#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAX_CHAR (16 * 1024)

#define ILLEGAL_ARGUMENT_EXIT_CODE 1
#define IO_ERROR_EXIT_CODE 2

static FILE * input;
static FILE * output;
static char inputFilePath[MAX_CHAR];
static char outputFilePath[MAX_CHAR];

static void usage(int argc, const char* argv[]);
static void showParameters(void);
static void countChars(void);
static void openFiles(void);
static void closeFiles(void);
static int readNextLine(char line[MAX_CHAR]);
static void writeNumber(int number);
static void kill(int exitCode);

void kill(int exitCode)
{
  closeFiles();
  exit(exitCode);
}

void openFiles(void)
{
   input = fopen(inputFilePath, "r");
   if (input == NULL)
   {
      fprintf(stderr, "O arquivo %s no foi encontrado.\n", inputFilePath);
      kill(IO_ERROR_EXIT_CODE);
   }

   output = fopen(outputFilePath, "w");
   if (output == NULL)
   {
      fprintf(stderr, "O arquivo %s no foi encontrado.\n", outputFilePath);
      kill(IO_ERROR_EXIT_CODE);
   }

   printf("Arquivos abertos: %s e %s.\n", inputFilePath, outputFilePath);
}

void closeFiles(void)
{
   if (output != NULL)
   {
      fclose(output);
      output = NULL;

      printf("Arquivo %s fechado.\n", outputFilePath);
   }

   if (input != NULL)
   {
      fclose(input);
      input = NULL;

      printf("Arquivo %s fechado.\n", inputFilePath);
   }

}

int readNextLine(char line[MAX_CHAR])
{
   if (fgets(line, MAX_CHAR, input) == NULL)
   {
     return 0;
   }
   line[strlen(line) - 1] = '\0';
   printf("Linha lida:\n");
   printf("(%s)\n", line);
   return 1;
}

void writeNumber(int number)
{
   fprintf(output, "%d\n", number);
   printf("Nmero escrito: %d.\n", number);
}

void countChars(void)
{
   char line[MAX_CHAR] = "";

   openFiles();
   while(readNextLine(line))
   {
      writeNumber(strlen(line));
   }
   closeFiles();
}

void showParameters(void)
{
  printf("Parmetros informados:\n");
  printf("Entrada: %s.\n", inputFilePath);
  printf("Saida: %s.\n", outputFilePath);
}

void usage(int argc, const char* argv[])
{
  int i;

  fprintf(stderr, "Parmetros errados.\n");
  fprintf(stderr, "Quantidade de parmetros fornecida: %d.\n", argc - 1);

  for (i = 1; i < argc; i++)
  {
     fprintf(stderr, "Parmetro %d : %s.\n", i, argv[i]);
  }

  fprintf(stderr, "Parmetros esperados:\n");
  fprintf(stderr, "<input> <output>\n");
  fprintf(stderr, "<input> := caminho para o arquivo de entrada.\n");
  fprintf(stderr, "<output> := caminho para o arquivo de sada.\n");
  
  exit(ILLEGAL_ARGUMENT_EXIT_CODE);
}

int main(int argc, const char* argv[])
{
   if (argc < 3)
   {
      fprintf(stderr, "Quantidade de parmetros errada.\n");
      usage(argc, argv);
   }

   strncpy(inputFilePath, argv[1], MAX_CHAR);
   strncpy(outputFilePath, argv[2], MAX_CHAR);

   showParameters();

   countChars();

   printf("Execuo terminada.\n");

   return 0;
}

