#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define ASCENDING "ASCENDING" 
#define DESCENDING "DESCENDING"

#define MAX_CHAR (16 * 1024)

#define ILLEGAL_ARGUMENT_EXIT_CODE 1
#define IO_ERROR_EXIT_CODE 2

static FILE * input;
static FILE * output;
static char order[MAX_CHAR];
static char inputFilePath[MAX_CHAR];
static char outputFilePath[MAX_CHAR];

static int (*isPreviousTo)(char, char);

static void usage(int argc, const char* argv[]);
static int isLargerThan(char ch1, char ch2);
static int isLessThan(char ch1, char ch2);
static void showParameters(void);
static void sortChars(void);
static void openFiles(void);
static void closeFiles(void);
static int readNextLine(char line[MAX_CHAR]);
static void sortLine(char line[MAX_CHAR]);
static void writeLine(const char line[MAX_CHAR]);
static void kill(int exitCode);

void kill(int exitCode)
{
  closeFiles();
  exit(exitCode);
}

void openFiles(void)
{
   input = fopen(inputFilePath, "r");
   if (input == NULL)
   {
      fprintf(stderr, "O arquivo %s no foi encontrado.\n", inputFilePath);
      kill(IO_ERROR_EXIT_CODE);
   }

   output = fopen(outputFilePath, "w");
   if (output == NULL)
   {
      fprintf(stderr, "O arquivo %s no foi encontrado.\n", outputFilePath);
      kill(IO_ERROR_EXIT_CODE);
   }

   printf("Arquivos abertos: %s e %s.\n", inputFilePath, outputFilePath);
}

void closeFiles(void)
{
   if (output != NULL)
   {
      fclose(output);
      output = NULL;

      printf("Arquivo %s fechado.\n", outputFilePath);
   }

   if (input != NULL)
   {
      fclose(input);
      input = NULL;

      printf("Arquivo %s fechado.\n", inputFilePath);
   }

}

int readNextLine(char line[MAX_CHAR])
{
   if (fgets(line, MAX_CHAR, input) == NULL) {
     return 0;
   }
   line[strlen(line) - 1] = '\0';
   printf("Linha lida:\n");
   printf("(%s)\n", line);
   return 1;
}

void sortLine(char line[MAX_CHAR])
{
  int i;
  int j;
  int size = strlen(line);

  for (i = 0; i < size - 1; i++)
  {
    char first = line[i];
    for (j = i; j < size; j++)
    {
      char ch = line[j];
      if (isPreviousTo(ch, first))
      {
        first = ch;
        line[j] = line[i];
        line[i] = ch;
      }
    }
  }
  printf("Linha ordenada:\n");
  printf("(%s)\n", line);
}

void writeLine(const char line[MAX_CHAR])
{
   fprintf(output, "%s\n", line);
   printf("Linha escrita:\n");
   printf("(%s)\n", line);
}

void sortChars(void)
{
   char line[MAX_CHAR] = "";

   openFiles();
   while(readNextLine(line))
   {
      sortLine(line);
      writeLine(line);
   }
   closeFiles();
}

void showParameters(void)
{
  printf("Parmetros informados:\n");
  printf("Ordem: %s.\n", order);
  printf("Entrada: %s.\n", inputFilePath);
  printf("Saida: %s.\n", outputFilePath);
}

int isLargerThan(char ch1, char ch2)
{
  return (ch1 > ch2);
}

int isLessThan(char ch1, char ch2)
{
  return (ch1 < ch2);
}

void usage(int argc, const char* argv[])
{
  int i;

  fprintf(stderr, "Parmetros errados.\n");
  fprintf(stderr, "Quantidade de parmetros fornecida: %d.\n", argc - 1);

  for (i = 1; i < argc; i++)
  {
     fprintf(stderr, "Parmetro %d : %s.\n", i, argv[i]);
  }

  fprintf(stderr, "Parmetros esperados:\n");
  fprintf(stderr, "<order> <input> <output>\n");
  fprintf(stderr, "<order> := (%s | %s)\n", ASCENDING, DESCENDING);
  fprintf(stderr, "<input> := caminho para o arquivo de entrada.\n");
  fprintf(stderr, "<output> := caminho para o arquivo de sada.\n");
  
  exit(ILLEGAL_ARGUMENT_EXIT_CODE);
}

int main(int argc, const char* argv[])
{
   if (argc < 4)
   {
      fprintf(stderr, "Quantidade de parmetros errada.\n");
      usage(argc, argv);
   }

   strncpy(order, argv[1], MAX_CHAR);
   strncpy(inputFilePath, argv[2], MAX_CHAR);
   strncpy(outputFilePath, argv[3], MAX_CHAR);

   if (strcmp(order, ASCENDING) == 0)
   {
     isPreviousTo = isLessThan;
   }
   else if (strcmp(order, DESCENDING) == 0)
   {
     isPreviousTo = isLargerThan;
   }
   else
   {
     fprintf(stderr, "O parmetro ordem est invlido.\n");
     usage(argc, argv);
   }

   showParameters();

   sortChars();

   printf("Execuo terminada.\n");

   return 0;
}

