package csbase.algorithms.flows;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.swing.Icon;

import tecgraf.javautils.gui.tree.DefaultNode;

import csbase.client.applications.ApplicationImages;
import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.AlgorithmVersionInfo;

/**
 * N Algoritmo.
 *
 * @author lmoreira
 */
public final class AlgorithmTreeNode extends DefaultNode implements
  Comparable<AlgorithmTreeNode> {
  /**
   * O algoritmo.
   */
  private AlgorithmInfo algorithm;

  /**
   * Cria o n.
   * 
   * @param algorithm O algoritmo (No aceita {@code null}).
   */
  public AlgorithmTreeNode(AlgorithmInfo algorithm) {
    super(algorithm.getName());
    this.algorithm = algorithm;
    List<AlgorithmVersionTreeNode> children = 
      new ArrayList<AlgorithmVersionTreeNode>();
    for (AlgorithmVersionInfo algorithmVersion : algorithm.getVersions()) {
      children.add(new AlgorithmVersionTreeNode(algorithmVersion));
    }
    Collections.sort(children);
    for (DefaultNode child : children) {
      addChild(child);
    }
  }

  public int compareTo(AlgorithmTreeNode node) {
    return getAlgorithm().compareTo(node.getAlgorithm());
  }  

  /**
   * Obtm o algoritmo.
   * 
   * @return .
   */
  public AlgorithmInfo getAlgorithm() {
    return this.algorithm;
  }

  /* (non-Javadoc)
   * @see tecgraf.javautils.gui.tree.DefaultNode#getIcon()
   */
  @Override
  public Icon getIcon() {
    return ApplicationImages.ICON_ALGORITHM_16;
  }
}
