package csbase.algorithms.flows;

import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import javax.swing.Icon;

import tecgraf.javautils.gui.tree.DefaultNode;

import csbase.client.applications.ApplicationImages;
import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.Category;

import static csbase.logic.algorithms.AlgorithmInfo.HIDE_ALGORITHM_PROPERTY;
import static csbase.logic.algorithms.AlgorithmInfo.HIDE_ALGORITHM_VALUE;

/**
 * N Algoritmo.
 * 
 * @author lmoreira
 */
public final class CategoryTreeNode extends DefaultNode implements
  Comparable<CategoryTreeNode> {
  /**
   * O algoritmo.
   */
  private Category category;

  /**
   * Cria o n.
   * 
   * @param category A categoria (No aceita {@code null}).
   */
  public CategoryTreeNode(Category category) {
    super(category.getName());
    this.category = category;
    List<CategoryTreeNode> categoryTreeNodes = new ArrayList<CategoryTreeNode>();
    for (Category childCategory : category.getCategories()) {
      categoryTreeNodes.add(new CategoryTreeNode(childCategory));
    }
    Collections.sort(categoryTreeNodes);
    List<AlgorithmTreeNode> algorithmTreeNodes = new ArrayList<AlgorithmTreeNode>();
    for (AlgorithmInfo algorithm : category.getAlgorithms()) {
      if (!HIDE_ALGORITHM_VALUE.equals(
        algorithm.getPropertyValue(HIDE_ALGORITHM_PROPERTY))) {
        AlgorithmTreeNode algorithmTreeNode = new AlgorithmTreeNode(algorithm);
        algorithmTreeNodes.add(algorithmTreeNode);
      }
    }
    Collections.sort(algorithmTreeNodes);
    List<DefaultNode> children = new LinkedList<DefaultNode>();
    children.addAll(categoryTreeNodes);
    children.addAll(algorithmTreeNodes);
    for (DefaultNode child : children) {
      addChild(child);
    }
  }

  public int compareTo(CategoryTreeNode node) {
    return getCategory().compareTo(node.getCategory());
  }

  /**
   * Obtm a categoria.
   * 
   * @return A categoria.
   */
  public Category getCategory() {
    return category;
  }

  @Override
  public Icon getIcon() {
    return ApplicationImages.ICON_FOLDER_16;
  }
}
