/**
 * $Id: FlowApplication.java 105493 2010-05-13 18:07:24Z pizzol $
 */

package csbase.algorithms.flows;

import java.awt.BorderLayout;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DragGestureEvent;
import java.awt.dnd.DragGestureListener;
import java.awt.dnd.DragSource;
import java.awt.geom.Point2D;
import java.io.IOException;
import java.rmi.RemoteException;
import java.text.MessageFormat;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JToolBar;
import javax.swing.JTree;
import javax.swing.ScrollPaneConstants;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import tecgraf.javautils.FileUtils;
import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.StandardDialogs;
import tecgraf.javautils.gui.tree.FilterableTreePanel;
import csbase.algorithms.flows.actions.AskForParameterValuesAction;
import csbase.algorithms.flows.actions.CloseAction;
import csbase.algorithms.flows.actions.ExecuteGraphAction;
import csbase.algorithms.flows.actions.LoadAction;
import csbase.algorithms.flows.actions.NewAction;
import csbase.algorithms.flows.actions.RemoveElementsAction;
import csbase.algorithms.flows.actions.SaveAction;
import csbase.algorithms.flows.actions.SaveAsAction;
import csbase.algorithms.flows.actions.ShowParameterValuesAction;
import csbase.algorithms.flows.actions.UpdateAlgorithmTreeAction;
import csbase.algorithms.flows.actions.UpdateAllVersionsAction;
import csbase.algorithms.flows.configurator.FlowAlgorithmConfiguratorView;
import csbase.algorithms.flows.filters.AskForParameterValuesActionFilter;
import csbase.algorithms.flows.filters.CopyAndPasteFilter;
import csbase.algorithms.flows.filters.HighlightElementFilter;
import csbase.algorithms.flows.filters.HintElementFilter;
import csbase.algorithms.flows.filters.PopupFilter;
import csbase.algorithms.flows.filters.ResizeNodeFilter;
import csbase.algorithms.flows.filters.SelectElementFilter;
import csbase.algorithms.flows.filters.UpdateStatusBarrFilter;
import csbase.algorithms.flows.filters.WorkspaceFilter;
import csbase.algorithms.flows.graph.FileTypeColorManager;
import csbase.algorithms.flows.graph.Graph;
import csbase.algorithms.flows.graph.GraphElement;
import csbase.algorithms.flows.graph.GraphFileDescriptor;
import csbase.algorithms.flows.graph.GraphLink;
import csbase.algorithms.flows.graph.GraphListener;
import csbase.algorithms.flows.graph.GraphNode;
import csbase.algorithms.flows.graph.actions.BreakLinkAction;
import csbase.algorithms.flows.graph.actions.BypassAction;
import csbase.algorithms.flows.graph.actions.CopyParameterValuesAction;
import csbase.algorithms.flows.graph.actions.PasteParameterValuesAction;
import csbase.algorithms.flows.graph.actions.RemoveSelectedElementsAction;
import csbase.algorithms.flows.graph.actions.UpdateVersionAction;
import csbase.algorithms.flows.messages.PickGraphMessage;
import csbase.algorithms.flows.messages.RemoveElementsMessage;
import csbase.algorithms.flows.messages.ResetMessage;
import csbase.algorithms.flows.messages.ShowParameterValuesMessage;
import csbase.algorithms.flows.zoom.ZoomControl;
import csbase.client.algorithms.commands.CommandPersistenceListener;
import csbase.client.algorithms.commands.CommandViewerPanel;
import csbase.client.algorithms.parameters.ParameterView.Mode;
import csbase.client.applicationmanager.ApplicationException;
import csbase.client.applications.ApplicationAboutAction;
import csbase.client.applications.ApplicationExitAction;
import csbase.client.applications.ApplicationProject;
import csbase.client.desktop.CommandPersistenceCommandNotificationHandler;
import csbase.client.desktop.DesktopFrame;
import csbase.client.desktop.RemoteTask;
import csbase.client.kernel.ClientException;
import csbase.client.project.ProjectTreePath;
import csbase.client.remote.srvproxies.AlgorithmManagementProxy;
import csbase.client.util.sga.CommandExecutionDialog;
import csbase.client.util.sga.CommandRequestedListener;
import csbase.exception.ConfigurationException;
import csbase.exception.OperationFailureException;
import csbase.exception.ParseException;
import csbase.exception.project.FileLockedException;
import csbase.logic.ClientProjectFile;
import csbase.logic.ClientProjectFileInputStream;
import csbase.logic.ClientProjectFileOutputStream;
import csbase.logic.CommandInfo;
import csbase.logic.algorithms.AlgorithmConfigurator;
import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.AlgorithmVersionInfo;
import csbase.logic.algorithms.Category;
import csbase.logic.algorithms.CategorySet;
import csbase.logic.algorithms.commands.CommandPropertiesFilter;
import csbase.logic.algorithms.flows.Flow;
import csbase.logic.algorithms.flows.FlowAlgorithmParser;
import csbase.logic.algorithms.flows.configurator.FlowAlgorithmConfigurator;

/**
 * Applicao de Construo de Fluxos de Algoritmo.
 */
public final class FlowApplication extends ApplicationProject {
  /** Posio relativa do splitter vertical. */
  private static final int VERTICAL_SPLITTER_LOCATION = 450;

  // Principais componentes da aplicao //
  /** rvore de algoritmos. */
  private FilterableTreePanel algorithmVersionTreePanel;
  /** Barra de status do editor. */
  private JPanel statusBarrPanel;

  /** Mensagem da barra de status. */
  private JLabel messageStatusLabel;

  /** Barra de ferramentas do editor. */
  private JToolBar toolBar;

  /** Espao de trabalho, onde o fluxo pode ser editado. */
  private final Workspace workspace;

  /** Visualizador de comandos do editor. */
  private final CommandViewerPanel commandViewerPanel;

  /** Controle de nvel de zoom. */
  private ZoomControl zoomControl;

  /** Painel de split horizontal, que contm a rvore de algoritmos e o fluxo */
  private JSplitPane horizontalSplitPane;

  // Monitorao de comandos //
  /**
   * Observadores interessados nos comandos solicitados com o dilogo de
   * execuo de comandos.
   */
  private final List<CommandRequestedListener> listeners;

  /**
   * Conjunto de comandos submetidos via essa aplicao, que devem ser
   * monitorados at que sejam terminados.
   */
  private final Set<String> commandIds;

  /** Observador de comandos persistidos. */
  private final CommandPersistenceListener commandPersistenceListener;

  // Filtros //
  /** Lista de itens de menu pop-up. */
  private List<WorkspaceFilter> popupFilters;

  /** Filtro para menu pop-up */
  private PopupFilter basicPopupFilter;

  /** Filtro para redimensionamento de ns. */
  private ResizeNodeFilter resizeNodeFilter;

  /** Filtro para ao de copiar/colar. */
  private CopyAndPasteFilter copyAndPasteFilter;

  /** Filtro para abrir a janela de configurao de parmetros. */
  private AskForParameterValuesActionFilter askForParameterValuesFilter;

  /** Filtro para destacar elementos do fluxo. */
  private HighlightElementFilter hightLightFilter;

  /** Filtro para a exibio de dicas. */
  private HintElementFilter hintFilter;

  /** Filtro para a seleo de elementos do fluxo. */
  private SelectElementFilter selectElementFilter;

  /** Filtro para atualizao da barra de status */
  private UpdateStatusBarrFilter updateStatusBarrFilter;

  // Aes //
  /** Ao de carregar um arquivo de fluxo. */
  private LoadAction loadAction;

  /** Ao de executar um fluxo. */
  private ExecuteGraphAction executeAction;
  /** Ao de criao de novo fluxo. */
  private NewAction newAction;

  /** Ao de remoo de elemento do fluxo. */
  private RemoveElementsAction removeAction;

  /** Ao de fechar fluxo. */
  private CloseAction closeAction;

  /** Ao de salvar fluxo. */
  private SaveAction saveAction;

  /** Aoo de salvar fluxo como um novo arquivo. */
  private SaveAsAction saveAsAction;

  /** Ao de atualizar a verso de todos os ns do fluxo. */
  private UpdateAllVersionsAction updateVersionsAction;

  /** Ao de atualizar rvore de algoritmos do editor. */
  private UpdateAlgorithmTreeAction updateAlgorithmTreeAction;

  /** Ao de mostrar propriedades de um elemento do fluxo. */
  private ShowParameterValuesAction showPropertiesAction;

  /**
   * Caminho para o arquivo de fluxo atual (se houver).
   */
  private ProjectTreePath currentFilePath;

  /**
   * Sinal que indica se o fluxo foi alterado depois de salvo.
   */
  private boolean wasChanged;

  /**
   * A descrio do comando a ser executado. Valor aparecer como default.
   */
  private String commandDescription;

  /**
   * Constri a aplicao de construo de fluxos de algoritmo.
   * 
   * @param id O identificador.
   * @throws ApplicationException
   */
  public FlowApplication(String id) throws ApplicationException {
    super(id);
    commandIds = new HashSet<String>();
    listeners = new LinkedList<CommandRequestedListener>();
    algorithmVersionTreePanel = createAlgorithmTreePanel();
    commandViewerPanel = new CommandViewerPanel(getApplicationFrame(),
      getApplicationProject().getId(), createCommandPropertiesFilter(),
      CommandViewerPanel.ToolBarLocation.LEFT) {
      @Override
      protected boolean exportCommand(final CommandInfo command) {
        RemoteTask<AlgorithmConfigurator> task = new RemoteTask<AlgorithmConfigurator>() {
          @Override
          public void performTask() throws Exception {
            AlgorithmConfigurator configurator = command.getConfigurator();
            setResult(configurator);
          }
        };
        String msg = getI18n("loadingConfigurator");
        if (!task.execute(getApplicationFrame(), msg, msg)) {
          return false;
        }
        AlgorithmConfigurator configurator = task.getResult();
        if (!(configurator instanceof FlowAlgorithmConfigurator)) {
          return false;
        }
        if (!createNewGraph()) {
          return false;
        }
        return editConfigurator((FlowAlgorithmConfigurator) configurator);
      }
    };
    workspace = new Workspace(getApplicationFrame(), true);
    workspace.getGraph().addGraphListener(new ChangedListener());
    JTree tree = algorithmVersionTreePanel.getTree();
    workspace.setTransferHandler(tree.getTransferHandler());
    createFilters();
    createGui();
    ListIterator<WorkspaceFilter> popupFilterIterator = popupFilters
      .listIterator(popupFilters.size());
    while (popupFilterIterator.hasPrevious()) {
      WorkspaceFilter filter = popupFilterIterator.previous();
      filter.attach();
    }
    selectElementFilter.attach();
    resizeNodeFilter.attach();
    hightLightFilter.attach();
    hintFilter.attach();
    updateStatusBarrFilter.attach();
    basicPopupFilter.attach();
    askForParameterValuesFilter.attach();
    copyAndPasteFilter.attach();
    createNewGraph();
    try {
      List<String> fileTypes = getStringListSpecificProperty("file.type.name");
      List<String> rgbs = getStringListSpecificProperty("file.type.rgb");
      FileTypeColorManager.createInstance(fileTypes, rgbs);
    }
    catch (ParseException e) {
      throw new ApplicationException(e.getLocalizedMessage(), e);
    }
    commandPersistenceListener = new CommandPersistenceListener() {

      public void commandWasRemoved(Object projectId, String commandId) {
        commandIds.remove(commandId);
      }

      public void commandWasSaved(Object projectId, String commandId) {
      }

      public void commandWasUpdated(Object projectId, String commandId) {
      }
    };
  }

  /**
   * Constri o painel com a rvore de algoritmos.
   * 
   * @return O painel com a rvore de algoritmos.
   */
  private FilterableTreePanel createAlgorithmTreePanel() {
    CategorySet categorySet = AlgorithmManagementProxy
      .retrieveAllCategories(DesktopFrame.getInstance().getDesktopFrame());
    AlgorithmInfo[] algorithms = AlgorithmManagementProxy
      .retrieveAllAlgorithmInfos(DesktopFrame.getInstance().getDesktopFrame());
    SortedSet<AlgorithmInfo> uncategorizedAlgorithms = new TreeSet<AlgorithmInfo>();
    uncategorizedAlgorithms.addAll(Arrays.asList(algorithms));
    for (Category category : categorySet.getCategories()) {
      Iterator<AlgorithmInfo> algorithmIterator = uncategorizedAlgorithms
        .iterator();
      while (algorithmIterator.hasNext()) {
        if (category.containsAlgorithmInChildren(algorithmIterator.next())) {
          algorithmIterator.remove();
        }
      }
    }
    FilterableTreePanel algorithmVersionTreePanel = new FilterableTreePanel(
      new AllCategoryTreeNode(categorySet.getCategories(),
        uncategorizedAlgorithms));
    DragSource dragSource = DragSource.getDefaultDragSource();
    JTree algorithmVersionTree = algorithmVersionTreePanel.getTree();
    dragSource.createDefaultDragGestureRecognizer(algorithmVersionTree,
      DnDConstants.ACTION_COPY, new DragGestureListener() {

        public void dragGestureRecognized(DragGestureEvent dge) {
          AlgorithmVersionInfo version = getSelectedAlgorithmVersion();

          if (version != null) {
            dge.startDrag(null, new AlgorithmVersionTransferable(version));
          }
        }

      });
    algorithmVersionTree.getSelectionModel().setSelectionMode(
      TreeSelectionModel.SINGLE_TREE_SELECTION);
    return algorithmVersionTreePanel;
  }

  /**
   * Atualiza a rvore de algoritmos.
   */
  public void reloadAlgorithmTree() {
    horizontalSplitPane.remove(algorithmVersionTreePanel);
    algorithmVersionTreePanel = createAlgorithmTreePanel();
    horizontalSplitPane.setLeftComponent(algorithmVersionTreePanel);
  }

  /**
   * Chama o configurador do algoritmo para obter os parmetros dos ns.
   */
  public void askForParameterValues() {
    PickGraphMessage pickGraphMessage = new PickGraphMessage();
    workspace.sendVO(pickGraphMessage);
    Graph graph = pickGraphMessage.getGraph();
    for (GraphNode node : graph.getNodeCollection()) {
      if (node.isSelected()) {
        node.askForParameterValues();
      }
    }
  }

  /**
   * Limpa o texto da parte da barra de status especfica para mensagens.
   */
  public void clearStatusBarrMessage() {
    messageStatusLabel.setText(" ");
  }

  /**
   * Fecha o fluxo atual se ele existir e cria um novo grafo.
   * 
   * @return <code>true</code> sucesso ou <code>false</code> se o usurio
   *         cancelou o fechamento do fluxo.
   */
  public boolean closeGraph() {
    return createNewGraph();
  }

  /**
   * Fecha o fluxo atual se ele existir e cria um novo grafo.
   * 
   * @return <code>true</code> sucesso ou <code>false</code> se o usurio
   *         cancelou o fechamento do fluxo.
   */
  public boolean createNewGraph() {
    if (!userCanKillApplication()) {
      return false;
    }
    workspace.setGraph(new Graph(getApplicationFrame()));
    return true;
  }

  /**
   * Edita o configurador do fluxo.
   * 
   * @param configurator configurador a ser editado
   * @return retorna true, se a edio foi feita com sucesso, caso contrrio,
   *         retorna false
   */
  public boolean editConfigurator(final FlowAlgorithmConfigurator configurator) {
    RemoteTask<Void> task = new RemoteTask<Void>() {
      @Override
      protected void performTask() throws Exception {
        Flow flow = configurator.getFlow();

        Graph graph = new Graph(getApplicationFrame(), flow);
        workspace.setGraph(graph);
      }
    };
    if (task.execute(getApplicationFrame(), "Editando fluxo",
      "Carregando informaes do fluxo.")) {
      updateApplicationTitle();
      return true;
    }
    return false;
  }

  /**
   * Executar o fluxo de algoritmos corrente.
   */
  public void executeGraph() {
    if (!getWorkspace().getGraph().isExecutable()) {
      throw new IllegalStateException(
        "O fluxo de algoritmos no est pronto para ser executado.");
    }
    String flowName;
    if (currentFilePath == null) {
      flowName = getI18n("anonymous");
    }
    else {
      ClientProjectFile currentFile = currentFilePath.getFile();
      String currentFilePathAsString = currentFile.getStringPath();
      if (wasChanged) {
        flowName = getI18n("flowModified", currentFilePathAsString);
      }
      else {
        flowName = currentFilePathAsString;
      }
    }
    Graph graph = workspace.getGraph();
    Flow flow = graph.toFlow();
    flow.setName(flowName);
    Graph newGraph;
    try {
      newGraph = new Graph(getApplicationFrame(), flow);
      final FlowAlgorithmConfiguratorView configuratorView = new FlowAlgorithmConfiguratorView(
        getApplicationFrame(), newGraph, Mode.CONFIGURATION);
      CommandExecutionDialog commandExecutionDialog = new CommandExecutionDialog(
        getApplicationFrame(), configuratorView, getApplicationProject());
      commandExecutionDialog.setCommandDescription(commandDescription);
      commandExecutionDialog
        .addCommandRequestedListener(new CommandRequestedListener() {
          public void commandsWereRequested(Set<CommandInfo> submittedCommands) {
            for (CommandInfo command : submittedCommands) {
              commandIds.add(command.getId());
            }
          }
        });
      addExternalListeners(commandExecutionDialog);
      commandExecutionDialog.setVisible(true);
    }
    catch (ParseException e) {
      showExceptionStack(e);
    }
    catch (ClientException e) {
      showExceptionStack(e);
    }
    catch (RemoteException e) {
      showExceptionStack(e);
    }
  }

  /**
   * 
   * Executa um determinado grafo que representa graficamente um fluxo de
   * algoritmos.
   * 
   * @param graph grafo que representa o fluxo de algoritmos
   * @param commandDescription descrio default para o comando a ser executado
   */
  public void executeGraph(Graph graph, String commandDescription) {
    workspace.setGraph(graph);
    setCommandDescription(commandDescription);
  }

  /**
   * Estabelece a descrio default do comando a ser gerado pelo algoritmo.
   * 
   * @param commandDescription descrio do comando
   */
  public void setCommandDescription(String commandDescription) {
    this.commandDescription = commandDescription;
  }

  /**
   * Adiciona listeners externos para o comando solicitado pela janela de
   * execuo de comandos. Esses listeners so cadastrados por classes que
   * executem essa aplicao.
   * 
   * @param commandExecutionDialog janela de execuo de comandos
   */
  private void addExternalListeners(
    CommandExecutionDialog commandExecutionDialog) {
    for (CommandRequestedListener listener : listeners) {
      commandExecutionDialog.addCommandRequestedListener(listener);
    }
  }

  /**
   * Adiciona um listener para um comando solicitado com o dilogo de execuo
   * de comandos.
   * 
   * @param listener listener de um comando solicitado com o dilogo de execuo
   *        de comandos
   */
  public void addCommandRequestedListener(CommandRequestedListener listener) {
    if (listener == null) {
      throw new IllegalArgumentException("O parmetro listener est nulo.");
    }
    listeners.add(listener);
  }

  /**
   * Obtm a rea de trabalho.
   * 
   * @return A rea de trabalho.
   */
  public Workspace getWorkspace() {
    return workspace;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void killApplication() {
    commandIds.clear();
    CommandPersistenceCommandNotificationHandler.getInstance().removeListener(
      commandPersistenceListener);
    commandViewerPanel.stop();
  }

  /**
   * Solicita ao usurio um arquivo que contenha informaes de um fluxo de
   * algoritmos e o carrega.
   * 
   * @return <code>true</code> em caso de sucesso, ou <code>false</code> caso
   *         contrrio.
   */
  public boolean openGraph() {
    try {
      if (!userCanKillApplication()) {
        return true;
      }
      ClientProjectFile file = browseFileOpen(getDefaultFileType());
      if (file != null) {
        load(file);
      }
      return true;
    }
    catch (ApplicationException e) {
      showExceptionStack(e);
    }
    return false;
  }

  /**
   * Remove os ns e conexes que esto selecionados.
   */
  public void removeSelectedElements() {
    workspace.sendVO(new RemoveElementsMessage());
    workspace.sendVO(new ResetMessage());
  }

  /**
   * Solicita ao usurio um arquivo que contenha informaes de um fluxo de
   * algoritmos e o salva o fluxo corrente no arquivo.
   * 
   * @return <code>true</code> em caso de sucesso, ou <code>false</code> caso
   *         contrrio.
   */
  public boolean saveAsGraph() {
    try {
      ClientProjectFile file = browseFileSave(getDefaultFileType());
      if (file != null) {
        if (save(file)) {
          currentFilePath = new ProjectTreePath(file);
          updateApplicationTitle();
          return true;
        }
      }
    }
    catch (ApplicationException e) {
      showExceptionStack(e);
    }
    return false;
  }

  /**
   * Seno houver um arquivo corrente em uso, ele solicita ao usurio um arquivo
   * que contenha informaes de um fluxo de algoritmos e o torna o arquivo
   * corrente, logo aps ele salva as informaes do algoritmo neste arquivo.
   * 
   * @return <code>true</code> em caso de sucesso, ou <code>false</code> caso
   *         contrrio.
   */
  public boolean saveGraph() {
    if (currentFilePath == null) {
      return saveAsGraph();
    }
    return save(currentFilePath.getFile());
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void sendMessage(String name, Object value) {
    if (value != null) {
      if (name.equals(PROJECT_FILE_MESSAGE)) {
        ClientProjectFile file = (ClientProjectFile) value;
        load(file);
        return;
      }
    }
    super.sendMessage(name, value);
  }

  /**
   * Atribui uma mensagem na barra de status.
   * 
   * @param hint A dica.
   * 
   * @throws IllegalArgumentException Se o parmetro hint est nulo.
   */
  public void setStatusBarrMessage(String hint) {
    if (hint == null) {
      throw new IllegalArgumentException("O parmetro hint est nulo.");
    }
    messageStatusLabel.setText(hint);
  }

  /**
   * Exibe a parametrizao.
   */
  public void showParameterValues() {
    workspace.sendVO(new ShowParameterValuesMessage());
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void startApplication() throws ApplicationException {
    super.startApplication();
    updateApplicationTitle();
    commandViewerPanel.start();
    CommandPersistenceCommandNotificationHandler.getInstance().addListener(
      commandPersistenceListener);
    commandIds.clear();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected boolean userCanKillApplication() {
    if (wasChanged) {
      switch (StandardDialogs.showYesNoCancelDialog(getApplicationFrame(),
        getName(), getI18n("askForSaving"))) {
        case JOptionPane.CANCEL_OPTION:
          return false;
        case JOptionPane.YES_OPTION:
          if (!saveGraph()) {
            return false;
          }
          break;
      }
    }
    currentFilePath = null;
    updateApplicationTitle();
    workspace.sendVO(new ResetMessage());
    return true;
  }

  /**
   * Constri as aes.
   */
  private void createActions() {
    closeAction = new CloseAction(this);
    executeAction = new ExecuteGraphAction(this);
    loadAction = new LoadAction(this);
    newAction = new NewAction(this);
    removeAction = new RemoveElementsAction(this);
    saveAction = new SaveAction(this);
    saveAsAction = new SaveAsAction(this);
    showPropertiesAction = new ShowParameterValuesAction(this);
    updateVersionsAction = new UpdateAllVersionsAction(this);
    updateAlgorithmTreeAction = new UpdateAlgorithmTreeAction(this);
  }

  /**
   * Cria o filtro de propriedades do comando.
   * 
   * @return filtro .
   */
  private CommandPropertiesFilter createCommandPropertiesFilter() {
    return new CommandPropertiesFilter() {

      public boolean accept(CommandInfo command) {
        return commandIds.contains(command.getId());
      }

    };
  }

  /**
   * Cria o menu Editar.
   * 
   * @return O menu Editar.
   */
  private JMenu createEditMenu() {
    JMenu editMenu = new JMenu(getI18n("editMenu"));
    editMenu.add(new JMenuItem(askForParameterValuesFilter.getAction()));
    editMenu.addSeparator();
    editMenu.add(new JMenuItem(removeAction));
    editMenu.addSeparator();
    editMenu.add(new JMenuItem(updateVersionsAction));
    return editMenu;
  }

  /**
   * Cria o menu Executar.
   * 
   * @return O menu Executar.
   */
  private JMenu createExecuteMenu() {
    JMenu executeMenu = new JMenu(getI18n("executeMenu"));
    executeMenu.add(new JMenuItem(executeAction));
    return executeMenu;
  }

  /**
   * Cria o menu Arquivo.
   * 
   * @return O menu Arquivo.
   */
  private JMenu createFileMenu() {
    JMenu fileMenu = new JMenu(getI18n("fileMenu"));
    fileMenu.add(new JMenuItem(newAction));
    fileMenu.add(new JMenuItem(loadAction));
    fileMenu.add(new JMenuItem(closeAction));
    fileMenu.add(new JMenuItem(saveAction));
    fileMenu.add(new JMenuItem(saveAsAction));
    fileMenu.add(new JMenuItem(updateAlgorithmTreeAction));
    fileMenu.addSeparator();
    fileMenu.add(new ApplicationExitAction(this));
    return fileMenu;
  }

  /**
   * Constri os filtros que a aplicao utiliza.
   */
  private void createFilters() {
    askForParameterValuesFilter = new AskForParameterValuesActionFilter(
      new AskForParameterValuesAction(this), workspace);
    basicPopupFilter = new PopupFilter(workspace);
    copyAndPasteFilter = new CopyAndPasteFilter(workspace);
    hightLightFilter = new HighlightElementFilter(workspace);
    hintFilter = new HintElementFilter(workspace);
    resizeNodeFilter = new ResizeNodeFilter(workspace);
    selectElementFilter = new SelectElementFilter(workspace);
    updateStatusBarrFilter = new UpdateStatusBarrFilter(this);
    createPopupFilters();
  }

  /**
   * Contri a interface grfica.
   */
  private void createGui() {
    createActions();
    horizontalSplitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
    horizontalSplitPane.setLeftComponent(algorithmVersionTreePanel);
    JScrollPane workspaceScrollPane = new JScrollPane(workspace,
      ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,
      ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
    horizontalSplitPane.setRightComponent(workspaceScrollPane);
    horizontalSplitPane.setOneTouchExpandable(true);
    horizontalSplitPane.setResizeWeight(0.0);
    createToolBar();
    createStatusBar();
    JSplitPane verticalSplitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
    verticalSplitPane.setTopComponent(horizontalSplitPane);
    verticalSplitPane.setBottomComponent(commandViewerPanel);
    verticalSplitPane.setOneTouchExpandable(true);
    verticalSplitPane.setResizeWeight(1.0);
    JPanel mainPanel = new JPanel(new BorderLayout());
    mainPanel.add(toolBar, BorderLayout.NORTH);
    mainPanel.add(verticalSplitPane, BorderLayout.CENTER);
    mainPanel.add(statusBarrPanel, BorderLayout.SOUTH);
    getApplicationFrame().setJMenuBar(createMenuBar());
    getApplicationFrame().setContentPane(mainPanel);
    getApplicationFrame().setSize(
      DesktopFrame.getInstance().getDesktopFrame().getSize());
    horizontalSplitPane.resetToPreferredSizes();
    verticalSplitPane.setDividerLocation(VERTICAL_SPLITTER_LOCATION);
    updateApplicationTitle();
  }

  /**
   * Cria o menu de ajuda.
   * 
   * @return o menu criado.
   */
  private JMenu createHelpMenu() {
    final JMenu helpMenu = new JMenu(getI18n("helpMenu"));
    helpMenu.add(new ApplicationAboutAction(this));
    return helpMenu;
  }

  /**
   * Cria a barra de menus.
   * 
   * @return A barra de menus.
   */
  private JMenuBar createMenuBar() {
    JMenuBar menuBar = new JMenuBar();
    menuBar.add(createFileMenu());
    menuBar.add(createEditMenu());
    menuBar.add(createViewMenu());
    menuBar.add(createExecuteMenu());
    menuBar.add(createHelpMenu());
    return menuBar;
  }

  /**
   * Constri os menus pop-up.
   */
  private void createPopupFilters() {
    popupFilters = Arrays.<WorkspaceFilter> asList(
      csbase.algorithms.flows.graph.actions.AskForParameterValuesAction
        .createFilter(workspace),
      csbase.algorithms.flows.graph.actions.ShowParameterValuesAction
        .createFilter(workspace), CopyParameterValuesAction
        .createFilter(workspace), PasteParameterValuesAction
        .createFilter(workspace), BreakLinkAction.createFilter(workspace),
      RemoveSelectedElementsAction.createFilter(workspace), UpdateVersionAction
        .createFilter(workspace), BypassAction.createFilter(workspace));
  }

  /**
   * Constri a barra de status da aplicao.
   */
  private void createStatusBar() {
    statusBarrPanel = new JPanel(new BorderLayout());
    messageStatusLabel = new JLabel(" ");
    messageStatusLabel.setBorder(BorderFactory.createLoweredBevelBorder());
    statusBarrPanel.add(messageStatusLabel, BorderLayout.CENTER);
  }

  /**
   * Constri a barra de ferramentas.
   */
  private void createToolBar() {
    toolBar = new JToolBar();
    toolBar.setBorder(BorderFactory.createEtchedBorder());
    toolBar.addSeparator();
    JButton executeButton = new JButton(executeAction);
    JButton loadButton = new JButton(loadAction);
    JButton newButton = new JButton(newAction);
    JButton removeButton = new JButton(removeAction);
    JButton saveButton = new JButton(saveAction);
    JButton updateVersionsButton = new JButton(updateVersionsAction);
    JButton showParameterButton = new JButton(askForParameterValuesFilter
      .getAction());
    JButton showPropertiesButton = new JButton(showPropertiesAction);

    executeButton.setText("");
    loadButton.setText("");
    newButton.setText("");
    removeButton.setText("");
    saveButton.setText("");
    updateVersionsButton.setText("");
    showParameterButton.setText("");
    showPropertiesButton.setText("");
    executeButton.setToolTipText(executeAction.getToolTipText());
    loadButton.setToolTipText(loadAction.getToolTipText());
    newButton.setToolTipText(newAction.getToolTipText());
    removeButton.setToolTipText(removeAction.getToolTipText());
    saveButton.setToolTipText(saveAction.getToolTipText());
    updateVersionsButton.setToolTipText(updateVersionsAction.getToolTipText());
    showParameterButton.setToolTipText(askForParameterValuesFilter.getAction()
      .getToolTipText());
    showPropertiesButton.setToolTipText(showPropertiesAction.getToolTipText());
    toolBar.add(newButton);
    toolBar.add(loadButton);
    toolBar.add(saveButton);
    toolBar.addSeparator();
    toolBar.add(removeButton);
    toolBar.addSeparator();
    toolBar.add(showPropertiesButton);
    toolBar.add(showParameterButton);
    toolBar.add(updateVersionsButton);
    toolBar.addSeparator();
    toolBar.add(executeButton);
    toolBar.addSeparator();
    zoomControl = new ZoomControl(workspace.getZoomModel());
    toolBar.add(zoomControl);
  }

  /**
   * Cria o menu Exibir.
   * 
   * @return O menu Exibir.
   */
  private JMenu createViewMenu() {
    JMenu viewMenu = new JMenu(getI18n("viewMenu"));
    JMenuItem zoomInMenuItem = new JMenuItem(zoomControl.getZoomInAction());
    JMenuItem zoomOutMenuItem = new JMenuItem(zoomControl.getZoomOutAction());
    viewMenu.add(new JMenuItem(showPropertiesAction));
    viewMenu.addSeparator();
    viewMenu.add(zoomOutMenuItem);
    viewMenu.add(zoomInMenuItem);
    return viewMenu;
  }

  /**
   * Retorna o tipo padro de arquivo.
   * 
   * @return O tipo padro de arquivo.
   */
  private String getDefaultFileType() {
    List<String> fileTypeVector = getApplicationRegistry().getFileTypes();
    if (fileTypeVector.isEmpty()) {
      return null;
    }
    return fileTypeVector.get(0);
  }

  /**
   * Retorna a string localizada a partir de uma chave.
   * 
   * @param key A chave.
   * @return A string localizada.
   */
  private String getI18n(String key) {
    return LNG.get(FlowApplication.class.getName() + "." + key);
  }

  /**
   * Retorna a string localizada a partir de uma chave e seus parmetros.
   * 
   * @param key A chave.
   * @param keyArgs Os parmetros.
   * @return A string localizada.
   */
  private String getI18n(String key, Object... keyArgs) {
    return MessageFormat.format(getI18n(key), keyArgs);
  }

  /**
   * Retorna as informaes de verso do algoritmo selecionado.
   * 
   * @return As informaes de verso do algoritmo.
   */
  private AlgorithmVersionInfo getSelectedAlgorithmVersion() {
    JTree algorithmVersionTree = algorithmVersionTreePanel.getTree();
    TreePath treePath = algorithmVersionTree.getSelectionPath();
    if (treePath == null) {
      return null;
    }
    tecgraf.javautils.gui.tree.Node node = (tecgraf.javautils.gui.tree.Node) treePath
      .getLastPathComponent();
    if (node instanceof AlgorithmVersionTreeNode) {
      AlgorithmVersionTreeNode algorithmVersionTreeNode = (AlgorithmVersionTreeNode) node;
      return algorithmVersionTreeNode.getAlgorithmVersion();
    }
    else if (node instanceof AlgorithmTreeNode) {
      AlgorithmTreeNode algorithmTreeNode = (AlgorithmTreeNode) node;
      AlgorithmInfo algorithmInfo = algorithmTreeNode.getAlgorithm();
      return algorithmInfo.getLastVersion();
    }
    return null;
  }

  /**
   * Carrega o arquivo de fluxo especificado.
   * 
   * @param file O arquivo de fluxo.
   */
  private void load(final ClientProjectFile file) {
    currentFilePath = null;
    RemoteTask<Void> task = new RemoteTask<Void>() {
      @Override
      protected void performTask() throws OperationFailureException,
        IOException, ParseException {
        ClientProjectFileInputStream inputStream = null;
        try {
          inputStream = new ClientProjectFileInputStream(file, 0);
          FlowAlgorithmParser parser = new FlowAlgorithmParser();
          Flow flow = parser.read(inputStream);
          Graph graph = new Graph(getApplicationFrame(), flow);
          workspace.setGraph(graph);
        }
        finally {
          if (inputStream != null) {
            try {
              inputStream.close();
            }
            catch (IOException e) {
              showExceptionStack(e);
            }
          }
        }
      }
    };
    String message = getI18n("loadingFile", file.getStringPath());
    if (task.execute(getApplicationFrame(), message, message)) {
      currentFilePath = new ProjectTreePath(file);
      updateApplicationTitle();
    }
  }

  /**
   * Salva o fluxo corrente em um arquivo.
   * 
   * @param file O arquivo.
   * 
   * @return <code>true</code> sucesso ou <code>false</code> em caso de erro.
   * 
   * @throws ConfigurationException
   */
  private boolean save(final ClientProjectFile file)
    throws ConfigurationException {
    RemoteTask<Void> task = new RemoteTask<Void>() {

      @Override
      protected void handleError(Exception error) {
        if (error instanceof FileLockedException) {
          StandardDialogs.showErrorDialog(getApplicationFrame(), getName(),
            String.format(getString("FlowApplication.msg.error.file.locked"),
              file.getName()));
        }
        else {
          super.handleError(error);
        }
      }

      @Override
      protected void performTask() throws Exception {
        Flow flow = getWorkspace().getGraph().toFlow();
        flow.setName(file.getName());
        ClientProjectFileOutputStream outputStream = null;
        try {
          outputStream = new ClientProjectFileOutputStream(file);
          FlowAlgorithmParser parser = new FlowAlgorithmParser();
          parser.write(outputStream, flow);
        }
        finally {
          if (outputStream != null) {
            try {
              outputStream.close();
            }
            catch (IOException e) {
              e.printStackTrace();
            }
          }
        }
      }
    };
    String message = getI18n("savingFile", FileUtils.joinPath('/', file
      .getSimplePath()));
    if (task.execute(getApplicationFrame(), message, message)) {
      wasChanged = false;
      return true;
    }
    return false;
  }

  /**
   * Atualiza o ttulo da aplicao construtora de fluxo.
   */
  private void updateApplicationTitle() {
    String titleDetails;
    if (currentFilePath == null) {
      titleDetails = getI18n("without_file");
    }
    else {
      ClientProjectFile currentFile = currentFilePath.getFile();
      String currentFilePathAsString = currentFile.getStringPath();
      titleDetails = currentFilePathAsString;
    }
    getApplicationFrame().setTitle(getName() + " - " + titleDetails);
  }

  /**
   * Esconde o painel com a rvore de algoritmos.
   */
  public void hideAlgorithmTreePanel() {
    horizontalSplitPane.setDividerLocation(0.0);
  }

  /**
   * Exibe o painel com a rvore de algoritmos.
   */
  public void showAlgorithmTreePanel() {
    horizontalSplitPane.setDividerLocation(horizontalSplitPane
      .getMinimumDividerLocation());
  }

  /**
   * Listener que percebe as modificao no grafo para que se possa perguntar ao
   * usurio se deseja salvar as alteraes feitas.
   */
  private final class ChangedListener implements GraphListener {
    /**
     * {@inheritDoc}
     */
    public void wasChangedWorkspace(Graph graph) {
    }

    /**
     * {@inheritDoc}
     */
    public void wasElementCreated(Graph graph, GraphElement element) {
      wasChanged = true;
    }

    /**
     * {@inheritDoc}
     */
    public void wasElementDragged(Graph graph, GraphElement element, double tx,
      double ty) {
      wasChanged = true;
    }

    /**
     * {@inheritDoc}
     */
    public void wasElementDragged(Graph graph, GraphElement element,
      Point2D startPoint, Point2D endPoint) {
      wasChanged = true;
    }

    /**
     * {@inheritDoc}
     */
    public void wasElementDropped(Graph graph, GraphElement element,
      Point2D point) {
      wasChanged = true;
    }

    /**
     * {@inheritDoc}
     */
    public void wasElementParametrized(Graph graph, GraphElement element) {
      wasChanged = true;
    }

    /**
     * {@inheritDoc}
     */
    public void wasElementRemoved(Graph graph, GraphElement element) {
      wasChanged = true;
    }

    /**
     * {@inheritDoc}
     */
    public void wasElementSelected(Graph graph, GraphElement element) {
    }

    /**
     * {@inheritDoc}
     */
    public void wasLinkAnchored(Graph graph, GraphLink link,
      GraphFileDescriptor fileDescriptor) {
      wasChanged = true;
    }

    /**
     * {@inheritDoc}
     */
    public void wasLinkIncreased(Graph graph, GraphLink link) {
      wasChanged = true;
    }

    /**
     * {@inheritDoc}
     */
    public void wasLinkStatusChanged(Graph graph, GraphLink link) {
      wasChanged = true;
    }

    /**
     * 
     * @see GraphListener#wasLinkUnanchored(Graph, GraphLink,
     *      GraphFileDescriptor)
     */
    public void wasLinkUnanchored(Graph graph, GraphLink link,
      GraphFileDescriptor fileDescriptor) {
      wasChanged = true;
    }

    /**
     * {@inheritDoc}
     */
    public void wasNodeResized(Graph graph, GraphNode node) {
      wasChanged = true;
    }

    /**
     * {@inheritDoc}
     */
    public void wasParameterSetEnabled(Graph graph, GraphNode node,
      String parameterName, boolean isEnabled) {
    }

    /**
     * {@inheritDoc}
     */
    public void wasParameterSetVisible(Graph graph, GraphNode node,
      String parameterName, boolean isVisible) {
    }

    /**
     * {@inheritDoc}
     */
    public void wasReseted(Graph graph) {
    }
  }
}
