package csbase.algorithms.flows;

import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.awt.image.ImageObserver;
import java.text.MessageFormat;

import tecgraf.javautils.vix.VO;

public final class ImageVO extends VO {

  private double bottomPadding;
  private double height;
  private Image image;
  private double leftPadding;
  private double rightPadding;
  private double topPadding;
  private double width;
  private double x;
  private double y;

  public ImageVO(Image image) {
    setImage(image);
    setWidth(this.image.getWidth(createImageObserver()));
    setHeight(this.image.getWidth(createImageObserver()));
  }

  @Override
  public void callbackRepaint(Graphics2D g) {
    super.callbackRepaint(g);
    AffineTransform matrix = new AffineTransform(1, 0, 0, 1, getX()
      + this.rightPadding, getY() + this.topPadding);
    g.drawImage(this.image, matrix, createImageObserver());
  }

  public Rectangle2D getBounds2D() {
    return new Rectangle2D.Double(getX(), getY(), getWidth(), getHeight());
  }

  public double getHeight() {
    return this.height;
  }

  public double getWidth() {
    return this.width;
  }

  public double getX() {
    return this.x;
  }

  public double getY() {
    return this.y;
  }

  public void setBottomPadding(double bottomPadding) {
    if (bottomPadding <= 0.0) {
      String errorMessage = MessageFormat.format(
        "O parmetro bottomPadding ({0}) tem que ser positivo.", bottomPadding);
      throw new IllegalArgumentException(errorMessage);
    }
    this.bottomPadding = bottomPadding;
    this.height += this.bottomPadding;
  }

  public void setCenterX(double x) {
    this.x = x - (getWidth() / 2.0);
  }

  public void setCenterY(double y) {
    this.y = y - (getHeight() / 2.0);
  }

  public void setLeftPadding(double leftPadding) {
    if (leftPadding <= 0.0) {
      String errorMessage = MessageFormat.format(
        "O parmetro leftPadding ({0}) tem que ser positivo.", leftPadding);
      throw new IllegalArgumentException(errorMessage);
    }
    this.leftPadding = leftPadding;
    this.width += this.leftPadding;
  }

  public void setRightPadding(double rightPadding) {
    if (rightPadding <= 0.0) {
      String errorMessage = MessageFormat.format(
        "O parmetro rightPadding ({0}) tem que ser positivo.", rightPadding);
      throw new IllegalArgumentException(errorMessage);
    }
    this.rightPadding = rightPadding;
    this.width += this.rightPadding;
  }

  public void setTopPadding(double topPadding) {
    if (topPadding <= 0.0) {
      String errorMessage = MessageFormat.format(
        "O parmetro topPadding ({0}) tem que ser positivo.", topPadding);
      throw new IllegalArgumentException(errorMessage);
    }
    this.topPadding = topPadding;
    this.height += this.topPadding;
  }

  public void setX(double x) {
    this.x = x;
  }

  public void setY(double y) {
    this.y = y;
  }

  private ImageObserver createImageObserver() {
    return new ImageObserver() {

      public boolean imageUpdate(Image img, int infoflags, int x, int y,
                                 int width, int height) {
        if ((infoflags & HEIGHT) == HEIGHT) {
          setHeight(height);
          return true;
        }
        if ((infoflags & WIDTH) == WIDTH) {
          setWidth(width);
          return true;

        }

        if ((infoflags & ALLBITS) == WIDTH) {
          return true;
        }

        return false;
      }

    };
  }

  private void setHeight(double height) {
    if (height <= 0.0) {
      String errorMessage = MessageFormat.format(
        "O parmetro height ({0}) tem que ser positivo.", height);
      throw new IllegalArgumentException(errorMessage);
    }
    this.height = height;
  }

  private void setImage(Image image) {
    if (image == null) {
      throw new IllegalArgumentException("O parmetro image est nulo.");
    }
    this.image = image;
  }

  private void setWidth(double width) {
    if (width <= 0.0) {
      String errorMessage = MessageFormat.format(
        "O parmetro width ({0}) tem que ser positivo.", width);
      throw new IllegalArgumentException(errorMessage);
    }
    this.width = width;
  }
}
