package csbase.algorithms.flows.actions;

import java.awt.geom.Point2D;

import javax.swing.ImageIcon;
import javax.swing.KeyStroke;

import csbase.algorithms.flows.FlowApplication;
import csbase.algorithms.flows.graph.Graph;
import csbase.algorithms.flows.graph.GraphElement;
import csbase.algorithms.flows.graph.GraphFileDescriptor;
import csbase.algorithms.flows.graph.GraphLink;
import csbase.algorithms.flows.graph.GraphListener;
import csbase.algorithms.flows.graph.GraphNode;

/**
 * Base para as aes da aplicao Construtor de Fluxos. Implementa um adaptador
 * para o listener do grafo do editor de fluxo e mantm uma referncia para a
 * aplicao.
 * 
 */
public abstract class FlowApplicationAction extends Action implements
  GraphListener {

  /**
   * A aplicao Construtor de Fluxos.
   */
  private FlowApplication application;

  /**
   * Construtor.
   * 
   * @param application A aplicao.
   * @param nameKey A chave utilizada no mecanismo de internacionalizao para
   *        obter o rtulo e a dica usada na ao.
   * @param icon O cone da ao.
   */
  protected FlowApplicationAction(
    FlowApplication application, String nameKey, ImageIcon icon) {
    super(nameKey, icon);
    setApplication(application);
  }

  /**
   * Construtor.
   * 
   * @param application A aplicao.
   * @param nameKey A chave utilizada no mecanismo de internacionalizao para
   *        obter o rtulo e a dica usada na ao.
   * @param icon O cone da ao.
   * @param acceleratorKeyStroke A tecla de acelerao para executar a ao.
   */
  protected FlowApplicationAction(
    FlowApplication application, String nameKey, ImageIcon icon,
    KeyStroke acceleratorKeyStroke) {
    super(nameKey, icon, acceleratorKeyStroke);
    setApplication(application);
  }

  /**
   * {@inheritDoc}
   */
  public void wasElementParametrized(Graph graph, GraphElement element) {
  }

  /**
   * {@inheritDoc}
   */
  public void wasElementCreated(Graph graph, GraphElement element) {
  }

  /**
   * {@inheritDoc}
   */
  public void wasElementDragged(
    Graph graph, GraphElement element, Point2D startPoint, Point2D endPoint) {
  }

  /**
   * {@inheritDoc}
   */
  public void wasElementDragged(
    Graph graph, GraphElement element, double tx, double ty) {
  }

  /**
   * {@inheritDoc}
   */
  public void wasElementDropped(
    Graph graph, GraphElement element, Point2D point) {
  }

  /**
   * {@inheritDoc}
   */
  public void wasElementRemoved(Graph graph, GraphElement element) {
  }

  /**
   * {@inheritDoc}
   */
  public void wasElementSelected(Graph graph, GraphElement element) {
  }

  /**
   * {@inheritDoc}
   */
  public void wasLinkAnchored(
    Graph graph, GraphLink link, GraphFileDescriptor fileDescriptor) {
  }

  /**
   * {@inheritDoc}
   */
  public void wasLinkIncreased(Graph graph, GraphLink link) {
  }

  /**
   * {@inheritDoc}
   */
  public void wasLinkStatusChanged(Graph graph, GraphLink link) {
  }

  /**
   * {@inheritDoc}
   */
  public void wasLinkUnanchored(
    Graph graph, GraphLink link, GraphFileDescriptor fileDescriptor) {
  }

  /**
   * {@inheritDoc}
   */
  public void wasChangedWorkspace(Graph graph) {
  }

  /**
   * {@inheritDoc}
   */
  public void wasNodeResized(Graph graph, GraphNode node) {
  }

  /**
   * {@inheritDoc}
   */
  public void wasParameterSetEnabled(
    Graph graph, GraphNode node, String parameterName, boolean isEnabled) {
  }

  /**
   * {@inheritDoc}
   */
  public void wasParameterSetVisible(
    Graph graph, GraphNode node, String parameterName, boolean isVisible) {
  }

  /**
   * {@inheritDoc}
   */
  public void wasReseted(Graph graph) {
  }

  /**
   * Retorna a instncia da aplicao de fluxos.
   * 
   * @return A instncia.
   */
  protected final FlowApplication getApplication() {
    return this.application;
  }

  /**
   * Atribui a instncia da aplicao de fluxos.
   * 
   * @param application A instncia.
   */
  private void setApplication(FlowApplication application) {
    this.application = application;
    this.application.getWorkspace().getGraph().addGraphListener(this);
  }
}
