package csbase.algorithms.flows.actions;

import java.awt.event.ActionEvent;

import csbase.algorithms.flows.FlowApplication;
import csbase.algorithms.flows.graph.Graph;
import csbase.algorithms.flows.graph.GraphElement;
import csbase.algorithms.flows.graph.GraphNode;
import csbase.client.applications.ApplicationImages;

/**
 * Ao de mostrar o painel com os valores atuais do parmetros de um n do
 * grafo.
 */
public final class ShowParameterValuesAction extends FlowApplicationAction {

  /**
   * Construtor.
   * 
   * @param application A aplicao.
   */
  public ShowParameterValuesAction(FlowApplication application) {
    super(application, ShowParameterValuesAction.class.getName() + ".name",
      ApplicationImages.ICON_VIEWPARAMS_16);
    setEnabled(false);
  }

  /**
   * {@inheritDoc}
   */
  public void actionPerformed(ActionEvent e) {
    getApplication().showParameterValues();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void wasElementCreated(Graph graph, GraphElement element) {
    updateEnabled(graph);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void wasElementRemoved(Graph graph, GraphElement element) {
    updateEnabled(graph);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void wasElementSelected(Graph graph, GraphElement element) {
    updateEnabled(graph);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void wasChangedWorkspace(Graph graph) {
    updateEnabled(graph);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void wasReseted(Graph graph) {
    updateEnabled(graph);
  }

  /**
   * Habilita a ao, caso exista pelo menos um n selecionado no grafo.
   * Desabilita, caso contrrio.
   * 
   * @param graph O grafo.
   */
  private void updateEnabled(Graph graph) {
    for (GraphNode node : graph.getNodeCollection()) {
      if (node.isSelected()) {
        setEnabled(true);
        return;
      }
    }
    setEnabled(false);
  }
}
