package csbase.algorithms.flows.actions;

import java.awt.event.ActionEvent;

import tecgraf.javautils.LNG;
import csbase.algorithms.flows.FlowApplication;
import csbase.algorithms.flows.graph.Graph;
import csbase.algorithms.flows.graph.GraphElement;
import csbase.algorithms.flows.graph.utils.GraphUpdateOperation;
import csbase.client.applications.ApplicationImages;
import csbase.client.util.StandardErrorDialogs;

/**
 * Ao que atualiza os algoritmos pertencentes ao fluxo corrente para utilizar
 * as suas respectivas verses mais atuais (se existirem).
 * 
 * @author isabella
 */
public class UpdateAllVersionsAction extends FlowApplicationAction {
  /**
   * Cria a ao
   * 
   * @param application A aplicao na qual essa ao vai ser adicionada.
   */
  public UpdateAllVersionsAction(FlowApplication application) {
    super(application, UpdateAllVersionsAction.class.getName() + ".name",
      ApplicationImages.ICON_REFRESH_16);
    setEnabled(false);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void actionPerformed(ActionEvent event) {
    Graph graph = getApplication().getWorkspace().getGraph();
    GraphUpdateOperation operation = new GraphUpdateOperation(graph);
    boolean result = operation.update();
    if (!result) {
      StandardErrorDialogs.showErrorDialog(super.getApplication()
        .getApplicationFrame(), LNG.get(UpdateAllVersionsAction.class.getName()
        + ".error"));
    }
    updateEnabledStatus(graph);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void wasElementRemoved(Graph graph, GraphElement element) {
    updateEnabledStatus(graph);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void wasElementCreated(Graph graph, GraphElement element) {
    updateEnabledStatus(graph);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void wasChangedWorkspace(Graph graph) {
    updateEnabledStatus(graph);
  }

  /**
   * @param graph
   */
  private void updateEnabledStatus(Graph graph) {
    setEnabled(GraphUpdateOperation.hasNewerVersion(graph));
  }

}
