package csbase.algorithms.flows.configurator;

import java.awt.Window;

import javax.swing.JComponent;

import csbase.algorithms.flows.Workspace;
import csbase.algorithms.flows.graph.Graph;
import csbase.client.algorithms.AlgorithmConfiguratorView;
import csbase.client.algorithms.parameters.ParameterView;
import csbase.client.algorithms.parameters.validators.ValidationMode;
import csbase.client.algorithms.parameters.validators.ValidationResult;
import csbase.client.algorithms.view.flow.FlowAlgorithmConfiguratorPanel;
import csbase.client.kernel.ClientException;
import csbase.logic.algorithms.AlgorithmConfigurator;
import csbase.logic.algorithms.flows.configurator.FlowAlgorithmConfigurator;

/**
 * Viso de configurador de fluxo de algoritmos.
 * 
 * @author lmoreira
 */
public final class FlowAlgorithmConfiguratorView extends
  AlgorithmConfiguratorView {
  /**
   * O componente principal.
   */
  private final FlowAlgorithmConfiguratorPanel mainComponent;

  /**
   * Cria uma viso de configurador de fluxo.
   * 
   * @param owner A janela que  pai desta viso (No aceita {@code null}).
   * @param configurator O configurador de fluxo.
   * @param mode Modo de visualizao do configurador. No aceita {@code null},
   *        os possveis valores so: {@link ParameterView.Mode#CONFIGURATION}
   *        ou {@link ParameterView.Mode#REPORT}
   * @throws ClientException
   */
  public FlowAlgorithmConfiguratorView(Window owner,
    FlowAlgorithmConfigurator configurator, ParameterView.Mode mode)
    throws ClientException {
    super(owner, configurator, mode);
    mainComponent =
      new FlowAlgorithmConfiguratorPanel(owner, configurator, mode);
  }

  /**
   * {@inheritDoc}
   * 
   * @see FlowAlgorithmConfiguratorPanel#getConfigurator()
   */
  @Override
  public AlgorithmConfigurator getConfigurator() {
    return null == mainComponent ? super.getConfigurator() : mainComponent
      .getConfigurator();
  }

  /**
   * Cria uma viso para configurador de fluxos de algoritmos.
   * 
   * @param window A janela (No aceita {@code null}).
   * @param graph A viso do fluxo (No aceita {@code null}).
   * @param mode Modo de visualizao do configurador. No aceita {@code null},
   *        os possveis valores so: {@link ParameterView.Mode#CONFIGURATION}
   *        ou {@link ParameterView.Mode#REPORT}
   * @throws ClientException
   */
  public FlowAlgorithmConfiguratorView(Window window, Graph graph,
    ParameterView.Mode mode) throws ClientException {
    this(window, new FlowAlgorithmConfigurator(graph.toFlow()), mode);
  }

  /**
   * Obtm a viso do fluxo.
   * 
   * @return A viso do fluxo.
   */
  public Graph getGraph() {
    return null == this.mainComponent ? null : this.mainComponent.getGraph();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void setValidationMode(ValidationMode validationMode) {
    super.setValidationMode(validationMode);
    mainComponent.setValidationMode(validationMode);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public ValidationResult validate(boolean complain) {
    return mainComponent.validate(complain);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public JComponent getMainComponent() {
    return mainComponent;
  }

  /**
   * Retorna o workspace onde o fluxo  desenhado.
   * 
   * @return o workspace onde o fluxo  desenhado.
   */
  public Workspace getWorkspace() {
    return null == this.mainComponent ? null : this.mainComponent
      .getWorkspace();
  }
}
