package csbase.algorithms.flows.filters;

import tecgraf.javautils.vix.Message;
import tecgraf.javautils.vix.TypeMessage;
import csbase.algorithms.flows.Workspace;
import csbase.algorithms.flows.actions.Action;
import csbase.algorithms.flows.graph.Graph;
import csbase.algorithms.flows.graph.actions.GraphAction;
import csbase.algorithms.flows.messages.GetPopupMessage;
import csbase.algorithms.flows.messages.PickGraphMessage;

/**
 * Filtro que adiciona {@link GraphAction aes de menu popup de fluxo de
 * algoritmos} ao menu popup.
 * 
 * @author lmoreira
 */
public abstract class AddGraphPopupActionFilter extends WorkspaceFilter {

  /**
   * Cria a ao.
   * 
   * @param workspace A rea de trabalho (No aceita {@code null}).
   */
  protected AddGraphPopupActionFilter(Workspace workspace) {
    super(workspace);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean msgHandlerVO(TypeMessage msg) {
    handleMessage(msg);
    return super.msgHandlerVO(msg);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean msgHandlerVS(TypeMessage msg) {
    handleMessage(msg);
    return super.msgHandlerVS(msg);
  }

  /**
   * Cria a ao.
   * 
   * @param graph O fluxo de algoritmos (No aceita {@code null}).
   * 
   * @return A ao.
   */
  protected abstract GraphAction createAction(Graph graph);

  /**
   * <p>
   * Trata as {@link Message mensages} relevantes a este filtro e ignora outras
   * mensagens. A nica mensagem relevante  {@link GetPopupMessage}.
   * 
   * @param msg A mensagem.
   */
  private void handleMessage(TypeMessage msg) {
    if (msg instanceof GetPopupMessage) {
      GetPopupMessage getPopupMessage = (GetPopupMessage) msg;
      PickGraphMessage pickGraphMessage = new PickGraphMessage();
      pickGraphMessage.sendVO(this);
      Graph graph = pickGraphMessage.getGraph();
      if (graph != null) {
        Action action = createAction(graph);
        if (action != null) {
          getPopupMessage.addPopupAction(action);
        }
      }
    }
  }
}
