package csbase.algorithms.flows.filters;

import java.awt.geom.Point2D;

import tecgraf.javautils.vix.Message;
import tecgraf.javautils.vix.TypeMessage;

import csbase.algorithms.flows.Workspace;
import csbase.algorithms.flows.actions.Action;
import csbase.algorithms.flows.graph.GraphLink;
import csbase.algorithms.flows.messages.GetPopupMessage;
import csbase.algorithms.flows.messages.PickLinkMessage;

/**
 * Filtro que adiciona aes de menu popup de conexo de fluxo de algoritmos ao 
 * menu popup. 
 *
 * @author lmoreira
 */
public abstract class AddLinkPopupActionFilter extends WorkspaceFilter {

  /**
   * Cria o filtro.
   * 
   * @param workspace A rea de trabalho (No aceita {@code null}).
   */
  protected AddLinkPopupActionFilter(Workspace workspace) {
    super(workspace);
  }

  @Override
  public boolean msgHandlerVO(TypeMessage msg) {
    handleMessage(msg);
    return super.msgHandlerVO(msg);
  }

  @Override
  public boolean msgHandlerVS(TypeMessage msg) {
    handleMessage(msg);
    return super.msgHandlerVS(msg);
  }

  /**
   * Cria a ao.
   * 
   * @param graphLink A conexo de fluxo de algoritmos
   * (No aceita {@code null}).
   * @param point O ponto onde o menu popup foi solicitado
   * (No aceita {@code null}).
   *  
   * @return A ao.
   */
  protected abstract Action createAction(GraphLink graphLink, Point2D point);

  /**
   * <p>Trata as {@link Message mensages} relevantes a este filtro e ignora 
   * outras mensagens. A nica mensagem relevante  {@link GetPopupMessage}.
   * 
   * @param msg A mensagem.
   */
  private void handleMessage(TypeMessage msg) {
    if (msg instanceof GetPopupMessage) {
      GetPopupMessage getPopupMessage = (GetPopupMessage) msg;
      Point2D point = getPopupMessage.getPoint();
      PickLinkMessage pickLinkMessage = 
        new PickLinkMessage(point);
      pickLinkMessage.sendVO(this);
      GraphLink node = pickLinkMessage.getLink();
      if (node != null) {
        Action action = createAction(node, point);
        if (action != null) {
          getPopupMessage.addPopupAction(action);
        }
      }
    }
  }  
}
