package csbase.algorithms.flows.filters;

import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.swing.JComponent;

import csbase.algorithms.flows.Workspace;
import csbase.algorithms.flows.graph.Graph;
import csbase.algorithms.flows.graph.GraphElement;
import csbase.algorithms.flows.messages.GetPopupMessage;
import csbase.algorithms.flows.messages.PickElementMessage;
import csbase.algorithms.flows.messages.PickGraphMessage;
import csbase.algorithms.flows.messages.SelectElementMessage;
import csbase.algorithms.flows.messages.SelectElementsMessage;
import csbase.algorithms.flows.messages.ShowPopupMessage;

public final class PopupFilter extends WorkspaceFilter {
  public PopupFilter(Workspace workspace) {
    super(workspace);
  }

  @Override
  public void callbackButton(Point2D pt, MouseEvent ev) {
    if (ev.isPopupTrigger()) {
      PickElementMessage pickElementMessage = new PickElementMessage(pt);
      if (pickElementMessage.sendVO(this)) {
        GraphElement element = pickElementMessage.getElement();
        if (!element.isSelected()) {
          PickGraphMessage pickGraphMessage = new PickGraphMessage();
          pickGraphMessage.sendVO(this);
          Graph graph = pickGraphMessage.getGraph();
          Set<GraphElement> selectedElements = graph.getSelectedElements();
          if (selectedElements.size() > 1) {
            Set<GraphElement> elementsToSelect = new HashSet<GraphElement>();
            elementsToSelect.addAll(selectedElements);
            elementsToSelect.add(element);
            new SelectElementsMessage(elementsToSelect).sendVO(this);
          }
          else {
            new SelectElementMessage(element, pt).sendVO(this);
          }
        }
      }
      GetPopupMessage getPopupMessage = new GetPopupMessage(pt);
      getPopupMessage.sendVO(this);
      List<JComponent> componentList = getPopupMessage.getPopupComponentList();
      if (!componentList.isEmpty()) {
        ShowPopupMessage showPopupMessage = 
          new ShowPopupMessage(pt, componentList);
        showPopupMessage.sendVS(this);
      }
      return;
    }
    super.callbackButton(pt, ev);
  }
}
