package csbase.algorithms.flows.graph;

import csbase.client.algorithms.parameters.validators.ValidationResult;

/**
 * Resultado da validao de um elemento especfico do tipo E.
 * 
 * @param <E> Tipo do elemento validado.
 */
public class ElementValidationResult<E> extends ValidationResult {

  /**
   * O elemento validado.
   */
  protected E element;

  /**
   * Construtor.
   * 
   * @param errorMessage A mensagem de erro (No aceita {@code null}).
   * @param element O elemento validado.
   */
  public ElementValidationResult(String errorMessage, E element) {
    super(errorMessage);
    this.element = element;
  }

  /**
   * Retorna o elemento validado.
   * 
   * @return element O elemento.
   */
  public E getElement() {
    return element;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public String toString() {
    if (element != null && getErrorMessage() != null) {
      return "(" + element.toString() + ") " + getErrorMessage();
    }
    return super.toString();
  }
}
