package csbase.algorithms.flows.graph.actions;

import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;
import java.awt.geom.Point2D;
import java.text.MessageFormat;

import tecgraf.javautils.LNG;
import tecgraf.javautils.vix.TypeVS;
import csbase.algorithms.flows.Workspace;
import csbase.algorithms.flows.actions.Action;
import csbase.algorithms.flows.filters.AddNodePopupActionFilter;
import csbase.algorithms.flows.filters.WorkspaceFilter;
import csbase.algorithms.flows.graph.GraphNode;
import csbase.algorithms.flows.graph.ParametersTransferable;
import csbase.algorithms.flows.graph.utils.CopyAndPasteOperation;
import csbase.algorithms.flows.graph.utils.OperationStatus;
import csbase.algorithms.flows.messages.ClipboardIsEmptyMessage;
import csbase.algorithms.flows.messages.PasteMessage;
import csbase.client.applications.ApplicationImages;
import csbase.client.util.StandardErrorDialogs;

/**
 * Ao de menu popup que colar os valores dos parmetros de um n de fluxo de
 * algoritmos.
 * 
 * @author lmoreira
 */
public final class PasteParameterValuesAction extends GraphElementAction {

  /**
   * Cria a ao.
   * 
   * @param node O n (No aceita {@code null}).
   */
  public PasteParameterValuesAction(GraphNode node) {
    super(node, PasteParameterValuesAction.class.getName() + ".name",
      ApplicationImages.ICON_PASTE_16);
    boolean clipboardIsEmpty = isClipboardEmpty(node);
    setEnabled(clipboardIsEmpty);
  }

  /**
   * Testa se tem algo armazenado no clipboard (algo que tenha sido copiado
   * previamente) de um determinado VS
   * 
   * @param vs VS a ser testado
   * @return Verdadeiro se o clipboard est vazio ou falso caso contrrio
   */
  private boolean isClipboardEmpty(TypeVS vs) {
    ClipboardIsEmptyMessage clipboardIsEmptyMessage =
      new ClipboardIsEmptyMessage();
    clipboardIsEmptyMessage.sendVS(vs);
    Boolean clipboardIsEmpty = clipboardIsEmptyMessage.clipboardIsEmpty();
    if (clipboardIsEmpty == null) {
      clipboardIsEmpty = false;
    }
    return clipboardIsEmpty;
  }

  /**
   * {@inheritDoc}
   */
  public void actionPerformed(ActionEvent event) {
    GraphNode node = (GraphNode) getElement();
    PasteMessage pasteMessage = new PasteMessage();
    pasteMessage.sendVS(node);
    Transferable transferable = pasteMessage.getTransferable();
    if (transferable instanceof ParametersTransferable) {
      ParametersTransferable parametersTransferable =
        (ParametersTransferable) transferable;
      CopyAndPasteOperation operation =
        new CopyAndPasteOperation(parametersTransferable, node);
      OperationStatus operationStatus = operation.getConfirmation();
      if (operationStatus.isConfirmed()) {
        boolean result = operation.pasteParameters();
        if (result) {
          if (!(operationStatus.getWarnings().isEmpty() && operationStatus
            .getErrors().isEmpty())) {
            node.askForParameterValues();
          }
        }
        else {
          StandardErrorDialogs.showErrorDialog(node.getGraph()
            .getParentWindow(), MessageFormat.format(LNG
            .get(PasteParameterValuesAction.class.getName() + ".error"), node
            .getAlgorithmName()));
        }
      }
    }

  }

  /**
   * Cria o filtro que cadastra a ao no menu popup.
   * 
   * @param workspace A rea de trabalho (No aceita {@code null}).
   * 
   * @return O filtro.
   */
  public static WorkspaceFilter createFilter(Workspace workspace) {
    if (workspace == null) {
      throw new IllegalArgumentException("O parmetro workspace est nulo.");
    }
    return new AddNodePopupActionFilter(workspace) {

      @Override
      protected Action createAction(GraphNode node, Point2D point) {
        if (!node.isWellFormed()) {
          return null;
        }
        return new PasteParameterValuesAction(node);
      }
    };
  }
}
