package csbase.algorithms.flows.graph.actions;

import java.awt.event.ActionEvent;
import java.awt.geom.Point2D;

import csbase.algorithms.flows.Workspace;
import csbase.algorithms.flows.actions.Action;
import csbase.algorithms.flows.filters.AddNodePopupActionFilter;
import csbase.algorithms.flows.filters.WorkspaceFilter;
import csbase.algorithms.flows.graph.GraphNode;
import csbase.client.applications.ApplicationImages;

/**
 * Ao de menu popup que exibe os valores dos parmetros de um n de fluxo de 
 * algoritmos.
 *
 * @author lmoreira
 */
public final class ShowParameterValuesAction extends GraphElementAction {
  /**
   * Cria a ao.
   * 
   * @param node O n (No aceita {@code null}).
   */
  public ShowParameterValuesAction(GraphNode node) {
    super(node, 
      ShowParameterValuesAction.class.getName()+".name", 
      ApplicationImages.ICON_VIEWPARAMS_16);
  }

  public void actionPerformed(ActionEvent e) {
    GraphNode node = (GraphNode)getElement();
    node.showParameterValues();
  }
  
  /**
   * Cria o filtro que cadastra a ao no menu popup.
   * 
   * @param workspace A rea de trabalho (No aceita {@code null}).
   * 
   * @return O filtro.
   */
  public static final WorkspaceFilter createFilter(Workspace workspace) {
    if (workspace == null) {
      throw new IllegalArgumentException("O parmetro workspace est nulo.");     }
    return new AddNodePopupActionFilter(workspace) {      
      @Override
      protected Action createAction(GraphNode node, Point2D point) {
        return new ShowParameterValuesAction(node);
      }
    };
  }  
}
