package csbase.algorithms.flows.graph.actions;

import java.awt.event.ActionEvent;
import java.awt.geom.Point2D;
import java.text.MessageFormat;

import tecgraf.javautils.LNG;
import csbase.algorithms.flows.Workspace;
import csbase.algorithms.flows.actions.Action;
import csbase.algorithms.flows.filters.AddNodePopupActionFilter;
import csbase.algorithms.flows.filters.WorkspaceFilter;
import csbase.algorithms.flows.graph.GraphNode;
import csbase.algorithms.flows.graph.utils.GraphNodeUpdateOperation;
import csbase.client.applications.ApplicationImages;

/**
 * Ao de menu popup que atualiza a verso do algoritmo do n escolhido para
 * utilizar a sua verso mais atual.
 * 
 * @author isabella
 */
public class UpdateVersionAction extends GraphElementAction {

  /**
   * Cria a ao
   * 
   * @param node N a ser atualizado
   */
  protected UpdateVersionAction(GraphNode node) {
    super(node, UpdateVersionAction.class.getName() + ".name",
      ApplicationImages.ICON_REFRESH_16);
    setEnabled(GraphNodeUpdateOperation.hasNewerVersion(node));
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void actionPerformed(ActionEvent event) {
    GraphNode node = (GraphNode) getElement();
    GraphNodeUpdateOperation operation = new GraphNodeUpdateOperation(node);
    boolean result = operation.update();
    if (!result) {
      showErrorDialog(MessageFormat.format(LNG.get(UpdateVersionAction.class
        .getName()
        + ".error"), node.getAlgorithmName()));
    }
  }

  /**
   * Cria o filtro que cadastra a ao no menu popup.
   * 
   * @param workspace A rea de trabalho (No aceita {@code null}).
   * 
   * @return O filtro.
   */
  public static WorkspaceFilter createFilter(Workspace workspace) {
    return new AddNodePopupActionFilter(workspace) {

      @Override
      protected Action createAction(GraphNode graphNode, Point2D point) {
        return new UpdateVersionAction(graphNode);
      }
    };
  }
}
