package csbase.algorithms.flows.zoom;

import javax.swing.AbstractSpinnerModel;

/**
 * <p>
 * Modelo de zoom para spinner.
 * </p>
 * 
 * <p>
 * Classe que adequa a modelos de zoom para ser usados como modelos para
 * spinners.
 * </p>
 *
 * @author lmoreira.
 */
public final class ZoomSpinnerModel extends AbstractSpinnerModel {
  private final ZoomModel zoomModel;

  /**
   * Cria um modelo de zoom para spinner.
   *
   * @param zoomModel O modelo de zoom.
   */
  public ZoomSpinnerModel(ZoomModel zoomModel) {
    this.zoomModel = zoomModel;
    this.zoomModel.addListener(new ZoomSpinnerModelListener());
  }

  public Object getNextValue() {
    return toSpinnerValue(this.zoomModel.getNextUnitValue());
  }

  public Object getPreviousValue() {
    return toSpinnerValue(this.zoomModel.getPreviousUnitValue());
  }

  public Object getValue() {
    return toSpinnerValue(this.zoomModel.getValue());
  }

  public void setValue(Object value) {
    this.zoomModel.setValue(fromSpinnerValue(value));
  }

  /**
   * Converte o valor vindo do spinner para o valor utilizado no modelo de
   * zoom.
   *
   * @param zoomValue O valor do spinner.
   *
   * @return .
   */
  private static double fromSpinnerValue(Object zoomValue) {
    String stringValue = (String)zoomValue;
    return Integer.parseInt(stringValue)/100.0;
  }

  /**
   * Converte o valor do modelo de zoom para o valor utilizado no spinner.
   *
   * @param zoomValue O valor de zoom.
   *
   * @return O valor convertido.
   */
  private static Object toSpinnerValue(double zoomValue) {
    int intValue = (int)Math.round((zoomValue*100));
    return Integer.toString(intValue);
  }

  /**
   * O listener de zoom que comunica ao modelo de zoom para spinners sobre
   * mudanas de zoom.
   */
  private class ZoomSpinnerModelListener implements ZoomListener {
    public void wasChanged(ZoomModel model) {
      fireStateChanged();
    }
  }
}
