/*
 * $Id: ClientKeyStrokeUtil.java 102600 2010-03-05 19:10:36Z clinio $
 */
package csbase.client;

import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;

import javax.swing.KeyStroke;

/**
 * Tradutor de teclas.
 * 
 * @author Tecgraf/PUC-Rio
 */
public enum ClientKeyStrokeUtil {
  /** Tecla */
  A('a', KeyEvent.VK_A),

  /** Tecla */
  B('b', KeyEvent.VK_B),

  /** Tecla */
  C('c', KeyEvent.VK_C),

  /** Tecla */
  D('d', KeyEvent.VK_D),

  /** Tecla */
  E('e', KeyEvent.VK_E),

  /** Tecla */
  F('f', KeyEvent.VK_F),

  /** Tecla */
  G('g', KeyEvent.VK_G),

  /** Tecla */
  H('h', KeyEvent.VK_H),

  /** Tecla */
  I('i', KeyEvent.VK_I),

  /** Tecla */
  J('j', KeyEvent.VK_J),

  /** Tecla */
  K('k', KeyEvent.VK_K),

  /** Tecla */
  L('l', KeyEvent.VK_L),

  /** Tecla */
  M('m', KeyEvent.VK_M),

  /** Tecla */
  N('n', KeyEvent.VK_N),

  /** Tecla */
  O('o', KeyEvent.VK_O),

  /** Tecla */
  P('p', KeyEvent.VK_P),

  /** Tecla */
  Q('q', KeyEvent.VK_Q),

  /** Tecla */
  R('r', KeyEvent.VK_R),

  /** Tecla */
  S('s', KeyEvent.VK_S),

  /** Tecla */
  T('t', KeyEvent.VK_T),

  /** Tecla */
  U('u', KeyEvent.VK_U),

  /** Tecla */
  V('v', KeyEvent.VK_V),

  /** Tecla */
  X('x', KeyEvent.VK_X),

  /** Tecla */
  Y('y', KeyEvent.VK_Y),

  /** Tecla */
  W('w', KeyEvent.VK_W),

  /** Tecla */
  Z('z', KeyEvent.VK_Z),

  /** Tecla */
  N0('0', KeyEvent.VK_0),

  /** Tecla */
  N1('1', KeyEvent.VK_1),

  /** Tecla */
  N2('2', KeyEvent.VK_2),

  /** Tecla */
  N3('3', KeyEvent.VK_3),

  /** Tecla */
  N4('4', KeyEvent.VK_4),

  /** Tecla */
  N5('5', KeyEvent.VK_5),

  /** Tecla */
  N6('6', KeyEvent.VK_6),

  /** Tecla */
  N7('7', KeyEvent.VK_7),

  /** Tecla */
  N8('8', KeyEvent.VK_8),

  /** Tecla */
  N9('9', KeyEvent.VK_9);

  /**
   * Nome
   */
  final private char keyName;

  /**
   * Tecla
   */
  final private int key;

  /**
   * Construtor
   * 
   * @param keyName nome
   * @param key tecla
   */
  private ClientKeyStrokeUtil(final char keyName, final int key) {
    this.keyName = keyName;
    this.key = key;
  }

  /**
   * Pega uma tecla com base no texto (primeiro caracter)
   * 
   * @param text o texto
   * @return a tecla
   */
  static private int getKey(final String text) {
    if (text == null || text.trim().isEmpty()) {
      throw new IllegalArgumentException("Null or empty text detected!");
    }
    final char kn = text.toLowerCase().charAt(0);
    for (ClientKeyStrokeUtil tr : values()) {
      if (kn == tr.keyName) {
        return tr.key;
      }
    }
    throw new IllegalArgumentException("Unhandled text detected!");
  }

  /**
   * Retorna o key-stroke com ctrl.
   * 
   * @param text o texto
   * @return o stroke
   */
  static public KeyStroke getCtrlAltKeyStroke(final String text) {
    final int k = getKey(text);
    final int mod = InputEvent.ALT_DOWN_MASK | InputEvent.CTRL_DOWN_MASK;
    return KeyStroke.getKeyStroke(k, mod);
  }

  /**
   * Retorna o key-stroke com ctrl.
   * 
   * @param text o texto
   * @return o stroke
   */
  static public KeyStroke getCtrlKeyStroke(final String text) {
    final int k = getKey(text);
    return KeyStroke.getKeyStroke(k, InputEvent.CTRL_DOWN_MASK);
  }

  /**
   * Retorna o key-stroke com alt.
   * 
   * @param text o texto
   * @return o stroke
   */
  static public KeyStroke getAltKeyStroke(final String text) {
    final int k = getKey(text);
    return KeyStroke.getKeyStroke(k, InputEvent.ALT_DOWN_MASK);
  }
}
