package csbase.client.algorithms.commands.view;

import java.awt.GridBagLayout;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.GBC;
import csbase.client.algorithms.AlgorithmConfiguratorFactory;
import csbase.client.algorithms.AlgorithmConfiguratorView;
import csbase.client.algorithms.view.simple.SimpleAlgorithmConfiguratorPanel;
import csbase.client.desktop.DesktopComponentFrame;
import csbase.client.kernel.ClientException;
import csbase.logic.CommandInfo;
import csbase.logic.algorithms.AlgorithmConfigurator;

/**
 * Dilogo de visualizao de comandos simples.
 * 
 * @author Tecgraf / PUC-Rio
 */
final class SimpleAlgorithmCommandView extends AbstractAlgorithmCommandView {

  /**
   * Construtor
   * 
   * @param index identificador da view. Usado para garantir a unicidade.
   * @param owner A janela pai deste dilogo (No aceita {@code null}).
   * @param command O comando (No aceita {@code null}).
   * @param configurator O configurador a ter seus parmetros apresentados.
   *        Deve-se ignorar o configurador do comando pois pode ser diferente
   *        deste. Isso ocorrerira no caso de estar representando um n de um
   *        comando de fluxo (No aceita {@code null}).
   * 
   * @throws ClientException Erro ao obter informaes sobre o comando.
   */
  public SimpleAlgorithmCommandView(final Object index,
    final DesktopComponentFrame owner, final CommandInfo command,
    final AlgorithmConfigurator configurator) throws ClientException {
    super(index, owner, command, configurator);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected ConfigurationReportTab createConfigurationReportTab(
    final AlgorithmConfigurator configurator) throws ClientException {
    return new ParametersTab(configurator);
  }

  /**
   * Tab para visualizar os parmetros do comando. Contm um painel do tipo
   * {@link SimpleAlgorithmConfiguratorPanel}.
   */
  private final class ParametersTab extends ConfigurationReportTab {

    /**
     * Ttulo.
     */
    private final String title;

    /**
     * Construtor
     * 
     * @param configurator configurador
     * @throws ClientException se houver problemas na criao do configurador.
     */
    ParametersTab(final AlgorithmConfigurator configurator)
      throws ClientException {

      String algorithmName, algorithmVersion;
      if (null == configurator) {
        algorithmName = "";
        algorithmVersion = "";
      }
      else {
        algorithmName = configurator.getAlgorithmName();
        algorithmVersion = configurator.getAlgorithmVersion().toString();

        this.setLayout(new GridBagLayout());

        AlgorithmConfiguratorFactory configuratorFactory =
          AlgorithmConfiguratorFactory.getInstance();
        AlgorithmConfiguratorView view =
          configuratorFactory.createReportView(owner, configurator);
        if (view != null) {
          this.add(view.getMainComponent(), new GBC(0, 0).both());
        }
      }

      this.title =
        String.format(LNG.get(SimpleAlgorithmCommandView.class.getSimpleName()
          + ".tab.params.title.template"), algorithmName, algorithmVersion);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected String getTitle() {
      return title;
    }
  }
}
