package csbase.client.algorithms.parameters;

import java.awt.Color;
import java.awt.Font;
import java.util.HashMap;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JTextField;
import javax.swing.border.Border;
import javax.swing.text.JTextComponent;

import csbase.client.algorithms.parameters.ParameterView.Mode;

public class ComponentProperties {

  private static final Map<Mode, ComponentProperties> COMPONENT_PROPERTIES;

  private Color bgColor;
  private Color fgColor;
  private Border border;
  private Font font;
  private boolean isOpaque;
  private boolean isEditable;

  static {
    COMPONENT_PROPERTIES = new HashMap<Mode, ComponentProperties>();
    for (Mode mode : Mode.values()) {
      COMPONENT_PROPERTIES.put(mode, createComponentProperties(mode));
    }
  }

  private ComponentProperties(Color bgColor, Color fgColor, Border border,
    Font font, boolean isOpaque, boolean isEditable) {
    super();
    this.bgColor = bgColor;
    this.fgColor = fgColor;
    this.border = border;
    this.font = font;
    this.isOpaque = isOpaque;
    this.isEditable = isEditable;
  }

  public static final ComponentProperties getInstance(Mode mode) {
    return COMPONENT_PROPERTIES.get(mode);
  }

  public static final void setProperties(JTextComponent component, Mode mode,
    boolean setBorder) {
    ComponentProperties properties = getInstance(mode);
    component.setEditable(properties.isEditable());
    setProperties(component, properties, setBorder);
  }

  public static final void setProperties(JComponent component, Mode mode,
    boolean setBorder) {
    ComponentProperties properties = getInstance(mode);
    setProperties(component, properties, setBorder);
  }

  public static final void setProperties(JComponent component,
    ComponentProperties properties, boolean setBorder) {
    component.setBackground(properties.getBackground());
    component.setForeground(properties.getForeGround());
    component.setFont(properties.getFont());
    component.setOpaque(properties.isOpaque());
    if (setBorder) {
      component.setBorder(properties.getBorder());
    }
  }

  public Color getBackground() {
    return bgColor;
  }

  public Color getForeGround() {
    return fgColor;
  }

  public Border getBorder() {
    return border;
  }

  public Font getFont() {
    return font;
  }

  public boolean isOpaque() {
    return isOpaque;
  }

  public boolean isEditable() {
    return isEditable;
  }

  private static ComponentProperties createComponentProperties(Mode mode) {
    JTextField template = new JTextField();
    template.setEditable(Mode.CONFIGURATION == mode);

    Color bgColor = template.getBackground();
    Color fgColor = template.getForeground();
    Font font = template.getFont();
    boolean isOpaque = template.isOpaque();
    boolean isEditable = template.isEditable();
    
    Color borderColor =
      Mode.CONFIGURATION == mode ? 
        new Color(0x7a, 0x8a, 0x99) : 
        new Color(0xb8, 0xcf, 0xe5);
    Border border = BorderFactory.createLineBorder(borderColor);

    return new ComponentProperties(bgColor, fgColor, border, font, isOpaque,
      isEditable);
  }
}
