package csbase.client.algorithms.parameters;

import java.awt.Window;

import javax.swing.JComponent;
import javax.swing.JTextField;

import csbase.client.algorithms.parameters.validators.DoubleParameterValidator;
import csbase.logic.algorithms.parameters.DoubleParameter;
import csbase.logic.algorithms.parameters.DoubleParameterListener;

/**
 * Viso para o {@link DoubleParameter Parmetro do Tipo Real}.
 * 
 * @author lmoreira
 */
public final class DoubleParameterView extends SimpleParameterView<Double> {

  /**
   * Cria a viso em modo {@link ParameterView.Mode#CONFIGURATION}.
   * 
   * @param window NO EST SENDO UTILIZADO. Existe somente para manter
   *        compatibilidade com o WebSintesi.
   * @param parameter O parmetro que  manipulador por esta viso (No aceita
   *        {@code null}).
   * 
   * @deprecated para manter compatibilidade com o WebSintesi
   */
  @Deprecated
  public DoubleParameterView(Window window, DoubleParameter parameter) {
    this(parameter, Mode.CONFIGURATION);
  }

  /**
   * Cria a viso.
   * 
   * @param parameter O parmetro que  manipulador por esta viso (No aceita
   *        {@code null}).
   * @param mode Modo de visualizao. No aceita {@code null}, os possveis
   *        valores so: {@link ParameterView.Mode#CONFIGURATION} ou
   *        {@link ParameterView.Mode#REPORT}.
   */
  public DoubleParameterView(DoubleParameter parameter, Mode mode) {
    super(parameter, mode);
    parameter.addDoubleParameterListener(new DoubleParameterListener() {

      public void minimumWasChanged(DoubleParameter parameter) {
        DoubleParameterValidator validator =
          (DoubleParameterValidator) getParameterValidator();
        validator.setMinimum(parameter.getMinimum(), parameter
          .isMinimumIncluded());
      }

      public void maximumWasChanged(DoubleParameter parameter) {
        DoubleParameterValidator validator =
          (DoubleParameterValidator) getParameterValidator();
        validator.setMaximum(parameter.getMaximum(), parameter
          .isMaximumIncluded());
      }

    });
    updateCapabilityView();
    updateVisibilyView();
  }

  /**
   * @see csbase.client.algorithms.parameters.ParameterView#getParameter()
   */
  @Override
  public DoubleParameter getParameter() {
    return (DoubleParameter) super.getParameter();
  }

  @Override
  protected DoubleParameterValidator createParameterValidator() {
    return new DoubleParameterValidator(getParameter().isOptional(),
      getParameter().getMaximum(), getParameter().isMaximumIncluded(),
      getParameter().getMinimum(), getParameter().isMinimumIncluded());
  }

  @Override
  protected JComponent createConfigurationComponent(Object... componentArgs) {
    return new DoubleConfigurationParameter();
  }

  @Override
  protected JComponent createReportComponent(Object... componentArgs) {
    return new DoubleReportParameter();
  }

  @Override
  protected void updateViewContents() {
    ((IDoubleParameterComponent) getComponent()).updateViewContents();
  }

  private interface IDoubleParameterComponent {
    /**
     * Atualiza o contedo exibido pela viso.
     */
    void updateViewContents();
  }

  private final class DoubleReportParameter extends JTextField implements
    IDoubleParameterComponent {

    DoubleReportParameter() {
      setToolTipText(getParameter().getDescription());
      ComponentProperties.setProperties(this, Mode.REPORT, true);
      super.setEditable(false);
      updateViewContents();
    }

    /**
     * @see csbase.client.algorithms.parameters.DoubleParameterView.IDoubleParameterComponent#updateViewContents()
     */
    public void updateViewContents() {
      Double value = getParameter().getValue();
      setText(null == value ? " " : value.toString());
    }

    /**
     * Este mtodo no faz nada por que este componente no pode ser editvel.
     * 
     * @see javax.swing.text.JTextComponent#setEditable(boolean)
     */
    @Override
    public void setEditable(boolean enable) {
    }
  }

  private final class DoubleConfigurationParameter extends DoubleTextField
    implements IDoubleParameterComponent {

    DoubleConfigurationParameter() {
      super(new AbstractNumberTextFieldModel<Double>() {
        public void setValue(Double value) {
          getParameter().setValue(value);
          fireChangedValue();
        }

        public Double getValue() {
          return getParameter().getValue();
        }
      });

      setToolTipText(getParameter().getDescription());
    }

    /**
     * @see csbase.client.algorithms.parameters.DoubleParameterView.IDoubleParameterComponent#updateViewContents()
     */
    public void updateViewContents() {
      updateView();
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean fillVerticalSpace() {
    return false;
  }
}
