package csbase.client.algorithms.parameters;

import java.awt.Window;

import javax.swing.JComponent;
import javax.swing.JTextField;

import csbase.client.algorithms.parameters.validators.IntegerParameterValidator;
import csbase.logic.algorithms.parameters.IntegerParameter;
import csbase.logic.algorithms.parameters.IntegerParameterListener;

/**
 * Viso para o {@link IntegerParameter Parmetro do Tipo Inteiro}.
 * 
 * @author lmoreira
 */
public final class IntegerParameterView extends SimpleParameterView<Integer> {

  /**
   * Cria uma viso em modo {@link ParameterView.Mode#CONFIGURATION}.
   * 
   * @param window NO EST SENDO UTILIZADO. Existe somente para manter
   *        compatibilidade com o WebSintesi.
   * @param parameter O parmetro (No aceita {@code null}).
   * 
   * @deprecated para manter compatibilidade com o WebSintesi
   */
  @Deprecated
  public IntegerParameterView(Window window, IntegerParameter parameter) {
    this(parameter, Mode.CONFIGURATION);
  }

  /**
   * Cria a viso.
   * 
   * @param parameter O parmetro que  manipulado por esta viso (No aceita
   *        {@code null}).
   * @param mode Modo de visualizao. No aceita {@code null}, os possveis
   *        valores so: {@link ParameterView.Mode#CONFIGURATION} ou
   *        {@link ParameterView.Mode#REPORT}.
   */
  public IntegerParameterView(IntegerParameter parameter, Mode mode) {
    super(parameter, mode);

    parameter.addIntegerParameterListener(new IntegerParameterListener() {
      public void minimumWasChanged(IntegerParameter parameter) {
        IntegerParameterValidator validator =
          (IntegerParameterValidator) getParameterValidator();
        validator.setMinimum(parameter.getMinimum());
      }

      public void maximumWasChanged(IntegerParameter parameter) {
        IntegerParameterValidator validator =
          (IntegerParameterValidator) getParameterValidator();
        validator.setMaximum(parameter.getMaximum());
      }
    });
    updateCapabilityView();
    updateVisibilyView();
  }

  /**
   * @see csbase.client.algorithms.parameters.ParameterView#getParameter()
   */
  @Override
  public IntegerParameter getParameter() {
    return (IntegerParameter) super.getParameter();
  }

  @Override
  protected IntegerParameterValidator createParameterValidator() {
    return new IntegerParameterValidator(getParameter().isOptional(),
      getParameter().getMaximum(), getParameter().getMinimum());
  }

  @Override
  protected JComponent createConfigurationComponent(Object... componentArgs) {
    return new IntegerConfigurationParameter();
  }

  @Override
  protected JComponent createReportComponent(Object... componentArgs) {
    return new IntegerReportParameter();
  }

  @Override
  protected void updateViewContents() {
    ((IIntegerParameterComponent) getComponent()).updateViewContents();
  }

  private interface IIntegerParameterComponent {
    /**
     * Atualiza o contedo exibido pela viso.
     */
    void updateViewContents();
  }

  private final class IntegerReportParameter extends JTextField implements
    IIntegerParameterComponent {

    IntegerReportParameter() {
      setToolTipText(getParameter().getDescription());
      ComponentProperties.setProperties(this, Mode.REPORT, true);
      super.setEditable(false);
      updateViewContents();
    }

    /**
     * @see csbase.client.algorithms.parameters.IntegerParameterView.IIntegerParameterComponent#updateViewContents()
     */
    public void updateViewContents() {
      Integer value = getParameter().getValue();
      setText(null == value ? " " : value.toString());
    }

    /**
     * Este mtodo no faz nada por que este componente no pode ser editvel.
     * 
     * @see javax.swing.text.JTextComponent#setEditable(boolean)
     */
    @Override
    public void setEditable(boolean enable) {
    }
  }

  private final class IntegerConfigurationParameter extends IntegerTextField
    implements IIntegerParameterComponent {

    IntegerConfigurationParameter() {
      super(new AbstractNumberTextFieldModel<Integer>() {
        public void setValue(Integer value) {
          getParameter().setValue(value);
          fireChangedValue();
        }

        public Integer getValue() {
          return getParameter().getValue();
        }
      });

      setToolTipText(getParameter().getDescription());
    }

    /**
     * @see csbase.client.algorithms.parameters.IntegerParameterView.IIntegerParameterComponent#updateViewContents()
     */
    public void updateViewContents() {
      updateView();
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean fillVerticalSpace() {
    return false;
  }
}
