package csbase.client.algorithms.parameters;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.List;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import tecgraf.javautils.gui.GBC;
import csbase.client.algorithms.parameters.validators.ListParameterValidator;
import csbase.client.algorithms.parameters.validators.SimpleParameterValidator;
import csbase.client.util.ClientUtilities;
import csbase.logic.algorithms.parameters.ListParameter;

/**
 * @param <E> Tipo do elemento contido na lista.
 * 
 * @author Tecgraf / PUC-Rio
 */
public abstract class ListParameterView<E> extends SimpleParameterView<List<E>> {

  /**
   * Cria a viso em modo {@link ParameterView.Mode#CONFIGURATION}.
   * 
   * @param window NO EST SENDO UTILIZADO. Existe somente para manter
   *        compatibilidade com o WebSintesi.
   * @param parameter O parmetro que  manipulado por esta viso (No aceita
   *        {@code null}).
   * @param parser
   * @param formatter
   * 
   * @deprecated para manter compatibilidade com o WebSintesi
   */
  @Deprecated
  protected ListParameterView(Window window, ListParameter<E> parameter,
    Parser<E> parser, Formatter<E> formatter) {
    this(parameter, Mode.CONFIGURATION, parser, formatter);
  }

  /**
   * Cria a viso.
   * 
   * @param parameter O parmetro que  manipulado por esta viso (No aceita
   *        {@code null}).
   * @param mode Modo de visualizao. No aceita {@code null}, os possveis
   *        valores so: {@link ParameterView.Mode#CONFIGURATION} ou
   *        {@link ParameterView.Mode#REPORT}.
   * @param parser
   * @param formatter
   */
  protected ListParameterView(ListParameter<E> parameter, Mode mode,
    Parser<E> parser, Formatter<E> formatter) {
    this(parameter, mode, parser, formatter, null);
  }

  /**
   * Cria a viso em modo {@link ParameterView.Mode#CONFIGURATION}.
   * 
   * @param window NO EST SENDO UTILIZADO. Existe somente para manter
   *        compatibilidade com o WebSintesi.
   * @param parameter O parmetro que  manipulado por esta viso (No aceita
   *        {@code null}).
   * @param parser
   * @param formatter
   * @param textField
   * 
   * @deprecated para manter compatibilidade com o WebSintesi
   */
  @Deprecated
  protected ListParameterView(Window window, ListParameter<E> parameter,
    Parser<E> parser, Formatter<E> formatter, JTextField textField) {
    this(parameter, Mode.CONFIGURATION, parser, formatter, textField);
  }

  /**
   * Cria a viso.
   * 
   * @param parameter O parmetro que  manipulado por esta viso (No aceita
   *        {@code null}).
   * @param mode Modo de visualizao. No aceita {@code null}, os possveis
   *        valores so: {@link ParameterView.Mode#CONFIGURATION} ou
   *        {@link ParameterView.Mode#REPORT}.
   * @param parser
   * @param formatter
   * @param textField
   */
  protected ListParameterView(ListParameter<E> parameter, Mode mode,
    Parser<E> parser, Formatter<E> formatter, JTextField textField) {
    super(parameter, mode, parser, formatter, textField);
    updateCapabilityView();
    updateVisibilyView();
  }

  /**
   * @see csbase.client.algorithms.parameters.ParameterView#getParameter()
   */
  @Override
  public ListParameter<E> getParameter() {
    return (ListParameter<E>) super.getParameter();
  }

  @Override
  protected final ListParameterValidator<E> createParameterValidator() {
    return new ListParameterValidator<E>(createItemValidator());
  }

  @Override
  protected JComponent createConfigurationComponent(Object... componentArgs) {
    Parser<E> parser = (Parser<E>) componentArgs[0];
    Formatter<E> formatter = (Formatter<E>) componentArgs[1];
    JTextField textField = (JTextField) componentArgs[2];
    return new ListConfigurationParameter(parser, formatter, textField);
  }

  @Override
  protected JComponent createReportComponent(Object... componentArgs) {
    Formatter<E> formatter = (Formatter<E>) componentArgs[1];
    return new ListReportParameter(formatter);
  }

  @Override
  protected void updateViewContents() {
    ((IListParameterComponent) getComponent()).updateViewContents();
  }

  protected abstract SimpleParameterValidator<E> createItemValidator();

  protected static interface Formatter<E> {
    String format(E element);
  }

  protected static interface Parser<E> {
    E parse(String text);
  }

  private interface IListParameterComponent {
    /**
     * Atualiza o contedo exibido pela viso.
     */
    void updateViewContents();
  }

  private final class ListReportParameter extends JPanel implements
    IListParameterComponent {

    private final Formatter<E> formatter;

    private JTextArea textArea;

    ListReportParameter(Formatter<E> formatter) {
      this.formatter = formatter;

      setLayout(new GridLayout());

      textArea = new JTextArea();
      textArea.setToolTipText(getParameter().getDescription());
      ComponentProperties.setProperties(textArea, Mode.REPORT, false);

      JScrollPane scroll = new JScrollPane(textArea);
      scroll
        .setBorder(ComponentProperties.getInstance(Mode.REPORT).getBorder());
      updateViewContents();
      add(scroll, new GBC(0, 0).both());
    }

    /**
     * @see csbase.client.algorithms.parameters.ListParameterView.IListParameterComponent#updateViewContents()
     */
    public void updateViewContents() {
      List<E> values = getParameter().getValue();
      if (values == null) {
        textArea.setRows(1);
      }
      else {
        textArea.setRows(Math.max(1, Math.min(values.size(), 6)));

        StringBuffer sb = new StringBuffer();
        for (E value : values) {
          sb.append(formatter.format(value)).append('\n');
        }
        // remove o ltimo breakLine
        String text = sb.substring(0, sb.length() - 1);
        textArea.setText(text);
      }
    }

    /**
     * @see javax.swing.JComponent#setEnabled(boolean)
     */
    @Override
    public void setEnabled(boolean enabled) {
      super.setEnabled(enabled);
      textArea.setEnabled(enabled);
    }
  }

  private final class ListConfigurationParameter extends JPanel implements
    IListParameterComponent {

    private Parser<E> parser;
    private Formatter<E> formatter;

    private final JButton addButton;
    private final JButton editButton;
    private final JList list;
    private final DefaultListModel model;
    private final JButton removeButton;
    private final JTextField textField;

    ListConfigurationParameter(Parser<E> parser, Formatter<E> formatter,
      JTextField textField) {
      setParser(parser);
      setFormatter(formatter);

      setLayout(new BorderLayout());

      this.textField = null == textField ? new JTextField(50) : textField;
      this.textField.addActionListener(new AddActionListener());
      this.textField.getDocument().addDocumentListener(
        new AddDocumentListener());
      this.textField.setToolTipText(getParameter().getDescription());

      this.model = new DefaultListModel();

      this.list = new JList(this.model);
      this.list.getSelectionModel().setSelectionMode(
        ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
      this.list.addKeyListener(new RemoveKeyListener());
      this.list.addListSelectionListener(new EditListSelectionListener());
      Dimension preferredSize = this.textField.getPreferredSize();
      this.list.setFixedCellWidth((int) preferredSize.getWidth());
      this.list.setFixedCellHeight((int) preferredSize.getHeight());
      this.list.setToolTipText(getParameter().getDescription());

      JPanel editPanel = new JPanel();
      editPanel.setLayout(new BorderLayout());
      editPanel.add(this.textField, BorderLayout.NORTH);
      editPanel.add(new JScrollPane(this.list), BorderLayout.CENTER);
      add(editPanel, BorderLayout.CENTER);

      this.addButton = new JButton("Adicionar");
      this.addButton.addActionListener(new AddActionListener());
      String description = getParameter().getDescription();
      this.addButton.setToolTipText(description);
      this.addButton.setEnabled(false);
      this.editButton = new JButton("Editar...");
      this.editButton.addActionListener(new EditActionListener());
      this.editButton.setToolTipText(description);
      this.editButton.setEnabled(false);
      this.removeButton = new JButton("Remover");
      this.removeButton.addActionListener(new RemoveActionListener());
      this.removeButton.setToolTipText(description);
      this.removeButton.setEnabled(false);
      ClientUtilities.adjustEqualSizes(this.addButton, this.editButton,
        this.removeButton);
      JPanel buttonPanel = new JPanel();
      buttonPanel.setLayout(new GridBagLayout());
      GridBagConstraints constraints = new GridBagConstraints();
      constraints.anchor = GridBagConstraints.CENTER;
      constraints.fill = GridBagConstraints.NONE;
      constraints.gridheight = 1;
      constraints.gridwidth = 1;
      constraints.gridx = 1;
      constraints.gridy = 1;
      constraints.insets = new Insets(2, 2, 2, 2);
      constraints.weightx = 0.0;
      constraints.weighty = 0.0;
      buttonPanel.add(this.addButton, constraints);
      constraints.gridy++;
      buttonPanel.add(this.editButton, constraints);
      constraints.gridy++;
      buttonPanel.add(this.removeButton, constraints);
      add(buttonPanel, BorderLayout.EAST);
    }

    /**
     * @see javax.swing.JComponent#setEnabled(boolean)
     */
    @Override
    public void setEnabled(boolean isEnabled) {
      super.setEnabled(isEnabled);
      this.textField.setEnabled(isEnabled);
      this.list.setEnabled(isEnabled);
      updateAddButton();
      updateEditButton();
      updateRemoveButton();
    }

    /**
     * @see csbase.client.algorithms.parameters.ListParameterView.IListParameterComponent#updateViewContents()
     */
    public void updateViewContents() {
      this.model.removeAllElements();
      List<E> values = getParameter().getValue();
      if (values != null) {
        for (E value : values) {
          this.model.addElement(getFormatter().format(value));
        }
      }
    }

    private Formatter<E> getFormatter() {
      return this.formatter;
    }

    private Parser<E> getParser() {
      return this.parser;
    }

    private void setFormatter(Formatter<E> formatter) {
      if (formatter == null) {
        throw new IllegalArgumentException("O parmetro formatter est nulo.");
      }
      this.formatter = formatter;
    }

    private void setParser(Parser<E> parser) {
      if (parser == null) {
        throw new IllegalArgumentException("O parmetro parser est nulo.");
      }
      this.parser = parser;
    }

    private DefaultListModel getModel() {
      return this.model;
    }

    private void removeSelection() {
      int[] indices = this.list.getSelectedIndices();
      for (int i = 0; i < indices.length; i++) {
        getParameter().removeElement(indices[i] - i);
      }
    }

    private void restoreText() {
      this.textField.setText("");
      this.textField.requestFocus();
    }

    private void updateAddButton() {
      String rawText = this.textField.getText();
      Object object = getParser().parse(rawText);
      this.addButton.setEnabled(isEnabled() && object != null);
    }

    private void updateEditButton() {
      this.editButton.setEnabled(isEnabled() && !this.list.isSelectionEmpty());
    }

    private void updateRemoveButton() {
      this.removeButton
        .setEnabled(isEnabled() && !this.list.isSelectionEmpty());
    }

    private final class AddActionListener implements ActionListener {
      /**
       * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
       */
      public void actionPerformed(ActionEvent e) {
        if (addButton.isEnabled()) {
          String text = textField.getText().trim();
          E element = getParser().parse(text);
          if (element != null) {
            getModel().addElement(getFormatter().format(element));
            ListParameter<E> parameter = getParameter();
            parameter.addElement(element);
            restoreText();
          }
        }
      }
    }

    private final class AddDocumentListener implements DocumentListener {
      /**
       * @see javax.swing.event.DocumentListener#changedUpdate(javax.swing.event.DocumentEvent)
       */
      public void changedUpdate(DocumentEvent e) {
        updateAddButton();
      }

      /**
       * @see javax.swing.event.DocumentListener#insertUpdate(javax.swing.event.DocumentEvent)
       */
      public void insertUpdate(DocumentEvent e) {
        updateAddButton();
      }

      /**
       * @see javax.swing.event.DocumentListener#removeUpdate(javax.swing.event.DocumentEvent)
       */
      public void removeUpdate(DocumentEvent e) {
        updateAddButton();
      }
    }

    private final class EditActionListener implements ActionListener {
      /**
       * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
       */
      public void actionPerformed(ActionEvent e) {
        String text =
          JOptionPane.showInputDialog("Editar", list.getSelectedValue());
        if (text != null) {
          E element = getParser().parse(text);
          if (element != null) {
            int index = list.getSelectedIndex();
            getModel().removeElementAt(index);
            getModel().add(index, getFormatter().format(element));
            getParameter().setElement(index, element);
          }
        }
        restoreText();
      }
    }

    private final class EditListSelectionListener implements
      ListSelectionListener {
      /**
       * @see javax.swing.event.ListSelectionListener#valueChanged(javax.swing.event.ListSelectionEvent)
       */
      public void valueChanged(ListSelectionEvent e) {
        if (list.isSelectionEmpty() || list.getSelectedIndices().length > 1) {
          editButton.setEnabled(false);
        }
        else {
          editButton.setEnabled(true);
        }
        if (list.isSelectionEmpty()) {
          removeButton.setEnabled(false);
        }
        else {
          removeButton.setEnabled(true);
        }
      }
    }

    private final class RemoveActionListener implements ActionListener {
      /**
       * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
       */
      public void actionPerformed(ActionEvent e) {
        removeSelection();
        restoreText();
      }
    }

    private final class RemoveKeyListener implements KeyListener {
      /**
       * @see java.awt.event.KeyListener#keyPressed(java.awt.event.KeyEvent)
       */
      public void keyPressed(KeyEvent e) {
        int keyCode = e.getKeyCode();
        switch (keyCode) {
          case KeyEvent.VK_BACK_SPACE:
          case KeyEvent.VK_DELETE:
            removeSelection();
        }
      }

      /**
       * @see java.awt.event.KeyListener#keyReleased(java.awt.event.KeyEvent)
       */
      public void keyReleased(KeyEvent e) {
      }

      /**
       * @see java.awt.event.KeyListener#keyTyped(java.awt.event.KeyEvent)
       */
      public void keyTyped(KeyEvent e) {
      }
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean fillVerticalSpace() {
    return true;
  }
}
