/**
 * $Id: OutputFileParameterView.java 97327 2009-10-09 01:14:44Z costa $
 */

package csbase.client.algorithms.parameters;

import java.awt.Window;
import java.text.MessageFormat;

import tecgraf.javautils.FileUtils;
import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.algorithms.parameters.validators.OutputFileParameterValidator;
import csbase.client.desktop.DesktopFrame;
import csbase.client.kernel.ClientException;
import csbase.client.project.ProjectDirectoryChooser;
import csbase.client.project.ProjectFileChooser;
import csbase.client.project.ProjectFileChooserSave;
import csbase.client.project.ProjectTreePath;
import csbase.logic.ClientProjectFile;
import csbase.logic.ProjectFileType;
import csbase.logic.algorithms.FileParameterValue;
import csbase.logic.algorithms.parameters.FileParameterMode;
import csbase.logic.algorithms.parameters.OutputFileParameter;

/**
 * A Viso para {@link OutputFileParameter Parmetro do Tipo Arquivo de Sada}.
 * 
 * @author lmoreira
 */
public final class OutputFileParameterView extends FileParameterView {

  /**
   * Cria uma viso em modo {@link ParameterView.Mode#CONFIGURATION}.
   * 
   * @param window NO EST SENDO UTILIZADO. Existe somente para manter
   *        compatibilidade com o WebSintesi.
   * @param parameter O parmetro (No aceita {@code null}).
   * 
   * @deprecated para manter compatibilidade com o WebSintesi
   */
  @Deprecated
  public OutputFileParameterView(Window window, OutputFileParameter parameter) {
    this(parameter, Mode.CONFIGURATION);
  }

  /**
   * Cria a viso.
   * 
   * @param parameter O parmetro (No aceita {@code null}).
   * @param mode Modo de visualizao. No aceita {@code null}, os possveis
   *        valores so: {@link ParameterView.Mode#CONFIGURATION} ou
   *        {@link ParameterView.Mode#REPORT}.
   */
  public OutputFileParameterView(OutputFileParameter parameter, Mode mode) {
    super(parameter, mode);
  }

  /**
   * @see csbase.client.algorithms.parameters.FileParameterView#getParameter()
   */
  @Override
  public OutputFileParameter getParameter() {
    return (OutputFileParameter) super.getParameter();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected FileParameterValue askForFile() {
    try {
      if (FileParameterMode.REGULAR_FILE.equals(getParameter().getMode())) {
        ProjectFileChooserSave chooser =
          new ProjectFileChooserSave(getWindow(), DesktopFrame.getInstance()
            .getProject(), ProjectFileChooser.FILE_ONLY, getParameter()
            .getFileType());
        ProjectTreePath projectTreePath = chooser.getSelectedPath();
        if (projectTreePath == null) {
          return null;
        }
        ClientProjectFile clientProjectFile = projectTreePath.getFile();
        String type;
        if (clientProjectFile == null) {
          type = getParameter().getFileType();
        }
        else if (clientProjectFile.isDirectory()) {
          type = ProjectFileType.DIRECTORY_TYPE;
        }
        else {
          type = clientProjectFile.getType();
        }
        String path = FileUtils.joinPath('/', projectTreePath.getSimplePath());
        return new FileParameterValue(path, type);
      }
      if (FileParameterMode.DIRECTORY.equals(getParameter().getMode())) {
        ProjectDirectoryChooser chooser =
          new ProjectDirectoryChooser(DesktopFrame.getInstance().getProject(),
            getWindow());
        ProjectTreePath projectTreePath = chooser.getSelectedDirectory();
        if (projectTreePath == null) {
          return null;
        }
        return new FileParameterValue(FileUtils.joinPath('/', projectTreePath
          .getSimplePath()), ProjectFileType.DIRECTORY_TYPE);
      }
      String errorMessage =
        MessageFormat.format("O modo de operao {0} no  suportado.",
          getParameter().getMode());
      throw new IllegalStateException(errorMessage);
    }
    catch (ClientException e) {
      String errorMessage =
        "Ocorreu um erro ao abrir o dilogo de seleo de arquivos.";
      if (e.getLocalizedMessage() != null) {
        errorMessage += "\nDetalhes:\n";
        errorMessage += e.getLocalizedMessage();
      }
      StandardDialogs.showErrorDialog(getWindow(),
        "Erro ao abrir o dilogo de seleo de arquivo", errorMessage);
      return null;
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected OutputFileParameterValidator createParameterValidator() {
    return new OutputFileParameterValidator(getParameter().isOptional());
  }
}
