/*
 * $Id: ParameterLoaderCommandListener.java 85068 2008-10-31 20:19:56Z isabella
 * $
 */

package csbase.client.algorithms.parameters;

import csbase.client.desktop.CommandListener;
import csbase.logic.CommandEndNotificationData;
import csbase.logic.CommandErrorNotificationData;
import csbase.logic.CommandNotificationData;
import csbase.logic.CommandSuccessNotificationData;
import csbase.logic.CommandWithNoExitCodeNotificationData;

/**
 * Esta classe monitora as notificaes sobre a execuo dos comandos do
 * carregador de parmetros.
 */
public class ParameterLoaderCommandListener implements CommandListener {
  /** Identificador do comando */
  private Object commandId;
  /** A task de execuo do comando */
  private ParameterLoaderCommandExecutionTask task;
  /** Indica se o listener deve ser removido aps receber a notificao. */
  private boolean remove;

  /**
   * Constri um <code>ParameterLoaderCommandListener</code>.
   * 
   * @param commandId
   */
  protected ParameterLoaderCommandListener(
    ParameterLoaderCommandExecutionTask task, Object commandId) {
    this.commandId = commandId;
    this.task = task;
    this.remove = false;
  }

  public void notifyCommand(CommandNotificationData data) {
    if (!isInterested(data)) {
      return;
    }
    task.executionCompleted(data instanceof CommandEndNotificationData
      || data instanceof CommandErrorNotificationData
      || data instanceof CommandSuccessNotificationData
      || data instanceof CommandWithNoExitCodeNotificationData);
    remove = true;
  }

  public boolean shouldRemove() {
    return remove;
  }

  private boolean isInterested(CommandNotificationData data) {
    return commandId.equals(data.getCommandId());
  }
}
