package csbase.client.algorithms.parameters;

import java.awt.Window;
import java.io.FileNotFoundException;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.Reader;

import csbase.client.desktop.DesktopFrame;
import csbase.client.desktop.RemoteTask;
import csbase.client.util.StandardErrorDialogs;
import csbase.logic.ClientProjectFile;
import csbase.logic.ClientProjectFileInputStream;
import csbase.logic.CommonClientProject;

import tecgraf.javautils.LNG;

/**
 * Tarefa responsvel por carregar os parmetros gerados pelo algoritmo
 * Extrator no configurador.
 */
public class ParameterLoaderTask extends RemoteTask<String> {
  /** A janela da aplicao [para dialogs de erro] */
  private Window window;
  /** A viso do componente do carregagor */
  private ParameterLoaderView parameter;
  /** Arquivo gerado pelo algoritmo extrator */
  private ClientProjectFile file = null;

  /**
   * @param window
   * @param parameter
   */
  public ParameterLoaderTask(Window window, ParameterLoaderView parameter) {
    this.window = window;
    this.parameter = parameter;
  }
   
  /**
   * Cancela a tarefa. Remove o arquivo temporrio que foi criado.
   */
  @Override
  protected void cancelTask() {
    super.cancelTask();
    try {
      // Apagar arquivos de parmetros
      file.remove();
    }
    catch (Exception e) {
      StandardErrorDialogs.showExceptionDialog(window,
        LNG.get("ParameterLoaderView.error.title"), e);
    }
  }

  /**
   * Executa a tarefa. 
   *
   * @throws Exception
   */
  @Override
  protected void performTask() throws Exception {
    String outputExtractFileName = parameter.getOutputExtractFileName();
    
    CommonClientProject currentProject = DesktopFrame.getInstance()
      .getProject();
    ClientProjectFile rootFile = (ClientProjectFile) currentProject.getTree();
    file = rootFile.getChild(outputExtractFileName);
    if (file == null) {
      throw new FileNotFoundException(String.format(
        LNG.get("ParameterLoaderView.file_not_found.error"),
        outputExtractFileName));
    }
    Reader reader = null;
    StringBuilder fileContents = new StringBuilder();
    try {
      reader = new InputStreamReader(
        new ClientProjectFileInputStream(file, 0));
      int character = reader.read();
      while (character != -1) {
        fileContents.append((char) character);
        character = reader.read();
      }
    }
    catch (Exception e) {
      // Falha no reader
      try {
        if (reader != null) {
          reader.close();
        }
      }
      catch (IOException ioe) {
      }
      try {
        // Apagar arquivos de parmetros
        file.remove();
      }
      catch (IOException ioe) {
      }
      throw e;
    }
    // Leitura efetuada com sucesso.
    try {
      if (reader != null) {
        reader.close();
      }
    }
    catch (Exception e) {
    }
    // Apagar arquivos de parmetros
    file.remove();
    setResult(fileContents.toString());
  }

  /**
   * Exibe mensagem de erro com exceo.
   * 
   * @param error exceo
   */
  @Override
  protected void handleError(Exception error) {
    StandardErrorDialogs.showExceptionDialog(window,
      LNG.get("ParameterLoaderView.error.title"), error);
  }
}

