/*
 * $Id:$
 */
package csbase.client.algorithms.parameters;

import java.awt.Window;

import javax.swing.JComponent;

import csbase.client.algorithms.AlgorithmConfiguratorView;
import csbase.client.algorithms.parameters.validators.ValidationMode;
import csbase.client.algorithms.parameters.validators.ValidationResult;
import csbase.client.algorithms.view.simple.SimpleAlgorithmConfiguratorPanel;
import csbase.client.kernel.ClientException;
import csbase.logic.algorithms.parameters.SimpleAlgorithmConfigurator;

/**
 * Viso para {@link SimpleAlgorithmConfigurator Configurador de Algoritmos
 * Simples}.
 * 
 * @author lmoreira
 */
public class SimpleAlgorithmConfiguratorView extends AlgorithmConfiguratorView {

  /**
   * O componente principal.
   */
  private SimpleAlgorithmConfiguratorPanel mainComponent;

  /**
   * Cria a viso.
   * 
   * @param owner A janela que  dona deste dilogo (No aceita {@code null}).
   * @param configurator O configurador (No aceita {@code null}).
   * @param mode Modo de visualizao do configurador. No aceita {@code null},
   *        os possveis valores so: {@link ParameterView.Mode#CONFIGURATION}
   *        ou {@link ParameterView.Mode#REPORT}
   * @throws ClientException
   */
  public SimpleAlgorithmConfiguratorView(Window owner,
    SimpleAlgorithmConfigurator configurator, ParameterView.Mode mode)
    throws ClientException {
    super(owner, configurator, mode);
    this.mainComponent = createMainComponent(configurator);
    this.mainComponent
      .addParameterGroupViewListener(new ParameterGroupViewListener() {
      @Override
      public void childParameterVisibilityChanged(ParameterGroupView groupView) {
        final AlgorithmConfiguratorView view =
          SimpleAlgorithmConfiguratorView.this;
        view.resizeOnVisibilityChanged();
      }
    });
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void setValidationMode(ValidationMode validationMode) {
    super.setValidationMode(validationMode);
    mainComponent.setValidationMode(validationMode);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public ValidationResult validate(boolean complain) {
    return mainComponent.validate(complain);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public JComponent getMainComponent() {
    return this.mainComponent;
  }

  /**
   * Cria o componente principal.
   * 
   * @param configurator O configurador do algoritmo a ser visualizado.
   * @return O painel que  o componente principal dessa viso.
   * 
   * @throws ClientException
   */
  private SimpleAlgorithmConfiguratorPanel createMainComponent(
    final SimpleAlgorithmConfigurator configurator) throws ClientException {

    return new SimpleAlgorithmConfiguratorPanel(getOwner(), configurator,
      getMode());
  }
}
