package csbase.client.algorithms.parameters.validators;

import csbase.logic.algorithms.parameters.FormulaParameter;
import csbase.logic.algorithms.parameters.SimpleParameter;

/**
 * <p>
 * Valida textos de usurio que so valores para parmetros da classe
 * {@link FormulaParameter}.
 * </p>
 * 
 * <p>
 * Veja {@link FormulaParameter} para saber o que  um texto vlido.
 * </p>
 * 
 * @author lmoreira
 */
public final class FormulaParameterValidator extends
  SimpleParameterValidator<String> {

  /**
   * Cria o validador.
   * 
   * @param isOptional Indica se o parmetro  opcional (aceita {@code null}
   *        como valor) ou  obrigatrio (no aceita {@code null} como valor).
   */
  public FormulaParameterValidator(boolean isOptional) {
    super(isOptional);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public ValidationResult validateValue(SimpleParameter<?> parameter,
    String value) {

    ValidationResult result = super.validateValue(parameter, value);
    if (!result.isWellSucceded()) {
      return result;
    }

    FormulaParameter formulaParameter = (FormulaParameter) parameter;
    String commandValue = formulaParameter.getCommandValue();
    if (commandValue != null) {
      if (!commandValue.matches("[A-Za-z0-9_]+")) {
        String message = getMessage(getClass(), "message");
        message = String.format(message, parameter.getName(), value);
        return new ValidationResult(message);
      }
    }
    return new ValidationResult();
  }
}
