package csbase.client.algorithms.parameters.validators;

import java.text.MessageFormat;
import java.util.List;

import csbase.logic.algorithms.parameters.SimpleParameter;

/**
 * Validador do {@link csbase.logic.algorithms.parameters.ListParameter
 * Parmetro do Tipo Lista}.
 * 
 * @param <E> O tipo do elemento armazenado na lista.
 * 
 * @author lmoreira
 */
public final class ListParameterValidator<E> extends
  SimpleParameterValidator<List<E>> {
  /**
   * O validador de elementos da lista.
   */
  private SimpleParameterValidator<E> itemValidator;

  /**
   * Cria o validador.
   * 
   * @param itemValidator O validador dos elementos da lista
   *        (No aceita {@code null}).
   */
  public ListParameterValidator(SimpleParameterValidator<E> itemValidator) {
    super(itemValidator.isOptional());
    this.itemValidator = itemValidator;
  }

  /**
   * Obtm o validador de elementos da lista.
   * 
   * @return .
   */
  public SimpleParameterValidator<E> getItemValidator() {
    return this.itemValidator;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void setMode(ValidationMode mode) {
    this.itemValidator.setMode(mode);
    super.setMode(mode);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public ValidationResult validateValue(SimpleParameter<?> parameter,
    List<E> values) {
    if (!parameter.isEnabled() || !parameter.isVisible()) {
      return new ValidationResult();
    }
    if ((values == null) || values.isEmpty()) {
      if (isOptional() || getMode().equals(ValidationMode.ALLOW_EMPY_VALUES)) {
        return new ValidationResult();
      }
      return new ValidationResult(MessageFormat.format(getMessage(
        ListParameterValidator.class, "empty_value"), parameter.getLabel()));
    }
    for (int i = 0; i < values.size(); i++) {
      E itemValue = values.get(i);
      ValidationResult result =
        this.itemValidator.validateValue(parameter, itemValue);
      if (!result.isWellSucceded()) {
        return new ValidationResult(result.getErrorMessage());
      }
    }
    return new ValidationResult();
  }
}
