/*
 * $Id: XAppAlgorithmConfiguratorView.java 101134 2010-01-27 20:22:25Z clinio $
 */
package csbase.client.algorithms.xapps;

import java.awt.Window;

import javax.swing.JComponent;

import csbase.client.algorithms.AlgorithmConfiguratorView;
import csbase.client.algorithms.parameters.ParameterView;
import csbase.client.algorithms.parameters.validators.ValidationResult;
import csbase.client.algorithms.view.xapp.XAppAlgorithmConfiguratorPanel;
import csbase.client.kernel.ClientException;
import csbase.logic.algorithms.xapps.XAppAlgorithmConfigurator;

/**
 * A viso do configurador de aplicao X.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class XAppAlgorithmConfiguratorView extends AlgorithmConfiguratorView {

  /**
   * O painel principal.
   */
  private XAppAlgorithmConfiguratorPanel mainComponent;

  /**
   * Cria a viso.
   * 
   * @param owner A janela que  dona do dilogo desta viso (No aceita {@code
   *        null}).
   * @param configurator O configurador (No aceita {@code null}).
   * @param mode Modo de visualizao do configurador. No aceita {@code null},
   *        os possveis valores so: {@link ParameterView.Mode#CONFIGURATION}
   *        ou {@link ParameterView.Mode#REPORT}
   * @throws ClientException em caso de erro.
   */
  public XAppAlgorithmConfiguratorView(Window owner,
    XAppAlgorithmConfigurator configurator, ParameterView.Mode mode)
    throws ClientException {
    super(owner, configurator, mode);
    this.mainComponent = createMainComponent(configurator);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public final ValidationResult validate(boolean complain) {
    return mainComponent.validate(complain);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public final JComponent getMainComponent() {
    return this.mainComponent;
  }

  /**
   * Cria o componente principal.
   * 
   * @param configurator o configurador
   * @return o painel
   * @throws ClientException em caso de erro.
   */
  private XAppAlgorithmConfiguratorPanel createMainComponent(
    XAppAlgorithmConfigurator configurator) throws ClientException {

    return new XAppAlgorithmConfiguratorPanel(getOwner(), configurator,
      ParameterView.Mode.CONFIGURATION);
  }
}
