package csbase.client.applications;

import javax.swing.JPanel;

/**
 * Painel genrico de uma aplicao, que contm algumas facilidades comuns a
 * todos os painis de uma aplicao, commo por exemplo, a obteno do valor de
 * uma propriedade de internacionalizao.
 * 
 * @param <T> Tipo da aplicao que est criando o painel.
 */
public abstract class AbstractSimpleApplicationPanel<T extends Application>
  extends JPanel {
  /**
   * Aplicao que contm o painel.
   */
  private T application;

  /**
   * Constri o painel.
   * 
   * @param application a aplicao
   */
  public AbstractSimpleApplicationPanel(T application) {
    this.application = application;
  }

  /**
   * Obtm a referncia para a aplicao.
   * 
   * @return a aplicao
   */
  final public T getApplication() {
    return application;
  }

  /**
   * Constri o painel.
   */
  protected abstract void buildPanel();

  /**
   * Obtm o texto de uma propriedade de internacionalizao.
   * 
   * @param key propriedade desejada
   * @return o texto de internacionalizao
   */
  final public String getString(final String key) {
    return application.getString(key);
  }

  /**
   * Obtm o texto de uma propriedade de internacionalizao que contenha
   * parmetros.
   * 
   * @param key propriedade para a mensagem que contm parmetros.
   * @param args argumentos para a mensagem.
   * 
   * @return mensagem preenchida pelos argumentos
   * 
   * @see java.text.MessageFormat
   */
  public final String getString(final String key, final Object[] args) {
    return application.getString(key, args);
  }

}
