package csbase.client.applications.algorithmsmanager;

import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.Category;
import csbase.logic.algorithms.CategorySet;

/**
 * A interface <code>AlgorithmsManagerListener</code> modela um listener para a
 * aplicao de gerncia de algoritmos.
 * 
 * Esse listener permite que determinados eventos ocorridos na aplicao sejam
 * repassados para objetos que precisam tratar tais eventos. Por exemplo, uma
 * mudana no conjunto de categorias ou no conjunto de algoritmos devem ser
 * tratadas nas janelas que possuem uma rvore de categorias de algoritmos e
 * algoritmos e que precisam redesenh-la.
 * 
 * @author pizzol
 */
public interface AlgorithmsManagerListener {

  /**
   * Notifica que uma nova categoria foi criada no conjunto de categorias
   * disponvel na aplicao.
   * 
   * @param category categoria criada
   * 
   */
  public void categoryCreated(Category category);

  /**
   * Notifica que uma categoria foi removida do conjunto de categorias
   * disponvel na aplicao.
   * 
   * @param category categoria removida
   * 
   */
  public void categoryRemoved(Category category);

  /**
   * Notifica que um conjunto de categorias foi modificado no conjunto de
   * categorias disponvel na aplicao.
   * 
   * @param modifiedCategorySet conjunto com as categorias modificadas
   * 
   */
  public void categoryUpdated(CategorySet modifiedCategorySet);

  /**
   * Notifica que um novo algoritmo foi criado no conjunto de algoritmos
   * disponvel na aplicao.
   * 
   * @param algoInfo algoritmo criado
   * 
   */
  public void algorithmCreated(AlgorithmInfo algoInfo);

  /**
   * Notifica que um algoritmo foi removido do conjunto de algoritmos disponvel
   * na aplicao.
   * 
   * @param algoInfo algoritmo removido
   * 
   */
  public void algorithmRemoved(AlgorithmInfo algoInfo);

  /**
   * Notifica que um algoritmo foi alterado no conjunto de algoritmos disponvel
   * na aplicao.
   * 
   * @param algoInfo algoritmo alterado
   * 
   */
  public void algorithmUpdated(AlgorithmInfo algoInfo);
}
