package csbase.client.applications.algorithmsmanager.actions;

import java.text.MessageFormat;

import javax.swing.ImageIcon;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.applications.algorithmsmanager.dialogs.AlgorithmVersionInfoPanel;
import csbase.client.applications.algorithmsmanager.dialogs.VersionDialog;
import csbase.client.remote.srvproxies.AlgorithmManagementProxy;
import csbase.client.util.StandardErrorDialogs;
import csbase.logic.algorithms.AlgorithmInfo;

/**
 * Essa classe representa a ao de remover uma verso de algoritmo, na
 * funcionalidade de gerenciamento de algoritmos.
 * 
 * O mtodo <code> handleEditOperation </code> deve ser redefinido para que seja
 * aberto o dilogo para confirmao da remoo da verso do algoritmo.
 * 
 */
public class VersionRemoveAction extends CommonVersionAction {
  /**
   * Constri a ao de importar um pacote de verso de algoritmo.
   * 
   * @param versionInfoPanel painel pai que criou a ao
   * @param icon imagem da ao
   */
  public VersionRemoveAction(AlgorithmVersionInfoPanel versionInfoPanel,
    ImageIcon icon) {
    super(versionInfoPanel, icon);
  }

  @Override
  protected void handleVersionOperation() {
    new RemoveVersionDialog().show();
  }

  /**
   * Classe que representa um dilogo de remoo de verses.
   * 
   */
  private class RemoveVersionDialog extends VersionDialog {

    /**
     * Cria o dilogo.
     */
    public RemoveVersionDialog() {
      super(getSelectedAlgorithm());
    }

    /**
     * Mostra o dilogo.
     */
    public void show() {
      Object algoId = algoInfo.getId();
      dialogTitle = MessageFormat.format(LNG
        .get("algomanager.title.remove_version"), new Object[] { algoInfo
        .getName() });
      Object[] options = { LNG.get("algomanager.button.remove"),
          LNG.get("algomanager.button.cancel") };
      int opt = StandardDialogs.showOptionDialog(getWindow(), dialogTitle,
        MessageFormat.format(LNG.get("algomanager.msg.confirm.version_remove"),
          new Object[] { getSelectedVersion().getId() }), options);
      if (opt != 0) {
        return;
      }
      AlgorithmInfo algo = AlgorithmManagementProxy.removeVersion(algoId,
        getSelectedVersion().getId(), getWindow());
      if (algo == null) {
        StandardErrorDialogs.showErrorDialog(getWindow(), LNG
          .get("algomanager.title.error")
          + " - " + dialogTitle, MessageFormat.format(LNG
          .get("algomanager.error.version_remove"),
          new Object[] { getSelectedVersion().getId() }));
      }
    }
  }

}
