package csbase.client.applications.algorithmsmanager.dialogs;

import java.awt.GridBagLayout;
import java.awt.Window;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.text.MessageFormat;
import java.util.Hashtable;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;

import tecgraf.javautils.gui.GBC;
import csbase.client.applications.algorithmsmanager.versiontree.PropertiesPanel;
import csbase.client.remote.srvproxies.AlgorithmManagementProxy;
import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.AlgorithmProperty;

/**
 * Painel com as informaes para criao de um novo algoritmo.
 * 
 */
public class AlgorithmCreateInfoPanel extends CommonInfoEditPanel {
  /** Campo texto com o nome do algoritmo */
  private JTextField tf_algorithmName;

  /**
   * Lista com as propriedades ordenadas dos algoritmos. Cada propriedade  um
   * mapeamento da chave ao Label (String de visualizaco).
   */
  private List<AlgorithmProperty> algorithmProperties;

  /** Painel de propriedades de algoritmos */
  private PropertiesPanel propertiesPanel;

  /** Painel principal */
  private JPanel mainPanel;

  /** Indica que est sendo feita uma edio */
  private boolean isEditing;

  /**
   * Constri o painel com as informaes bsicas para criao de um algoritmo.
   * 
   * @param algoPane painel com os dados de criao de um algoritmo
   */
  public AlgorithmCreateInfoPanel(CommonEditTabbedPanel algoPane) {
    super(algoPane);
    setCancelActionState(true);
  }

  /**
   * Obtm a janela da aplicao Gerenciador de Algoritmos.
   * 
   * @return a janela da aplcicao
   */
  private Window getWindow() {
    return getApplication().getApplicationFrame();
  }

  /**
   * Obtm o painel com os campos para alterao de uma categoria.
   * 
   * @return o painel criado
   */
  private JPanel getAlgoCreatePanel() {
    JPanel mainPanel = new JPanel(new GridBagLayout());
    mainPanel.setBorder(BorderFactory.createEmptyBorder());

    algorithmProperties = AlgorithmManagementProxy
      .getAlgorithmProperties(getWindow());
    propertiesPanel = new PropertiesPanel(algorithmProperties,
      new Hashtable<String, String>(), getApplication().getString(
        "CategoryCreateInfoPanel.label.properties"), getWindow(), true);

    if (this.algorithmProperties.size() > 0) {
      mainPanel.add(propertiesPanel.getPropertiesPanel(), new GBC(0, 0)
        .horizontal().west().insets(0, 0, 0, 0));
    }
    mainPanel.add(buildEmptyPanel(), new GBC(0, 1).both().west().insets(0, 0,
      0, 0));
    return mainPanel;
  }

  /**
   * Obtm o painel com o nome do algoritmo.
   * 
   * @return o painel criado
   */
  private JPanel getAlgorithmNamePanel() {
    JPanel panel = new JPanel(new GridBagLayout());
    String maxSize = Integer.toString(AlgorithmInfo.MAX_NAME_SIZE);
    String maxInLabel = MessageFormat.format(getApplication().getString(
      "CategoryCreateInfoPanel.msg.name_max"), new Object[] { maxSize });
    JLabel nameLabel = new JLabel(getApplication().getString(
      "CategoryCreateInfoPanel.label.name")
      + " " + maxInLabel);
    tf_algorithmName = new JTextField(30);
    tf_algorithmName.addKeyListener(new KeyAdapter() {
      @Override
      public void keyReleased(KeyEvent e) {
        if (e.getKeyCode() == KeyEvent.VK_ENTER) {
          apply();
        }
      }
    });
    tf_algorithmName.addCaretListener(new CaretListener() {

      @Override
      public void caretUpdate(CaretEvent e) {
        JTextField tf = (JTextField) e.getSource();
        boolean hasName = !(tf.getText().trim().isEmpty());
        isEditing = hasName;
        setApplyActionState(hasName);
      }
    });

    panel.add(nameLabel, new GBC(0, 0).none().west().insets(5, 5, 5, 5));
    panel.add(tf_algorithmName, new GBC(1, 0).horizontal().west().insets(5, 5,
      5, 5));
    return panel;
  }

  /**
   * Obtm o nome da categoria especificado pelo usurio.
   * 
   * @return o nome da categoria
   */
  protected String getFieldCategoryName() {
    return tf_algorithmName.getText();
  }

  @Override
  public void initializeData() {
    tf_algorithmName.requestFocus();
  }

  @Override
  protected JPanel buildMainInfoPanel() {
    if (mainPanel == null) {
      mainPanel = new JPanel(new GridBagLayout());

      JPanel namePanel = getAlgorithmNamePanel();
      JPanel algoPanel = getAlgoCreatePanel();
      mainPanel.add(namePanel, new GBC(0, 0).horizontal().west().insets(0, 0,
        0, 0));
      mainPanel.add(algoPanel, new GBC(0, 1).both().west().insets(0, 0, 0, 0));
    }
    return mainPanel;
  }

  /**
   * A partir do nome do algoritmo e suas propriedades adicionadas pelo usurio,
   * o sero efetivamente criadas no servidor.
   * 
   * @return retorna true se o algoritmo foi criado com sucesso, caso contrrio,
   *         retorna false
   */
  protected boolean createAlgorithm() {
    String algoName = tf_algorithmName.getText().trim();
    if (getApplication().containsAlgorithm(algoName)) {
      getApplication()
        .showError(
          getString("AlgorithmCreateInfoPanel.msg.error.create.algorithm_exists"));
      return false;
    }
    Hashtable<String, String> fieldValues = propertiesPanel.getProperties();
    AlgorithmInfo algorithmCreated = getAlgorithmCreateDataPanel()
      .createAlgorithm(algoName, fieldValues);
    if (algorithmCreated == null) {
      getApplication().showError(
        getString("AlgorithmCreateInfoPanel.msg.error.create.no_sucess"));
      return false;
    }
    return true;
  }

  @Override
  protected void apply() {
    isEditing = false;
    if (!createAlgorithm()) {
      tf_algorithmName.requestFocus();
    }
  }

  @Override
  protected void cancel() {
    boolean confirmCancelling = !wasModified()
      || (wasModified() && confirmCancelling());
    if (confirmCancelling) {
      getAlgorithmCreateDataPanel().cancel();
      getSelectionPanel().selectAlgorithm(
        getSelectionPanel().getSelectedAlgorithm().getItem());
    }
  }

  /**
   * Obtm o painel de seleo de algoritmos.
   * 
   * @return o painel de seleo de algoritmos
   */
  private AlgorithmSelectionPanel getSelectionPanel() {
    return getAlgorithmCreateDataPanel().getSelectionPanel();
  }

  /**
   * Obtm o painel global de criao de dados de algoritmo.
   * 
   * @return o painel global de criao de dados de algoritmo
   */
  private AlgorithmCreateDataPanel getAlgorithmCreateDataPanel() {
    return (AlgorithmCreateDataPanel) getEditPanel();
  }

  /**
   * Confirma com o usurio se a operao de cancelamento deve ser efetivada, j
   * que perder todas as alteraes feitas sobre o dado.
   * 
   * @return retorna true, se a alterao do dado deve ser cancelada, caso
   *         contrrio, retorna false
   */
  private boolean confirmCancelling() {
    int confirm = getApplication().showOptionDialog(
      getString("AlgorithmCreateInfoPanel.msg.cancel.confirm"),
      new String[] {
          getString("AlgorithmCreateInfoPanel.msg.cancel.confirm.yes"),
          getString("AlgorithmCreateInfoPanel.msg.cancel.confirm.no") });
    if (confirm == 0) {
      isEditing = false;
      return true;
    }
    return false;
  }

  @Override
  public void setDataChanged() {
    isEditing = false;
    changeOperationsState(isEditing);
  }

  @Override
  protected String getTitle() {
    return getApplication().getString("CommonEditTabbedPanel.tab.info.basic");
  }

  @Override
  public boolean wasModified() {
    return isEditing;
  }
}
