package csbase.client.applications.algorithmsmanager.dialogs;

import java.awt.GridBagLayout;
import java.io.File;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Vector;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.SortOrder;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;

import tecgraf.javautils.FileUtils;
import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.GBC;
import tecgraf.javautils.gui.table.SortableTable;
import csbase.client.algorithms.ShowHistoryAction;
import csbase.client.applications.algorithmsmanager.models.AlgorithmListItem;
import csbase.client.kernel.ClientException;
import csbase.client.remote.srvproxies.AlgorithmManagementProxy;
import csbase.client.util.DateTableCellRenderer;
import csbase.logic.algorithms.HistoryProvider;
import csbase.logic.algorithms.HistoryRecord;

/**
 * Painel com as informaes de histrico do algoritmo selecionado para edio.
 * 
 */
public class AlgorithmHistoryInfoPanel extends CommonInfoEditPanel {
  /** Campo texto com o nome do algoritmo */
  private JTextField nameText;

  /** Objeto provedor do histrico a ser exibido */
  private HistoryProvider historyProvider;

  /** Painel principal */
  private JPanel mainPanel;

  /** Painel com o histrico do algoritmo selecionado */
  private JPanel historyPanel;

  /**
   * Constri o painel com as informaes de categorias do algoritmo.
   * 
   * @param algoPane painel com os dados de atualizao de um algoritmo
   */
  public AlgorithmHistoryInfoPanel(AlgorithmEditDataPanel algoPane) {
    super(algoPane);
    showOperationButtons(false);
  }

  /**
   * Obtm o painel global de edio de dados de algoritmo.
   * 
   * @return o painel global de edio de dados de algoritmo
   */
  private AlgorithmEditDataPanel getAlgorithmEditDataPanel() {
    return (AlgorithmEditDataPanel) getEditPanel();
  }

  /**
   * Obtm o algoritmo selecionado para edio.
   * 
   * @return o algoritmo selecionado para edio
   */
  private AlgorithmListItem getSelectedAlgorithm() {
    return getAlgorithmEditDataPanel().getSelectedAlgorithm();
  }

  @Override
  public void initializeData() {
    nameText.setText(getSelectedAlgorithm().getName());
    updateHistoryPanel();
  }

  /**
   * Altera o painel com a rvore da verso para o algoritmo selecionado.
   * 
   */
  private void updateHistoryPanel() {
    mainPanel.remove(historyPanel);
    historyPanel = getHistoryPanel();
    mainPanel.add(historyPanel, new GBC(0, 1).both().west().insets(0, 0, 0, 0));
    mainPanel.revalidate();
    mainPanel.repaint();
  }

  @Override
  protected JPanel buildMainInfoPanel() {
    if (mainPanel == null) {
      mainPanel = new JPanel(new GridBagLayout());
      JPanel namePanel = getAlgorithmNamePanel();
      historyPanel = getHistoryPanel();

      mainPanel.add(namePanel, new GBC(0, 0).horizontal().west().insets(0, 0,
        0, 0));
      mainPanel.add(historyPanel, new GBC(0, 1).both().west()
        .insets(0, 0, 0, 0));
    }
    return mainPanel;
  }

  /**
   * Obtm o painel com o nome do algoritmo.
   * 
   * @return o painel criado
   */
  private JPanel getAlgorithmNamePanel() {
    JPanel panel = new JPanel(new GridBagLayout());
    JLabel nameLabel = new JLabel(getApplication().getString(
      "AlgorithmBasicInfoPanel.label.name"));
    nameText = new JTextField(30);
    nameText.setEditable(false);

    panel.add(nameLabel, new GBC(0, 0).none().west().insets(5, 5, 5, 5));
    panel.add(nameText, new GBC(1, 0).horizontal().west().insets(5, 5, 5, 5));
    return panel;
  }

  /**
   * Obtm o painel com o histrico do algoritmo.
   * 
   * @return o painel com o histrico do algoritmo
   */
  private JPanel getHistoryPanel() {
    historyPanel = new JPanel(new GridBagLayout());
    historyProvider = new AlgorithmHistory();
    try {
      historyPanel.add(new JScrollPane(makeTable()), new GBC(0, 0).both()
        .west().insets(5, 5, 5, 5));
    }
    catch (ClientException e) {
      getAlgorithmEditDataPanel().getApplication().showException(
        "Ocorreu um problema na obteno do histrico do algoritmo: "
          + getSelectedAlgorithm().getName(), e);
    }
    return historyPanel;
  }

  @Override
  protected void apply() {
  }

  @Override
  protected void cancel() {
  }

  @Override
  public void setDataChanged() {
  }

  /**
   * Constri a tabela que exibe o histrico.
   * 
   * @return tabela que exibe o histrico.
   * @throws ClientException
   */
  private JTable makeTable() throws ClientException {
    List rows = (this.historyProvider == null) ? Collections.emptyList()
      : this.historyProvider.getHistory();
    if (rows == null) {
      throw new ClientException(LNG.get(ShowHistoryAction.class.getName()
        + ".error"));
    }
    TableModel m = new HistoryTableModel(rows);

    SortableTable table = new SortableTable(m);
    table.sort(0, SortOrder.DESCENDING);
    table.setDefaultRenderer(Date.class, new DateTableCellRenderer());
    table.adjustColumnWidth();
    return table;
  }

  /**
   * Modelo para a tabela
   */
  private class HistoryTableModel extends AbstractTableModel {
    /** Lista de <code>HistoryRecord</code> */
    private List recordList;

    /** Cabealhos para as colunas */
    private String[] columnNames = {
        LNG.get(ShowHistoryAction.class.getName() + ".dateColumn"),
        LNG.get(ShowHistoryAction.class.getName() + ".pathColumn"),
        LNG.get(ShowHistoryAction.class.getName() + ".loginColumn"),
        LNG.get(ShowHistoryAction.class.getName() + ".nameColumn"),
        LNG.get(ShowHistoryAction.class.getName() + ".descriptionColumn") };

    /** Classes das colunas */
    private Class[] columnClasses = { Date.class, String.class, String.class,
        String.class, String.class };

    /**
     * Cria o modelo.
     * 
     * @param recordList Lista de <code>HistoryRecord</code>.
     */
    public HistoryTableModel(List recordList) {
      this.recordList = recordList;
    }

    /**
     * {@inheritDoc}
     */
    public int getRowCount() {
      return this.recordList.size();
    }

    /**
     * {@inheritDoc}
     */
    public int getColumnCount() {
      return this.columnNames.length;
    }

    /**
     * {@inheritDoc}
     */
    public Object getValueAt(int rowIndex, int columnIndex) {
      HistoryRecord rec = (HistoryRecord) this.recordList.get(rowIndex);
      switch (columnIndex) {
        case HistoryRecord.DATE_COLUMN_INDEX:
          return rec.getDate();
        case HistoryRecord.PATH_COLUMN_INDEX:
          return FileUtils.joinPath(rec.getDirPath());
        case HistoryRecord.LOGIN_COLUMN_INDEX:
          return rec.getUserLogin();
        case HistoryRecord.NAME_COLUMN_INDEX:
          return rec.getUserName();
        case HistoryRecord.DESCRIPTION_COLUMN_INDEX:
          return rec.getDescription();
        default:
          return null;
      }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getColumnName(int columnIndex) {
      return this.columnNames[columnIndex];
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Class getColumnClass(int columnIndex) {
      return this.columnClasses[columnIndex];
    }
  }

  /**
   * Classe que implementa o provedor de histrico do algoritmo selecionado.
   * 
   */
  private class AlgorithmHistory implements HistoryProvider {

    @Override
    public List getHistory() {
      if (getSelectedAlgorithm() == null) {
        return new Vector<HistoryRecord>();
      }
      String repositoryName = LNG.get("algomanager.tree.root");
      String[] spath = new String[] { repositoryName,
          getSelectedAlgorithm().getName() };
      return AlgorithmManagementProxy.retrieveHistory(spath);
    }

    @Override
    public String getName() {
      if (getSelectedAlgorithm() == null) {
        return "";
      }
      String repositoryName = getString("AlgorithmHistoryInfoPanel.algorithm.repository.name");
      return repositoryName + File.separator + getSelectedAlgorithm().getName();
    }
  }

  @Override
  protected String getTitle() {
    return getString("AlgorithmHistoryInfoPanel.tab.history");
  }

  @Override
  public boolean wasModified() {
    return false;
  }
}
