/*
 * $Id$
 */

package csbase.client.applications.algorithmsmanager.versiontree;

import java.awt.datatransfer.DataFlavor;

import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;

import tecgraf.javautils.LNG;
import csbase.client.applications.algorithmsmanager.versiontree.actions.CopyFile;
import csbase.client.applications.algorithmsmanager.versiontree.actions.ExportConfigurationFile;
import csbase.client.applications.algorithmsmanager.versiontree.actions.PasteFile;
import csbase.client.applications.algorithmsmanager.versiontree.actions.RemoveConfigurationFiles;
import csbase.client.applications.algorithmsmanager.versiontree.actions.VersionWarningActionDecorator;
import csbase.logic.FileInfo;
import csbase.logic.IPathFactory;
import csbase.logic.algorithms.AlgorithmVersionInfo;
import csbase.logic.algorithms.ConfigurationPathFactory;

/**
 * @author Tecgraf / PUC-Rio
 * 
 *         N que representa arquivos de configurao.
 */
final class ConfigurationFileNode extends AbstractFileInfoNode {

  /**
   * Descreve o formato de objeto que pode ser transferido pelas operaces de
   * transferncia (drag&drop ou clipboard) na rvore.
   */
  public static final DataFlavor DATA_FLAVOR = new DataFlavor(
    ConfigurationFileNode.class, ConfigurationFileNode.class.getSimpleName());

  /**
   * Construtor.
   * 
   * @param tree rvore que detm este n.
   * @param version Verso que detm o arquivo representado por este n.
   * @param file Arquivo representado por este n.
   */
  protected ConfigurationFileNode(final VersionTree tree,
    final AlgorithmVersionInfo version, final FileInfo file) {
    super(tree, version, file, new IChildrenFactory() {
      @Override
      public AbstractFileInfoNode createNode(FileInfo file) {
        return new ConfigurationFileNode(tree, version, file);
      }
    }, false);
  }

  /**
   * Cria um menu pop-up para este n.
   * 
   * @return um menu pop-up para este n.
   */
  @Override
  public JPopupMenu createPopupMenu() {
    JPopupMenu popupMenu = new JPopupMenu();
    popupMenu.add(new JMenuItem(new VersionWarningActionDecorator(new CopyFile(
      getTree(), LNG.get("algomanager.menu.config.copy")))));
    if (getFile().isDirectory()) {
      popupMenu.add(new JMenuItem(new VersionWarningActionDecorator(
        new PasteFile(getTree(), LNG.get("algomanager.menu.config.paste"),
          ConfigurationFileNode.DATA_FLAVOR))));
    }
    popupMenu.add(new JMenuItem(
      new VersionWarningActionDecorator(new RemoveConfigurationFiles(getTree(),
        getVersion(), getSelectedFiles()))));
    if (!getFile().isDirectory()) {
      popupMenu.add(new JMenuItem(new VersionWarningActionDecorator(
        new ExportConfigurationFile(getTree(), getVersion(), getFile()))));
    }

    return popupMenu;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public DataFlavor getDataFlavor() {
    return DATA_FLAVOR;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected final IPathFactory createSourcePathFactory(DataTransferObject dto) {
    AlgorithmVersionInfo sourceVersion = dto.getVersion();
    return new ConfigurationPathFactory(sourceVersion);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected final IPathFactory createTargetPathFactory() {
    return new ConfigurationPathFactory(getVersion());
  }
}
