/*
 * $Id: FileClient.java 105848 2010-05-20 23:29:45Z pizzol $
 */

package csbase.client.applications.algorithmsmanager.versiontree.niotransfer;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.nio.channels.SocketChannel;

import csbase.client.externalresources.LocalFile;
import csbase.remote.niotransfer.RequestTransferInfo;

/**
 * Cliente para exportao de arquivos, semelhante a um cliente FTP, mas usando
 * a API NIO do Java.
 * 
 * Cliente de uma aplicao simples cliente/servidor de transferncia de
 * arquivos.
 * 
 * @author Tecgraf
 */
public class FileClient {
  /**
   * Objeto que implementa o protocolo de comunicao no cliente.
   */
  private ClientSimpleFileTransferProtocol protocol;
  
  /**
   * Informaes necessrias para solicitar a transferncia.
   */
  private RequestTransferInfo requestInfo;

  /**
   * Cria uma instncia de cliente, conectando-se  sua contraparte no servidor.
   * 
   * @param serverName nome do servidor.
   * @param info informaes necessrias para solicitar a transferncia.
   * 
   * @throws IOException caso ocorra algum problema durante a conexo.
   */
  public FileClient(String serverName, RequestTransferInfo info)
    throws IOException {
    this.requestInfo = info;
    SocketChannel serverChannel = null;
    int port = requestInfo.getPort();
    int bufferSize = 1024 * 1024;
    serverChannel = SocketChannel.open();
    serverChannel.connect(new InetSocketAddress(serverName, port));
    protocol = new ClientSimpleFileTransferProtocol(bufferSize, serverChannel);
  }

  /**
   * Envia um arquivo local para o servidor.
   * 
   * @param sourceFile referncia para o arquivo no sistema de arquivos local.
   * 
   * @throws IOException em caso de erro de I/O.
   */
  public void send(LocalFile sourceFile) throws IOException {
    protocol.writeFile(sourceFile, requestInfo);
  }

  /**
   * Recebe um arquivo remoto e grava no sistema de arquivos local.
   * 
   * @param targetFile localizao do arquivo de destino (se existir,
   *        sobrescreve).
   * 
   * @throws IOException em caso de erro de I/O.
   */
  public void receive(LocalFile targetFile) throws IOException {
    protocol.readFile(requestInfo, targetFile);
  }

}
