/*
 * $Id$
 */

package csbase.client.applications.commandsmonitor.table.script;

import csbase.logic.CommandInfo;
import csbase.logic.CommandStatus;
import csbase.logic.Priority;

/**
 * Implementa um comando no modificvel para ser utilizada no valor do elemento
 * &lt;expression&gt; que ser interpretado como um script para filtrar
 * comandos.
 * 
 * @author Tecgraf / PUC-Rio
 */
public class ScriptCommand {

  /** Comando representado por esta instncia. */
  private CommandInfo command;
  /** Verso no modificvel da data de submisso do comando. */
  private ScriptDate submittedDate;

  /**
   * Construtor.
   * 
   * @param command comando a ser representado.
   */
  public ScriptCommand(CommandInfo command) {
    this.command = command;
    this.submittedDate = new ScriptDate(command.getSubmittedDate());
  }

  /**
   * @see CommandInfo#getCpuPerc()
   */
  public Double getCpu() {
    return command.getCpuPerc();
  }

  /**
   * @see CommandInfo#getDescription()
   */
  public String getDescription() {
    return command.getDescription();
  }

  /**
   * @see CommandInfo#getPriority()
   */
  public String getId() {
    return command.getId();
  }

  /**
   * @see CommandInfo#getPriority()
   */
  public Priority getPriority() {
    return command.getPriority();
  }

  /**
   * @see CommandInfo#getProjectId()
   */
  public Object getProjectId() {
    return command.getProjectId();
  }

  /**
   * @see CommandInfo#getRAMMemoryPerc()
   */
  public Double getRam() {
    return command.getRAMMemoryPerc();
  }

  /**
   * @see CommandInfo#getSgaName()
   */
  public String getSga() {
    return command.getSgaName();
  }

  /**
   * @see CommandInfo#getSgaName()
   */
  public CommandStatus getStatus() {
    return command.getStatus();
  }

  /**
   * @return a {@link ScriptDate data} de submisso do comando.
   */
  public ScriptDate getSubmission() {
    return submittedDate;
  }

  /**
   * @see CommandInfo#getSwapMemoryPerc()
   */
  public Double getSwap() {
    return command.getSwapMemoryPerc();
  }

  /**
   * @see CommandInfo#getTip()
   */
  public String getTip() {
    return command.getTip();
  }

  /**
   * @see CommandInfo#getUserId()
   */
  public Object getUserId() {
    return command.getUserId();
  }

  /**
   * @see CommandInfo#getWallTimeSec()
   */
  public int getWallTime() {
    return command.getWallTimeSec();
  }
}
