package csbase.client.applications.fileexchanger.logic;

import javax.swing.Icon;
import javax.swing.ImageIcon;

import csbase.client.applications.fileexchanger.FileExchangerUI;
import csbase.client.util.AnimatedIcon;

/**
 * Estado de uma operao.
 * 
 * @author Tecgraf/PUC-Rio
 */
public enum ExchangeState {
  /**
   * Criado apenas, (estado inicial)
   */
  CREATED,

  /**
   * Na fila de transferncia do programa
   */
  QUEUED,

  /**
   * Em andamento
   */
  RUNNING(true),
  /**
   * Terminado com sucesso
   */
  FINISHED,

  /**
   * Interrompido
   */
  INTERRUPTED,

  /**
   * Em pausa
   */
  PAUSED,

  /**
   * Em estado de erro
   */
  ERROR,

  /**
   * Estado de retirado da lista de programa (seja por trmino bem sucedido ou
   * no)
   */
  REMOVED;

  /**
   * Indica se o cone que representa esse estado deve ser animado.
   */
  private boolean hasAnimatedIcon;

  /**
   * Construtor.
   * 
   * @param hasAnimatedIcon Indica se o cone que representa esse estado deve
   *        ser animado.
   */
  ExchangeState(boolean hasAnimatedIcon) {
    this.hasAnimatedIcon = hasAnimatedIcon;
  }

  /**
   * Construtor padro de estados cujos cones no so animados.
   */
  ExchangeState() {
    this.hasAnimatedIcon = false;
  }

  /**
   * Consulta o texto da descrio.
   * 
   * @return o texto
   */
  public String getDescription() {
    final String tag = toString() + ".description";
    return FileExchangerUI.getString(this.getClass(), tag);
  }

  /**
   * Obtem prefixo de tag.
   * 
   * @return prefixo.
   */
  private String getPrefixTag() {
    final String className = getClass().getSimpleName();
    final String prefixTag = className + "." + toString();
    return prefixTag;
  }

  /**
   * Obtem a imagem que representa o estado.
   * 
   * @return a imagem
   */
  public Icon getImage() {
    final String prefixTag = getPrefixTag();
    final String path = "states/" + prefixTag + ".16.gif";
    ImageIcon imageIcon = FileExchangerUI.getImageIcon(path);
    if (imageIcon != null) {
      if (isAnimated()) {
        return new AnimatedIcon(imageIcon);
      }
      return imageIcon;
    }
    return null;
  }

  /**
   * Retorna verdadeiro se o cone que representa este estado  animado ou
   * falso, caso contrrio.
   * 
   * @return verdadeiro se o cone  animado ou falso, caso contrrio.
   */
  private boolean isAnimated() {
    return hasAnimatedIcon;
  }

}
