/*
 * $Id: FileTransfer.java 104950 2010-04-30 21:36:26Z clinio $
 */

package csbase.client.applications.filetransfer;

import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Arrays;
import java.util.Observable;
import java.util.Observer;
import java.util.Set;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import tecgraf.javautils.gui.GBC;
import tecgraf.javautils.gui.GUIResources;
import csbase.client.applications.Application;
import csbase.client.applications.ApplicationExitAction;
import csbase.client.applications.ApplicationFrame;
import csbase.client.applications.ApplicationImages;
import csbase.client.desktop.DesktopFrame;
import csbase.client.desktop.RemoteTask;
import csbase.client.kernel.ClientException;
import csbase.client.project.ProjectTree;
import csbase.client.project.ProjectTreeFactory;
import csbase.client.project.ProjectTreePopupMenu;
import csbase.client.remote.ProjectAdminObserver;
import csbase.client.remote.ProjectObserver;
import csbase.client.util.StandardErrorDialogs;
import csbase.client.util.gui.MultiImageButton;
import csbase.logic.CommonClientProject;
import csbase.logic.CommonProjectInfo;
import csbase.logic.ProjectUserEvent;
import csbase.logic.User;
import csbase.logic.UserProjectInfo;

/**
 * A classe <code>FileTransfer</code> implementa o dilogo de cpia de arquivos
 * entre projetos. No momento, as combos s mostram projetos do usurio corrente
 * e ProjectComboBox no est sendo notificada de incluso/excluso do usurio
 * em algum projeto de outro usurio.
 * 
 * @author Ana Lodi
 * @version $Revision: 104950 $
 */
public class FileTransfer extends Application {
  /** Dilogo principal do visualizador. */
  private ApplicationFrame mainFrame = null;

  /** A rvore de diretrios do projeto origem */
  private ProjectTree sourceProjectTree;

  /** A rvore de diretrios do projeto destino */
  private ProjectTree targetProjectTree;

  /** Combo de Projeto origem */
  private ProjectComboBox sourceProjectCombo;

  /** Combo de Projeto destino */
  private ProjectComboBox targetProjectCombo;

  /** Projeto origem */
  private CommonClientProject sourceProject = null;

  /** Projeto destino */
  private CommonClientProject targetProject = null;

  /**
   * Exibe a janela
   */
  private void showFrame() {
    mainFrame.setSize(new Dimension(800, 600));
    mainFrame.center(DesktopFrame.getInstance().getDesktopFrame());
    mainFrame.setVisible(true);
  }

  /**
   * Cria a janela principal
   * 
   * @return referncia para a janela criada
   */
  private ApplicationFrame createMainFrame() {
    mainFrame = getApplicationFrame();
    mainFrame.setTitle(getString("FILE_TRANSFER_TITLE"));
    Container cp = mainFrame.getContentPane();
    cp.setLayout(new GridBagLayout());
    cp.add(createTextPanel(), new GBC(0, 0).horizontal().north().insets(10, 10,
      0, 10));
    cp.add(createTransferPanel(), new GBC(0, 1).both().center().insets(10, 11,
      0, 10));
    cp.add(createButtonPanel(), new GBC(0, 2).horizontal().center().insets(10,
      10, 5, 10));
    return mainFrame;
  }

  /**
   * Cria o painel de Transferncia
   * 
   * @return .
   */
  private JPanel createTransferPanel() {
    JPanel tp = new JPanel(new GridBagLayout());
    try {
      tp.add(createSourceTreePanel(mainFrame), new GBC(0, 0).both().northwest()
        .insets(0, 0, 0, 0));
      tp.add(createActionButtonsPanel(mainFrame), new GBC(1, 0).fillx()
        .center().insets(0, 10, 0, 10));
      tp.add(createTargetTreePanel(mainFrame), new GBC(2, 0).both().northeast()
        .insets(0, 0, 0, 0));
    }
    catch (Exception e) {
      StandardErrorDialogs.showErrorDialog(mainFrame, mainFrame.getTitle(), e);
    }
    return tp;
  }

  /**
   * Cria o painel com o boto de Fechar
   * 
   * @return .
   */
  private JPanel createButtonPanel() {
    JButton exitButton = new JButton(new ApplicationExitAction(this));
    exitButton.setIcon(null);
    exitButton.setText(getString("FILE_TRANSFER_EXIT"));
    JPanel bp = new JPanel(new FlowLayout());
    bp.add(exitButton);
    return bp;
  }

  /**
   * Cria o Painel com o Texto explicativo
   * 
   * @return .
   */
  private JPanel createTextPanel() {
    JPanel tp = new JPanel(new GridBagLayout());
    tp.setForeground(Color.GRAY);
    JLabel icon = new JLabel(ApplicationImages.ICON_INFORMATION_32);
    JLabel lb1 = new JLabel(getString("FILE_TRANSFER_TEXT_MOVE"));
    JLabel lb2 = new JLabel(getString("FILE_TRANSFER_TEXT_COPY"));
    lb1.setForeground(Color.GRAY);
    lb2.setForeground(Color.GRAY);
    tp.add(icon, new GBC(0, 0).height(2).none().insets(0, 0, 0, 5));
    tp.add(lb1, new GBC(1, 0).horizontal().northwest());
    tp.add(lb2, new GBC(1, 1).horizontal().northwest());
    return tp;
  }

  /**
   * Cria o painel de projeto Origem.
   * 
   * @param frame Dilogo que usar o painel.
   * 
   * @return o painel
   * 
   * @throws ClientException
   */
  private JComponent createSourceTreePanel(ApplicationFrame frame)
    throws ClientException {
    JPanel treePanel = new JPanel(new GridBagLayout());
    treePanel.add(createSourceComboPanel(), new GBC(0, 0).none().northwest()
      .insets(5, 0, 8, 0));
    try {
      // Criando a rvore de projeto origem   
      sourceProjectTree = createProjectTree(frame, sourceProject);
      JScrollPane sourceProjectPanel =
        new JScrollPane(sourceProjectTree.getTree());
      sourceProjectPanel.setPreferredSize(new Dimension(200, 465));

      treePanel.add(sourceProjectPanel, new GBC(0, 1).both().northwest()
        .insets(0, 0, 0, 0));
    }
    catch (Exception e) {
      e.printStackTrace();
    }

    //  O Projeto Origem  o mesmo do projeto que est aberto 
    // (ou null, se no houver projeto aberto)
    CommonClientProject currentProject =
      DesktopFrame.getInstance().getProject();
    /* Se houver projeto aberto, ficar selecionado na combo */
    if (currentProject != null) {
      sourceProjectCombo.setSelectedItem(new UserProjectInfo(currentProject
        .getUserId(), currentProject.getName()));
    }
    return treePanel;
  }

  /**
   * Cria o painel de projeto Destino.
   * 
   * @param frame Dilogo que usar o painel.
   * 
   * @return O painel.
   * 
   * @throws ClientException
   */
  private JComponent createTargetTreePanel(ApplicationFrame frame)
    throws ClientException {
    JPanel treePanel = new JPanel(new GridBagLayout());
    treePanel.add(createTargetComboPanel(), new GBC(0, 0).none().northwest()
      .insets(5, 0, 8, 0));

    targetProjectTree = createProjectTree(frame, null);
    JScrollPane targetProjectPanel =
      new JScrollPane(targetProjectTree.getTree());
    targetProjectPanel.setPreferredSize(new Dimension(200, 400));

    treePanel.add(targetProjectPanel, new GBC(0, 1).both().northwest().insets(
      0, 0, 0, 0));
    return treePanel;
  }

  /**
   * Cria o painel de botes para transferncia de arquivos.
   * 
   * @param frame - Dilogo que usar o painel.
   * 
   * @return O painel.
   */
  private JComponent createActionButtonsPanel(ApplicationFrame frame) {
    JPanel buttonsPanel = new JPanel(new GridBagLayout());

    MultiImageButton moveRightButton =
      new MultiImageButton(null, getString("FILE_TRANSFER_BUTTON_MOVE"),
        GUIResources.SELECTOR_ADD_ICON);
    moveRightButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        moveFiles(sourceProjectTree, targetProjectTree);
      }
    });

    MultiImageButton copyRightButton =
      new MultiImageButton(null, getString("FILE_TRANSFER_BUTTON_COPY"),
        GUIResources.SELECTOR_ADD_ICON);
    copyRightButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        copyFiles(sourceProjectTree, targetProjectTree);
      }
    });

    MultiImageButton moveLeftButton =
      new MultiImageButton(GUIResources.SELECTOR_REMOVE_ICON,
        getString("FILE_TRANSFER_BUTTON_MOVE"), null);
    moveLeftButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        moveFiles(targetProjectTree, sourceProjectTree);
      }
    });

    MultiImageButton copyLeftButton =
      new MultiImageButton(GUIResources.SELECTOR_REMOVE_ICON,
        getString("FILE_TRANSFER_BUTTON_COPY"), null);
    copyLeftButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        copyFiles(targetProjectTree, sourceProjectTree);
      }
    });

    buttonsPanel.add(moveRightButton, new GBC(0, 0).fillx().center());
    buttonsPanel.add(copyRightButton, new GBC(0, 1).fillx().center().insets(5,
      0, 0, 0));
    buttonsPanel.add(moveLeftButton, new GBC(0, 2).fillx().center().insets(30,
      0, 0, 0));
    buttonsPanel.add(copyLeftButton, new GBC(0, 3).fillx().center().insets(5,
      0, 0, 0));
    return buttonsPanel;
  }

  /**
   * Realiza a cpia dos arquivos, caso hajam arquivos selecionados no projeto
   * de origem e um diretrio selecionado no projeto de destino.
   * 
   * @param source Projeto de origem.
   * @param target Projeto de destino.
   */
  private void copyFiles(ProjectTree source, ProjectTree target) {
    if (canTransfer(source, target)) {
      source.startCopyAction();
      target.startPasteAction();
    }
  }

  /**
   * Realiza a movimentao dos arquivos, caso hajam arquivos selecionados no
   * projeto de origem e um diretrio selecionado no projeto de destino.
   * 
   * @param source Projeto de origem.
   * @param target Projeto de destino.
   */
  private void moveFiles(ProjectTree source, ProjectTree target) {

    if (canTransfer(source, target)) {
      source.startCutAction();
      target.startPasteAction();
    }
  }

  /**
   * Valida se  possvel iniciar a transferncia de arquivos, ou seja, se
   * existem arquivos selecionados no projeto de origem e um diretrio
   * selecionado no projeto de destino.
   * 
   * @param source Projeto de origem.
   * @param target Projeto de destino.
   * 
   * @return True, se existem arquivos selecionados no projeto de origem e um
   *         diretrio selecionado no projeto de destino. False, caso qualquer
   *         uma das condies abaixo seja verdadeira: <br>
   *         - Nenhum arquivo de origem selecionado. <br>
   *         - Nenhum diretrio de destino selecionado. <br>
   *         - Mais de um diretrio de destino selecionado. <br>
   *         - Arquivo selecionado como destino.
   */
  private boolean canTransfer(ProjectTree source, ProjectTree target) {

    if (source.getSelectionCount() == 0) {
      // Nenhum arquivo de origem selecionado.
      StandardErrorDialogs.showErrorDialog(mainFrame, mainFrame.getTitle(),
        getString("FILE_TRANSFER_MISSING_SOURCE"));
      return false;
    }
    if (target.getSelectionCount() == 1) {
      // Apenas um destino selecionado.
      if (target.getSelectedFile().isDirectory()) {
        // Destino selecionado  um diretrio.
        return true;
      }
    }
    StandardErrorDialogs.showErrorDialog(mainFrame, mainFrame.getTitle(),
      getString("FILE_TRANSFER_MISSING_TARGET"));
    return false;
  }

  /**
   * Cria a rvore de projeto.
   * 
   * @param frame .
   * @param project .
   * 
   * @return a ProjectTree
   * 
   * @throws ClientException
   */
  private ProjectTree createProjectTree(ApplicationFrame frame,
    CommonClientProject project) throws ClientException {
    ProjectTree projectTree;
    if (project != null) {
      projectTree =
        ProjectTreeFactory.getInstance().makeProjectTree(frame,
          frame.getTitle(), project);
    }
    else {
      projectTree =
        ProjectTreeFactory.getInstance().makeProjectTree(frame,
          frame.getTitle());
    }
    projectTree.enableRenameAction(false);
    projectTree.enableDeleteAction(false);
    /* Habilitando menu popup */
    projectTree.addMouseListener();

    /* Adaptando PopupMenu Root */
    ProjectTreePopupMenu popupRoot =
      (ProjectTreePopupMenu) projectTree.getPopupRoot();
    popupRoot.removeItem(getString("PRJ_DOWNLOAD"));
    popupRoot.removeItem(getString("PRJ_UPLOAD_FILE"));

    /* Adaptando PopupMenu Dir */
    ProjectTreePopupMenu popupDir =
      (ProjectTreePopupMenu) projectTree.getPopupDir();
    popupDir.removeItem(getString("PRJ_DOWNLOAD"));
    popupDir.removeItem(getString("PRJ_UPLOAD_FILE"));

    /* Adaptando PopupMenu File */
    ProjectTreePopupMenu popupFile =
      (ProjectTreePopupMenu) projectTree.getPopupFile();
    popupFile.removeItem(getString("PRJ_MANAGE_PUBLICATION"));
    popupFile.removeItem(getString("PRJ_CHANGE_TYPE"));
    popupFile.removeItem(getString("PRJ_DOWNLOAD_FILE"));

    /* Adaptando PopupMenu MultSelection */
    ProjectTreePopupMenu popupMult =
      (ProjectTreePopupMenu) projectTree.getPopupMultSelection();
    popupMult.removeItem(getString("PRJ_MANAGE_PUBLICATION"));
    popupMult.removeItem(getString("PRJ_CHANGE_TYPE"));
    return projectTree;
  }

  /**
   * Cria o Painel com a combo de seleo de projetos Origem
   * 
   * @return painel com a combo de seleo de projetos Origem
   */
  private JComponent createSourceComboPanel() {
    JPanel comboPanel = new JPanel(new GridBagLayout());
    // Criando combo de Projeto Origem
    sourceProjectCombo = new ProjectComboBox(mainFrame);
    sourceProjectCombo.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ae) {
        sourceProject =
          openProjectAction(sourceProject, sourceProjectCombo,
            sourceProjectTree);
      }
    });
    comboPanel.add(new JLabel(getString("FILE_TRANSFER_PROJECT_SOURCE")),
      new GBC(0, 0).none());
    comboPanel.add(sourceProjectCombo, new GBC(1, 0).none().insets(0, 5, 0, 0));
    return comboPanel;
  }

  /**
   * Cria o Painel com a combo de seleo de projetos Destino
   * 
   * @return painel com a combo de seleo de projetos Destino
   */
  private JComponent createTargetComboPanel() {
    JPanel comboPanel = new JPanel(new GridBagLayout());
    // Criando combo de Projeto Destino
    targetProjectCombo = new ProjectComboBox(mainFrame);
    targetProjectCombo.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ae) {
        targetProject =
          openProjectAction(targetProject, targetProjectCombo,
            targetProjectTree);
      }
    });
    comboPanel.add(new JLabel(getString("FILE_TRANSFER_PROJECT_TARGET")),
      new GBC(0, 0).none());
    comboPanel.add(targetProjectCombo, new GBC(1, 0).none().insets(0, 5, 0, 0));
    return comboPanel;
  }

  /**
   * Abre o projeto selecionado na combo e atualiza a rvore.
   * 
   * @param project Projeto atualmente aberto na rvore.
   * @param projectCombo Combo onde foi selecionado o projeto que ser aberto.
   * @param projectTree rvore de diretrios que exibe o contedo do projeto.
   * 
   * @return o projeto aberto ou null em caso de erro.
   */
  private CommonClientProject openProjectAction(
    final CommonClientProject project, final ProjectComboBox projectCombo,
    final ProjectTree projectTree) {
    if (projectCombo.getSelectedItem() == null) {
      return null;
    }
    RemoteTask<CommonClientProject> task =
      new RemoteTask<CommonClientProject>() {
        @Override
        public void performTask() throws Exception {
          // Desinstalando o observador do projeto selecionado anteriormente
          if ((project != null)) {
            project.uninstallObserver();
          }

          // Abrindo o projeto correspondente ao item selecionado da combo.
          UserProjectInfo prj =
            (UserProjectInfo) projectCombo.getSelectedItem();
          CommonClientProject newProject =
            CommonClientProject.openProject(prj.getUserId(), prj
              .getProjectName(), true);
          setResult(newProject);
          // Instalando o observador desse projeto.
          newProject.installObserver(new ProjectObserver(newProject));
          projectTree.setProject(newProject);
        }
      };
    if (task.execute(mainFrame, getName(),
      getString("FILE_TRANSFER_WAIT_MESSAGE"))) {
      return task.getResult();
    }
    return null;
  }

  /**
   * Trmino ASAP da aplicao.
   */
  @Override
  public void killApplication() {
    close();
  }

  /**
   * Procedimento de finalizao da aplicao.
   */
  private void close() {
    try {
      // Desinstalando os observadores dos projeto selecionados
      if (sourceProject != null) {
        sourceProject.uninstallObserver();
      }
      if (targetProject != null) {
        targetProject.uninstallObserver();
      }
      // Removendo as combos de ProjectAdminObserver
      sourceProjectCombo.deleteObserver();
      targetProjectCombo.deleteObserver();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  /**
   * Mtodo que permite sempre ao usurio matar a aplicao.
   * 
   * @return o flag indicativo.
   */
  @Override
  public boolean userCanKillApplication() {
    return (true);
  }

  /**
   * Constri e exibe a frame de Transferncia de arquivos.
   * 
   * @param id .
   */
  public FileTransfer(String id) {
    super(id);
    mainFrame = createMainFrame();
    showFrame();
  }

  /**
   * Classe que representa uma combo para seleo de projetos do usurio
   * corrente. Se cadastra como observadora de ProjectAdminObserver para se
   * atualizar caso algum projeto seja removido ou criado.
   */
  private final class ProjectComboBox extends JComboBox implements Observer {
    /** Janela da aplicao */
    private JFrame frame;

    /** Identificador do usurio corrente */
    private Object userId;

    /**
     * Construtor.
     * 
     * @param frame o dilogo que ir conter a combo.
     */
    public ProjectComboBox(JFrame frame) {
      this.frame = frame;
      userId = User.getLoggedUser().getId();
      /* Carregando os projetos do usurio */
      loadProjects();
      setSelectedIndex(-1);
      /* Se cadastrando como observador de projetos em ProjectAdminObserver */
      ProjectAdminObserver.getInstance().addObserver(this);
    }

    /**
     * Atualiza a projectCombo com os projetos existentes para o usurio
     */
    private void loadProjects() {
      try {
        Set<UserProjectInfo> projects =
          CommonClientProject.getAllProjectsInfo(userId);
        final UserProjectInfo[] projsArray = projects.toArray(new UserProjectInfo[0]);
        Arrays.sort(projsArray);
        setModel(new DefaultComboBoxModel(projsArray));
      }
      catch (Exception e) {
        StandardErrorDialogs
          .showErrorDialog(frame, frame.getTitle(), e);
      }
    }

    /**
     * Chamado por notityObservers de ProjectAdminObserver quando um projeto 
     * criado ou removido no servidor. arg ProjectUserEvent
     * 
     * @param o .
     * @param arg .
     */
    public void update(Observable o, Object arg) {
      if (!(arg instanceof ProjectUserEvent)) {
        return;
      }
      ProjectUserEvent event = (ProjectUserEvent) arg;
      CommonProjectInfo info = (CommonProjectInfo) event.item;
      switch (event.type) {
        case ProjectUserEvent.CREATE:
          addItem(info);
          break;
        case ProjectUserEvent.DELETE:
          if ((getSelectedItem()).equals(info.name)) {
            setSelectedIndex(-1);
          }
          removeItem(info);
          break;
      }
    }

    /**
     * Remove a combo como observadora de ProjectAdminObserver
     */
    public void deleteObserver() {
      ProjectAdminObserver.getInstance().deleteObserver(this);
    }
  }
}
