/*
 * $Id:$
 */

package csbase.client.applications.projectsmanager.actions;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import javax.swing.ImageIcon;

import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.applications.ApplicationFrame;
import csbase.client.applications.ApplicationImages;
import csbase.client.applications.projectsmanager.ProjectsManager;
import csbase.client.applications.projectsmanager.actions.core.ProjectsManagerAction;
import csbase.client.applications.projectsmanager.dialogs.StatusDialog;
import csbase.client.applications.projectsmanager.models.ProjectInfoData;
import csbase.client.applications.projectsmanager.models.ProjectsManagerData;
import csbase.client.applications.projectsmanager.proxy.FillProjectsTableAdminTask;
import csbase.client.applications.projectsmanager.proxy.FillProjectsTableUserTask;
import csbase.client.applications.projectsmanager.proxy.core.ProjectsManagerTask;

/**
 * Ao responsvel por criar a thread que ir oter os dados dos projetos para
 * insero na tabela.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class FillProjectsTableAction extends ProjectsManagerAction {

  /**
   * Indica se o preenchimento da tabela deve ser feito no modo administrador ou
   * usurio.
   */
  final private boolean adminMode;

  /**
   * Indica se o espao ocupado dos projetos deve ser carregado tambm.
   */
  final private boolean loadOccupiedSpace;

  /**
   * {@inheritDoc}
   */
  @Override
  public ImageIcon getStandardImageIcon() {
    return ApplicationImages.ICON_REFRESH_16;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void actionDone() throws Exception {
    final ProjectsManager projectsManager = getProjectsManager();

    projectsManager.clearTable();
    projectsManager.clearFilters();

    final ProjectsManagerTask<ProjectInfoData> fptt;
    if (adminMode) {
      fptt = new FillProjectsTableAdminTask(projectsManager, loadOccupiedSpace);
    }
    else {
      fptt = new FillProjectsTableUserTask(projectsManager, loadOccupiedSpace);
    }

    final ApplicationFrame appFrame = projectsManager.getApplicationFrame();
    fptt.execute(appFrame, getString("title"), getString("message"));

    if (fptt.wasCancelled()) {
      final String err = getString("cancelled.message");
      fptt.showError(err);
      return;
    }

    if (fptt.getStatus() != true) {
      final Exception exception = fptt.getError();
      throw exception;
    }

    final ProjectInfoData pid = fptt.getResult();
    final List<ProjectsManagerData> projects = pid.getProjectsData();
    final Set<String> failedProjectNames = pid.getFailedProjectNames();

    if (projects == null) {
      final String errMsg = getString("reading.error.message");
      final String errTitle = getString("reading.error.title");
      StandardDialogs.showErrorDialog(appFrame, errTitle, errMsg);
      return;
    }

    final String message = getString("project.file.error.message");
    if (failedProjectNames.size() > 0) {
      //      for (String projectName : failedProjectNames) {
      //        message.append("\n\u25CF\t " + projectName);
      //      }

      final String conclusionMessage = getString("project.loading.error.title");
      StatusDialog.showLoadingFailureDialog(projectsManager, message,
        new ArrayList<String>(failedProjectNames), conclusionMessage);
    }

    for (ProjectsManagerData prj : projects) {
      projectsManager.addProject(prj, false);
    }
    projectsManager.refreshProjectsTable();
  }

  /**
   * Construtor
   * 
   * @param projectsManager a aplicao
   * @param adminMode indicativo de admin (uso da ao)
   * @param loadOccupiedSpace indicativo de carga de espao ocupado.
   */
  public FillProjectsTableAction(final ProjectsManager projectsManager,
    final boolean adminMode, final boolean loadOccupiedSpace) {
    super(projectsManager);
    this.adminMode = adminMode;
    this.loadOccupiedSpace = loadOccupiedSpace;
  }
}
