package csbase.client.applications.projectsmanager.actions;

import java.util.List;

import javax.swing.ImageIcon;

import csbase.client.applications.ApplicationImages;
import csbase.client.applications.projectsmanager.ProjectsManager;
import csbase.client.applications.projectsmanager.actions.core.ProjectsManagerAction;
import csbase.client.applications.projectsmanager.dialogs.AdminMailDialog;
import csbase.client.applications.projectsmanager.models.ProjectsManagerData;

/**
 * Ao que abre dilogo para envio de emails referentes a projetos.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class OpenMailDialogAction extends ProjectsManagerAction {
  /**
   * Lista de projetos selecionados.
   */
  private List<ProjectsManagerData> projectsManagerData;

  /**
   * Construtor
   * 
   * @param projectsManager A aplicao
   * @param projectsManagerData Lista de projetos selecionados.
   */
  public OpenMailDialogAction(final ProjectsManager projectsManager,
    final List<ProjectsManagerData> projectsManagerData) {
    super(projectsManager);
    this.projectsManagerData = projectsManagerData;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void actionDone() throws Exception {
    if (projectsManagerData == null || projectsManagerData.size() == 0) {
      return;
    }

    final ProjectsManager projectsManager = getProjectsManager();
    final AdminMailDialog amd =
      new AdminMailDialog(projectsManager, projectsManagerData);
    amd.pack();
    amd.center();
    amd.setVisible(true);
  }

  /**
   * Atualiza lista de projetos sobre os quais a ao ser realizada.
   * 
   * @param projectsManagerData lista de projetos.
   */
  public void setProjectsManagerData(
    final List<ProjectsManagerData> projectsManagerData) {
    this.projectsManagerData = projectsManagerData;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public ImageIcon getStandardImageIcon() {
    return ApplicationImages.ICON_SENDMAIL_16;
  }
}
