package csbase.client.applications.projectsmanager.proxy;

import java.rmi.RemoteException;

import csbase.client.applications.projectsmanager.ProjectsManager;
import csbase.client.applications.projectsmanager.proxy.core.ProjectsManagerTask;
import csbase.remote.ClientRemoteLocator;
import csbase.remote.DiskUsageServiceInterface;

/**
 * Task responsvel por obter o tamanho ocupado por diversos projetos
 * selecionados.
 * 
 * @author jnlopes
 * 
 */
public class MultipleProjectSpaceTask extends ProjectsManagerTask<Double[]> {

  /**
   * Nomes dos projetos.
   */
  final private String[] projectsNames;

  /**
   * Logins dos donos de projeto.
   */
  final private String[] ownersIds;

  /**
   * Construtor
   * 
   * @param projectsManager A aplicao.
   * @param projectsNames nomes dos projetos.
   * @param ownersIds logins dos donos dos projetos.
   */
  public MultipleProjectSpaceTask(final ProjectsManager projectsManager,
    final String[] projectsNames, final String[] ownersIds) {
    super(projectsManager);

    this.projectsNames = projectsNames;
    this.ownersIds = ownersIds;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected void performTask() throws Exception {
    final Double[] projectsSpace = getProjectsSpace();
    setResult(projectsSpace);
  }

  /**
   * Calcula o espao ocupado por uma lista de projetos.
   * 
   * @return Espao ocupado por uma lista de projetos.
   * @throws RemoteException se {@link DiskUsageServiceInterface} o fizer.
   */
  private Double[] getProjectsSpace() throws RemoteException {

    final Double[] projectsOccupiedSpace = new Double[projectsNames.length];
    final DiskUsageServiceInterface dus = ClientRemoteLocator.diskUsageService;

    for (int i = 0; i < projectsNames.length; i++) {
      projectsOccupiedSpace[i] =
        dus.getUsedSpaceForProjectMb(ownersIds[i], projectsNames[i]);
    }

    return projectsOccupiedSpace;
  }

}
