package csbase.client.applications.projectsmanager.proxy;

import java.rmi.RemoteException;

import csbase.client.applications.projectsmanager.ProjectsManager;
import csbase.client.applications.projectsmanager.proxy.core.ProjectsManagerTask;
import csbase.remote.ClientRemoteLocator;
import csbase.remote.ProjectServiceInterface;
import csbase.server.services.projectservice.ServerProject;

/**
 * Remote task para remoo de projetos.
 * 
 * @author jnlopes
 * 
 */
public class RemoveProjectTask extends ProjectsManagerTask<Boolean> {

  /**
   * Nome dos projetos.
   */
  final private String[] projectNames;

  /**
   * Ids dos donos dos projetos.
   */
  final private String[] ownerIds;

  /**
   * Construtor
   * 
   * @param projectsManager A aplicao
   * @param projectNames Nomes dos projetos.
   * @param ownerIds Ids dos donos dos projetos.
   */
  public RemoveProjectTask(final ProjectsManager projectsManager,
    final String[] projectNames, final String[] ownerIds) {
    super(projectsManager);
    this.projectNames = projectNames;
    this.ownerIds = ownerIds;

  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected void performTask() {

    boolean res = true;
    final ProjectServiceInterface psi = ClientRemoteLocator.projectService;
    for (int i = 0; i < projectNames.length; i++) {
      try {
        /*
         * FIXME - Deveria haver um mtodo no servio de projetos que indicasse
         * se um projeto est aberto.
         */
        final ServerProject sp =
          ServerProject.getProject(ServerProject.getId(ownerIds[i],
            projectNames[i]));
        if (sp != null) {
          psi.closeProject(sp.getId(), true);
        }
        psi.removeProject(ownerIds[i], projectNames[i]);
      }
      catch (RemoteException re) {
        re.printStackTrace();
        res = false;
      }
    }
    setResult(res);
  }

}
