/*
 * CPULoadDistributionPanel.java
 *
 * $Author: moura $
 * $Revision: 36552 $  - $Date: 2006-02-02 18:13:31 -0200 (Thu, 02 Feb 2006) $
 */
package csbase.client.applications.sgamonitor;

import java.awt.BorderLayout;
import java.awt.Color;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.plaf.basic.BasicProgressBarUI;

import tecgraf.javautils.gui.GUIUtils;
import csbase.client.applications.Application;
import csbase.client.util.MonitoringTheme;
import csbase.logic.SGAInfo;
import csbase.logic.SGASet;

/**
 * A classe <code>CPULoadDistributionPanel</code> modela o painel de
 * monitorao que exibe a distribuio de ocupao de CPU nos ns de um
 * cluster.
 *
 * @author Ana
 * @version $Revision: 36552 $
 */
class CPULoadDistributionPanel extends DynamicMonitorPanel {
  private Application app;
  private int numNodes;
  private JProgressBar unavailableUsage;
  private JProgressBar lightUsage;
  private JProgressBar mediumUsage;
  private JProgressBar highUsage;
  private JProgressBar veryHighUsage;
  private JProgressBar superHighUsage;

  /**
   * Retorna o ttulo do painel.
   *
   * @return ttulo do painel
   */
  public String getTitle() {
    return app.getString("MON_CLUSTER_CPU_USAGE");
  }

  /**
   * Constroi um Gauge de monitorao
   *
   * @param c largura do Gauge
   *
   * @return .
   */
  private JProgressBar createGauge(Color c) {
    JProgressBar gauge = new JProgressBar(JProgressBar.HORIZONTAL, 0, numNodes);
    gauge.setUI(
      new BasicProgressBarUI() {
        protected Color getSelectionBackground() {
          return Color.BLACK;
        }

        protected Color getSelectionForeground() {
          return Color.BLACK;
        }
      });
    gauge.setStringPainted(true);
    gauge.setForeground(c);
    return gauge;
  }

  /**
   * Atualiza as informaes dinmicas exibidas
   *
   * @param obj informaes do servidor
   */
  public void updateDynamicInfo(Object obj) {
    SGASet sga = (SGASet)obj;
    SGAInfo[] dynamicData = sga.getAllInfo();
    int unavailable = 0;
    int light = 0;
    int medium = 0;
    int high = 0;
    int veryHigh = 0;
    int superHigh = 0;
    for (int i = 0; i < numNodes; i++) {
      SGAInfo serverData = dynamicData[i];
      int usage = (int)serverData.getCPULoad1();
      if (usage < 0) {
        unavailable++;
      }
      else if (usage <= 25) {
        light++;
      }
      else if (usage <= 50) {
        medium++;
      }
      else if (usage <= 75) {
        high++;
      }
      else if (usage <= 100) {
        veryHigh++;
      }
      else {
        superHigh++;
      }
    }
    unavailableUsage.setValue(unavailable);
    unavailableUsage.setString("" + unavailable + " " +
      app.getString("MON_NODES"));
    lightUsage.setValue(light);
    lightUsage.setString("" + light + " " + app.getString("MON_NODES"));
    mediumUsage.setValue(medium);
    mediumUsage.setString("" + medium + " " + app.getString("MON_NODES"));
    highUsage.setValue(high);
    highUsage.setString("" + high + " " + app.getString("MON_NODES"));
    veryHighUsage.setValue(veryHigh);
    veryHighUsage.setString("" + veryHigh + " " + app.getString("MON_NODES"));
    superHighUsage.setValue(superHigh);
    superHighUsage.setString("" + superHigh + " " + app.getString("MON_NODES"));
  }

  /**
   * Constri o painel que contm as informaes dinmicas do servidor.
   *
   * @param app .
   * @param server .
   */
  public CPULoadDistributionPanel(Application app, SGASet server) {
    super();
    this.app = app;
    numNodes = server.getNumNodes();
    setLayout(new BorderLayout());
    setBorder(
      BorderFactory.createCompoundBorder(
        BorderFactory.createCompoundBorder(
          BorderFactory.createEmptyBorder(10, 10, 10, 10),
          BorderFactory.createTitledBorder(
            app.getString("MON_LOAD_DISTRIBUTION"))),
        BorderFactory.createEmptyBorder(10, 10, 10, 10)));
    unavailableUsage = createGauge(MonitoringTheme.getRangeErrorColor());
    lightUsage = createGauge(MonitoringTheme.getRangeNormalColor());
    mediumUsage = createGauge(MonitoringTheme.getRangeNormalColor());
    highUsage = createGauge(MonitoringTheme.getRangeWarningColor());
    veryHighUsage = createGauge(MonitoringTheme.getRangeAlertColor());
    superHighUsage = createGauge(MonitoringTheme.getRangePanicColor());
    JComponent[][] components =
    {
      { new JLabel(" 0  -  25 %"), lightUsage },
      { new JLabel("25  -  50 %"), mediumUsage },
      { new JLabel("50  -  75 %"), highUsage },
      { new JLabel("75  - 100 %"), veryHighUsage },
      { new JLabel("> 100 %"), superHighUsage },
      { new JLabel("-----"), unavailableUsage }
    };
    JPanel mp = GUIUtils.createBasicGridPanel(components);
    mp.setBackground(Color.WHITE);
    mp.setBorder(
      BorderFactory.createCompoundBorder(
        BorderFactory.createLoweredBevelBorder(),
        BorderFactory.createEmptyBorder(10, 10, 10, 10)));
    add(mp);
  }
}
