/*
 * ClusterCPULoadInfoHandler.java
 * $Revision: 33587 $
 */
package csbase.client.applications.sgamonitor;

import csbase.logic.SGAInfo;

/**
 * A classe <code>ClusterCPULoadInfoHandler</code> modela um gerenciador de
 * informaes de carga de ns de um cluster.
 *
 * @author Ana
 */
class ClusterCPULoadInfoHandler implements DynamicInfoHandler {
  /** Informaes do cluster */
  private SGAInfo[] clusterData;

  /**
   * Atualiza as informaes dinmicas.
   *
   * @param dynamicData informaes do cluster
   */
  public void updateInfo(Object dynamicData) {
    this.clusterData = (SGAInfo[])dynamicData;
  }

  /**
   * Obtm o nmero total de ns.
   *
   * @return .
   */
  public int getTotalSize() {
    return clusterData.length;
  }

  /**
   * Obtm o nmero de ns vlidos. Em um cluster, todos os ns so vlidos. Um
   * n pode no estar acessvel, porm sempre ser conhecido.
   *
   * @return .
   */
  public int getValidSize() {
    return clusterData.length;
  }

  /**
   * Verifica se um n  vlido.
   *
   * @param index ndice do n
   *
   * @return true se o n  vlido, false caso contrrio
   */
  public boolean isValid(int index) {
    if ((index < clusterData.length) && (clusterData[index] != null)) {
      return true;
    }
    else {
      return false;
    }
  }

  /**
   * Obtm o nome de um n.
   *
   * @param index ndice do n
   *
   * @return nome do n
   */
  public String getName(int index) {
    return clusterData[index].getHostName();
  }

  /**
   * Obtm a carga corrente de um n.
   *
   * @param index ndice do n
   *
   * @return carga do n
   */
  public double getCurrentValue(int index) {
    return clusterData[index].getCPULoad1();
  }

  /**
   * Constri o gerenciador.
   *
   * @param clusterData informaes dos ns do cluster (primeira coleta)
   */
  public ClusterCPULoadInfoHandler(SGAInfo[] clusterData) {
    this.clusterData = clusterData;
  }
}
