/*
 * SGAInfoFrame.java
 *
 * $Author: ururahy $
 * $Revision: 67318 $  - $Date: 2007-09-10 17:03:59 -0300 (Mon, 10 Sep 2007) $
 */
package csbase.client.applications.sgamonitor;

import csbase.client.applications.*;
import csbase.client.remote.srvproxies.SGAProxy;
import csbase.client.util.StandardErrorDialogs;
import csbase.logic.SGASet;
import csbase.remote.ClientRemoteLocator;
import csbase.remote.SGAServiceInterface;

import java.awt.*;
import java.awt.event.*;
import java.rmi.RemoteException;
import java.text.MessageFormat;
import java.util.Vector;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * A classe <code>SGAInfoFrame</code>  responsvel por exibir as informaes
 * detalhadas (estticas e dinmicas) de um servidor (SGA).
 *
 * @author Ana
 * @version $Revision: 67318 $
 */
public class SGAInfoFrame {
  private static int MON_PANEL = 0;
  private static int WARNING_SERVER_PANEL = 1;
  private static int WARNING_SSI_PANEL = 2;

  /** A Aplicao que chamou SGAInfoFrame */
  private Application app;

  /** Nome do servidor monitorado */
  private String sgaName;

  /** Thread responsvel pela atualizao das informaes do SGA */
  private Thread updateThread = null;

  /** Flag indicativo de sada da thread de atualizao das informaes do SGA */
  private boolean exitUpdateThread = false;

  /** Intervalo entre atualizaes de informao */
  private long updateInterval;

  /** Servidor SGA */
  private SGASet server;

  /** Painis de monitorao */
  private Vector monPanels;

  /** Elementos de Interface */
  private ApplicationComponentFrame mainFrame;
  private String mainFrameTitle;
  private JTextField serverState;

  /** Painel com informaes dinmicas do servidor */
  private DynamicInfoPanel dynamicInfoPanel;

  /**
   * Termina a monitorao de um SGA.
   */
  private void close() {
    exitUpdateThread = true;
    try {
      updateThread.interrupt();
    }
    catch (Throwable t) {} // Applet nao pode fazer isto
  }

  /**
   * Constri os elementos da interface de monitorao de um servidor
   */
  private void createWidgets() {
    dynamicInfoPanel = new DynamicInfoPanel();
    mainFrame = createMainFrame();
  }

  /**
   * Exibe a janela de monitorao de um servidor.
   */
  private void display() {
    mainFrame.pack();
    mainFrame.center(app.getApplicationFrame());
    mainFrame.setVisible(true);
  }

  /**
   * Cria a janela de monitorao de um servidor.
   *
   * @return referncia para a janela criada
   */
  private ApplicationComponentFrame createMainFrame() {
    ApplicationComponentFrame mainFrame =
      new ApplicationComponentFrame(mainFrameTitle, app);
    mainFrame.addWindowListener(
      new WindowAdapter() {
        public void windowClosing(WindowEvent we) {
          close();
        }
      });

    Container cp = mainFrame.getContentPane();
    cp.setLayout(new BorderLayout(5, 5));
    cp.add(createGeneralInfoPanel(), BorderLayout.NORTH);
    cp.add(dynamicInfoPanel, BorderLayout.CENTER);
    cp.add(createButtonsPanel(), BorderLayout.SOUTH);
    return mainFrame;
  }

  /**
   * Constri o painel que contm as informaes gerais do servidor.
   *
   * @return o painel criado
   */
  private JPanel createGeneralInfoPanel() {
    JPanel generalPanel = new JPanel(new GridBagLayout());
    GridBagConstraints constraints = new GridBagConstraints();

    // Atributos (constraints) comuns a todos os elementos
    Insets defInsets = new Insets(2, 2, 2, 2);
    Insets leftMarginInsets = new Insets(2, 20, 2, 2);
    constraints.anchor = GridBagConstraints.WEST; // alinhado  esquerda
    constraints.weightx = 100; // a distribuio de espao  uniforme
    constraints.weighty = 100;
    constraints.gridwidth = 1;
    constraints.gridheight = 1;
    constraints.insets = defInsets;

    // Nome do servidor
    constraints.fill = GridBagConstraints.NONE; // sem resize
    constraints.gridx = 0;
    constraints.gridy = 0;
    generalPanel.add(
      new JLabel(app.getString("MON_SERVER")+" : "), constraints);
    constraints.fill = GridBagConstraints.HORIZONTAL; // sem resize
    constraints.gridx = 1;
    constraints.gridy = 0;
    JTextField name = new JTextField(server.getName());
    name.setEditable(false);
    generalPanel.add(name, constraints);

    // Nmero de ns
    constraints.fill = GridBagConstraints.NONE; // sem resize
    constraints.gridx = 0;
    constraints.gridy = 1;
    generalPanel.add(
      new JLabel(app.getString("MON_NODES_NUM")+" : "), constraints);
    constraints.fill = GridBagConstraints.HORIZONTAL; // sem resize
    constraints.gridx = 1;
    constraints.gridy = 1;
    JTextField nodes = new JTextField(""+server.getNumNodes());
    nodes.setEditable(false);
    generalPanel.add(nodes, constraints);

    // Plataforma
    constraints.fill = GridBagConstraints.NONE; // sem resize
    constraints.insets = leftMarginInsets;
    constraints.gridx = 2;
    constraints.gridy = 0;
    generalPanel.add(
      new JLabel(app.getString("MON_PLATFORM")+" : "), constraints);
    constraints.fill = GridBagConstraints.HORIZONTAL; // sem resize
    constraints.insets = defInsets;
    constraints.gridx = 3;
    constraints.gridy = 0;
    JTextField plat = new JTextField(server.getPlatformId());
    plat.setEditable(false);
    generalPanel.add(plat, constraints);

    // Estado
    constraints.fill = GridBagConstraints.NONE; // sem resize
    constraints.insets = leftMarginInsets;
    constraints.gridx = 2;
    constraints.gridy = 1;
    generalPanel.add(new JLabel(app.getString("MON_STATE")+" : "), constraints);
    constraints.fill = GridBagConstraints.HORIZONTAL; // sem resize
    constraints.insets = defInsets;
    constraints.gridx = 3;
    constraints.gridy = 1;
    serverState = new JTextField();
    serverState.setEditable(false);
    updateServerState();
    generalPanel.add(serverState, constraints);
    return generalPanel;
  }

  /**
   * Atualiza a informao de estado do servidor.
   */
  private void updateServerState() {
    serverState.setText(app.getString("MON_"+server.getStatus()));
  }

  /**
   * Cria o painel de botes.
   *
   * @return painel de botes
   */
  private JPanel createButtonsPanel() {
    JPanel bp = new JPanel(new FlowLayout());

    // Boto de fechamento da janela
    JButton closeButton = new JButton(app.getString("MON_CLOSE"));
    closeButton.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent ev) {
          close();
          mainFrame.setVisible(false);
        }
      });
    bp.add(closeButton);
    return bp;
  }

  /**
   * Cria os painis de monitorao para um servidor "single" (no cluster).
   *
   * @return um vetor com os painis criados
   */
  private Vector createSingleServerMonitorPanels() {
    Vector<DynamicMonitorPanel> panels = new Vector<DynamicMonitorPanel>();
    panels.add(new MainNodeUsagePanel(app, server));
    panels.add(new TrendUsagePanel(app, updateInterval/1000)); // seconds
    return panels;
  }

  /**
   * Cria os painis de monitorao para um servidor do tipo cluster
   *
   * @return um vetor com os painis criados
   */
  private Vector createClusterServerMonitorPanels() {
    Vector<DynamicMonitorPanel> panels = new Vector<DynamicMonitorPanel>();
    panels.add(new MainNodeUsagePanel(app, server));
    panels.add(new ClusterStarUsagePanel(app, server.getAllInfo()));
    panels.add(new ClusterInfoPanel(app, server));
    panels.add(new CPULoadDistributionPanel(app, server));
    panels.add(new MemoryDistributionPanel(app, server));
    return panels;
  }

  /**
   * Painel que contm as informaes dinmicas do servidor.
   */
  private class DynamicInfoPanel extends JPanel {
    private int currentDynPanel;
    private DynamicMonitorPanel currentMonPanel;

    /**
     * Atualiza as informaes dinmicas do servidor. Essa atualizao 
     * comandada pela thread de tratamento de eventos do AWT.
     */
    public void updateInfo() {
      // Verifica o estado da comunicao com o servidor
      if (!server.getAlive()) {
        if (currentDynPanel != WARNING_SERVER_PANEL) {
          ((CardLayout)getLayout()).show(
            this, String.valueOf(WARNING_SERVER_PANEL));
          currentDynPanel = WARNING_SERVER_PANEL;
        }
      }
      else {
        if (
          (currentDynPanel == WARNING_SERVER_PANEL) ||
          (currentDynPanel == WARNING_SSI_PANEL)) {
          ((CardLayout)getLayout()).show(this, String.valueOf(MON_PANEL));
          currentDynPanel = MON_PANEL;
        }

        // Atualiza informaes em todos os painis registrados
        for (int i = 0; i < monPanels.size(); i++) {
          DynamicMonitorPanel mp = (DynamicMonitorPanel)monPanels.get(i);
          mp.updateDynamicInfo(server);
        }

        // Redesenha o painel de monitorao corrente
        if (currentMonPanel != null) {
          currentMonPanel.repaint();
        }
      }
    }

    /**
     * Invalida a monitorao (mostrando painel de erro).
     */
    private void invalidateInfo() {
      if (currentDynPanel != WARNING_SSI_PANEL) {
        ((CardLayout)getLayout()).show(this, String.valueOf(WARNING_SSI_PANEL));
        currentDynPanel = WARNING_SSI_PANEL;
      }
    }

    /**
     * Constri o painel que contm as informaes dinmicas do servidor. Este
     * painel  diagramado com um CardLayout para permitir a  visualizao dos
     * painis de alarme em sobreposio ao painel de monitorao.
     */
    public DynamicInfoPanel() {
      super(new CardLayout());

      // Cria os painis de monitorao de acordo com o tipo de servidor
      if (server.isCluster()) {
        monPanels = createClusterServerMonitorPanels();
      }
      else {
        monPanels = createSingleServerMonitorPanels();
      }
      final JTabbedPane monitorTabs = new JTabbedPane();
      if (monPanels != null) {
        for (int i = 0; i < monPanels.size(); i++) {
          DynamicMonitorPanel dp = (DynamicMonitorPanel)monPanels.get(i);
          monitorTabs.add(dp.getTitle(), dp);
        }
        currentMonPanel = (DynamicMonitorPanel)monPanels.get(0);
      }
      else {
        currentMonPanel = null;
      }
      monitorTabs.addChangeListener(
        new ChangeListener() {
          public void stateChanged(ChangeEvent ce) {
            DynamicMonitorPanel dp =
              (DynamicMonitorPanel)monitorTabs.getSelectedComponent();
            if (dp != null) {
              currentMonPanel = dp;
              currentMonPanel.repaint();
            }
          }
        });
      add(monitorTabs, String.valueOf(MON_PANEL));
      currentDynPanel = MON_PANEL;

      // Cria o painel de alarme (erro comunicao com servidor SGA)
      JPanel warningPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
      JLabel warningMessage =
        new JLabel(
          MessageFormat.format(
            app.getString("MON_ERROR_COMM_SGA"), new Object[] { sgaName }));
      warningMessage.setIcon(ApplicationImages.ICON_WARNING_32);
      warningPanel.add(warningMessage);
      add(warningPanel, String.valueOf(WARNING_SERVER_PANEL));

      // Cria o painel de alarme (erro comunicao com SSI)
      JPanel warningSSIPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
      JLabel warningSSIMessage =
        new JLabel(
          MessageFormat.format(
            app.getString("MON_SERVER_NO_INFO"), new Object[] { sgaName }));
      warningSSIMessage.setIcon(ApplicationImages.ICON_ERROR_32);
      warningSSIPanel.add(warningSSIMessage);
      add(warningSSIPanel, String.valueOf(WARNING_SSI_PANEL));
    }
  }

  /**
   * Constri o monitorador detalhado de um servidor/SGA.
   *
   * @param _sgaName nome do servidor/SGA
   * @param app .
   */
  public SGAInfoFrame(String _sgaName, final Application app) {
    this.app = app;
    this.sgaName = _sgaName;
    mainFrameTitle =
      MessageFormat.format(
        app.getString("MON_SERVER_TITLE"), new Object[] { sgaName });

    /* Obtem as informaes iniciais do servidor
       e o intervalo de atualizao */
    server = null;
    int updateIntervalAsInt = SGAProxy.getUpdateInterval();
    if (updateIntervalAsInt == -1) {
      return;
    }
    updateInterval = (long)updateIntervalAsInt*(long)1000;
    server = SGAProxy.getSGASet(sgaName);
    if (server == null) {
      //Pegando a frame principal da aplicao
      JFrame jFrame = app.getApplicationFrame();
      if (jFrame == null) {
        return;
      }
      String msg =
        MessageFormat.format(
          app.getString("MON_ERROR_COMM_SGA"), new Object[] { sgaName });
      StandardErrorDialogs.showErrorDialog(jFrame, mainFrameTitle, msg);
      return;
    }

    /* Constri e exibe os elementos de interface */
    createWidgets();
    display();
    dynamicInfoPanel.updateInfo();

    /*
     * Thread responsvel pela atualizao das informaes dinmicas
     */
    updateThread =
      new Thread(
        new Runnable() {
          public void run() {
            while (!exitUpdateThread) {
              try {
                SGAServiceInterface sgaService = ClientRemoteLocator.sgaService;
                server = sgaService.getSGASet(sgaName);

                // Enfileira a atualizao das informaes dinmicas na janela
                SwingUtilities.invokeLater(
                  new Runnable() {
                    public void run() {
                      if (server == null) {
                        StandardErrorDialogs.showErrorDialog(
                          mainFrame, mainFrameTitle,
                          MessageFormat.format(
                            app.getString("MON_SERVER_DEREG"),
                            new Object[] { sgaName }));
                        close();
                        mainFrame.setVisible(false);
                      }
                      else {
                        updateServerState();
                        dynamicInfoPanel.updateInfo();
                      }
                    }
                  });
                Thread.sleep(updateInterval); // miliseconds
              }
              catch (InterruptedException e) {}
              catch (RemoteException e) {
                dynamicInfoPanel.invalidateInfo();
              }
            }
          }
        });
    updateThread.start();
  }
}
