package csbase.client.applications.sgamonitor;

import csbase.client.applicationmanager.ApplicationException;
import csbase.client.applications.Application;
import csbase.logic.SGAAdminPermission;
import csbase.logic.User;

/**
 * Aplicao SGAMonitor - de acordo com a propriedade monitoringType, abre
 * ServerMonitorFrame ou GroupServerMonitorFrame
 * 
 * @author Ana Lcia Moura
 * @author Ana Lodi
 */
public class SGAMonitor extends Application {

  /**
   * Tipo de monitorao: Servidor.
   */
  private static String server = "server";

  /**
   * Tipo de monitorao: Grupos de servidores.
   */
  private static String serverGroup = "servergroup";

  /**
   * Tipo de monitorao: Desativao dos servidores.
   */
  private static String sgaDisable = "sgadisable";

  /**
   * Tipo de monitorao: Comandos.
   */
  private static String command = "command";

  /**
   * Tipo da Aplicao.
   */
  private String type;

  /**
   * A janela.
   */
  private MonitorFrame mon;

  /**
   * Trmino ASAP da aplicao.
   */
  @Override
  public void killApplication() {
    mon.close();
  }

  /**
   * Mtodo que permite sempre ao usurio matar a aplicao.
   * 
   * @return sempre vedadeiro.
   */
  @Override
  public boolean userCanKillApplication() {
    return true;
  }

  /**
   * Construtor padro.
   * 
   * @param id identificator da aplicao.
   * @throws ApplicationException em caso de erro.
   */
  public SGAMonitor(String id) throws ApplicationException {
    super(id);
    type = getStringSpecificProperty("monitoringType");
    if (type == null) {
      throw new ApplicationException("undefined type");
    }
    if (type.equals(server)) {
      User user = User.getLoggedUser();
      SGAAdminPermission sgaPermission =
        user.getPermission(SGAAdminPermission.class);
      // Se o usurio for admin ou tiver permisso para executar sgas, o frame
      // a ser mostrado no menu de monitorao ser o mesmo a ser mostrado no
      // menu de gerenciamento de servidores (dilogo para reinicializao e 
      // desativao de servidores), enquanto que se o usurio no for admin ou 
      // no tiver permisso para executar sgas, o frame a ser carregado  o de 
      // monitorao de servidores.
      if (user.isAdmin() || (sgaPermission != null))
        mon = new SGADisableFrame(this);
      else
        mon = new ServerMonitorFrame(this);
    }
    else if (type.equals(serverGroup)) {
      mon = new ServerGroupMonitorFrame(this);
    }
    else if (type.equals(sgaDisable)) {
      mon = new SGADisableFrame(this);
    }
    else if (type.equals(command)) {
      // Obtm a propriedade que habilita a monitorao da fila de processos.
      boolean queueMonitoring = getBooleanSpecificProperty("queue.monitoring", false);
      mon = new CommandMonitorFrame(this, queueMonitoring);
    }
  }
}
