/*
 * $Id: DirectoryContentsTransferHandler.java 75203 2008-04-18 12:06:37Z taciana
 * $
 */
package csbase.client.desktop.dircontents;

import java.awt.Window;
import java.awt.datatransfer.DataFlavor;

import javax.swing.JComponent;
import javax.swing.JTable;

import csbase.client.project.AbstractProjectFileTransferHandler;
import csbase.logic.ClientProjectFile;

/**
 * @author Tecgraf/PUC-Rio
 */
public class DirectoryContentsTransferHandler extends
  AbstractProjectFileTransferHandler {

  /**
   * Constri um handler de transferncia de objetos de uma tabela de projeto.
   * 
   * @param ownerWindow A janela sobre a qual uma DesktopTask deve ser efetuada.
   * @param ownerTitle O ttulo da janela de DesktopTask
   */
  public DirectoryContentsTransferHandler(Window ownerWindow, String ownerTitle) {
    super(ownerWindow, ownerTitle);
  }

  /**
   * Verifica se o componente pode fazer importao.
   * 
   * @param c O componente sobre o qual a transferncia foi iniciada.
   * @param flavor Os formatos de dados dos objetos que esto sendo
   *        transferidos.
   * 
   * @return Verdadeiro, se o componente permite iniciar a transferncia dos
   *         objetos com o formato especificado.
   */
  @Override
  public boolean canImport(JComponent c, DataFlavor[] flavor) {
    if (!(c instanceof JTable)) {
      return false;
    }
    JTable table = (JTable) c;
    if (table.getEditingRow() != -1) {
      return false;
    }
    return super.canImport(c, flavor);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected ClientProjectFile[] getFilesToExport(JComponent source) {
    if (source instanceof JTable) {
      JTable table = (JTable) source;

      int[] selectedRows = table.getSelectedRows();

      if (0 < selectedRows.length) {
        ClientProjectFile[] files = new ClientProjectFile[selectedRows.length];
        for (int inx = 0; inx < files.length; inx++) {
          files[inx] = (ClientProjectFile) table.getValueAt(selectedRows[inx],
            DirectoryContentsPanel.NAME_COL_INDEX);
          System.out.println(files[inx].getName() + " - "
            + files[inx].getParent());
        }
        return files;
      }
    }

    return null;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected ClientProjectFile getDestinationDirectory(JComponent destination) {
    if (destination instanceof JTable) {
      JTable table = (JTable) destination;

      int selectedRow = table.getSelectedRow();
      ClientProjectFile dir = (ClientProjectFile) table.getValueAt(selectedRow,
        DirectoryContentsPanel.NAME_COL_INDEX);

      return dir;
    }
    else {
      return null;
    }
  }
}
