/*
 * XLauncherFrame.java
 * $Author: taciana $
 * $Revision: 76954 $  - $Date: 2008-06-06 16:24:00 -0300 (Fri, 06 Jun 2008) $
 */
package csbase.client.facilities.xlauncher;

import csbase.client.Client;
import csbase.client.desktop.*;
import csbase.client.remote.srvproxies.SGAProxy;
import csbase.client.remote.srvproxies.XAppProxy;
import csbase.client.util.ClientUtilities;
import csbase.client.util.StandardErrorDialogs;
import csbase.logic.*;

import java.awt.*;
import java.awt.event.*;
import java.net.InetAddress;
import java.util.*;

import javax.swing.*;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.GUIUtils;
import tecgraf.javautils.gui.StandardDialogs;

/**
 * A classe <code>XLauncherFrame</code> implementa o dilogo de  execuo de
 * aplicaes.
 *
 * @author Andr Luiz Clinio
 * @version $Revision: 76954 $
 */
public class XLauncherFrame {
  /** Modelo de lista vazia. */
  private final DefaultComboBoxModel emptyModel = new DefaultComboBoxModel();

  /** Ttulo da janela */
  private String frameTitle = LNG.get("XLAUNCHER_TITLE");

  /** Thread de atualizao das informaes provenientes do SSI. */
  private Thread updateThread = null;

  /** Flag que indica que a thread de atualizao deve ser terminada */
  private boolean exitUpdateThread = false;

  /** Dilogo principal do lanador de aplicaes X-Windows. */
  private DesktopComponentFrame mainFrame;

  /** Label do nome da aplicao selecionada */
  private JLabel appLabel;

  /** Lista com a aplicao selecionada. */
  private JComboBox appCombo = null;

  /** Label do display utilizado. */
  private JLabel displayLabel;

  /** Lista com o display selecionado. */
  private JTextField displayText = null;

  /** Toggle de seleo manual de display */
  private JCheckBox manualDisplayToggle = null;

  /** Label do servidor utilizado. */
  private JLabel serverLabel;

  /** Lista com o servidor selecionado. */
  private JComboBox serverCombo = null;

  /** Toggle de seleo manual de servidor */
  private JCheckBox manualServerToggle = null;

  /** Label do no' utilizado. */
  private JLabel nodeLabel;

  /** Lista com o no' selecionado. */
  private JComboBox nodeCombo = null;

  /** Toggle de seleo manual de no' */
  private JCheckBox manualNodeToggle = null;

  /** Boto de execuo. */
  private JButton runButton = null;

  /** Eventos de erro sinalizados na interface */
  private DesktopErrorEvent currentError = null;

  /** Servidores registrados e suas informaes */
  private Hashtable registeredServers = null;

  /**
   * Trmino do dilogo
   */
  private void close() {
    exitUpdateThread = true;
    if (updateThread != null) {
      try {
        updateThread.interrupt();
      }
      catch (Throwable t) {} // Applet nao pode fazer isto
    }
    if (mainFrame != null) {
      mainFrame.close();
    }
  }

  /**
   * Exibe a janela de execuo de aplicaes
   */
  private void startFrame() {
    if (mainFrame != null) {
      mainFrame.showStatusBar();
      mainFrame.pack();
      mainFrame.setResizable(false);
      mainFrame.center(DesktopFrame.getInstance().getDesktopFrame());
      mainFrame.setVisible(true);
    }
  }

  /**
   * Cria a janela de execuo de aplicaes
   *
   * @return referncia para a janela criada
   */
  private DesktopComponentFrame createMainFrame() {
    DesktopComponentFrame mainFrame = new DesktopComponentFrame(frameTitle);
    mainFrame.addWindowListener(
      new WindowAdapter() {
        public void windowClosing(WindowEvent we) {
          close();
        }
      });

    Container cp = mainFrame.getContentPane();
    cp.add(createSelectionPanel(), BorderLayout.CENTER);
    cp.add(createButtonsPanel(), BorderLayout.SOUTH);
    return mainFrame;
  }

  /**
   * Cria o painel de botes.
   *
   * @return painel de botes
   */
  private JPanel createButtonsPanel() {
    JPanel bp = new JPanel(new FlowLayout());

    // Boto de fechamento da janela
    JButton cancelButton = new JButton(LNG.get("XLAUNCHER_CLOSE_BUTTON"));
    cancelButton.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent ev) {
          close();
        }
      });
    // Boto de execuo da janela
    runButton = new JButton(LNG.get("XLAUNCHER_EXECUTE_BUTTON"));
    runButton.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent ev) {
          runApplication();
        }
      });
    bp.add(runButton);
    bp.add(cancelButton);
    ClientUtilities.adjustEqualSizes(cancelButton, runButton);
    return bp;
  }

  /**
   * Monta o display default do cliente.
   *
   * @return uma string com esse valor
   */
  private String getDefaultDisplay() {
    try {
      String addr = Client.getInstance().getParameter("client_address");
      if (addr == null) {
        addr = InetAddress.getLocalHost().getHostName();
      }
      return addr+":0.0";
    }
    catch (Exception e) {
      return "";
    }
  }

  /**
   * Cria o painel de seleo de appritmo e servidor
   *
   * @return painel de seleo
   */
  private JPanel createSelectionPanel() {
    appLabel = new JLabel(LNG.get("XLAUNCHER_APPLICATION_LABEL"));
    appCombo = new JComboBox(new DefaultComboBoxModel());
    /* Display */
    displayLabel = new JLabel(LNG.get("XLAUNCHER_DISPLAY_LABEL"));
    displayText = new JTextField(15);
    displayText.setText(getDefaultDisplay());
    manualDisplayToggle = new JCheckBox(
        LNG.get("XLAUNCHER_DISPLAY_TOGGLE"), false);
    manualDisplayToggle.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent ae) {
          updateDisplayList();
        }
      });
    updateDisplayList();
    /* Servidor */
    serverLabel = new JLabel(LNG.get("XLAUNCHER_SERVER_LABEL"));
    serverCombo = new JComboBox(new DefaultComboBoxModel());
    serverCombo.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent ae) {
          updateNodeList();
        }
      });
    manualServerToggle = new JCheckBox(
        LNG.get("XLAUNCHER_SERVER_TOGGLE"), true);
    /* ACCOXX */
    manualServerToggle.setEnabled(true);
    serverCombo.setEnabled(true);
    manualServerToggle.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent ae) {
          updateServerWidgets();
        }
      });
    /* No do cluster */
    nodeLabel = new JLabel(LNG.get("XLAUNCHER_NODE_LABEL"));
    nodeCombo = new JComboBox(new DefaultComboBoxModel());
    manualNodeToggle = new JCheckBox(LNG.get("XLAUNCHER_NODE_TOGGLE"), false);
    /* ACCOXX */
    manualNodeToggle.setEnabled(false);
    nodeCombo.setEnabled(false);
    manualNodeToggle.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent ae) {
          updateNodeWidgets();
        }
      });
    updateNodeList();
    JComponent[][] selPanelComponents =
      {
        { appLabel, appCombo },
        { displayLabel, manualDisplayToggle, displayText },
        { serverLabel, manualServerToggle, serverCombo },
        { nodeLabel, manualNodeToggle, nodeCombo },
      };
    JPanel selPanel = GUIUtils.createBasicGridPanel(selPanelComponents);
    JPanel allPanel = new JPanel(new BorderLayout());
    allPanel.add(selPanel, BorderLayout.CENTER);
    return allPanel;
  }

  /**
   * Habilitao da interface (recuperao de erro).
   */
  private void resetError() {
    if (currentError != null) {
      mainFrame.resetError(currentError);
      currentError = null;
    }
  }

  /**
   * Desabilitao da interface por erro.
   *
   * @param errMsg .
   * @param disabled .
   */
  private void signalError(String errMsg, JComponent[] disabled) {
    if (currentError != null) {
      resetError();
    }
    final String _errMsg = errMsg;
    final JComponent[] _disabled = disabled;
    SwingUtilities.invokeLater(
      new Runnable() {
        public void run() {
          currentError = new DesktopErrorEvent(_errMsg, null, _disabled);
          mainFrame.signalError(currentError);
        }
      });
  }

  /**
   * Atualizao da interface dinmica da lista de servidores.
   */
  private void updateDisplayList() {
    if (manualDisplayToggle.isSelected()) {
      displayText.setEnabled(true);
      displayText.setEditable(true);
    }
    else {
      displayText.setEnabled(false);
      displayText.setEditable(false);
      displayText.setText(getDefaultDisplay());
    }
  }

  /**
   * Atualizao da interface dinmica da lista de servidores.
   *
   * @return .
   */
  private boolean updateAppList() {
    //Object[] ids = XAppProxy.getAllInfo();
    Object[] ids = null;
    if ((ids == null) || (ids.length == 0)) {
      JComponent[] disabled = { appCombo, runButton };
      signalError(LNG.get("XLAUNCHER_ERROR_NO_APPS"), disabled);
      appCombo.setModel(emptyModel);
      return false;
    }
    DefaultComboBoxModel appModel = new DefaultComboBoxModel(ids);
    appCombo.setModel(appModel);
    return true;
  }

  /**
   * Atualiza estado dos elementos relacionados ao servidor
   */
  private void updateServerWidgets() {
    if (manualServerToggle.isSelected()) {
      serverCombo.setEnabled(true);
      updateNodeList();
    }
    else {
      serverCombo.setEnabled(false);
      invalidateNodeSelection();
    }
  }

  /**
   * Atualizao da interface dinmica da lista de servidores.
   *
   * @param forced .
   *
   * @return .
   */
  private boolean updateServerList(boolean forced) {
    if (!forced) {
      if (manualServerToggle.isSelected()) {
        return true;
      }
    }
    Vector<String> sgaNames = new Vector<String>();
    Enumeration sgas = registeredServers.elements();
    while (sgas.hasMoreElements()) {
      SGASet sgaInfo = (SGASet)sgas.nextElement();
      if (sgaInfo.mayExecuteCommand()) {
        sgaNames.add(sgaInfo.getName());
      }
    }
    if (sgaNames.size() == 0) {
      JComponent[] disabled = { serverCombo, runButton };
      signalError(LNG.get("XLAUNCHER_ERROR_NO_SERVERS"), disabled);
      serverCombo.setModel(emptyModel);
      return false;
    }
    Collections.sort(sgaNames);
    DefaultComboBoxModel sgaModel = new DefaultComboBoxModel(sgaNames);
    serverCombo.setModel(sgaModel);
    updateNodeList();
    return true;
  }

  /**
   * Invalida a seleo de ns.
   */
  private void invalidateNodeSelection() {
    manualNodeToggle.setSelected(false);
    manualNodeToggle.setEnabled(false);
    nodeCombo.setModel(emptyModel);
    nodeCombo.setEnabled(false);
  }

  /**
   * Atualizao da interface dinmica da lista de ns
   */
  private void updateNodeList() {
    String selectedSGA = (String)serverCombo.getSelectedItem();
    if (selectedSGA == null) {
      return;
    }
    SGASet sga = SGAProxy.getSGASet(selectedSGA);
    if (!sga.isCluster()) {
      invalidateNodeSelection();
      return;
    }
    String[] nodesNames = sga.getAllNames();
    Arrays.sort(nodesNames);
    DefaultComboBoxModel sgaModel = new DefaultComboBoxModel(nodesNames);
    nodeCombo.setModel(sgaModel);
    if (manualServerToggle.isSelected()) {
      manualNodeToggle.setEnabled(true);
    }
  }

  /**
   * Atualiza estado dos elementos relacionados ao n
   */
  private void updateNodeWidgets() {
    if (manualNodeToggle.isEnabled() && manualNodeToggle.isSelected()) {
      nodeCombo.setEnabled(true);
    }
    else {
      nodeCombo.setEnabled(false);
    }
  }

  /**
   * Atualizao da interface dinmica.
   */
  private void updateInterface() {
    SwingUtilities.invokeLater(
      new Runnable() {
        public void run() {
          if (!updateServerList(false)) {
            return;
          }
          resetError();
        }
      });
  }

  /**
   * Thread de atualizao de informaes provenientes do SSI.
   */
  private void runThread() {
    updateThread =
      new Thread(
        new Runnable() {
          public void run() {
            while (!exitUpdateThread) {
              try {
                updateInterface();
                Thread.sleep(5000);
              }
              catch (InterruptedException ie) {}
            }
          }
        });
    updateThread.start();
  }

  /**
   * Retorna o identificador da aplicao selecionada.
   *
   * @return .
   */
  private Object getApplicationId() {
    XApplicationInfo info = (XApplicationInfo)appCombo.getSelectedItem();
    return info.getId();
  }

  /**
   * Retorna o nome do SGA requisitado pelo usurio ou null se este ajuste for
   * feito de modo automtico.
   *
   * @return .
   */
  private String getHostName() {
    try {
      if (manualServerToggle.isSelected()) {
        return (String)serverCombo.getSelectedItem();
      }
      return null;
    }
    catch (Exception e) {
      return null;
    }
  }

  /**
   * Retorna o nome do n do SGA requisitado pelo usurio ou null se este
   * ajuste for feito de modo automtico.
   *
   * @return .
   */
  private String getNodeName() {
    try {
      if (manualNodeToggle.isSelected()) {
        return (String)nodeCombo.getSelectedItem();
      }
      return null;
    }
    catch (Exception e) {
      return null;
    }
  }

  /**
   * Retorna o nome do display requisitado pelo usurio ou null se este ajuste
   * for feito de modo automtico.
   *
   * @return .
   */
  private String getDisplayName() {
    String txt = displayText.getText().trim();
    if (txt.equals("")) {
      return null;
    }
    return txt;
  }

  /**
   * Solicita a execuo de uma aplicao.
   */
  private void runApplication() {
    try {
      Object appId = getApplicationId();
      if (appId == null) {
        return;
      }
      String prjName = getProject().getName();
      Object prjOwner = getProject().getUserId();
      String dspName = getDisplayName();
      if (dspName == null) {
        ClientUtilities.showErrorByTextField(
          null, LNG.get("XLAUNCHER_ERROR_NO_DISPLAY"), displayLabel, displayText);
        return;
      }
      String sgaName = getHostName();
      boolean status =
        XAppProxy.runApplication(getProject().getId(), appId, dspName, sgaName);
      if (status) {
        close();
      }
    }
    catch (Exception e) {
      StandardErrorDialogs.showErrorDialog(
        null, LNG.get("XLAUNCHER_TITLE"), e.getMessage(), e);
      close();
    }
  }

  /**
   * Consulta ao projeto corrente.
   *
   * @return o projeto corrente.
   */
  private CommonClientProject getProject() {
    DesktopFrame desktop = DesktopFrame.getInstance();
    if (desktop == null) {
      return null;
    }
    return desktop.getProject();
  }

  /**
   * Constri e exibe o dilogo de execuo de aplicaes
   *
   * @param owner a janela a partir da qual este dilogo foi criado
   */
  public XLauncherFrame(Window owner) {
    if (true) {
      StandardDialogs.showErrorDialog(
        owner, LNG.get("XAPPLICATION_EXECUTE"),
        "As aplicaes X esto em processo de desativao.\n" +
        "Contacte o administrador do sistema.");
      return;
    }
    try {
      registeredServers = SGAProxy.getRegisteredServers();
      CommonClientProject project = getProject();
      if (project == null) {
        throw new Exception("project == null");
      }

      // #TODO Essa varivel nunca  lida:  necessria a execuo do mtodo
      // XAppProxy.getAllInfo()?
      //Object[] ids = XAppProxy.getAllInfo();
    }
    catch (Exception e) {
      StandardErrorDialogs.showErrorDialog(owner, frameTitle, e.getMessage(), e);
      return;
    }
    mainFrame = createMainFrame();
    if (!updateAppList()) {
      StandardDialogs.showErrorDialog(
        owner, LNG.get("XAPPLICATION_EXECUTE"),
        LNG.get("XLAUNCHER_ERROR_NO_APPS"));
      return;
    }
    updateServerList(true);
    resetError();
    startFrame();
    runThread();
  }
}
