package csbase.client.ias;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import tecgraf.javautils.LNG;
import tecgraf.javautils.configurationmanager.Configuration;
import tecgraf.javautils.configurationmanager.ConfigurationManager;
import tecgraf.javautils.configurationmanager.ConfigurationManagerException;
import tecgraf.javautils.gui.StandardDialogs;
import tecgraf.javautils.gui.table.DefaultObjectTableProvider;
import tecgraf.javautils.gui.table.ObjectTableModel;
import tecgraf.javautils.gui.table.ObjectTableProvider;
import tecgraf.javautils.gui.table.SortableTable;
import csbase.client.desktop.DesktopComponentDialog;
import csbase.client.desktop.DesktopFrame;
import csbase.client.desktop.LocalTask;
import csbase.client.desktop.RemoteTask;
import csbase.client.kernel.ClientException;
import csbase.client.project.ClientProjectFileType;
import csbase.client.project.ProjectFileChooser;
import csbase.client.project.ProjectFileChooserSave;
import csbase.client.project.ProjectTreePath;
import csbase.client.project.tasks.CreateFileTask;
import csbase.client.remote.srvproxies.UserProxy;
import csbase.client.util.ClientUtilities;
import csbase.client.util.StandardErrorDialogs;
import csbase.exception.project.FileLockedException;
import csbase.logic.ClientProjectFile;
import csbase.logic.ClientProjectFileOutputStream;
import csbase.logic.CommonClientProject;
import csbase.logic.ProjectFileType;
import csbase.logic.UserOutline;

public final class ExportUsersDialog {
  private Window owner;
  private String windowLabel;
  private DesktopComponentDialog window;
  private Container contentPane;
  private JButton actionButton;
  private SortableTable sortableTable;
  private static final String FILE_TYPE = "CSV";
  private static final int LOGIN_COLUMN = 0;
  private static final int NAME_COLUMN = 1;
  private static final int E_MAIL_COLUMN = 2;
  private static final int USERGROUP_COLUMN = 3;
  private static final char FIELD_SEPARATOR = '\t';
  private static final char REGISTER_SEPARATOR = '\n';

  public ExportUsersDialog(Window owner) {
    this.owner = owner;
    this.windowLabel = LNG.get("IAS_EXPORT_USERS_TITLE");
    this.window = new DesktopComponentDialog((JFrame) owner, this.windowLabel);
    this.window.center(owner);
    this.contentPane = this.window.getContentPane();
  }

  public void showDialog() {
    createUsersPanel();
    createButtonPanel();
    this.window.pack();
    this.window.center(this.owner);
    this.window.setVisible(true);
  }

  private void createUsersPanel() {
    JPanel usersPanel = new JPanel();
    usersPanel.setLayout(new BorderLayout());
    createTable();
    usersPanel.add(new JScrollPane(this.sortableTable), BorderLayout.CENTER);
    this.contentPane.add(usersPanel, BorderLayout.CENTER);
  }

  private void createTable() {
    Vector<UserOutline> allOutlines =
      UserProxy.getAllOutlines(this.owner, this.windowLabel, "MESSAGE");
    ObjectTableProvider provider = new DefaultObjectTableProvider() {

      /**
       * {@inheritDoc}
       */
      @Override
      public Object[] getCellValues(Object item) {
        UserOutline line = (UserOutline) item;
        return (line == null) ? null : new String[] { line.getLogin(),
            line.getName(), line.getMail(), line.getUserGroup() };
      }

      /**
       * {@inheritDoc}
       */
      public String[] getColumnNames() {
        return new String[] { LNG.get("IAS_USER"), LNG.get("IAS_USER_NAME"),
            LNG.get("IAS_USER_EMAIL"), LNG.get("IAS_USERGROUP") };
      }

      /**
       * {@inheritDoc}
       */
      public Class<?>[] getColumnClasses() {
        return new Class[] { String.class, String.class, String.class,
            String.class };
      }
    };
    ObjectTableModel model = new ObjectTableModel(allOutlines, provider);
    this.sortableTable = new SortableTable(model);
    this.sortableTable.addMouseListener(new MouseAdapter() {
      @Override
      public void mousePressed(MouseEvent e) {
        updateActionButton();
      }

      private void updateActionButton() {
        if (sortableTable.getSelectedRowCount() > 0) {
          actionButton.setEnabled(true);
        }
        else {
          actionButton.setEnabled(false);
        }
      }
    });
  }

  private void createButtonPanel() {
    createActionButton();
    JButton closeButton = createCloseButton();
    ClientUtilities.adjustEqualSizes(actionButton, closeButton);
    JPanel buttonsPanel = new JPanel();
    buttonsPanel.add(actionButton);
    buttonsPanel.add(closeButton);
    this.contentPane.add(buttonsPanel, BorderLayout.SOUTH);
  }

  private void createActionButton() {
    actionButton = new JButton(LNG.get("IAS_EXPORT_USERS_ACTION_BUTTON"));
    actionButton.setEnabled(false);
    actionButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ae) {
        if (useProject()) {
          showProjectFileChooser();
        }
        else {
          showFileChooser();
        }
      }
    });
  }

  /**
   * Exibe o sistema de arquivo local para escolha do diretrio onde o arquivo
   * ser gravado.
   */
  private void showFileChooser() {
    JFileChooser fc = new JFileChooser();
    fc.setAcceptAllFileFilterUsed(true);
    fc.addChoosableFileFilter(new Filter("txt", LNG
      .get("ias.file_type.txt.description")));
    fc.addChoosableFileFilter(new Filter("csv", LNG
      .get("ias.file_type.csv.description")));
    int returnVal = fc.showSaveDialog(window);
    if (returnVal == JFileChooser.APPROVE_OPTION) {
      File file = fc.getSelectedFile();
      StringBuffer buffer = generateData();
      if (writeFile(file, buffer)) {
        StandardDialogs.showInfoDialog(window, windowLabel, LNG
          .get("IAS_EXPORT_USERS_SUCCESS"));
        window.close();
      }
    }
  }

  /**
   * Indica se a aplicao utiliza o conceito de projeto.
   * 
   * @return true, se aplicao utiliza projeto, false caso contrrio.
   */
  private boolean useProject() {
    try {
      Configuration configuration =
        ConfigurationManager.getInstance().getConfiguration(
          UserManagerPanel.class);
      return configuration.getOptionalBooleanProperty(
        "application.use.project", true);
    }
    catch (ConfigurationManagerException cme) {
      return true;
    }
  }

  /**
   * Grava o arquivo no disco local do usurio.
   * 
   * @param file o arquivo que ser criado.
   * @param buffer o contedo que ser gravado no arquivo.
   * 
   * @return true, se a gravao finalizar com sucesso, false caso contrrio.
   */
  private boolean writeFile(final File file, final StringBuffer buffer) {
    LocalTask<Void> localTask = new LocalTask<Void>() {

      @Override
      protected void performTask() throws Exception {
        BufferedWriter out = null;
        try {
          FileWriter fstream = new FileWriter(file);
          out = new BufferedWriter(fstream);
          out.write(buffer.toString());
          out.close();
        }
        finally {
          if (out != null) {
            out.close();
          }
        }
      }

      @Override
      protected void handleError(Exception error) {
        if (error instanceof IOException) {
          StandardDialogs.showErrorDialog(window, windowLabel, String.format(
            LNG.get("ExportUsersDialog.save.file.failed"), file.getName()));
        }
        else {
          super.handleError(error);
        }
      }

    };
    return localTask.execute(window, windowLabel, String.format(LNG
      .get("ExportUsersDialog.msg.writing.file"), file.getName()));
  }

  /**
   * Exibe a rvore de projeto para escolha do diretrio onde o arquivo ser
   * gravado.
   */
  private void showProjectFileChooser() {
    ClientProjectFile clientProjectFile = getChosenFile();
    if (clientProjectFile == null) {
      return;
    }
    StringBuffer buffer = generateData();
    if (writeFile(clientProjectFile, buffer)) {
      StandardDialogs.showInfoDialog(window, windowLabel, LNG
        .get("IAS_EXPORT_USERS_SUCCESS"));
      window.close();
    }
  }

  /**
   * Grava o arquivo no servidor, na rea de projeto.
   * 
   * @param clientProjectFile o arquivo que ser criado.
   * @param buffer o contedo que ser gravado no arquivo.
   * 
   * @return true, se a gravao finalizar com sucesso, false caso contrrio.
   */
  private boolean writeFile(final ClientProjectFile clientProjectFile,
    final StringBuffer buffer) {
    RemoteTask<Void> task = new RemoteTask<Void>() {

      @Override
      protected void performTask() throws Exception {
        OutputStreamWriter writer = null;
        try {
          ClientProjectFileOutputStream stream =
            new ClientProjectFileOutputStream(clientProjectFile);
          writer = new OutputStreamWriter(stream);
          writer.write(buffer.toString());
          writer.flush();
          writer.close();
        }
        finally {
          if (writer != null) {
            writer.close();
          }
        }
      }

      @Override
      protected void handleError(Exception error) {
        if (error instanceof FileLockedException) {
          StandardDialogs.showErrorDialog(window, windowLabel, String.format(
            LNG.get("csbase.file.locked.error"), clientProjectFile.getName()));
        }
        else {
          super.handleError(error);
        }
      }
    };
    return task.execute(window, windowLabel, String.format(LNG
      .get("ExportUsersDialog.msg.writing.file"), clientProjectFile.getName()));

  }

  private ClientProjectFile getChosenFile() {
    final CommonClientProject currentProject =
      DesktopFrame.getInstance().getProject();
    ProjectFileChooserSave chooser = null;
    // Se no existe arquivo CSV no sistema, usa o TEXT.
    String fileTypeExt = FILE_TYPE;
    ClientProjectFileType projFileType =
      ClientProjectFileType.getFileType(FILE_TYPE);
    if (projFileType.getCode().equals(ProjectFileType.UNKNOWN)) {
      fileTypeExt = "TEXT";
    }
    try {
      chooser =
        new ProjectFileChooserSave(window, currentProject,
          ProjectFileChooser.FILE_ONLY, fileTypeExt);
    }
    catch (ClientException ce) {
      StandardErrorDialogs.showErrorDialog(window, windowLabel, LNG
        .get("ExportUsersDialog.error.creating.chooser"), ce);
      return null;
    }
    final ProjectTreePath treePath = chooser.getSelectedPath();
    if (treePath == null) {
      return null;
    }
    if (treePath.exists()) {
      return treePath.getFile();
    }
    String[] path = treePath.getSimplePath();
    final String name = path[path.length - 1];

    // criamos o arquivo no projeto
    CreateFileTask task =
      new CreateFileTask(currentProject, treePath.getParent(), name, chooser
        .getFileType());
    final boolean succeeded =
      task.execute(window, windowLabel, String.format(LNG
        .get("ExportUsersDialog.msg.creating.file"), name));
    if (succeeded) {
      return task.getResult();
    }
    return null;
  }

  private StringBuffer generateData() {
    StringBuffer buffer = new StringBuffer();
    int[] selectedRows = sortableTable.getSelectedRows();
    for (int i = 0; i < selectedRows.length; i++) {
      String login =
        (String) sortableTable.getValueAt(selectedRows[i], LOGIN_COLUMN);
      String name =
        (String) sortableTable.getValueAt(selectedRows[i], NAME_COLUMN);
      String eMail =
        (String) sortableTable.getValueAt(selectedRows[i], E_MAIL_COLUMN);
      String userGroup =
        (String) sortableTable.getValueAt(selectedRows[i], USERGROUP_COLUMN);
      buffer.append(login);
      buffer.append(FIELD_SEPARATOR);
      buffer.append(name);
      buffer.append(FIELD_SEPARATOR);
      buffer.append(eMail);
      buffer.append(FIELD_SEPARATOR);
      buffer.append(userGroup);
      buffer.append(REGISTER_SEPARATOR);
    }
    return buffer;
  }

  private JButton createCloseButton() {
    JButton closeButton = new JButton(LNG.get("IAS_CLOSE"));
    closeButton.setEnabled(true);
    closeButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        window.close();
      }
    });
    return closeButton;
  }
}
