/*
 * $Id$
 */

package csbase.client.ias.algorithm;

import java.awt.Component;
import java.awt.Rectangle;
import java.awt.Window;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.text.MessageFormat;
import java.util.Arrays;
import java.util.Enumeration;
import java.util.Observable;
import java.util.Observer;

import javax.swing.ImageIcon;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreePath;

import tecgraf.javautils.LNG;
import tecgraf.javautils.configurationmanager.Configuration;
import tecgraf.javautils.configurationmanager.ConfigurationManager;
import tecgraf.javautils.configurationmanager.ConfigurationManagerException;
import tecgraf.javautils.configurationmanager.MissingPropertyException;
import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.applications.ApplicationImages;
import csbase.client.applications.algorithmsmanager.versiontree.Mode;
import csbase.client.ias.algorithm.dnd.AlgorithmTreeTransferHandler;
import csbase.client.remote.srvproxies.AlgorithmManagementProxy;
import csbase.exception.ConfigurationException;
import csbase.logic.AlgoEvent;
import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.AlgorithmVersionInfo;

/**
 * @author Tecgraf / PUC-Rio
 * 
 * rvore utilizada na visualizao e gerncia dos algoritmos.<br>
 * A estrutura da rvore  a seguinte:<br>
 * <ul>
 * <li>
 * @{link RootNode algoritmos}<br>
 *        <ul>
 *        <li>
 * @{link AlgorithmNode &lt;algoritmo&gt;}<br>
 *        <ul>
 *        <li>
 * @{link VersionNode &lt;verso&gt;}<br>
 *        <ul>
 *        <li>
 * @{link ExecutableNode bin}<br>
 *        <ul>
 *        <li>
 * @{link PlatformNode &lt;plataforma&gt;}
 *        <ul>
 *        <li>
 * @{link ExecutableFileNode &lt;arquivos/diretrios&gt;}... </li>
 *        </ul>
 *        </li>
 *        </ul>
 *        </li>
 *        <li> {@link ConfigurationNode &lt;config&gt;}<br>
 *        <ul>
 *        <li>
 * @{link ConfigurationFileNode &lt;arquivos/diretrios&gt;}... </li>
 *        </ul>
 *        </li>
 *        <li> {@link DocumentationNode &lt;html&gt;}<br>
 *        <ul>
 *        <li>
 * @{link DocumentationFileNode &lt;arquivos/diretrios&gt;}... </li>
 *        </ul>
 *        </li>
 *        </ul>
 *        </li>
 *        </ul>
 *        </li>
 *        </ul>
 *        </li>
 *        <ul>
 */
@Deprecated
public final class AlgorithmTree extends AlgorithmManagementCommonTree {

  /** Modelo da rvore de algoritmos */
  private AlgorithmTreeModel model;

  /** Backup de ns expandidos da rvore. */
  private Enumeration<TreePath> expandedNodes;
  /** Backupo da poro da rvore visvel dentro do ScrollPane. */
  private Rectangle visibleRectangle;

  /**
   * Construtor.
   * 
   * @param owner janela que criou a rvore.
   */
  public AlgorithmTree(Window owner) {
    super(owner);

    RootNode root = createRootNode(owner);
    this.model = new AlgorithmTreeModel(root);
    setModel(model);

    setTransferHandler(new AlgorithmTreeTransferHandler());
    setDragEnabled(true);

    addHotKeys();
    addTreeSelectionListener(new SelectionListener());
    addMouseListener(new MouseListener());
    addListeners();
    setCellRenderer(new CellRenderer());
  }

  /**
   * Obtm todos os algoritmos.
   * 
   * @param owner janela que criou essa rvore
   * 
   * @return um array com as informaes de todos os algoritmos
   */
  private static AlgorithmInfo[] getAllAlgorithms(Window owner) {
    AlgorithmInfo[] algorithms = AlgorithmManagementProxy
      .retrieveAllAlgorithmInfos(owner);
    return algorithms;
  }

  /**
   * (non-Javadoc)
   * 
   * @see csbase.client.ias.algorithm.AlgorithmManagementCommonTree#contains(java.lang.String)
   */
  @Override
  public boolean contains(String algorithmName) {
    if (null == this.model || null == this.model.getRoot()) {
      return false;
    }
    else {
      return 0 <= ((RootNode) this.model.getRoot()).getNodeIndex(algorithmName);
    }
  }

  /**
   * Adiciona atalhos de teclado a rvore.<br>
   * Os atalhos so:<br>
   * <ul>
   * <li>Copiar: CTRL+C</li>
   * <li>Colar: CTRL+V</li>
   * </ul>
   */
  private void addHotKeys() {
    /*
     * AbstractAlgorithmTreeNodeAction copy = new VersionWarningActionDecorator(
     * new CopyFile(this)); getActionMap().put(copy.getName(), copy);
     * getInputMap().put( KeyStroke.getKeyStroke(KeyEvent.VK_C,
     * KeyEvent.CTRL_DOWN_MASK), copy.getName());
     * AbstractAlgorithmTreeNodeAction paste = new
     * VersionWarningActionDecorator( new PasteFile(this));
     * getActionMap().put(paste.getName(), paste); getInputMap().put(
     * KeyStroke.getKeyStroke(KeyEvent.VK_P, KeyEvent.CTRL_DOWN_MASK),
     * paste.getName());
     */
  }

  /**
   * Cria um n raiz.
   * 
   * @param owner Janela que requisitou a criao desta rvore.
   * 
   * @return um n raiz.
   */
  private RootNode createRootNode(Window owner) {
    AlgorithmInfo[] algorithms = AlgorithmManagementProxy
      .retrieveAllAlgorithmInfos(owner);
    if (null == algorithms) {
      return null;
    }
    Arrays.sort(algorithms);
    return new RootNode(this, algorithms);
  }

  private void addListeners() {
    AlgorithmInfo.addObserver(new Observer() {
      @Override
      public void update(Observable o, Object arg) {
        AlgoEvent event = (AlgoEvent) arg;
        AlgorithmTree.this.update(event);
      }
    });
  }

  private AbstractAlgorithmTreeNode getSelectedNode(TreePath path) {
    if (null != path) {
      Object tmp = path.getLastPathComponent();
      if (AbstractAlgorithmTreeNode.class.isAssignableFrom(tmp.getClass())) {
        return (AbstractAlgorithmTreeNode) tmp;
      }
    }
    return null;
  }

  /**
   * (non-Javadoc)
   * 
   * @see csbase.client.ias.algorithm.AlgorithmManagementCommonTree#getDefaultMode()
   */
  @Override
  public Mode getDefaultMode() {
    try {
      Configuration configuration = ConfigurationManager.getInstance()
        .getConfiguration(AlgorithmTree.class);
      String modeName = configuration.getMandatoryProperty("mode");
      for (Mode aMode : Mode.values()) {
        if (modeName.equalsIgnoreCase(aMode.name())) {
          return aMode;
        }
      }
      String errorMessage = String.format(
        "Modo de funcionamento desconhecido.\nModo informado: %s", modeName);
      throw new ConfigurationException(errorMessage);

    }
    catch (MissingPropertyException e) {
      throw new ConfigurationException(e);
    }
    catch (ConfigurationManagerException e) {
      throw new ConfigurationException(e);
    }
  }

  /**
   * Gerencia a seleo mltipla de ns da rvore. <br>
   * Se a rvore j tiver n(s) selecionado(s), o novo n dever ser aceitos por
   * estes, caso contrrio sua seleo ser desfeita.
   */
  class SelectionListener implements TreeSelectionListener {
    /**
     * {@inheritDoc}
     */
    @Override
    public void valueChanged(TreeSelectionEvent ev) {
      JTree tree = (JTree) ev.getSource();

      /*
       * Se no tiver nenhum ou apenas um selecionado, ele dever continuar
       * assim.
       */
      if (tree.isSelectionEmpty() || 1 == tree.getSelectionCount()) {
        return;
      }

      /*
       * Define quem ser o n lider da seleo que ter a funo de validar os
       * demais. Se o antigo lder ainda estiver entre os selecionados, ele ser
       * utilizado para validar os novos ns, caso contrrio, o ltimo n
       * selecionado ir validar os demais.
       */
      AbstractAlgorithmTreeNode leadNode;
      TreePath oldLeadPath = ev.getOldLeadSelectionPath();
      if (null != oldLeadPath && isPathSelected(oldLeadPath)) {
        leadNode = getSelectedNode(ev.getOldLeadSelectionPath());
      }
      else {
        leadNode = getSelectedNode(ev.getNewLeadSelectionPath());
      }
      if (null == leadNode) {
        return;
      }
      /*
       * Valida com o n lder os novos ns selecionados. Os que no forem
       * aceitos devem ser removidos.
       */
      TreePath[] paths = ev.getPaths();
      for (TreePath aPath : paths) {
        AbstractAlgorithmTreeNode aSelectedNode = getSelectedNode(aPath);
        if (null != aSelectedNode
          && !leadNode.allowMultipleSelection(aSelectedNode)) {
          tree.removeSelectionPath(aPath);
        }
      }
    }
  }

  /**
   * Gerencia as requisies aos ns por menus pop-up.
   */
  class MouseListener extends MouseAdapter {
    /**
     * {@inheritDoc}
     */
    @Override
    public void mousePressed(MouseEvent ev) {
      showPopup(ev);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void mouseReleased(MouseEvent ev) {
      showPopup(ev);
    }

    /**
     * Caso o evento de mouse tenha sido uma requisio de pop-up, pede para o
     * n aonde ele ocorreu para que construa um menu pop-up e mostra esse menu
     * ao usurio.
     * 
     * @param ev evento de mouse.
     */
    private void showPopup(MouseEvent ev) {
      if (!ev.isPopupTrigger()) {
        return;
      }

      JTree tree = (JTree) ev.getSource();
      TreePath newPath = tree.getPathForLocation(ev.getX(), ev.getY());
      if (null != newPath) {

        // Pega o n lder selecionado.
        TreePath leadPath = getLeadSelectionPath();
        AbstractAlgorithmTreeNode leadNode = getSelectedNode(leadPath);
        // Pega o n cujo pop-up foi requisitado.
        AbstractAlgorithmTreeNode newNode = getSelectedNode(newPath);

        /*
         * Verifica se j havia algum n selecionado e se ele aceita o n do
         * pop-up.
         */
        if (null != leadPath && newNode.allowMultipleSelection(leadNode)) {
          /*
           * Verifica se o usurio segurava o CTRL ou o SHIFT quando requisitou
           * o pop-up. Se sim, adiciona o novo n a lista.
           */
          boolean ctrlOrShiftPressed = (ev.getModifiersEx() & MouseEvent.CTRL_DOWN_MASK) == MouseEvent.CTRL_DOWN_MASK
            || (ev.getModifiersEx() & MouseEvent.SHIFT_DOWN_MASK) == MouseEvent.SHIFT_DOWN_MASK;
          if (ctrlOrShiftPressed) {
            tree.addSelectionPath(newPath);
          }
          else {
            /*
             * Caso contrrio e se o n origem do pop-up ainda no estiver na
             * lista, atribui ele como nico elemento selecionado.
             */

            // Verifica se o n j estava selecionado.
            boolean alreadySelected = false;
            for (TreePath path : tree.getSelectionPaths()) {
              if (path.equals(newPath)) {
                alreadySelected = true;
                break;
              }
            }
            if (!alreadySelected) {
              tree.setSelectionPath(newPath);
            }
          }
        }
        else {
          // Caso contrrio, o n do pop-up ser o nico n selecionado.
          tree.setSelectionPath(newPath);
        }
        // Cria e executa o menu pop-up do n escolhido.
        Object selected = newPath.getLastPathComponent();
        if (selected instanceof AbstractAlgorithmTreeNode) {
          AbstractAlgorithmTreeNode node = (AbstractAlgorithmTreeNode) selected;
          node.createPopupMenu().show(tree, ev.getX(), ev.getY());
        }
      }
    }
  }

  /**
   * Renderiza os ns da rvore de acordo com cones providos pelos mesmos.
   */
  class CellRenderer extends DefaultTreeCellRenderer {

    /**
     * {@inheritDoc}
     */
    @Override
    public Component getTreeCellRendererComponent(JTree tree, Object value,
      boolean selected, boolean expanded, boolean leaf, int row,
      boolean hasFocus) {

      super.getTreeCellRendererComponent(tree, value, selected, expanded, leaf,
        row, hasFocus);

      if (value instanceof AbstractAlgorithmTreeNode) {
        AbstractAlgorithmTreeNode node = (AbstractAlgorithmTreeNode) value;
        ImageIcon icon = node.getImageIcon();
        this.setIcon(icon);
      }
      else {
        this.setIcon(ApplicationImages.ICON_FOLDER_16);
      }
      return this;
    }
  }

  /**
   * Atualiza a rvore, sincronizando a sua representao visual no cliente com
   * sua estrutura real no servidor.
   */
  public void update() {
    backupExpandedNodes();

    boolean reloaded = AlgorithmManagementProxy.reloadAlgorithms(getOwner());
    if (reloaded) {
      StandardDialogs.showInfoDialog(getOwner(), LNG
        .get("algorithm.title.reload"), LNG.get("algorithm.msg.reload"));
    }
    RootNode root = createRootNode(getOwner());
    model.setRoot(root);

    restoreExpandedNodes();
  }

  /**
   * Atualiza a representao da rvore, sempre que for recebida uma notificao
   * de alterao no servidor.
   * 
   * @param event
   * 
   * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
   */
  public void update(AlgoEvent event) {
    AlgorithmInfo algoInfo = (AlgorithmInfo) event.item;
    switch (event.type) {
      case AlgoEvent.CREATE:
        addAlgoNode(algoInfo);
        break;
      case AlgoEvent.MODIFY:
        backupExpandedNodes();
        removeAlgoNode(algoInfo.getId());
        addAlgoNode(algoInfo);
        break;
      case AlgoEvent.DELETE:
        removeAlgoNode(algoInfo.getId());
        break;
      default:
        String msg = MessageFormat.format(LNG
          .get("algomanager.error.unknown_event"), new Object[] { new Integer(
          event.type) });
        System.err.println(msg);
        break;
    }
    restoreExpandedNodes();
  }

  /**
   * Remove um n de algoritmo do modelo da rvore.
   * 
   * @param algoId identificador do algoritmo.
   */
  private void removeAlgoNode(Object algoId) {
    int removedNodeIndex = ((RootNode) model.getRoot()).getNodeIndex(algoId);
    if (removedNodeIndex == -1) {
      return;
    }
    model.removeAlgorithm(removedNodeIndex);
  }

  /**
   * Adiciona um n de algoritmo ao modelo da rvore.
   * 
   * @param newAlgoInfo informaes sobre o algoritmo cujo n ser criado.
   */
  private void addAlgoNode(AlgorithmInfo newAlgoInfo) {
    DefaultMutableTreeNode newAlgoNode = new AlgorithmNode(this, newAlgoInfo);
    model.addAlgorithm(newAlgoNode);
  }

  /**
   * Guarda uma cpia do estado atual de expanso dos ns da rvore.
   */
  private void backupExpandedNodes() {
    TreePath rootPath = new TreePath(model.getRoot());
    expandedNodes = this.getExpandedDescendants(rootPath);
    visibleRectangle = this.getVisibleRect();
  }

  /**
   * Restaura o estado de expanso dos ns da rvore, modificado aps alguma
   * atualizao.
   */
  private void restoreExpandedNodes() {
    if (expandedNodes == null) {
      return;
    }
    while (expandedNodes.hasMoreElements()) {
      TreePath oldPath = (TreePath) expandedNodes.nextElement();
      TreePath newPath = getNewPath(oldPath);
      if (this.isCollapsed(newPath)) {
        this.expandPath(newPath);
      }
    }
    this.scrollRectToVisible(visibleRectangle);
  }

  /**
   * Obtm o novo caminho equivalente a um caminho antigo, aps a substituio
   * do modelo da rvore.
   * 
   * @param oldPath caminho para um n no modelo antigo.
   * 
   * @return novo caminho equivalente no novo modelo, ou null se o caminho no
   *         mais existir.
   */
  private TreePath getNewPath(TreePath oldPath) {
    Object[] path = oldPath.getPath();
    if (path.length == 1) {
      return new TreePath(model.getRoot());
    }
    DefaultMutableTreeNode node = (DefaultMutableTreeNode) model.getRoot();
    for (int i = 1; i < path.length; i++) {
      DefaultMutableTreeNode oldNode = (DefaultMutableTreeNode) path[i];
      Enumeration children = node.children();
      if (children.equals(DefaultMutableTreeNode.EMPTY_ENUMERATION)) {
        return null;
      }
      DefaultMutableTreeNode childNode = null;
      boolean found = false;
      while (children.hasMoreElements()) {
        childNode = (DefaultMutableTreeNode) children.nextElement();
        if (oldNode.getUserObject() instanceof AlgorithmVersionInfo
          && childNode.getUserObject() instanceof AlgorithmVersionInfo) {
          AlgorithmVersionInfo oldVersion = (AlgorithmVersionInfo) oldNode
            .getUserObject();
          AlgorithmVersionInfo childVersion = (AlgorithmVersionInfo) childNode
            .getUserObject();
          if (oldVersion.getId().equals(childVersion.getId())) {
            found = true;
            break;
          }
          continue;
        }
        if (childNode.toString().equals(path[i].toString())) {
          found = true;
          break;
        }
      }
      if (!found) {
        return null;
      }
      node = childNode;
    }
    return new TreePath(node.getPath());
  }

  class Clipboard {
    AbstractAlgorithmTreeNode[] transferable;

  }

}
