/*
 * $Id$
 */

package csbase.client.ias.algorithm.action;

import java.awt.event.ActionEvent;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.applications.algorithmsmanager.versiontree.Mode;
import csbase.client.ias.algorithm.AlgorithmManagementCommonTree;

/**
 * @author Tecgraf / PUC-Rio
 * 
 *         Ao abstrata de construo de verso.<br>
 *         Caso a rvore de algoritmos esteja no modo {@link Mode#WIZARD}, o
 *         usurio ser questionado se deseja proseguir com a ao filha desta.
 */
public abstract class AbstractVersionFactoryAction extends
  AbstractAlgorithmTreeNodeAction {

  /**
   * Construtor.
   * 
   * @param tree rvore que detm o n fonte da ao.
   * @param name Nome da ao.
   */
  public AbstractVersionFactoryAction(AlgorithmManagementCommonTree tree, String name) {
    super(tree, name);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void actionPerformed(ActionEvent e) {
    if (getTree().getMode() == Mode.WIZARD) {
      if (!warningCreateVersion()) {
        return;
      }
    }

    createVersion();
  }

  protected abstract void createVersion();

  /**
   * Exibe um dilogo alertando ao usurio que incluir uma verso de algoritmo
   * manualmente  perigoso e d a opo para ele abortar a operao.
   * 
   * @return {@code true} se o usurio desejar continuar ou {@code false} caso
   *         contrrio.
   */
  protected boolean warningCreateVersion() {
    String message = LNG.get("algomanager.warning.include.version.message");
    String title = LNG.get("algomanager.warning.include.version.title");
    String continueMessage = LNG.get("algomanager.option.continue");
    String cancelMessage = LNG.get("algomanager.option.cancel");
    int continueIndex = 0;
    int cancelIndex = 1;
    int optionsCount = 2;
    Object[] options = new Object[optionsCount];
    options[continueIndex] = continueMessage;
    options[cancelIndex] = cancelMessage;
    int option =
      StandardDialogs.showOptionDialog(getWindow(), title, message, options);
    if (option == continueIndex) {
      return true;
    }
    else if (option == cancelIndex) {
      return false;
    }
    else {
      String errorMessage =
        String.format("Opo invlida. Opo: %d.\n", option);
      throw new IllegalStateException(errorMessage);
    }
  }
}
