/*
 * $Id$
 */

package csbase.client.ias.algorithm.action;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.text.MessageFormat;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JPanel;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.applications.algorithmsmanager.dialogs.VersionDialog;
import csbase.client.applications.algorithmsmanager.versiontree.PropertiesPanel;
import csbase.client.desktop.DesktopComponentDialog;
import csbase.client.ias.algorithm.AlgorithmManagementCommonTree;
import csbase.client.remote.srvproxies.AlgorithmManagementProxy;
import csbase.client.util.ClientUtilities;
import csbase.client.util.StandardErrorDialogs;
import csbase.logic.algorithms.AlgorithmProperty;
import csbase.logic.algorithms.AlgorithmVersionInfo;

/**
 * @author Tecgraf / PUC-Rio
 * 
 *         Ao para abrir um dilogo apresentando as propriedades de um
 *         algoritmo.
 */
@Deprecated
public class ShowVersionProperties extends AbstractAlgorithmTreeNodeAction {

  // Verso a ter suas propriedades apresentadas.
  private final AlgorithmVersionInfo version;

  /**
   * Construtor.
   * 
   * @param tree rvore que detm o n fonte da ao.
   * @param version Verso a ter suas propriedades apresentadas.
   */
  public ShowVersionProperties(AlgorithmManagementCommonTree tree,
    AlgorithmVersionInfo version) {
    super(tree, LNG.get("algomanager.menu.algorithm.properties"));

    this.version = version;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void actionPerformed(ActionEvent e) {
    new VersionPropertiesDialog().show();
  }

  /**
   * Classe que representa um dilogo com as propriedades de um algoritmo
   * (incluindo, caso existam, as propriedades estendidas)
   * 
   * @author Daniel de Paula
   */
  private class VersionPropertiesDialog extends VersionDialog {

    /** Dilogo em si */
    protected DesktopComponentDialog dialog;

    /**
     * Lista com as propriedades dos algoritmos.
     */
    private List<AlgorithmProperty> algorithmProperties;

    /**
     * Lista com as propriedades das verses do algoritmo.
     */
    private List<AlgorithmProperty> versionProperties;

    /** Painel de propriedades do algoritmo */
    private PropertiesPanel algoPropertiesPanel;

    /** Painel de propriedades da verso */
    private PropertiesPanel versionPropertiesPanel;

    /**
     * Cria o dilogo.
     */
    public VersionPropertiesDialog() {
      super(version.getInfo());

      algorithmProperties = AlgorithmManagementProxy
        .getAlgorithmProperties(getWindow());
      versionProperties = AlgorithmManagementProxy
        .getAlgorithmVersionProperties(getWindow());

      algoPropertiesPanel = new PropertiesPanel(algorithmProperties, algoInfo
        .getPropertyValues(), String.format(LNG
        .get("algomanager.title.algo_properties"), algoInfo.getName()),
        getWindow(), false);
      versionPropertiesPanel = new PropertiesPanel(versionProperties, algoInfo
        .getVersionInfo(version.getId()).getPropertyValues(), LNG
        .get("algomanager.title.border.version_properties"), getWindow(), true);
    }

    /**
     * Exibe dilogo
     */
    public void show() {
      String dialogTitle = String.format(LNG
        .get("algomanager.title.version_properties"), version.getId()
        .toString(), algoInfo.getName());
      dialog = DesktopComponentDialog.createDialog(getWindow());
      dialog.setTitle(dialogTitle);
      dialog.addWindowListener(new WindowAdapter() {
        @Override
        public void windowClosing(WindowEvent e) {
          close();
        }
      });

      /* Painel com os atributos e seus valores (labels e trextfields) */
      JPanel infoPanel = new JPanel();
      infoPanel.setLayout(new BoxLayout(infoPanel, BoxLayout.Y_AXIS));
      infoPanel.setBorder(BorderFactory.createEmptyBorder(12, 12, 12, 12));
      if (algorithmProperties.size() > 0) {
        infoPanel.add(algoPropertiesPanel.getPropertiesPanel(),
          BorderLayout.CENTER);
      }
      if (versionProperties.size() > 0) {
        infoPanel.add(versionPropertiesPanel.getPropertiesPanel(),
          BorderLayout.CENTER);
      }

      /* Painel com os botoes */
      JPanel buttonsPanel = new JPanel();
      JButton includeButton = new JButton(LNG.get("algomanager.button.change"));
      JButton cancelButton = new JButton(LNG.get("algomanager.button.cancel"));
      buttonsPanel.add(includeButton);
      buttonsPanel.add(cancelButton);
      /* Listeners do dialog */
      includeButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          if (versionPropertiesPanel.validateProperties()) {
            analyzePropertyChange();
            dialog.close();
          }
        }
      });
      cancelButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          close();
        }
      });
      /* ajustes */
      ClientUtilities.adjustEqualSizes(includeButton, cancelButton);
      dialog.add(infoPanel, BorderLayout.CENTER);
      dialog.add(buttonsPanel, BorderLayout.SOUTH);
      dialog.pack();
      dialog.center(getWindow());
      dialog.setVisible(true);
    }

    /**
     * Fecha o dilogo.
     */
    private void close() {
      if (versionPropertiesPanel.wasModified()) {
        if (askConfirmation() == 0) {
          dialog.close();
        }
      }
      else {
        dialog.close();
      }
    }

    /**
     * Analisa se ouve ou nao mudanca em alguma das propriedades. Em caso
     * positivo, executa a atualizacao (apos pedir confirmacao)
     */
    private void analyzePropertyChange() {

      /* Caso alguma propriedade tenha mudado, pergunta e processa a mudanca */
      if (versionPropertiesPanel.wasModified()) {
        String name = algoInfo.getName();
        algoInfo = AlgorithmManagementProxy.changeVersionProperties(name,
          version.getId(), versionPropertiesPanel.getProperties(), getWindow());
        String dialogTitle = LNG.get("algomanager.title.algo_properties");
        if (algoInfo == null) {
          String title = LNG.get("algomanager.title.error") + " - "
            + dialogTitle;
          StandardErrorDialogs.showErrorDialog(getWindow(), title,
            MessageFormat.format(LNG.get("algomanager.error.update"),
              new Object[] { name }));
          return;
        }
      }
    }

    /**
     * Exibe janela de confirmacao para atualizar propriedades do algoritmo
     * 
     * @return opcao selecionada
     */
    private int askConfirmation() {
      String algoName = algoInfo.getName();
      String dialogTitle = LNG.get("algomanager.title.properties_change");
      int opt = StandardDialogs.showYesNoDialog(getWindow(), dialogTitle,
        MessageFormat.format(LNG
          .get("algomanager.msg.confirm.properties_update"),
          new Object[] { algoName }));
      return opt;
    }
  }
}
