/**
 * $Id$
 */
package csbase.client.login;

import java.rmi.RemoteException;
import java.util.Locale;
import java.util.TimeZone;

import csbase.exception.CSBaseException;
import csbase.logic.Session;
import csbase.remote.ServerEntryPoint;

/**
 * Representa um autenticador de usurios no servidor atravs do mtodo de login
 * e senha.
 * 
 * @author Tecgraf/PUC-RIo
 */
public final class UserPasswordAuthenticator implements Authenticator {
  /**
   * O login do usurio.
   */
  private String login;
  /**
   * A senha do usurio.
   */
  private String password;
  /**
   * A locale escolhida pelo usurio.
   */
  private Locale locale;

  /**
   * Cria um autenticador de usurios no servidor atravs do mtodo de login e
   * senha.
   * 
   * @param login O login do usurio.
   * @param password A senha do usurio.
   * @param locale A locale escolhida pelo usurio.
   */
  public UserPasswordAuthenticator(String login, String password, Locale locale) {
    this.login = login;
    this.password = password;
    this.locale = locale;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Session authenticate(ServerEntryPoint server) throws CSBaseException {
    TimeZone tz = TimeZone.getDefault();
    try {
      return server.login(this.login, this.password, this.locale, tz);
    }
    catch (RemoteException e) {
      throw new CSBaseException(e);
    }
  }
}
