/**
 * $Id$
 */
package csbase.client.openbus;

import java.rmi.RemoteException;
import java.util.Locale;
import java.util.TimeZone;

import csbase.client.login.Authenticator;
import csbase.exception.CSBaseException;
import csbase.logic.Session;
import csbase.logic.openbus.Credential;
import csbase.remote.ServerEntryPoint;

/**
 * Representa um autenticador de usurios no servidor atravs de uma credencial
 * do OpenBus.
 * 
 * @author Tecgraf/PUC-RIo
 */
public final class OpenBusAuthenticator implements Authenticator {
  /**
   * A credencial do usurio.
   */
  private Credential credential;
  /**
   * A locale escolhida pelo usurio.
   */
  private Locale locale;

  /**
   * Cria o autenticador de usurios no servidor atravs de uma credencial do
   * OpenBus.
   * 
   * @param credential A credencial do usurio.
   * @param locale A locale escolhida pelo usurio.
   */
  public OpenBusAuthenticator(Credential credential, Locale locale) {
    this.credential = credential;
    this.locale = locale;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Session authenticate(ServerEntryPoint server) throws CSBaseException {
    TimeZone tz = TimeZone.getDefault();
    try {
      System.out.println("Autenticando via OpenBus.");
      return server.login(this.credential, this.locale, tz);
    }
    catch (RemoteException e) {
      throw new CSBaseException(e);
    }
  }

}
