package csbase.client.openbus;

import org.omg.CORBA.NO_PERMISSION;
import org.omg.CORBA.SystemException;

import tecgraf.javautils.LNG;
import csbase.client.desktop.RemoteTask;
import csbase.client.util.StandardErrorDialogs;

public abstract class OpenbusTask<R> extends RemoteTask<R> {
  /**
   * Prefixo utilizado nos nomes das chaves de traduo.
   */
  private static final String LNG_PREFIX = "OpenbusTask.";

  /**
   * Construtor.
   */
  protected OpenbusTask() {
    /** nada a fazer */
  }

  /**
   * Tratamento de erros especfico para cada tipo de tarefa. Esse tratamento 
   * chamado na thread do Swing. Pode ser redefinido por uma especializao
   * dessa classe, porm com cautela, para que a padronizao do tratamento de
   * erros no seja quebrada!
   * 
   * @param error exceo que representa erro durante execuo da tarefa
   */
  @Override
  protected void handleError(Exception error) {
    if (error instanceof NO_PERMISSION) {
      StandardErrorDialogs.showErrorDialog(parentWindow, taskTitle, LNG
        .get(LNG_PREFIX + "no_permission"), null, getAdditionalInfo());
    }
    else if (error instanceof SystemException) {
      StandardErrorDialogs.showErrorDialog(parentWindow, taskTitle, LNG
        .get(LNG_PREFIX + "system_exception"), null, getAdditionalInfo());
    }
    else {
      super.handleError(error);
    }
  }
}
