package csbase.client.project;

import java.util.List;

import javax.swing.ListSelectionModel;

import tecgraf.javautils.gui.table.SortableTable;
import csbase.client.project.ProjectTableModel.RowData;

/**
 * Tabela de projetos.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class ProjectTableSortableTable extends SortableTable {

  /**
   * Construtor.
   * 
   * @param projects Lista com os pares (usurio, nome do projeto) que sero
   *        listados pela tabela.
   * 
   * @see ProjectTableModel.RowData
   */
  public ProjectTableSortableTable(List<RowData> projects) {
    super(new ProjectTableModel(projects));

    setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    setRowSelectionAllowed(true);
    setFillsViewportHeight(true);
  }

  /**
   * Obtm o par (usurio, nome do projeto) associado a linha que est
   * selecionada na tabela.
   * 
   * @return Par (usurio, nome do projeto) associado  linha.
   * @see ProjectTableModel.RowData
   */
  public RowData getSelectedRowData() {
    ProjectTableModel model = (ProjectTableModel) getModel();
    return model.getRow(convertRowIndexToModel(getSelectedRow()));
  }

  /**
   * Troca as os projetos atuais da tabela por novos projetos.
   * 
   * @param projectList Lista com os pares (usurio, nome do projeto) que sero
   *        listados pela tabela.
   * @see ProjectTableModel.RowData
   */
  public void setNewValues(List<RowData> projectList) {
    ProjectTableModel model = (ProjectTableModel) getModel();
    model.replaceRows(projectList);
  }

  /**
   * Habilita/Desabilita a visualizao da coluna "Usurios".
   * 
   * @param visible <code>True</code> para habilitar a visualizao,
   *        <code>False</code> para desabilitar.
   */
  public void setUserColumnVisible(boolean visible) {
    ProjectTableModel model = (ProjectTableModel) getModel();
    model.setUserColumnVisible(visible);
  }

  /**
   * Seleciona a 1a. linha da tabela, se a tabela no for vazia.
   */
  public void setSelectedFirstRow() {
    if (getRowCount() > 0) {
      setRowSelectionInterval(0, 0);
    }
  }

}
