/**
 * $Id: ProjectTreeStructureListenerImpl.java 105206 2010-05-06 23:06:04Z costa $
 */
package csbase.client.project;

import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JTree;
import javax.swing.event.TreeModelEvent;
import javax.swing.tree.TreePath;

/**
 * Implementao de um listener para eventos da rvore de projetos.
 * 
 * @author Tecgraf
 */
final class ProjectTreeStructureListenerImpl implements
  ProjectTreeStructureListener {
  /**
   * {@link JTree} que representa a rvore de projetos.
   */
  private final JTree tree;

  /**
   * @param projectTree
   */
  ProjectTreeStructureListenerImpl(JTree projectTree) {
    this.tree = projectTree;
  }

  /**
   * Lista de paths expandidos ("abertos").
   */
  private List<TreePath> expandedPaths;
  /**
   * Lista de paths selecionados.
   */
  private TreePath[] selectedPaths;
  /**
   * Backup da poro da rvore visvel dentro do ScrollPane.
   */
  private Rectangle visibleRectangle;

  /**
   * {@inheritDoc}
   */
  public void treeWillChange(TreeModelEvent event) {
    expandedPaths = this.getExpandedPaths();
    this.visibleRectangle = tree.getVisibleRect();
    this.selectedPaths = tree.getSelectionPaths();
  }

  /**
   * @return lista de paths expandidos ("abertos")
   */
  private List<TreePath> getExpandedPaths() {
    ProjectTreeNode rootNode = (ProjectTreeNode) tree.getModel().getRoot();
    if (rootNode == null) {
      return null;
    }
    TreePath treePath = new TreePath(rootNode);
    Enumeration<TreePath> expandedPathsEnum =
      tree.getExpandedDescendants(treePath);

    if (null == expandedPathsEnum) {
      return null;
    }

    List<TreePath> expandedPaths = new ArrayList<TreePath>();
    while (expandedPathsEnum.hasMoreElements()) {
      TreePath path = expandedPathsEnum.nextElement();
      expandedPaths.add(path);
    }
    return expandedPaths;
  }

  /**
   * {@inheritDoc}
   */
  public void treeWasChanged(TreeModelEvent event) {
    this.expandPaths();
    this.updateSelectedPaths();
    tree.setSelectionPaths(this.selectedPaths);
  }

  /**
   * Atualiza a lista de ns selecionados removendo paths que no existam mais
   * na rvore.
   */
  private void updateSelectedPaths() {
    ProjectTreeModel treeModel = (ProjectTreeModel) tree.getModel();
    List<TreePath> validSelectedPaths = new LinkedList<TreePath>();
    for (TreePath selectedPath : this.selectedPaths) {
      if (treeModel.exists(selectedPath)) {
        validSelectedPaths.add(selectedPath);
      }
    }
    this.selectedPaths =
      validSelectedPaths.toArray(new TreePath[validSelectedPaths.size()]);
  }

  /**
   * Garante que os paths marcados como "expandidos" estejam visveis.
   */
  private void expandPaths() {
    if (this.expandedPaths != null) {
      for (TreePath path : expandedPaths) {
        if (tree.isCollapsed(path)) {
          tree.expandPath(path);
        }
      }
      tree.scrollRectToVisible(visibleRectangle);
    }
  }
}