/*
 * PublishedFileRetrieverDialog.java
 *
 * $Id: PublishedFileRetrieverDialog.java 72732 2008-02-15 12:15:26Z taciana $
 */
package csbase.client.project;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.GUIUtils;
import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.desktop.*;
import csbase.client.kernel.ClientException;
import csbase.client.util.*;
import csbase.logic.*;

/**
 * Modela uma janela de dilogo que permite ao usurio obter dados publicados
 * para ele por outros usurios.
 *
 * @author Glaucio
 * @version $Revision: 72732 $
 */
public class PublishedFileRetrieverDialog {
  /** Ttulo da janela. */
  private String mainFrameTitle;

  /** Descritores das colunas da tabela de arquivos publicados */
  private TableColumnAttributes[] tableColumns;
  private int lastColumnSorted = -1;
  private int firstColumnSorted = -1;

  /**
   * Contador para auxiliar na ordenao inversa ou normal de uma determinada
   * coluna da tabela.  incrementado quando clica-se no header da coluna que
   * foi a ltima a ser ordenada, ento uma nova ordenacao sobre esta  feita,
   * s que de maneira inversa a anterior. Seu valor  zerado assim que  pedida
   * uma ordenao numa coluna diferente da que foi ordenada previamente.
   * NUMEROS PARES a ordenao  a default. NUMEROS IMPARES a ordenao  a
   * inversa.
   */
  private int inverseOrdering = 0;

  /** Vetor com as informaes sobre publicaes de arquivos. */
  private Vector<PublicationInfo> publishedData;

  /** Modelo da tabela de publicaes. */
  private DataTableModel dataTableModel;

  /** Elementos de Interface */
  private DesktopComponentFrame mainFrame;
  private JButton retrieveButton;
  private JTable dataTable;
  private FileTextField fileTextField;
  private ProjectTreePath choosedTreePath = null;

  /*
   * Constantes que definem o tamanha da rea onde os resultados da recuperao
   * so apresentados.
   */
  private final int ROWS = 12;
  private final int COLS = 50;
  private static final String DIRECTORY_SEPARATOR = "/";

  /**
   * Exibe a janela de obteno de dados publicados.
   */
  private void display() {
    mainFrame.pack();
    mainFrame.center(DesktopFrame.getInstance().getDesktopFrame());
    mainFrame.setVisible(true);
  }

  /**
   * Cria a janela de recuperao dos dados publicados.
   */
  private void createMainFrame() {
    mainFrameTitle = LNG.get("PRJ_PUBLISHED_FILE_RETRIEVER") + " - "
      + LNG.get("SERVER");
    mainFrame = new DesktopComponentFrame(mainFrameTitle);
    mainFrame.addWindowListener(new WindowAdapter() {
      @Override
      public void windowClosing(WindowEvent we) {
        mainFrame.close();
      }
    });

    Container cp = mainFrame.getContentPane();
    cp.add(new JScrollPane(createDataTable()), BorderLayout.CENTER);
    JPanel paux = new JPanel(new BorderLayout());
    paux.add(createPathChooser(), BorderLayout.NORTH);
    paux.add(createButtons(), BorderLayout.SOUTH);
    cp.add(paux, BorderLayout.SOUTH);
  }

  /**
   * Cria o painel com os componentes para exibir e selecionar um diretrio do
   * projeto onde sero colocados os dados publicados.
   *
   * @return .
   */
  private JPanel createPathChooser() {
    this.fileTextField = new FileTextField(this.mainFrame, LNG
      .get("PRJ_DIR_TARGET"), 0, DesktopFrame.getInstance().getProject(),
      ProjectFileChooser.DIRECTORY_ONLY, false);
    this.fileTextField.getField().addKeyListener(new KeyAdapter() {
      @Override
      public void keyReleased(KeyEvent e) {
        this.update();
      }

      private void update() {
        choosedTreePath = null;
        updateRetrieveButton();
      }
    });
    this.fileTextField.getButton().setToolTipText(
      LNG.get("PRJ_PUBLISHED_DIR_CHOOSER"));
    this.fileTextField.getButton().addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ev) {
        try {
          choosedTreePath = fileTextField.getButtonPath();
        }
        catch (ClientException e) {
          e.printStackTrace();
          return;
        }
        if (choosedTreePath == null) {
          return;
        }
        fileTextField.getField().setText(
          DIRECTORY_SEPARATOR
            + ProjectTreePath.makePath(DIRECTORY_SEPARATOR, choosedTreePath
              .getSimplePath()));
        updateRetrieveButton();
      }
    });

    JComponent[][] components = { { fileTextField.getLabel(),
        fileTextField.getField(), fileTextField.getButton() } };
    return GUIUtils.createBasicGridPanel(components);
  }

  /**
   *
   */
  protected void updateRetrieveButton() {
    if ((fileTextField.getField().getText().length() > 0)
      && (dataTable.getSelectedRowCount() > 0)) {
      retrieveButton.setEnabled(true);
    }
    else {
      retrieveButton.setEnabled(false);
    }
  }

  /**
   * Cria o painel de botes.
   *
   * @return o painel de botes criado
   */
  private JPanel createButtons() {
    /* Boto de cancelamento da operao. */
    JButton cancelButton = new JButton(LNG.get("PRJ_CANCEL"));
    cancelButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ev) {
        mainFrame.close();
      }
    });

    /* Boto de obteno dos dados publicados. */
    retrieveButton = new JButton(LNG.get("PRJ_RETRIEVE"));
    retrieveButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if (choosedTreePath == null) {
          choosedTreePath = fileTextField.getFieldPath();
        }
        if (choosedTreePath == null) {
          return;
        }
        if (linkToPublishedData(choosedTreePath.getFile())) {
          mainFrame.close();
        }
      }
    });
    retrieveButton.setEnabled(false);
    addSelectionListener(retrieveButton);
    /* Boto de atualizao da lista de dados publicados */
    JButton refreshButton = new JButton(LNG
      .get("PRJ_PUBLISHED_FILE_LIST_UPDATE"));
    refreshButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        publishedData.removeAllElements();
        loadPublishedData();
        Collections.sort(publishedData, tableColumns[lastColumnSorted]);
        dataTableModel.fireTableChanged(new TableModelEvent(dataTableModel));
      }
    });
    ClientUtilities.adjustEqualSizes(new JButton[] { retrieveButton,
        refreshButton, cancelButton });
    /* Painel com os botes */
    JPanel panel = new JPanel(new FlowLayout());
    panel.add(retrieveButton);
    panel.add(refreshButton);
    panel.add(cancelButton);
    return panel;
  }

  /**
   * Cria a tabela de visualizacao dos arquivos publicados.
   *
   * @return .
   */
  private JTable createDataTable() {
    publishedData = new Vector<PublicationInfo>();
    dataTableModel = new DataTableModel();
    dataTable = new JTable(dataTableModel);
    JTableHeader th = dataTable.getTableHeader();
    th.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseClicked(MouseEvent e) {
        TableColumnModel columnModel = dataTable.getColumnModel();
        int viewColumn = columnModel.getColumnIndexAtX(e.getX());
        int column = dataTable.convertColumnIndexToModel(viewColumn);

        // criterio de ordenacao e ajustado
        if (lastColumnSorted == column) {
          inverseOrdering++;
        }
        else {
          inverseOrdering = 0;
        }
        boldHeaderFont(column);
        dataTableModel.sort(column);
      }
    });
    dataTable.setRowSelectionAllowed(true);
    dataTable.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
    dataTable.setPreferredScrollableViewportSize(new Dimension(640, 100));
    boldHeaderFont(firstColumnSorted);
    lastColumnSorted = firstColumnSorted;
    return dataTable;
  }

  /**
   * Carrega o vetor do modelo da tabela de arquivos publicados com os dados dos
   * arquivos disponiveis para o usurio.
   */
  public void loadPublishedData() {
    final Object userId = User.getLoggedUser().getId();
    RemoteTask<Vector<PublicationInfo>> task =
      new RemoteTask<Vector<PublicationInfo>>() {
      @Override
      public void performTask() throws Exception {
        setResult(CommonClientProject.getAllPublicationInfo(userId));
      }
    };
    if  (task.execute(mainFrame, mainFrameTitle, LNG
      .get("PRJ_WAITING_FIND_PROJECT_NAMES"))) {
      publishedData.addAll(task.getResult());
    }
  }

  /**
   * Cria vrios links no diretrio selecionado, um para cada arquivo
   * selecionado na lista de publicaes.
   *
   * @param directory .
   *
   * @return Verdadeiro, se todas as recuperaes foram efetuadas com sucesso ou
   *         falso, caso tenha ocorrido erro em alguma das recuperaes.
   */
  private boolean linkToPublishedData(final ClientProjectFile directory) {
    final int[] selectedRows = dataTable.getSelectedRows();
    final Vector<PublicationInfo> successList = new Vector<PublicationInfo>();
    final Vector<PublicationError> notSuccessList =
      new Vector<PublicationError>();
    RemoteTask<Void> task = new RemoteTask<Void>() {
      @Override
      public void performTask() {
        for (int index = 0; index < selectedRows.length; index++) {
          PublicationInfo info = publishedData
            .elementAt(selectedRows[index]);
          try {
            directory.linkToPublishedFile(info);
            successList.addElement(info);
          }
          catch (Exception e) {
            notSuccessList.addElement(new PublicationError(info, e));
          }
        }
      }
    };
    task.execute(mainFrame, mainFrameTitle, LNG
      .get("PRJ_WAITING_LINK_TO_PUBLISHED_FILES"));
    return showResult(successList, notSuccessList);
  }

  /**
   * Adiciona ouvidor para seleo de linhas, e habilita o boto se houver
   * alguma linha em selecionada, seno o boto  desabilitado.
   *
   * @param button O boto a ser habilitado ou no de acordo com a seleo de
   *        linhas.
   */
  private void addSelectionListener(final JButton button) {
    ListSelectionModel selectModel = dataTable.getSelectionModel();
    selectModel.addListSelectionListener(new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent e) {
        if ((dataTable.getSelectedRowCount() == 0)
          || (fileTextField.getField().getText().length() == 0)) {
          button.setEnabled(false);
        }
        else {
          button.setEnabled(true);
        }
      }
    });
  }

  /**
   * Modifica a fonte do ttulo da coluna que determina a ordenao da tabela.
   *
   * @param column ndice da coluna no modelo
   */
  private void boldHeaderFont(int column) {
    int viewColumn;
    if (lastColumnSorted != column) {
      if (lastColumnSorted != -1) {
        viewColumn = dataTable.convertColumnIndexToView(lastColumnSorted);
        String lastColumn = dataTable.getColumnName(viewColumn);
        modifyHeaderFont(lastColumn, Font.PLAIN);
      }
      viewColumn = dataTable.convertColumnIndexToView(column);
      String sortColumn = dataTable.getColumnName(viewColumn);
      modifyHeaderFont(sortColumn, Font.BOLD);
    }
  }

  /**
   * Aplica um estilo de fonte no ttulo de uma coluna.
   *
   * @param columnName ttulo (identificador) da coluna
   * @param fontStyle estilo a ser aplicado
   */
  private void modifyHeaderFont(String columnName, final int fontStyle) {
    dataTable.getColumn(columnName).setHeaderRenderer(new TableCellRenderer() {
      public Component getTableCellRendererComponent(JTable table,
        Object value, boolean isSelected, boolean hasFocus, int row, int column) {
        TableCellRenderer renderer = table.getTableHeader()
          .getDefaultRenderer();
        Component c = renderer.getTableCellRendererComponent(table, value,
          isSelected, hasFocus, row, column);
        c.setFont(c.getFont().deriveFont(fontStyle));
        return c;
      }
    });
  }

  /**
   * Define os atributos das colunas da tabela de dados publicados.
   */
  private void defineTableColumns() {
    tableColumns = new TableColumnAttributes[] { getPublishedByAttributes(),
        getFilePathAttributes(), getSizeAttributes() };
    firstColumnSorted = 0;
  }

  /**
   * Atributos da coluna que exibe o nome de quem publicou o arquivo
   *
   * @return .
   */
  private TableColumnAttributes getPublishedByAttributes() {
    return new TableColumnAttributes() {
      @Override
      public String getColumnName() {
        return LNG.get("PRJ_PUBLISHED_BY");
      }

      @Override
      public Object getColumnValue(Object o) {
        return getPublishedByRepr((PublicationInfo) o);
      }

      @Override
      public int compareReverse(Object o1, Object o2) {
        String name1 = getPublishedByRepr((PublicationInfo) o1);
        String name2 = getPublishedByRepr((PublicationInfo) o2);
        int comp = name1.compareTo(name2);
        return ((inverseOrdering % 2) == 0) ? comp : (-comp);
      }

      @Override
      public void setColumnValue(Object o, Object value) {
      }

    };
  }

  /**
   * Atributos da coluna que exibe o nome do arquivo.
   *
   * @return .
   */
  private TableColumnAttributes getFilePathAttributes() {
    return new TableColumnAttributes() {
      @Override
      public String getColumnName() {
        return LNG.get("PRJ_PUBLISHED_FILE_NAME");
      }

      @Override
      public Object getColumnValue(Object o) {
        PublicationInfo info = (PublicationInfo) o;
        return ProjectTreePath.makePath(info.getProjectName(),
          DIRECTORY_SEPARATOR, info.getPath());
      }

      @Override
      public int compareReverse(Object o1, Object o2) {
        PublicationInfo info1 = (PublicationInfo) o1;
        PublicationInfo info2 = (PublicationInfo) o2;
        String path1 = ProjectTreePath.makePath(info1.getProjectName(),
          DIRECTORY_SEPARATOR, info1.getPath());
        String path2 = ProjectTreePath.makePath(info2.getProjectName(),
          DIRECTORY_SEPARATOR, info2.getPath());
        int comp = path1.compareTo(path2);
        return ((inverseOrdering % 2) == 0) ? comp : (-comp);
      }

      @Override
      public void setColumnValue(Object o, Object value) {
      }

    };
  }

  /**
   * Atributos da coluna que exibe o tamanho do arquivo publicado
   *
   * @return .
   */
  private TableColumnAttributes getSizeAttributes() {
    return new TableColumnAttributes() {
      @Override
      public String getColumnName() {
        return LNG.get("PRJ_SIZE");
      }

      @Override
      public Object getColumnValue(Object o) {
        return getSizeRepr((PublicationInfo) o);
      }

      @Override
      public int compareReverse(Object o1, Object o2) {
        long size1 = ((PublicationInfo) o1).getFileSize();
        long size2 = ((PublicationInfo) o2).getFileSize();
        int comp;
        if (size1 < size2) {
          comp = -1;
        }
        else {
          comp = (size1 == size2) ? 0 : 1;
        }
        return ((inverseOrdering % 2) == 0) ? comp : (-comp);
      }

      private String getSizeRepr(PublicationInfo pub) {
        long size = pub.getFileSize();
        return Long.toString(size);
      }

      @Override
      public void setColumnValue(Object o, Object value) {
      }

    };
  }

  /**
   * Mostra a janela com os resultados da recuperao.
   *
   * @param successList .
   * @param notSuccessList .
   *
   * @return .
   */
  private boolean showResult(Vector successList, Vector notSuccessList) {
    StringBuffer msg = new StringBuffer();
    boolean hasDetails = false;
    Vector detailList = new Vector();
    if (successList.size() > 0) {
      msg.append(LNG.get("PUB_LINK_FILES_RETRIEVED_WITH_SUCCESS") + "\n");
      for (Enumeration elem = successList.elements(); elem.hasMoreElements();) {
        PublicationInfo info = (PublicationInfo) elem.nextElement();
        msg.append(" - " + LNG.get("PUB_LINK_FILE_ITEM") + " ");
        msg.append(ProjectTreePath.makePath(info.getProjectName(),
          DIRECTORY_SEPARATOR, info.getPath()));
        msg.append("\n\n");
      }
    }
    if (notSuccessList.size() > 0) {
      msg.append(LNG.get("PUB_LINK_FILES_NOT_RETRIEVED") + "\n");
      for (Enumeration elem = notSuccessList.elements(); elem.hasMoreElements();) {
        PublicationError error = (PublicationError) elem.nextElement();
        if (error != null) {
          hasDetails = true;
          detailList.add(error);
        }
        msg.append(" - " + LNG.get("PUB_LINK_FILE_ITEM") + " ");
        msg.append(ProjectTreePath.makePath(error.info.getProjectName(),
          DIRECTORY_SEPARATOR, error.info.getPath()));
        msg.append("\n");
        msg.append("   " + LNG.get("PUB_LINK_ERROR_ITEM") + " ");
        msg.append(error.toString());
        msg.append("\n\n");
      }
    }
    if (!hasDetails) {
      JScrollPane msgArea = new JScrollPane(new JTextArea(msg.toString(), ROWS,
        COLS));
      StandardDialogs.showInfoDialog(mainFrame, LNG
        .get("PUB_LINK_FILES_RETRIEVER_RESULT_TITLE")
        + " - " + LNG.get("SERVER"), msgArea);
    }
    else {
      msg.append(LNG.get("GET_IN_TOUCH_DETAIL"));
      Object[] options = { LNG.get("PUB_LINK_DETAIL"),
          LNG.get("PUB_LINK_CLOSE") };
      JScrollPane msgArea = new JScrollPane(new JTextArea(msg.toString(), ROWS,
        COLS));
      int option = StandardDialogs.showOptionDialog(mainFrame, LNG
        .get("PUB_LINK_FILES_RETRIEVER_RESULT_TITLE")
        + " - " + LNG.get("SERVER"), msgArea, options);
      if (option == 0) {
        showDetails(detailList);
      }
    }
    return (notSuccessList.size() == 0);
  }

  /**
   * Mostra a janela com os resultados da recuperao.
   *
   * @param detailList .
   */
  private void showDetails(Vector detailList) {
    StringBuffer msg = new StringBuffer();
    for (Enumeration elem = detailList.elements(); elem.hasMoreElements();) {
      PublicationError error = (PublicationError) elem.nextElement();
      msg.append(LNG.get("PUB_LINK_FILE_ITEM") + " ");
      msg.append(ProjectTreePath.makePath(error.info.getProjectName(),
        DIRECTORY_SEPARATOR, error.info.getPath()));
      msg.append("\n");
      msg.append(LNG.get("PUB_LINK_ERROR_ITEM") + " ");
      msg.append(error.toString());
      msg.append("\n");
      msg.append(LNG.get("PUB_LINK_ERROR_DESCRIPTION_ITEM"));
      msg.append("\n");
      StackTraceElement[] trace = error.e.getStackTrace();
      for (int i = 0; i < trace.length; i++) {
        msg.append(trace[i].toString()).append("\n");
      }
      msg.append("\n");
      for (int i = 0; i < COLS; i++) {
        msg.append("=");
      }
    }
    JScrollPane msgArea = new JScrollPane(new JTextArea(msg.toString(), ROWS,
      COLS));
    StandardDialogs.showInfoDialog(mainFrame, LNG
      .get("PUB_LINK_FILES_RETRIEVER_RESULT_TITLE")
      + " - " + LNG.get("SERVER"), msgArea);
  }

  /**
   * Obtm o nome do usurio que publicou o arquivo.
   *
   * @param pub .
   *
   * @return .
   */
  private String getPublishedByRepr(PublicationInfo pub) {
    String strUser = new String();
    try {
      User user = User.getUser(pub.getUserWhoPublished());
      strUser = user.getName();
    }
    catch (Exception e) {
      strUser = LNG.get("PRJ_UNKNOWN");
    }
    return strUser;
  }

  /**
   * Classe que modela os erros que podem ocorrer durante a recuperao de um
   * arquivo publicado.
   */
  private class PublicationError {
    PublicationInfo info;
    Exception e;

    @Override
    public String toString() {
      return e.getMessage();
    }

    PublicationError(PublicationInfo info, Exception e) {
      this.info = info;
      this.e = e;
    }
  }

  /**
   * Modelo utilizado para a tabela de arquivos publicados.
   */
  public class DataTableModel extends AbstractTableModel {
    /**
     * Define o nmero de linhas da tabela.
     *
     * @return nmero de linhas
     */
    public int getRowCount() {
      return publishedData.size();
    }

    /**
     * Define o nmero de colunas da tabela.
     *
     * @return nmero de colunas
     */
    public int getColumnCount() {
      return tableColumns.length;
    }

    /**
     * Obtm o contedo a ser exibido em uma clula da tabela.
     *
     * @param row nmero da linha
     * @param column nmero da coluna
     *
     * @return o contedo a ser exibido
     */
    public Object getValueAt(int row, int column) {
      Object info = publishedData.elementAt(row);
      return tableColumns[column].getColumnValue(info);
    }

    /**
     * Ordena a tabela conforme o contedo de uma determinada coluna.
     *
     * @param column nmero da coluna
     */
    public void sort(int column) {
      Collections.sort(publishedData, tableColumns[column]);
      lastColumnSorted = column;
      fireTableChanged(new TableModelEvent(this));
    }

    /**
     * Obtm o ttulo de uma coluna.
     *
     * @param column nmero da coluna
     *
     * @return .
     */
    @Override
    public String getColumnName(int column) {
      return tableColumns[column].getColumnName();
    }
  }

  /**
   * Constri um dilogo para obteno de dados publicados.
   */
  public PublishedFileRetrieverDialog() {
    defineTableColumns();
    createMainFrame();
    loadPublishedData();
    display();
  }
}
