/*
 * $Id: CommonFileDownloadAction.java 68790 2007-10-09 19:49:38Z clinio $
 */
package csbase.client.project.action;

import csbase.client.externalresources.ExternalResources;
import csbase.client.project.ProjectFileContainer;
import csbase.client.remote.srvproxies.HttpProxy;
import csbase.client.remote.srvproxies.NotificationProxy;
import csbase.client.util.StandardErrorDialogs;
import csbase.logic.ClientProjectFile;
import csbase.logic.CommonClientProject;
import csbase.logic.ProjectFile;
import csbase.logic.User;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.StandardDialogs;

import java.awt.event.ActionEvent;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;

import java.net.URL;

import java.text.MessageFormat;

import javax.swing.JFileChooser;

/**
 * Ao de download de arquivo
 * 
 * @author Tecgraf/PUC-Rio
 */
public class CommonFileDownloadAction extends CommonProjectAction {
  
  /**
   * Indica se o arquivo a ser publicado foi originado da rvore do projeto ou
   * no
   */
  private boolean isFileFromTree;

  /**
   * Arquivo a ser exportado, no originado da rvore do projeto
   */
  private ProjectFile exportFile;

  /**
   * Construtor
   * 
   * @param container
   */
  public CommonFileDownloadAction(ProjectFileContainer container) {
    super(container);
    this.isFileFromTree = true;
  }

  /**
   * Constri a ao a partir de um arquivo que no est selecionado na rvore
   * do projeto.
   * 
   * @param container rvore do projeto.
   * @param file Arquivo sobre o qual a ao ser realizada.
   */
  public CommonFileDownloadAction(ProjectFileContainer container,
    ProjectFile file) {
    super(container);
    this.isFileFromTree = false;
    this.exportFile = file;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void actionPerformed(ActionEvent e) {
    ProjectFile file = (isFileFromTree) ? (ClientProjectFile) getSelectedFile()
      : this.exportFile;
    User user = User.getLoggedUser();
    CommonClientProject project = getProject();
    try {
      // Chamada ao Proxy.
      String addr = HttpProxy.getDownloadURL(user.getId(), project.getUserId(),
        project.getName(), file.getPath());
      if (addr == null) {
        StandardErrorDialogs.showErrorDialog(getWindow(), LNG
          .get("PRJ_PROJECT_FILE_DOWNLOAD_URL_ERROR"));
        return;
      }
      final URL url = new URL(addr);
      if (ExternalResources.getInstance().isEnabled()) {
        ExternalResources.getInstance().showDocument(url);
        return;
      }
      /* Tenta abrir o FileChooser. Caso nao consiga, esta rodando */
      /* como applet */
      try {
        JFileChooser chooser = new JFileChooser();
        try {
          int returnVal = chooser.showSaveDialog(getWindow());
          if (returnVal == JFileChooser.APPROVE_OPTION) {
            File fileChoosed = chooser.getSelectedFile();
            final FileOutputStream newFile = new FileOutputStream(fileChoosed);
            final String fileName = fileChoosed.getName();
            final Object userId = user.getId();
            Thread work = new Thread() {
              @Override
              public void run() {
                try {
                  String msg = MessageFormat.format(LNG
                    .get("PRJ_PROJECT_FILE_DOWNLOAD_START"),
                    new Object[] { fileName });
                  StandardDialogs.showInfoDialog(getWindow(), LNG
                    .get("PRJ_PROJECT_FILE_DOWNLOAD_TITLE"), msg);
                  InputStream urlIn = url.openStream();
                  InputStream in = new BufferedInputStream(urlIn);
                  int i;
                  byte[] b = new byte[1024];
                  while ((i = in.read(b)) != -1) {
                    newFile.write(b, 0, i);
                  }
                  urlIn.close();
                  msg = MessageFormat.format(LNG
                    .get("PRJ_PROJECT_FILE_DOWNLOAD_SUCCESS"),
                    new Object[] { fileName });
                  NotificationProxy.notifyTo(LNG
                    .get("PRJ_PROJECT_FILE_DOWNLOAD_TITLE"), userId, msg, true,
                    false);
                  newFile.close();
                }
                catch (Exception e) {
                  StandardErrorDialogs.showErrorDialog(getWindow(), LNG
                    .get("PRJ_PROJECT_FILE_DOWNLOAD_IO_ERROR"), e);
                  return;
                }
              }
            };
            work.start();
          }
        }
        catch (Exception ioEx) {
          StandardErrorDialogs.showErrorDialog(getWindow(), LNG
            .get("PRJ_PROJECT_FILE_DOWNLOAD_IO_ERROR"), ioEx);
        }
        /* Se deu erro, e porque esta sendo chamado pelo applet */
      }

      /*
       * catch (Exception ex) { if (Client.getInstance() == null) { return; }
       * try { Client.getInstance().showDocument(url); }
       */
      catch (Exception appletEx) {
        StandardErrorDialogs.showErrorDialog(getWindow(), LNG
          .get("PRJ_PROJECT_FILE_DOWNLOAD_APPLET_ERROR"), appletEx);
      }

      // }
    }
    catch (Exception URLEx) {
      StandardErrorDialogs.showErrorDialog(getWindow(), LNG
        .get("PRJ_PROJECT_FILE_DOWNLOAD_URL_ERROR"), URLEx);
      return;
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public String getName() {
    return LNG.get("PRJ_DOWNLOAD_FILE");
  }
}
