/*
 * FileDoubleClickAction.java
 * 
 * $Author: jnlopes $ $Revision: 100840 $ - $Date: 2007-08-15 17:07:03 -0300
 * (Wed, 15 Aug 2007) $
 */
package csbase.client.project.action;

import java.awt.event.ActionEvent;

import javax.swing.Action;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.applicationmanager.ApplicationException;
import csbase.client.applicationmanager.ApplicationManager;
import csbase.client.applicationmanager.ApplicationType;
import csbase.client.applications.ApplicationProject;
import csbase.client.project.ProjectFileContainer;
import csbase.client.util.StandardErrorDialogs;
import csbase.logic.ProjectFile;
import csbase.logic.ProjectFileType;

/**
 * Ao que abre o arquivo na aplicao informada no construtor ou associada ao
 * tipo.
 */
public class CommonFileOpenAction extends CommonProjectAction {

  /** Identificador da aplicao que ir abrir o arquivo selecionado. */
  private String applicationId;

  /** Indica se alguma aplicao foi configurada para abrir arquivos */
  private boolean appSet;

  /** Nome da ao */
  private String name;

  /**
   * Constri a ao com uma aplicao padro para abrir arquivos.
   * 
   * @param container formato de dados que representa um
   *        <code>ProjectFile</code>.
   * @param application identificador da aplicao que ir abrir o arquivo
   *        selecionado
   * @param name nome da ao
   */
  public CommonFileOpenAction(ProjectFileContainer container,
    String application, String name) {
    super(container);
    this.appSet = true;
    this.applicationId = application;
    putValue(Action.NAME, name);
    this.name = name;
  }

  /**
   * Constri a ao sem uma aplicao para abrir arquivos.
   * 
   * @param container formato de dados que representa um
   *        <code>ProjectFile</code>.
   */
  public CommonFileOpenAction(ProjectFileContainer container) {
    super(container);
    this.appSet = false;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void actionPerformed(ActionEvent e) {
    openFile(getSelectedFile());
  }

  /**
   * Abre o arquivo na aplicao.
   * 
   * @param file arquivo a ser aberto
   */
  public void openFile(ProjectFile file) {
    String appId = getApplicationId();
    ProjectFile selectedFile = getSelectedFile();
    if (appId == null) {
      ProjectFileType pft = ProjectFileType.getFileType(selectedFile.getType());
      String msg =
        String.format(LNG.get("CommonFileOpenAction.error"), pft
          .getDescription());
      StandardDialogs.showInfoDialog(getWindow(), String.format(LNG
        .get("CommonFileOpenAction.title"), selectedFile.getName()), msg);
      return;
    }
    ApplicationType app;
    try {
      app = ApplicationManager.getInstance().runApplication(appId);
      app.sendMessage(ApplicationProject.PROJECT_FILE_MESSAGE, selectedFile);
    }
    catch (ApplicationException ex) {
      StandardErrorDialogs.showErrorDialog(getWindow(), ex.getMessage(), ex);
    }
  }

  /**
   * Retorna o identificador da aplicao, se uma aplicao tiver sido
   * configurada. Se no procura uma aplicao que abre o tipo de arquivo
   * selecionado.
   * 
   * @return o identificador da aplicao que abre o arquivo ou null caso no
   *         encontre aplicao cadastrada para abr-lo
   */
  private String getApplicationId() {
    if (appSet) {
      return applicationId;
    }
    return ApplicationManager.getInstance().getApplicationFromType(
      getSelectedFile().getType());
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public String getName() {
    if (name == null) {
      return LNG.get("CommonFileOpenAction.name");
    }
    return name;
  }

}
