/*
 * $Id$
 */
package csbase.client.project.dialogs;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.ScrollPaneConstants;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import tecgraf.javautils.LNG;
import tecgraf.javautils.configurationmanager.Configuration;
import tecgraf.javautils.configurationmanager.ConfigurationManager;
import tecgraf.javautils.gui.GUIUtils;
import tecgraf.javautils.gui.StandardDialogs;
import csbase.client.desktop.DesktopComponentDialog;
import csbase.client.desktop.DesktopFrame;
import csbase.client.desktop.RemoteTask;
import csbase.client.project.action.CommonProjectOpenAnyAction;
import csbase.client.project.tasks.OpenProjectOnTreeRemoteTask;
import csbase.logic.CommonClientProject;
import csbase.logic.User;
import csbase.logic.UserOutline;
import csbase.util.StringUtils;

/**
 * Classe responsvel por implementar a interface de abertura de todos os
 * projetos. Esta interface  visvel apenas pelo administrador por ser o nico
 * que pode abrir qualquer projeto do sistema. Exibe um dilogo com duas listas:
 * a da esquerda para seleo de usurio (passo 1) e a da direita para seleo
 * do projeto (passo 2). A lista da direita  modificada a cada nova seleo de
 * usurio.
 * 
 * @deprecated esta classe no  mais necessria dado s era usada por
 *             {@link CommonProjectOpenAnyAction}, que foi marcada como
 *             deprecated
 */
@Deprecated
public class OpenAnyProjectDialog {

  /**
   * A janela de seleo de projeto.
   */
  private DesktopComponentDialog dialog;

  /**
   * Projeto selecionado.
   */
  private CommonClientProject project;

  /**
   * Janela pai do dilogo.
   */
  private Window owner;

  /**
   * Boto default da janela
   */
  private JButton openButton;

  /**
   * Componente com a lista de usurios
   */
  private JList usersList;

  /**
   * Componente com a lista de projetos
   */
  private JList projectsList;

  /**
   * Cache de usurios do sistema com os seus projetos. Associa o login do
   * usurio a um array de projetos.
   */
  private HashMap<Object, String[]> userProjects;

  /**
   * Estrutura criada para representar a lista de usurios. Poderia usar o
   * <code>UserOutline</code>, mas o mtodo <code>toString</code> exibe
   * informaes demais.
   */
  private class UserData implements Comparable<UserData> {
    /**
     * Id
     */
    final Object id;
    /**
     * Nome
     */
    final String name;

    /**
     * Construtor
     * 
     * @param id id
     * @param name nome
     */
    UserData(Object id, String name) {
      this.id = id;
      this.name = name;
    }

    /**
     * {@inheritDoc}
     */
    public int compareTo(UserData o) {
      return StringUtils.compare(name, o.name);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String toString() {
      return name;
    }
  };

  /**
   * Construtor.
   * 
   * @param owner Janela pai.
   */
  private OpenAnyProjectDialog(Window owner) {
    this.owner = owner;
    userProjects = new HashMap<Object, String[]>();
  }

  /**
   * Mostra um dilogo com um a lista de nomes dos usurios e outra com os nomes
   * dos projetos e aguarda que um projeto seja selecionado. O dilogo  modal.
   * No caso da confimao da seleo, o projeto  aberto.
   * 
   * @param owner A janela a partir da qual esse dilogo foi chamado.
   * 
   * @return O projeto selecionado.
   */
  public static CommonClientProject openProject(Window owner) {
    OpenAnyProjectDialog openDialog = new OpenAnyProjectDialog(owner);
    openDialog.buildDialog();
    return openDialog.project;
  }

  /**
   * Coloca os componentes da interface na janela.
   */
  private void buildDialog() {
    JPanel mainPanel = new JPanel(new BorderLayout());
    mainPanel.add(makeProjectPanel(), BorderLayout.CENTER);
    mainPanel.add(makeButtonsPanel(), BorderLayout.SOUTH);

    // Cria um dilogo modal
    if (owner instanceof JFrame) {
      dialog = new DesktopComponentDialog((JFrame) owner, getTitle());
    }
    else {
      dialog = new DesktopComponentDialog((JDialog) owner, getTitle());
    }
    dialog.addWindowListener(new WindowAdapter() {
      @Override
      public void windowClosing(WindowEvent e) {
        dialog.close();
      }
    });
    dialog.getContentPane().add(mainPanel);
    dialog.pack();
    dialog.center(owner);
    dialog.getRootPane().setDefaultButton(openButton);
    dialog.setVisible(true);
  }

  /**
   * Constri o painel com a lista de usurios de de projetos.
   * 
   * @return o painel.
   */
  private JPanel makeProjectPanel() {
    JPanel panel = new JPanel(new GridBagLayout());

    int width;
    int height;
    final int defaultWidth = 230;
    final int defaultHeight = 170;
    try {
      final ConfigurationManager mng = ConfigurationManager.getInstance();
      final Configuration cnf = mng.getConfiguration(this.getClass());
      width = cnf.getOptionalIntegerProperty("list.width", defaultWidth);
      height = cnf.getOptionalIntegerProperty("list.height", defaultHeight);
    }
    catch (Exception e) {
      width = defaultWidth;
      height = defaultHeight;
    }
    final Dimension listPreferredSize = new Dimension(width, height);

    final String explanationMsg = LNG.get("OpenAnyProjectDialog.explanation");
    JLabel explanationLabel = new JLabel(explanationMsg);
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.gridx = 0;
    constraints.gridy = 0;
    constraints.gridwidth = 2;
    constraints.anchor = GridBagConstraints.WEST;
    constraints.fill = GridBagConstraints.HORIZONTAL;
    constraints.insets = new Insets(10, 10, 5, 5);
    panel.add(explanationLabel, constraints);

    JLabel usersLabel = new JLabel(LNG.get("OpenAnyProjectDialog.users"));
    constraints = new GridBagConstraints();
    constraints.gridx = 0;
    constraints.gridy = 1;
    constraints.anchor = GridBagConstraints.WEST;
    constraints.fill = GridBagConstraints.HORIZONTAL;
    constraints.insets = new Insets(5, 10, 5, 5);
    panel.add(usersLabel, constraints);

    usersList = new JList(loadUsers());
    usersList.addListSelectionListener(new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent e) {
        loadProjects();
      }
    });
    usersList.getSelectionModel().setSelectionMode(
      ListSelectionModel.SINGLE_SELECTION);

    constraints = new GridBagConstraints();
    constraints.gridx = 0;
    constraints.gridy = 2;
    constraints.fill = GridBagConstraints.BOTH;
    constraints.weightx = 1.0;
    constraints.weighty = 1.0;
    constraints.anchor = GridBagConstraints.WEST;
    constraints.insets = new Insets(0, 10, 5, 5);
    JScrollPane usersScrollPane =
      new JScrollPane(usersList,
        ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
        ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);

    usersScrollPane.setPreferredSize(listPreferredSize);
    panel.add(usersScrollPane, constraints);

    JLabel projectsLabel = new JLabel(LNG.get("OpenAnyProjectDialog.projects"));
    constraints = new GridBagConstraints();
    constraints.gridx = 1;
    constraints.gridy = 1;
    constraints.anchor = GridBagConstraints.WEST;
    constraints.fill = GridBagConstraints.HORIZONTAL;
    constraints.insets = new Insets(5, 5, 5, 10);
    panel.add(projectsLabel, constraints);

    projectsList = new JList();
    projectsList.getSelectionModel().setSelectionMode(
      ListSelectionModel.SINGLE_SELECTION);

    // Associa o duplo clique a ao principal do dilogo
    MouseListener mouseListener = new MouseAdapter() {
      @Override
      public void mouseClicked(MouseEvent e) {
        if (e.getClickCount() == 2) {
          openProject();
        }
      }
    };
    projectsList.addMouseListener(mouseListener);

    constraints = new GridBagConstraints();
    constraints.gridx = 1;
    constraints.gridy = 2;
    constraints.fill = GridBagConstraints.BOTH;
    constraints.weightx = 1.0;
    constraints.weighty = 1.0;
    constraints.anchor = GridBagConstraints.WEST;
    constraints.insets = new Insets(0, 5, 5, 10);
    JScrollPane projectsScrollPane =
      new JScrollPane(projectsList,
        ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
        ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    projectsScrollPane.setPreferredSize(listPreferredSize);
    panel.add(projectsScrollPane, constraints);

    // Seleciona o primeiro usurio
    usersList.setSelectedIndex(0);

    return panel;
  }

  /**
   * Constri o painel com os botes.
   * 
   * @return o painel.
   */
  private JPanel makeButtonsPanel() {
    JPanel panel = new JPanel();

    openButton = new JButton(LNG.get("OpenAnyProjectDialog.open"));
    openButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        openProject();
      }
    });
    panel.add(openButton);

    JButton cancelButton = new JButton(LNG.get("OpenAnyProjectDialog.cancel"));
    cancelButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        dialog.close();
      }
    });
    panel.add(cancelButton);

    GUIUtils.matchPreferredSizes(new JComponent[] { openButton, cancelButton });

    return panel;
  }

  /**
   * Informa o ttulo que vai ser utilizado na janela de dilogo.
   * 
   * @return O ttulo da janela.
   */
  private String getTitle() {
    return LNG.get("OpenAnyProjectDialog.title");
  }

  /**
   * Obtm todos os usurio do sistema.
   * 
   * @return a lista (vetor de usurios)
   */
  private Vector<UserData> loadUsers() {
    Vector<UserData> usersData = new Vector<UserData>();
    RemoteTask<List<UserOutline>> task = new RemoteTask<List<UserOutline>>() {
      @Override
      protected void performTask() throws Exception {
        setResult(User.getAllOutlines());
      }
    };

    if (task.execute(owner, getTitle(), LNG
      .get("OpenAnyProjectDialog.getUsers"))) {
      List<UserOutline> result = task.getResult();
      for (int i = 0; i < result.size(); i++) {
        UserOutline outline = result.get(i);
        usersData.add(new UserData(outline.getId(), outline.getName()));
      }
      // Ordena
      Collections.sort(usersData);
    }
    return usersData;
  }

  /**
   * Preenche a lista com os projetos.
   */
  private void loadProjects() {
    UserData user = (UserData) usersList.getSelectedValue();
    if (user == null) {
      return;
    }
    final String[] projects = getProjects(user.id);
    if (projects != null) {
      final String[] projectNames = StringUtils.sort(projects);
      projectsList.setListData(projectNames);
      projectsList.setSelectedIndex(0);
    }
    else {
      projectsList.setListData(new Object[0]);
    }
  }

  /**
   * Obtm os projetos do usurio
   * 
   * @param userLogin Identificador do usurio.
   * @return Os nomes do projeto
   */
  private String[] getProjects(final Object userLogin) {
    String[] projectNames = userProjects.get(userLogin);
    if (projectNames != null) {
      return projectNames;
    }

    RemoteTask<String[]> task = new RemoteTask<String[]>() {
      @Override
      protected void performTask() throws Exception {
        setResult(CommonClientProject.getAllProjectNames(userLogin));
      }
    };

    if (task.execute(owner, getTitle(), String.format(LNG
      .get("OpenAnyProjectDialog.getProjects"), userLogin))) {
      projectNames = task.getResult();
      if (projectNames != null) {
        Arrays.sort(projectNames);
        userProjects.put(userLogin, projectNames);
      }
    }
    return projectNames;
  }

  /**
   * Obtm o projeto e o atribui a varivel local <code>project</code> Em
   * seguida, fecha a janela.
   */
  private void openProject() {
    UserData user = (UserData) usersList.getSelectedValue();
    String projectName = (String) projectsList.getSelectedValue();
    if (user == null || projectName == null) {
      StandardDialogs.showErrorDialog(dialog, getTitle(), LNG
        .get("OpenAnyProjectDialog.info.select.project"));
      return;
    }

    OpenProjectOnTreeRemoteTask openProjectTask =
      new OpenProjectOnTreeRemoteTask(projectName, user.id, DesktopFrame
        .getInstance().getTree());
    if (openProjectTask.execute(dialog, LNG.get("OpenAnyProjectDialog.title"),
      String.format(LNG.get("OpenAnyProjectDialog.opening.project"),
        projectName, user.name))) {
      this.project = openProjectTask.getResult();
    }
    dialog.close();
  }
}
