/*
 * $Id: CoreTask.java 79318 2008-06-27 22:25:25Z clinio $
 */
package csbase.client.project.tasks;

import tecgraf.javautils.LNG;
import csbase.client.desktop.DesktopComponentFrame;
import csbase.client.desktop.DesktopFrame;
import csbase.client.desktop.RemoteTask;

/**
 * Classe que implementa a tarefa de ajustar o out-of-date em todo
 * projeto.
 * 
 * @author Tecgraf/PUC-Rio
 * @param <T> o tipo do resultado da task conforme {@link RemoteTask}.
 */
abstract class CoreTask<T> extends RemoteTask<T> {

  /**
   * Mtodo interno de convenincia.
   * @param msg o texto da task. 
   * @return o indicativo de {@link RemoteTask#execute(java.awt.Window, String, String)}
   */
  private boolean execute(final String msg) {
    final DesktopFrame dskFrame = DesktopFrame.getInstance();
    final DesktopComponentFrame frm = dskFrame.getDesktopFrame();
    
    final String className = getClass().getSimpleName();
    final String title = LNG.get(className+".title");

    return execute(frm, title, msg);
  }

  /**
   * Mtodo de convenincia para execuo da task com base
   * na janela de desktop e com textos ttulo e mensagem padronizados.
   * @return o indicativo de {@link #execute(String)}. 
   */
  protected boolean execute() {
    final String className = getClass().getSimpleName();
    final String msg = LNG.get(className+".msg");
    return execute(msg);
  }

  /**
   * Mtodo de convenincia para execuo da task com base
   * na janela de desktop e com texto ttulo padronizado; a mensagem
   * ser formatada conforme o parmetro
   * @param args os argumentos de formatao da mensagem.
   * @return o indicativo de {@link #execute(String)}. 
   */
  protected boolean execute(final Object[] args) {
    final String className = getClass().getSimpleName();
    final String fmt = LNG.get(className+".msg");
    final String msg = String.format(fmt, args);
    return execute(msg);
  }

}
