/*
 * $Id: CreateFileTask.java 68848 2007-10-10 18:45:37Z taciana $
 */
package csbase.client.project.tasks;

import java.rmi.RemoteException;

import csbase.client.desktop.RemoteTask;
import csbase.exception.CSBaseRuntimeException;
import csbase.logic.ClientProjectFile;
import csbase.logic.CommonClientProject;
import csbase.logic.ProjectFile;

/**
 * A classe <code>CreateFileTask</code> implementa uma tarefa que cria um novo
 * arquivo em um projeto. O tratamento de RemoteException  delegado  super-
 * classe. O resultado da execuo pode ser verificado atravs do mtodo 
 * {@link #getErrorCode()}.
 */
public class CreateFileTask extends RemoteTask<ClientProjectFile> {

  /**
   * Cdigos de erro.
   */
  public static enum ErrorCode {
    NO_ERROR,
    NOT_EXECUTED,
    RMI_FAILURE,
    UNKNOWN, 
  }
  
  /**
   * Projeto da forma como ele  visto pelo cliente.
   */
  private CommonClientProject clientProject;
  /**
   * Diretrio onde ser criado o arquivo.
   */
  private ProjectFile parent;
  /**
   * Nome do novo arquivo.
   */
  private String fileName;
  /**
   * Tipo do novo arquivo.
   */
  private String fileType;
  /**
   * Cdigo de erro associado  execuo. 
   */
  private ErrorCode errorCode = ErrorCode.NOT_EXECUTED;

  /**
   * Construtor.
   * 
   * @param clientProject - projeto (viso do cliente)
   * @param parent - diretrio onde ser criado o arquivo
   * @param fileName - nome do novo arquivo
   * @param fileType - tipo do novo arquivo
   */
  public CreateFileTask(CommonClientProject clientProject,
                        ProjectFile parent,
                        String fileName,
                        String fileType) {
    this.clientProject = clientProject;
    this.parent = parent;
    this.fileName = fileName;
    this.fileType = fileType;
  }

  /**
   * Est provisoriamente chamando o handle de RemoteTask apenas para 
   * RemoteException e CSBaseRuntimeException. Verificar se  mesmo 
   * necessrio continuar fazendo outros tratamentos.
   * {@inheritDoc}
   */
  @Override
  protected void handleError(Exception error) {
    if (error instanceof RemoteException) {
      super.handleError(error);
      errorCode = ErrorCode.RMI_FAILURE;
    }
    else if (error instanceof CSBaseRuntimeException){
      super.handleError(error);
      errorCode = ErrorCode.UNKNOWN;
    }
    else {
      errorCode = ErrorCode.UNKNOWN;
    }
    setResult(null);
  }

  /**
   * Retorna o cdigo de erro associado  execuo.
   * 
   * @return cdigo de erro associado  execuo
   */
  public ErrorCode getErrorCode() {
    return errorCode;
  }
  
  /**
   * Cria o arquivo.
   */
  @Override
  protected void performTask() throws Exception {
    ClientProjectFile newFile = (ClientProjectFile) clientProject
      .createAndWaitForFile(parent, fileName, fileType);
    setResult(newFile);
    errorCode = ErrorCode.NO_ERROR;
  }

}
