/*
 * AdministrationObserver.java
 *
 * $Author: taciana $
 * $Revision: 54777 $  - $Date: 2006-08-15 11:01:24 -0300 (Tue, 15 Aug 2006) $
 */
package csbase.client.remote;

import csbase.client.util.StandardErrorDialogs;
import csbase.logic.AdministrationEvent;
import csbase.logic.Permission;
import csbase.logic.Platform;
import csbase.logic.Role;
import csbase.logic.User;
import csbase.logic.UserGroup;
import csbase.remote.AdministrationServiceInterface;
import csbase.remote.RemoteObserver;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

import tecgraf.javautils.LNG;

/**
 * A classe <code>AdministrationObserver</code> implementa um observador remoto
 * do servio de administrao. Possui apenas uma nica instncia. Interage
 * com as classes {@link csbase.logic.User User}, {@link
 * csbase.logic.UserGroup UserGroup}, {@link csbase.logic.Role Role} e {@link
 * csbase.logic.Platform Platform} para manter as informaes dos objetos
 * atualizadas.
 *
 * @author $Author: taciana $
 * @version $Revision: 54777 $
 */
public class AdministrationObserver extends UnicastRemoteObject
implements RemoteObserver {
  /** Instncia nica do observador */
  private static AdministrationObserver instance = null;
  private Object arg;

  /**
   * Inicializa os servios desta classe: cria o nico objeto deste tipo e o
   * registra como observador do AdministrationService.
   *
   * @throws Exception .
   */
  public static void start()
  throws Exception {
    if (instance != null) {
      return;
    }
    instance = new AdministrationObserver();
  }

  public static void stop()
  throws RemoteException {
    if (instance == null) {
      return;
    }
    instance.uninstall();
  }

  /**
   * Registra no servidor um observador.
   *
   * @throws RemoteException .
   */
  public void install()
  throws RemoteException {
    User loggedUser = User.getLoggedUser();
    if (loggedUser == null || loggedUser.isAdmin()) {
      arg = null;
    }
    else {
      //Cadastra o observador do usurio
      arg = loggedUser.getId();
    }
    ClientRemoteMonitor.getInstance().addObserver(
      AdministrationServiceInterface.SERVICE_NAME, this, arg);
  }

  /**
   * Remove do servidor o registro desse observador.
   *
   * @throws RemoteException .
   */
  public void uninstall()
  throws RemoteException {
    ClientRemoteMonitor.getInstance().deleteObserver(
      AdministrationServiceInterface.SERVICE_NAME, this, arg);
    instance = null;
    unexportObject(this, true);
  }

  /**
   * Esse mtodo  chamado sempre que alguma alterao ocorre no servio de
   * administrao. Informa  classe correspondente da lgica sobre o evento
   * ocorrido. Esse evento  representado por um objeto da classe {@link
   * csbase.logic.AdministrationEvent AdministrationEvent}.
   *
   * @param remoteEvent Descreve o evento ocorrido no servio de administrao
   *
   * @throws RemoteException .
   */
  public void update(Object remoteEvent)
  throws RemoteException {
    try {
      AdministrationEvent event = (AdministrationEvent)remoteEvent;
      if (event.item instanceof User) {
        User.update(event);
      }
      else if (event.item instanceof UserGroup) {
        UserGroup.update(event);
      }
      else if (event.item instanceof Role) {
        Role.update(event);
      }
      else if (event.item instanceof Platform) {
        Platform.update(event);
      }
      else if (event.item instanceof Permission) {
        Permission.update(event);
      }
    }
    catch (Exception e) {
      StandardErrorDialogs.showErrorDialog(null, LNG.get("ERRO"), e);
    }
  }

  /**
   * Cria um observador remoto de um provedor de usurios. Exporta esse objeto
   * para o sistema rmi.
   *
   * @throws Exception
   */
  protected AdministrationObserver()
  throws Exception {
    install();
  }
}
