/**
 * $Id: ProjectObserver.java 105206 2010-05-06 23:06:04Z costa $
 */
package csbase.client.remote;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

import javax.swing.SwingUtilities;

import csbase.logic.CommonClientProject;
import csbase.logic.ProjectArgObserver;
import csbase.logic.ProjectEvent;
import csbase.remote.ProjectServiceInterface;
import csbase.remote.RemoteObserver;

/**
 * Permite aos clientes observar mudanas em projetos abertos.
 */
public class ProjectObserver extends UnicastRemoteObject implements
  RemoteObserver {
  /** O projeto correspondente a esse observador. */
  private CommonClientProject project;

  /**
   * Registra no servidor um observador.
   * 
   * @throws RemoteException 
   */
  public void install() throws RemoteException {
    if (!ClientRemoteMonitor.getInstance().addObserver(
      ProjectServiceInterface.SERVICE_NAME, this,
      new ProjectArgObserver(project.getUserId(), project.getName()))) {
      try {
        project.reopen();
        ClientRemoteMonitor.getInstance().addObserver(
          ProjectServiceInterface.SERVICE_NAME, this,
          new ProjectArgObserver(project.getUserId(), project.getName()));
      }
      catch (Exception e) {
        throw new RemoteException("ProjectObserver.install: " + e);
      }
    }
  }

  /**
   * Remove do servidor o registro desse observador.
   * 
   * @throws RemoteException 
   */
  public void uninstall() throws RemoteException {
    ClientRemoteMonitor.getInstance().deleteObserver(
      ProjectServiceInterface.SERVICE_NAME, this,
      new ProjectArgObserver(project.getUserId(), project.getName()));
    unexportObject(this, true);
  }

  /**
   * Trata os eventos que notificam qualquer mudana sobre um projeto. Eventos e
   * informaes necessrias no tratamento: - mudana nos dados do projeto .
   * objeto do tipo ProjectInfo contendo as novas informaes - remoo do
   * projeto . nenhuma informao necessria - mudanas na rvore do projeto -
   * incluso de um arquivo . caminho do arquivo . objeto do tipo
   * ClientProjectFile que represente o novo arquivo - remoo de um arquivo .
   * caminho do arquivo - mudana de nome de um arquivo . caminho do arquivo .
   * novo nome do arquivo - mudana da localizao de um arquivo . caminho do
   * arquivo . novo caminho do arquivo - mudana do estado do arquivo (global,
   * publicado, construo, normal) . caminho do arquivo . novo estado do
   * arquivo
   * 
   * @param arg o argumento.
   * 
   * @throws RemoteException
   */
  public void update(final Object arg) throws RemoteException {
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        ProjectObserver.this.project.update((ProjectEvent) arg);
      }
    });
  }

  /**
   * Cria um observador para um projeto.
   * 
   * @param cp O projeto que est usando esse observador.
   * 
   * @throws Exception de rmi.
   */
  public ProjectObserver(CommonClientProject cp) throws Exception {
    project = cp;
    install();
  }
}
