/*
 * $Id: DiskUsageProxy.java 83625 2008-10-01 19:59:50Z clinio $
 */

package csbase.client.remote.srvproxies;

import java.util.Hashtable;

import javax.swing.JFrame;

import tecgraf.javautils.LNG;
import csbase.client.desktop.DesktopFrame;
import csbase.client.desktop.RemoteTask;
import csbase.client.util.StandardErrorDialogs;
import csbase.logic.diskusageservice.DiskOccupation;
import csbase.remote.ClientRemoteLocator;
import csbase.remote.DiskUsageServiceInterface;

/**
 * Classe que modela um proxy ISI/DiskUsageService.
 *
 * @author Tecgraf/PUC-Rio
 */
public class DiskUsageProxy {

  /**
   * Consulta um texto de internacionalizao.
   * @param tag o tag
   * @return o texto
   */
  private static String getString(final String tag) {
    final String className = DiskUsageProxy.class.getSimpleName();
    final String newTag = className + "." + tag;
    return LNG.get(newTag);
  }

  /**
   * Obtm o uso de disco por usurios.
   *
   * @return tabela (hash) contendo uso de disco (em Mb) para cada usurio
   */
  public static Hashtable<Object, Double> getUsersList() {
    RemoteTask<Hashtable<Object, Double>> task = new RemoteTask<Hashtable<Object, Double>>() {
      @Override
      public void performTask() throws Exception {
        final DiskUsageServiceInterface dskService = ClientRemoteLocator.diskUsageService;
        if (dskService == null) {
          return;
        }
        setResult(dskService.getUsersList());
      }
    };


    final DesktopFrame mainFrame = DesktopFrame.getInstance();
    if ((mainFrame == null)) {
      return null;
    }

    final JFrame desktop = mainFrame.getDesktopFrame();
    final String PRE = "disk.request";

    final String title = getString(PRE+".title");
    final String message = getString(PRE+".message");
    if (task.execute(desktop, title, message)) {
      Hashtable<Object, Double> info = task.getResult();
      if (info != null) {
        return info;
      }
    }

    final String errTitle = getString(PRE+".error.title");
    final String errMessage = getString(PRE+".error.message");
    StandardErrorDialogs.showErrorDialog(desktop, errTitle, errMessage);
    return null;
  }

  /**
   * Obtm o uso de disco de um projeto do usurio
   *
   * @param ownerId o identificador de owner.
   * @param projName o identificador de projeto.
   * @return o valor em (Mb)
   */
  public static double getUsedSpaceForProjectMb(final Object ownerId,
    final String projName) {
    RemoteTask<Double> task = new RemoteTask<Double>() {
      @Override
      public void performTask() throws Exception {
        final DiskUsageServiceInterface dskService = ClientRemoteLocator.diskUsageService;
        if (dskService == null) {
          return;
        }
        setResult(dskService.getUsedSpaceForProjectMb(ownerId, projName));
      }
    };
    final String PRE = "project.usage";
    final JFrame desktop = DesktopFrame.getInstance().getDesktopFrame();
    final String title = getString(PRE+".title");
    final String message = getString(PRE+".message");
    if (task.execute(desktop, title, message)) {
      final double used = task.getResult();
      return used;
    }

    final String errTitle = getString(PRE+".error.title");
    final String errMessage = getString(PRE+".error.message");
    StandardErrorDialogs.showErrorDialog(desktop, errTitle, errMessage);
    return 0.0;
  }

  /**
   * Obtm informao do servio de ocupao de espao em disco. Em caso de erro
   * na obteno da informao (valor -1 informado pelo servio) exibe janela de
   * erro.
   * @param task a tarefa de consulta ao servidor
   * @return informao solicitada 0 em caso de erro
   */
  private static DiskOccupation getInfo(RemoteTask<DiskOccupation> task) {
    final JFrame desktop = DesktopFrame.getInstance().getDesktopFrame();

    /*
     * O servio de ocupao de disco no lana excees por erro na obteno da
     * informao (retorna -1). Falhas na chamada a execute ento resultam de
     * erros de comunicao (j sinalizado ao usurio) ou de cancelamento (no
     * necessita sinalizao)
     */
    if (task.execute(desktop, LNG.get("DISK_REQUEST_TITLE"), LNG
      .get("DISK_REQUEST_MESSAGE"))) {
      return task.getResult();
    }
    return new DiskOccupation();
  }

  /**
   * Mtodo para obter o espao livre em um disco.
   *
   * @return espao livre em um disco ou 0 em caso de erro
   */
  public static DiskOccupation getServerOccupation() {
    RemoteTask<DiskOccupation> task = new RemoteTask<DiskOccupation>() {
      @Override
      public void performTask() throws Exception {
        final DiskUsageServiceInterface diskUsageService = ClientRemoteLocator.diskUsageService;
        final DiskOccupation serverOccupation = diskUsageService.getServerOccupation();
        setResult(serverOccupation);
      }
    };
    return getInfo(task);
  }

  /**
   * Mtodo para obter o espao livre em um disco.
   *
   * @return espao livre em um disco ou 0 em caso de erro
   */
  public static DiskOccupation getProjectOccupation() {
    RemoteTask<DiskOccupation> task = new RemoteTask<DiskOccupation>() {
      @Override
      public void performTask() throws Exception {
        final DiskUsageServiceInterface diskUsageService = ClientRemoteLocator.diskUsageService;
        final DiskOccupation projectOccupation = diskUsageService.getProjectOccupation();
        setResult(projectOccupation);
      }
    };
    return getInfo(task);
  }
}
