package csbase.client.util;

import javax.swing.ImageIcon;
import javax.swing.JLabel;

import csbase.client.applications.ApplicationImages;

/**
 * Representa uma barra de status a ser utilizada em interfaces de usurio.
 * Obs. Uma barra de status  apenas um label com borda.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class StatusBar extends JLabel {

  private static final String _CLEAR_STATUS_STRING = " ";
   
  /**
   * Construtor
   */
  public StatusBar() {
    super();
    super.setVisible(true);
        
    hideStatusBar();
  }
  
  /**
   * Faz com que o componente seja visvel ou invisvel.
   * 
   * @param visible
   * 
   * @deprecated use {@link #showStatusBar()} ou {@link #hideStatusBar()}
   * @see #showStatusBar()
   * @see #hideStatusBar()
   */
  @Override
  @Deprecated 
  public void setVisible(boolean visible) {
    super.setVisible(visible);
  }
  
  /**
   * Mostra a barra de status se a condio dada for verdadeira.
   * 
   * @param condition
   */
  public void showStatusBar(final boolean condition) {
    setStatus(condition && isHidden() , _CLEAR_STATUS_STRING);
  }  
    
  /**
   * Mostra a barra de status.
   */
  public void showStatusBar() {
    setStatus(true, _CLEAR_STATUS_STRING);
  }
 
  /**
   * Esconde a barra de status se a condio dada for verdadeira.
   * 
   * @param condition
   */
  public void hideStatusBar(final boolean condition) {
    setStatus(condition, null);
  }    
  
  /**
   * Esconde a barra de status.
   */
  public void hideStatusBar() {
    setStatus(true, null);
  }  

  /**
   * @return true se a barra de status estiver escondida.
   */
  public boolean isHidden() {
    return null == getText() || "".equals(getText());
  }    
  
  /**
   * Limpa a barra de status se a condio dada for verdadeira.
   * Obs. Esse mtodo no faz com que a barra de status aparea 
   * caso ela esteja escondida.<br/>
   * 
   * @param condition
   */
  public void clearStatus(final boolean condition) {
    setStatus(condition && !isHidden(), _CLEAR_STATUS_STRING);         
  }  
  
  /**
   * Limpa a barra de status.
   */
  public void clearStatus() {
    clearStatus(true);
  }

  /**
   * Escreve uma mensagem na barra de status precedida por um cone 
   * se a condio dada for verdadeira.<br/>
   * Obs. caso a barra de status estivesse escondida, ela ir aparecer 
   * para apresentar a mensagem.<br/>
   * Obs. se a mensagem for vazia (<code>""</code>) ou nula (<code>null</code>)
   * e se o cone for nulo (<code>null</code>), ser o mesmo que chamar 
   * {@link #hideStatusBar()}.
   *  
   * @param condition 
   * @param icon
   * @param message
   */
  public void setStatus(final boolean condition, 
    final ImageIcon icon, final String message) {
    
    if (condition) {
      setIcon(icon);
      setText(message);
    }
  }  
  
  /**
   * Escreve uma mensagem na barra de status precedida por um cone.
   * Obs. caso a barra de status estivesse escondida, ela ir aparecer 
   * para apresentar a mensagem.<br/>
   * Obs. se a mensagem for vazia (<code>""</code>) ou nula (<code>null</code>)
   * e se o cone for nulo (<code>null</code>), ser o mesmo que chamar 
   * {@link #hideStatusBar()}.
   *  
   * @param icon
   * @param message
   */
  public void setStatus(final ImageIcon icon, final String message) {
    this.setStatus(true, icon, message);
  }
  
  /**
   * Escreve uma mensagem na barra de status se a condio dada for verdadeira.
   * Obs. caso a barra de status estivesse escondida, ela ir aparecer 
   * para apresentar a mensagem.<br/>
   * Obs. se a mensagem for vazia (<code>""</code>) ou nula (<code>null</code>)
   * e se o cone for nulo (<code>null</code>), ser o mesmo que chamar 
   * {@link #hideStatusBar()}.
   *  
   * @param condition 
   * @param message
   */
  public void setStatus(final boolean condition, final String message) {
    this.setStatus(condition, null, message);
  }  
  
  /**
   * Escreve uma mensagem na barra de status.
   * Obs. caso a barra de status estivesse escondida, ela ir aparecer 
   * para apresentar a mensagem.<br/>
   * Obs. se a mensagem for vazia (<code>""</code>) ou nula (<code>null</code>)
   * e se o cone for nulo (<code>null</code>), ser o mesmo que chamar 
   * {@link #hideStatusBar()}.
   *  
   * @param message
   */
  public void setStatus(final String message) {
    this.setStatus(true, message);
  }  

  /**
   * Escreve uma mensagem na barra de status precedida por um 
   * cone de informao se a condio dada for verdadeira.
   * Obs. caso a barra de status estivesse escondida, ela ir aparecer 
   * para apresentar a mensagem.<br/>
   *  
   * @param condition 
   * @param message
   */
  public void setInfo(final boolean condition, final String message) {
    setStatus(ApplicationImages.ICON_INFORMATION_16, message);
  }   
  
  /**
   * Escreve uma mensagem na barra de status precedida por um 
   * cone de informao.
   * Obs. caso a barra de status estivesse escondida, ela ir aparecer 
   * para apresentar a mensagem.<br/>
   *  
   * @param message
   */
  public void setInfo(final String message) {
    setInfo(true, message);
  }  

  /**
   * Escreve uma mensagem na barra de status precedida por 
   * um cone de aviso se a condio dada for verdadeira.
   * Obs. caso a barra de status estivesse escondida, ela ir aparecer 
   * para apresentar a mensagem.<br/>
   *  
   * @param condition   
   * @param message
   */  
  public void setWarning(final boolean condition, final String message) {
    setStatus(condition, ApplicationImages.ICON_WARNING_16, message);
  }
  
  /**
   * Escreve uma mensagem na barra de status precedida por um cone de aviso.
   * Obs. caso a barra de status estivesse escondida, ela ir aparecer 
   * para apresentar a mensagem.<br/>
   *  
   * @param message
   */  
  public void setWarning(final String message) {
    setWarning(true, message);
  }  
  
  /**
   * Escreve uma mensagem na barra de status precedida por 
   * um cone de erro se a condio dada for verdadeira.
   * Obs. caso a barra de status estivesse escondida, ela ir aparecer 
   * para apresentar a mensagem.<br/>
   *  
   * @param condition 
   * @param message
   */
  public void setError(final boolean condition, final String message) {
    setStatus(condition, ApplicationImages.ICON_ERROR_16, message);
  }  
  
  /**
   * Escreve uma mensagem na barra de status precedida por um cone de erro.
   * Obs. caso a barra de status estivesse escondida, ela ir aparecer 
   * para apresentar a mensagem.<br/>
   *  
   * @param message
   */
  public void setError(final String message) {
    setError(true, message);
  }    
}
