/*
 * $Author: taciana $
 * $Date: 2008-06-06 16:24:00 -0300 (Fri, 06 Jun 2008) $
 * $Release:$
 */
package csbase.client.util.sga;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import tecgraf.javautils.LNG;
import csbase.client.remote.srvproxies.SGAProxy;
import csbase.logic.SGASet;

/**
 * Esta classe representa um cache de SGAs. Ela ser descontinuada depois
 * que o SGAProxy estiver com cache. Ela tenta imitar o SGAProxy.
 */
public class SGACache {
  private Map<String, SGASet> sgaSetMap;
  private boolean error;

  /**
   * Mantm uma cache de SGAs.
   */
  public SGACache() {
    this.sgaSetMap = new HashMap<String, SGASet>();
    Vector<String> sgaNameVector = SGAProxy.getAllSGANames();
    if (sgaNameVector == null) {
      System.err.println(
        LNG.get("csbase.client.facilities.sga.exception.load_sgas"));
      this.error = true;
      return;
    }
    Iterator<String> sgaNameIterator = sgaNameVector.iterator();
    while (sgaNameIterator.hasNext()) {
      String sgaName = sgaNameIterator.next();
      SGASet sgaSet = SGAProxy.getSGASet(sgaName);
      this.sgaSetMap.put(sgaName, sgaSet);
    }
  }

  /**
   * Obtm todos os nomes dos SGAs contidos no servidor.
   * 
   * @return .
   */
  public Vector<String> getAllSGANames() {
    if (this.error) {
      return null;
    }
    return new Vector<String>(this.sgaSetMap.keySet());
  }

  /**
   * Obtm a representao lgica de um SGA.
   * @param sgaName O nome do SGA do qual se deseja obter a representao.
   * 
   * @return a representao lgica de um SGA.
   */
  public SGASet getSGASet(String sgaName) {
    if (this.error) {
      return null;
    }
    return this.sgaSetMap.get(sgaName);
  }

  /**
   * Encontra um SGA que satisfaa a determinado critria e que seja de um 
   * determinado conjunto de plataformas.
   * 
   * @param platforms conjunto de plataformas do qual o SGA deve fazer parte.
   * @param criteria .
   * 
   * @return o nome do SGA selecionado.
   */
  public String chooseSGA(final Vector<String> platforms, final int criteria) {
    String selectedSGA = null;
    double minCPULoad = Double.MAX_VALUE;
    Iterator<SGASet> sgaIterator = this.sgaSetMap.values().iterator();
    while (sgaIterator.hasNext()) {
      SGASet sgaSet = sgaIterator.next();
      String sgaName = sgaSet.getName();
      if (
        sgaSet.isPlatformSupported(platforms) && sgaSet.mayExecuteCommand()) {
        if (sgaSet.getCPULoad1() < minCPULoad) {
          minCPULoad = sgaSet.getCPULoad1();
          selectedSGA = sgaName;
        }
/* XXX
        switch (criteria) {
          case SGAServiceInterface.SGA_CHOOSE_DONTCARE:
            return sgaName;
          case SGAServiceInterface.SGA_CHOOSE_CPULOAD:
            if (sgaSet.getCPULoad1() < minCPULoad) {
              minCPULoad = sgaSet.getCPULoad1();
              selectedSGA = sgaName;
            }
            break;
          default:
            throw new IllegalStateException(
              "Critrio de seleo de SGA invlido: "+criteria);
        }
*/
      }
    }
    return selectedSGA;
  }
}
