package csbase.client.util.table;

import java.awt.Component;

import javax.swing.JProgressBar;
import javax.swing.JTable;
import javax.swing.table.TableCellRenderer;

/**
 * Mostra barra de progresso. O valor esperado  entre 0 e 100. O tipo do valor
 * esperado pode ser do tipo 'Number', que inclui 'Double', 'Integer', etc, ou
 * pode tambm ser uma combinao de String + Number, vinda como um Object[]. O
 * string especificado ser ento mostrado na barra de progresso, ao invs da
 * porcentagem. Se o valor for Null ou Double.NaN, apenas o texto  mostrado, se
 * existir um.
 */
public class ProgressCellRenderer extends JProgressBar implements
  TableCellRenderer {

  /**
   * Construtor.
   */
  public ProgressCellRenderer() {
    setStringPainted(true);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Component getTableCellRendererComponent(JTable table, Object value,
    boolean isSelected, boolean hasFocus, int row, int column) {

    if (value == null) {
      setValue(null);
      setString("--");
    }
    else if (value instanceof Object[]) {
      // temos tanto o texto quanto o valor ... usar ambos:
      Object[] valueData = (Object[]) value;

      // definir texto da barra:
      setString((String) valueData[0]);

      // definir valor do progresso:
      setValue((Number) valueData[1]);
    }
    else {
      // temos apenas o valor ... criar um texto pelo valor ...
      // definir valor do progresso:
      Number number = (Number) value;
      setValue(number);

      // definir texto:
      double progressValue = number.doubleValue();
      String text = String.format("%.0f%%", progressValue);
      setString(text);
    }

    setEnabled(table.isEnabled());
    return this;
  }

  /**
   * @param number - novo valor para o progresso. Null ou Double.NaN no alteram
   *        o valor.
   */
  private void setValue(Number number) {
    if (number != null && !number.equals(Double.NaN))
      super.setValue(number.intValue());
  }
}
