# $Id: csbase.sh 106565 2010-06-07 17:24:30Z csgrid $
#
# Este arquivo define variveis de ambiente correspondentes s libs
# disponibilizadas pelo CSBase para os outros projetos, para facilitar
# e centralizar as referncias a estas. Upgrades de verses de
# bibliotecas sero atualizados aqui, e todos os scripts que fizerem
# uso deste arquivo automaticamente utilizaro as novas verses.
# 
# O nico pr-requisito  que a varivel de ambiente CSBASE_DIR seja
# definida para apontar para o diretrio-raiz do CSBase. Como este local
# varia de projeto para projeto, esta definio deve ser feita nos
# scripts _antes_ da incluso deste script.
#
# O uso tpico deste arquivo :
# ...
# CSBASE_DIR=[path para o CSBase]
# . $CSBASE_DIR/conf/csbase.sh
# ...

# Funo para trocar o separador ":" por espaos. Funciona apenas para
# as variveis UNIX.
#
# Exemplos de uso:
# FILES=`withSpaces $JAR_JACORB`
# cp `withSpaces $JAR_JACORB` /tmp
withSpaces () {
    echo "$@" | tr : \ 
}

# a varivel CSBASE_LIBS deve ter sido previamente definida
export CSBASE_LIBS=$CSBASE_DIR/libs
export CSBASE_LIBS_WIN=`echo $CSBASE_LIBS | tr / \\\\\\\ `

# definies das variveis para cada JAR

# Acme
export JAR_ACME=$CSBASE_LIBS/acme/Acme.jar
export JAR_ACME_WIN=$CSBASE_LIBS_WIN\\acme\\Acme.jar

# Ant
export JAR_ANT=$CSBASE_LIBS/ant/ant-launcher.jar
export JAR_ANT_WIN=$CSBASE_LIBS_WIN\\ant\\ant-launcher.jar

# args4j
export JAR_ARGS4J=$CSBASE_LIBS/args4j/args4j-2.0.12.jar
export JAR_ARGS4J_WIN=$CSBASE_LIBS_WIN\\ARGS4J\\args4j-2.0.12.jar

# Cos
export JAR_COS=$CSBASE_LIBS/cos/cos.jar
export JAR_COS_WIN=$CSBASE_LIBS_WIN\\cos\\cos.jar

# FTC
export JAR_FTC=$CSBASE_LIBS/ftc/ftc-1.0.1.jar
export JAR_FTC_WIN=$CSBASE_LIBS_WIN\\ftc\\ftc-1.0.1.jar

# Jacorb + dependncias
JACORB_DIR=$CSBASE_LIBS/jacorb
JACORB_DIR_WIN=$CSBASE_LIBS_WIN\\jacorb
export JAR_JACORB=$JACORB_DIR:$JACORB_DIR/antlr-2.7.2.jar:$JACORB_DIR/avalon-framework-4.1.5.jar:$JACORB_DIR/concurrent-1.3.2.jar:$JACORB_DIR/idl.jar:$JACORB_DIR/jacorb-2.3.0.jar:$JACORB_DIR/logkit-1.2.jar
export JAR_JACORB_WIN="$JACORB_DIR_WIN;$JACORB_DIR_WIN\\antlr-2.7.2.jar;$JACORB_DIR_WIN\\avalon-framework-4.1.5.jar;$JACORB_DIR_WIN\\concurrent-1.3.2.jar;$JACORB_DIR_WIN\\idl.jar;$JACORB_DIR_WIN\\jacorb-2.3.0.jar;$JACORB_DIR_WIN\\logkit-1.2.jar"

# Jaf
export JAR_JAF=$CSBASE_LIBS/jaf/activation.jar
export JAR_JAF_WIN=$CSBASE_LIBS_WIN\\jaf\\activation.jar

# Javamail
export JAR_JAVAMAIL=$CSBASE_LIBS/javamail/mail.jar
export JAR_JAVAMAIL_WIN=$CSBASE_LIBS_WIN\\javamail\\mail.jar

# Javautils
export JAR_JAVAUTILS=$CSBASE_LIBS/javautils/javautils.jar
export JAR_JAVAUTILS_WIN=$CSBASE_LIBS_WIN\\javautils\\javautils.jar

# Java WebStart
export JAR_JAVAWEBSTART=$CSBASE_LIBS/javawebstart/javaws.jar
export JAR_JAVAWEBSTART_WIN=$CSBASE_LIBS_WIN\\javawebstart\\javaws.jar

# JFreechart + dependncias
export VERSION_JFREECHART=jfreechart-1.0.11.jar
export VERSION_JCOMMON=jcommon-1.0.14.jar
export JAR_JFREECHART=$CSBASE_LIBS/jfreechart/$VERSION_JFREECHART:$CSBASE_LIBS/jfreechart/$VERSION_JCOMMON
export JAR_JFREECHART_WIN="$CSBASE_LIBS_WIN\\jfreechart\\$VERSION_JFREECHART;$CSBASE_LIBS\\jfreechart\\$VERSION_JCOMMON"

# JUnit
export JAR_JUNIT=$CSBASE_LIBS/junit/junit-4.6.jar
export JAR_JUNIT_WIN=$CSBASE_LIBS_WIN\\junit\\junit-4.6.jar

# LDAP
export JAR_LDAP=$CSBASE_LIBS/ldap/ldap.jar
export JAR_LDAP_WIN=$CSBASE_LIBS_WIN\\ldap\\ldap.jar

# Numal
export JAR_NUMAL=$CSBASE_LIBS/numal/numal.jar
export JAR_NUMAL_WIN=$CSBASE_LIBS_WIN\\numal\\numal.jar

# Ptolemy
export JAR_PTOLEMY=$CSBASE_LIBS/ptolemy/plotapplication.jar
export JAR_PTOLEMY_WIN=$CSBASE_LIBS_WIN\\ptolemy\\plotapplication.jar

# Servlet
export JAR_SERVLET=$CSBASE_LIBS/servlet/servlet.jar
export JAR_SERVLET_WIN=$CSBASE_LIBS_WIN\\servlet\\servlet.jar

# SGAIDL
export JAR_SGAIDL=$CSBASE_LIBS/sgaidl/sgaidl.jar
export JAR_SGAIDL_WIN=$CSBASE_LIBS_WIN\\sgaidl\\sgaidl.jar

# WIOIDL
export JAR_WIOIDL=$CSBASE_LIBS/wioidl/wioidl.jar
export JAR_WIOIDL_WIN=$CSBASE_LIBS_WIN\\wioidl\\wioidl.jar

# CSFS
export JAR_CSFS=$CSBASE_LIBS/csfs/csfs.jar
export JAR_CSFS_WIN=$CSBASE_LIBS_WIN\\csfs\\csfs.jar

# VIX
export JAR_VIX=$CSBASE_LIBS/vix/vix.jar
export JAR_VIX_WIN=$CSBASE_LIBS_WIN\\vix\\vix.jar

# MiGLayout
export JAR_MIGLAYOUT=$CSBASE_LIBS/miglayout/miglayout-3.7.2-swing.jar
export JAR_MIGLAYOUT_WIN=$CSBASE_LIBS_WIN\\miglayout\\miglayout-3.7.2-swing.jar

# OpenBus
export JAR_OPENBUS=$CSBASE_LIBS/openbus/openbus-api-1.4.5.jar:$CSBASE_LIBS/openbus/openbus-idl-jacorb-1.4.5.jar:$CSBASE_LIBS/openbus/openbus-data_service-api-1.0.1.jar:$CSBASE_LIBS/openbus/openbus-data_service-idl-jacorb-1.0.1.jar:$CSBASE_LIBS/openbus/openbus-data_service-valuetype-1.0.1.jar:$CSBASE_LIBS/openbus/openbus-data_service-project-idl-jacorb-1.0.1.jar:$CSBASE_LIBS/openbus/openbus-data_service-project-valuetype-1.0.1.jar
export JAR_OPENBUS_WIN="$CSBASE_LIBS\\openbus\\openbus-api-1.4.5.jar:$CSBASE_LIBS\\openbus\\openbus-idl-jacorb-1.4.5.jar:$CSBASE_LIBS\\openbus\\openbus-data_service-api-1.0.1.jar:$CSBASE_LIBS\\openbus\\openbus-data_service-idl-jacorb-1.0.1.jar:$CSBASE_LIBS\\openbus\\openbus-data_service-valuetype-1.0.1.jar:$CSBASE_LIBS\\openbus\\openbus-data_service-project-idl-jacorb-1.0.1.jar:$CSBASE_LIBS\\openbus\\openbus-data_service-project-valuetype-1.0.1.jar"

# SCS
export JAR_SCS=$CSBASE_LIBS/scs/scs-core-1.0.1.jar:$CSBASE_LIBS/scs/scs-idl-jacorb-1.0.1.jar
export JAR_SCS_WIN="$CSBASE_LIBS\\scs\\scs-core-1.0.1.jar;$CSBASE_LIBS\\scs\\scs-idl-jacorb-1.0.1.jar"

# DRMAA (OpenDreams)
export JAR_DRMAA=$CSBASE_LIBS/drmaa/drmaa.jar
export JAR_DRMAA_WIN=$CSBASE_LIBS_WIN\\drmaa\\drmaa.jar


