package csbase.logic;

/**
 * Permisso para indicar se um usurio, em um determinado sistema, pode
 * executar um ou mais algoritmos instalados no sistema atual.
 * 
 * Essa permisso possui dois tipos de atributos:
 * 
 * - sistema=<id_do_sistema> : representa a identificao (string nica) do
 * sistema em que o usurio ter permisso para executa os algoritmos
 * especificados na permisso. Para representar o sistema atual, voc deve criar
 * um atributo "sistema=." Um exemplo de uso desse atributo com o valor
 * diferente de "." seria no caso de uso do OpenDreams para execuo de comandos
 * por fora do sistema atual. Nesse caso, o mesmo valor usado pelo Opendreams
 * para representar o sistema deve ser passado para o administrador do sistema
 * atual, para ser inserido na permisso.
 * 
 * - nome=<nome do algoritmo> : representa o nome do algoritmo que o usuario
 * pode executar ou uma expresso regular. Por exemplo, para permitir executar
 * todos os algoritmos, basta criar um atributo "nome=.*".
 * 
 */
public class AlgorithmExecutionPermission extends AttributesPermission {
  /** Atributo que define o nome do algoritmo */
  public static final String ALGORITHM_NAME = "nome=";

  /**
   * Atributo que define o identificador (string) nico do sistema em que essa
   * permisso vai ser vlida.
   */
  public static final String SYSTEM_ID = "sistema=";

  /**
   * Constri uma permisso para execuo de algoritmos.
   */
  public AlgorithmExecutionPermission() {
    super();
  }

  /**
   * Constri uma permisso para execuo de algoritmos.
   * 
   * @param name nome da permisso
   * @param description descrio
   * @param attributes atributos da permisso
   */
  public AlgorithmExecutionPermission(String name, String description,
    String[] attributes) {
    super(name, description, attributes);
  }
}