/*
 * $Id: ClientProjectFileOutputStream.java 102488 2010-03-03 22:26:31Z clinio $
 */

package csbase.logic;

import java.io.IOException;
import java.io.OutputStream;
import java.nio.ByteBuffer;
import java.rmi.RemoteException;

/**
 * Implementa um escritor (stream) de um arquivo (client-side).
 * 
 * @author Tecgraf/PUC-Rio
 */
public class ClientProjectFileOutputStream extends OutputStream {
  /**
   * Arquivo a ser lido
   */
  private final ClientProjectFile file;

  /**
   * Posio corrente no arquivo
   */
  private long currentFilePosition = 0;

  /**
   * {@inheritDoc}
   */
  @Override
  public void close() throws IOException {
    truncate();
    file.close();
  }

  /**
   * {@inheritDoc}
   * 
   * No faz nada.
   */
  @Override
  public void flush() throws IOException {
    // vazio
  }

  /**
   * Consulta o objeto file associado.
   * 
   * @return o {@link ClientProjectFile} associado a este stream
   */
  public ClientProjectFile getClientProjectFile() {
    return file;
  }

  /**
   * Trunca o arquivo de acordo o seu tamanho, e reinicia o controle interno da
   * posio corrente do arquivo.
   * 
   * @throws IOException em caso de erro de I/O.
   */
  private void truncate() throws IOException {
    file.truncate(currentFilePosition);
    currentFilePosition = 0;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void write(final byte[] b) throws IOException {
    try {
      final ByteBuffer auxBuffer = ByteBuffer.wrap(b);
      file.write(auxBuffer, currentFilePosition);
      currentFilePosition += b.length;
    }
    catch (final IOException e) {
      currentFilePosition = 0;
      throw e;
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void write(final byte[] b, final int off, final int len)
    throws IOException {
    try {
      final ByteBuffer auxBuffer = ByteBuffer.wrap(b, off, len);
      file.write(auxBuffer, currentFilePosition);
      currentFilePosition += len;
    }
    catch (final IOException e) {
      currentFilePosition = 0;
      throw e;
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void write(final int b) throws IOException {
    try {
      final byte[] array = new byte[1];
      array[0] = (byte) b;
      final ByteBuffer auxBuffer = ByteBuffer.wrap(array);
      file.write(auxBuffer, currentFilePosition);
      currentFilePosition++;
    }
    catch (final IOException e) {
      currentFilePosition = 0;
      throw e;
    }
  }

  /**
   * Cria um output stream a partir de um arquivo da rea de projetos, abrido-o
   * para escrita.
   * 
   * @param file Arquivo da rea de projetos (um ClientProjectFile).
   * @throws RemoteException falha de rmi
   */
  public ClientProjectFileOutputStream(final ClientProjectFile file)
    throws RemoteException {
    if (file == null) {
      throw new IllegalArgumentException("O parmetro file est nulo.");
    }
    this.file = file;
    file.openFileChannel(false);
  }
}
