/*
 * CommandEndNotificationData.java
 * 
 * $Author: pasti $ $Date: 2010-03-19 15:49:20 -0300 (Fri, 19 Mar 2010) $
 * $Revision: 103286 $
 */

package csbase.logic;

/**
 * Classe que representa os dados de mensagem de fim de comando em um SGA,
 * quando o comando no d informaes sobre como a execuo terminou (com
 * sucesso ou com erro).
 */
public class CommandEndNotificationData extends ExtendedCommandFinishedNotificationData {

  /**
   * Indica se os tempos de execuo dos comandos esto definidos na notificao
   */
  private boolean timeInfo;

  /**
   * Retorna verdadeiro se possui os tempos de execuo dos comandos.
   * 
   * @return verdadeiro se possui os tempos de execuo dos comandos.
   */
  public boolean hasTimeInfo() {
    return timeInfo;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected String getStatusString() {
    return "END";
  }

  /**
   * Constri o evento de trmino normal de comando.
   * 
   * @param commandId identificador do comando.
   * @param cmdDesc descrio do comando.
   * @param tip Dica para auxiliar a identificao do algoritmo do comando.
   * @param elapsedTimeSec tempo de parede.
   * @param userTimeSec tempo em modo usurio.
   * @param cpuTimeSec tempo em modo sistema.
   * @param startTime Hora de incio
   * @param endTime Hora de trmino.
   * @param execHost Servidor de execuo.
   * @param projectId identificador do projeto
   */
  public CommandEndNotificationData(Object commandId, String cmdDesc,
    String tip, Integer elapsedTimeSec, Integer userTimeSec, Integer cpuTimeSec,
    long startTime, long endTime, String execHost, Object projectId) {
    super(commandId, cmdDesc, tip, elapsedTimeSec, userTimeSec, cpuTimeSec,
      startTime, endTime, execHost, projectId);
    
    this.timeInfo =
      (elapsedTimeSec != null && elapsedTimeSec >= 0)
        || (userTimeSec != null && userTimeSec >= 0)
        || (cpuTimeSec != null && cpuTimeSec >= 0);
  }

  /**
   * Constri o evento de trmino de comando sem dados de tempo.
   * 
   * @param commandId identificador do comando.
   * @param cmdDesc descrio do comando.
   * @param tip Dica para auxiliar a identificao do algoritmo do comando.
   * @param startTime Hora de incio
   * @param endTime Hora de trmino.
   * @param execHost Servidor de execuo.
   * @param projectId identificador do projeto
   */
  public CommandEndNotificationData(Object commandId, String cmdDesc,
    String tip, long startTime, long endTime, String execHost, Object projectId) {
    super(commandId, cmdDesc, tip, null, null, null, startTime, endTime, execHost,
      projectId);
    this.timeInfo = false;
  }

}
