package csbase.logic;

/**
 * Modela um projeto.
 */
public interface CommonProject {
  /**
   * Fornece o identificador do usurio ao qual o projeto pertence.
   * 
   * @return Identificador do usurio.
   */
  Object getUserId();

  /**
   * Fornece o nome do projeto.
   * 
   * @return Nome do projeto.
   */
  String getName();

  /**
   * Fornece a dscrio do projeto.
   * 
   * @return A descrio do projeto.
   */
  String getDescription();

  /**
   * Altera a descrio do projeto. Essa modificao s ser persistida quando o
   * mtodo {@link #modify()} for chamado.
   * 
   * @param description nova descrio.
   */
  void setDescription(String description);

  /**
   * Retorna o arquivo associado  raiz do projeto.
   * <p>
   * IMPORTANTE: no se deve assumir que a rvore est preenchida, s 
   * garantido que o n-raiz esteja definido
   * 
   * @return arquivo associado  raiz do projeto. Seus filhos podem no ter sido
   *         ainda definidos
   */
  ProjectFile getTree();

  /**
   * Persiste as modificaes feitas nas informaes do projeto. Caso alguma
   * informao no possa ser alterada, seu valor original ser reestabelecido.
   * 
   * @throws Exception
   */
  void modify() throws Exception;

  /**
   * Fecha um projeto. Aps chamado este mtodo, o objeto que representa o
   * projeto no poder mais ser utilizado.
   * 
   * @param notify indicativo de notificao aos clientes.
   * @throws Exception
   */
  void close(boolean notify) throws Exception;
}
