/*
 * NewProjectFileEvent.java
 * 
 * $author$ $Revision: 82436 $ - $Date: 2008-07-08 14:48:53 -0300 (Tue, 08 Jul
 * 2008) $
 */
package csbase.logic;

/**
 * Representa o evento de incluso de arquivo na rvore do projeto. Objetos
 * dessa classe so criados pelo servidor.
 */
public class NewProjectFileEvent extends ProjectEvent {
  
  /** Caminho do diretrio aonde o arquivo foi criados. */
  private String[] path;

  /** Arquivo criado na rvore do projeto. */
  private ProjectFile file;

  /**
   * Obtm um {@link ProjectFile} representando o caminho de cada arquivo
   * criado.<br>
   * Obs.: este caminho  relativo ao diretrio aonde os projetos so
   * armazenados. Em outras palavras, o ancestral mais antigo ser o diretrio
   * do projeto aonde o arquivo foi criado.
   * 
   * @return um {@link ProjectFile} representando o caminho de cada arquivo
   *         criado.
   */
  public ProjectFile getFile() {
    return file;
  }

  /**
   * Obtm o caminho do diretrio no qual foi criado um arquivo como resultado
   * do evento ocorrido.
   * 
   * @return O caminho do diretrio onde o arquivo foi criado.
   */
  public String[] getPath() {
    return path;
  }

  /**
   * Cria uma instncia de <code>ProjectEvent</code> para o evento de incluso
   * de um arquivo na rvore de um projeto.
   * 
   * @param path o caminho do diretrio aonde o arquivo foi criado.<br>
   *        Obs.: este caminho deve ser relativo ao diretrio aonde os projetos
   *        so armazenados. Em outras palavras, o ancestral mais antigo dever
   *        ser o diretrio do projeto aonde o arquivo foi criado.
   * @param file um {@link ProjectFile} representando o arquivo criado. <br>
   *        O mtodo {@link ProjectFile#getParent()} ir retornar null, pois por
   *        motivos de consistncia, este valor no foi atribuido no servidor.
   */
  NewProjectFileEvent(String[] path, ProjectFile file) {
    event = NEW_FILE;
    this.file = file;
    this.path = path;
  }
}
