/*
 * ProjectFileRenamedEvent.java
 *
 * $author$
 * $Revision: 55308 $  - $Date: 2006-08-23 18:11:09 -0300 (Wed, 23 Aug 2006) $
 */
package csbase.logic;


/**
 * Representa o evento de renomear um arquivo na rvore do projeto.
 */
public class ProjectFileRenamedEvent extends ProjectEvent {
  private String[] path;
  private String newName;
  private String newType;
  private ProjectFile file;

  /**
   * Retorna o caminho do arquivo com o nome antigo.
   *
   * @return O caminho do arquivo com o nome antigo.
   */
  public String[] getPath() {
    return path;
  }

  /**
   * Retorna o novo nome do arquivo como resultado do evento ocorrido.
   *
   * @return O novo nome do arquivo
   */
  public String getNewName() {
    return newName;
  }

  /**
   * Retorna o novo tipo do arquivo como resultado do evento ocorrido.
   *
   * @return O novo tipo do arquivo
   */
  public String getNewType() {
    return newType;
  }

  /**
   * Atribui o arquivo cujo nome foi alterado.
   *
   * @param file O arquivo cujo nome foi alterado.
   */
  public void setFile(ProjectFile file) {
    this.file = file;
  }

  /**
   * Retorna o arquivo cujo nome foi alterado.
   *
   * @return O arquivo cujo nome foi alterado.
   */
  public ProjectFile getFile() {
    return file;
  }

  /**
   * Cria uma instncia de <code>ProjectEvent</code> para o evento de renomear
   * um arquivo na rvore de um projeto.
   *
   * @param path O caminho do arquivo com o nome antigo
   * @param newName O novo nome do arquivo
   * @param newType O novo tipo do arquivo
   */
  public ProjectFileRenamedEvent(String[] path, String newName, String newType) {
    event = NEW_FILE_NAME;
    this.path = path;
    this.newName = newName;
    this.newType = newType;
  }
}
