/*
 * ProjectFileStateChangedEvent.java
 *
 * $Author: taciana $
 * $Revision: 76954 $  - $Date: 2008-06-06 16:24:00 -0300 (Fri, 06 Jun 2008) $
 */
package csbase.logic;


/**
 * Representa o evento de mudana de estado de um arquivo na rvore do projeto.
 * So os seguintes os estados de um arquivo:
 *
 * <ul>
 * <li>
 * global
 * </li>
 * <li>
 * publicado
 * </li>
 * <li>
 * em construo
 * </li>
 * </ul>
 */
public class ProjectFileStateChangedEvent extends ProjectEvent {
  private String[] path;
  private String name;
  private boolean[] state;
  private ProjectFile file;
  private Object whoPublished;
  private Object projectId;
  private long modificationDate;
  private long size;
  /** Indica se o arquivo est bloqueado */
  private boolean isLocked;

  /**
   * Retorna o caminho do arquivo cujo estado foi alterado.
   *
   * @return O caminho do arquivo.
   */
  public String[] getPath() {
    return path;
  }

  /**
   * Retorna o nome do arquivo cujo estado foi alterado.
   *
   * @return O caminho do arquivo.
   */
  public String getName() {
    return name;
  }

  /**
   * Retorna o novo estado do arquivo.
   *
   * @return O novo estado.
   */
  public boolean[] getNewState() {
    return state;
  }

  /**
   * Retorna o tamanho atualizado do arquivo.
   *
   * @return O tamanho atualizado do arquivo.
   */
  public long getSize() {
    return size;
  }

  /**
   * Retorna a data de modificao atualizada do arquivo.
   *
   * @return a data de modificao atualizada do arquivo.
   */
  public long getModificationDate() {
    return modificationDate;
  }

  /**
   * Retorna quem publicou o arquivo.
   *
   * @return O identificador do usurio que publicou o arquivo.
   */
  public Object whoPublished() {
    return whoPublished;
  }

  /**
   * Retorna o projeto que publicou o arquivo.
   *
   * @return O identificador do projeto a partir do qual o arquivo foi
   *         publicado.
   */
  public Object whichProjectPublished() {
    return projectId;
  }

  /**
   * Indica se o arquivo est bloqueado
   * @return true se o arquivo estiver bloqueado
   */
  public boolean isLocked(){
    return isLocked;
  }

  /**
   * Atribui o arquivo cujo estado foi alterado.
   *
   * @param file O arquivo cujo estado foi alterado.
   */
  public void setFile(ProjectFile file) {
    this.file = file;
  }

  /**
   * Retorna o arquivo cujo estado foi alterado.
   *
   * @return O arquivo cujo estado fo alterado.
   */
  public ProjectFile getFile() {
    return file;
  }

  /**
   * Cria uma instncia de <code>ProjectEvent</code> para o evento de mudana
   * de estado de um arquivo na rvore de um projeto.
   *
   * @param path O caminho do arquivo cujo estado foi alterado.
   * @param name nome do arquivo
   * @param state Um array de boolean com os novos estados do arquivo.
   * @param whoPublished identificador do projeto que publicou o arquivo
   * @param projectId identificador do projeto
   * @param modificationDate da de modificao do arquivo
   * @param size tamanho do arquivo
   * @param isLocked indica se o arquivo est bloqueado
   */
  public ProjectFileStateChangedEvent(
    String[] path, String name, boolean[] state, Object whoPublished,
    Object projectId, long modificationDate, long size, boolean isLocked) {
    event = FILE_STATE_CHANGED;
    this.path = path;
    this.name = name;
    this.state = state;
    this.whoPublished = whoPublished;
    this.projectId = projectId;
    this.modificationDate = modificationDate;
    this.size = size;
    this.isLocked = isLocked;
  }
}
