/*
 * SGANotificationData.java
 * $Author: lbarros $
 * $Revision: 62029 $ - $Date: 2007-03-28 15:58:39 -0300 (Wed, 28 Mar 2007) $
 */
package csbase.logic;

/**
 * Classe que modela uma notificao de start-up/shutdownr/init do SGA
 * @author Andr Luiz Clinio
 */
public class SGANotificationData extends NotificationData {

  /** Flag de start-up */
  public static final int SGA_START_UP = 0;

  /** Flag de shutdown */
  public static final int SGA_SHUTDOWN = 1;

  /** Flag de reincio */
  public static final int SGA_REINIT = 2;
  
  /** Strings de auditagem dos flags de notificao */
  private final String[] strFlag = { 
    "START-UP", "SHUTDOWN", "REINIT"  
  };

  /** Atributo de SGA novo detectado */
  private SGASet info;

  /** Atributo de estado */
  private int flag;

  /**
   * Definio da volatilidade da notificao.
   * @return flag indicativo.
   */
  public boolean isVolatile() {
    return true;
  }

  /**
   * Consulta ao flag de evento do SGA
   *
   * @return o cdigo do evento.
   */
  public int getFlag() {
    return flag;
  }

  /**
   * Consulta ao nome do SGA
   *
   * @return uma string com o nome.
   */
  public SGASet getMainInfo() {
    return info;
  }

  /**
   * String para auditagem.
   *
   * @return uma string
   */
  public String toString() {
    return "SGA/NOTIFICATION (cod." + flag + " / " + strFlag[flag] + ") " +
    " - " + getMainInfo().getName() + " (" + getMainInfo().getPlatformId() +
    ")";
  }

  /**
   * Constri o evento de notificao de SGA.
   *
   * @param flag estado on/off.
   * @param info .
   */
  public SGANotificationData(int flag, SGASet info) {
    this.flag = flag;
    this.info = info;
  }
}
