/*
 * $Id: UsersNotificationData.java 91193 2009-05-06 17:24:04Z costa $
 */
package csbase.logic;

import java.util.Collection;

/**
 * Classe que modela uma notificao de publicao.
 */
public class UsersNotificationData extends NotificationData {

  /** Ocorrncia de um insert */
  public static final int INSERT = 0;

  /** Ocorrncia de um remove */
  public static final int REMOVE = 1;

  /** Ocorrncia de um remove */
  public static final int REMOVE_PROJECT = 2;

  /** Strings de auditagem da notificao de gerencimento de usurios */
  private final String[] strType = { "INSERT", "REMOVE", "REMOVE_PROJECT" };

  /** Informao sobre o gerenciamento: administrador do projeto */
  private Object ownerID;

  /** Informao sobre o gerenciamento: quem est sendo inserido/removido */
  private Collection<Object> usersToNotify;

  /** * Informao sobre o gerenciamento: - qual projeto */
  private String projectName;

  /** Atributo que indica a ao de publicao ou despublicao. */
  private int type;

  /**
   * Definio da volatilidade da notificao.
   * 
   * @return flag indicativo.
   */
  @Override
  public boolean isVolatile() {
    return false;
  }

  /**
   * Consulta o tipo de evento de gerenciamento de usurios.
   * 
   * @return o cdigo do evento, que pode ser INSERT ou REMOVE.
   */
  public int getType() {
    return type;
  }

  /**
   * Consulta o usurio que inseriu ou removeu o usurio do projeto.
   * 
   * @return o identificador do usurio que originou o evento.
   */
  public Object getUserId() {
    return ownerID;
  }

  /**
   * Consulta o nome do projeto ao qual o usurio foi inserido ou removido.
   * 
   * @return o nome do projeto.
   */
  public String getProjectName() {
    return projectName;
  }

  /**
   * String para auditagem.
   * 
   * @return uma string
   */
  @Override
  public String toString() {
    return String.format("PRJ_PERMS (%s) From: %s Project: %s To: %s",
      strType[type], ownerID, projectName, usersToNotify.toString());
  }

  /**
   * Cria um evento para notificao de compartilhamento de projeto. Notifica um
   * conjunto de usurios avisandoq que estes foram inseridos ou removidos do
   * compartilhamento de um determinado projeto.
   * 
   * @param type - tipo do evento (insero ou remoo)
   * @param usersToNotify - usurios que sero notificados
   * @param ownerID - ID do dono do projeto que deu origem  notificao
   * @param projectName - nome do projeto que deu origem  notificao
   */
  public UsersNotificationData(int type, Collection<Object> usersToNotify,
    Object ownerID, String projectName) {
    this.type = type;
    this.usersToNotify = usersToNotify;
    this.ownerID = ownerID;
    this.projectName = projectName;
  }
}
