/*
 * $Id: AlgorithmCommand.java 101015 2010-01-25 19:32:39Z clinio $
 */

package csbase.logic.algorithms.commands;

import java.io.Serializable;

import csbase.logic.algorithms.AlgorithmConfigurator;

/**
 * Um comando de algortmo  criado quando h uma requisio de execuo de um
 * algoritmo.
 */
public class AlgorithmCommand implements Serializable {

  /**
   * As propriedades do comando.
   */
  private CommandProperties commandProperties;
  /**
   * O configurador de algoritmos.
   */
  private AlgorithmConfigurator algorithmConfigurator;

  /**
   * Obtm o configurador de algoritmo.
   * 
   * @return O configurador de algoritmo.
   */
  public AlgorithmConfigurator getAlgorithmConfigurator() {
    return algorithmConfigurator;
  }

  /**
   * Obtm as propriedades do comando.
   * 
   * @return As proprieades do comando.
   */
  public CommandProperties getCommandProperties() {
    return commandProperties;
  }

  /**
   * Atribui o configurador do algoritmo a este comando.
   * 
   * @param algorithmConfigurator O configurador de algoritmo (no aceita
   *        {@code null}).
   */
  private void setAlgorithmConfigurator(
    final AlgorithmConfigurator algorithmConfigurator) {
    if (null == algorithmConfigurator) {
      throw new IllegalArgumentException(
        "O parmetro algorithmConfigurator est nulo.");
    }

    this.algorithmConfigurator = algorithmConfigurator;
  }

  /**
   * Atribui as propriedades de comando deste comando.
   * 
   * @param commandProperties As proprieades do comando (no aceita {@code null}
   *        ).
   */
  private void setCommandProperties(final CommandProperties commandProperties) {
    if (null == commandProperties) {
      throw new IllegalArgumentException(
        "O parmetro commandProperties est nulo.");
    }

    this.commandProperties = commandProperties;
  }

  /**
   * Cria o comando de algoritmo.
   * 
   * @param commandProperties As propriedades do comando (no aceita {@code
   *        null}).
   * @param algorithmConfigurator O configurador do(s) algoritmo(s) (no aceita
   *        {@code null}).
   */
  public AlgorithmCommand(final CommandProperties commandProperties,
    final AlgorithmConfigurator algorithmConfigurator) {

    setCommandProperties(commandProperties);
    setAlgorithmConfigurator(algorithmConfigurator);
  }
}
