/*
 * $Id: FlowCommand.java 101016 2010-01-25 19:36:52Z clinio $
 */

package csbase.logic.algorithms.commands;

import java.io.Serializable;

import csbase.logic.algorithms.flows.configurator.FlowAlgorithmConfigurator;

/**
 * Comando de Fluxo de Algoritmos.
 * 
 * @author Tecgraf/PUC-Rio
 */
public final class FlowCommand implements Serializable {

  /**
   * O configurador de fluxo de algoritmos.
   */
  private FlowAlgorithmConfigurator flowAlgorithmConfigurator;

  /**
   * Obtm o configurador de fluxo de algoritmos.
   * 
   * @return O configurador de fluxo de algoritmos.
   */
  public FlowAlgorithmConfigurator getFlowAlgorithmConfigurator() {
    return flowAlgorithmConfigurator;
  }

  /**
   * Atribui as propriedades de comando deste comando.
   * 
   * @param commandProperties proprieades do comando (no aceita {@code null}).
   */
  private void setCommandProperties(final CommandProperties commandProperties) {
    if (commandProperties == null) {
      final String err = "O parmetro commandProperties est nulo.";
      throw new IllegalArgumentException(err);
    }
  }

  /**
   * Atribui o configurador de fluxos de algoritmos a este comando.
   * 
   * @param flowAlgorithmConfigurator O configurador de fluxo de algoritmos (no
   *        aceita {@code null}).
   */
  private void setFlowAlgorithmConfigurator(
    final FlowAlgorithmConfigurator flowAlgorithmConfigurator) {
    if (flowAlgorithmConfigurator == null) {
      final String err = "O parmetro flowAlgorithmConfigurator est nulo.";
      throw new IllegalArgumentException(err);
    }
    this.flowAlgorithmConfigurator = flowAlgorithmConfigurator;
  }

  /**
   * Cria o comando de fluxo de algoritmos.
   * 
   * @param commandProperties As propriedades do comando (no aceita {@code
   *        null}).
   * @param flowAlgorithmConfigurator O configurador de fluxo de algoritmos (no
   *        aceita {@code null}).
   */
  public FlowCommand(final CommandProperties commandProperties,
    final FlowAlgorithmConfigurator flowAlgorithmConfigurator) {
    setCommandProperties(commandProperties);
    setFlowAlgorithmConfigurator(flowAlgorithmConfigurator);
  }
}
