package csbase.logic.algorithms.parameters;

import java.util.List;

import csbase.exception.algorithms.ExpressionFunctionExecutionException;

public abstract class ListExpressionFunction extends ExpressionFunction {
  public ListExpressionFunction(String name, Class... parameterTypes) {
    super(name, createParameterTypes(parameterTypes));
  }

  protected abstract Object doOperation(List<Object> values,
                                        Object... arguments)
    throws ExpressionFunctionExecutionException;

  @Override
  protected final Object doOperation(SimpleAlgorithmConfigurator configurator,
                                     Object... parameters)
    throws ExpressionFunctionExecutionException {
    if (configurator == null) {
      throw new IllegalArgumentException("O parmetro configurator est nulo.");
    }
    if (parameters == null) {
      throw new IllegalArgumentException("O parmetro parameters est nulo.");
    }
    if (parameters.length == 0) {
      throw new ExpressionFunctionExecutionException(
        "Esta funo trabalha sobre listas.\n"
          + "O primeiro parmetro tem que ser uma lista, "
          + "porm no foram informados valores.\n"
          + "Primeiro fornea o nome da lista a ser utilizada.\n"
          + "Funo: {0}.", this);
    }
    if (!(parameters[0] instanceof String)) {
      throw new ExpressionFunctionExecutionException(
        "Esta funo trabalha sobre listas.\n"
          + "O primeiro valor tem que ser uma lista, "
          + "porm foi informado um valor que no  um nome de lista.\n"
          + "Primeiro fornea o nome da lista a ser utilizada.\n"
          + "Funo: {0}.\nValor: {1}.", this, parameters[0]);
    }
    String parameterName = (String) parameters[0];
    SimpleParameter parameter = configurator.getSimpleParameter(parameterName);
    if (parameter == null) {
      throw new ExpressionFunctionExecutionException(
        "A lista {0} no foi encontrada no configurador {1}.\n"
          + "Funo: {2}.", parameterName, configurator, this);
    }
    if (!(parameter instanceof ListParameter)) {
      throw new ExpressionFunctionExecutionException(
        "O parmetro {0} do configurador {1} no  uma lista.\n"
          + "Funo: {2}.", parameterName, configurator, this);
    }
    ListParameter listParameter = (ListParameter) parameter;
    Object[] newParameters = new Object[parameters.length - 1];
    for (int i = 0; i < newParameters.length; i++) {
      newParameters[i] = parameters[i + 1];
    }
    List<Object> values = (List) listParameter.getValue();
    return doOperation(values, newParameters);
  }

  private static Class[] createParameterTypes(Class[] parameterTypes) {
    if (parameterTypes == null) {
      throw new IllegalArgumentException(
        "O parmetro parameterTypes est nulo.");
    }
    Class[] newParameterTypes = new Class[parameterTypes.length + 1];
    newParameterTypes[0] = String.class;
    for (int i = 1; i < newParameterTypes.length; i++) {
      newParameterTypes[i] = parameterTypes[i - 1];
    }
    return newParameterTypes;
  }
}
