package csbase.logic.algorithms.parameters;

import java.text.MessageFormat;

/**
 * Parmetro do tipo Lista de Texto.
 *
 * @author lmoreira
 */
public final class TextListParameter extends ListParameter<String> {
  /**
   * O nmero mximo de caracteres.
   */
  private Integer maxCharacters;
  
  /**
   * Cria um parmetro do tipo lista de texto.
   * 
   * @param name O nome do parmetro (No aceita {@code null}).
   * @param label O rtulo do parmetro (No aceita {@code null}).
   * @param description A descrio do parmetro (No aceita {@code null}).
   * @param isOptional Indica se  opcional.
   * @param commandLinePattern O padro para construo da linha de comando. O 
   *        padro ser utilizado para escrever o trecho da linha do comando 
   *        referente ao parmetro. Esta string ser passada para o mtodo
   *        MessageFormat.format(String,Object...). O primeiro formato ({0})
   *         referente ao nome e o segundo formato ({1})  referente
   *        ao valor. Se {@code null} o parmetro no produzir
   *        sada na linha de comando. 
   * @param maxCharacters O nmero mximo de caracteres para que o valor
   *        de uma clula seja considerado vlido (Aceita {@code null}).
   */
  public TextListParameter(
    String name, String label, String description, boolean isOptional, 
    String commandLinePattern, Integer maxCharacters) {
    super(name, label,
      description, isOptional, commandLinePattern);
    setMaxCharacters(maxCharacters);
  }

  @Override
  protected String getItemValueFromText(String itemValue) {
    return itemValue;
  }

  @Override
  public String getType() {
    return "LIST_OF_TEXTS";
  }

  /**
   * Obtm o nmero mximo de caracteres para que o valor de uma clula seja 
   * considerado vlido.
   *
   * @return O nmero mximo de caracteres ou {@code null} caso no haja 
   *         um nmero mximo de caracteres definido.
   */
  public Integer getMaxCharacters() {
    return this.maxCharacters;
  }
  
  /**
   * Atribui o nmero mximo de caracteres para que o valor de uma clula seja 
   * considerado vlido.
   *
   * @param maxCharacters O nmero mximo de caracteres ou {@code null} caso 
   *        no haja um nmero mximo de caracteres definido.
   */
  private void setMaxCharacters(Integer maxCharacters) {
    if (maxCharacters != null && maxCharacters.intValue() <= 0) {
      throw new IllegalArgumentException(
        MessageFormat.format(
          "O parmetro maxCharacters deveria ser positivo.\n"+
          "Valor encontrado: {0}.", new Object[] { maxCharacters }));
    }
    this.maxCharacters = maxCharacters;
  }
}
