package csbase.logic.algorithms.parsers;

import csbase.exception.ParseException;
import csbase.logic.algorithms.parameters.FileParameterMode;
import csbase.logic.algorithms.parameters.FileParameterPipeAcceptance;
import csbase.logic.algorithms.parameters.InputFileParameter;
import csbase.logic.algorithms.parameters.ParameterGroup;

/**
 * <p>
 * Analisador de {@link InputFileParameter}.
 * </p>
 * 
 * <p>
 * Este parser l os atributos de parmetros do tipo "arquivo de entrada"
 * simples. O elemento corrente do {@link XmlParser analisador de XML} precisa
 * ser um elemento {@link InputFileParameter}.
 * </p>
 * 
 */
public class InputFileParameterParser extends
  AbstractFileParameterParser<InputFileParameter> {

  /**
   * <p>
   * O elemento {@value #INPUT_FILE_PARAMETER_ELEMENT}: descreve as propriedades
   * de um {@link InputFileParameter parmetro do tipo arquivo de entrada}.
   * </p>
   * <p>
   *  filho do elemento {@link ParameterGroup}.
   * </p>
   */
  static final String INPUT_FILE_PARAMETER_ELEMENT = "arquivo_de_entrada";

  /**
   * {@inheritDoc}
   */
  @Override
  public InputFileParameter createFileParameter(XmlParser parser, String name,
    String label, String description, boolean isOptional,
    String commandLinePattern, String type) throws ParseException {

    final boolean sortDefined =
      parser.hasAttributeValue(FILE_PARAMETER_ELEMENT_MUST_SORT_ATTRIBUTE);
    if (sortDefined) {
      boolean mustSort =
        parser.extractAttributeValueAsBoolean(
          FILE_PARAMETER_ELEMENT_MUST_SORT_ATTRIBUTE,
          FILE_PARAMETER_ELEMENT_MUST_SORT_DEFAULT_VALUE);
      if (mustSort) {
        throw new ParseException(
          "Arquivo de entrada (seleo nica) no admite atributo: "
            + FILE_PARAMETER_ELEMENT_MUST_SORT_ATTRIBUTE);
      }
    }

    FileParameterPipeAcceptance usesPipe =
      parser.extractAttributeValueAsEnumeration(
        FILE_PARAMETER_ELEMENT_CAN_USE_PIPE_ATTRIBUTE,
        FILE_PARAMETER_ELEMENT_CAN_USE_PIPE_DEFAULT_VALUE,
        new FileParameterPipeAcceptanceConverter());
    String modeName =
      parser.extractAttributeValue(FILE_PARAMETER_ELEMENT_CATEGORY_ATTRIBUTE,
        FILE_PARAMETER_ELEMENT_CATEGORY_VALUE_REGULAR_FILE);
    FileParameterMode mode = null;
    if (modeName.equals(FILE_PARAMETER_ELEMENT_CATEGORY_VALUE_DIRECTORY)) {
      mode = FileParameterMode.DIRECTORY;
    }
    else if (modeName
      .equals(FILE_PARAMETER_ELEMENT_CATEGORY_VALUE_DIRECTORY_AND_REGULAR_FILE)) {
      mode = FileParameterMode.DIRECTORY_AND_REGULAR_FILE;
    }
    else if (modeName
      .equals(FILE_PARAMETER_ELEMENT_CATEGORY_VALUE_REGULAR_FILE)) {
      mode = FileParameterMode.REGULAR_FILE;
    }
    else {
      throw new ParseException("A categoria informada {0} no  vlida.\n"
        + "Categorias vlidas:\n{1};\n{2};{3}.", new Object[] { modeName,
          FILE_PARAMETER_ELEMENT_CATEGORY_VALUE_REGULAR_FILE,
          FILE_PARAMETER_ELEMENT_CATEGORY_VALUE_DIRECTORY,
          FILE_PARAMETER_ELEMENT_CATEGORY_VALUE_DIRECTORY_AND_REGULAR_FILE });
    }
    InputFileParameter parameter =
      new InputFileParameter(name, label, description, isOptional,
        commandLinePattern, type, mode, usesPipe);
    return parameter;
  }

}
