package csbase.logic.algorithms.xml.category;

import java.io.BufferedWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;

import tecgraf.javautils.xml.XMLWriter;
import tecgraf.javautils.xml.exception.XMLException;
import csbase.logic.algorithms.CategorySet;

/**
 * Esta classe implementa o escritor XML de arquivos de categorias de algoritmos
 * existentes no sistema.
 * 
 */
public class XmlCategoriesWriter extends XMLWriter {

  /** Tamanho do buffer de sada */
  final static public int OUT_BUFFER_SIZE = 128 * 1024;

  /**
   * Constri um escritor XML de arquivos de categorias de algoritmos.
   * 
   * @param categorySet conjunto de categorias de algoritmos
   * @throws XMLException exceo de XML
   */
  public XmlCategoriesWriter(CategorySet categorySet) throws XMLException {
    super(categorySet, new XmlCategoriesFactory(categorySet));
  }

  /**
   * Escreve as categorias existentes no sistema na stream de sada, no formato
   * XML.
   * 
   * @param out stream de sada
   * @throws IOException erro de I/O
   */
  public void write(OutputStream out) throws IOException {
    write(new BufferedWriter(new OutputStreamWriter(out), OUT_BUFFER_SIZE));
    out.close();
  }

}
