/*
 * $Id: DiskOccupation.java 83625 2008-10-01 19:59:50Z clinio $
 */

package csbase.logic.diskusageservice;

import java.io.Serializable;

/**
 * Estrutura que armazena o ocupao (livre/total) do disco.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class DiskOccupation implements Serializable {
  /**
   * Espao livre
   */
  final private long free;

  /**
   * Espao total
   */
  final private long total;

  /**
   * Converte um valor de bytes para mega-bytes
   * 
   * @param bytes o valor em bytes.
   * @return o valor em mega bytes.
   */
  final private double bytesToMb(final long bytes) {
    return bytes / (1024 * 1024); // Mb
  }

  /**
   * @return o valor
   */
  final public double getFreeSpaceMb() {
    return bytesToMb(free);
  }

  /**
   * @return o valor
   */
  final public double getTotalSpaceMb() {
    return bytesToMb(total);
  }

  /**
   * @return o valor
   */
  final public double getUsedSpaceMb() {
    final long used = total - free;
    return bytesToMb(used);
  }

  /**
   * @return o valor
   */
  final public double getFreeSpacePerc() {
    final double totalSpaceMb = getTotalSpaceMb();
    if (totalSpaceMb <= 0.0) {
      return 0.0;
    }
    return getFreeSpaceMb() / totalSpaceMb * 100;
  }

  /**
   * @return o valor
   */
  final public double getUsedSpacePerc() {
    final double totalSpaceMb = getTotalSpaceMb();
    if (totalSpaceMb <= 0.0) {
      return 100.0;
    }
    return getUsedSpaceMb() / totalSpaceMb * 100;
  }

  /**
   * Construtor para objeto vlido
   * 
   * @param free espao livre em bytes
   * @param total espao total em bytes
   */
  public DiskOccupation(final long free, final long total) {
    this.free = free;
    this.total = total;
  }

  /**
   * Construtor para objeto invlido
   * 
   */
  public DiskOccupation() {
    this.free = -1;
    this.total = -1;
  }

}
