/*
 * ServiceLogFormatter.java
 *
 * $Author: taciana $
 * $Revision: 69519 $  -  $Date: 2007-10-27 20:17:45 -0200 (Sat, 27 Oct 2007) $
 */
package csbase.server;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.text.SimpleDateFormat;
import java.util.logging.Formatter;
import java.util.logging.LogRecord;

/**
 * Imprime um resumo do objeto <code>LogRecord</code> em formato texto de facil
 * leitura.
 *
 * @see java.util.logging.SimpleFormatter
 * @since 1.4
 */
public class ServerLogFormatter extends Formatter {

  /** Separador de linha */
  private final String LINE_SEPARATOR = "\n";

  /**
   * Formata o <code>LogRecord</code> dado.
   *
   * @param record registro de log a ser formatado.
   *
   * @return registro de log formatado.
   */
  @Override
  public synchronized String format(LogRecord record) {
    StringBuffer sb = new StringBuffer();

    SimpleDateFormat dateHourFormat =
      new SimpleDateFormat("dd/MM/yyyy HH:mm:ss");
    dateHourFormat.setLenient(false);
    String date = dateHourFormat.format(record.getMillis());

    sb.append(record.getLevel().getLocalizedName());
    sb.append(": ");
    sb.append(date);
    sb.append(" --- ");

    if (record instanceof ServerLogRecord) {
      ServerLogRecord serverRecord = (ServerLogRecord) record;
      if (serverRecord.getUser() != null) {
        sb.append("(");
        sb.append(serverRecord.getUser().getId());
        sb.append(")");
      }
    }

    String message = formatMessage(record);
    sb.append(message);
    sb.append(LINE_SEPARATOR);
    if (record.getThrown() != null) {
      sb.append(format(record.getThrown()));
    }
    return sb.toString();
  }

  /**
   * Formata uma exceo.
   * @param t a exceo
   * @return o texto que descreve a exceo
   */
  private static String format(Throwable t) {
    StringWriter sw = new StringWriter();
    PrintWriter pw = new PrintWriter(sw);
    String tab ="";
    format(tab, pw, t);
    pw.close();
    return sw.toString();
  }

  /**
   * Formata uma exceo.
   * @param tab a string de formatao
   * @param pw onde ser escrito o texto da exceo
   * @param t a exceo
   */
  private static void format(String tab, PrintWriter pw, Throwable t) {
    pw.println(tab + "Exceo: ");
    pw.println(tab + "Tipo: " + t.getClass().getName());
    pw.println(tab + "Mensagem: " + t.getMessage());
    pw.println(tab + "Pilha de execuo:");
    t.printStackTrace(pw);
    if (t.getCause() != null && t.getCause() != t) {
      pw.println("Causada por: ");
      format(tab + "  ", pw, t.getCause());
    }
  }

  /**
   * Retorna uma mensagem qualificada, isto , contendo informaes acerca do
   * ponto no cdigo onde ela foi gerada. O formato retornado :
   * [nome_longo_da_classe.nome_do_metodo:numero_da_linha] mensagem.
   *
   * @param msg mensagem a ser qualificada.
   *
   * @return mensagem qualificada.
   */
  public static String getQualifiedMessage(String msg) {
    StackTraceElement[] stack = (new Throwable()).getStackTrace();
    StackTraceElement method = stack[2];
    StringBuilder sb = new StringBuilder("[");
    sb.append(method.getClassName());
    sb.append(".");
    sb.append(method.getMethodName());
    sb.append(":");
    sb.append(method.getLineNumber());
    sb.append("] ");
    sb.append(msg);
    return sb.toString();
  }
}
