/*
 * $Id: ServiceProxy.java 88748 2009-02-27 19:41:22Z clinio $
 */

package csbase.server;

import csbase.logic.User;
import csbase.remote.ServiceInterface;

import java.rmi.RemoteException;
import java.rmi.server.RemoteObject;
import java.rmi.server.UnicastRemoteObject;

/**
 * Super-classe dos proxies remotos para acesso aos servios. Garante que o
 * usurio que realizou a chamada possa ser identificado durante a execuo do
 * mtodo do servio. Para isso, ser criado um objeto proxy para cada servio
 * para cada usurio que se conectar ao sistema.
 * 
 * @author Tecgraf/PUC-Rio
 */
public abstract class ServiceProxy extends RemoteObject implements
  ServiceInterface {

  /**
   * A chave
   */
  protected Object key;

  /**
   * Registro de falhas detetadas durante a execuo de um pedido de servio.
   * 
   * @param serviceName nome do servio
   * @param throwable a exceo que representa a falha
   */
  protected void logException(final String serviceName,
    final Throwable throwable) {
    final Service service = Service.getInstance(serviceName);
    final User user = Service.getUser();
    final Object uid = user.getId();
    final String msg = "Falha em pedido de servio para usurio: ";
    service.logSevereMessage(msg + uid, throwable);
  }

  /**
   * Termina o proxy.
   * 
   * @throws RemoteException se houver falha
   */
  public void shutdownProxy() throws RemoteException {
    UnicastRemoteObject.unexportObject(this, true);
  }

  /**
   * Cria um proxy para um usurio.
   * 
   * @param key a chave que identifica o usurio
   * 
   * @throws RemoteException se houver falha.
   */
  public ServiceProxy(final Object key) throws RemoteException {
    this.key = key;
    UnicastRemoteObject.exportObject(this, 0);
  }
}
