package csbase.server.services.applicationservice;

import java.rmi.RemoteException;
import java.util.Hashtable;

import csbase.exception.BugException;
import csbase.exception.CSBaseRuntimeException;
import csbase.exception.ServiceFailureException;
import csbase.logic.applicationservice.ApplicationCategory;
import csbase.logic.applicationservice.ApplicationRegistry;
import csbase.remote.ApplicationServiceInterface;
import csbase.server.Service;
import csbase.util.remote.RemoteInputStream;

/**
 * @author Tecgraf/PUC-Rio
 */
public class ApplicationServiceProxy extends csbase.server.ServiceProxy
  implements ApplicationServiceInterface {

  /**
   * Construtor
   * @param param_0
   * @throws java.rmi.RemoteException
   */
  public ApplicationServiceProxy(java.lang.Object param_0)
    throws java.rmi.RemoteException {
    super(param_0);
  }

  /**
   * {@inheritDoc}
   */
  public Hashtable<String, ApplicationRegistry> getApplicationRegistries()
    throws java.rmi.RemoteException {
    try {
      Service.setKey(this.key);
      return ApplicationService.getInstance().getApplicationRegistries();
    }
    catch (ServiceFailureException fe) {
      logException(SERVICE_NAME, fe);
      throw new ServiceFailureException(fe.getClientMessage());
    }
    catch (CSBaseRuntimeException re) {
      throw re;
    }
    catch (Throwable t) {
      logException(SERVICE_NAME, t);
      throw new BugException();
    }
  }

  /**
   * {@inheritDoc}
   */
  public Hashtable<String, ApplicationCategory> getApplicationCategories()
    throws java.rmi.RemoteException {
    try {
      Service.setKey(this.key);
      return ApplicationService.getInstance().getApplicationCategories();
    }
    catch (ServiceFailureException fe) {
      logException(SERVICE_NAME, fe);
      throw new ServiceFailureException(fe.getClientMessage());
    }
    catch (CSBaseRuntimeException re) {
      throw re;
    }
    catch (Throwable t) {
      logException(SERVICE_NAME, t);
      throw new BugException();
    }
  }

  /**
   * {@inheritDoc}
   */
  public boolean isAlive() throws java.rmi.RemoteException {
    try {
      Service.setKey(this.key);
      return ApplicationService.getInstance().isAlive();
    }
    catch (ServiceFailureException fe) {
      logException(SERVICE_NAME, fe);
      throw new ServiceFailureException(fe.getClientMessage());
    }
    catch (CSBaseRuntimeException re) {
      throw re;
    }
    catch (Throwable t) {
      logException(SERVICE_NAME, t);
      throw new BugException();
    }
  }

  /**
   * {@inheritDoc}
   */
  public RemoteInputStream getApplicationResourceAsStream(String appId,
    String[] resourcePath) throws RemoteException {
    try {
      Service.setKey(this.key);
      return ApplicationService.getInstance().getApplicationResourceAsStream(
        appId, resourcePath);
    }
    catch (ServiceFailureException fe) {
      logException(SERVICE_NAME, fe);
      throw new ServiceFailureException(fe.getClientMessage());
    }
    catch (CSBaseRuntimeException re) {
      throw re;
    }
    catch (Throwable t) {
      logException(SERVICE_NAME, t);
      throw new BugException();
    }
  }
}