/*
 * $Id: MonitoredDirectory.java 94183 2009-07-27 17:01:05Z clinio $
 */

package csbase.server.services.diskusageservice;

import java.io.File;

import csbase.server.ServerException;

/**
 * Entidade de monitorao do servio.
 * 
 * @author Tecgraf/PUC-Rio
 */
class MonitoredDirectory {

  /**
   * Diretrio a ser analisado.
   */
  final private File directory;

  /**
   * Identificador
   */
  final private String id;

  /**
   * Construtor
   * 
   * @param id identificador.
   * @param dirPath path do diretrio a ser monitorado.
   * @throws ServerException em caso de m configurao.
   */
  public MonitoredDirectory(final String id, final String dirPath)
    throws ServerException {
    final File dir = new File(dirPath);
    if (!dir.exists()) {
      final String prefix = "No existe o diretrio ";
      final String sufix = " para monitorao de disco.";
      final String err = prefix + "[" + dirPath + "]" + sufix;
      throw new ServerException(err);
    }

    if (!dir.isDirectory()) {
      final String pth = dir.getAbsolutePath();
      final String prefix = "Monitorao (" + id + "): ";
      final String message = prefix + "[" + pth + "] no  diretrio!";
      throw new ServerException(message);
    }

    this.id = id;
    this.directory = dir;
  }

  /**
   * @return o campo id
   */
  final String getId() {
    return id;
  }

  /**
   * Consulta o diretrio a ser monitorado.
   * 
   * @return o diretrio
   */
  final File getDirectory() {
    return directory;
  }
}
