/*
 * $Id: FileTransferJob.java 90593 2009-04-17 00:02:58Z clinio $
 */

package csbase.server.services.filetransferservice;

import csbase.logic.filetransferservice.FileTransferRequest;

/**
 * Thread de transferncia
 * 
 * @author Tecgraf/PUC-Rio
 */
public abstract class FileTransferJob implements Runnable {

  /**
   * Requisio
   */
  final private FileTransferRequest request;

  /**
   * Consulta o valor de request
   * 
   * @return o valor
   */
  public final FileTransferRequest getRequest() {
    return request;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  final public void run() {
    request.markRunning();
    final FileTransferService service = FileTransferService.getInstance();
    final String requestId = request.getId();
    try {
      transfer();
    }
    catch (final Exception e) {
      request.markError(e.getMessage());
      service.logSevereMessage("Falha em: "+requestId, e);
    }
    request.markEnd();
    
    service.moveToEndedList(requestId);
  }

  /**
   * Mtodo que efetivamente faz a transferncia do dado.
   * 
   * @throws Exception em caso de falha na transferncia.
   */
  abstract protected void transfer() throws Exception;

  /**
   * Construtor
   * 
   * @param request requisio.
   */
  public FileTransferJob(final FileTransferRequest request) {
    this.request = request;
  }

}
