/*
 * $Id:$
 */

package csbase.server.services.filetransferservice.ftp;

import java.io.FileInputStream;
import java.io.InputStream;
import java.io.OutputStream;

import org.apache.commons.net.ftp.FTPClient;

import csbase.exception.ServiceFailureException;
import csbase.logic.filetransferservice.FileTransferConnection;
import csbase.logic.filetransferservice.FileTransferRequest;
import csbase.logic.filetransferservice.FileTransferRequestStatus;
import csbase.server.services.filetransferservice.FileTransferJob;
import csbase.server.services.filetransferservice.FileTransferService;
import csbase.server.services.projectservice.ServerProject;
import csbase.server.services.projectservice.ServerProjectFile;

/**
 * Job de upload no protocolo FTP.
 * 
 * @author Tecgraf/PUC-Rio
 */
class FTPUploadJob extends FileTransferJob {

  /**
   * {@inheritDoc}
   */
  @Override
  final protected void transfer() throws Exception {
    final FileTransferRequest request = getRequest();
    final FileTransferConnection connection = request.getConnection();
    final Object projectId = connection.getProjectId();
    final ServerProject project = ServerProject.getProject(projectId);
    final String[] localPath = request.getLocalFilePath();
    final String remoteFilePath = request.getRemoteFilePath();
    final String password = connection.getPassword();
    final String userName = connection.getUserName();
    final String serverName = connection.getServerName();

    final FTPClient ftp = new FTPClient();
    InputStream inStream = null;
    OutputStream outStream = null;

    try {
      ftp.connect(serverName);
      final boolean loginOk = ftp.login(userName, password);
      if (!loginOk) {
        throw new ServiceFailureException("bad login");
      }
      final String[] fullPath = project.makeFullPath(localPath);
      final ServerProjectFile spf =
        ServerProjectFile.findFile(projectId, fullPath);

      inStream = new FileInputStream(spf.getRealFile());
      outStream = ftp.storeFileStream(remoteFilePath);
      final int chunkSize = FileTransferService.CHUNK_SIZE;
      final byte[] b = new byte[chunkSize];
      int bytesRead = 0;
      while ((bytesRead = inStream.read(b)) != -1) {
        outStream.write(b, 0, bytesRead);
        request.addTransferredSize(bytesRead);
        if (request.getStatus() == FileTransferRequestStatus.INTERRUPTED) {
          return;
        }
      }
    }
    catch (final Exception e) {
      try {
        if (ftp.isConnected()) {
          ftp.disconnect();
        }
      }
      catch (final Exception e2) {
        // No faz nada, pois a falha foi justamente no disconnect.
      }
      throw new ServiceFailureException(e.getMessage());
    }
    finally {
      if (outStream != null) {
        outStream.close();
      }
      if (inStream != null) {
        inStream.close();
      }
    }
  }

  /**
   * Construtor
   * 
   * @param request requisio.
   */
  FTPUploadJob(final FileTransferRequest request) {
    super(request);
  }
}
