package csbase.server.services.opendreamsservice;

import scs.core.servant.ComponentContext;
import tecgraf.openbus.Openbus;
import tecgraf.openbus.DRMAA.AuthorizationException;
import tecgraf.openbus.DRMAA.InternalException;
import tecgraf.openbus.DRMAA.Session;
import tecgraf.openbus.DRMAA.SessionHelper;
import tecgraf.openbus.opendreams.IOpenDreamsPOA;
import csbase.server.services.openbusservice.OpenBusService;

/**
 * Implementa a idl <code>IOpenDreams</code>.
 * 
 * Recupera a sesso ativa associada a um determinado usurio e a um projeto
 * csbase. Se a sesso ainda no existir, cria a sesso.
 * 
 * @author Tecgraf PUC-Rio
 */
public class OpenDreams extends IOpenDreamsPOA {
  private Session session;
  private SessionManager sessionManagement;
  private final OpenDreamsService service;

  /**
   * Constri o objeto que implementa a interface <code>IOpenDremas</code>.
   * 
   * @param context contexto do componente dono da faceta.
   */
  public OpenDreams(ComponentContext context) {
    sessionManagement = new SessionManager();
    service = OpenDreamsService.getInstance();
  }

  @Override
  public Session getSession(String projectName) throws AuthorizationException,
    InternalException {
    org.omg.CORBA.Object obj;
    String currentUser = OpenBusService.getInstance().initCSBaseAccess();
    Openbus bus = Openbus.getInstance();
    try {
      obj = bus.getRootPOA().servant_to_reference(
        sessionManagement.getSession(currentUser, projectName));
    }
    catch (Throwable e) {
      String msg = "Erro na recuperao da sesso do usurio " + currentUser
        + " para o projeto " + projectName;
      service.logSevereMessage(msg, e);
      throw new InternalException(OpenDreams.formatMessage(e, msg));
    }
    session = SessionHelper.narrow(obj);
    OpenBusService.getInstance().finishCSBaseAccess();
    return session;
  }

  /**
   * Cria uma mensagem formatada com o texto de uma mensagem junto com
   * informaes mais detalhadas sobre a prpria exceo.
   * 
   * @param e a exceo que ocorreu
   * @param msg uma mensagem descritiva do contexto da exceo
   * @return um texto formatado
   */
  public static String formatMessage(Throwable e, String msg) {
    StringBuffer message = new StringBuffer(msg);
    message.append(": " + e.toString());
    message.append("\nCaused by:\n");
    StackTraceElement[] stackTrace = e.getStackTrace();
    for (int i = 0; i < stackTrace.length; i++) {
      message.append(stackTrace[i].toString() + "\n");
    }
    return message.toString();
  }
}
