/*
 * $Id$
 */
package csbase.server.services.projectservice;

import java.util.Arrays;

/**
 * Representa um caminho de arquivo.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class UpdatableFileLocation {
  /**
   * O identificador do dono do projeto ao qual o arquivo pertence.
   */
  private Object projectOwnerId;

  /**
   * O caminho do arquivo.
   */
  private String[] path;

  /**
   * Constri um caminho de arquivo.
   * 
   * @param projectOwnerId O identificador do dono do projeto.
   * @param path O caminho do arquivo.
   */
  UpdatableFileLocation(Object projectOwnerId, String[] path) {
    this.projectOwnerId = projectOwnerId;
    this.path = path;
  }

  /**
   * Obtm o identificador do dono do projeto ao qual o arquivo pertence.
   * 
   * @return O identificador do dono do projeto ao qual o arquivo pertence.
   */
  public Object getProjectOwnerId() {
    return this.projectOwnerId;
  }

  /**
   * Obtm o caminho do arquivo.
   * 
   * @return O caminho do arquivo.
   */
  public String[] getPath() {
    return this.path;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean equals(Object obj) {
    if (obj == null) {
      return false;

    }
    if (!obj.getClass().equals(UpdatableFileLocation.class)) {
      return false;
    }
    UpdatableFileLocation fileLocation = (UpdatableFileLocation) obj;
    if (!this.projectOwnerId.equals(fileLocation.projectOwnerId)) {
      return false;
    }
    return Arrays.equals(this.path, fileLocation.path);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public int hashCode() {
    return this.projectOwnerId.hashCode() + Arrays.hashCode(this.path);
  }
}