/*
 * $Id$
 */
package csbase.server.services.projectservice.openbus;

import java.io.Serializable;

import openbusidl.acs.Credential;

import org.omg.CORBA.UserException;

import tecgraf.openbus.Openbus;
import tecgraf.openbus.data_service.DataAccessDenied;
import tecgraf.openbus.data_service.DataKey;
import tecgraf.openbus.data_service.DataNotFound;
import tecgraf.openbus.data_service.InvalidDataKey;
import csbase.logic.FileUpdateNotificationData;
import csbase.logic.FileUpdateNotificationData.Result;
import csbase.server.Service;
import csbase.server.services.notificationservice.NotificationService;
import csbase.server.services.openbusservice.OpenBusService;
import csbase.server.services.projectservice.ProjectFileUpdater;
import csbase.server.services.projectservice.ProjectService;
import csbase.server.services.projectservice.UpdatableFileLocation;

/**
 * Representa um atualizador de arquivo de projeto obtidos atravs do OpenBus.
 * 
 * @author Tecgraf/PUC-Rio
 */
public final class OpenBusProjectFileUpdater implements ProjectFileUpdater {
  /**
   * {@inheritDoc}
   */
  @Override
  public void update(UpdatableFileLocation fileLocation, Serializable userData) {
    if (!OpenBusService.getInstance().isEnabled()) {
      return;
    }
    Openbus bus = Openbus.getInstance();
    Credential serverCredential = bus.getCredential();
    if (serverCredential == null) {
      return;
    }

    final Credential credential =
      new Credential(serverCredential.identifier, serverCredential.owner,
        Service.getUser().getLogin());
    bus.setThreadCredential(credential);
    try {
      String userId = credential.owner;
      if (!credential.delegate.equals("")) {
        userId = credential.delegate;
      }
      Service.setUserId(userId);

      String[] fileLocationPath = fileLocation.getPath();
      String filePath = fileLocationPath[0];
      for (int i = 1; i < fileLocationPath.length; i++) {
        filePath = filePath + "/" + fileLocationPath[i];
      }
      ProjectDataService projectDataService =
        ProjectService.getInstance().getDataService();
      String targetDataId =
        ProjectDataService.generateDataId(fileLocation.getProjectOwnerId(),
          fileLocationPath);
      DataKey targetDataKey;
      try {
        targetDataKey = projectDataService.generateDataKey(targetDataId);
      }
      catch (InvalidDataKey e) {
        NotificationService.getInstance().notifyUserFromService(
          ProjectService.getInstance(), userId,
          new FileUpdateNotificationData(filePath, Result.UNEXPECTED_ERROR));
        ProjectService.getInstance().logSevereMessage(
          "Erro ao criar chave unvoca do arquivo {0}.",
          new Object[] { filePath }, e);
        return;
      }

      byte[] sourceDataKey = (byte[]) userData;
      try {
        projectDataService
          .updateDataFrom(targetDataKey.getKey(), sourceDataKey);
      }
      catch (DataNotFound e) {
        NotificationService.getInstance().notifyUserFromService(
          ProjectService.getInstance(), userId,
          new FileUpdateNotificationData(filePath, Result.SOURCE_NOT_FOUND));
        ProjectService.getInstance().logSevereMessage(
          "O arquivo de origem do arquivo {0} no foi encontrado.",
          new Object[] { filePath }, e);
        return;
      }
      catch (DataAccessDenied e) {
        NotificationService.getInstance().notifyUserFromService(
          ProjectService.getInstance(), userId,
          new FileUpdateNotificationData(filePath, Result.NO_PERMISSION));
        ProjectService.getInstance().logSevereMessage(
          "Sem permisso para atualizar o arquivo {0}.",
          new Object[] { filePath }, e);
        return;
      }
      catch (UserException e) {
        NotificationService.getInstance().notifyUserFromService(
          ProjectService.getInstance(), userId,
          new FileUpdateNotificationData(filePath, Result.UNEXPECTED_ERROR));
        ProjectService.getInstance().logSevereMessage(
          "Erro ao atualizar o arquivo {0}.", new Object[] { filePath }, e);
        return;
      }

      NotificationService.getInstance().notifyUserFromService(
        ProjectService.getInstance(), userId,
        new FileUpdateNotificationData(filePath, Result.SUCCESS));
    }
    finally {
      bus.setThreadCredential(null);
    }
  }
}
