package csbase.client.algorithms.parameters;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.MessageFormat;
import java.util.Hashtable;
import java.util.List;
import java.util.Locale;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.WindowConstants;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.GUIUtils;
import csbase.client.algorithms.AlgorithmConfiguratorView;
import csbase.client.algorithms.AlgorithmConfiguratorViewListener;
import csbase.client.algorithms.parameters.ParameterView.Mode;
import csbase.client.kernel.ClientException;
import csbase.logic.FileInfo;
import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.AlgorithmVersionId;
import csbase.logic.algorithms.AlgorithmVersionInfo;
import csbase.logic.algorithms.ExecutionType;
import csbase.logic.algorithms.parameters.Parameter;
import csbase.logic.algorithms.parameters.ParameterGroup;
import csbase.logic.algorithms.parameters.SimpleAlgorithmConfigurator;
import csbase.logic.algorithms.parameters.SimpleParameter;
import csbase.logic.algorithms.parameters.SimpleParameterListener;

public final class ParameterViewTester<V> {
  private JFrame testFrame;
  private Parameter<V> parameter;
  private JLabel statusLabel;

  /**
   * Pattern padro.
   */
  private static final String CMD_PATTERN = "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO";
  
  static {
    Locale locale = new Locale("pt", "BR");
    LNG.load("csbase.client.resources.properties.language.idiom", locale);
    LNG.load("resources.language.idiom", locale);
  }

  public ParameterViewTester(Parameter<V> parameter) {
    if (parameter == null) {
      throw new IllegalArgumentException("O parmetro parameter est nulo.");
    }
    this.parameter = parameter;
    this.testFrame = new JFrame("Teste de visualizao de parmetros");
    this.testFrame.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
    this.testFrame.setSize(800, 600);
    JPanel buttonPanel = new JPanel();
    JButton commandButton = new JButton("Linha de comando");
    commandButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        showMessage("Linha de comando: "
          + ParameterViewTester.this.parameter.getCommandLine(
            "projects/user/project", "/"));
      }
    });
    buttonPanel.add(commandButton);
    JPanel statusPanel = new JPanel(new BorderLayout());
    this.statusLabel = new JLabel();
    statusPanel.add(this.statusLabel, BorderLayout.CENTER);
    JPanel panel = new JPanel(new BorderLayout());
    panel.add(statusPanel, BorderLayout.NORTH);
    panel.add(buttonPanel, BorderLayout.SOUTH);
    this.testFrame.getContentPane().add(panel, BorderLayout.SOUTH);
    GUIUtils.centerOnScreen(this.testFrame);
    this.testFrame.setVisible(true);
  }

  public void test() throws ClientException {
    AlgorithmInfo algorithm =
      new AlgorithmInfo("Id do algoritmo", "Algoritmo de teste", "Diretrio",
        new Hashtable<String, String>());
    AlgorithmVersionInfo algorithmVersion =
      new AlgorithmVersionInfo(algorithm, new AlgorithmVersionId(0, 0, 0),
        new Hashtable<String, List<FileInfo>>(),
        new Hashtable<String, String>());
    SimpleAlgorithmConfigurator configurator =
      new SimpleAlgorithmConfigurator(algorithmVersion, null,
        ExecutionType.SIMPLE, null, false, "comando", null, null, true, CMD_PATTERN);
    ParameterGroup group = new ParameterGroup("Parmetro de teste");
    configurator.addGroup(group);
    group.addParameter(this.parameter);
    if (this.parameter instanceof SimpleParameter) {
      final SimpleParameter<V> simpleParameter =
        SimpleParameter.class.cast(this.parameter);
      simpleParameter
        .addSimpleParameterListener(new SimpleParameterListener<V>() {
          public void capabilityWasChanged(SimpleParameter<V> parameter) {
            showMessage("Mudana de estado habilitado/desabilitado."
              + "\nParmetro: {0};\nStatus: ({1}).\n", simpleParameter,
              simpleParameter.isEnabled());
          }

          public void defaultValueWasChanged(SimpleParameter<V> parameter) {
            showMessage(
              "Mudana de valor-padro.\nParmetro: {0};\nValor: ({1}).\n",
              simpleParameter, simpleParameter.getValueAsText());
          }

          public void valueWasChanged(SimpleParameter<V> parameter) {
            showMessage("Mudana de valor.\nParmetro: {0};\nValor: ({1}).\n",
              simpleParameter, simpleParameter.getValueAsText());
          }

          public void visibilityWasChanged(SimpleParameter<V> parameter) {
            showMessage(
              "Mudana de visibilidade.\nParmetro: {0};\nStatus: ({1}).\n",
              simpleParameter, simpleParameter.isVisible());
          }

          public void labelWasChanged(SimpleParameter<V> parameter) {
            showMessage("Mudana de rtulo.\nParmetro: {0};\nRtulo: {1}.\n",
              simpleParameter.getName(), simpleParameter.getLabel());
          }
        });
    }
    SimpleAlgorithmConfiguratorView view =
      new SimpleAlgorithmConfiguratorView(this.testFrame, configurator,
        Mode.CONFIGURATION);
    view
      .addAlgorithmConfiguratorViewListener(new AlgorithmConfiguratorViewListener() {
        public void wasConfirmed(AlgorithmConfiguratorView view) {
          System.out.println("wasConfirmed " + view.getConfigurator());
        }

        public void wasCancelled(AlgorithmConfiguratorView view) {
          System.out.println("wasCancelled " + view.getConfigurator());
        }
      });
    view.launch();
  }

  private void showMessage(String message) {
    String text = "";
    text += "<html>";
    text += "<center>";
    text += message.replaceAll("\n", "<br>");
    text += "</center>";
    text += "</html>";
    this.statusLabel.setText(text);
  }

  private void showMessage(String messageFormat, Object... parameterFormat) {
    showMessage(MessageFormat.format(messageFormat, parameterFormat));
  }
}
