package csbase.client.algorithms.parameters.validators;

import java.util.Locale;

import tecgraf.javautils.LNG;
import csbase.client.algorithms.parameters.validators.SimpleParameterValidator;
import csbase.logic.algorithms.parameters.MockSimpleParameter;
import junit.framework.TestCase;

public final class SimpleParameterValidatorTest extends TestCase {
  static {
    LNG.load("csbase.client.resources.properties.language.idiom", new Locale(
      "pt", "BR"));
  }
  
  public void testSimpleParameterValidator1() {
    boolean isOptional = true;
    SimpleParameterValidator validator = new MockParameterValidator(isOptional);
    assertEquals(isOptional, validator.isOptional());
  }

  public void testSimpleParameterValidator2() {
    boolean isOptional = false;
    SimpleParameterValidator validator = new MockParameterValidator(isOptional);
    assertEquals(isOptional, validator.isOptional());
  }

  public void testValidateValue1() {
    boolean isOptional = true;
    SimpleParameterValidator validator = new MockParameterValidator(isOptional);
    assertEquals(isOptional, validator.isOptional());
    MockSimpleParameter parameter = new MockSimpleParameter(
      "Nome do parmetro", "Rtulo do parmetro", "Descrio do parmetro",
      null, isOptional, "{0}{1}");
    assertNotNull(validator.validateValue(parameter, null));
    assertTrue(validator.validateValue(parameter, null).isWellSucceded());
  }

  public void testValidateValue2() {
    boolean isOptional = false;
    SimpleParameterValidator validator = new MockParameterValidator(isOptional);
    assertEquals(isOptional, validator.isOptional());
    MockSimpleParameter parameter = new MockSimpleParameter(
      "Nome do parmetro", "Rtulo do parmetro", "Descrio do parmetro",
      null, isOptional, "{0}{1}");
    assertNotNull(validator.validateValue(parameter, null));
    assertFalse(validator.validateValue(parameter, null).isWellSucceded());
  }

  public void testValidateValue3() {
    boolean isOptional = true;
    SimpleParameterValidator validator = new MockParameterValidator(isOptional);
    assertEquals(isOptional, validator.isOptional());
    MockSimpleParameter parameter = new MockSimpleParameter(
      "Nome do parmetro", "Rtulo do parmetro", "Descrio do parmetro",
      null, isOptional, "{0}{1}");
    assertNotNull(validator.validateValue(parameter, "Algum valor"));
    assertTrue(validator.validateValue(parameter, "Algum valor")
      .isWellSucceded());
  }

  public void testValidateValue4() {
    boolean isOptional = false;
    SimpleParameterValidator validator = new MockParameterValidator(isOptional);
    assertEquals(isOptional, validator.isOptional());
    MockSimpleParameter parameter = new MockSimpleParameter(
      "Nome do parmetro", "Rtulo do parmetro", "Descrio do parmetro",
      null, isOptional, "{0}{1}");
    assertNotNull(validator.validateValue(parameter, "Algum valor"));
    assertTrue(validator.validateValue(parameter, "Algum valor")
      .isWellSucceded());
  }
}
