package csbase.logic;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.Stack;

public class MockProjectFile implements ProjectFile {

  private Object creatorId;
  private Set<MockProjectFile> children;
  private boolean isDirectory;
  private String name;
  private MockProjectFile parent;
  private Object projectId;
  private String type;

  public MockProjectFile(Object projectId, String name, Object userId) {
    setProjectId(projectId);
    setName(name);
    setCreator(userId);
    this.children = new HashSet<MockProjectFile>();
    this.isDirectory = true;
  }

  public MockProjectFile(MockProjectFile parent, String name,
    boolean isDirectory, String type) {
    this(parent.getProjectId(), name, parent.whoCreated());
    setParent(parent);
    addChild(this);
    this.type = type;
    this.isDirectory = isDirectory;
  }

  public String getName() {
    return this.name;
  }

  public String[] getPath() {
    Stack<String> pathStack = new Stack<String>();
    for (ProjectFile file = this; file != null; file = file.getParent()) {
      pathStack.push(file.getName());
    }
    String[] path = new String[pathStack.size()];
    for (int i = 0; i < path.length; i++) {
      path[i] = pathStack.pop();
    }
    return path;
  }

  public String getAbsolutePath() {
    String fileSeparator = "/";
    String absolutePath = fileSeparator;
    absolutePath += getProjectId();
    absolutePath += fileSeparator;
    absolutePath += whoCreated();
    String[] path = getPath();
    for (String component : path) {
      absolutePath += fileSeparator;
      absolutePath += component;
    }
    return absolutePath;
  }

  public String getType() {
    return this.type;
  }

  public long getCreationDate() {
    return 0;
  }

  public long getModificationDate() {
    return 0;
  }

  public void changeType(String newType) {
    this.type = newType;
  }

  public Object whoPublished() {
    return null;
  }

  public Object whichProjectPublished() {
    return null;
  }

  public Object whoCreated() {
    return this.creatorId;
  }

  public ProjectFile getParent() {
    return this.parent;
  }

  public ProjectFile[] getChildren() {
    ProjectFile[] newChildren = new ProjectFile[this.children.size()];
    Iterator<MockProjectFile> childIterator = this.children.iterator();
    for (int i = 0; i < newChildren.length; i++) {
      newChildren[i] = childIterator.next();
    }
    return newChildren;
  }

  public ProjectFile[] getChildren(String fileType) {
    Set<MockProjectFile> acceptedChildren =
      new HashSet<MockProjectFile>(this.children);
    for (Iterator<MockProjectFile> childIterator = acceptedChildren.iterator(); childIterator
      .hasNext();) {
      MockProjectFile child = childIterator.next();
      if (fileType == null && child.getType() != null) {
        childIterator.remove();
      }
      else if (!fileType.equals(child.getType())) {
        childIterator.remove();
      }
    }
    ProjectFile[] newChildren = new ProjectFile[acceptedChildren.size()];
    Iterator<MockProjectFile> childIterator = acceptedChildren.iterator();
    for (int i = 0; i < newChildren.length; i++) {
      newChildren[i] = childIterator.next();
    }
    return newChildren;
  }

  public ProjectFile getChild(String name) {
    for (MockProjectFile child : this.children) {
      if (child.getName().equals(name)) {
        return child;
      }
    }
    return null;
  }

  public Object getProjectId() {
    return this.projectId;
  }

  public void createDirectory(String name) {
  }

  public void rename(String name) {
    setName(name);
  }

  public void copy(ProjectFile directory) {
  }

  public void move(ProjectFile directory) {
  }

  public void remove() {
  }

  public boolean isDirectory() {
    return this.isDirectory;
  }

  public boolean isGlobalFile() {
    return false;
  }

  public boolean isPublishedFile() {
    return false;
  }

  public boolean isLocked() {
    return false;
  }

  public boolean isUnderConstruction() {
    return false;
  }

  public boolean isRegularFile() {
    return !isDirectory();
  }

  public String getDescription() {
    return "Dummy file";
  }

  public void setDescription(String text) {
  }

  public void appendDescription(String text) {
  }

  public boolean publish(Object[] userIds, Object whoPublishedId) {
    return false;
  }

  public boolean updatePublishAccess(Object[] userIds, Object userId) {
    return false;
  }

  public Object[] getPublishAccess() {
    return null;
  }

  public long size() {
    return 0;
  }

  public boolean wasPublishedBy(Object projId) {
    return false;
  }

  private void addChild(MockProjectFile child) {
    this.children.add(child);
  }

  private void setCreator(Object userId) {
    if (userId == null) {
      throw new IllegalArgumentException("O parmetro userId est nulo.");
    }
    this.creatorId = userId;
  }

  private void setName(String name) {
    if (name == null) {
      throw new IllegalArgumentException("O parmetro name est nulo.");
    }
    this.name = name;
  }

  private void setParent(MockProjectFile parent) {
    if (parent == null) {
      throw new IllegalArgumentException("O parmetro parent est nulo.");
    }
    this.parent = parent;
  }

  private void setProjectId(Object projectId) {
    if (projectId == null) {
      throw new IllegalArgumentException("O parmetro projectId est nulo.");
    }
    this.projectId = projectId;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean isRoot() {
    return parent == null;
  }
}
