/*
 * $Author:$ $Date:$ $Release:$
 */
package csbase.logic.algorithms;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Hashtable;
import java.util.List;

import junit.framework.TestCase;
import csbase.logic.FileInfo;

public final class AlgorithmConfiguratorTest extends TestCase {
  public void testAlgorithmConfigurator1() {
    AlgorithmInfo algorithm = createAlgorithm("Nome do algoritmo");
    AlgorithmVersionId id = new AlgorithmVersionId(1, 2, 3);
    AlgorithmVersionInfo algorithmVersion =
      new AlgorithmVersionInfo(
        algorithm, id, new Hashtable<String, List<FileInfo>>(),
        new Hashtable<String, String>());
    String description = "Descrio da verso";
    ExecutionType executionType = ExecutionType.SIMPLE;
    String abbreviation = "Abreviatura da verso";
    boolean provideId = false;
    AlgorithmConfigurator configurator =
      new MockAlgorithmConfigurator(
        algorithmVersion, description, executionType, abbreviation, provideId);
    assertConfigurator(
      algorithm, id, description, executionType, abbreviation, provideId,
      configurator);
  }

  public void testAlgorithmConfigurator2() {
    AlgorithmInfo algorithm = createAlgorithm("Nome do algoritmo");
    AlgorithmVersionId id = new AlgorithmVersionId(1, 2, 3);
    AlgorithmVersionInfo algorithmVersion =
      new AlgorithmVersionInfo(
        algorithm, id, new Hashtable<String, List<FileInfo>>(),
        new Hashtable<String, String>());
    String description = "Descrio da verso";
    ExecutionType executionType = ExecutionType.SIMPLE;
    String abbreviation = "Abreviatura da verso";
    boolean provideId = true;
    AlgorithmConfigurator configurator =
      new MockAlgorithmConfigurator(
        algorithmVersion, description, executionType, abbreviation, provideId);
    assertConfigurator(
      algorithm, id, description, executionType, abbreviation, provideId,
      configurator);
  }

  public void testAlgorithmConfigurator3() {
    AlgorithmInfo algorithm = createAlgorithm("Nome do algoritmo");
    AlgorithmVersionId id = new AlgorithmVersionId(1, 2, 3);
    AlgorithmVersionInfo algorithmVersion =
      new AlgorithmVersionInfo(
        algorithm, id, new Hashtable<String, List<FileInfo>>(),
        new Hashtable<String, String>());
    String description = "Descrio da verso";
    ExecutionType executionType = ExecutionType.SIMPLE;
    String abbreviation = null;
    boolean provideId = false;
    AlgorithmConfigurator configurator =
      new MockAlgorithmConfigurator(
        algorithmVersion, description, executionType, abbreviation, provideId);
    //algorithm.addVersion(configurator);
    assertConfigurator(
      algorithm, id, description, executionType, abbreviation, provideId,
      configurator);
  }

  public void testAlgorithmConfigurator4() {
    AlgorithmVersionInfo algorithmVersion = null;
    String description = "Descrio da verso";
    ExecutionType executionType = ExecutionType.SIMPLE;
    String abbreviation = "Abreviatura da verso";
    boolean provideId = false;
    try {
      new MockAlgorithmConfigurator(
        algorithmVersion, description, executionType, abbreviation, provideId);
      fail("Esperada IllegalArgumentException");
    }
    catch (IllegalArgumentException e) {
    }
  }

  public void testAlgorithmConfigurator5() {
    AlgorithmInfo algorithm = createAlgorithm("Algoritmo");
    AlgorithmVersionId id = AlgorithmVersionIdTest.create();
    AlgorithmVersionInfo algorithmVersion =
      new AlgorithmVersionInfo(
        algorithm, id, new Hashtable<String, List<FileInfo>>(),
        new Hashtable<String, String>());
    String description = null;
    ExecutionType executionType = ExecutionType.SIMPLE;
    String abbreviation = "Abreviatura da verso";
    boolean provideId = false;
    AlgorithmConfigurator configurator =
      new MockAlgorithmConfigurator(
        algorithmVersion, description, executionType, abbreviation, provideId);
    assertConfigurator(
      algorithm, id, description, executionType, abbreviation, provideId,
      configurator);
  }

  public void testAlgorithmConfigurator6() {
    try {
      new MockAlgorithmConfigurator(
        createAlgorithmVersion(1, 2, 3), "Descrio da verso",
        null, "Abreviatura da verso", false);
      fail("Esperada IllegalArgumentException");
    }
    catch (IllegalArgumentException e) {
    }
  }

  public void testEquals1() {
    AlgorithmVersionInfo algorithmVersion =
      createAlgorithmVersion(1, 2, 3);
    String description1 = "Descrio da verso 1";
    ExecutionType executionType1 = ExecutionType.MULTIPLE;
    String abbreviation1 = "Abreviatura 1";
    boolean provideId1 = true;
    AlgorithmConfigurator configurator1 =
      new MockAlgorithmConfigurator(
        algorithmVersion, description1, executionType1, abbreviation1,
        provideId1);
    String description2 = "Descrio da verso 2";
    ExecutionType executionType2 = ExecutionType.PARALLEL;
    String abbreviation2 = "Abreviatura 2";
    boolean provideId2 = false;
    AlgorithmConfigurator configurator2 =
      new MockAlgorithmConfigurator(
        algorithmVersion, description2, executionType2, abbreviation2,
        provideId2);
    assertTrue(configurator1.equals(configurator2));
  }

  public void testEquals2() {
    String filePath = "diretorio/arquivo.xml";
    String description1 = "Descrio da verso 1";
    ExecutionType executionType1 = ExecutionType.MULTIPLE;
    String abbreviation1 = "Abreviatura 1";
    boolean provideId1 = true;
    AlgorithmConfigurator configurator1 =
      new MockAlgorithmConfigurator(
        filePath, description1, executionType1, abbreviation1, provideId1);
    String description2 = "Descrio da verso 2";
    ExecutionType executionType2 = ExecutionType.PARALLEL;
    String abbreviation2 = "Abreviatura 2";
    boolean provideId2 = false;
    AlgorithmConfigurator configurator2 =
      new MockAlgorithmConfigurator(
        filePath, description2, executionType2, abbreviation2, provideId2);
    assertTrue(configurator1.equals(configurator2));
  }

  public void testEquals3() {
    AlgorithmVersionInfo algorithmVersion1 =
      createAlgorithmVersion(1, 1, 1);
    String description = "Descrio da verso 1";
    ExecutionType executionType = ExecutionType.MULTIPLE;
    String abbreviation = "Abreviatura 1";
    boolean provideId = true;
    AlgorithmConfigurator configurator1 =
      new MockAlgorithmConfigurator(
        algorithmVersion1, description, executionType, abbreviation, provideId);
    AlgorithmVersionInfo algorithmVersion2 =
      createAlgorithmVersion(2, 2, 2);
    AlgorithmConfigurator configurator2 =
      new MockAlgorithmConfigurator(
        algorithmVersion2, description, executionType, abbreviation, provideId);
    assertFalse(configurator1.equals(configurator2));
  }

  public void testEquals4() {
    String filePath1 = "diretorio/arquivo1.xml";
    String description = "Descrio da verso 1";
    ExecutionType executionType = ExecutionType.MULTIPLE;
    String abbreviation = "Abreviatura 1";
    boolean provideId = true;
    AlgorithmConfigurator configurator1 =
      new MockAlgorithmConfigurator(
        filePath1, description, executionType, abbreviation, provideId);
    String filePath2 = "diretorio/arquivo2.xml";
    AlgorithmConfigurator configurator2 =
      new MockAlgorithmConfigurator(
        filePath2, description, executionType, abbreviation, provideId);
    assertFalse(configurator1.equals(configurator2));
  }

  public void testEquals5() {
    String filePath = "diretorio/arquivo.xml";
    String description = "Descrio do verso";
    ExecutionType executionType = ExecutionType.PARALLEL;
    String abbreviation = "Abreviatura";
    boolean provideId = false;
    AlgorithmConfigurator configurator1 =
      new MockAlgorithmConfigurator(
        filePath, description, executionType, abbreviation, provideId);
    assertFalse(configurator1.equals(null));
  }

  public void testEquals6() {
    String filePath = "diretorio/arquivo.xml";
    String description = "Descrio do verso";
    ExecutionType executionType = ExecutionType.PARALLEL;
    String abbreviation = "Abreviatura";
    boolean provideId = false;
    AlgorithmConfigurator configurator1 =
      new MockAlgorithmConfigurator(
        filePath, description, executionType, abbreviation, provideId);
    assertFalse(configurator1.equals("Objeto de outra classe"));
  }

  public void testHashCode1() {
    AlgorithmVersionInfo algorithmVersion =
      createAlgorithmVersion(1, 2, 3);
    String description1 = "Descrio da verso 1";
    ExecutionType executionType1 = ExecutionType.MULTIPLE;
    String abbreviation1 = "Abreviatura 1";
    boolean provideId1 = true;
    AlgorithmConfigurator configurator1 =
      new MockAlgorithmConfigurator(
        algorithmVersion, description1, executionType1, abbreviation1,
        provideId1);
    String description2 = "Descrio da verso 2";
    ExecutionType executionType2 = ExecutionType.PARALLEL;
    String abbreviation2 = "Abreviatura 2";
    boolean provideId2 = false;
    AlgorithmConfigurator configurator2 =
      new MockAlgorithmConfigurator(
        algorithmVersion, description2, executionType2, abbreviation2,
        provideId2);
    assertEquals(configurator1.hashCode(), configurator2.hashCode());
  }

  public void testHashCode2() {
    String filePath = "diretorio/arquivo.xml";
    String description1 = "Descrio da verso 1";
    ExecutionType executionType1 = ExecutionType.MULTIPLE;
    String abbreviation1 = "Abreviatura 1";
    boolean provideId1 = true;
    AlgorithmConfigurator configurator1 =
      new MockAlgorithmConfigurator(
        filePath, description1, executionType1, abbreviation1, provideId1);
    String description2 = "Descrio da verso 2";
    ExecutionType executionType2 = ExecutionType.PARALLEL;
    String abbreviation2 = "Abreviatura 2";
    boolean provideId2 = false;
    AlgorithmConfigurator configurator2 =
      new MockAlgorithmConfigurator(
        filePath, description2, executionType2, abbreviation2, provideId2);
    assertEquals(configurator1.hashCode(), configurator2.hashCode());
  }

  public void testToString1() {
    String name = "Nome do algoritmo";
    AlgorithmVersionId versionId = AlgorithmVersionIdTest.create();
    AlgorithmConfigurator configurator =
      createConfigurator(createAlgorithm(name), versionId);
    assertNotNull(configurator.toString());
    String toString = name+" ("+versionId+")";
    assertEquals(toString, configurator.toString());
  }

  public void testToString2() {
    String name = "config.xml";
    AlgorithmConfigurator configurator = createConfigurator(name);
    assertNotNull(configurator.toString());
    assertEquals("", configurator.toString());
  }

  public void testAddAlgorithmConfiguratorListener2() {
    AlgorithmVersionInfo version =
      createAlgorithmVersion(3, 2, 1);
    AlgorithmConfigurator configurator =
      new MockAlgorithmConfigurator(
        version, null, ExecutionType.SIMPLE, null, false);
    try {
      configurator.addAlgorithmConfiguratorListener(null);
      fail("Esperada IllegalArgumentException.");
    }
    catch (IllegalArgumentException e) {
    }
  }
  
  public void testSerialize1()
  throws IOException, ClassNotFoundException {
    AlgorithmVersionInfo algorithmVersion =
      createAlgorithmVersion(1, 2, 3);
    String description = "Descrio do algoritmo";
    String abbreviation = "Abreviatura do algoritmo";
    boolean provideId = true;
    AlgorithmConfigurator expectedVersion =
      new MockAlgorithmConfigurator(
        algorithmVersion, description, ExecutionType.SIMPLE, abbreviation,
        provideId);
    ByteArrayOutputStream outputSource = new ByteArrayOutputStream();
    ObjectOutputStream objectOutput = new ObjectOutputStream(outputSource);
    objectOutput.writeObject(expectedVersion);
    ByteArrayInputStream inputSource =
      new ByteArrayInputStream(outputSource.toByteArray());
    ObjectInputStream objectInput = new ObjectInputStream(inputSource);
    AlgorithmConfigurator loadedVersion =
      (AlgorithmConfigurator)objectInput.readObject();
    assertEquals(
      expectedVersion.getAbbreviation(), loadedVersion.getAbbreviation());
    assertEquals(
      expectedVersion.getAlgorithmVersion(), loadedVersion.getAlgorithmVersion());
    assertEquals(
      expectedVersion.getDescription(), loadedVersion.getDescription());
    assertSame(
      expectedVersion.getExecutionType(), loadedVersion.getExecutionType());
    assertEquals(expectedVersion.provideId(), loadedVersion.provideId());
  }

  private static AlgorithmInfo createAlgorithm(String algorithmName) {
    return new AlgorithmInfo(
      "ID "+algorithmName, algorithmName, "diretrio", new Hashtable<String, String>());
  }

  private static AlgorithmVersionInfo createAlgorithmVersion(
    int major, int minor, int patch) {
    AlgorithmInfo algorithm = createAlgorithm("Algoritmo");
    AlgorithmVersionId versionId = new AlgorithmVersionId(major, minor, patch);
    return new AlgorithmVersionInfo(
      algorithm, versionId, new Hashtable<String, List<FileInfo>>(),
      new Hashtable<String, String>());
  }

  private static AlgorithmConfigurator createConfigurator(String filePath) {
    return new MockAlgorithmConfigurator(
      filePath, "Descrio da verso", ExecutionType.SIMPLE, null, false);
  }

  private static AlgorithmConfigurator createConfigurator(
    AlgorithmInfo algorithm, AlgorithmVersionId versionId) {
    AlgorithmVersionInfo algorithmVersion =
      new AlgorithmVersionInfo(
        algorithm, versionId, new Hashtable<String, List<FileInfo>>(),
        new Hashtable<String, String>());
    return new MockAlgorithmConfigurator(
      algorithmVersion, "Descrio da verso", ExecutionType.SIMPLE, null, false);
  }

  private void assertConfigurator(
    AlgorithmInfo algorithm, AlgorithmVersionId id, String description,
    ExecutionType executionType, String abbreviation, boolean provideId,
    AlgorithmConfigurator configurator) {
    assertEquals(algorithm, configurator.getAlgorithmVersion().getInfo());
    assertEquals(id, configurator.getAlgorithmVersion().getId());
    assertConfigurator(
      description, executionType, abbreviation, provideId, configurator);
  }

  private void assertConfigurator(
    String description, ExecutionType executionType, String abbreviation,
    boolean provideId, AlgorithmConfigurator configurator) {
    assertEquals(description, configurator.getDescription());
    assertEquals(executionType, configurator.getExecutionType());
    assertEquals(abbreviation, configurator.getAbbreviation());
    assertEquals(provideId, configurator.provideId());
  }
}
