package csbase.logic.algorithms.parameters;

import junit.framework.Assert;

/**
 * <p>
 * Observador Verificador de Eventos de {@link IntegerParameter}:
 * </p>
 * 
 * <ul>
 * Ele  responsvel por:
 * <li>Verificar se eventos inesperados esto ocorrendo.</li>
 * <li>Verificar se eventos esperados esto ocorrendo.</li>
 * <li>Verificar se a quantidade de eventos esperados est dentro do esperado.
 * </li>
 * </ul>
 * 
 * @author lmoreira
 */
final class CheckIntegerParameterListener implements IntegerParameterListener {

  /**
   * Libera o evento
   * {@link IntegerParameterListener#maximumWasChanged(IntegerParameter)}.
   */
  private boolean allowMaximumWasChangedEvent;
  /**
   * Libera o evento
   * {@link IntegerParameterListener#minimumWasChanged(IntegerParameter)}.
   */
  private boolean allowMinimumWasChangedEvent;

  /**
   * Contador de eventos ocorridos.
   */
  private int eventCount;

  /**
   * <p>
   * Cria um verificador.
   * </p>
   * 
   * <p>
   * Ele proibe todos os eventos.
   * </p>
   * 
   */
  CheckIntegerParameterListener() {
    allowMaximumWasChangedEvent = false;
    allowMinimumWasChangedEvent = false;
    eventCount = 0;
  }

  /**
   * Permite que haja eventos
   * {@link IntegerParameterListener#maximumWasChanged(IntegerParameter)}.
   */
  public void allowMaximumWasChangedEvent() {
    allowMaximumWasChangedEvent = true;
  }

  /**
   * Permite que haja eventos
   * {@link IntegerParameterListener#minimumWasChanged(IntegerParameter)}.
   */
  public void allowMinimumWasChangedEvent() {
    allowMinimumWasChangedEvent = true;
  }

  /**
   * Obtm o nmero de eventos ocorridos.
   * 
   * @return .
   */
  public int getEventCount() {
    return eventCount;
  }

  public void maximumWasChanged(IntegerParameter parameter) {
    Assert.assertTrue(allowMaximumWasChangedEvent);
    eventCount++;
  }

  public void minimumWasChanged(IntegerParameter parameter) {
    Assert.assertTrue(allowMinimumWasChangedEvent);
    eventCount++;
  }
}
