/*
 * $Author:$ $Date:$ $Release:$
 */
package csbase.logic.algorithms.parameters;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import csbase.logic.algorithms.parameters.DoubleColumn;

import junit.framework.TestCase;

/**
 * Testes unitrios para a classe {@link DoubleColumn}.
 * 
 * @author lmoreira
 */
public final class DoubleColumnTest extends TestCase {
  /**
   * Testando criar uma coluna de reais opcional.
   */
  public void testDoubleColumn1() {
    String id = "Identificador da coluna";
    String label = "Nome do parmetro";
    boolean isEditable = true;
    boolean isOptional = true;
    Double maximum = new Double(10.10);
    boolean includeMaximum = true;
    Double minimum = new Double(1.1);
    boolean includeMinimum = true;
    Double defaultValue = new Double(123.456);
    DoubleColumn column = new DoubleColumn(id, label, defaultValue, isOptional,
      isEditable, maximum, includeMaximum, minimum, includeMinimum);
    assertEquals(id, column.getId());
    assertEquals(label, column.getLabel());
    assertEquals(defaultValue, column.getDefaultValue(0));
    assertEquals(isOptional, column.isOptional());
    assertEquals(isEditable, column.isOptional());
    assertEquals(maximum, column.getMaximum());
    assertEquals(includeMaximum, column.isMaximumIncluded());
    assertEquals(minimum, column.getMinimum());
    assertEquals(includeMinimum, column.isMinimumIncluded());
  }

  /**
   * Testando criar uma coluna de reais obrigatria.
   */
  public void testDoubleColumn2() {
    String id = "Identificador da coluna";
    String label = "Nome do parmetro";
    boolean isEditable = true;
    boolean isOptional = false;
    Double maximum = new Double(10.10);
    boolean includeMaximum = true;
    Double minimum = new Double(1.1);
    boolean includeMinimum = true;
    Double defaultValue = new Double(1234.45679);
    DoubleColumn column = new DoubleColumn(id, label, defaultValue, isOptional,
      isEditable, maximum, includeMaximum, minimum, includeMinimum);
    assertEquals(id, column.getId());
    assertEquals(label, column.getLabel());
    assertEquals(isOptional, column.isOptional());
    assertEquals(isEditable, column.isEditable());
    assertEquals(defaultValue, column.getDefaultValue(0));
    assertEquals(maximum, column.getMaximum());
    assertEquals(includeMaximum, column.isMaximumIncluded());
    assertEquals(minimum, column.getMinimum());
    assertEquals(includeMinimum, column.isMinimumIncluded());
  }

  /**
   * Testando criar uma coluna de reais sem valor-padro.
   */
  public void testDoubleColumn3() {
    String id = "Identificador do parmetro";
    String label = "Nome do parmetro";
    boolean isOptional = true;
    boolean isEditable = true;
    Double maximum = new Double(10.10);
    boolean includeMaximum = true;
    Double minimum = new Double(1.1);
    boolean includeMinimum = true;
    Double defaultValue = null;
    DoubleColumn column = new DoubleColumn(id, label, defaultValue, isOptional,
      isEditable, maximum, includeMaximum, minimum, includeMinimum);
    assertEquals(id, column.getId());
    assertEquals(label, column.getLabel());
    assertEquals(defaultValue, column.getDefaultValue(0));
    assertEquals(isOptional, column.isOptional());
    assertEquals(isEditable, column.isEditable());
    assertEquals(maximum, column.getMaximum());
    assertEquals(includeMaximum, column.isMaximumIncluded());
    assertEquals(minimum, column.getMinimum());
    assertEquals(includeMinimum, column.isMinimumIncluded());
  }

  /**
   * Testando criar uma coluna de reais com incluir mximo falso.
   */
  public void testDoubleColumn4() {
    String id = "Identificador do parmetro";
    String label = "Nome do parmetro";
    boolean isEditable = true;
    boolean isOptional = true;
    Double maximum = new Double(10.10);
    boolean includeMaximum = false;
    Double minimum = new Double(1.1);
    boolean includeMinimum = true;
    Double defaultValue = new Double(1234.45679);
    DoubleColumn column = new DoubleColumn(id, label, defaultValue, isOptional,
      isEditable, maximum, includeMaximum, minimum, includeMinimum);
    assertEquals(id, column.getId());
    assertEquals(label, column.getLabel());
    assertEquals(defaultValue, column.getDefaultValue(0));
    assertEquals(isOptional, column.isOptional());
    assertEquals(isEditable, column.isEditable());
    assertEquals(maximum, column.getMaximum());
    assertEquals(includeMaximum, column.isMaximumIncluded());
    assertEquals(minimum, column.getMinimum());
    assertEquals(includeMinimum, column.isMinimumIncluded());
    assertEquals(isOptional, column.isOptional());
  }

  /**
   * Testando criar uma coluna de reais com incluir mnimo falso.
   */
  public void testDoubleColumn5() {
    String id = "Identificador do parmetro";
    String label = "Nome do parmetro";
    boolean isEditable = true;
    boolean isOptional = true;
    Double maximum = new Double(10.10);
    boolean includeMaximum = true;
    Double minimum = new Double(1.1);
    boolean includeMinimum = false;
    Double defaultValue = new Double(1234.56789);
    DoubleColumn column = new DoubleColumn(id, label, defaultValue, isOptional,
      isEditable, maximum, includeMaximum, minimum, includeMinimum);
    assertEquals(id, column.getId());
    assertEquals(label, column.getLabel());
    assertEquals(isEditable, column.isEditable());
    assertEquals(defaultValue, column.getDefaultValue(0));
    assertEquals(maximum, column.getMaximum());
    assertEquals(includeMaximum, column.isMaximumIncluded());
    assertEquals(minimum, column.getMinimum());
    assertEquals(includeMinimum, column.isMinimumIncluded());
    assertEquals(isOptional, column.isOptional());
  }

  /**
   * Testando criar uma coluna de reais sem mximo.
   */
  public void testDoubleColumn6() {
    String id = "Identificador do parmetro";
    String label = "Nome do parmetro";
    boolean isOptional = true;
    boolean isEditable = true;
    boolean includeMaximum = true;
    Double minimum = new Double(1.1);
    boolean includeMinimum = true;
    Double defaultValue = new Double(1234.56789);
    DoubleColumn column = new DoubleColumn(id, label, defaultValue, isOptional,
      isEditable, null, includeMaximum, minimum, includeMinimum);
    assertEquals(id, column.getId());
    assertEquals(label, column.getLabel());
    assertEquals(defaultValue, column.getDefaultValue(0));
    assertNull(column.getMaximum());
    assertFalse(column.isMaximumIncluded());
    assertEquals(minimum, column.getMinimum());
    assertEquals(includeMinimum, column.isMinimumIncluded());
    assertEquals(isOptional, column.isOptional());
  }

  /**
   * Testando criar uma coluna de reais sem mnimo.
   */
  public void testDoubleColumn7() {
    String id = "Identificador da coluna";
    String label = "Nome do parmetro";
    boolean isOptional = true;
    boolean isEditable = true;
    Double maximum = new Double(10.10);
    boolean includeMaximum = true;
    boolean includeMinimum = true;
    Double defaultValue = new Double(1234);
    DoubleColumn column = new DoubleColumn(id, label, defaultValue, isOptional,
      isEditable, maximum, includeMaximum, null, includeMinimum);
    assertEquals(id, column.getId());
    assertEquals(label, column.getLabel());
    assertEquals(defaultValue, column.getDefaultValue(0));
    assertEquals(maximum, column.getMaximum());
    assertEquals(includeMaximum, column.isMaximumIncluded());
    assertNull(column.getMinimum());
    assertFalse(column.isMinimumIncluded());
    assertEquals(isOptional, column.isOptional());
  }

  /**
   * Testando criar uma coluna de reais com mnimo maior do que o mximo.
   */
  public void testDoubleColumn8() {
    String id = "Identificador do parmetro";
    String label = "Nome do parmetro";
    Double defaultValue = new Double(1234.56789);
    boolean isEditable = true;
    boolean isOptional = true;
    Double maximum = new Double(1.1);
    boolean includeMaximum = false;
    Double minimum = new Double(10.01);
    boolean includeMinimum = true;
    try {
      new DoubleColumn(id, label, defaultValue, isOptional, isEditable,
        maximum, includeMaximum, minimum, includeMinimum);
      fail("No deveria ser possvel criar um parmetro real com minmo maior do que o mximo.");
    } catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando criar uma coluna de reais sem rtulo.
   */
  public void testDoubleColumn9() {
    String id = "Identificador do parmetro";
    Double defaultValue = new Double(1234.56789);
    boolean isOptional = true;
    boolean isEditable = true;
    Double maximum = new Double(1.1);
    boolean includeMaximum = false;
    Double minimum = new Double(10.01);
    boolean includeMinimum = true;
    try {
      new DoubleColumn(id, null, defaultValue, isOptional, isEditable, maximum,
        includeMaximum, minimum, includeMinimum);
      fail("No deveria ser possvel criar um parmetro real sem um nome.");
    } catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando criar uma coluna de reais no-editvel.
   */
  public void testDoubleColumn10() {
    String id = "Identificador do parmetro";
    String label = "Nome do parmetro";
    boolean isEditable = false;
    boolean isOptional = true;
    Double maximum = new Double(10.10);
    boolean includeMaximum = true;
    Double minimum = new Double(1.1);
    boolean includeMinimum = true;
    Double defaultValue = new Double(123.456);
    DoubleColumn column = new DoubleColumn(id, label, defaultValue, isOptional,
      isEditable, maximum, includeMaximum, minimum, includeMinimum);
    assertEquals(id, column.getId());
    assertEquals(label, column.getLabel());
    assertEquals(defaultValue, column.getDefaultValue(0));
    assertEquals(isOptional, column.isOptional());
    assertEquals(isEditable, column.isEditable());
    assertEquals(maximum, column.getMaximum());
    assertEquals(includeMaximum, column.isMaximumIncluded());
    assertEquals(minimum, column.getMinimum());
    assertEquals(includeMinimum, column.isMinimumIncluded());
  }

  /**
   * Testando criar uma coluna de reais sem rtulo.
   */
  public void testDoubleColumn11() {
    String label = "Rtulo do parmetro";
    Double defaultValue = new Double(1234.56789);
    boolean isOptional = true;
    boolean isEditable = true;
    Double maximum = new Double(1.1);
    boolean includeMaximum = false;
    Double minimum = new Double(10.01);
    boolean includeMinimum = true;
    try {
      new DoubleColumn(null, label, defaultValue, isOptional, isEditable,
        maximum, includeMaximum, minimum, includeMinimum);
      fail("No deveria ser possvel criar um parmetro real sem um identificador.");
    } catch (IllegalArgumentException e) {
    }
  }
  
  /**
   * Testando serializar e deserializar uma coluna de reais.
   * 
   * @throws IOException Bug.
   * @throws ClassNotFoundException Bug.
   */
  public void testSerialize() throws IOException, ClassNotFoundException {
    String id = "Identificador do parmetro";
    String label = "Nome do parmetro";
    boolean isOptional = true;
    boolean isEditable = true;
    Double maximum = new Double(10.10);
    boolean includeMaximum = true;
    Double minimum = new Double(1.1);
    boolean includeMinimum = true;
    Double defaultValue = new Double(1234.56789);
    DoubleColumn outputColumn = new DoubleColumn(id, label, defaultValue,
      isOptional, isEditable, maximum, includeMaximum, minimum, includeMinimum);
    ByteArrayOutputStream outputSource = new ByteArrayOutputStream();
    ObjectOutputStream objectOutput = new ObjectOutputStream(outputSource);
    objectOutput.writeObject(outputColumn);
    ByteArrayInputStream inputSource = new ByteArrayInputStream(outputSource
      .toByteArray());
    ObjectInputStream objectInput = new ObjectInputStream(inputSource);
    DoubleColumn inputColumn = (DoubleColumn) objectInput.readObject();
    assertEquals(outputColumn.getId(), inputColumn.getId());
    assertEquals(outputColumn.getLabel(), inputColumn.getLabel());
    assertEquals(outputColumn.getDefaultValue(0), inputColumn
      .getDefaultValue(0));
    assertEquals(outputColumn.isOptional(), inputColumn.isOptional());
    assertEquals(outputColumn.isEditable(), inputColumn.isEditable());
    assertEquals(outputColumn.getMaximum(), inputColumn.getMaximum());
    assertEquals(outputColumn.isMaximumIncluded(), inputColumn
      .isMaximumIncluded());
    assertEquals(outputColumn.getMinimum(), inputColumn.getMinimum());
    assertEquals(outputColumn.isMinimumIncluded(), inputColumn
      .isMinimumIncluded());
  }
}
