/*
 * $Author:$
 * $Date:$
 * $Release:$
 */
package csbase.logic.algorithms.parameters;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import junit.framework.TestCase;
import csbase.exception.ParseException;

public final class DoubleParameterTest extends TestCase {
  public void testDoubleParameter1() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    Double defaultValue = new Double(5.5);
    boolean isOptional = true;
    String commandLinePattern = "--{0} {1}";
    Double maximum = new Double(10.10);
    boolean includeMaximum = true;
    Double minimum = new Double(1.1);
    boolean includeMinimum = true;
    DoubleParameter parameter = new DoubleParameter(name, label, description,
      defaultValue, isOptional, commandLinePattern, maximum, includeMaximum,
      minimum, includeMinimum);
    assertEquals(name, label, description, defaultValue, isOptional, commandLinePattern, maximum,
      includeMaximum, minimum, includeMinimum, parameter);
  }

  public void testDoubleParameter2() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    Double defaultValue = new Double(5.5);
    boolean isOptional = false;
    String commandLinePattern = "--{0} {1}";
    Double maximum = new Double(10.10);
    boolean includeMaximum = true;
    Double minimum = new Double(1.1);
    boolean includeMinimum = true;
    DoubleParameter parameter = new DoubleParameter(name, label, description,
      defaultValue, isOptional, commandLinePattern, maximum, includeMaximum,
      minimum, includeMinimum);
    assertEquals(name, label, description, defaultValue, isOptional,
      commandLinePattern, maximum, includeMaximum, minimum, includeMinimum,
      parameter);
  }

  public void testDoubleParameter3() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    Double defaultValue = new Double(5.5);
    boolean isOptional = true;
    String commandLinePattern = "--{0} {1}";
    Double maximum = new Double(10.10);
    boolean includeMaximum = false;
    Double minimum = new Double(1.1);
    boolean includeMinimum = true;
    DoubleParameter parameter =
      new DoubleParameter(name, label, description,
      defaultValue, isOptional, commandLinePattern, maximum, includeMaximum,
      minimum, includeMinimum);
    assertEquals(name, label, description, defaultValue, isOptional,
      commandLinePattern, maximum, includeMaximum, minimum, includeMinimum,
      parameter);
  }

  public void testDoubleParameter4() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    Double defaultValue = new Double(5.5);
    boolean isOptional = true;
    String commandLinePattern = "--{0} {1}";
    Double maximum = new Double(10.10);
    boolean includeMaximum = true;
    Double minimum = new Double(1.1);
    boolean includeMinimum = false;
    DoubleParameter parameter =
      new DoubleParameter(name, label, description,
      defaultValue, isOptional, commandLinePattern, maximum, includeMaximum,
      minimum, includeMinimum);
    assertEquals(name, label, description, defaultValue, isOptional,
      commandLinePattern, maximum, includeMaximum, minimum, includeMinimum,
      parameter);
  }

  public void testDoubleParameter5() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    Double defaultValue = new Double(5.5);
    boolean isOptional = true;
    String commandLinePattern = "--{0} {1}";
    Double maximum = null;
    boolean includeMaximum = false;
    Double minimum = new Double(1.1);
    boolean includeMinimum = true;
    DoubleParameter parameter =
      new DoubleParameter(name, label, description,
      defaultValue, isOptional, commandLinePattern, maximum, includeMaximum,
      minimum, includeMinimum);
    assertEquals(name, label, description, defaultValue, isOptional,
      commandLinePattern, maximum, includeMaximum, minimum, includeMinimum,
      parameter);
  }

  public void testDoubleParameter6() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    Double defaultValue = new Double(5.5);
    boolean isOptional = true;
    String commandLinePattern = "--{0} {1}";
    Double maximum = new Double(10.10);
    boolean includeMaximum = true;
    Double minimum = null;
    boolean includeMinimum = false;
    DoubleParameter parameter =
      new DoubleParameter(name, label, description,
      defaultValue, isOptional, commandLinePattern, maximum, includeMaximum,
      minimum, includeMinimum);
    assertEquals(name, label, description, defaultValue, isOptional,
      commandLinePattern, maximum, includeMaximum, minimum, includeMinimum,
      parameter);

    
  }

  public void testDoubleParameter7() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    Double defaultValue = new Double(5.5);
    boolean isOptional = true;
    String commandLinePattern = "--{0} {1}";
    Double maximum = new Double(1.1);
    boolean includeMaximum = false;
    Double minimum = new Double(10.01);
    boolean includeMinimum = true;
    try {
      new DoubleParameter(name, label, description, defaultValue, isOptional,
        commandLinePattern, maximum, includeMaximum, minimum, includeMinimum);
      fail(
        "No deveria ser possvel criar um parmetro real com minmo maior " +
        "do que o mximo.");
    }
    catch (IllegalArgumentException e) {}
  }

  public void testDoubleParameter8() {
    String name = null;
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    Double defaultValue = new Double(5.5);
    boolean isOptional = true;
    String commandLinePattern = "--{0} {1}";
    Double maximum = new Double(10.1);
    boolean includeMaximum = false;
    Double minimum = new Double(1.01);
    boolean includeMinimum = true;
    try {
      new DoubleParameter(name, label, description, defaultValue, isOptional,
        commandLinePattern, maximum, includeMaximum, minimum, includeMinimum);
      fail("No deveria ser possvel criar um parmetro real sem um nome.");
    }
    catch (IllegalArgumentException e) {}
  }

  public void testDoubleParameter9() {
    String name = "Nome do parmetro";
    String label = null;
    String description = "Descrio do parmetro";
    Double defaultValue = new Double(5.5);
    boolean isOptional = true;
    String commandLinePattern = "--{0} {1}";
    Double maximum = new Double(10.1);
    boolean includeMaximum = false;
    Double minimum = new Double(1.01);
    boolean includeMinimum = true;
    try {
      new DoubleParameter(name, label, description, defaultValue, isOptional,
        commandLinePattern, maximum, includeMaximum, minimum, includeMinimum);
      fail("No deveria ser possvel criar um parmetro real sem um rtulo.");
    }
    catch (IllegalArgumentException e) {}
  }

  public void testDoubleParameter10() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = null;
    Double defaultValue = new Double(5.5);
    boolean isOptional = true;
    String commandLinePattern = "--{0} {1}";
    Double maximum = new Double(10.1);
    boolean includeMaximum = false;
    Double minimum = new Double(1.01);
    boolean includeMinimum = true;
    try {
      new DoubleParameter(name, label, description, defaultValue, isOptional,
        commandLinePattern, maximum, includeMaximum, minimum, includeMinimum);
      fail(
        "No deveria ser possvel criar um parmetro real sem uma descrio.");
    }
    catch (IllegalArgumentException e) {}
  }

  public void testDoubleParameter11() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    Double defaultValue = null;
    boolean isOptional = true;
    String commandLinePattern = "--{0} {1}";
    Double maximum = new Double(10.10);
    boolean includeMaximum = true;
    Double minimum = new Double(1.1);
    boolean includeMinimum = true;
    DoubleParameter parameter = new DoubleParameter(name, label, description,
      defaultValue, isOptional, commandLinePattern, maximum, includeMaximum,
      minimum, includeMinimum);
    assertEquals(name, label, description, defaultValue, isOptional,
      commandLinePattern, maximum, includeMaximum, minimum, includeMinimum,
      parameter);
  }

  public void testSetValueAsText() throws ParseException{
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    DoubleParameter parameter =
      new DoubleParameter(
        name, label, description, null, false, null, null, false, null, false);
    Double value = new Double(1.2e4);
    parameter.setValueAsText(value.toString());
    assertEquals(value.toString(), parameter.getValueAsText());
    assertEquals(value, parameter.getValue());
  }

  public void testSetValueAsTextWithInvalidValue() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    DoubleParameter parameter =
      new DoubleParameter(
        name, label, description, null, false, null, null, false, null, false);
    try {
      parameter.setValueAsText("Valor invlido.");
      fail("Esperada ParseException");
    }
    catch (ParseException e) {}
  }

  public void testSetValueAsTextWithNull() throws ParseException{
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    DoubleParameter parameter = new DoubleParameter(name, label, description,
      new Double(10), false, null, null, false, null, false);
    parameter.setValueAsText(null);
    assertNull(parameter.getValue());
    assertNull(parameter.getValueAsText());
  }

  public void testSerialize()
  throws IOException, ClassNotFoundException {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    Double defaultValue = new Double(5.5);
    boolean isOptional = true;
    String commandLinePattern = "--{0} {1}";
    Double maximum = new Double(10.10);
    boolean includeMaximum = true;
    Double minimum = new Double(1.1);
    boolean includeMinimum = true;
    DoubleParameter outputParameter = new DoubleParameter(name, label,
      description, defaultValue, isOptional, commandLinePattern, maximum,
      includeMaximum, minimum, includeMinimum);
    ByteArrayOutputStream outputSource = new ByteArrayOutputStream();
    ObjectOutputStream objectOutput = new ObjectOutputStream(outputSource);
    objectOutput.writeObject(outputParameter);
    ByteArrayInputStream inputSource =
      new ByteArrayInputStream(outputSource.toByteArray());
    ObjectInputStream objectInput = new ObjectInputStream(inputSource);
    DoubleParameter inputParameter = (DoubleParameter)objectInput.readObject();
    assertEquals(outputParameter.getName(), outputParameter.getLabel(),
      outputParameter.getDescription(), outputParameter.getDefaultValue(),
      outputParameter.isOptional(), outputParameter.getCommandLinePattern(),
      outputParameter.getMaximum(), outputParameter.isMaximumIncluded(),
      outputParameter.getMinimum(), outputParameter.isMinimumIncluded(),
      inputParameter);
  }

  /**
   * Verifica se os atributos fornecidos so iguais aos atributos do parmetro.
   * 
   * @param name O nome (Aceita {@code null}). 
   * @param label O rtulo (Aceita {@code null}).
   * @param description A descrio (Aceita {@code null}).
   * @param defaultValue O valor-padro (Aceita {@code null}).
   * @param isOptional Indica se  opcional/obrigatrio.
   * @param commandLinePattern O padro da linha de comando (Aceita {@code null}).
   * @param maximum O mximo (Aceita {@code null}).
   * @param includeMaximum Indica se o valor mximo  considerado vlido.
   * @param minimum O mnimo (Aceita {@code null}).
   * @param includeMinimum Indica se o valor mnimo  considerado vlido.
   * @param parameter O parmetro (No aceita {@code null}).
   */
  private void assertEquals(String name, String label, String description,
    Double defaultValue, boolean isOptional, String commandLinePattern,
    Double maximum, boolean includeMaximum, Double minimum,
    boolean includeMinimum, DoubleParameter parameter) {
    assertEquals(name, parameter.getName());
    assertEquals(label, parameter.getLabel());
    assertEquals(description, parameter.getDescription());
    assertEquals(defaultValue, parameter.getDefaultValue());
    assertEquals(isOptional, parameter.isOptional());
    assertEquals(commandLinePattern, parameter.getCommandLinePattern());
    assertEquals(maximum, parameter.getMaximum());
    assertEquals(includeMaximum, parameter.isMaximumIncluded());
    assertEquals(minimum, parameter.getMinimum());
    assertEquals(includeMinimum, parameter.isMinimumIncluded());
  }
}
