package csbase.logic.algorithms.parameters;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import junit.framework.TestCase;

public final class EnumerationItemTest extends TestCase {
  public void testEnumerationItem() {
    String id = "item 1";
    String label = "Enumeration's item's label";
    String value = "Enumeration's item's value";
    String description = "Description";
    EnumerationItem item = new EnumerationItem(id, label, value, description);
    assertEquals(id, item.getId());
    assertNotNull(item.getLabel());
    assertEquals(label, item.getLabel());
    assertNotNull(item.getValue());
    assertEquals(value, item.getValue());
    assertEquals(description, item.getDescription());
  }

  public void testEnumerationItemWithNullDescription() {
    String id = "item 1";
    String label = "Enumeration's item's label";
    String value = "Enumeration's item's value";
    String description = null;
    EnumerationItem item = new EnumerationItem(id, label, value, description);
    assertEquals(id, item.getId());
    assertNotNull(item.getLabel());
    assertEquals(label, item.getLabel());
    assertNotNull(item.getValue());
    assertEquals(value, item.getValue());
    assertEquals(description, item.getDescription());
  }

  public void testEnumerationItemWithIdNull() {
    String id = null;
    String value = "Enumeration's item's value";
    String description = "Description";
    try {
      new EnumerationItem(id, null, value, description);
      fail("Todo item de enumerao tem que ter um nome.");
    } catch (IllegalArgumentException e) {
    }
  }

  public void testEnumerationItemWithNameNull() {
    String id = "item 1";
    String value = "Enumeration's item's value";
    String description = "Description";
    try {
      new EnumerationItem(id, null, value, description);
      fail("Todo item de enumerao tem que ter um nome.");
    } catch (IllegalArgumentException e) {
    }
  }

  public void testEnumerationItemWithValueNull() {
    String id = "item 1";
    String label = "Enumeration's item's label";
    try {
      new EnumerationItem(id, label, null, null);
      fail("Todo item de enumerao tem que ter um valor.");
    } catch (IllegalArgumentException e) {
    }
  }

  public void testEqualsObjectAllEquals() {
    String id = "item 1";
    String label = "Enumeration's item's label";
    String value = "Enumeration's item's value";
    String description = "Description";
    EnumerationItem item = new EnumerationItem(id, label, value, description);
    EnumerationItem allEqualsItem = new EnumerationItem(id, label, value,
      description);
    assertEquals(item, allEqualsItem);
  }

  public void testEqualsObjectDifferentId() {
    String id1 = "1";
    String label = "Enumeration's item's label";
    String value = "Enumeration's item's value";
    String description = "Description";
    EnumerationItem item = new EnumerationItem(id1, label, value, description);
    String id2 = "2";
    EnumerationItem differentNameItem = new EnumerationItem(id2, label, value,
      description);
    assertFalse("Itens de enumerao com nomes diferentes so diferentes.",
      item.equals(differentNameItem));
  }

  public void testEqualsObjectDifferentValues() {
    String id = "item 1";
    String label = "Enumeration's item's label";
    String value = "Enumeration's item's value";
    String description = "Description";
    EnumerationItem item = new EnumerationItem(id, label, value, description);
    String anotherLabel = "Another enumeration's item's label";
    String anotherValue = "Another enumeration's item's value";
    String anotherDescription = "Another description";
    EnumerationItem differentValueItem = new EnumerationItem(id, anotherLabel,
      anotherValue, anotherDescription);
    assertEquals(item, differentValueItem);
  }

  public void testEqualsObjectWithObjectNull() {
    String id = "item 1";
    String label = "Enumeration's item's label";
    String value = "Enumeration's item's value";
    String description = "Description";
    EnumerationItem item = new EnumerationItem(id, label, value, description);
    assertFalse("O item de enumeracao no pode ser igual a null.", item
      .equals(null));
  }

  public void testEqualsObjectWithObjectAnotherClass() {
    String id = "item 1";
    String label = "Enumeration's item's label";
    String value = "Enumeration's item's value";
    String description = "Description";
    EnumerationItem item = new EnumerationItem(id, label, value, description);
    String anotherClassObject = "Another class object";
    assertFalse(
      "O item de enumeracao no pode ser igual a um objeto de outra classe.",
      item.equals(anotherClassObject));
  }

  public void testHashCode() {
    String id = "item 1";
    String label = "Enumeration's item's label";
    String value = "Enumeration's item's value";
    String description = "Description";
    EnumerationItem item = new EnumerationItem(id, label, value, description);
    String anotherLabel = "Another enumeration's item's label";
    String anotherValue = "Another enumeration's item's value";
    String anotherDescription = "Another description";
    EnumerationItem equalsItem = new EnumerationItem(id, anotherLabel,
      anotherValue, anotherDescription);
    assertEquals(item.hashCode(), equalsItem.hashCode());
  }

  public void testToString() {
    String id = "item 1";
    String label = "Enumeration's item's label";
    String value = "Enumeration's item's value";
    String description = "Description";
    EnumerationItem item = new EnumerationItem(id, label, value, description);
    assertNotNull(item.toString());
    assertEquals(label, item.toString());
  }

  public void testCompareToEquals() {
    String id = "item 1";
    String label = "Enumeration's item's label";
    String value = "Enumeration's item's value";
    String description = "Description";
    EnumerationItem item = new EnumerationItem(id, label, value, description);
    String anotherValue = "Another enumeration's item's value";
    String anotherDescription = "Another description";
    EnumerationItem equalsItem = new EnumerationItem(id, label, anotherValue,
      anotherDescription);
    assertEquals(0, item.compareTo(equalsItem));
  }

  public void testCompareToGreaterThan() {
    String id = "item 1";
    String label = "B: Enumeration's item's label";
    String value = "Enumeration's item's value";
    String description = "Description";
    EnumerationItem item = new EnumerationItem(id, label, value, description);
    String anotherLabel = "A: Enumeration's item's label";
    EnumerationItem theLestItem = new EnumerationItem(id, anotherLabel,
      value, description);
    assertTrue(item.compareTo(theLestItem) > 0);
  }

  public void testCompareToLesserThan() {
    String id = "item 1";
    String label = "A: Enumeration's item's label";
    String value = "Enumeration's item's value";
    String description = "Description";
    EnumerationItem item = new EnumerationItem(id, label, value, description);
    String anotherLabel = "B: Enumeration's item's label";
    EnumerationItem theGreatestItem = new EnumerationItem(id, anotherLabel,
      value, description);
    assertTrue(item.compareTo(theGreatestItem) < 0);
  }

  public void testCompareToWithNull() {
    String id = "item 1";
    String label = "A: Enumeration's item's label";
    String value = "Enumeration's item's value";
    String description = "Description";
    EnumerationItem item = new EnumerationItem(id, label, value, description);
    try {
      item.compareTo(null);
      fail("Deveria lanar um " + NullPointerException.class.getName());
    } catch (NullPointerException e) {
    }
  }

  public void testSerialize() throws IOException, ClassNotFoundException {
    String id = "item 1";
    String label = "Nome do item";
    String value = "Valor do item";
    String description = "Description";
    EnumerationItem outputItem = new EnumerationItem(id, label, value,
      description);
    ByteArrayOutputStream outputSource = new ByteArrayOutputStream();
    ObjectOutputStream objectOutput = new ObjectOutputStream(outputSource);
    objectOutput.writeObject(outputItem);
    ByteArrayInputStream inputSource = new ByteArrayInputStream(outputSource
      .toByteArray());
    ObjectInputStream objectInput = new ObjectInputStream(inputSource);
    EnumerationItem inputItem = (EnumerationItem) objectInput.readObject();
    assertEquals(outputItem.getLabel(), inputItem.getLabel());
    assertEquals(outputItem.getValue(), inputItem.getValue());
    assertEquals(outputItem.getDescription(), inputItem.getDescription());
  }
}
