package csbase.logic.algorithms.parameters;

import junit.framework.TestCase;

public final class HiddenParameterTest extends TestCase {

  public void testHiddenParameter1() {
    String name = "NOME";
    String value = "VALOR";
    String commandLinePattern = "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO";
    HiddenParameter parameter = new HiddenParameter(name, value, commandLinePattern);
    assertEquals(name, parameter.getName());
    assertEquals(value, parameter.getValue());
    assertEquals(commandLinePattern, parameter.getCommandLinePattern());
  }
  
  public void testHiddenParameter2() {
    try {
      new HiddenParameter(null, "VALOR", "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO");  
      fail("Esperada IllegalArgumentException.");
    } catch (IllegalArgumentException e) {
    }
  }
  
  public void testHiddenParameter3() {
    try {
      new HiddenParameter("NOME", null, "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO");  
      fail("Esperada IllegalArgumentException.");
    } catch (IllegalArgumentException e) {
    }
  }

  public void testHiddenParameter4() {
    try {
      new HiddenParameter("NOME", "VALOR", null);  
      fail("Esperada IllegalArgumentException.");
    } catch (IllegalArgumentException e) {
    }  }

  public void testGetCommandValue1() {
    String name = "nome_do_parametro";
    String value = "valor_do_parametro";
    String commandLinePattern = "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO";
    HiddenParameter parameter = new HiddenParameter(name, value,
      commandLinePattern);
    assertEquals(name+"="+value, parameter.getCommandLine());
  }
  
  public void testGetCommandValue2() {
    String name = "nome_do_parametro";
    String value = "valor_do_parametro";
    String commandLinePattern = "--$NOME_DO_PARAMETRO";
    HiddenParameter parameter = new HiddenParameter(name, value,
      commandLinePattern);
    assertEquals("--"+name, parameter.getCommandLine());
  }
  
  public void testGetCommandValue3() {
    String name = "nome_do_parametro";
    String value = "valor_do_parametro";
    String commandLinePattern = "$VALOR_DO_PARAMETRO";
    HiddenParameter parameter = new HiddenParameter(name, value,
      commandLinePattern);
    assertEquals(value, parameter.getCommandLine());
  }
}
