/*
 * $Author:$
 * $Date:$
 * $Release:$
 */
package csbase.logic.algorithms.parameters;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import junit.framework.TestCase;

/**
 * Testes unitrios para a classe {@link IntegerListParameter}.
 *
 * @author lmoreira
 */
public final class IntegerListParameterTest extends TestCase {
  /**
   * Testando criar parmetros lista de inteiros opcionais.
   */
  public void testIntegerListParameter1() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = true;
    String commandLinePattern = "--{0} {1}";
    Integer maximum = new Integer(100);
    Integer minimum = new Integer(0);
    IntegerListParameter parameter = new IntegerListParameter(name, label,
      description, isOptional, commandLinePattern, maximum, minimum);
    assertEquals(name, parameter.getName());
    assertEquals(label, parameter.getLabel());
    assertEquals(description, parameter.getDescription());
    assertEquals(isOptional, parameter.isOptional());
    assertEquals(maximum, parameter.getMaximum());
    assertEquals(minimum, parameter.getMinimum());
    assertNull(parameter.getDefaultValue());
  }

  /**
   * Testando criar parmetros lista de inteiros obrigatrios.
   */
  public void testIntegerListParameter2() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = false;
    String commandLinePattern = "--{0} {1}";
    Integer maximum = new Integer(100);
    Integer minimum = new Integer(0);
    IntegerListParameter parameter = new IntegerListParameter(name, label,
      description, isOptional, commandLinePattern, maximum, minimum);
    assertEquals(name, parameter.getName());
    assertEquals(label, parameter.getLabel());
    assertEquals(description, parameter.getDescription());
    assertEquals(isOptional, parameter.isOptional());
    assertEquals(maximum, parameter.getMaximum());
    assertEquals(minimum, parameter.getMinimum());
    assertNull(parameter.getDefaultValue());
  }

  /**
   * Testando criar parmetros lista de inteiros com mximo.
   */
  public void testIntegerListParameter3() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = false;
    String commandLinePattern = "--{0} {1}";
    Integer maximum = null;
    Integer minimum = new Integer(0);
    IntegerListParameter parameter = new IntegerListParameter(name, label,
      description, isOptional, commandLinePattern, maximum, minimum);
    assertEquals(name, parameter.getName());
    assertEquals(label, parameter.getLabel());
    assertEquals(description, parameter.getDescription());
    assertEquals(isOptional, parameter.isOptional());
    assertNull(parameter.getMaximum());
    assertEquals(minimum, parameter.getMinimum());
    assertNull(parameter.getDefaultValue());
  }

  /**
   * Testando criar parmetros lista de inteiros com mnimo.
   */
  public void testIntegerListParameter4() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = false;
    String commandLinePattern = "--{0} {1}";
    Integer maximum = new Integer(100);
    Integer minimum = null;
    IntegerListParameter parameter = new IntegerListParameter(name, label,
      description, isOptional, commandLinePattern, maximum, minimum);
    assertEquals(name, parameter.getName());
    assertEquals(label, parameter.getLabel());
    assertEquals(description, parameter.getDescription());
    assertEquals(isOptional, parameter.isOptional());
    assertEquals(maximum, parameter.getMaximum());
    assertNull(parameter.getMinimum());
    assertNull(parameter.getDefaultValue());
  }
  /**
   * Testando criar parmetros lista de inteiros sem nome.
   */
  public void testIntegerListParameter5() {
    String name = null;
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = false;
    String commandLinePattern = "--{0} {1}";
    Integer maximum = null;
    Integer minimum = null;
    try {
      new IntegerListParameter(name, label, description, isOptional,
        commandLinePattern, maximum, minimum);
      fail("No deveria ser possvel criar um parmetro sem nome.");
    }
    catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando criar parmetros lista de inteiros sem rtulo.
   */
  public void testIntegerListParameter6() {
    String name = "Nome do parmetro";
    String label = null;
    String description = "Descrio do parmetro";
    boolean isOptional = false;
    String commandLinePattern = "--{0} {1}";
    Integer maximum = null;
    Integer minimum = null;
    try {
      new IntegerListParameter(name, label, description, isOptional,
        commandLinePattern, maximum, minimum);
      fail("No deveria ser possvel criar um parmetro sem rtulo.");
    }
    catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando criar parmetros lista de inteiros sem descrio.
   */
  public void testIntegerListParameter7() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = null;
    boolean isOptional = false;
    String commandLinePattern = "--{0} {1}";
    Integer maximum = null;
    Integer minimum = null;
    try {
      new IntegerListParameter(name, label, description, isOptional,
        commandLinePattern, maximum, minimum);
      fail("No deveria ser possvel criar um parmetro sem descrio.");
    }
    catch (IllegalArgumentException e) {
    }
  }
  
  /**
   * Testando o mtodo {@link IntegerListParameter#setMaximum(Integer)} com um
   * mximo diferente do mximo anterior.
   */
  public void testSetMaximum1() {
    CheckIntegerListParameterListener checkListener = new CheckIntegerListParameterListener();
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = false;
    String commandLinePattern = "--{0} {1}";
    Integer maximum = 1000;
    Integer minimum = 0;
    IntegerListParameter IntegerListParameter = new IntegerListParameter(name,
      label, description, isOptional, commandLinePattern, maximum, minimum);
    IntegerListParameter.addIntegerListParameterListener(checkListener);
    checkListener.allowMaximumWasChangedEvent();
    Integer newMaximum = 2000;
    assertTrue(IntegerListParameter.setMaximum(newMaximum));
    assertEquals(1, checkListener.getEventCount());
    assertEquals(newMaximum, IntegerListParameter.getMaximum());
  }

  /**
   * Testando o mtodo {@link IntegerListParameter#setMaximum(Integer)} com um
   * mximo e sem mximo anteriormente.
   */
  public void testSetMaximum2() {
    CheckIntegerListParameterListener checkListener = new CheckIntegerListParameterListener();
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = false;
    String commandLinePattern = "--{0} {1}";
    Integer maximum = null;
    Integer minimum = 0;
    IntegerListParameter IntegerListParameter = new IntegerListParameter(name,
      label, description, isOptional, commandLinePattern, maximum, minimum);
    IntegerListParameter.addIntegerListParameterListener(checkListener);
    checkListener.allowMaximumWasChangedEvent();
    Integer newMaximum = 2000;
    assertTrue(IntegerListParameter.setMaximum(newMaximum));
    assertEquals(1, checkListener.getEventCount());
    assertEquals(newMaximum, IntegerListParameter.getMaximum());
  }

  /**
   * Testando o mtodo {@link IntegerListParameter#setMaximum(Integer)} sem um
   * mximo e com um mximo anterior.
   */
  public void testSetMaximum3() {
    CheckIntegerListParameterListener checkListener = new CheckIntegerListParameterListener();
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    String commandLinePattern = "--{0} {1}";
    boolean isOptional = false;
    Integer maximum = 1000;
    Integer minimum = 0;
    IntegerListParameter IntegerListParameter = new IntegerListParameter(name,
      label, description, isOptional, commandLinePattern, maximum, minimum);
    IntegerListParameter.addIntegerListParameterListener(checkListener);
    checkListener.allowMaximumWasChangedEvent();
    Integer newMaximum = null;
    assertTrue(IntegerListParameter.setMaximum(newMaximum));
    assertEquals(1, checkListener.getEventCount());
    assertEquals(newMaximum, IntegerListParameter.getMaximum());
  }

  /**
   * Testando o mtodo {@link IntegerListParameter#setMaximum(Integer)} com um
   * mximo igual ao anterior.
   */
  public void testSetMaximum4() {
    CheckIntegerListParameterListener checkListener = new CheckIntegerListParameterListener();
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    String commandLinePattern = "--{0} {1}";
    boolean isOptional = false;
    Integer maximum = 1000;
    Integer minimum = 0;
    IntegerListParameter IntegerListParameter = new IntegerListParameter(name,
      label, description, isOptional, commandLinePattern, maximum, minimum);
    IntegerListParameter.addIntegerListParameterListener(checkListener);
    Integer newMaximum = maximum;
    assertFalse(IntegerListParameter.setMaximum(newMaximum));
    assertEquals(0, checkListener.getEventCount());
    assertEquals(newMaximum, IntegerListParameter.getMaximum());
  }

  /**
   * Testando o mtodo {@link IntegerListParameter#setMaximum(Integer)} com um
   * mximo atual e anterior {@code null}.
   */
  public void testSetMaximum5() {
    CheckIntegerListParameterListener checkListener = new CheckIntegerListParameterListener();
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    String commandLinePattern = "--{0} {1}";
    boolean isOptional = false;
    Integer maximum = null;
    Integer minimum = 0;
    IntegerListParameter IntegerListParameter = new IntegerListParameter(name,
      label, description, isOptional, commandLinePattern, maximum, minimum);
    IntegerListParameter.addIntegerListParameterListener(checkListener);
    Integer newMaximum = maximum;
    assertFalse(IntegerListParameter.setMaximum(newMaximum));
    assertEquals(0, checkListener.getEventCount());
    assertEquals(newMaximum, IntegerListParameter.getMaximum());
  }

  /**
   * Testando o mtodo {@link IntegerListParameter#setMinimum(Integer)} com um
   * mnimo diferente do mnimo anterior.
   */
  public void testSetMinimum1() {
    CheckIntegerListParameterListener checkListener = new CheckIntegerListParameterListener();
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    String commandLinePattern = "--{0} {1}";
    boolean isOptional = false;
    Integer maximum = 1000;
    Integer minimum = -1000;
    IntegerListParameter IntegerListParameter = new IntegerListParameter(name,
      label, description, isOptional, commandLinePattern, maximum, minimum);
    IntegerListParameter.addIntegerListParameterListener(checkListener);
    checkListener.allowMinimumWasChangedEvent();
    Integer newMinimum = -2000;
    assertTrue(IntegerListParameter.setMinimum(newMinimum));
    assertEquals(1, checkListener.getEventCount());
    assertEquals(newMinimum, IntegerListParameter.getMinimum());
  }

  /**
   * Testando o mtodo {@link IntegerListParameter#setMinimum(Integer)} com um
   * mnimo e sem mnimo anteriormente.
   */
  public void testSetMinimum2() {
    CheckIntegerListParameterListener checkListener = new CheckIntegerListParameterListener();
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    String commandLinePattern = "--{0} {1}";
    boolean isOptional = false;
    Integer maximum = 1000;
    Integer minimum = null;
    IntegerListParameter IntegerListParameter = new IntegerListParameter(name,
      label, description, isOptional, commandLinePattern, maximum, minimum);
    IntegerListParameter.addIntegerListParameterListener(checkListener);
    checkListener.allowMinimumWasChangedEvent();
    Integer newMinimum = -2000;
    assertTrue(IntegerListParameter.setMinimum(newMinimum));
    assertEquals(1, checkListener.getEventCount());
    assertEquals(newMinimum, IntegerListParameter.getMinimum());
  }

  /**
   * Testando o mtodo {@link IntegerListParameter#setMinimum(Integer)} sem um
   * mnimo e com um mnimo anterior.
   */
  public void testSetMinimum3() {
    CheckIntegerListParameterListener checkListener = new CheckIntegerListParameterListener();
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    String commandLinePattern = "--{0} {1}";
    boolean isOptional = false;
    Integer maximum = 1000;
    Integer minimum = -1000;
    IntegerListParameter IntegerListParameter = new IntegerListParameter(name,
      label, description, isOptional, commandLinePattern, maximum, minimum);
    IntegerListParameter.addIntegerListParameterListener(checkListener);
    checkListener.allowMinimumWasChangedEvent();
    Integer newMinimum = null;
    assertTrue(IntegerListParameter.setMinimum(newMinimum));
    assertEquals(1, checkListener.getEventCount());
    assertEquals(newMinimum, IntegerListParameter.getMinimum());
  }

  /**
   * Testando o mtodo {@link IntegerListParameter#setMinimum(Integer)} com um
   * mnimo igual ao anterior.
   */
  public void testSetMinimum4() {
    CheckIntegerListParameterListener checkListener = new CheckIntegerListParameterListener();
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = false;
    String commandLinePattern = "--{0} {1}";
    Integer maximum = 1000;
    Integer minimum = 0;
    IntegerListParameter integerListParameter = new IntegerListParameter(name,
      label, description, isOptional, commandLinePattern, maximum, minimum);
    integerListParameter.addIntegerListParameterListener(checkListener);
    Integer newMinimum = minimum;
    assertFalse(integerListParameter.setMinimum(newMinimum));
    assertEquals(0, checkListener.getEventCount());
    assertEquals(newMinimum, integerListParameter.getMinimum());
  }

  /**
   * Testando o mtodo {@link IntegerListParameter#setMinimum(Integer)} com um
   * mnimo atual e anterior {@code null}.
   */
  public void testSetMinimum5() {
    CheckIntegerListParameterListener checkListener = new CheckIntegerListParameterListener();
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = false;
    String commandLinePattern = "--{0} {1}";
    Integer maximum = null;
    Integer minimum = 0;
    IntegerListParameter integerListParameter = new IntegerListParameter(name,
      label, description, isOptional, commandLinePattern, maximum, minimum);
    integerListParameter.addIntegerListParameterListener(checkListener);
    Integer newMinimum = minimum;
    assertFalse(integerListParameter.setMinimum(newMinimum));
    assertEquals(0, checkListener.getEventCount());
    assertEquals(newMinimum, integerListParameter.getMinimum());
  }
  
  /**
   * Testando serializao.
   * 
   * @throws IOException Bug.
   * @throws ClassNotFoundException Bug.
   */
  public void testSerialize()
  throws IOException, ClassNotFoundException {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = true;
    String commandLinePattern = "--{0} {1}";
    Integer maximum = new Integer(100);
    Integer minimum = new Integer(0);
    IntegerListParameter outputParameter = new IntegerListParameter(name,
      label, description, isOptional, commandLinePattern, maximum, minimum);
    ByteArrayOutputStream outputSource = new ByteArrayOutputStream();
    ObjectOutputStream objectOutput = new ObjectOutputStream(outputSource);
    objectOutput.writeObject(outputParameter);
    ByteArrayInputStream inputSource =
      new ByteArrayInputStream(outputSource.toByteArray());
    ObjectInputStream objectInput = new ObjectInputStream(inputSource);
    IntegerListParameter inputParameter =
      (IntegerListParameter)objectInput.readObject();
    assertEquals(outputParameter.getName(), inputParameter.getName());
    assertEquals(outputParameter.getLabel(), inputParameter.getLabel());
    assertEquals(
      outputParameter.getDescription(), inputParameter.getDescription());
    assertEquals(
      outputParameter.getDefaultValue(), inputParameter.getDefaultValue());
    assertEquals(outputParameter.isOptional(), inputParameter.isOptional());
    assertEquals(outputParameter.getCommandLinePattern(), inputParameter
      .getCommandLinePattern());
    assertEquals(outputParameter.getMaximum(), inputParameter.getMaximum());
    assertEquals(outputParameter.getMinimum(), inputParameter.getMinimum());
  }
}
