package csbase.logic.algorithms.parameters;

import junit.framework.TestCase;

/**
 * Testes unitrios para a classe {@link OutputFileParameter}.
 * 
 * @author lmoreira
 */
public final class OutputFileParameterTest extends TestCase {
  /**
   * Testando o mtodo
   * {@link OutputFileParameter#OutputFileParameter(String, String, String, boolean, String, String, FileParameterMode, boolean, boolean)}
   * .
   */
  public void testOutputFileParameter01() {
    String name = "NOME_DO_PARAMETRO";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = true;
    String fileType = "Tipo de arquivo";
    FileParameterMode mode = FileParameterMode.REGULAR_FILE;
    FileParameterPipeAcceptance canUsePipe = FileParameterPipeAcceptance.TRUE;
    boolean mustForceExtension = false;
    String commandLinePattern = "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO";
    OutputFileParameter parameter =
      new OutputFileParameter(name, label, description, isOptional,
        commandLinePattern, fileType, mode, canUsePipe, mustForceExtension);
    assertEquals(name, label, description, isOptional, commandLinePattern,
      fileType, mode, canUsePipe, mustForceExtension, parameter);
  }

  /**
   * Testando o mtodo
   * {@link OutputFileParameter#OutputFileParameter(String, String, String, boolean, String, String, FileParameterMode, boolean, boolean)}
   * : o parmetro  obrigatrio.
   */
  public void testOutputFileParameter02() {
    String name = "NOME_DO_PARAMETRO";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = false;
    String fileType = "Tipo de arquivo";
    FileParameterMode mode = FileParameterMode.REGULAR_FILE;
    FileParameterPipeAcceptance canUsePipe = FileParameterPipeAcceptance.TRUE;
    boolean mustForceExtension = false;
    String commandLinePattern = "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO";
    OutputFileParameter parameter =
      new OutputFileParameter(name, label, description, isOptional,
        commandLinePattern, fileType, mode, canUsePipe, mustForceExtension);
    assertEquals(name, label, description, isOptional, commandLinePattern,
      fileType, mode, canUsePipe, mustForceExtension, parameter);
  }

  /**
   * Testando o mtodo
   * {@link OutputFileParameter#OutputFileParameter(String, String, String, boolean, String, String, FileParameterMode, boolean, boolean)}
   * : o padro para formao da linha de comando  {@code null}.
   */
  public void testOutputFileParameter03() {
    String name = "NOME_DO_PARAMETRO";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = false;
    String fileType = "Tipo de arquivo";
    FileParameterMode mode = FileParameterMode.REGULAR_FILE;
    FileParameterPipeAcceptance canUsePipe = FileParameterPipeAcceptance.TRUE;
    boolean mustForceExtension = false;
    String commandLinePattern = null;
    OutputFileParameter parameter =
      new OutputFileParameter(name, label, description, isOptional,
        commandLinePattern, fileType, mode, canUsePipe, mustForceExtension);
    assertEquals(name, label, description, isOptional, commandLinePattern,
      fileType, mode, canUsePipe, mustForceExtension, parameter);
  }

  /**
   * Testando o mtodo
   * {@link OutputFileParameter#OutputFileParameter(String, String, String, boolean, String, String, FileParameterMode, boolean, boolean)}
   * : o tipo de arquivo  {@code null}.
   */
  public void testOutputFileParameter04() {
    String name = "NOME_DO_PARAMETRO";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = false;
    String fileType = null;
    FileParameterMode mode = FileParameterMode.REGULAR_FILE;
    FileParameterPipeAcceptance canUsePipe = FileParameterPipeAcceptance.TRUE;
    boolean mustForceExtension = false;
    String commandLinePattern = "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO";
    OutputFileParameter parameter =
      new OutputFileParameter(name, label, description, isOptional,
        commandLinePattern, fileType, mode, canUsePipe, mustForceExtension);
    assertEquals(name, label, description, isOptional, commandLinePattern,
      fileType, mode, canUsePipe, mustForceExtension, parameter);
  }

  /**
   * Testando o mtodo
   * {@link OutputFileParameter#OutputFileParameter(String, String, String, boolean, String, String, FileParameterMode, boolean, boolean)}
   * : o modo  {@link FileParameterMode#DIRECTORY}.
   */
  public void testOutputFileParameter05() {
    String name = "NOME_DO_PARAMETRO";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = false;
    String fileType = null;
    FileParameterMode mode = FileParameterMode.DIRECTORY;
    FileParameterPipeAcceptance canUsePipe = FileParameterPipeAcceptance.TRUE;
    boolean mustForceExtension = false;
    String commandLinePattern = "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO";
    OutputFileParameter parameter =
      new OutputFileParameter(name, label, description, isOptional,
        commandLinePattern, fileType, mode, canUsePipe, mustForceExtension);
    assertEquals(name, label, description, isOptional, commandLinePattern,
      fileType, mode, canUsePipe, mustForceExtension, parameter);
  }

  /**
   * Testando o mtodo
   * {@link OutputFileParameter#OutputFileParameter(String, String, String, boolean, String, String, FileParameterMode, boolean, boolean)}
   * : o parmetro no pode utilizar pipes.
   */
  public void testOutputFileParameter06() {
    String name = "NOME_DO_PARAMETRO";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = false;
    String fileType = null;
    FileParameterMode mode = FileParameterMode.REGULAR_FILE;
    FileParameterPipeAcceptance canUsePipe = FileParameterPipeAcceptance.FALSE;
    boolean mustForceExtension = false;
    String commandLinePattern = "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO";
    OutputFileParameter parameter =
      new OutputFileParameter(name, label, description, isOptional,
        commandLinePattern, fileType, mode, canUsePipe, mustForceExtension);
    assertEquals(name, label, description, isOptional, commandLinePattern,
      fileType, mode, canUsePipe, mustForceExtension, parameter);
  }

  /**
   * Testando o mtodo
   * {@link OutputFileParameter#OutputFileParameter(String, String, String, boolean, String, String, FileParameterMode, boolean, boolean)}
   * : o parmetro precisa que a extenso seja forada.
   */
  public void testOutputFileParameter07() {
    String name = "NOME_DO_PARAMETRO";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = false;
    String fileType = null;
    FileParameterMode mode = FileParameterMode.REGULAR_FILE;
    FileParameterPipeAcceptance canUsePipe = FileParameterPipeAcceptance.TRUE;
    boolean mustForceExtension = true;
    String commandLinePattern = "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO";
    OutputFileParameter parameter =
      new OutputFileParameter(name, label, description, isOptional,
        commandLinePattern, fileType, mode, canUsePipe, mustForceExtension);
    assertEquals(name, label, description, isOptional, commandLinePattern,
      fileType, mode, canUsePipe, mustForceExtension, parameter);
  }

  /**
   * Testando o mtodo
   * {@link OutputFileParameter#OutputFileParameter(String, String, String, boolean, String, String, FileParameterMode, boolean, boolean)}
   * : parmetro com nome {@code null}.
   */
  public void testOutputFileParameter08() {
    String name = null;
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = true;
    String fileType = "Tipo de arquivo";
    FileParameterMode mode = FileParameterMode.REGULAR_FILE;
    FileParameterPipeAcceptance canUsePipe = FileParameterPipeAcceptance.TRUE;
    boolean mustForceExtension = false;
    String commandLinePattern = "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO";
    try {
      new OutputFileParameter(name, label, description, isOptional,
        commandLinePattern, fileType, mode, canUsePipe, mustForceExtension);
      fail();
    }
    catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando o mtodo
   * {@link OutputFileParameter#OutputFileParameter(String, String, String, boolean, String, String, FileParameterMode, boolean, boolean)}
   * : parmetro com rtulo {@code null}.
   */
  public void testOutputFileParameter09() {
    String name = "NOME_DO_PARAMETRO";
    String label = null;
    String description = "Descrio do parmetro";
    boolean isOptional = true;
    String fileType = "Tipo de arquivo";
    FileParameterMode mode = FileParameterMode.REGULAR_FILE;
    FileParameterPipeAcceptance canUsePipe = FileParameterPipeAcceptance.TRUE;
    boolean mustForceExtension = false;
    String commandLinePattern = "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO";
    try {
      new OutputFileParameter(name, label, description, isOptional,
        commandLinePattern, fileType, mode, canUsePipe, mustForceExtension);
      fail();
    }
    catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando o mtodo
   * {@link OutputFileParameter#OutputFileParameter(String, String, String, boolean, String, String, FileParameterMode, boolean, boolean)}
   * : parmetro com descrio {@code null}.
   */
  public void testOutputFileParameter10() {
    String name = "NOME_DO_PARAMETRO";
    String label = "Rtulo do parmetro";
    String description = null;
    boolean isOptional = true;
    String fileType = "Tipo de arquivo";
    FileParameterMode mode = FileParameterMode.REGULAR_FILE;
    FileParameterPipeAcceptance canUsePipe = FileParameterPipeAcceptance.TRUE;
    boolean mustForceExtension = false;
    String commandLinePattern = "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO";
    try {
      new OutputFileParameter(name, label, description, isOptional,
        commandLinePattern, fileType, mode, canUsePipe, mustForceExtension);
      fail();
    }
    catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando o mtodo
   * {@link OutputFileParameter#OutputFileParameter(String, String, String, boolean, String, String, FileParameterMode, boolean, boolean)}
   * : parmetro com modo {@code null}.
   */
  public void testOutputFileParameter11() {
    String name = "NOME_DO_PARAMETRO";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = true;
    String fileType = "Tipo de arquivo";
    FileParameterMode mode = null;
    FileParameterPipeAcceptance canUsePipe = FileParameterPipeAcceptance.TRUE;
    boolean mustForceExtension = false;
    String commandLinePattern = "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO";
    try {
      new OutputFileParameter(name, label, description, isOptional,
        commandLinePattern, fileType, mode, canUsePipe, mustForceExtension);
      fail();
    }
    catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando o mtodo
   * {@link OutputFileParameter#OutputFileParameter(String, String, String, boolean, String, String, FileParameterMode, boolean, boolean)}
   * : o modo  {@link FileParameterMode#DIRECTORY_AND_REGULAR_FILE}.
   */
  public void testOutputFileParameter12() {
    String name = "NOME_DO_PARAMETRO";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    boolean isOptional = false;
    String fileType = null;
    FileParameterMode mode = FileParameterMode.DIRECTORY_AND_REGULAR_FILE;
    FileParameterPipeAcceptance canUsePipe = FileParameterPipeAcceptance.TRUE;
    boolean mustForceExtension = false;
    String commandLinePattern = "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO";
    try {
      new OutputFileParameter(name, label, description, isOptional,
        commandLinePattern, fileType, mode, canUsePipe, mustForceExtension);
      fail();
    }
    catch (IllegalArgumentException e) {
    }
  }

  /**
   * Verifica se todos os atributos esperados so iguais aos atributos do
   * parmetro encontrado.
   * 
   * @param name O nome (Aceita {@code null}).
   * @param label O rtulo (Aceita {@code null}).
   * @param description A descrio (Aceita {@code null}).
   * @param isOptional Indica se o parmetro  opcional/obrigatrio.
   * @param commandLinePattern O formato da linha de comando (Aceita {@code
   *        null}).
   * @param fileType O tipo do arquivo (Aceita {@code null}).
   * @param mode O modo do parmetro (Aceita {@code null}).
   * @param canUsePipe Indica se o parmetro pode utilizar pipes.
   * @param mustForceExtension Indica se o parmetro precisa forar a extenso.
   * @param parameter O parmetro encontrado (Aceita {@code null}).
   */
  private void assertEquals(String name, String label, String description,
    boolean isOptional, String commandLinePattern, String fileType,
    FileParameterMode mode, FileParameterPipeAcceptance canUsePipe,
    boolean mustForceExtension, OutputFileParameter parameter) {
    assertEquals(name, parameter.getName());
    assertEquals(label, parameter.getLabel());
    assertEquals(description, parameter.getDescription());
    assertEquals(isOptional, parameter.isOptional());
    assertEquals(commandLinePattern, parameter.getCommandLinePattern());
    assertEquals(fileType, parameter.getFileType());
    assertEquals(mode, parameter.getMode());
    assertNull(parameter.getDefaultValue());
    assertEquals(canUsePipe, parameter.usesPipe());
    assertEquals(mustForceExtension, parameter.mustForceExtension());
  }
}
