/*
 * $Author:$ $Date:$ $Release:$
 */
package csbase.logic.algorithms.parameters;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import junit.framework.TestCase;

public final class ParameterGroupTest extends TestCase {
  public void testParameterGroup1() {
    String name = "Nome do grupo";
    ParameterGroup parameterGroup = new ParameterGroup(name);
    assertNotNull(parameterGroup.getName());
    assertEquals(name, parameterGroup.getName());
    assertNotNull(parameterGroup.getInputFileParameters());
    assertTrue(parameterGroup.getInputFileParameters().isEmpty());
    assertNotNull(parameterGroup.getOutputFileParameters());
    assertTrue(parameterGroup.getOutputFileParameters().isEmpty());
    assertNotNull(parameterGroup.getParameters());
    assertTrue(parameterGroup.getParameters().isEmpty());
    assertNotNull(parameterGroup.getParameters());
    assertTrue(parameterGroup.getParameters().isEmpty());
  }

  public void testParameterGroup2() {
    try {
      new ParameterGroup(null);
      fail("No deveria ser criado um grupo de parmetro como nome nulo.");
    }
    catch (IllegalArgumentException e) {
    }
  }

  public void testToString() {
    String groupName = "Nome do grupo";
    ParameterGroup parameterGroup = new ParameterGroup(groupName);
    assertNotNull(parameterGroup.toString());
    assertEquals(groupName, parameterGroup.toString());
  }

  public void testAddParameter1() {
    String groupName = "Nome do grupo";
    ParameterGroup parameterGroup = new ParameterGroup(groupName);
    List<Parameter> parameters = new ArrayList<Parameter>();
    List<InputFileParameter> inputFileParameters =
      new LinkedList<InputFileParameter>();
    for (int i = 0; i < 10; i++) {
      String parameterName = "Nome do parmetro " + i;
      String parameterLabel = "Rtulo do parmetro " + i;
      String parameterDescription = "Descrio do parmetro " + i;
      InputFileParameter parameter =
        new InputFileParameter(parameterName, parameterLabel,
          parameterDescription, true, null, null,
          FileParameterMode.REGULAR_FILE, FileParameterPipeAcceptance.TRUE);
      assertTrue(parameterGroup.addParameter(parameter));
      parameters.add(parameter);
      inputFileParameters.add(parameter);
    }
    assertEquals(parameters, parameterGroup.getParameters());
    assertEquals(inputFileParameters, parameterGroup.getInputFileParameters());
    assertEquals(new LinkedList<OutputFileParameter>(), parameterGroup
      .getOutputFileParameters());
  }

  public void testAddParameter2() {
    String groupName = "Nome do grupo";
    ParameterGroup parameterGroup = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    InputFileParameter parameter =
      new InputFileParameter(parameterName, parameterLabel,
        parameterDescription, true, null, null, FileParameterMode.REGULAR_FILE,
        FileParameterPipeAcceptance.TRUE);
    assertTrue(parameterGroup.addParameter(parameter));
    List<Parameter> parameters = new ArrayList<Parameter>();
    parameters.add(parameter);
    List<InputFileParameter> inputFileParameters =
      new LinkedList<InputFileParameter>();
    inputFileParameters.add(parameter);
    assertFalse(parameterGroup.addParameter(parameter));
    assertEquals(parameters, parameterGroup.getParameters());
    assertEquals(inputFileParameters, parameterGroup.getInputFileParameters());
    assertEquals(new LinkedList<OutputFileParameter>(), parameterGroup
      .getOutputFileParameters());
  }

  public void testAddParameter3() {
    String groupName = "Nome do grupo";
    ParameterGroup parameterGroup = new ParameterGroup(groupName);
    List<Parameter> parameters = new ArrayList<Parameter>();
    List<OutputFileParameter> outputFileParameters =
      new LinkedList<OutputFileParameter>();
    for (int i = 0; i < 10; i++) {
      String parameterName = "Nome do parmetro " + i;
      String parameterLabel = "Rtulo do parmetro " + i;
      String parameterDescription = "Descrio do parmetro " + i;
      OutputFileParameter parameter =
        new OutputFileParameter(parameterName, parameterLabel,
          parameterDescription, true, null, null,
          FileParameterMode.REGULAR_FILE, FileParameterPipeAcceptance.TRUE,
          false);
      assertTrue(parameterGroup.addParameter(parameter));
      parameters.add(parameter);
      outputFileParameters.add(parameter);
    }
    assertEquals(parameters, parameterGroup.getParameters());
    assertEquals(outputFileParameters, parameterGroup.getOutputFileParameters());
    assertEquals(new LinkedList<InputFileParameter>(), parameterGroup
      .getInputFileParameters());
  }

  public void testAddParameter4() {
    String groupName = "Nome do grupo";
    ParameterGroup parameterGroup = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    OutputFileParameter parameter =
      new OutputFileParameter(parameterName, parameterLabel,
        parameterDescription, true, null, null, FileParameterMode.REGULAR_FILE,
        FileParameterPipeAcceptance.TRUE, false);
    assertTrue(parameterGroup.addParameter(parameter));
    List<Parameter> parameters = new ArrayList<Parameter>();
    parameters.add(parameter);
    List<OutputFileParameter> outputFileParameters =
      new LinkedList<OutputFileParameter>();
    outputFileParameters.add(parameter);
    assertFalse(parameterGroup.addParameter(parameter));
    assertEquals(parameters, parameterGroup.getParameters());
    assertEquals(outputFileParameters, parameterGroup.getOutputFileParameters());
    assertEquals(new LinkedList<OutputFileParameter>(), parameterGroup
      .getInputFileParameters());
  }

  public void testAddParameter5() {
    String groupName = "Nome do grupo";
    ParameterGroup parameterGroup = new ParameterGroup(groupName);
    List<Parameter> parameters = new ArrayList<Parameter>();
    Set<SimpleParameter> simpleParameters = new HashSet<SimpleParameter>();
    for (int i = 0; i < 10; i++) {
      String parameterName = "Nome do parmetro " + i;
      String parameterLabel = "Rtulo do parmetro " + i;
      String parameterDescription = "Descrio do parmetro " + i;
      String parameterDefaultValue = "Valor-padro do parmetro " + i;
      SimpleParameter parameter =
        new TextParameter(parameterName, parameterLabel, parameterDescription,
          parameterDefaultValue, true, null, null);
      assertTrue(parameterGroup.addParameter(parameter));
      parameters.add(parameter);
      simpleParameters.add(parameter);
    }
    assertEquals(parameters, parameterGroup.getParameters());
    assertEquals(simpleParameters, parameterGroup.getSimpleParameters());
    assertEquals(new LinkedList<InputFileParameter>(), parameterGroup
      .getOutputFileParameters());
    assertEquals(new LinkedList<InputFileParameter>(), parameterGroup
      .getInputFileParameters());
  }

  public void testAddParameter6() {
    String groupName = "Nome do grupo";
    ParameterGroup parameterGroup = new ParameterGroup(groupName);
    String parameterName = "Nome do parmetro";
    String parameterLabel = "Rtulo do parmetro";
    String parameterDescription = "Descrio do parmetro";
    String parameterDefaultValue = "Valor-padro do parmetro";
    SimpleParameter parameter =
      new TextParameter(parameterName, parameterLabel, parameterDescription,
        parameterDefaultValue, true, null, null);
    assertTrue(parameterGroup.addParameter(parameter));
    List<Parameter> parameters = new ArrayList<Parameter>();
    parameters.add(parameter);
    Set<SimpleParameter> simpleParameters = new HashSet<SimpleParameter>();
    simpleParameters.add(parameter);
    assertFalse(parameterGroup.addParameter(parameter));
    assertEquals(parameters, parameterGroup.getParameters());
    assertEquals(simpleParameters, parameterGroup.getSimpleParameters());
    assertEquals(new LinkedList<InputFileParameter>(), parameterGroup
      .getOutputFileParameters());
    assertEquals(new LinkedList<InputFileParameter>(), parameterGroup
      .getInputFileParameters());
  }

  public void testAddParameter7() {
    String parentGroupName = "Grupo pai";
    ParameterGroup parameterGroup = new ParameterGroup(parentGroupName);
    String childGroupName = "Grupo filho";
    ParameterGroup childGroup = new ParameterGroup(childGroupName);
    assertTrue(parameterGroup.addParameter(childGroup));
    List<Parameter> parameters = new ArrayList<Parameter>();
    parameters.add(childGroup);
    assertEquals(parameters, parameterGroup.getParameters());
    assertEquals(new HashSet<SimpleParameter>(), parameterGroup
      .getSimpleParameters());
    assertEquals(new LinkedList<InputFileParameter>(), parameterGroup
      .getOutputFileParameters());
    assertEquals(new LinkedList<InputFileParameter>(), parameterGroup
      .getInputFileParameters());
  }

  public void testAddParameter8() {
    String parentGroupName = "Grupo pai";
    ParameterGroup parameterGroup = new ParameterGroup(parentGroupName);
    String childGroupName = "Grupo filho";
    ParameterGroup childGroup = new ParameterGroup(childGroupName);
    assertTrue(parameterGroup.addParameter(childGroup));
    assertFalse(parameterGroup.addParameter(childGroup));
    List<Parameter> parameters = new ArrayList<Parameter>();
    parameters.add(childGroup);
    assertEquals(parameters, parameterGroup.getParameters());
    assertEquals(new HashSet<SimpleParameter>(), parameterGroup
      .getSimpleParameters());
    assertEquals(new LinkedList<InputFileParameter>(), parameterGroup
      .getOutputFileParameters());
    assertEquals(new LinkedList<InputFileParameter>(), parameterGroup
      .getInputFileParameters());
  }

  public void testAddParameter9() {
    String groupName = "Nome do grupo";
    ParameterGroup parameterGroup = new ParameterGroup(groupName);
    try {
      parameterGroup.addParameter(null);
      fail("No deveria ser possivel adicionar um parmetro nulo.");
    }
    catch (IllegalArgumentException e) {
    }
  }

  public void testGetParameters() {
    String groupName = "Nome do grupo";
    ParameterGroup parameterGroup = new ParameterGroup(groupName);
    try {
      parameterGroup.getParameters().clear();
      fail("No deveria ser possvel alterar a lista de parmetro diretamente.");
    }
    catch (UnsupportedOperationException e) {
    }
  }

  public void testGetInputFileParameters1() {
    String groupName = "Nome do grupo";
    ParameterGroup parameterGroup = new ParameterGroup(groupName);
    try {
      parameterGroup.getInputFileParameters().clear();
      fail("No deveria ser possvel alterar a lista de parmetro diretamente.");
    }
    catch (UnsupportedOperationException e) {
    }
  }

  public void testGetInputFileParameters2() {
    String groupName = "Nome do grupo";
    ParameterGroup parameterGroup = new ParameterGroup(groupName);
    InputFileParameter parameter1 =
      new InputFileParameter("Nome do parmetro 1", "Rtulo do parmetro 1",
        "Descrio do parmetro 1", false, null, null,
        FileParameterMode.REGULAR_FILE, FileParameterPipeAcceptance.TRUE);
    InputFileParameter parameter2 =
      new InputFileParameter("Nome do parmetro 2", "Rtulo do parmetro 2",
        "Descrio do parmetro 2", false, null, null,
        FileParameterMode.REGULAR_FILE, FileParameterPipeAcceptance.TRUE);
    List<InputFileParameter> parameters = new LinkedList<InputFileParameter>();
    parameters.add(parameter1);
    parameters.add(parameter2);
    parameterGroup.addParameter(parameter1);
    parameterGroup.addParameter(parameter2);
    assertEquals(parameters, parameterGroup.getInputFileParameters());
  }

  public void testGetInputFileParameters3() {
    String groupName = "Nome do grupo";
    ParameterGroup parameterGroup = new ParameterGroup(groupName);
    SimpleParameter parameter1 =
      new MockSimpleParameter("Nome do parmetro 1", "Rtulo do parmetro 1",
        "Descrio do parmetro 1", null, false, null);
    InputFileParameter parameter2 =
      new InputFileParameter("Nome do parmetro 2", "Rtulo do parmetro 2",
        "Descrio do parmetro 2", false, null, null,
        FileParameterMode.REGULAR_FILE, FileParameterPipeAcceptance.TRUE);
    InputFileParameter parameter3 =
      new InputFileParameter("Nome do parmetro 3", "Rtulo do parmetro 3",
        "Descrio do parmetro 3", false, null, null,
        FileParameterMode.REGULAR_FILE, FileParameterPipeAcceptance.TRUE);
    List<InputFileParameter> parameters = new LinkedList<InputFileParameter>();
    parameters.add(parameter2);
    parameters.add(parameter3);
    parameterGroup.addParameter(parameter1);
    parameterGroup.addParameter(parameter2);
    parameterGroup.addParameter(parameter3);
    assertEquals(parameters, parameterGroup.getInputFileParameters());
  }

  public void testGetInputFileParameters4() {
    String groupName = "Nome do grupo";
    ParameterGroup parameterGroup = new ParameterGroup(groupName);
    SimpleParameter parameter1 =
      new MockSimpleParameter("Nome do parmetro 1", "Rtulo do parmetro 1",
        "Descrio do parmetro 1", null, false, null);
    InputFileParameter parameter2 =
      new InputFileParameter("Nome do parmetro 2", "Rtulo do parmetro 2",
        "Descrio do parmetro 2", false, null, null,
        FileParameterMode.REGULAR_FILE, FileParameterPipeAcceptance.TRUE);
    InputFileParameter parameter3 =
      new InputFileParameter("Nome do parmetro 3", "Rtulo do parmetro 3",
        "Descrio do parmetro 3", false, null, null,
        FileParameterMode.REGULAR_FILE, FileParameterPipeAcceptance.TRUE);
    InputFileParameter parameter4 =
      new InputFileParameter("Nome do parmetro 4", "Rtulo do parmetro 4",
        "Descrio do parmetro 4", false, null, null,
        FileParameterMode.REGULAR_FILE, FileParameterPipeAcceptance.TRUE);
    InputFileParameter parameter5 =
      new InputFileParameter("Nome do parmetro 5", "Rtulo do parmetro 5",
        "Descrio do parmetro 5", false, null, null,
        FileParameterMode.REGULAR_FILE, FileParameterPipeAcceptance.TRUE);
    ParameterGroup childGroup = new ParameterGroup("Grupo filho");
    parameterGroup.addParameter(parameter1);
    parameterGroup.addParameter(parameter2);
    parameterGroup.addParameter(parameter3);
    childGroup.addParameter(parameter4);
    childGroup.addParameter(parameter5);
    parameterGroup.addParameter(childGroup);
    List<InputFileParameter> parameters = new LinkedList<InputFileParameter>();
    parameters.add(parameter2);
    parameters.add(parameter3);
    parameters.add(parameter4);
    parameters.add(parameter5);
    assertEquals(parameters, parameterGroup.getInputFileParameters());
  }

  public void testGetInputFileParameters5() {
    String groupName = "Nome do grupo";
    ParameterGroup parameterGroup = new ParameterGroup(groupName);
    SimpleParameter parameter1 =
      new MockSimpleParameter("Nome do parmetro 1", "Rtulo do parmetro 1",
        "Descrio do parmetro 1", null, false, null);
    InputFileParameter parameter2 =
      new InputFileParameter("Nome do parmetro 1", "Rtulo do parmetro 1",
        "Descrio do parmetro 1", false, null, null,
        FileParameterMode.REGULAR_FILE, FileParameterPipeAcceptance.TRUE);
    InputFileParameter parameter3 =
      new InputFileParameter("Nome do parmetro 2", "Rtulo do parmetro 2",
        "Descrio do parmetro 2", false, null, null,
        FileParameterMode.REGULAR_FILE, FileParameterPipeAcceptance.TRUE);
    ParameterGroup childGroup = new ParameterGroup("Grupo filho");
    parameterGroup.addParameter(childGroup);
    List<InputFileParameter> parameters = new LinkedList<InputFileParameter>();
    parameters.add(parameter2);
    parameters.add(parameter3);
    parameterGroup.addParameter(parameter1);
    parameterGroup.addParameter(parameter2);
    parameterGroup.addParameter(parameter3);
    assertEquals(parameters, parameterGroup.getInputFileParameters());
  }

  public void testGetOutputFileParameters1() {
    String groupName = "Nome do grupo";
    ParameterGroup parameterGroup = new ParameterGroup(groupName);
    try {
      parameterGroup.getOutputFileParameters().clear();
      fail("No deveria ser possvel alterar a lista de parmetro diretamente.");
    }
    catch (UnsupportedOperationException e) {
    }
  }

  public void testGetOutputFileParameters2() {
    String groupName = "Nome do grupo";
    ParameterGroup parameterGroup = new ParameterGroup(groupName);
    OutputFileParameter parameter1 =
      new OutputFileParameter("Nome do parmetro 1", "Rtulo do parmetro 1",
        "Descrio do parmetro 1", false, null, null,
        FileParameterMode.REGULAR_FILE, FileParameterPipeAcceptance.TRUE, false);
    OutputFileParameter parameter2 =
      new OutputFileParameter("Nome do parmetro 2", "Rtulo do parmetro 2",
        "Descrio do parmetro 2", false, null, null,
        FileParameterMode.REGULAR_FILE, FileParameterPipeAcceptance.TRUE, false);
    List<OutputFileParameter> parameters =
      new LinkedList<OutputFileParameter>();
    parameters.add(parameter1);
    parameters.add(parameter2);
    parameterGroup.addParameter(parameter1);
    parameterGroup.addParameter(parameter2);
    assertEquals(parameters, parameterGroup.getOutputFileParameters());
  }

  public void testGetOutputFileParameters3() {
    String groupName = "Nome do grupo";
    ParameterGroup parameterGroup = new ParameterGroup(groupName);
    SimpleParameter parameter1 =
      new MockSimpleParameter("Nome do parmetro 1", "Rtulo do parmetro 1",
        "Descrio do parmetro 1", null, false, null);
    OutputFileParameter parameter2 =
      new OutputFileParameter("Nome do parmetro 2", "Rtulo do parmetro 2",
        "Descrio do parmetro 2", false, null, null,
        FileParameterMode.REGULAR_FILE, FileParameterPipeAcceptance.TRUE, false);
    OutputFileParameter parameter3 =
      new OutputFileParameter("Nome do parmetro 3", "Rtulo do parmetro 3",
        "Descrio do parmetro 3", false, null, null,
        FileParameterMode.REGULAR_FILE, FileParameterPipeAcceptance.TRUE, false);
    List<OutputFileParameter> parameters =
      new LinkedList<OutputFileParameter>();
    parameters.add(parameter2);
    parameters.add(parameter3);
    parameterGroup.addParameter(parameter1);
    parameterGroup.addParameter(parameter2);
    parameterGroup.addParameter(parameter3);
    assertEquals(parameters, parameterGroup.getOutputFileParameters());
  }

  public void testGetOutputFileParameters4() {
    String groupName = "Nome do grupo";
    ParameterGroup parameterGroup = new ParameterGroup(groupName);
    SimpleParameter parameter1 =
      new MockSimpleParameter("Nome do parmetro 1", "Rtulo do parmetro 1",
        "Descrio do parmetro 1", null, false, null);
    OutputFileParameter parameter2 =
      new OutputFileParameter("Nome do parmetro 2", "Rtulo do parmetro 2",
        "Descrio do parmetro 2", false, null, null,
        FileParameterMode.REGULAR_FILE, FileParameterPipeAcceptance.TRUE, false);
    OutputFileParameter parameter3 =
      new OutputFileParameter("Nome do parmetro 3", "Rtulo do parmetro 3",
        "Descrio do parmetro 3", false, null, null,
        FileParameterMode.REGULAR_FILE, FileParameterPipeAcceptance.TRUE, false);
    OutputFileParameter parameter4 =
      new OutputFileParameter("Nome do parmetro 4", "Rtulo do parmetro 4",
        "Descrio do parmetro 4", false, null, null,
        FileParameterMode.REGULAR_FILE, FileParameterPipeAcceptance.TRUE, false);
    OutputFileParameter parameter5 =
      new OutputFileParameter("Nome do parmetro 5", "Rtulo do parmetro 5",
        "Descrio do parmetro 5", false, null, null,
        FileParameterMode.REGULAR_FILE, FileParameterPipeAcceptance.TRUE, false);
    ParameterGroup childGroup = new ParameterGroup("Grupo filho");
    parameterGroup.addParameter(parameter1);
    parameterGroup.addParameter(parameter2);
    parameterGroup.addParameter(parameter3);
    childGroup.addParameter(parameter4);
    childGroup.addParameter(parameter5);
    parameterGroup.addParameter(childGroup);
    List<OutputFileParameter> parameters =
      new LinkedList<OutputFileParameter>();
    parameters.add(parameter2);
    parameters.add(parameter3);
    parameters.add(parameter4);
    parameters.add(parameter5);
    assertEquals(parameters, parameterGroup.getOutputFileParameters());
  }

  public void testGetOutputFileParameters5() {
    String groupName = "Nome do grupo";
    ParameterGroup parameterGroup = new ParameterGroup(groupName);
    SimpleParameter parameter1 =
      new MockSimpleParameter("Nome do parmetro 1", "Rtulo do parmetro 1",
        "Descrio do parmetro 1", null, false, null);
    OutputFileParameter parameter2 =
      new OutputFileParameter("Nome do parmetro 1", "Rtulo do parmetro 1",
        "Descrio do parmetro 1", false, null, null,
        FileParameterMode.REGULAR_FILE, FileParameterPipeAcceptance.TRUE, false);
    OutputFileParameter parameter3 =
      new OutputFileParameter("Nome do parmetro 2", "Rtulo do parmetro 2",
        "Descrio do parmetro 2", false, null, null,
        FileParameterMode.REGULAR_FILE, FileParameterPipeAcceptance.TRUE, false);
    ParameterGroup childGroup = new ParameterGroup("Grupo filho");
    parameterGroup.addParameter(childGroup);
    List<OutputFileParameter> parameters =
      new LinkedList<OutputFileParameter>();
    parameters.add(parameter2);
    parameters.add(parameter3);
    parameterGroup.addParameter(parameter1);
    parameterGroup.addParameter(parameter2);
    parameterGroup.addParameter(parameter3);
    assertEquals(parameters, parameterGroup.getOutputFileParameters());
  }

  public void testGetSimpleParameters1() {
    String groupName = "Nome do grupo";
    ParameterGroup parameterGroup = new ParameterGroup(groupName);
    try {
      parameterGroup.getSimpleParameters().clear();
      fail("No deveria ser possvel alterar a lista de parmetro diretamente.");
    }
    catch (UnsupportedOperationException e) {
    }
  }

  public void testGetSimpleParameters2() {
    String groupName = "Nome do grupo";
    ParameterGroup parameterGroup = new ParameterGroup(groupName);
    SimpleParameter parameter1 =
      new MockSimpleParameter("Nome do parmetro 1", "Rtulo do parmetro 1",
        "Descrio do parmetro 1", null, false, null);
    SimpleParameter parameter2 =
      new MockSimpleParameter("Nome do parmetro 1", "Rtulo do parmetro 1",
        "Descrio do parmetro 1", null, false, null);
    Set<SimpleParameter> parameters = new HashSet<SimpleParameter>();
    parameters.add(parameter1);
    parameters.add(parameter2);
    parameterGroup.addParameter(parameter1);
    parameterGroup.addParameter(parameter2);
    assertEquals(parameters, parameterGroup.getSimpleParameters());
  }

  public void testGetSimpleParameters3() {
    String groupName = "Nome do grupo";
    ParameterGroup parameterGroup = new ParameterGroup(groupName);
    SimpleParameter parameter1 =
      new MockSimpleParameter("Nome do parmetro 1", "Rtulo do parmetro 1",
        "Descrio do parmetro 1", null, false, null);
    SimpleParameter parameter2 =
      new MockSimpleParameter("Nome do parmetro 1", "Rtulo do parmetro 1",
        "Descrio do parmetro 1", null, false, null);
    SimpleParameter parameter3 =
      new MockSimpleParameter("Nome do parmetro 1", "Rtulo do parmetro 1",
        "Descrio do parmetro 1", null, false, null);
    Set<SimpleParameter> parameters = new HashSet<SimpleParameter>();
    parameters.add(parameter2);
    parameters.add(parameter3);
    parameterGroup.addParameter(parameter1);
    parameterGroup.addParameter(parameter2);
    parameterGroup.addParameter(parameter3);
    assertEquals(parameters, parameterGroup.getSimpleParameters());
  }

  public void testGetSimpleParameters4() {
    String groupName = "Nome do grupo";
    ParameterGroup parameterGroup = new ParameterGroup(groupName);
    SimpleParameter parameter1 =
      new MockSimpleParameter("Nome do parmetro 1", "Rtulo do parmetro 1",
        "Descrio do parmetro 1", null, false, null);
    SimpleParameter parameter2 =
      new MockSimpleParameter("Nome do parmetro 1", "Rtulo do parmetro 1",
        "Descrio do parmetro 1", null, false, null);
    SimpleParameter parameter3 =
      new MockSimpleParameter("Nome do parmetro 1", "Rtulo do parmetro 1",
        "Descrio do parmetro 1", null, false, null);
    SimpleParameter parameter4 =
      new MockSimpleParameter("Nome do parmetro 1", "Rtulo do parmetro 1",
        "Descrio do parmetro 1", null, false, null);
    SimpleParameter parameter5 =
      new MockSimpleParameter("Nome do parmetro 1", "Rtulo do parmetro 1",
        "Descrio do parmetro 1", null, false, null);
    ParameterGroup childGroup = new ParameterGroup("Grupo filho");
    parameterGroup.addParameter(parameter1);
    parameterGroup.addParameter(parameter2);
    parameterGroup.addParameter(parameter3);
    childGroup.addParameter(parameter4);
    childGroup.addParameter(parameter5);
    parameterGroup.addParameter(childGroup);
    Set<SimpleParameter> parameters = new HashSet<SimpleParameter>();
    parameters.add(parameter2);
    parameters.add(parameter3);
    parameters.add(parameter4);
    parameters.add(parameter5);
    assertEquals(parameters, parameterGroup.getSimpleParameters());
  }

  public void testGetSimpleParameters5() {
    String groupName = "Nome do grupo";
    ParameterGroup parameterGroup = new ParameterGroup(groupName);
    SimpleParameter parameter1 =
      new MockSimpleParameter("Nome do parmetro 1", "Rtulo do parmetro 1",
        "Descrio do parmetro 1", null, false, null);
    SimpleParameter parameter2 =
      new MockSimpleParameter("Nome do parmetro 1", "Rtulo do parmetro 1",
        "Descrio do parmetro 1", null, false, null);
    SimpleParameter parameter3 =
      new MockSimpleParameter("Nome do parmetro 1", "Rtulo do parmetro 1",
        "Descrio do parmetro 1", null, false, null);
    ParameterGroup childGroup = new ParameterGroup("Grupo filho");
    parameterGroup.addParameter(childGroup);
    Set<SimpleParameter> parameters = new HashSet<SimpleParameter>();
    parameters.add(parameter2);
    parameters.add(parameter3);
    parameterGroup.addParameter(parameter1);
    parameterGroup.addParameter(parameter2);
    parameterGroup.addParameter(parameter3);
    assertEquals(parameters, parameterGroup.getSimpleParameters());
  }

  public void testEqualsObject1() {
    String groupName = "Nome do grupo";
    ParameterGroup parameterGroup = new ParameterGroup(groupName);
    ParameterGroup equalsParameterGroup = new ParameterGroup(groupName);
    assertEquals(parameterGroup, equalsParameterGroup);
  }

  public void testEqualsObject2() {
    String groupName = "Nome do grupo";
    ParameterGroup parameterGroup = new ParameterGroup(groupName);
    assertFalse(parameterGroup.equals(null));
  }

  public void testEqualsObject3() {
    String groupName = "Nome do grupo";
    ParameterGroup parameterGroup = new ParameterGroup(groupName);
    String objectOfAnotherClass = "Objeto de outra classe";
    assertFalse(parameterGroup.equals(objectOfAnotherClass));
  }

  public void testEqualsObject4() {
    String groupName = "Nome do grupo";
    ParameterGroup parameterGroup = new ParameterGroup(groupName);
    String anotherGroupName = "Outro grupo";
    ParameterGroup equalsParameterGroup = new ParameterGroup(anotherGroupName);
    assertFalse(parameterGroup.equals(equalsParameterGroup));
  }

  public void testHashCode() {
    String groupName = "Nome do grupo";
    ParameterGroup parameterGroup = new ParameterGroup(groupName);
    ParameterGroup equalsParameterGroup = new ParameterGroup(groupName);
    assertEquals(parameterGroup.hashCode(), equalsParameterGroup.hashCode());
  }

  public void testSerialize() throws IOException, ClassNotFoundException {
    ParameterGroup expectedGroup = new ParameterGroup("Nome do grupo");
    InputFileParameter inputFileParameter =
      new InputFileParameter("Entrada", "Entrada", "Entrada", false, null,
        null, FileParameterMode.REGULAR_FILE, FileParameterPipeAcceptance.TRUE);
    expectedGroup.addParameter(inputFileParameter);
    OutputFileParameter outputFileParameter =
      new OutputFileParameter("Saida", "Saida", "Saida", false, null, null,
        FileParameterMode.REGULAR_FILE, FileParameterPipeAcceptance.TRUE, false);
    expectedGroup.addParameter(outputFileParameter);
    SimpleParameter simpleParameter =
      new MockSimpleParameter("Simples", "Simples", "Simples", null, false,
        null);
    expectedGroup.addParameter(simpleParameter);
    ByteArrayOutputStream outputSource = new ByteArrayOutputStream();
    ObjectOutputStream objectOutput = new ObjectOutputStream(outputSource);
    objectOutput.writeObject(expectedGroup);
    ByteArrayInputStream inputSource =
      new ByteArrayInputStream(outputSource.toByteArray());
    ObjectInputStream objectInput = new ObjectInputStream(inputSource);
    ParameterGroup loadedGroup = (ParameterGroup) objectInput.readObject();
    assertEquals(expectedGroup.getName(), loadedGroup.getName());
    assertEquals(expectedGroup.getInputFileParameters(), loadedGroup
      .getInputFileParameters());
    assertEquals(expectedGroup.getOutputFileParameters(), loadedGroup
      .getOutputFileParameters());
    assertEquals(expectedGroup.getParameters(), loadedGroup.getParameters());
  }
}
