/*
 * $Id$
 */
package csbase.logic.algorithms.parameters;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import junit.framework.TestCase;
import csbase.exception.ParseException;
import csbase.exception.algorithms.FormulaCreationException;
import csbase.logic.FileInfo;
import csbase.logic.ProjectFileType;
import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.AlgorithmVersionId;
import csbase.logic.algorithms.AlgorithmVersionInfo;
import csbase.logic.algorithms.CheckAlgorithmConfiguratorListener;
import csbase.logic.algorithms.CommandLineContext;
import csbase.logic.algorithms.ExecutionType;
import csbase.logic.algorithms.FileParameterValue;
import csbase.logic.algorithms.parameters.conditions.Condition;
import csbase.logic.algorithms.parameters.conditions.MockCondition;
import csbase.logic.algorithms.parameters.conditions.SimpleCondition;
import csbase.logic.algorithms.parameters.triggers.CheckTrigger;
import csbase.logic.algorithms.parameters.triggers.MockTrigger;
import csbase.logic.algorithms.parameters.triggers.Trigger;

public final class SimpleAlgorithmConfiguratorTest extends TestCase {

  /**
   * Pattern padro.
   */
  private static final String CMD_PATTERN = "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO";
  
  
  public void testSimpleAlgorithmConfigurator1() {
    AlgorithmVersionInfo algorithmVersionInfo = createAlgorithmVersion();
    SimpleAlgorithmConfigurator configurator =
      new SimpleAlgorithmConfigurator(algorithmVersionInfo,
        VERSION_DESCRIPTION, SIMPLE_EXECUTION, ABBREVIATION, DONT_PROVIDE_ID,
        COMMAND, SHELL, INPUT_FILE_PARAMETER_NAME, true, CMD_PATTERN);
    assertEquals(algorithmVersionInfo, configurator.getAlgorithmVersion());
    assertEquals(VERSION_DESCRIPTION, configurator.getDescription());
    assertEquals(SIMPLE_EXECUTION, configurator.getExecutionType());
    assertEquals(ABBREVIATION, configurator.getAbbreviation());
    assertEquals(DONT_PROVIDE_ID, configurator.provideId());
    assertEquals(COMMAND, configurator.getCommandBinaryName());
    assertEquals(SHELL, configurator.getShell());
    assertEquals(INPUT_FILE_PARAMETER_NAME, configurator
      .getDefaultInputFileParameterName());
    assertNotNull(configurator.getTriggers());
    assertTrue(configurator.getTriggers().isEmpty());
    assertNotNull(configurator.getExpressions());
    assertTrue(configurator.getExpressions().isEmpty());
    assertNotNull(configurator.getGroups());
    assertNull(configurator.getFilePath());
  }

  public void testSimpleAlgorithmConfigurator2() {
    AlgorithmVersionInfo algorithmVersionInfo = null;
    try {
      new SimpleAlgorithmConfigurator(algorithmVersionInfo,
        VERSION_DESCRIPTION, SIMPLE_EXECUTION, ABBREVIATION, DONT_PROVIDE_ID,
        COMMAND, SHELL, INPUT_FILE_PARAMETER_NAME, true, CMD_PATTERN);
      fail("Esperada IllegalArgumentException");
    }
    catch (IllegalArgumentException e) {
    }
  }

  public void testSimpleAlgorithmConfigurator3() {
    AlgorithmVersionInfo algorithmVersionInfo = createAlgorithmVersion();
    String description = null;
    SimpleAlgorithmConfigurator configurator =
      new SimpleAlgorithmConfigurator(algorithmVersionInfo, description,
        SIMPLE_EXECUTION, ABBREVIATION, DONT_PROVIDE_ID, COMMAND, SHELL,
        INPUT_FILE_PARAMETER_NAME, true, CMD_PATTERN);
    assertEquals(algorithmVersionInfo, configurator.getAlgorithmVersion());
    assertEquals(description, configurator.getDescription());
    assertEquals(SIMPLE_EXECUTION, configurator.getExecutionType());
    assertEquals(ABBREVIATION, configurator.getAbbreviation());
    assertEquals(DONT_PROVIDE_ID, configurator.provideId());
    assertEquals(COMMAND, configurator.getCommandBinaryName());
    assertEquals(SHELL, configurator.getShell());
    assertEquals(INPUT_FILE_PARAMETER_NAME, configurator
      .getDefaultInputFileParameterName());
    assertNotNull(configurator.getTriggers());
    assertTrue(configurator.getTriggers().isEmpty());
    assertNotNull(configurator.getExpressions());
    assertTrue(configurator.getExpressions().isEmpty());
    assertNotNull(configurator.getGroups());
    assertNull(configurator.getFilePath());
  }

  public void testSimpleAlgorithmConfigurator4() {
    AlgorithmVersionInfo algorithmVersionInfo = createAlgorithmVersion();
    ExecutionType executionType = null;
    try {
      new SimpleAlgorithmConfigurator(algorithmVersionInfo,
        VERSION_DESCRIPTION, executionType, ABBREVIATION, DONT_PROVIDE_ID,
        COMMAND, SHELL, INPUT_FILE_PARAMETER_NAME, true, CMD_PATTERN);
      fail("Esperada IllegalArgumentException");
    }
    catch (IllegalArgumentException e) {
    }
  }

  public void testSimpleAlgorithmConfigurator5() {
    AlgorithmVersionInfo algorithmVersionInfo = createAlgorithmVersion();
    String abbreviation = null;
    SimpleAlgorithmConfigurator configurator =
      new SimpleAlgorithmConfigurator(algorithmVersionInfo,
        ALGORITHM_DESCRIPTION, SIMPLE_EXECUTION, abbreviation, DONT_PROVIDE_ID,
        COMMAND, SHELL, INPUT_FILE_PARAMETER_NAME, true, CMD_PATTERN);
    assertEquals(algorithmVersionInfo, configurator.getAlgorithmVersion());
    assertEquals(ALGORITHM_DESCRIPTION, configurator.getDescription());
    assertEquals(SIMPLE_EXECUTION, configurator.getExecutionType());
    assertEquals(abbreviation, configurator.getAbbreviation());
    assertEquals(DONT_PROVIDE_ID, configurator.provideId());
    assertEquals(COMMAND, configurator.getCommandBinaryName());
    assertEquals(SHELL, configurator.getShell());
    assertEquals(INPUT_FILE_PARAMETER_NAME, configurator
      .getDefaultInputFileParameterName());
    assertNotNull(configurator.getTriggers());
    assertTrue(configurator.getTriggers().isEmpty());
    assertNotNull(configurator.getExpressions());
    assertTrue(configurator.getExpressions().isEmpty());
    assertNotNull(configurator.getGroups());
    assertNull(configurator.getFilePath());
  }

  public void testSimpleAlgorithmConfigurator6() {
    AlgorithmVersionInfo algorithmVersionInfo = createAlgorithmVersion();
    String abbreviation = null;
    SimpleAlgorithmConfigurator configurator =
      new SimpleAlgorithmConfigurator(algorithmVersionInfo,
        ALGORITHM_DESCRIPTION, SIMPLE_EXECUTION, abbreviation, PROVIDE_ID,
        COMMAND, SHELL, INPUT_FILE_PARAMETER_NAME, true, CMD_PATTERN);
    assertEquals(algorithmVersionInfo, configurator.getAlgorithmVersion());
    assertEquals(ALGORITHM_DESCRIPTION, configurator.getDescription());
    assertEquals(SIMPLE_EXECUTION, configurator.getExecutionType());
    assertEquals(abbreviation, configurator.getAbbreviation());
    assertEquals(PROVIDE_ID, configurator.provideId());
    assertEquals(COMMAND, configurator.getCommandBinaryName());
    assertEquals(SHELL, configurator.getShell());
    assertEquals(INPUT_FILE_PARAMETER_NAME, configurator
      .getDefaultInputFileParameterName());
    assertNotNull(configurator.getTriggers());
    assertTrue(configurator.getTriggers().isEmpty());
    assertNotNull(configurator.getExpressions());
    assertTrue(configurator.getExpressions().isEmpty());
    assertNotNull(configurator.getGroups());
    assertNull(configurator.getFilePath());
  }

  public void testSimpleAlgorithmConfigurator7() {
    AlgorithmVersionInfo algorithmVersionInfo = createAlgorithmVersion();
    String command = null;
    try {
      new SimpleAlgorithmConfigurator(algorithmVersionInfo,
        VERSION_DESCRIPTION, SIMPLE_EXECUTION, ABBREVIATION, DONT_PROVIDE_ID,
        command, SHELL, INPUT_FILE_PARAMETER_NAME, true, CMD_PATTERN);
      fail("Esperada IllegalArgumentException");
    }
    catch (IllegalArgumentException e) {
    }
  }

  public void testSimpleAlgorithmConfigurator8() {
    AlgorithmVersionInfo algorithmVersionInfo = createAlgorithmVersion();
    String shell = null;
    SimpleAlgorithmConfigurator configurator =
      new SimpleAlgorithmConfigurator(algorithmVersionInfo,
        ALGORITHM_DESCRIPTION, SIMPLE_EXECUTION, ABBREVIATION, PROVIDE_ID,
        COMMAND, shell, INPUT_FILE_PARAMETER_NAME, true, CMD_PATTERN);
    assertEquals(algorithmVersionInfo, configurator.getAlgorithmVersion());
    assertEquals(ALGORITHM_DESCRIPTION, configurator.getDescription());
    assertEquals(SIMPLE_EXECUTION, configurator.getExecutionType());
    assertEquals(ABBREVIATION, configurator.getAbbreviation());
    assertEquals(PROVIDE_ID, configurator.provideId());
    assertEquals(COMMAND, configurator.getCommandBinaryName());
    assertEquals(shell, configurator.getShell());
    assertEquals(INPUT_FILE_PARAMETER_NAME, configurator
      .getDefaultInputFileParameterName());
    assertNotNull(configurator.getTriggers());
    assertTrue(configurator.getTriggers().isEmpty());
    assertNotNull(configurator.getExpressions());
    assertTrue(configurator.getExpressions().isEmpty());
    assertNotNull(configurator.getGroups());
    assertNull(configurator.getFilePath());
  }

  public void testSimpleAlgorithmConfigurator9() {
    AlgorithmVersionInfo algorithmVersionInfo = createAlgorithmVersion();
    String inputFileParameterName = null;
    SimpleAlgorithmConfigurator configurator =
      new SimpleAlgorithmConfigurator(algorithmVersionInfo,
        ALGORITHM_DESCRIPTION, SIMPLE_EXECUTION, ABBREVIATION, PROVIDE_ID,
        COMMAND, SHELL, inputFileParameterName, true, CMD_PATTERN);
    assertEquals(algorithmVersionInfo, configurator.getAlgorithmVersion());
    assertEquals(ALGORITHM_DESCRIPTION, configurator.getDescription());
    assertEquals(SIMPLE_EXECUTION, configurator.getExecutionType());
    assertEquals(ABBREVIATION, configurator.getAbbreviation());
    assertEquals(PROVIDE_ID, configurator.provideId());
    assertEquals(COMMAND, configurator.getCommandBinaryName());
    assertEquals(SHELL, configurator.getShell());
    assertEquals(inputFileParameterName, configurator
      .getDefaultInputFileParameterName());
    assertNotNull(configurator.getTriggers());
    assertTrue(configurator.getTriggers().isEmpty());
    assertNotNull(configurator.getExpressions());
    assertTrue(configurator.getExpressions().isEmpty());
    assertNotNull(configurator.getGroups());
    assertNull(configurator.getFilePath());
  }

  public void testAddExpression1() throws FormulaCreationException,
    ParseException {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    Set<ValidationExpression> expressions = new HashSet<ValidationExpression>();
    for (int i = 0; i < 10; i++) {
      ValidationExpression expression =
        new ValidationExpression(configurator, Integer.toString(i), "Erro " + i);
      expressions.add(expression);
      assertTrue(configurator.addExpression(expression));
      assertEquals(expressions, configurator.getExpressions());
    }
  }

  public void testAddExpression2() throws FormulaCreationException,
    ParseException {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    Set<ValidationExpression> expressions = new HashSet<ValidationExpression>();
    for (int i = 0; i < 10; i++) {
      ValidationExpression expression =
        new ValidationExpression(configurator, Integer.toString(i), "Erro " + i);
      expressions.add(expression);
      assertTrue(configurator.addExpression(expression));
      assertFalse(configurator.addExpression(expression));
      assertEquals(expressions, configurator.getExpressions());
    }
  }

  public void testAddExpression3() {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    try {
      configurator.addExpression(null);
      fail("No deveria ser possvel adicionar um arquivo de ajuda nulo.");
    }
    catch (IllegalArgumentException e) {
    }
  }

  public void testAddGroup1() {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    List<ParameterGroup> groups = new LinkedList<ParameterGroup>();
    for (int i = 0; i < 10; i++) {
      ParameterGroup group = new ParameterGroup(GROUP_PARAM_NAME + i);
      groups.add(group);
      assertTrue(configurator.addGroup(group));
    }
    assertEquals(groups, configurator.getGroups());
  }

  public void testAddGroup2() {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    List<ParameterGroup> groups = new LinkedList<ParameterGroup>();
    for (int i = 0; i < 10; i++) {
      ParameterGroup group = new ParameterGroup(GROUP_PARAM_NAME + i);
      groups.add(group);
      assertTrue(configurator.addGroup(group));
      assertFalse(configurator.addGroup(group));
    }
    assertEquals(groups, configurator.getGroups());
  }

  public void testAddGroup3() {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    try {
      configurator.addGroup(null);
      fail("No deveria ser possvel adicionar um grupo nulo.");
    }
    catch (IllegalArgumentException e) {
    }
  }

  public void testAddTrigger1() {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup(GROUP_PARAM_NAME);
    configurator.addGroup(group);
    SimpleParameter<Object> parameter =
      new MockSimpleParameter(PARAM_NAME, PARAM_LABEL, PARAM_DESCRIPTION,
        PARAM_DEFAULT_VALUE, false, ALTERNATE_COMMAND_LINE_PATTERN);
    group.addParameter(parameter);
    Set<Trigger> triggers = new HashSet<Trigger>();
    Condition condition = new MockCondition(true);
    Trigger trigger = new MockTrigger(parameter, condition);
    assertTrue(configurator.addTrigger(trigger));
    triggers.add(trigger);
    assertEquals(triggers, configurator.getTriggers());
  }

  public void testAddTrigger2() {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup(GROUP_PARAM_NAME);
    configurator.addGroup(group);
    SimpleParameter<Object> parameter =
      new MockSimpleParameter(PARAM_NAME, PARAM_LABEL, PARAM_DESCRIPTION,
        PARAM_DEFAULT_VALUE, false, ALTERNATE_COMMAND_LINE_PATTERN);
    group.addParameter(parameter);
    Set<Trigger> triggers = new HashSet<Trigger>();
    Condition condition = new MockCondition(true);
    Trigger trigger = new MockTrigger(parameter, condition);
    assertTrue(configurator.addTrigger(trigger));
    assertFalse(configurator.addTrigger(trigger));
    triggers.add(trigger);
    assertEquals(triggers, configurator.getTriggers());
  }

  public void testAddTrigger3() {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    try {
      configurator.addTrigger(null);
      fail("Esperada IllegalArgumentException.");
    }
    catch (IllegalArgumentException e) {
    }
  }

  public void testGetExpressions1() throws FormulaCreationException,
    ParseException {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    Set<ValidationExpression> expressions = new HashSet<ValidationExpression>();
    for (int i = 0; i < 10; i++) {
      ValidationExpression expression =
        new ValidationExpression(configurator, Integer.toString(i), "Erro " + i);
      expressions.add(expression);
      assertTrue(configurator.addExpression(expression));
    }
    assertEquals(expressions, configurator.getExpressions());
    try {
      configurator.getExpressions().clear();
      fail("No deveria ser possvel criar modificar a lista de expresses diretamente.");
    }
    catch (UnsupportedOperationException e) {
    }
    assertEquals(expressions, configurator.getExpressions());
  }

  public void testGetGroups1() {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    List<ParameterGroup> groups = new LinkedList<ParameterGroup>();
    for (int i = 0; i < 10; i++) {
      ParameterGroup group = new ParameterGroup(GROUP_PARAM_NAME + i);
      groups.add(group);
      assertTrue(configurator.addGroup(group));
    }

    assertEquals(groups, configurator.getGroups());
    try {
      configurator.getGroups().clear();
      fail("No deveria ser possvel alterar a lista de grupos diretamente.");
    }
    catch (UnsupportedOperationException e) {
    }
    assertEquals(groups, configurator.getGroups());
  }

  public void testGetTriggers1() {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup(GROUP_PARAM_NAME);
    configurator.addGroup(group);
    SimpleParameter<Object> parameter =
      new MockSimpleParameter(PARAM_NAME, PARAM_LABEL, PARAM_DESCRIPTION,
        PARAM_DEFAULT_VALUE, false, ALTERNATE_COMMAND_LINE_PATTERN);
    group.addParameter(parameter);
    Set<Trigger> triggers = new HashSet<Trigger>();
    Condition condition = new MockCondition(true);
    Trigger trigger = new MockTrigger(parameter, condition);
    assertTrue(configurator.addTrigger(trigger));
    triggers.add(trigger);
    assertEquals(triggers, configurator.getTriggers());
    try {
      configurator.getTriggers().clear();
      fail("No deveria ser possvel alterar a lista de restries diretamente.");
    }
    catch (UnsupportedOperationException e) {
    }
    assertEquals(triggers, configurator.getTriggers());
  }

  public void testGetInputFileParameters1() {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup(GROUP_PARAM_NAME);
    configurator.addGroup(group);
    assertNotNull(configurator.getInputFileParameters());
    assertTrue(configurator.getInputFileParameters().isEmpty());
  }

  public void testGetInputFileParameters2() {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup(GROUP_PARAM_NAME);
    configurator.addGroup(group);
    List<InputFileParameter> inputFileParameters =
      new LinkedList<InputFileParameter>();
    for (int i = 0; i < 10; i++) {
      InputFileParameter inputFileParameter =
        new InputFileParameter(PARAM_NAME + i, PARAM_LABEL + i,
          PARAM_DESCRIPTION + i, true, DEFAULT_COMMAND_LINE_PATTERN, null,
          FileParameterMode.REGULAR_FILE, FileParameterPipeAcceptance.TRUE);
      inputFileParameters.add(inputFileParameter);
      group.addParameter(inputFileParameter);
    }
    assertEquals(inputFileParameters, configurator.getInputFileParameters());
  }

  public void testGetOutputFileParameters1() {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup(GROUP_PARAM_NAME);
    configurator.addGroup(group);
    List<OutputFileParameter> outputFileParameters =
      new LinkedList<OutputFileParameter>();
    assertEquals(outputFileParameters, configurator.getOutputFileParameters());
  }

  public void testGetOutputFileParameters2() {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup(GROUP_PARAM_NAME);
    configurator.addGroup(group);
    List<OutputFileParameter> outputFileParameters =
      new LinkedList<OutputFileParameter>();
    for (int i = 0; i < 10; i++) {
      OutputFileParameter outputFileParameter =
        new OutputFileParameter(PARAM_NAME + i, PARAM_LABEL + i,
          PARAM_DESCRIPTION + i, true, null, null,
          FileParameterMode.REGULAR_FILE, FileParameterPipeAcceptance.TRUE,
          false);
      outputFileParameters.add(outputFileParameter);
      group.addParameter(outputFileParameter);
    }
    assertEquals(outputFileParameters, configurator.getOutputFileParameters());
  }

  @SuppressWarnings("unchecked")
  public void testResetValuess1() {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    for (int i = 0; i < 10; i++) {
      ParameterGroup group = new ParameterGroup(GROUP_PARAM_NAME + i);
      for (int j = 0; j < 10; j++) {
        Object[] formats = new Object[] { new Integer(i), new Integer(j) };
        SimpleParameter parameter =
          new MockSimpleParameter(MessageFormat
            .format("Nome do parmetro $NOME_DO_PARAMETRO.$VALOR_DO_PARAMETRO",
              formats), MessageFormat.format(
            "Rtulo do parmetro $NOME_DO_PARAMETRO.$VALOR_DO_PARAMETRO",
            formats), MessageFormat.format(
            "Descrio do parmetro $NOME_DO_PARAMETRO.$VALOR_DO_PARAMETRO",
            formats), MessageFormat.format(
            "Valor do parmetro $NOME_DO_PARAMETRO.$VALOR_DO_PARAMETRO",
            formats), false, null);
        group.addParameter(parameter);
      }
      configurator.addGroup(group);
    }
    int count = 0;
    for (ParameterGroup group : configurator.getGroups()) {
      for (Parameter parameter : group.getParameters()) {
        SimpleParameter simpleParameter = (SimpleParameter) parameter;
        Object value = PARAM_DEFAULT_VALUE + count;
        count++;
        simpleParameter.setValue(value);
        assertEquals(value, simpleParameter.getValue());
      }
    }
    configurator.resetValues();
    for (ParameterGroup group : configurator.getGroups()) {
      for (Parameter parameter : group.getParameters()) {
        SimpleParameter simpleParameter = (SimpleParameter) parameter;
        assertEquals(simpleParameter.getDefaultValue(), simpleParameter
          .getValue());
      }
    }
  }

  @SuppressWarnings("unchecked")
  public void testImportValues1() throws ParseException {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    for (int i = 0; i < 10; i++) {
      ParameterGroup group = new ParameterGroup(GROUP_PARAM_NAME + i);
      for (int j = 0; j < 10; j++) {
        Object[] formats = new Object[] { new Integer(i), new Integer(j) };
        SimpleParameter parameter =
          new MockSimpleParameter(MessageFormat
            .format("Nome do parmetro $NOME_DO_PARAMETRO.$VALOR_DO_PARAMETRO",
              formats), MessageFormat.format(
            "Rtulo do parmetro $NOME_DO_PARAMETRO.$VALOR_DO_PARAMETRO",
            formats), MessageFormat.format(
            "Descrio do parmetro $NOME_DO_PARAMETRO.$VALOR_DO_PARAMETRO",
            formats), MessageFormat.format(
            "Valor do parmetro $NOME_DO_PARAMETRO.$VALOR_DO_PARAMETRO",
            formats), false, null);
        group.addParameter(parameter);
      }
      configurator.addGroup(group);
    }
    Map exportedValues = configurator.exportValues();
    assertNotNull(exportedValues);
    Iterator groupIterator = configurator.getGroups().iterator();
    int count = 0;
    while (groupIterator.hasNext()) {
      ParameterGroup group = (ParameterGroup) groupIterator.next();
      Iterator parameterIterator = group.getParameters().iterator();
      while (parameterIterator.hasNext()) {
        SimpleParameter parameter = (SimpleParameter) parameterIterator.next();
        Object value = PARAM_DEFAULT_VALUE + count;
        count++;
        parameter.setValue(value);
        assertEquals(value, parameter.getValue());
      }
    }
    configurator.importValues(exportedValues);
    groupIterator = configurator.getGroups().iterator();
    while (groupIterator.hasNext()) {
      ParameterGroup group = (ParameterGroup) groupIterator.next();
      Iterator parameterIterator = group.getParameters().iterator();
      while (parameterIterator.hasNext()) {
        SimpleParameter parameter = (SimpleParameter) parameterIterator.next();
        assertEquals(parameter.getDefaultValue(), parameter.getValue());
      }
    }
  }

  public void testImportValues2() throws ParseException {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    try {
      configurator.importValues(null);
      fail("No deveria ser possvel importar valor nulo.");
    }
    catch (IllegalArgumentException e) {
    }
  }

  public void testUpdateTriggers1() {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup(GROUP_PARAM_NAME);
    MockSimpleParameter parameter =
      new MockSimpleParameter(PARAM_NAME, PARAM_LABEL, PARAM_DESCRIPTION, null,
        true, null);
    group.addParameter(parameter);
    configurator.addGroup(group);
    SimpleCondition condition =
      new SimpleCondition(parameter.getName(), PARAM_NEW_VALUE);
    CheckTrigger trigger = new CheckTrigger(parameter, condition);
    trigger.allowUndo();
    configurator.addTrigger(trigger);
    assertTrue(trigger.wasUndone());
    trigger.prohibitUndo();
    trigger.allowDo();
    assertTrue(parameter.setValue(PARAM_NEW_VALUE));
    assertTrue(trigger.wasDone());
  }

  public void testUpdateTriggers2() {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup(GROUP_PARAM_NAME);
    MockSimpleParameter parameter =
      new MockSimpleParameter(PARAM_NAME, PARAM_LABEL, PARAM_DESCRIPTION,
        PARAM_DEFAULT_VALUE, true, null);
    group.addParameter(parameter);
    configurator.addGroup(group);
    SimpleCondition condition = new SimpleCondition(parameter.getName(), null);
    CheckTrigger trigger = new CheckTrigger(parameter, condition);
    trigger.allowUndo();
    configurator.addTrigger(trigger);
    assertTrue(trigger.wasUndone());
    assertTrue(parameter.setValue(PARAM_NEW_VALUE));
    assertTrue(trigger.wasUndone());
  }

  /**
   * Testando o mtodo
   * {@link SimpleAlgorithmConfigurator#makeCommandLine(CommandLineContext)}:
   * separador de arquivos padro UNIX.
   */
  public void testMakeCommandLine1() {
    SimpleAlgorithmConfigurator configurator =
      new SimpleAlgorithmConfigurator(createAlgorithmVersion(), null,
        SIMPLE_EXECUTION, null, false, COMMAND, null, null, true, CMD_PATTERN);
    String cmd = "";
    cmd += "(";
    cmd += CD_TO_ALGORITHM_DIR_COMMAND_UNIX;
    cmd += " ";
    cmd += "(." + UNIX_SEPARATOR + COMMAND + ")";
    cmd += ")";

    CommandLineContext context =
      CommandLineContext.createSimpleContext(COMMAND_ID, PLATFORM_ID,
        UNIX_SEPARATOR, SERVER_DIR_UNIX, PROJECT_DIR_UNIX);
    assertEquals(cmd, configurator.makeCommandLine(context));
  }

  /**
   * Testando o mtodo
   * {@link SimpleAlgorithmConfigurator#makeCommandLine(CommandLineContext)}:
   * separador de arquivos padro Windows.
   */
  public void testMakeCommandLine2() {
    SimpleAlgorithmConfigurator configurator =
      new SimpleAlgorithmConfigurator(createAlgorithmVersion(), null,
        SIMPLE_EXECUTION, null, false, COMMAND, null, null, true, CMD_PATTERN);
    String cmd = "";
    cmd += "(";
    cmd += CD_TO_ALGORITHM_DIR_COMMAND_WINDOWS;
    cmd += " ";
    cmd += "(." + WINDOWS_SEPARATOR + COMMAND + ")";
    cmd += ")";
    CommandLineContext commandLineContext =
      CommandLineContext.createSimpleContext(COMMAND_ID, PLATFORM_ID,
        WINDOWS_SEPARATOR, SERVER_DIR_WINDOWS, "PROJECT_DIR");
    assertEquals(cmd, configurator.makeCommandLine(commandLineContext));
  }

  /**
   * Testando o mtodo
   * {@link SimpleAlgorithmConfigurator#makeCommandLine(CommandLineContext)}:
   * parmetros tipo booleano.
   */
  public void testMakeCommandLine3() {
    SimpleAlgorithmConfigurator configurator =
      new SimpleAlgorithmConfigurator(createAlgorithmVersion(), null,
        SIMPLE_EXECUTION, null, false, COMMAND, null, null, true, CMD_PATTERN);
    ParameterGroup group1 = new ParameterGroup("Grupo 1");
    String parameterName11 = "Parmetro_1.1";
    boolean parameterValue11 = false;
    String falseCommand11 = "FALSO 1.1";
    String trueCommand11 = "VERDADEIRO 1.1";
    String parameterLabel11 = "Rtulo do parmetro 1.1";
    String parameterDescription11 = "Descrio do parmetro 1.1";
    BooleanParameter parameter11 =
      new BooleanParameter(parameterName11, parameterLabel11,
        parameterDescription11, parameterValue11, DEFAULT_COMMAND_LINE_PATTERN,
        falseCommand11, trueCommand11);
    group1.addParameter(parameter11);
    String parameterName12 = "Parmetro_1.2";
    boolean parameterValue12 = true;
    String falseCommand12 = "FALSO 1.2";
    String trueCommand12 = "VERDADEIRO 1.2";
    String parameterLabel12 = "Rtulo do parmetro 1.2";
    String parameterDescription12 = "Descrio do parmetro 1.2";
    BooleanParameter parameter12 =
      new BooleanParameter(parameterName12, parameterLabel12,
        parameterDescription12, parameterValue12, DEFAULT_COMMAND_LINE_PATTERN,
        falseCommand12, trueCommand12);
    group1.addParameter(parameter12);
    ParameterGroup group2 = new ParameterGroup("Grupo 2");
    String parameterName21 = "Parmetro_2.1";
    boolean parameterValue21 = true;
    String falseCommand21 = "FALSO 2.1";
    String trueCommand21 = "VERDADEIRO 2.1";
    String parameterLabel21 = "Rtulo do parmetro 2.1";
    String parameterDescription21 = "Descrio do parmetro 2.1";
    BooleanParameter parameter21 =
      new BooleanParameter(parameterName21, parameterLabel21,
        parameterDescription21, parameterValue21, DEFAULT_COMMAND_LINE_PATTERN,
        falseCommand21, trueCommand21);
    group2.addParameter(parameter21);
    String parameterName22 = "Parmetro_2.2";
    boolean parameterValue22 = false;
    String falseCommand22 = "FALSO 2.2";
    String trueCommand22 = "VERDADEIRO 2.2";
    String parameterLabel22 = "Rtulo do parmetro 2.2";
    String parameterDescription22 = "Descrio do parmetro 2.2";
    BooleanParameter parameter22 =
      new BooleanParameter(parameterName22, parameterLabel22,
        parameterDescription22, parameterValue22, DEFAULT_COMMAND_LINE_PATTERN,
        falseCommand22, trueCommand22);
    group2.addParameter(parameter22);
    configurator.addGroup(group1);
    configurator.addGroup(group2);
    String cmd = "";
    cmd += "(";
    cmd += CD_TO_ALGORITHM_DIR_COMMAND_UNIX;
    cmd += " ";
    cmd += "(." + UNIX_SEPARATOR + COMMAND;
    cmd += " ";
    cmd += "Parmetro_1.1=FALSO 1.1 ";
    cmd += "Parmetro_1.2=VERDADEIRO 1.2 ";
    cmd += "Parmetro_2.1=VERDADEIRO 2.1 ";
    cmd += "Parmetro_2.2=FALSO 2.2";
    cmd += "))";
    CommandLineContext context =
      CommandLineContext.createSimpleContext(COMMAND_ID, PLATFORM_ID,
        UNIX_SEPARATOR, SERVER_DIR_UNIX, PROJECT_DIR_UNIX);
    assertEquals(cmd, configurator.makeCommandLine(context));
  }

  /**
   * Testando o mtodo
   * {@link SimpleAlgorithmConfigurator#makeCommandLine(CommandLineContext)}:
   * parmetros tipo texto.
   */
  public void testMakeCommandLine4() {
    SimpleAlgorithmConfigurator configurator =
      new SimpleAlgorithmConfigurator(createAlgorithmVersion(), null,
        SIMPLE_EXECUTION, null, false, COMMAND, null, null, true, CMD_PATTERN);
    ParameterGroup group1 = new ParameterGroup("Grupo 1");
    String parameterName11 = "Parmetro_1.1";
    String parameterLabel11 = "Rtulo do parmetro 1.1";
    String parameterDescription11 = "Descrio do parmetro 1.1";
    String parameterValue11 = "Valor 1.1";
    TextParameter parameter11 =
      new TextParameter(parameterName11, parameterLabel11,
        parameterDescription11, parameterValue11, true,
        DEFAULT_COMMAND_LINE_PATTERN, null);
    group1.addParameter(parameter11);
    ParameterGroup group2 = new ParameterGroup("Grupo 2");
    String parameterName21 = "Parmetro_2.1";
    String parameterLabel21 = "Rtulo do parmetro 2.1";
    String parameterDescription21 = "Descrio do parmetro 2.1";
    String parameterValue21 = null;
    TextParameter parameter21 =
      new TextParameter(parameterName21, parameterLabel21,
        parameterDescription21, parameterValue21, true,
        DEFAULT_COMMAND_LINE_PATTERN, null);
    group2.addParameter(parameter21);
    configurator.addGroup(group1);
    configurator.addGroup(group2);
    String cmd = "";
    cmd += "(";
    cmd += CD_TO_ALGORITHM_DIR_COMMAND_UNIX;
    cmd += " ";
    cmd += "(." + UNIX_SEPARATOR + COMMAND;
    cmd += " ";
    cmd += "Parmetro_1.1=Valor 1.1";
    cmd += "))";

    String fileSeparator = UNIX_SEPARATOR;
    CommandLineContext context =
      CommandLineContext.createSimpleContext(COMMAND_ID, PLATFORM_ID,
        fileSeparator, SERVER_DIR_UNIX, PROJECT_DIR_UNIX);
    assertEquals(cmd, configurator.makeCommandLine(context));
  }

  /**
   * Testando o mtodo
   * {@link SimpleAlgorithmConfigurator#makeCommandLine(CommandLineContext)}:
   * parmetros tipo arquivo.
   */
  public void testMakeCommandLine5() {

    String fileSeparator = UNIX_SEPARATOR;

    String logPath = "um_diretorio/arquivo_de_log.txt";
    String logFullPath =
      PATH_ALGORITHM_TO_PROJECT + PROJECT_DIR_UNIX + UNIX_SEPARATOR + logPath;
    String stdOut = "out.log";
    StringBuilder cmd1 = new StringBuilder();
    cmd1.append('(');
    cmd1.append("cd ").append(SERVER_DIR_UNIX).append(UNIX_SEPARATOR).append(
      ALGORITHM_DIR_UNIX).append(COMMAND_SEPARATOR);
    cmd1.append(' ');
    cmd1.append("(." + UNIX_SEPARATOR + COMMAND);
    cmd1.append(' ');
    cmd1.append("LOG_FILE=").append(logFullPath);
    cmd1.append(' ');
    cmd1.append("2>&1) | tee -a");
    cmd1.append(' ');
    cmd1.append(PATH_ALGORITHM_TO_PROJECT).append(PROJECT_DIR_UNIX).append(
      UNIX_SEPARATOR).append(stdOut);
    cmd1.append(')');

    /*
     * Cria um configurador com apenas o parmetro de log para testar a linha de
     * comando gerada por ele.
     */
    FileParameterValue file = new FileParameterValue(logPath, TEXT_FILE_TYPE);

    OutputFileParameter logFileParameter =
      OutputFileParameter.createLogFile(DEFAULT_COMMAND_LINE_PATTERN);
    logFileParameter.setValue(file);

    ParameterGroup group = new ParameterGroup(GROUP_PARAM_NAME);
    group.addParameter(logFileParameter);

    SimpleAlgorithmConfigurator configurator =
      new SimpleAlgorithmConfigurator(createAlgorithmVersion(), null,
        SIMPLE_EXECUTION, null, false, COMMAND, null, null, true, CMD_PATTERN);
    configurator.addGroup(group);
    configurator.setStandardOutputFile(new FileParameterValue(stdOut));

    CommandLineContext context =
      CommandLineContext.createSimpleContext(COMMAND_ID, PLATFORM_ID,
        fileSeparator, SERVER_DIR_UNIX, PROJECT_DIR_UNIX);
    String cmd2 = configurator.makeCommandLine(context);

    assertEquals(cmd1.toString(), cmd2);
  }

  /**
   * Testando o mtodo
   * {@link SimpleAlgorithmConfigurator#makeCommandLine(CommandLineContext)}:
   * fornecendo o identificador do comando.
   */
  public void testMakeCommandLine6() {

    SimpleAlgorithmConfigurator configurator =
      new SimpleAlgorithmConfigurator(createAlgorithmVersion(), null,
        SIMPLE_EXECUTION, null, PROVIDE_ID, COMMAND, null, null, true, CMD_PATTERN);

    String cmd1 = "";
    cmd1 += "(";
    cmd1 += CD_TO_ALGORITHM_DIR_COMMAND_UNIX;
    cmd1 += " ";
    cmd1 += "(." + UNIX_SEPARATOR + COMMAND;
    cmd1 += " ";
    cmd1 += "cmdId=" + COMMAND_ID;
    cmd1 += "))";
    CommandLineContext context =
      CommandLineContext.createSimpleContext(COMMAND_ID, PLATFORM_ID,
        UNIX_SEPARATOR, SERVER_DIR_UNIX, "PROJECT_DIR");
    String cmd2 = configurator.makeCommandLine(context);
    assertEquals(cmd1, cmd2);
  }

  /**
   * Testando o mtodo
   * {@link SimpleAlgorithmConfigurator#makeCommandLine(CommandLineContext)}:
   * execuo paralela.
   */
  public void testMakeCommandLine8() {
    SimpleAlgorithmConfigurator configurator =
      new SimpleAlgorithmConfigurator(createAlgorithmVersion(), null,
        ExecutionType.PARALLEL, null, false, COMMAND, null, null, true, CMD_PATTERN);
    int cpuCount = 3;
    String cmd1 = "";
    cmd1 += "(";
    cmd1 += CD_TO_ALGORITHM_DIR_COMMAND_UNIX;
    cmd1 += " ";
    cmd1 += "(." + UNIX_SEPARATOR + COMMAND;
    cmd1 += " ";
    cmd1 += "numCPU=" + cpuCount;
    cmd1 += "))";

    String fileSeparator = UNIX_SEPARATOR;

    CommandLineContext context =
      CommandLineContext.createParallelContext("TEST@TEST", PLATFORM_ID,
        fileSeparator, SERVER_DIR_UNIX, PROJECT_DIR_UNIX, cpuCount);
    String cmd2 = configurator.makeCommandLine(context);
    assertEquals(cmd1, cmd2);
  }

  /**
   * Testando o mtodo
   * {@link SimpleAlgorithmConfigurator#makeCommandLine(CommandLineContext)}:
   * parmetro oculto.
   */
  public void testMakeCommandLine14() {
    SimpleAlgorithmConfigurator configurator =
      new SimpleAlgorithmConfigurator(createAlgorithmVersion(), null,
        SIMPLE_EXECUTION, null, false, COMMAND, null, null, true, CMD_PATTERN);
    HiddenParameter parameter1 =
      new HiddenParameter("PARAM_1", "VALOR_1", DEFAULT_COMMAND_LINE_PATTERN);
    configurator.addHiddenParameter(parameter1);
    String cmd1 = "";
    cmd1 += "(";
    cmd1 += CD_TO_ALGORITHM_DIR_COMMAND_UNIX;
    cmd1 += " ";
    cmd1 += "(." + UNIX_SEPARATOR + COMMAND + " PARAM_1=VALOR_1";
    cmd1 += "))";
    CommandLineContext context =
      CommandLineContext.createSimpleContext(COMMAND_ID, PLATFORM_ID,
        UNIX_SEPARATOR, SERVER_DIR_UNIX, "PROJECT_DIR");
    String cmd2 = configurator.makeCommandLine(context);
    assertEquals(cmd1, cmd2);
  }

  /**
   * Testando o mtodo
   * {@link SimpleAlgorithmConfigurator#makeCommandLine(CommandLineContext)}:
   * execuo distribuda.
   */
  public void testMakeCommandLine15() {
    SimpleAlgorithmConfigurator configurator =
      new SimpleAlgorithmConfigurator(createAlgorithmVersion(), null,
        ExecutionType.DISTRIBUTED, null, false, COMMAND, null, null, true, CMD_PATTERN);
    int executionCount = 10;
    List<String> sgaNames = new ArrayList<String>();
    String sgaName1 = "sga1";
    String sgaName2 = "sga2";
    sgaNames.add(sgaName1);
    sgaNames.add(sgaName2);
    String cmd1 = "";
    cmd1 += "(";
    cmd1 += CD_TO_ALGORITHM_DIR_COMMAND_UNIX;
    cmd1 += " ";
    cmd1 += "(." + UNIX_SEPARATOR + COMMAND;
    cmd1 += " nexec=" + executionCount + " ";
    cmd1 += "servidores=";
    String separator = "";
    for (String sgaName : sgaNames) {
      cmd1 += separator;
      cmd1 += sgaName;
      separator = ",";
    }
    cmd1 += "))";

    String fileSeparator = UNIX_SEPARATOR;
    CommandLineContext context =
      CommandLineContext.createDistributedContext(COMMAND_ID, PLATFORM_ID,
        fileSeparator, SERVER_DIR_UNIX, PROJECT_DIR_UNIX, sgaNames,
        executionCount);
    String cmd2 = configurator.makeCommandLine(context);
    assertEquals(cmd1, cmd2);
  }

  /**
   * Testando o mtodo
   * {@link SimpleAlgorithmConfigurator#makeCommandLine(CommandLineContext)}:
   * com parmetros utilizando diferentes formatos de linha de comando.
   */
  public void testMakeCommandLine16() {
    SimpleAlgorithmConfigurator configurator =
      new SimpleAlgorithmConfigurator(createAlgorithmVersion(), null,
        SIMPLE_EXECUTION, null, false, COMMAND, null, null, true, CMD_PATTERN);
    ParameterGroup group = new ParameterGroup(GROUP_PARAM_NAME);
    TextParameter textParameter1 =
      new TextParameter(PARAM_NAME + 1, PARAM_LABEL + 1, PARAM_DESCRIPTION + 1,
        PARAM_DEFAULT_VALUE + 1, false, DEFAULT_COMMAND_LINE_PATTERN, null);
    TextParameter textParameter2 =
      new TextParameter(PARAM_NAME + 2, PARAM_LABEL + 2, PARAM_DESCRIPTION + 2,
        PARAM_DEFAULT_VALUE + 2, false, null, null);
    TextParameter textParameter3 =
      new TextParameter(PARAM_NAME + 3, PARAM_LABEL + 3, PARAM_DESCRIPTION + 3,
        PARAM_DEFAULT_VALUE + 3, false,
        "--$NOME_DO_PARAMETRO [$VALOR_DO_PARAMETRO]", null);
    TextParameter textParameter4 =
      new TextParameter(PARAM_NAME + 4, PARAM_LABEL + 4, PARAM_DESCRIPTION + 4,
        PARAM_DEFAULT_VALUE + 4, false, "-$NOME_DO_PARAMETRO", null);
    TextParameter textParameter5 =
      new TextParameter(PARAM_NAME + 5, PARAM_LABEL + 5, PARAM_DESCRIPTION + 5,
        PARAM_DEFAULT_VALUE + 5, false, "($VALOR_DO_PARAMETRO)", null);
    group.addParameter(textParameter1);
    group.addParameter(textParameter2);
    group.addParameter(textParameter3);
    group.addParameter(textParameter4);
    group.addParameter(textParameter5);
    configurator.addGroup(group);
    String cmd = "";
    cmd += "(";
    cmd += CD_TO_ALGORITHM_DIR_COMMAND_UNIX;
    cmd += " ";
    cmd += "(." + UNIX_SEPARATOR + COMMAND;
    cmd += " ";
    cmd += PARAM_NAME + "1=" + PARAM_DEFAULT_VALUE + "1";
    cmd += " ";
    cmd += "--" + PARAM_NAME + "3 [" + PARAM_DEFAULT_VALUE + "3]";
    cmd += " ";
    cmd += "-" + PARAM_NAME + "4";
    cmd += " ";
    cmd += "(" + PARAM_DEFAULT_VALUE + "5)";
    cmd += "))";
    CommandLineContext context =
      CommandLineContext.createSimpleContext(COMMAND_ID, PLATFORM_ID,
        UNIX_SEPARATOR, SERVER_DIR_UNIX, "PROJECT_DIR");
    assertEquals(cmd, configurator.makeCommandLine(context));
  }

  public void testGetInputFiles1() {
    SimpleAlgorithmConfigurator configurator =
      new SimpleAlgorithmConfigurator(createAlgorithmVersion(), null,
        ExecutionType.MULTIPLE, null, false, COMMAND, null, null, true, CMD_PATTERN);
    assertNotNull(configurator.getInputFiles());
    assertTrue(configurator.getInputFiles().isEmpty());
  }

  public void testGetInputFiles2() {
    SimpleAlgorithmConfigurator configurator =
      new SimpleAlgorithmConfigurator(createAlgorithmVersion(), null,
        ExecutionType.MULTIPLE, null, false, COMMAND, null, null, true, CMD_PATTERN);
    InputFileParameter inputFileParameter =
      new InputFileParameter(INPUT_FILE_PARAMETER_NAME,
        INPUT_FILE_PARAMETER_LABEL, INPUT_FILE_PARAMETER_LABEL, false,
        DEFAULT_COMMAND_LINE_PATTERN, null,
        FileParameterMode.DIRECTORY_AND_REGULAR_FILE,
        FileParameterPipeAcceptance.TRUE);
    ParameterGroup group = new ParameterGroup(GROUP_PARAM_NAME);
    group.addParameter(inputFileParameter);
    configurator.addGroup(group);
    assertNotNull(configurator.getInputFiles());
    assertTrue(configurator.getInputFiles().isEmpty());
  }

  public void testGetInputFiles3() {
    SimpleAlgorithmConfigurator configurator =
      new SimpleAlgorithmConfigurator(createAlgorithmVersion(), null,
        ExecutionType.MULTIPLE, null, false, COMMAND, null, null, true, CMD_PATTERN);
    InputFileParameter inputFileParameter =
      new InputFileParameter(INPUT_FILE_PARAMETER_NAME,
        INPUT_FILE_PARAMETER_LABEL, INPUT_FILE_PARAMETER_LABEL, false,
        DEFAULT_COMMAND_LINE_PATTERN, null,
        FileParameterMode.DIRECTORY_AND_REGULAR_FILE,
        FileParameterPipeAcceptance.TRUE);
    inputFileParameter.setValue(new FileParameterValue(DIR_NAME,
      ProjectFileType.DIRECTORY_TYPE));
    ParameterGroup group = new ParameterGroup(GROUP_PARAM_NAME);
    group.addParameter(inputFileParameter);
    configurator.addGroup(group);
    assertNotNull(configurator.getInputFiles());
    assertTrue(configurator.getInputFiles().isEmpty());
  }

  public void testGetInputFiles4() {
    SimpleAlgorithmConfigurator configurator =
      new SimpleAlgorithmConfigurator(createAlgorithmVersion(), null,
        ExecutionType.MULTIPLE, null, false, COMMAND, null, null, true, CMD_PATTERN);
    InputFileParameter inputFileParameter =
      new InputFileParameter(INPUT_FILE_PARAMETER_NAME,
        INPUT_FILE_PARAMETER_LABEL, INPUT_FILE_PARAMETER_LABEL, false,
        DEFAULT_COMMAND_LINE_PATTERN, null,
        FileParameterMode.DIRECTORY_AND_REGULAR_FILE,
        FileParameterPipeAcceptance.TRUE);
    Set<FileParameterValue> inputFiles = new HashSet<FileParameterValue>();
    FileParameterValue file =
      new FileParameterValue(TEXT_FILE_NAME, TEXT_FILE_TYPE);
    inputFiles.add(file);
    inputFileParameter.setValue(file);
    ParameterGroup group = new ParameterGroup(GROUP_PARAM_NAME);
    group.addParameter(inputFileParameter);
    configurator.addGroup(group);
    assertEquals(inputFiles, configurator.getInputFiles());
  }

  public void testGetInputFiles5() {
    SimpleAlgorithmConfigurator configurator =
      new SimpleAlgorithmConfigurator(createAlgorithmVersion(), null,
        ExecutionType.MULTIPLE, null, false, COMMAND, null, null, true, CMD_PATTERN);
    InputFileListParameter inputFileListParameter =
      new InputFileListParameter(INPUT_FILE_PARAMETER_NAME,
        INPUT_FILE_PARAMETER_LABEL, INPUT_FILE_PARAMETER_LABEL, false,
        DEFAULT_COMMAND_LINE_PATTERN, null,
        FileParameterMode.DIRECTORY_AND_REGULAR_FILE, true);
    ParameterGroup group = new ParameterGroup(GROUP_PARAM_NAME);
    group.addParameter(inputFileListParameter);
    configurator.addGroup(group);
    assertNotNull(configurator.getInputFiles());
    assertTrue(configurator.getInputFiles().isEmpty());
  }

  public void testGetInputFiles6() {
    SimpleAlgorithmConfigurator configurator =
      new SimpleAlgorithmConfigurator(createAlgorithmVersion(), null,
        ExecutionType.MULTIPLE, null, false, COMMAND, null, null, true, CMD_PATTERN);
    InputFileListParameter inputFileListParameter =
      new InputFileListParameter(INPUT_FILE_PARAMETER_NAME,
        INPUT_FILE_PARAMETER_LABEL, INPUT_FILE_PARAMETER_LABEL, false,
        DEFAULT_COMMAND_LINE_PATTERN, null,
        FileParameterMode.DIRECTORY_AND_REGULAR_FILE, true);
    FileParameterValue file1 = new FileParameterValue(GENERIC_FILE_NAME + 1);
    FileParameterValue file2 = new FileParameterValue(GENERIC_FILE_NAME + 2);
    inputFileListParameter.addFile(file1);
    inputFileListParameter.addFile(file2);
    Set<FileParameterValue> files = new HashSet<FileParameterValue>();
    files.add(file1);
    files.add(file2);
    ParameterGroup group = new ParameterGroup(GROUP_PARAM_NAME);
    group.addParameter(inputFileListParameter);
    configurator.addGroup(group);
    assertEquals(files, configurator.getInputFiles());
  }

  public void testGetInputFiles7() {
    SimpleAlgorithmConfigurator configurator =
      new SimpleAlgorithmConfigurator(createAlgorithmVersion(), null,
        ExecutionType.MULTIPLE, null, false, COMMAND, null, null, true, CMD_PATTERN);
    InputFileListParameter inputFileListParameter =
      new InputFileListParameter(INPUT_FILE_PARAMETER_NAME,
        INPUT_FILE_PARAMETER_LABEL, INPUT_FILE_PARAMETER_LABEL, false,
        DEFAULT_COMMAND_LINE_PATTERN, null,
        FileParameterMode.DIRECTORY_AND_REGULAR_FILE, true);
    FileParameterValue file1 = new FileParameterValue(GENERIC_FILE_NAME + 1);
    FileParameterValue file2 = new FileParameterValue(GENERIC_FILE_NAME + 2);
    FileParameterValue directory =
      new FileParameterValue(DIR_NAME, ProjectFileType.DIRECTORY_TYPE);
    inputFileListParameter.addFile(file1);
    inputFileListParameter.addFile(file2);
    inputFileListParameter.addFile(directory);
    Set<FileParameterValue> files = new HashSet<FileParameterValue>();
    files.add(file1);
    files.add(file2);
    ParameterGroup group = new ParameterGroup(GROUP_PARAM_NAME);
    group.addParameter(inputFileListParameter);
    configurator.addGroup(group);
    assertEquals(files, configurator.getInputFiles());
  }

  public void testGetInputFiles8() {
    SimpleAlgorithmConfigurator configurator =
      new SimpleAlgorithmConfigurator(createAlgorithmVersion(), null,
        ExecutionType.MULTIPLE, null, false, COMMAND, null, null, true, CMD_PATTERN);
    InputFileListParameter inputFileListParameter =
      new InputFileListParameter(INPUT_FILE_PARAMETER_NAME,
        INPUT_FILE_PARAMETER_LABEL, INPUT_FILE_PARAMETER_LABEL, false,
        DEFAULT_COMMAND_LINE_PATTERN, null,
        FileParameterMode.DIRECTORY_AND_REGULAR_FILE, true);
    FileParameterValue directory1 =
      new FileParameterValue(DIR_NAME + 1, ProjectFileType.DIRECTORY_TYPE);
    FileParameterValue directory2 =
      new FileParameterValue(DIR_NAME + 2, ProjectFileType.DIRECTORY_TYPE);
    inputFileListParameter.addFile(directory1);
    inputFileListParameter.addFile(directory2);
    ParameterGroup group = new ParameterGroup(GROUP_PARAM_NAME);
    group.addParameter(inputFileListParameter);
    configurator.addGroup(group);
    assertNotNull(configurator.getInputFiles());
    assertTrue(configurator.getInputFiles().isEmpty());
  }

  public void testGetOutputFiles1() {
    SimpleAlgorithmConfigurator configurator =
      new SimpleAlgorithmConfigurator(createAlgorithmVersion(), null,
        ExecutionType.MULTIPLE, null, false, COMMAND, null, null, true, CMD_PATTERN);
    assertNotNull(configurator.getOutputFiles());
    assertTrue(configurator.getOutputFiles().isEmpty());
  }

  public void testGetOutputFiles2() {
    SimpleAlgorithmConfigurator configurator =
      new SimpleAlgorithmConfigurator(createAlgorithmVersion(), null,
        ExecutionType.MULTIPLE, null, false, COMMAND, null, null, true, CMD_PATTERN);
    OutputFileParameter outputFileParameter =
      new OutputFileParameter(OUTPUT_FILE_PARAMETER_NAME,
        OUTPUT_FILE_PARAMETER_LABEL, OUTPUT_FILE_PARAMETER_LABEL, false,
        DEFAULT_COMMAND_LINE_PATTERN, null, FileParameterMode.REGULAR_FILE,
        FileParameterPipeAcceptance.TRUE, false);
    ParameterGroup group = new ParameterGroup(GROUP_PARAM_NAME);
    group.addParameter(outputFileParameter);
    configurator.addGroup(group);
    assertNotNull(configurator.getOutputFiles());
    assertTrue(configurator.getOutputFiles().isEmpty());
  }

  public void testGetOutputFiles3() {
    SimpleAlgorithmConfigurator configurator =
      new SimpleAlgorithmConfigurator(createAlgorithmVersion(), null,
        ExecutionType.MULTIPLE, null, false, COMMAND, null, null, true, CMD_PATTERN);
    OutputFileParameter outputFileParameter =
      new OutputFileParameter(OUTPUT_FILE_PARAMETER_NAME,
        OUTPUT_FILE_PARAMETER_LABEL, OUTPUT_FILE_PARAMETER_LABEL, false,
        DEFAULT_COMMAND_LINE_PATTERN, null, FileParameterMode.REGULAR_FILE,
        FileParameterPipeAcceptance.TRUE, false);
    outputFileParameter.setValue(new FileParameterValue(DIR_NAME,
      ProjectFileType.DIRECTORY_TYPE));
    ParameterGroup group = new ParameterGroup(GROUP_PARAM_NAME);
    group.addParameter(outputFileParameter);
    configurator.addGroup(group);
    assertNotNull(configurator.getOutputFiles());
    assertTrue(configurator.getOutputFiles().isEmpty());
  }

  public void testGetOutputFiles4() {
    SimpleAlgorithmConfigurator configurator =
      new SimpleAlgorithmConfigurator(createAlgorithmVersion(), null,
        ExecutionType.MULTIPLE, null, false, COMMAND, null, null, true, CMD_PATTERN);
    OutputFileParameter outputFileParameter =
      new OutputFileParameter(OUTPUT_FILE_PARAMETER_NAME,
        OUTPUT_FILE_PARAMETER_LABEL, OUTPUT_FILE_PARAMETER_LABEL, false,
        DEFAULT_COMMAND_LINE_PATTERN, null, FileParameterMode.REGULAR_FILE,
        FileParameterPipeAcceptance.TRUE, false);
    Set<FileParameterValue> outputFiles = new HashSet<FileParameterValue>();
    FileParameterValue file =
      new FileParameterValue(TEXT_FILE_NAME, TEXT_FILE_TYPE);
    outputFiles.add(file);
    outputFileParameter.setValue(file);
    ParameterGroup group = new ParameterGroup(GROUP_PARAM_NAME);
    group.addParameter(outputFileParameter);
    configurator.addGroup(group);
    assertEquals(outputFiles, configurator.getOutputFiles());
  }

  public void testGetDirectories1() {
    SimpleAlgorithmConfigurator configurator =
      new SimpleAlgorithmConfigurator(createAlgorithmVersion(), null,
        ExecutionType.MULTIPLE, null, false, COMMAND, null, null, true, CMD_PATTERN);
    assertNotNull(configurator.getDirectories());
    assertTrue(configurator.getDirectories().isEmpty());
  }

  public void testGetDirectories2() {
    SimpleAlgorithmConfigurator configurator =
      new SimpleAlgorithmConfigurator(createAlgorithmVersion(), null,
        ExecutionType.MULTIPLE, null, false, COMMAND, null, null, true, CMD_PATTERN);
    InputFileParameter inputFileParameter =
      new InputFileParameter(INPUT_FILE_PARAMETER_NAME,
        INPUT_FILE_PARAMETER_LABEL, INPUT_FILE_PARAMETER_LABEL, false,
        DEFAULT_COMMAND_LINE_PATTERN, null,
        FileParameterMode.DIRECTORY_AND_REGULAR_FILE,
        FileParameterPipeAcceptance.TRUE);
    OutputFileParameter outputFileParameter =
      new OutputFileParameter(OUTPUT_FILE_PARAMETER_NAME,
        OUTPUT_FILE_PARAMETER_LABEL, OUTPUT_FILE_PARAMETER_LABEL, false,
        DEFAULT_COMMAND_LINE_PATTERN, null, FileParameterMode.DIRECTORY,
        FileParameterPipeAcceptance.TRUE, false);
    ParameterGroup group = new ParameterGroup(GROUP_PARAM_NAME);
    group.addParameter(inputFileParameter);
    group.addParameter(outputFileParameter);
    configurator.addGroup(group);
    assertNotNull(configurator.getDirectories());
    assertTrue(configurator.getDirectories().isEmpty());
  }

  public void testGetDirectories3() {
    SimpleAlgorithmConfigurator configurator =
      new SimpleAlgorithmConfigurator(createAlgorithmVersion(), null,
        ExecutionType.MULTIPLE, null, false, COMMAND, null, null, true, CMD_PATTERN);
    InputFileParameter inputFileParameter =
      new InputFileParameter(INPUT_FILE_PARAMETER_NAME,
        INPUT_FILE_PARAMETER_LABEL, INPUT_FILE_PARAMETER_LABEL, false,
        DEFAULT_COMMAND_LINE_PATTERN, null,
        FileParameterMode.DIRECTORY_AND_REGULAR_FILE,
        FileParameterPipeAcceptance.TRUE);
    FileParameterValue inputFile =
      new FileParameterValue("entrada.txt", TEXT_FILE_TYPE);
    inputFileParameter.setValue(inputFile);
    OutputFileParameter outputFileParameter =
      new OutputFileParameter(OUTPUT_FILE_PARAMETER_NAME,
        OUTPUT_FILE_PARAMETER_LABEL, OUTPUT_FILE_PARAMETER_LABEL, false,
        DEFAULT_COMMAND_LINE_PATTERN, null, FileParameterMode.DIRECTORY,
        FileParameterPipeAcceptance.TRUE, false);
    FileParameterValue outputFile =
      new FileParameterValue("saida.txt", TEXT_FILE_TYPE);
    outputFileParameter.setValue(outputFile);
    ParameterGroup group = new ParameterGroup(GROUP_PARAM_NAME);
    group.addParameter(inputFileParameter);
    group.addParameter(outputFileParameter);
    configurator.addGroup(group);
    assertNotNull(configurator.getDirectories());
    assertTrue(configurator.getDirectories().isEmpty());
  }

  public void testGetDirectories4() {
    SimpleAlgorithmConfigurator configurator =
      new SimpleAlgorithmConfigurator(createAlgorithmVersion(), null,
        ExecutionType.MULTIPLE, null, false, COMMAND, null, null, true, CMD_PATTERN);
    InputFileParameter inputFileParameter =
      new InputFileParameter(INPUT_FILE_PARAMETER_NAME,
        INPUT_FILE_PARAMETER_LABEL, INPUT_FILE_PARAMETER_LABEL, false,
        DEFAULT_COMMAND_LINE_PATTERN, null,
        FileParameterMode.DIRECTORY_AND_REGULAR_FILE,
        FileParameterPipeAcceptance.TRUE);
    FileParameterValue inputFile =
      new FileParameterValue("entrada", ProjectFileType.DIRECTORY_TYPE);
    inputFileParameter.setValue(inputFile);
    OutputFileParameter outputFileParameter =
      new OutputFileParameter(OUTPUT_FILE_PARAMETER_NAME,
        OUTPUT_FILE_PARAMETER_LABEL, OUTPUT_FILE_PARAMETER_LABEL, false,
        DEFAULT_COMMAND_LINE_PATTERN, null, FileParameterMode.DIRECTORY,
        FileParameterPipeAcceptance.TRUE, false);
    FileParameterValue outputFile =
      new FileParameterValue("saida", ProjectFileType.DIRECTORY_TYPE);
    outputFileParameter.setValue(outputFile);
    Set<FileParameterValue> directories = new HashSet<FileParameterValue>();
    directories.add(inputFile);
    directories.add(outputFile);
    ParameterGroup group = new ParameterGroup(GROUP_PARAM_NAME);
    group.addParameter(inputFileParameter);
    group.addParameter(outputFileParameter);
    configurator.addGroup(group);
    assertEquals(directories, configurator.getDirectories());
  }

  public void testGetDirectories5() {
    SimpleAlgorithmConfigurator configurator =
      new SimpleAlgorithmConfigurator(createAlgorithmVersion(), null,
        ExecutionType.MULTIPLE, null, false, COMMAND, null, null, true, CMD_PATTERN);
    InputFileListParameter inputFileListParameter =
      new InputFileListParameter(INPUT_FILE_PARAMETER_NAME,
        INPUT_FILE_PARAMETER_LABEL, INPUT_FILE_PARAMETER_LABEL, false,
        DEFAULT_COMMAND_LINE_PATTERN, null,
        FileParameterMode.DIRECTORY_AND_REGULAR_FILE, true);
    ParameterGroup group = new ParameterGroup(GROUP_PARAM_NAME);
    group.addParameter(inputFileListParameter);
    configurator.addGroup(group);
    assertNotNull(configurator.getDirectories());
    assertTrue(configurator.getDirectories().isEmpty());
  }

  public void testGetDirectories6() {
    SimpleAlgorithmConfigurator configurator =
      new SimpleAlgorithmConfigurator(createAlgorithmVersion(), null,
        ExecutionType.MULTIPLE, null, false, COMMAND, null, null, true, CMD_PATTERN);
    InputFileListParameter inputFileListParameter =
      new InputFileListParameter(INPUT_FILE_PARAMETER_NAME,
        INPUT_FILE_PARAMETER_LABEL, INPUT_FILE_PARAMETER_LABEL, false,
        DEFAULT_COMMAND_LINE_PATTERN, null,
        FileParameterMode.DIRECTORY_AND_REGULAR_FILE, true);
    FileParameterValue file1 = new FileParameterValue(GENERIC_FILE_NAME + 1);
    FileParameterValue file2 = new FileParameterValue(GENERIC_FILE_NAME + 2);
    inputFileListParameter.addFile(file1);
    inputFileListParameter.addFile(file2);
    ParameterGroup group = new ParameterGroup(GROUP_PARAM_NAME);
    group.addParameter(inputFileListParameter);
    configurator.addGroup(group);
    assertNotNull(configurator.getDirectories());
    assertTrue(configurator.getDirectories().isEmpty());
  }

  public void testGetDirectories7() {
    SimpleAlgorithmConfigurator configurator =
      new SimpleAlgorithmConfigurator(createAlgorithmVersion(), null,
        ExecutionType.MULTIPLE, null, false, COMMAND, null, null, true, CMD_PATTERN);
    InputFileListParameter inputFileListParameter =
      new InputFileListParameter(INPUT_FILE_PARAMETER_NAME,
        INPUT_FILE_PARAMETER_LABEL, INPUT_FILE_PARAMETER_LABEL, false,
        DEFAULT_COMMAND_LINE_PATTERN, null,
        FileParameterMode.DIRECTORY_AND_REGULAR_FILE, true);
    FileParameterValue file1 = new FileParameterValue(GENERIC_FILE_NAME + 1);
    FileParameterValue file2 = new FileParameterValue(GENERIC_FILE_NAME + 2);
    FileParameterValue directory1 =
      new FileParameterValue(DIR_NAME + 1, ProjectFileType.DIRECTORY_TYPE);
    FileParameterValue directory2 =
      new FileParameterValue(DIR_NAME + 2, ProjectFileType.DIRECTORY_TYPE);
    inputFileListParameter.addFile(file1);
    inputFileListParameter.addFile(file2);
    inputFileListParameter.addFile(directory1);
    inputFileListParameter.addFile(directory2);
    Set<FileParameterValue> files = new HashSet<FileParameterValue>();
    files.add(directory1);
    files.add(directory2);
    ParameterGroup group = new ParameterGroup(GROUP_PARAM_NAME);
    group.addParameter(inputFileListParameter);
    configurator.addGroup(group);
    assertEquals(files, configurator.getDirectories());
  }

  public void testGetDirectories8() {
    SimpleAlgorithmConfigurator configurator =
      new SimpleAlgorithmConfigurator(createAlgorithmVersion(), null,
        ExecutionType.MULTIPLE, null, false, COMMAND, null, null, true, CMD_PATTERN);
    InputFileListParameter inputFileListParameter =
      new InputFileListParameter(INPUT_FILE_PARAMETER_NAME,
        INPUT_FILE_PARAMETER_LABEL, INPUT_FILE_PARAMETER_LABEL, false,
        DEFAULT_COMMAND_LINE_PATTERN, null,
        FileParameterMode.DIRECTORY_AND_REGULAR_FILE, true);
    FileParameterValue directory1 =
      new FileParameterValue(DIR_NAME + 1, ProjectFileType.DIRECTORY_TYPE);
    FileParameterValue directory2 =
      new FileParameterValue(DIR_NAME + 2, ProjectFileType.DIRECTORY_TYPE);
    inputFileListParameter.addFile(directory1);
    inputFileListParameter.addFile(directory2);
    ParameterGroup group = new ParameterGroup(GROUP_PARAM_NAME);
    group.addParameter(inputFileListParameter);
    configurator.addGroup(group);
    Set<FileParameterValue> files = new HashSet<FileParameterValue>();
    files.add(directory1);
    files.add(directory2);
    assertEquals(files, configurator.getDirectories());
  }

  public void testGetBinaries1() {
    SimpleAlgorithmConfigurator configurator =
      new SimpleAlgorithmConfigurator(createAlgorithmVersion(), null,
        ExecutionType.MULTIPLE, null, false, COMMAND, null, null, true, CMD_PATTERN);
    Set<String> actualBinaryDirectories =
      configurator.getBinaryDirectories(PLATFORM_ID, UNIX_SEPARATOR);
    SortedSet<String> expectedBinaryDirectories = new TreeSet<String>();
    expectedBinaryDirectories.add(ALGORITHM_DIR_UNIX);
    assertEquals(expectedBinaryDirectories, actualBinaryDirectories);
  }

  public void testGetBinaries2() {
    SimpleAlgorithmConfigurator configurator =
      new SimpleAlgorithmConfigurator(createAlgorithmVersion(), null,
        ExecutionType.MULTIPLE, null, false, COMMAND, null, null, true, CMD_PATTERN);
    SortedSet<String> expectedBinaryDirectories = new TreeSet<String>();
    expectedBinaryDirectories.add(ALGORITHM_DIR_WINDOWS);
    Set<String> actualBinaryDirectories =
      configurator.getBinaryDirectories(PLATFORM_ID, WINDOWS_SEPARATOR);
    assertEquals(expectedBinaryDirectories, actualBinaryDirectories);
  }

  public void testParameterWasSetVisible1() {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup(GROUP_PARAM_NAME);
    SimpleParameter<Object> parameter =
      new MockSimpleParameter(PARAM_NAME, PARAM_LABEL, PARAM_DESCRIPTION, null,
        false, DEFAULT_COMMAND_LINE_PATTERN);
    group.addParameter(parameter);
    CheckAlgorithmConfiguratorListener listener =
      new CheckAlgorithmConfiguratorListener();
    listener.allowParameterWasSetVisible();
    configurator.addGroup(group);
    configurator.addAlgorithmConfiguratorListener(listener);
    parameter.setVisible(false);
    assertTrue(listener.parameterWasSetVisible());
    assertEquals(1, listener.getEventCount());
  }

  public void testParameterWasSetVisible2() {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup(GROUP_PARAM_NAME);
    SimpleParameter<Object> parameter =
      new MockSimpleParameter(PARAM_NAME, PARAM_LABEL, PARAM_DESCRIPTION, null,
        false, DEFAULT_COMMAND_LINE_PATTERN);
    parameter.setVisible(false);
    group.addParameter(parameter);
    CheckAlgorithmConfiguratorListener listener =
      new CheckAlgorithmConfiguratorListener();
    listener.allowParameterWasSetVisible();
    configurator.addGroup(group);
    configurator.addAlgorithmConfiguratorListener(listener);
    parameter.setVisible(true);
    assertTrue(listener.parameterWasSetVisible());
    assertEquals(1, listener.getEventCount());
  }

  public void testParameterWasSetEnabled1() {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup(GROUP_PARAM_NAME);
    SimpleParameter<Object> parameter =
      new MockSimpleParameter(PARAM_NAME, PARAM_LABEL, PARAM_DESCRIPTION, null,
        false, DEFAULT_COMMAND_LINE_PATTERN);
    group.addParameter(parameter);
    CheckAlgorithmConfiguratorListener listener =
      new CheckAlgorithmConfiguratorListener();
    listener.allowParameterWasSetEnabled();
    configurator.addGroup(group);
    configurator.addAlgorithmConfiguratorListener(listener);
    parameter.setEnabled(false);
    assertTrue(listener.parameterWasSetEnabled());
    assertEquals(1, listener.getEventCount());
  }

  public void testParameterWasSetEnabled2() {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup(GROUP_PARAM_NAME);
    SimpleParameter<Object> parameter =
      new MockSimpleParameter(PARAM_NAME, PARAM_LABEL, PARAM_DESCRIPTION, null,
        false, DEFAULT_COMMAND_LINE_PATTERN);
    parameter.setEnabled(false);
    group.addParameter(parameter);
    CheckAlgorithmConfiguratorListener listener =
      new CheckAlgorithmConfiguratorListener();
    listener.allowParameterWasSetEnabled();
    configurator.addGroup(group);
    configurator.addAlgorithmConfiguratorListener(listener);
    parameter.setEnabled(true);
    assertTrue(listener.parameterWasSetEnabled());
    assertEquals(1, listener.getEventCount());
  }

  private SimpleAlgorithmConfigurator createAlgorithmConfigurator() {
    AlgorithmVersionInfo algorithmVersionInfo = createAlgorithmVersion();
    String abbreviation = null;
    String shell = null;
    String inputFileParameterName = null;
    return new SimpleAlgorithmConfigurator(algorithmVersionInfo,
      ALGORITHM_DESCRIPTION, SIMPLE_EXECUTION, abbreviation, DONT_PROVIDE_ID,
      COMMAND, shell, inputFileParameterName, true, CMD_PATTERN);
  }

  private AlgorithmVersionInfo createAlgorithmVersion() {
    String algorithmName = "Algoritmo";
    int major = 1;
    int minor = 2;
    int patch = 3;
    AlgorithmVersionInfo algorithmVersionInfo =
      createAlgorithmVersion(algorithmName, major, minor, patch);
    return algorithmVersionInfo;
  }

  private AlgorithmVersionInfo createAlgorithmVersion(String algorithmName,
    int major, int minor, int patch) {
    AlgorithmInfo algorithmInfo =
      new AlgorithmInfo("ID " + algorithmName, algorithmName, "DIR "
        + algorithmName, new Hashtable<String, String>());
    AlgorithmVersionId algorithmVersionId =
      new AlgorithmVersionId(major, minor, patch);
    AlgorithmVersionInfo algorithmVersionInfo =
      new AlgorithmVersionInfo(algorithmInfo, algorithmVersionId,
        new Hashtable<String, List<FileInfo>>(),
        new Hashtable<String, String>());
    return algorithmVersionInfo;
  }

  private static final String PARAM_DEFAULT_VALUE = "Valor";
  private static final String PARAM_NEW_VALUE = "NovoValor";
  private static final String PARAM_DESCRIPTION = "Descrio";
  private static final String PARAM_LABEL = "Rtulo";
  private static final String PARAM_NAME = "Nome";

  private static final String TEXT_FILE_TYPE = "TEXT";

  private static final String OUTPUT_FILE_PARAMETER_LABEL = "Saida";
  private static final String INPUT_FILE_PARAMETER_LABEL = "Entrada";
  private static final String OUTPUT_FILE_PARAMETER_NAME = "OUT";
  private static final String INPUT_FILE_PARAMETER_NAME = "IN";
  private static final String GROUP_PARAM_NAME = "Grupo de parmetros";

  private static final String DIR_NAME = "diretorio";
  private static final String TEXT_FILE_NAME = "arquivo.txt";
  private static final String GENERIC_FILE_NAME = "arquivo";

  private static final boolean DONT_PROVIDE_ID = false;
  private static final boolean PROVIDE_ID = true;

  private static final String ALGORITHM_DESCRIPTION = "Descrio";
  private static final String ABBREVIATION = "Abreviatura";
  private static final String VERSION_DESCRIPTION = "Descrio da verso";

  private static final String PLATFORM_ID = "id_da_plataforma";
  private static final String COMMAND_ID = "USR@TEST";
  private static final ExecutionType SIMPLE_EXECUTION = ExecutionType.SIMPLE;
  private static final String SHELL = "Shell";
  private static final String COMMAND = "comando";

  private static final String ALTERNATE_COMMAND_LINE_PATTERN =
    "--$NOME_DO_PARAMETRO $VALOR_DO_PARAMETRO";
  private static final String DEFAULT_COMMAND_LINE_PATTERN =
    "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO";

  private static final String COMMAND_SEPARATOR = ";";
  private static final String UNIX_SEPARATOR = "/";
  private static final String WINDOWS_SEPARATOR = "\\";

  private static final String PROJECT_DIR_UNIX =
    "projects/id_do_usuario/nome_do_projeto";
  private static final String SERVER_DIR_UNIX = "../../src";
  private static final String ALGORITHM_DIR_WINDOWS =
    "algorithms\\DIR Algoritmo\\versions\\v_001_002_003\\bin\\" + PLATFORM_ID;
  private static final String SERVER_DIR_WINDOWS = "..\\..\\src";
  private static final String PATH_ALGORITHM_TO_PROJECT = "../../../../../../";
  private static final String ALGORITHM_DIR_UNIX =
    "algorithms/DIR Algoritmo/versions/v_001_002_003/bin/" + PLATFORM_ID;

  private static final String CD_TO_ALGORITHM_DIR_COMMAND_WINDOWS =
    "cd " + SERVER_DIR_WINDOWS + WINDOWS_SEPARATOR + ALGORITHM_DIR_WINDOWS
      + COMMAND_SEPARATOR;
  private static final String CD_TO_ALGORITHM_DIR_COMMAND_UNIX =
    "cd " + SERVER_DIR_UNIX + UNIX_SEPARATOR + ALGORITHM_DIR_UNIX
      + COMMAND_SEPARATOR;
}
