package csbase.logic.algorithms.parameters;

import java.util.List;

import junit.framework.TestCase;
import csbase.util.parser.exception.AutomatonException;

public final class TableParameterParserTest extends TestCase {
  public void testParseTextWithoutLines()
  throws AutomatonException {
    TableParameterParser parser = new TableParameterParser();
    List<List<String>> elementRowsList = parser.parseText("{}");
    assertNotNull(elementRowsList);
    assertTrue(elementRowsList.isEmpty());
  }
  
  public void testParseTextWithElements()
  throws AutomatonException {
    TableParameterParser parser = new TableParameterParser();
    List<List<String>> elementRowsList = parser
      .parseText("{{1,2},{3,4,5},{6,7,8,9}}");
    assertNotNull(elementRowsList);
    assertEquals(3, elementRowsList.size());
    List<String> elements1 = elementRowsList.get(0);
    assertEquals(2, elements1.size());
    assertEquals("1", elements1.get(0));
    assertEquals("2", elements1.get(1));
    List<String> elements2 = elementRowsList.get(1);
    assertEquals(3, elements2.size());
    assertEquals("3", elements2.get(0));
    assertEquals("4", elements2.get(1));
    assertEquals("5", elements2.get(2));
    List<String> elements3 = elementRowsList.get(2);
    assertEquals(4, elements3.size());
    assertEquals("6", elements3.get(0));
    assertEquals("7", elements3.get(1));
    assertEquals("8", elements3.get(2));
    assertEquals("9", elements3.get(3));
  }
  
  
  public void testParseTextWithEmptyCells()
  throws AutomatonException {
    TableParameterParser parser = new TableParameterParser();
    List<List<String>> elementRowsList = parser
      .parseText("{{1,},{,4,5},{6,,8,9}}");
    assertNotNull(elementRowsList);
    assertEquals(3, elementRowsList.size());
    List<String> elements1 = elementRowsList.get(0);
    assertEquals(2, elements1.size());
    assertEquals("1", elements1.get(0));
    assertEquals(null, elements1.get(1));
    List<String> elements2 = elementRowsList.get(1);
    assertEquals(3, elements2.size());
    assertEquals(null, elements2.get(0));
    assertEquals("4", elements2.get(1));
    assertEquals("5", elements2.get(2));
    List<String> elements3 = elementRowsList.get(2);
    assertEquals(4, elements3.size());
    assertEquals("6", elements3.get(0));
    assertEquals(null, elements3.get(1));
    assertEquals("8", elements3.get(2));
    assertEquals("9", elements3.get(3));
  }
}
