/*
 * $Author:$
 * $Date:$
 * $Release:$
 */
package csbase.logic.algorithms.parameters;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import junit.framework.TestCase;

public final class TextParameterTest extends TestCase {
  public void testTextParameterWithIsOptionalTrue() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    String defaultValue = "Valor-padro";
    boolean isOptional = true;
    String commandLinePattern = "{0}={1}";
    Integer maxCharacters = new Integer(1234);
    TextParameter parameter = new TextParameter(name, label, description,
      defaultValue, isOptional, commandLinePattern, maxCharacters);
    assertEquals(name, parameter.getName());
    assertEquals(label, parameter.getLabel());
    assertEquals(description, parameter.getDescription());
    assertEquals(defaultValue, parameter.getDefaultValue());
    assertEquals(isOptional, parameter.isOptional());
    
    assertEquals(maxCharacters, parameter.getMaxCharacters());
  }

  public void testTextParameterWithIsOptionalFalse() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    String defaultValue = "Valor-padro";
    boolean isOptional = false;
    String commandLinePattern = "{0}={1}";
    Integer maxCharacters = new Integer(1234);
    TextParameter parameter = new TextParameter(name, label, description,
      defaultValue, isOptional, commandLinePattern, maxCharacters);
    assertEquals(name, parameter.getName());
    assertEquals(label, parameter.getLabel());
    assertEquals(description, parameter.getDescription());
    assertEquals(defaultValue, parameter.getDefaultValue());
    assertEquals(isOptional, parameter.isOptional());
    
    assertEquals(maxCharacters, parameter.getMaxCharacters());
  }

  public void testTextParameterWithNullName() {
    String name = null;
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    String defaultValue = "Valor-padro";
    boolean isOptional = false;
    String commandLinePattern = "{0}={1}";
    Integer maxCharacters = new Integer(1234);
    try {
      new TextParameter(name, label, description, defaultValue, isOptional,
        commandLinePattern, maxCharacters);
      fail("No deveria ser possvel criar um parmetro com nome nulo.");
    }
    catch (IllegalArgumentException e) {}
  }

  public void testTextParameterWithNullLabel() {
    String name = "Nome do parmetro";
    String label = null;
    String description = "Descrio do parmetro";
    String defaultValue = "Valor-padro";
    boolean isOptional = false;
    String commandLinePattern = "{0}={1}";
    Integer maxCharacters = new Integer(1234);
    try {
      new TextParameter(name, label, description, defaultValue, isOptional,
        commandLinePattern, maxCharacters);
      fail("No deveria ser possvel criar um parmetro com rtulo nulo.");
    }
    catch (IllegalArgumentException e) {}
  }

  public void testTextParameterWithNullDescription() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = null;
    String defaultValue = "Valor-padro";
    boolean isOptional = false;
    String commandLinePattern = "{0}={1}";
    Integer maxCharacters = new Integer(1234);
    try {
      new TextParameter(name, label, description, defaultValue, isOptional,
        commandLinePattern, maxCharacters);
      fail(
        "No deveria ser possvel criar um parmetro com descrio "+"nula.");
    }
    catch (IllegalArgumentException e) {}
  }

  public void testTextParameterWithNullDefaultValue() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    String defaultValue = null;
    boolean isOptional = true;
    String commandLinePattern = "{0}={1}";
    Integer maxCharacters = new Integer(1234);
    TextParameter parameter = new TextParameter(name, label, description,
      defaultValue, isOptional, commandLinePattern, maxCharacters);
    assertEquals(name, parameter.getName());
    assertEquals(label, parameter.getLabel());
    assertEquals(description, parameter.getDescription());
    assertEquals(defaultValue, parameter.getDefaultValue());
    assertEquals(isOptional, parameter.isOptional());
    
    assertEquals(maxCharacters, parameter.getMaxCharacters());
  }

  public void testTextParameterWithNullMaxCharacters() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    String defaultValue = "Valor-padro";
    boolean isOptional = true;
    String commandLinePattern = "{0}={1}";
    Integer maxCharacters = null;
    TextParameter parameter = new TextParameter(name, label, description,
      defaultValue, isOptional, commandLinePattern, maxCharacters);
    assertEquals(name, parameter.getName());
    assertEquals(label, parameter.getLabel());
    assertEquals(description, parameter.getDescription());
    assertEquals(defaultValue, parameter.getDefaultValue());
    assertEquals(isOptional, parameter.isOptional());
    
    assertNull(parameter.getMaxCharacters());
  }

  public void testGetValueAsText() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    String defaultValue = "Valor-padro do parmetro";
    boolean isOptional = true;
    String commandLinePattern = "{0}={1}";
    TextParameter textParameter = new TextParameter(name, label, description,
      defaultValue, isOptional, commandLinePattern, null);
    assertEquals(defaultValue, textParameter.getValueAsText());
  }

  public void testGetValueAsTextWithNullValue() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    String defaultValue = null;
    boolean isOptional = true;
    String commandLinePattern = "{0}={1}";
    TextParameter textParameter = new TextParameter(name, label, description,
      defaultValue, isOptional, commandLinePattern, null);
    assertNull(textParameter.getValueAsText());
  }

  public void testSetValueAsText() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    String defaultValue = "Valor padro do parmetro";
    boolean isOptional = true;
    String commandLinePattern = "{0}={1}";
    TextParameter textParameter = new TextParameter(name, label, description,
      defaultValue, isOptional, commandLinePattern, null);
    String value = "Valor do parmetro";
    textParameter.setValueAsText(value);
    assertEquals(value, textParameter.getValueAsText());
    assertEquals(value, textParameter.getValue());
  }

  public void testSetValueAsTextWithNull() {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    String defaultValue = "Valor padro do parmetro";
    boolean isOptional = true;
    String commandLinePattern = "{0}={1}";
    TextParameter textParameter = new TextParameter(name, label, description,
      defaultValue, isOptional, commandLinePattern, null);
    textParameter.setValueAsText(null);
    assertNull(textParameter.getValueAsText());
    assertNull(textParameter.getValue());
  }

  public void testSerialize()
  throws IOException, ClassNotFoundException {
    String name = "Nome do parmetro";
    String label = "Rtulo do parmetro";
    String description = "Descrio do parmetro";
    String defaultValue = "Valor-padro";
    boolean isOptional = true;
    String commandLinePattern = "{0}={1}";
    Integer maxCharacters = new Integer(1234);
    TextParameter outputParameter = new TextParameter(name, label, description,
      defaultValue, isOptional, commandLinePattern, maxCharacters);
    ByteArrayOutputStream outputSource = new ByteArrayOutputStream();
    ObjectOutputStream objectOutput = new ObjectOutputStream(outputSource);
    objectOutput.writeObject(outputParameter);
    ByteArrayInputStream inputSource =
      new ByteArrayInputStream(outputSource.toByteArray());
    ObjectInputStream objectInput = new ObjectInputStream(inputSource);
    TextParameter inputParameter = (TextParameter)objectInput.readObject();
    assertEquals(outputParameter, inputParameter);
    assertEquals(outputParameter.getName(), inputParameter.getName());
    assertEquals(outputParameter.getLabel(), inputParameter.getLabel());
    assertEquals(
      outputParameter.getDescription(), inputParameter.getDescription());
    assertEquals(
      outputParameter.getDefaultValue(), inputParameter.getDefaultValue());
    assertEquals(outputParameter.isOptional(), inputParameter.isOptional());
    assertEquals(
      outputParameter.getMaxCharacters(), inputParameter.getMaxCharacters());
  }
}
