/*
 * $Author:$ $Date:$ $Release:$
 */
package csbase.logic.algorithms.parameters;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.text.MessageFormat;
import java.util.Arrays;
import java.util.Hashtable;
import java.util.List;

import junit.framework.TestCase;
import tecgraf.javautils.configurationmanager.ConfigurationManager;
import csbase.exception.ParseException;
import csbase.exception.algorithms.FormulaCreationException;
import csbase.exception.algorithms.FormulaEvaluationException;
import csbase.logic.FileInfo;
import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.AlgorithmVersionId;
import csbase.logic.algorithms.AlgorithmVersionInfo;
import csbase.logic.algorithms.ExecutionType;

public final class ValidationExpressionTest extends TestCase {
 
  /**
   * Pattern padro.
   */
  private static final String CMD_PATTERN = "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO";
  
  public void testExpression1() throws FormulaCreationException, ParseException {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup("Nome do grupo");
    configurator.addGroup(group);
    String parameterName = "Nome_do_Parametro";
    SimpleParameter parameter =
      new MockSimpleParameter(parameterName, "Rtulo do parmetro",
        "Descrio do parmetro", "Valor_do_parametro", false, "{0}={1}");
    group.addParameter(parameter);
    String expressionText =
      MessageFormat.format("{0} && {0}", new Object[] { parameterName });
    String errorMessage = "Mensagem de erro";
    ValidationExpression expression =
      new ValidationExpression(configurator, expressionText, errorMessage);
    configurator.addExpression(expression);
    assertEquals(expressionText, expression.toString());
    assertEquals(errorMessage, expression.getErrorMessage());
  }

  public void testExpression2() throws FormulaCreationException, ParseException {
    String expressionText = "!Parametro";
    String errorMessage = "Mensagem de erro";
    try {
      new ValidationExpression(null, expressionText, errorMessage);
      fail("No deveria ser possvel criar uma expresso sem uma verso.");
    }
    catch (IllegalArgumentException e) {
    }
  }

  public void testExpression3() throws FormulaCreationException, ParseException {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup("Nome do grupo");
    configurator.addGroup(group);
    String parameterName = "Nome_do_Parametro";
    SimpleParameter parameter =
      new MockSimpleParameter(parameterName, "Rtulo do parmetro",
        "Descrio do parmetro", "Valor_do_parametro", false, "{0}={1}");
    group.addParameter(parameter);
    String errorMessage = "Mensagem de erro";
    try {
      new ValidationExpression(configurator, null, errorMessage);
      fail("No deveria ser possvel criar uma expresso sem o texto.");
    }
    catch (IllegalArgumentException e) {
    }
  }

  public void testExpression4() throws FormulaCreationException {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup("Nome do grupo");
    configurator.addGroup(group);
    String parameterName = "Nome_do_Parametro";
    SimpleParameter parameter =
      new BooleanParameter(parameterName, "Parmetro", "Parmetro", true,
        "{0}={1}", null, null);
    group.addParameter(parameter);
    String expressionText =
      MessageFormat.format("+++{0}", new Object[] { parameterName });
    String errorMessage = "Mensagem de erro";
    try {
      new ValidationExpression(configurator, expressionText, errorMessage);
      fail("Deveria ter ocorrido um erro de sintaxe.");
    }
    catch (ParseException e) {
    }
  }

  public void testExpression5() throws FormulaCreationException, ParseException {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup("Nome do grupo");
    configurator.addGroup(group);
    String parameterName = "Nome_do_Parametro";
    SimpleParameter parameter =
      new MockSimpleParameter(parameterName, "Rtulo do parmetro",
        "Descrio do parmetro", "Valor_do_parametro", false, "{0}={1}");
    group.addParameter(parameter);
    String expressionText =
      MessageFormat.format("{0} && {0}", new Object[] { parameterName });
    try {
      new ValidationExpression(configurator, expressionText, null);
      fail("No deveria ser possvel criar uma expresso sem uma mensagem de erro.");
    }
    catch (IllegalArgumentException e) {
    }
  }

  public void testEquals1() throws FormulaCreationException, ParseException {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup("Nome do grupo");
    configurator.addGroup(group);
    String parameterName = "Nome_do_Parametro";
    SimpleParameter parameter =
      new MockSimpleParameter(parameterName, "Rtulo do parmetro",
        "Descrio do parmetro", "Valor_do_parametro", false, "{0}={1}");
    group.addParameter(parameter);
    String expressionText =
      MessageFormat.format("{0} && {0}", new Object[] { parameterName });
    String errorMessage1 = "Mensagem de erro 1";
    String errorMessage2 = "Mensagem de erro 2";
    ValidationExpression expression1 =
      new ValidationExpression(configurator, expressionText, errorMessage1);
    ValidationExpression expression2 =
      new ValidationExpression(configurator, expressionText, errorMessage2);
    assertEquals(expression1, expression2);
  }

  public void testEquals2() throws FormulaCreationException, ParseException,
    FormulaCreationException, ParseException {
    SimpleAlgorithmConfigurator configurator1 =
      createAlgorithmConfigurator("Algoritmo 1", 1, 1, 1);
    ParameterGroup group1 = new ParameterGroup("Nome do grupo");
    configurator1.addGroup(group1);
    String parameterName = "Nome_do_Parametro";
    SimpleParameter parameter1 =
      new MockSimpleParameter(parameterName, "Rtulo do parmetro",
        "Descrio do parmetro", "Valor_do_parametro", false, "{0}={1}");
    group1.addParameter(parameter1);
    SimpleAlgorithmConfigurator configurator2 =
      createAlgorithmConfigurator("Algoritmo 2", 2, 2, 2);
    ParameterGroup group2 = new ParameterGroup("Nome do grupo");
    configurator2.addGroup(group2);
    SimpleParameter parameter2 =
      new MockSimpleParameter(parameterName, "Rtulo do parmetro",
        "Descrio do parmetro", "Valor_do_parametro", false, "{0}={1}");
    group2.addParameter(parameter2);
    String expressionText =
      MessageFormat.format("{0} && {0}", new Object[] { parameterName });
    String errorMessage = "Mensagem de erro";
    ValidationExpression expression1 =
      new ValidationExpression(configurator1, expressionText, errorMessage);
    ValidationExpression expression2 =
      new ValidationExpression(configurator2, expressionText, errorMessage);
    assertFalse(expression1.equals(expression2));
  }

  public void testEquals3() throws FormulaCreationException, ParseException {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup("Nome do grupo");
    configurator.addGroup(group);
    String parameterName = "Nome_do_Parametro";
    SimpleParameter parameter =
      new MockSimpleParameter(parameterName, "Rtulo do parmetro",
        "Descrio do parmetro", "Valor_do_parametro", false, "{0}={1}");
    group.addParameter(parameter);
    String expressionText1 =
      MessageFormat.format("{0} && {0}", new Object[] { parameterName });
    String expressionText2 =
      MessageFormat.format("{0} || {0}", new Object[] { parameterName });
    String errorMessage1 = "Mensagem de erro 1";
    ValidationExpression expression1 =
      new ValidationExpression(configurator, expressionText1, errorMessage1);
    ValidationExpression expression2 =
      new ValidationExpression(configurator, expressionText2, errorMessage1);
    assertFalse(expression1.equals(expression2));
  }

  public void testEquals4() throws FormulaCreationException, ParseException {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup("Nome do grupo");
    configurator.addGroup(group);
    String parameterName = "Nome_do_Parametro";
    SimpleParameter parameter =
      new MockSimpleParameter(parameterName, "Rtulo do parmetro",
        "Descrio do parmetro", "Valor_do_parametro", false, "{0}={1}");
    group.addParameter(parameter);
    String expressionText =
      MessageFormat.format("{0} && {0}", new Object[] { parameterName });
    String errorMessage = "Mensagem de erro";
    ValidationExpression expression =
      new ValidationExpression(configurator, expressionText, errorMessage);
    configurator.addExpression(expression);
    assertFalse(expression.equals(null));
  }

  public void testEquals5() throws FormulaCreationException, ParseException {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup("Nome do grupo");
    configurator.addGroup(group);
    String parameterName = "Nome_do_Parametro";
    SimpleParameter parameter =
      new MockSimpleParameter(parameterName, "Rtulo do parmetro",
        "Descrio do parmetro", "Valor_do_parametro", false, "{0}={1}");
    group.addParameter(parameter);
    String expressionText =
      MessageFormat.format("{0} && {0}", new Object[] { parameterName });
    String errorMessage = "Mensagem de erro";
    ValidationExpression expression =
      new ValidationExpression(configurator, expressionText, errorMessage);
    configurator.addExpression(expression);
    String objectOfAnotherClass = "Objeto de outra classe";
    assertFalse(expression.equals(objectOfAnotherClass));
  }

  public void testHashCode() throws FormulaCreationException, ParseException {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup("Nome do grupo");
    configurator.addGroup(group);
    String parameterName = "Nome_do_Parametro";
    SimpleParameter parameter =
      new MockSimpleParameter(parameterName, "Rtulo do parmetro",
        "Descrio do parmetro", "Valor_do_parametro", false, "{0}={1}");
    group.addParameter(parameter);
    String expressionText =
      MessageFormat.format("{0} && {0}", new Object[] { parameterName });
    String errorMessage1 = "Mensagem de erro 1";
    String errorMessage2 = "Mensagem de erro 2";
    ValidationExpression expression1 =
      new ValidationExpression(configurator, expressionText, errorMessage1);
    ValidationExpression expression2 =
      new ValidationExpression(configurator, expressionText, errorMessage2);
    assertEquals(expression1.hashCode(), expression2.hashCode());
  }

  public void testEvaluate1() throws FormulaCreationException, ParseException,
    FormulaEvaluationException {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup("Nome do grupo");
    configurator.addGroup(group);
    String parameterName = "Nome_do_Parametro";
    SimpleParameter parameter =
      new BooleanParameter(parameterName, "Parmetro", "Parmetro", false,
        "{0}={1}", null, null);
    group.addParameter(parameter);
    String expressionText =
      MessageFormat.format("{0}", new Object[] { parameterName });
    String errorMessage = "Mensagem de erro";
    ValidationExpression expression =
      new ValidationExpression(configurator, expressionText, errorMessage);
    configurator.addExpression(expression);
    assertFalse(expression.evaluate());
  }

  public void testEvaluate2() throws FormulaCreationException, ParseException,
    FormulaEvaluationException {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup("Nome do grupo");
    configurator.addGroup(group);
    String parameterName = "Nome_do_Parametro";
    SimpleParameter parameter =
      new BooleanParameter(parameterName, "Parmetro", "Parmetro", true,
        "{0}={1}", null, null);
    group.addParameter(parameter);
    String expressionText =
      MessageFormat.format("{0}", new Object[] { parameterName });
    String errorMessage = "Mensagem de erro";
    ValidationExpression expression =
      new ValidationExpression(configurator, expressionText, errorMessage);
    configurator.addExpression(expression);
    assertTrue(expression.evaluate());
  }

  public void testEvaluate3() throws FormulaCreationException, ParseException,
    FormulaEvaluationException {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup("Nome do grupo");
    configurator.addGroup(group);
    String parameterName1 = "Nome_do_Parametro_1";
    int value1 = 2;
    IntegerParameter parameter1 =
      new IntegerParameter(parameterName1, "Rtulo do parmetro 1",
        "Descrio do parmtro 1", value1, false, "{0}={1}", null, null);
    group.addParameter(parameter1);
    String parameterName2 = "Nome_do_Parametro_2";
    int value2 = 4;
    IntegerParameter parameter2 =
      new IntegerParameter(parameterName2, "Rtulo do parmetro 2",
        "Descrio do parmtro 2", value2, false, "{0}={1}", null, null);
    group.addParameter(parameter2);
    String expressionText =
      MessageFormat.format("{0} < {1}", new Object[] { parameterName1,
          parameterName2 });
    String errorMessage = "Mensagem de erro";
    ValidationExpression expression =
      new ValidationExpression(configurator, expressionText, errorMessage);
    configurator.addExpression(expression);
    assertTrue(expression.evaluate());
  }

  public void testEvaluate4() throws FormulaCreationException, ParseException,
    FormulaEvaluationException {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup("Nome do grupo");
    configurator.addGroup(group);
    String parameterName = "Nome_do_Parametro";
    double value = 90;
    DoubleParameter parameter =
      new DoubleParameter(parameterName, "Rtulo do parmetro",
        "Descrio do parmtro", value, false, "{0}={1}", null, false, null,
        false);
    group.addParameter(parameter);
    String expressionText =
      MessageFormat.format("{0} < 180", new Object[] { parameterName,
          Double.toString(value) });
    String errorMessage = "Mensagem de erro";
    ValidationExpression expression =
      new ValidationExpression(configurator, expressionText, errorMessage);
    configurator.addExpression(expression);
    assertTrue(expression.evaluate());
  }

  public void testEvaluate5() throws FormulaCreationException, ParseException,
    FormulaEvaluationException {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup("Nome do grupo");
    configurator.addGroup(group);
    String parameterName = "Nome_do_Parametro";
    double angle = 90;
    double sin = Math.sin(angle);
    SimpleParameter parameter =
      new DoubleParameter(parameterName, "Rtulo do parmetro",
        "Descrio do parmtro", angle, false, "{0}={1}", null, false, null,
        false);
    group.addParameter(parameter);
    String expressionText =
      MessageFormat.format("sin({0}) == {1}", new Object[] { parameterName,
          new Double(sin).toString() });
    String errorMessage = "Mensagem de erro";
    ValidationExpression expression =
      new ValidationExpression(configurator, expressionText, errorMessage);
    configurator.addExpression(expression);
    assertTrue(expression.evaluate());
  }

  public void testEvaluate6() throws FormulaCreationException, ParseException,
    FormulaEvaluationException {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup("Nome do grupo");
    configurator.addGroup(group);
    String parameterName = "Nome_do_Parametro";
    SimpleParameter parameter =
      new DoubleParameter(parameterName, "Rtulo do parmetro",
        "Descrio do parmetro", null, true, "{0}={1}", null, false, null,
        false);
    group.addParameter(parameter);
    String expressionText = parameterName;
    String errorMessage = "Mensagem de erro";
    ValidationExpression expression =
      new ValidationExpression(configurator, expressionText, errorMessage);
    configurator.addExpression(expression);
    assertFalse(expression.evaluate());
  }

  public void testEvaluate7() throws FormulaCreationException, ParseException {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup("Nome do grupo");
    configurator.addGroup(group);
    String parameterName = "Nome_do_Parametro";
    SimpleParameter parameter =
      new BooleanParameter(parameterName, "Parmetro", "Parmetro", true,
        "{0}={1}", null, null);
    group.addParameter(parameter);
    String expressionText = "Outro_parametro && " + parameterName;
    String errorMessage = "Mensagem de erro";
    ValidationExpression expression =
      new ValidationExpression(configurator, expressionText, errorMessage);
    try {
      expression.evaluate();
      fail("Deveria ter ocorrido um erro.");
    }
    catch (FormulaEvaluationException e) {
    }
  }

  public void testEvaluate8() throws FormulaCreationException, ParseException {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup("Nome do grupo");
    configurator.addGroup(group);
    String parameterName = "Nome_do_Parametro";
    SimpleParameter parameter =
      new BooleanParameter(parameterName, "Parmetro", "Parmetro", true,
        "{0}={1}", null, null);
    group.addParameter(parameter);
    String expressionText =
      MessageFormat.format("funcao({0})", new Object[] { parameterName });
    String errorMessage = "Mensagem de erro";
    ValidationExpression expression =
      new ValidationExpression(configurator, expressionText, errorMessage);
    try {
      expression.evaluate();
      fail("Deveria ter ocorrido um erro.");
    }
    catch (FormulaEvaluationException e) {
    }
  }

  public void testEvaluate9() throws FormulaEvaluationException,
    FormulaCreationException, ParseException {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup("Nome do grupo");
    configurator.addGroup(group);
    String parameterName = "parametro";
    DoubleListParameter parameter =
      new DoubleListParameter(parameterName, "Rtulo do parmetro",
        "Descrio do parmetro", false, "{0}={1}", null, false, null, false);
    parameter.addElement(3.0);
    parameter.addElement(2.0);
    parameter.addElement(1.0);
    parameter.addElement(4.0);
    group.addParameter(parameter);
    String expressionText =
      MessageFormat.format("list_max({0}) == 4.0", parameterName);
    String errorMessage = "Mensagem de erro";
    ValidationExpression expression =
      new ValidationExpression(configurator, expressionText, errorMessage);
    assertTrue(expression.evaluate());
  }

  public void testEvaluate10() throws FormulaEvaluationException,
    FormulaCreationException, ParseException {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup("Nome do grupo");
    configurator.addGroup(group);
    String parameterName = "parametro";
    DoubleListParameter parameter =
      new DoubleListParameter(parameterName, "Rtulo do parmetro",
        "Descrio do parmetro", false, "{0}={1}", null, false, null, false);
    parameter.addElement(3.0);
    parameter.addElement(2.0);
    parameter.addElement(1.0);
    parameter.addElement(4.0);
    group.addParameter(parameter);
    String expressionText =
      MessageFormat.format("list_min({0}) == 1.0", parameterName);
    String errorMessage = "Mensagem de erro";
    ValidationExpression expression =
      new ValidationExpression(configurator, expressionText, errorMessage);
    assertTrue(expression.evaluate());
  }

  public void testEvaluate11() throws FormulaEvaluationException,
    FormulaCreationException, ParseException {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup("Nome do grupo");
    configurator.addGroup(group);
    String parameterName = "Nome_do_parametro";
    String columnName = "Nome_da_coluna";
    Column<?> doubleColumn =
      new DoubleColumn(columnName, columnName, null, false, true, null, false,
        null, false);
    TableParameter parameter =
      new TableParameter(parameterName, "Rtulo do parmetro",
        "Descrio do parmetro", false, "{0}={1}",
        Arrays.asList(doubleColumn), false, false, null, null);
    parameter.createRow();
    parameter.createRow();
    parameter.createRow();
    parameter.createRow();
    parameter.setItemValue(2.0, 0, 0);
    parameter.setItemValue(3.0, 1, 0);
    parameter.setItemValue(4.0, 2, 0);
    parameter.setItemValue(1.0, 3, 0);
    group.addParameter(parameter);
    String expressionText =
      MessageFormat.format("table_max({0}, 0) == 4.0", parameterName);
    String errorMessage = "Mensagem de erro";
    ValidationExpression expression =
      new ValidationExpression(configurator, expressionText, errorMessage);
    assertTrue(expression.evaluate());
  }

  public void testEvaluate12() throws FormulaEvaluationException,
    FormulaCreationException, ParseException {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup("Nome do grupo");
    configurator.addGroup(group);
    String parameterName = "Nome_do_parametro";
    String columnName = "Nome_da_coluna";
    Column<?> doubleColumn =
      new DoubleColumn(columnName, columnName, null, false, true, null, false,
        null, false);
    TableParameter parameter =
      new TableParameter(parameterName, "Rtulo do parmetro",
        "Descrio do parmetro", false, "{0}={1}",
        Arrays.asList(doubleColumn), false, false, null, null);
    parameter.createRow();
    parameter.createRow();
    parameter.createRow();
    parameter.createRow();
    parameter.setItemValue(2.0, 0, 0);
    parameter.setItemValue(3.0, 1, 0);
    parameter.setItemValue(4.0, 2, 0);
    parameter.setItemValue(1.0, 3, 0);
    group.addParameter(parameter);
    String expressionText =
      MessageFormat.format("table_min({0}, 0) == 1.0", parameterName);
    String errorMessage = "Mensagem de erro";
    ValidationExpression expression =
      new ValidationExpression(configurator, expressionText, errorMessage);
    assertTrue(expression.evaluate());
  }

  public void testSerialize() throws FormulaCreationException, ParseException,
    IOException, ClassNotFoundException {
    SimpleAlgorithmConfigurator configurator = createAlgorithmConfigurator();
    String expressionText = "parametro == 0";
    String errorMessage = "Mensagem de erro";
    ValidationExpression expectedExpression =
      new ValidationExpression(configurator, expressionText, errorMessage);
    ByteArrayOutputStream outputSource = new ByteArrayOutputStream();
    ObjectOutputStream objectOutput = new ObjectOutputStream(outputSource);
    objectOutput.writeObject(expectedExpression);
    ByteArrayInputStream inputSource =
      new ByteArrayInputStream(outputSource.toByteArray());
    ObjectInputStream objectInput = new ObjectInputStream(inputSource);
    ValidationExpression loadedExpression =
      (ValidationExpression) objectInput.readObject();
    assertEquals(expectedExpression.getErrorMessage(), loadedExpression
      .getErrorMessage());
    assertEquals(expectedExpression.toString(), loadedExpression.toString());
    assertEquals(expectedExpression.getConfigurator(), loadedExpression
      .getConfigurator());
  }

  private SimpleAlgorithmConfigurator createAlgorithmConfigurator() {
    String algorithmName = "Algoritmo";
    int major = 1;
    int minor = 2;
    int patch = 3;
    return createAlgorithmConfigurator(algorithmName, major, minor, patch);
  }

  private SimpleAlgorithmConfigurator createAlgorithmConfigurator(
    String algorithmName, int major, int minor, int patch) {
    AlgorithmInfo algorithm =
      new AlgorithmInfo("ID " + algorithmName, algorithmName, "DIR "
        + algorithmName, new Hashtable<String, String>());
    AlgorithmVersionId algorithmVersionId =
      new AlgorithmVersionId(major, minor, patch);
    AlgorithmVersionInfo algorithmVersion =
      new AlgorithmVersionInfo(algorithm, algorithmVersionId,
        new Hashtable<String, List<FileInfo>>(),
        new Hashtable<String, String>());
    return new SimpleAlgorithmConfigurator(algorithmVersion,
      "Descrio da verso", ExecutionType.SIMPLE, null, false, "comando",
      null, null, true, CMD_PATTERN);
  }
}
