/*
 * $Author:$
 * $Date:$
 * $Release:$
 */
package csbase.logic.algorithms.parameters.conditions;

import java.util.Arrays;

import junit.framework.TestCase;

public final class CompositeOperatorConditionTest extends TestCase {
  public void testBinaryOperatorCondition() {
    Condition firstCondition = new MockCondition(true);
    Condition secondCondition = new MockCondition(true);
    CompositeOperatorCondition condition = new MockCompositeOperatorCondition(
      firstCondition, secondCondition);
    assertEquals(Arrays.asList(firstCondition, secondCondition), condition
      .getConditions());
  }

  public void testEqualsObject() {
    Condition firstCondition1 = new MockCondition(true);
    Condition secondCondition1 = new MockCondition(true);
    CompositeOperatorCondition condition1 =
      new MockCompositeOperatorCondition(firstCondition1, secondCondition1);
    CompositeOperatorCondition condition2 =
      new MockCompositeOperatorCondition(firstCondition1, secondCondition1);
    assertEquals(condition1, condition2);
  }

  public void testEqualsObjectWithNull() {
    Condition firstCondition1 = new MockCondition(true);
    Condition secondCondition1 = new MockCondition(true);
    CompositeOperatorCondition condition1 =
      new MockCompositeOperatorCondition(firstCondition1, secondCondition1);
    assertFalse(condition1.equals(null));
  }

  public void testEqualsObjectWithAnotherClass() {
    Condition firstCondition1 = new MockCondition(true);
    Condition secondCondition1 = new MockCondition(true);
    CompositeOperatorCondition condition1 =
      new MockCompositeOperatorCondition(firstCondition1, secondCondition1);
    assertFalse(condition1.equals("Outra classe"));
  }

  public void testEqualsObjectWithDifferentFirstCondition() {
    Condition firstCondition1 = new MockCondition(true);
    Condition secondCondition1 = new MockCondition(true);
    CompositeOperatorCondition condition1 =
      new MockCompositeOperatorCondition(firstCondition1, secondCondition1);
    Condition firstCondition2 = new MockCondition(true);
    CompositeOperatorCondition condition2 =
      new MockCompositeOperatorCondition(firstCondition2, secondCondition1);
    assertFalse(condition1.equals(condition2));
  }

  public void testEqualsObjectWithDifferentSecondCondition() {
    Condition firstCondition1 = new MockCondition(true);
    Condition secondCondition1 = new MockCondition(true);
    CompositeOperatorCondition condition1 =
      new MockCompositeOperatorCondition(firstCondition1, secondCondition1);
    Condition secondCondition2 = new MockCondition(true);
    CompositeOperatorCondition condition2 =
      new MockCompositeOperatorCondition(firstCondition1, secondCondition2);
    assertFalse(condition1.equals(condition2));
  }

  public void testEqualsObjectWithDifferentConditions() {
    Condition firstCondition1 = new MockCondition(true);
    Condition secondCondition1 = new MockCondition(true);
    CompositeOperatorCondition condition1 =
      new MockCompositeOperatorCondition(firstCondition1, secondCondition1);
    Condition firstCondition2 = new MockCondition(true);
    Condition secondCondition2 = new MockCondition(true);
    CompositeOperatorCondition condition2 =
      new MockCompositeOperatorCondition(firstCondition2, secondCondition2);
    assertFalse(condition1.equals(condition2));
  }

  public void testHashCode() {
    Condition firstCondition1 = new MockCondition(true);
    Condition secondCondition1 = new MockCondition(true);
    CompositeOperatorCondition condition1 =
      new MockCompositeOperatorCondition(firstCondition1, secondCondition1);
    CompositeOperatorCondition condition2 =
      new MockCompositeOperatorCondition(firstCondition1, secondCondition1);
    assertEquals(condition1.hashCode(), condition2.hashCode());
  }
}
