package csbase.logic.algorithms.parameters.triggers;

import java.util.Hashtable;
import java.util.List;

import junit.framework.TestCase;
import csbase.logic.FileInfo;
import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.AlgorithmVersionId;
import csbase.logic.algorithms.AlgorithmVersionInfo;
import csbase.logic.algorithms.ExecutionType;
import csbase.logic.algorithms.parameters.DoubleListParameter;
import csbase.logic.algorithms.parameters.DoubleParameter;
import csbase.logic.algorithms.parameters.ParameterGroup;
import csbase.logic.algorithms.parameters.SimpleAlgorithmConfigurator;
import csbase.logic.algorithms.parameters.conditions.Condition;
import csbase.logic.algorithms.parameters.conditions.SimpleCondition;

/**
 * Testes unitrios para a classe {@link ChangeMaximumForDoublesTrigger}.
 * 
 * @author lmoreira
 */
public final class ChangeMaximumForDoublesTriggerTest extends TestCase {

  private static final String CMD_PATTERN = "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO";
  
  /**
   * Testando criar um objeto da classe {@link ChangeMaximumForDoublesTrigger} utilizando
   * um parmetro da classe {@link DoubleParameter} com mximo e incluir mximo
   * verdadeiro.
   */
  public void testChangeMaximumTrigger1() {
    DoubleParameter parameter = createMockDoubleParameter();
    Condition condition = createMockCondition();
    Double maximum = 10.5;
    boolean includeMaximum = true;
    ChangeMaximumForDoublesTrigger trigger = new ChangeMaximumForDoublesTrigger(parameter,
      condition, maximum, includeMaximum);
    assertEquals(parameter, trigger.getParameter());
    assertEquals(condition, trigger.getCondition());
    assertEquals(maximum, trigger.getMaximum());
    assertEquals(includeMaximum, trigger.isMaximumIncluded());
  }

  /**
   * Testando criar um objeto da classe {@link ChangeMaximumForDoublesTrigger} utilizando
   * um parmetro da classe {@link DoubleParameter} com mximo e incluir mximo
   * falso.
   */
  public void testChangeMaximumTrigger2() {
    DoubleParameter parameter = createMockDoubleParameter();
    Condition condition = createMockCondition();
    Double maximum = 10.5;
    boolean includeMaximum = false;
    ChangeMaximumForDoublesTrigger trigger = new ChangeMaximumForDoublesTrigger(parameter,
      condition, maximum, includeMaximum);
    assertEquals(parameter, trigger.getParameter());
    assertEquals(condition, trigger.getCondition());
    assertEquals(maximum, trigger.getMaximum());
    assertEquals(includeMaximum, trigger.isMaximumIncluded());
  }

  /**
   * Testando criar um objeto da classe {@link ChangeMaximumForDoublesTrigger} utilizando
   * um parmetro da classe {@link DoubleParameter} sem mximo.
   */
  public void testChangeMaximumTrigger3() {
    DoubleParameter parameter = createMockDoubleParameter();
    Condition condition = createMockCondition();
    Double maximum = null;
    boolean includeMaximum = false;
    ChangeMaximumForDoublesTrigger trigger = new ChangeMaximumForDoublesTrigger(parameter,
      condition, maximum, includeMaximum);
    assertEquals(parameter, trigger.getParameter());
    assertEquals(condition, trigger.getCondition());
    assertEquals(maximum, trigger.getMaximum());
    assertEquals(includeMaximum, trigger.isMaximumIncluded());
  }

  /**
   * Testando criar um objeto da classe {@link ChangeMaximumForDoublesTrigger} utilizando
   * um parmetro da classe {@link DoubleListParameter} com mximo e incluir
   * mximo verdadeiro.
   */
  public void testChangeMaximumTrigger4() {
    DoubleListParameter parameter = createMockDoubleListParameter();
    Condition condition = createMockCondition();
    Double maximum = 10.5;
    boolean includeMaximum = true;
    ChangeMaximumForDoublesTrigger trigger = new ChangeMaximumForDoublesTrigger(parameter,
      condition, maximum, includeMaximum);
    assertEquals(parameter, trigger.getParameter());
    assertEquals(condition, trigger.getCondition());
    assertEquals(maximum, trigger.getMaximum());
    assertEquals(includeMaximum, trigger.isMaximumIncluded());
  }

  /**
   * Testando criar um objeto da classe {@link ChangeMaximumForDoublesTrigger} utilizando
   * um parmetro da classe {@link DoubleListParameter} com mximo e incluir
   * mximo falso.
   */
  public void testChangeMaximumTrigger5() {
    DoubleListParameter parameter = createMockDoubleListParameter();
    Condition condition = createMockCondition();
    Double maximum = 10.5;
    boolean includeMaximum = false;
    ChangeMaximumForDoublesTrigger trigger = new ChangeMaximumForDoublesTrigger(parameter,
      condition, maximum, includeMaximum);
    assertEquals(parameter, trigger.getParameter());
    assertEquals(condition, trigger.getCondition());
    assertEquals(maximum, trigger.getMaximum());
    assertEquals(includeMaximum, trigger.isMaximumIncluded());
  }

  /**
   * Testando criar um objeto da classe {@link ChangeMaximumForDoublesTrigger} utilizando
   * um parmetro da classe {@link DoubleListParameter} sem mximo.
   */
  public void testChangeMaximumTrigger6() {
    DoubleListParameter parameter = createMockDoubleListParameter();
    Condition condition = createMockCondition();
    Double maximum = null;
    boolean includeMaximum = false;
    ChangeMaximumForDoublesTrigger trigger = new ChangeMaximumForDoublesTrigger(parameter,
      condition, maximum, includeMaximum);
    assertEquals(parameter, trigger.getParameter());
    assertEquals(condition, trigger.getCondition());
    assertEquals(maximum, trigger.getMaximum());
    assertEquals(includeMaximum, trigger.isMaximumIncluded());
  }

  /**
   * Testando criar um objeto da classe {@link ChangeMaximumForDoublesTrigger} com
   * parmetro {@link DoubleParameter} {@code null}.
   */
  public void testChangeMaximumTrigger7() {
    DoubleParameter parameter = null;
    Condition condition = createMockCondition();
    Double maximum = 10.5;
    boolean includeMaximum = true;
    try {
      new ChangeMaximumForDoublesTrigger(parameter, condition, maximum, includeMaximum);
      fail();
    } catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando criar um objeto da classe {@link ChangeMaximumForDoublesTrigger} com
   * parmetro {@link DoubleListParameter} {@code null}.
   */
  public void testChangeMaximumTrigger8() {
    DoubleListParameter parameter = null;
    Condition condition = createMockCondition();
    Double maximum = 10.5;
    boolean includeMaximum = true;
    try {
      new ChangeMaximumForDoublesTrigger(parameter, condition, maximum, includeMaximum);
      fail();
    } catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando criar um objeto da classe {@link ChangeMaximumForDoublesTrigger} com
   * parmetro {@link DoubleParameter} e condio {@code null}.
   */
  public void testChangeMaximumTrigger9() {
    DoubleParameter parameter = createMockDoubleParameter();
    Condition condition = null;
    Double maximum = 10.5;
    boolean includeMaximum = true;
    try {
      new ChangeMaximumForDoublesTrigger(parameter, condition, maximum, includeMaximum);
      fail();
    } catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando criar um objeto da classe {@link ChangeMaximumForDoublesTrigger} com
   * parmetro {@link DoubleListParameter} e condio {@code null}.
   */
  public void testChangeMaximumTrigger10() {
    DoubleListParameter parameter = createMockDoubleListParameter();
    Condition condition = null;
    Double maximum = 10.5;
    boolean includeMaximum = true;
    try {
      new ChangeMaximumForDoublesTrigger(parameter, condition, maximum, includeMaximum);
      fail();
    } catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando o mtodo {@link ChangeMaximumForDoublesTrigger#doAction()} com parmetro do
   * tipo {@link DoubleParameter} com condio verdadeira.
   */
  public void testDoAction1() {
    String parameterName = "Nome do parmetro";
    Double defaultValue = 10.5;
    Double maximum = 21.0;
    boolean isMaximumIncluded = true;
    DoubleParameter parameter = createMockDoubleParameter(parameterName,
      defaultValue, maximum, isMaximumIncluded);
    assertEquals(maximum, parameter.getMaximum());
    assertEquals(isMaximumIncluded, parameter.isMaximumIncluded());
    Condition condition = new SimpleCondition(parameterName, defaultValue);
    SimpleAlgorithmConfigurator configurator = createMockAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup("Grupo de parmetros");
    group.addParameter(parameter);
    configurator.addGroup(group);
    assertTrue(condition.evaluate(configurator));
    Double newMaximum = 31.5;
    boolean newIsMaximumIncluded = !isMaximumIncluded;
    ChangeMaximumForDoublesTrigger trigger = new ChangeMaximumForDoublesTrigger(parameter,
      condition, newMaximum, newIsMaximumIncluded);
    assertTrue(trigger.update(configurator));
    assertEquals(newMaximum, parameter.getMaximum());
    assertEquals(newIsMaximumIncluded, parameter.isMaximumIncluded());
  }

  /**
   * Testando o mtodo {@link ChangeMaximumForDoublesTrigger#doAction()} com parmetro do
   * tipo {@link DoubleListParameter} com condio verdadeira.
   */
  public void testDoAction2() {
    String parameterName = "Nome do parmetro";
    Double maximum = 21.0;
    boolean isMaximumIncluded = true;
    DoubleListParameter parameter = createMockDoubleListParameter(
      parameterName, maximum, isMaximumIncluded);
    assertEquals(maximum, parameter.getMaximum());
    assertEquals(isMaximumIncluded, parameter.isMaximumIncluded());
    Condition condition = new SimpleCondition(parameterName, null);
    SimpleAlgorithmConfigurator configurator = createMockAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup("Grupo de parmetros");
    group.addParameter(parameter);
    configurator.addGroup(group);
    assertTrue(condition.evaluate(configurator));
    Double newMaximum = 31.5;
    boolean newIsMaximumIncluded = !isMaximumIncluded;
    ChangeMaximumForDoublesTrigger trigger = new ChangeMaximumForDoublesTrigger(parameter,
      condition, newMaximum, newIsMaximumIncluded);
    assertTrue(trigger.update(configurator));
    assertEquals(newMaximum, parameter.getMaximum());
    assertEquals(newIsMaximumIncluded, parameter.isMaximumIncluded());
  }


  /**
   * Testando o mtodo {@link ChangeMaximumForDoublesTrigger#doAction()} com parmetro do
   * tipo {@link DoubleParameter} com condio falsa.
   */
  public void testDoAction3() {
    String parameterName = "Nome do parmetro";
    Double defaultValue = 10.5;
    Double maximum = 21.0;
    boolean isMaximumIncluded = true;
    DoubleParameter parameter = createMockDoubleParameter(parameterName,
      defaultValue, maximum, isMaximumIncluded);
    assertEquals(maximum, parameter.getMaximum());
    assertEquals(isMaximumIncluded, parameter.isMaximumIncluded());
    Condition condition = new SimpleCondition(parameterName, null);
    SimpleAlgorithmConfigurator configurator = createMockAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup("Grupo de parmetros");
    group.addParameter(parameter);
    configurator.addGroup(group);
    assertFalse(condition.evaluate(configurator));
    Double newMaximum = 31.5;
    boolean newIsMaximumIncluded = !isMaximumIncluded;
    ChangeMaximumForDoublesTrigger trigger = new ChangeMaximumForDoublesTrigger(parameter,
      condition, newMaximum, newIsMaximumIncluded);
    assertFalse(trigger.update(configurator));
    assertEquals(maximum, parameter.getMaximum());
    assertEquals(isMaximumIncluded, parameter.isMaximumIncluded());
  }

  /**
   * Testando o mtodo {@link ChangeMaximumForDoublesTrigger#doAction()} com parmetro do
   * tipo {@link DoubleListParameter} com condio falsa.
   */
  public void testDoAction4() {
    String parameterName = "Nome do parmetro";
    Double maximum = 21.0;
    boolean isMaximumIncluded = true;
    DoubleListParameter parameter = createMockDoubleListParameter(
      parameterName, maximum, isMaximumIncluded);
    assertEquals(maximum, parameter.getMaximum());
    assertEquals(isMaximumIncluded, parameter.isMaximumIncluded());
    Double value = 34.5;
    Condition condition = new SimpleCondition(parameterName, value);
    SimpleAlgorithmConfigurator configurator = createMockAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup("Grupo de parmetros");
    group.addParameter(parameter);
    configurator.addGroup(group);
    assertFalse(condition.evaluate(configurator));
    Double newMaximum = 31.5;
    boolean newIsMaximumIncluded = !isMaximumIncluded;
    ChangeMaximumForDoublesTrigger trigger = new ChangeMaximumForDoublesTrigger(parameter,
      condition, newMaximum, newIsMaximumIncluded);
    assertFalse(trigger.update(configurator));
    assertEquals(maximum, parameter.getMaximum());
    assertEquals(isMaximumIncluded, parameter.isMaximumIncluded());
  }
  
  /**
   * Testando o mtodo {@link ChangeMaximumForDoublesTrigger#equals(Object)} com parmetro
   * do tipo {@link DoubleParameter} e com gatilhos iguais.
   */
  public void testEquals1() {
    String parameterName = "Nome do parmetro";
    DoubleParameter parameter = createMockDoubleParameter(parameterName);
    Double maximum = 10.5;
    boolean isMaximumIncluded = true;
    Condition condition1 = new SimpleCondition("Nome do parmetro 1", "Valor 1");
    Condition condition2 = new SimpleCondition("Nome do parmetro 2", "Valor 2");
    ChangeMaximumForDoublesTrigger trigger1 = new ChangeMaximumForDoublesTrigger(parameter,
      condition1, maximum, isMaximumIncluded);
    ChangeMaximumForDoublesTrigger trigger2 = new ChangeMaximumForDoublesTrigger(parameter,
      condition2, maximum, isMaximumIncluded);
    assertTrue(trigger1.equals(trigger2));
  }

  /**
   * Testando o mtodo {@link ChangeMaximumForDoublesTrigger#equals(Object)} com parmetro
   * do tipo {@link DoubleListParameter} e com gatilhos iguais.
   */
  public void testEquals2() {
    String parameterName = "Nome do parmetro";
    DoubleListParameter parameter = createMockDoubleListParameter(parameterName);
    Double maximum = 10.5;
    boolean isMaximumIncluded = true;
    Condition condition1 = new SimpleCondition("Nome do parmetro 1", "Valor 1");
    Condition condition2 = new SimpleCondition("Nome do parmetro 2", "Valor 2");
    ChangeMaximumForDoublesTrigger trigger1 = new ChangeMaximumForDoublesTrigger(parameter,
      condition1, maximum, isMaximumIncluded);
    ChangeMaximumForDoublesTrigger trigger2 = new ChangeMaximumForDoublesTrigger(parameter,
      condition2, maximum, isMaximumIncluded);
    assertTrue(trigger1.equals(trigger2));
  }

  /**
   * Testando o mtodo {@link ChangeMaximumForDoublesTrigger#equals(Object)} com parmetro
   * do tipo {@link DoubleParameter} e com nomes de parmetro diferentes.
   */
  public void testEquals3() {
    String parameterName1 = "Nome do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    DoubleParameter parameter1 = createMockDoubleParameter(parameterName1);
    DoubleParameter parameter2 = createMockDoubleParameter(parameterName2);
    Double maximum = 10.5;
    boolean isMaximumIncluded = true;
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    ChangeMaximumForDoublesTrigger trigger1 = new ChangeMaximumForDoublesTrigger(parameter1,
      condition, maximum, isMaximumIncluded);
    ChangeMaximumForDoublesTrigger trigger2 = new ChangeMaximumForDoublesTrigger(parameter2,
      condition, maximum, isMaximumIncluded);
    assertFalse(trigger1.equals(trigger2));
  }

  /**
   * Testando o mtodo {@link ChangeMaximumForDoublesTrigger#equals(Object)} com parmetro
   * do tipo {@link DoubleListParameter} e com nomes de parmetro diferentes.
   */
  public void testEquals4() {
    String parameterName1 = "Nome do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    DoubleListParameter parameter1 = createMockDoubleListParameter(parameterName1);
    DoubleListParameter parameter2 = createMockDoubleListParameter(parameterName2);
    Double maximum = 10.5;
    boolean isMaximumIncluded = true;
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    ChangeMaximumForDoublesTrigger trigger1 = new ChangeMaximumForDoublesTrigger(parameter1,
      condition, maximum, isMaximumIncluded);
    ChangeMaximumForDoublesTrigger trigger2 = new ChangeMaximumForDoublesTrigger(parameter2,
      condition, maximum, isMaximumIncluded);
    assertFalse(trigger1.equals(trigger2));
  }

  /**
   * Testando o mtodo {@link ChangeMaximumForDoublesTrigger#equals(Object)} com parmetro
   * do tipo {@link DoubleParameter} e com mximos diferentes.
   */
  public void testEquals5() {
    String parameterName = "Nome do parmetro";
    DoubleParameter parameter = createMockDoubleParameter(parameterName);
    Double maximum1 = 11.1;
    Double maximum2 = 22.2;
    boolean isMaximumIncluded = true;
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    ChangeMaximumForDoublesTrigger trigger1 = new ChangeMaximumForDoublesTrigger(parameter,
      condition, maximum1, isMaximumIncluded);
    ChangeMaximumForDoublesTrigger trigger2 = new ChangeMaximumForDoublesTrigger(parameter,
      condition, maximum2, isMaximumIncluded);
    assertFalse(trigger1.equals(trigger2));
  }

  /**
   * Testando o mtodo {@link ChangeMaximumForDoublesTrigger#equals(Object)} com parmetro
   * do tipo {@link DoubleListParameter} e com mximos diferentes.
   */
  public void testEquals6() {
    String parameterName = "Nome do parmetro";
    DoubleListParameter parameter = createMockDoubleListParameter(parameterName);
    Double maximum1 = 11.1;
    Double maximum2 = 22.2;
    boolean isMaximumIncluded = true;
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    ChangeMaximumForDoublesTrigger trigger1 = new ChangeMaximumForDoublesTrigger(parameter,
      condition, maximum1, isMaximumIncluded);
    ChangeMaximumForDoublesTrigger trigger2 = new ChangeMaximumForDoublesTrigger(parameter,
      condition, maximum2, isMaximumIncluded);
    assertFalse(trigger1.equals(trigger2));
  }

  /**
   * Testando o mtodo {@link ChangeMaximumForDoublesTrigger#equals(Object)} com parmetro
   * do tipo {@link DoubleParameter} e com incluir mximo diferentes.
   */
  public void testEquals7() {
    String parameterName = "Nome do parmetro";
    DoubleParameter parameter = createMockDoubleParameter(parameterName);
    Double maximum = 10.5;
    boolean isMaximumIncluded1 = true;
    boolean isMaximumIncluded2 = false;
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    ChangeMaximumForDoublesTrigger trigger1 = new ChangeMaximumForDoublesTrigger(parameter,
      condition, maximum, isMaximumIncluded1);
    ChangeMaximumForDoublesTrigger trigger2 = new ChangeMaximumForDoublesTrigger(parameter,
      condition, maximum, isMaximumIncluded2);
    assertFalse(trigger1.equals(trigger2));
  }

  /**
   * Testando o mtodo {@link ChangeMaximumForDoublesTrigger#equals(Object)} com parmetro
   * do tipo {@link DoubleListParameter} e com incluir mximo diferentes.
   */
  public void testEquals8() {
    String parameterName = "Nome do parmetro";
    DoubleListParameter parameter = createMockDoubleListParameter(parameterName);
    Double maximum = 10.5;
    boolean isMaximumIncluded1 = true;
    boolean isMaximumIncluded2 = false;
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    ChangeMaximumForDoublesTrigger trigger1 = new ChangeMaximumForDoublesTrigger(parameter,
      condition, maximum, isMaximumIncluded1);
    ChangeMaximumForDoublesTrigger trigger2 = new ChangeMaximumForDoublesTrigger(parameter,
      condition, maximum, isMaximumIncluded2);
    assertFalse(trigger1.equals(trigger2));
  }

  /**
   * Testando o mtodo {@link ChangeMaximumForDoublesTrigger#equals(Object)} com parmetro
   * com parmetro de tipos diferentes.
   */
  public void testEquals9() {
    String parameterName = "Nome do parmetro";
    DoubleListParameter parameter1 = createMockDoubleListParameter(parameterName);
    DoubleParameter parameter2 = createMockDoubleParameter(parameterName);
    Double maximum = 10.5;
    boolean isMaximumIncluded = true;
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    ChangeMaximumForDoublesTrigger trigger1 = new ChangeMaximumForDoublesTrigger(parameter1,
      condition, maximum, isMaximumIncluded);
    ChangeMaximumForDoublesTrigger trigger2 = new ChangeMaximumForDoublesTrigger(parameter2,
      condition, maximum, isMaximumIncluded);
    assertFalse(trigger1.equals(trigger2));
  }

  /**
   * Testando o mtodo {@link ChangeMaximumForDoublesTrigger#equals(Object)} com parmetro
   * do tipo {@link DoubleParameter} e com argumento {@code null}.
   */
  public void testEquals10() {
    String parameterName = "Nome do parmetro";
    DoubleParameter parameter = createMockDoubleParameter(parameterName);
    Double maximum = 10.5;
    boolean isMaximumIncluded = true;
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    ChangeMaximumForDoublesTrigger trigger = new ChangeMaximumForDoublesTrigger(parameter,
      condition, maximum, isMaximumIncluded);
    assertFalse(trigger.equals(null));
  }

  /**
   * Testando o mtodo {@link ChangeMaximumForDoublesTrigger#equals(Object)} com parmetro
   * do tipo {@link DoubleParameter} e com argumento de outra classe.
   */
  public void testEquals11() {
    String parameterName = "Nome do parmetro";
    DoubleParameter parameter = createMockDoubleParameter(parameterName);
    Double maximum = 10.5;
    boolean isMaximumIncluded = true;
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    ChangeMaximumForDoublesTrigger trigger = new ChangeMaximumForDoublesTrigger(parameter,
      condition, maximum, isMaximumIncluded);
    assertFalse(trigger.equals("Outra classe"));
  }

  /**
   * Testando o mtodo {@link ChangeMaximumForDoublesTrigger#equals(Object)} com parmetro
   * do tipo {@link DoubleListParameter} e com argumento {@code null}.
   */
  public void testEquals12() {
    String parameterName = "Nome do parmetro";
    DoubleListParameter parameter = createMockDoubleListParameter(parameterName);
    Double maximum = 10.5;
    boolean isMaximumIncluded = true;
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    ChangeMaximumForDoublesTrigger trigger = new ChangeMaximumForDoublesTrigger(parameter,
      condition, maximum, isMaximumIncluded);
    assertFalse(trigger.equals(null));
  }

  /**
   * Testando o mtodo {@link ChangeMaximumForDoublesTrigger#equals(Object)} com parmetro
   * do tipo {@link DoubleListParameter} e com argumento de outra classe.
   */
  public void testEquals13() {
    String parameterName = "Nome do parmetro";
    DoubleListParameter parameter = createMockDoubleListParameter(parameterName);
    Double maximum = 10.5;
    boolean isMaximumIncluded = true;
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    ChangeMaximumForDoublesTrigger trigger = new ChangeMaximumForDoublesTrigger(parameter,
      condition, maximum, isMaximumIncluded);
    assertFalse(trigger.equals("Outra classe"));
  }

  /**
   * Cria um configurador de algorithms qualquer.
   * 
   * @return .
   */
  private SimpleAlgorithmConfigurator createMockAlgorithmConfigurator() {
    AlgorithmInfo algorithmInfo = new AlgorithmInfo("ID Algoritmo",
      "Algoritmo", "DIR Algoritmo", new Hashtable<String, String>());
    AlgorithmVersionId algorithmVersionId = new AlgorithmVersionId(1, 2, 3);
    AlgorithmVersionInfo algorithmVersionInfo = new AlgorithmVersionInfo(
      algorithmInfo, algorithmVersionId,
      new Hashtable<String, List<FileInfo>>(),
      new Hashtable<String, String>());
    return new SimpleAlgorithmConfigurator(algorithmVersionInfo, null,
      ExecutionType.SIMPLE, null, false, "Comando", null, null, true, CMD_PATTERN);
  }

  /**
   * Cria um parmetro qualquer do tipo {@link DoubleListParameter}.
   * 
   * @return .
   */
  private DoubleListParameter createMockDoubleListParameter() {
    String parameterName = "Nome do parmetro";
    return createMockDoubleListParameter(parameterName);
  }

  /**
   * Cria um parmetro qualquer do tipo {@link DoubleListParameter}.
   * 
   * @param parameterName O nome do parmetro (No aceita {@code null}).
   * 
   * @return .
   */
  private DoubleListParameter createMockDoubleListParameter(String parameterName) {
    Double maximum = null;
    boolean isMaximumIncluded = false;
    return createMockDoubleListParameter(parameterName, maximum,
      isMaximumIncluded);
  }

  /**
   * Cria um parmetro qualquer do tipo {@link DoubleListParameter}.
   * 
   * @param parameterName O nome do parmetro (No aceita {@code null}).
   * @param maximum O mximo (Aceita {@code null}).
   * @param isMaximumIncluded Indica se o mximo deve ser considerado um dos
   *        valores vlidos.
   * 
   * @return .
   */
  private DoubleListParameter createMockDoubleListParameter(
    String parameterName, Double maximum, boolean isMaximumIncluded) {
    return new DoubleListParameter(parameterName, "Rtulo do parmetro",
      "Descrio do parmetro", false, "{0}={1}", maximum, isMaximumIncluded,
      null, false);
  }

  /**
   * Cria um parmetro qualquer do tipo {@link DoubleParameter}.
   * 
   * @return .
   */
  private DoubleParameter createMockDoubleParameter() {
    String parameterName = "Nome do parmetro";
    return createMockDoubleParameter(parameterName);
  }

  /**
   * Cria um parmetro qualquer do tipo {@link DoubleParameter}.
   * 
   * @param parameterName O nome do parmetro (No aceita {@code null}).
   * 
   * @return .
   */
  private DoubleParameter createMockDoubleParameter(String parameterName) {
    Double defaultValue = null;
    Double maximum = null;
    boolean isMaximumIncluded = false;
    return createMockDoubleParameter(parameterName, defaultValue, maximum,
      isMaximumIncluded);
  }

  /**
   * Cria um parmetro qualquer do tipo {@link DoubleParameter}.
   * 
   * @param parameterName O nome do parmetro (No aceita {@code null}).
   * @param defaultValue O valor-padro (Aceita {@code null}).
   * @param maximum O mximo (Aceita {@code null}).
   * @param isMaximumIncluded Indica se o mximo deve ser considerado um dos
   *        valores vlidos.
   * 
   * @return .
   */
  private DoubleParameter createMockDoubleParameter(String parameterName,
    Double defaultValue, Double maximum, boolean isMaximumIncluded) {
    return new DoubleParameter(parameterName, "Rtulo do parmetro",
      "Descrio do parmetro", defaultValue, false, "{0}={1}", maximum,
      isMaximumIncluded, null, false);
  }

  /**
   * Cria uma condio qualquer.
   * 
   * @return .
   */
  private Condition createMockCondition() {
    return new SimpleCondition("Nome do parmetro", "Valor qualquer");
  }

}
