package csbase.logic.algorithms.parameters.triggers;

import java.util.Hashtable;
import java.util.List;

import junit.framework.TestCase;
import csbase.logic.FileInfo;
import csbase.logic.algorithms.AlgorithmInfo;
import csbase.logic.algorithms.AlgorithmVersionId;
import csbase.logic.algorithms.AlgorithmVersionInfo;
import csbase.logic.algorithms.ExecutionType;
import csbase.logic.algorithms.parameters.IntegerListParameter;
import csbase.logic.algorithms.parameters.IntegerParameter;
import csbase.logic.algorithms.parameters.ParameterGroup;
import csbase.logic.algorithms.parameters.SimpleAlgorithmConfigurator;
import csbase.logic.algorithms.parameters.conditions.Condition;
import csbase.logic.algorithms.parameters.conditions.SimpleCondition;

/**
 * Testes unitrios para a classe {@link ChangeMinimumForIntegersTrigger}.
 * 
 * @author lmoreira
 */
public final class ChangeMinimumForIntegersTriggerTest extends TestCase {

  /**
   * Pattern
   */
  private static final String CMD_PATTERN = "$NOME_DO_PARAMETRO=$VALOR_DO_PARAMETRO";
  
  /**
   * Testando criar um objeto da classe {@link ChangeMinimumForIntegersTrigger} 
   * utilizando um parmetro da classe {@link IntegerParameter} com mnimo.
   */
  public void testChangeMinimumTrigger1() {
    IntegerParameter parameter = createMockIntegerParameter();
    Condition condition = createMockCondition();
    Integer minimum = 10;
    ChangeMinimumForIntegersTrigger trigger = new ChangeMinimumForIntegersTrigger(parameter,
      condition, minimum);
    assertEquals(parameter, trigger.getParameter());
    assertEquals(condition, trigger.getCondition());
    assertEquals(minimum, trigger.getMinimum());
  }

  /**
   * Testando criar um objeto da classe {@link ChangeMinimumForIntegersTrigger} utilizando
   * um parmetro da classe {@link IntegerParameter} sem mnimo.
   */
  public void testChangeMinimumTrigger2() {
    IntegerParameter parameter = createMockIntegerParameter();
    Condition condition = createMockCondition();
    Integer minimum = null;
    ChangeMinimumForIntegersTrigger trigger = new ChangeMinimumForIntegersTrigger(parameter,
      condition, minimum);
    assertEquals(parameter, trigger.getParameter());
    assertEquals(condition, trigger.getCondition());
    assertEquals(minimum, trigger.getMinimum());
  }

  /**
   * Testando criar um objeto da classe {@link ChangeMinimumForIntegersTrigger} utilizando
   * um parmetro da classe {@link IntegerListParameter} com mnimo.
   */
  public void testChangeMinimumTrigger3() {
    IntegerListParameter parameter = createMockIntegerListParameter();
    Condition condition = createMockCondition();
    Integer minimum = 10;
    ChangeMinimumForIntegersTrigger trigger = new ChangeMinimumForIntegersTrigger(parameter,
      condition, minimum);
    assertEquals(parameter, trigger.getParameter());
    assertEquals(condition, trigger.getCondition());
    assertEquals(minimum, trigger.getMinimum());    
  }

  /**
   * Testando criar um objeto da classe {@link ChangeMinimumForIntegersTrigger} utilizando
   * um parmetro da classe {@link IntegerListParameter} sem mnimo.
   */
  public void testChangeMinimumTrigger4() {
    IntegerListParameter parameter = createMockIntegerListParameter();
    Condition condition = createMockCondition();
    Integer minimum = null;
    ChangeMinimumForIntegersTrigger trigger = new ChangeMinimumForIntegersTrigger(parameter,
      condition, minimum);
    assertEquals(parameter, trigger.getParameter());
    assertEquals(condition, trigger.getCondition());
    assertEquals(minimum, trigger.getMinimum());    
  }

  /**
   * Testando criar um objeto da classe {@link ChangeMinimumForIntegersTrigger} com
   * parmetro {@link IntegerParameter} {@code null}.
   */
  public void testChangeMinimumTrigger5() {
    IntegerParameter parameter = null;
    Condition condition = createMockCondition();
    Integer minimum = 10;
    try {
      new ChangeMinimumForIntegersTrigger(parameter, condition, minimum);
      fail();
    } catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando criar um objeto da classe {@link ChangeMinimumForIntegersTrigger} com
   * parmetro {@link IntegerListParameter} {@code null}.
   */
  public void testChangeMinimumTrigger6() {
    IntegerListParameter parameter = null;
    Condition condition = createMockCondition();
    Integer minimum = 10;
    try {
      new ChangeMinimumForIntegersTrigger(parameter, condition, minimum);
      fail();
    } catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando criar um objeto da classe {@link ChangeMinimumForIntegersTrigger} com
   * parmetro {@link IntegerParameter} e condio {@code null}.
   */
  public void testChangeMinimumTrigger7() {
    IntegerParameter parameter = createMockIntegerParameter();
    Condition condition = null;
    Integer minimum = 10;    
    try {
      new ChangeMinimumForIntegersTrigger(parameter, condition, minimum);
      fail();
    } catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando criar um objeto da classe {@link ChangeMinimumForIntegersTrigger} com
   * parmetro {@link IntegerListParameter} e condio {@code null}.
   */
  public void testChangeMinimumTrigger8() {
    IntegerListParameter parameter = createMockIntegerListParameter();
    Condition condition = null;
    Integer minimum = 10;    
    try {
      new ChangeMinimumForIntegersTrigger(parameter, condition, minimum);
      fail();
    } catch (IllegalArgumentException e) {
    }
  }

  /**
   * Testando o mtodo {@link ChangeMinimumForIntegersTrigger#doAction()} com parmetro do
   * tipo {@link IntegerParameter} com condio verdadeira.
   */
  public void testDoAction1() {
    String parameterName = "Nome do parmetro";
    Integer defaultValue = 10;
    Integer minimum = 21;
    IntegerParameter parameter = createMockIntegerParameter(parameterName,
      defaultValue, minimum);
    assertEquals(minimum, parameter.getMinimum());
    Condition condition = new SimpleCondition(parameterName, defaultValue);
    SimpleAlgorithmConfigurator configurator = createMockAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup("Grupo de parmetros");
    group.addParameter(parameter);
    configurator.addGroup(group);
    assertTrue(condition.evaluate(configurator));
    Integer newMinimum = 31;
    ChangeMinimumForIntegersTrigger trigger = new ChangeMinimumForIntegersTrigger(parameter,
      condition, newMinimum);
    assertTrue(trigger.update(configurator));
    assertEquals(newMinimum, parameter.getMinimum());
  }

  /**
   * Testando o mtodo {@link ChangeMinimumForIntegersTrigger#doAction()} com parmetro do
   * tipo {@link IntegerListParameter} com condio verdadeira.
   */
  public void testDoAction2() {
    String parameterName = "Nome do parmetro";
    Integer minimum = 21;
    IntegerListParameter parameter = createMockIntegerListParameter(
      parameterName, minimum);
    assertEquals(minimum, parameter.getMinimum());
    Condition condition = new SimpleCondition(parameterName, null);
    SimpleAlgorithmConfigurator configurator = createMockAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup("Grupo de parmetros");
    group.addParameter(parameter);
    configurator.addGroup(group);
    assertTrue(condition.evaluate(configurator));
    Integer newMinimum = 31;
    ChangeMinimumForIntegersTrigger trigger = new ChangeMinimumForIntegersTrigger(parameter,
      condition, newMinimum);
    assertTrue(trigger.update(configurator));
    assertEquals(newMinimum, parameter.getMinimum());
  }


  /**
   * Testando o mtodo {@link ChangeMinimumForIntegersTrigger#doAction()} com parmetro do
   * tipo {@link IntegerParameter} com condio falsa.
   */
  public void testDoAction3() {
    String parameterName = "Nome do parmetro";
    Integer defaultValue = 10;
    Integer minimum = 21;
    IntegerParameter parameter = createMockIntegerParameter(parameterName,
      defaultValue, minimum);
    assertEquals(minimum, parameter.getMinimum());
    Condition condition = new SimpleCondition(parameterName, null);
    SimpleAlgorithmConfigurator configurator = createMockAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup("Grupo de parmetros");
    group.addParameter(parameter);
    configurator.addGroup(group);
    assertFalse(condition.evaluate(configurator));
    Integer newMinimum = 31;
    ChangeMinimumForIntegersTrigger trigger = new ChangeMinimumForIntegersTrigger(parameter,
      condition, newMinimum);
    assertFalse(trigger.update(configurator));
    assertEquals(minimum, parameter.getMinimum());
    
  }

  /**
   * Testando o mtodo {@link ChangeMinimumForIntegersTrigger#doAction()} com parmetro do
   * tipo {@link IntegerListParameter} com condio falsa.
   */
  public void testDoAction4() {
    String parameterName = "Nome do parmetro";
    Integer minimum = 21;
    IntegerListParameter parameter = createMockIntegerListParameter(
      parameterName, minimum);
    assertEquals(minimum, parameter.getMinimum());
    Integer value = 34;
    Condition condition = new SimpleCondition(parameterName, value);
    SimpleAlgorithmConfigurator configurator = createMockAlgorithmConfigurator();
    ParameterGroup group = new ParameterGroup("Grupo de parmetros");
    group.addParameter(parameter);
    configurator.addGroup(group);
    assertFalse(condition.evaluate(configurator));
    Integer newMinimum = 31;
    ChangeMinimumForIntegersTrigger trigger = new ChangeMinimumForIntegersTrigger(parameter,
      condition, newMinimum);
    assertFalse(trigger.update(configurator));
    assertEquals(minimum, parameter.getMinimum());
    
  }
  
  /**
   * Testando o mtodo {@link ChangeMinimumForIntegersTrigger#equals(Object)} com parmetro
   * do tipo {@link IntegerParameter} e com gatilhos iguais.
   */
  public void testEquals1() {
    String parameterName = "Nome do parmetro";
    IntegerParameter parameter = createMockIntegerParameter(parameterName);
    Integer minimum = 10;
    Condition condition1 = new SimpleCondition("Nome do parmetro 1", "Valor 1");
    Condition condition2 = new SimpleCondition("Nome do parmetro 2", "Valor 2");
    ChangeMinimumForIntegersTrigger trigger1 = new ChangeMinimumForIntegersTrigger(parameter,
      condition1, minimum);
    ChangeMinimumForIntegersTrigger trigger2 = new ChangeMinimumForIntegersTrigger(parameter,
      condition2, minimum);
    assertTrue(trigger1.equals(trigger2));
  }

  /**
   * Testando o mtodo {@link ChangeMinimumForIntegersTrigger#equals(Object)} com parmetro
   * do tipo {@link IntegerListParameter} e com gatilhos iguais.
   */
  public void testEquals2() {
    String parameterName = "Nome do parmetro";
    IntegerListParameter parameter = createMockIntegerListParameter(parameterName);
    Integer minimum = 10;
    Condition condition1 = new SimpleCondition("Nome do parmetro 1", "Valor 1");
    Condition condition2 = new SimpleCondition("Nome do parmetro 2", "Valor 2");
    ChangeMinimumForIntegersTrigger trigger1 = new ChangeMinimumForIntegersTrigger(parameter,
      condition1, minimum);
    ChangeMinimumForIntegersTrigger trigger2 = new ChangeMinimumForIntegersTrigger(parameter,
      condition2, minimum);
    assertTrue(trigger1.equals(trigger2));
  }

  /**
   * Testando o mtodo {@link ChangeMinimumForIntegersTrigger#equals(Object)} com parmetro
   * do tipo {@link IntegerParameter} e com nomes de parmetro diferentes.
   */
  public void testEquals3() {
    String parameterName1 = "Nome do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    IntegerParameter parameter1 = createMockIntegerParameter(parameterName1);
    IntegerParameter parameter2 = createMockIntegerParameter(parameterName2);
    Integer minimum = 10;
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    ChangeMinimumForIntegersTrigger trigger1 = new ChangeMinimumForIntegersTrigger(parameter1,
      condition, minimum);
    ChangeMinimumForIntegersTrigger trigger2 = new ChangeMinimumForIntegersTrigger(parameter2,
      condition, minimum);
    assertFalse(trigger1.equals(trigger2));
  }

  /**
   * Testando o mtodo {@link ChangeMinimumForIntegersTrigger#equals(Object)} com parmetro
   * do tipo {@link IntegerListParameter} e com nomes de parmetro diferentes.
   */
  public void testEquals4() {
    String parameterName1 = "Nome do parmetro 1";
    String parameterName2 = "Nome do parmetro 2";
    IntegerListParameter parameter1 = createMockIntegerListParameter(parameterName1);
    IntegerListParameter parameter2 = createMockIntegerListParameter(parameterName2);
    Integer minimum = 10;
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    ChangeMinimumForIntegersTrigger trigger1 = new ChangeMinimumForIntegersTrigger(parameter1,
      condition, minimum);
    ChangeMinimumForIntegersTrigger trigger2 = new ChangeMinimumForIntegersTrigger(parameter2,
      condition, minimum);
    assertFalse(trigger1.equals(trigger2));
  }

  /**
   * Testando o mtodo {@link ChangeMinimumForIntegersTrigger#equals(Object)} com parmetro
   * do tipo {@link IntegerParameter} e com mnimos diferentes.
   */
  public void testEquals5() {
    String parameterName = "Nome do parmetro";
    IntegerParameter parameter = createMockIntegerParameter(parameterName);
    Integer minimum1 = 11;
    Integer minimum2 = 22;
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    ChangeMinimumForIntegersTrigger trigger1 = new ChangeMinimumForIntegersTrigger(parameter,
      condition, minimum1);
    ChangeMinimumForIntegersTrigger trigger2 = new ChangeMinimumForIntegersTrigger(parameter,
      condition, minimum2);
    assertFalse(trigger1.equals(trigger2));
  }

  /**
   * Testando o mtodo {@link ChangeMinimumForIntegersTrigger#equals(Object)} com parmetro
   * do tipo {@link IntegerListParameter} e com mnimos diferentes.
   */
  public void testEquals6() {
    String parameterName = "Nome do parmetro";
    IntegerListParameter parameter = createMockIntegerListParameter(parameterName);
    Integer minimum1 = 11;
    Integer minimum2 = 22;
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    ChangeMinimumForIntegersTrigger trigger1 = new ChangeMinimumForIntegersTrigger(parameter,
      condition, minimum1);
    ChangeMinimumForIntegersTrigger trigger2 = new ChangeMinimumForIntegersTrigger(parameter,
      condition, minimum2);
    assertFalse(trigger1.equals(trigger2));
  }

  /**
   * Testando o mtodo {@link ChangeMinimumForIntegersTrigger#equals(Object)} com parmetro
   * com parmetro de tipos diferentes.
   */
  public void testEquals7() {
    String parameterName = "Nome do parmetro";
    IntegerListParameter parameter1 = createMockIntegerListParameter(parameterName);
    IntegerParameter parameter2 = createMockIntegerParameter(parameterName);
    Integer minimum = 10;
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    ChangeMinimumForIntegersTrigger trigger1 = new ChangeMinimumForIntegersTrigger(parameter1,
      condition, minimum);
    ChangeMinimumForIntegersTrigger trigger2 = new ChangeMinimumForIntegersTrigger(parameter2,
      condition, minimum);
    assertFalse(trigger1.equals(trigger2));
  }

  /**
   * Testando o mtodo {@link ChangeMinimumForIntegersTrigger#equals(Object)} com parmetro
   * do tipo {@link IntegerParameter} e com argumento {@code null}.
   */
  public void testEquals8() {
    String parameterName = "Nome do parmetro";
    IntegerParameter parameter = createMockIntegerParameter(parameterName);
    Integer minimum = 10;
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    ChangeMinimumForIntegersTrigger trigger = new ChangeMinimumForIntegersTrigger(parameter,
      condition, minimum);
    assertFalse(trigger.equals(null));
  }

  /**
   * Testando o mtodo {@link ChangeMinimumForIntegersTrigger#equals(Object)} com parmetro
   * do tipo {@link IntegerParameter} e com argumento de outra classe.
   */
  public void testEquals9() {
    String parameterName = "Nome do parmetro";
    IntegerParameter parameter = createMockIntegerParameter(parameterName);
    Integer minimum = 10;
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    ChangeMinimumForIntegersTrigger trigger = new ChangeMinimumForIntegersTrigger(parameter,
      condition, minimum);
    assertFalse(trigger.equals("Outra classe"));
  }

  /**
   * Testando o mtodo {@link ChangeMinimumForIntegersTrigger#equals(Object)} com parmetro
   * do tipo {@link IntegerListParameter} e com argumento {@code null}.
   */
  public void testEquals10() {
    String parameterName = "Nome do parmetro";
    IntegerListParameter parameter = createMockIntegerListParameter(parameterName);
    Integer minimum = 10;
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    ChangeMinimumForIntegersTrigger trigger = new ChangeMinimumForIntegersTrigger(parameter,
      condition, minimum);
    assertFalse(trigger.equals(null));
  }

  /**
   * Testando o mtodo {@link ChangeMinimumForIntegersTrigger#equals(Object)} com parmetro
   * do tipo {@link IntegerListParameter} e com argumento de outra classe.
   */
  public void testEquals13() {
    String parameterName = "Nome do parmetro";
    IntegerListParameter parameter = createMockIntegerListParameter(parameterName);
    Integer minimum = 10;
    Condition condition = new SimpleCondition("Nome do parmetro", "Valor");
    ChangeMinimumForIntegersTrigger trigger = new ChangeMinimumForIntegersTrigger(parameter,
      condition, minimum);
    assertFalse(trigger.equals("Outra classe"));
  }

  /**
   * Cria um configurador de algorithms qualquer.
   * 
   * @return .
   */
  private SimpleAlgorithmConfigurator createMockAlgorithmConfigurator() {
    AlgorithmInfo algorithmInfo = new AlgorithmInfo("ID Algoritmo",
      "Algoritmo", "DIR Algoritmo", new Hashtable<String, String>());
    AlgorithmVersionId algorithmVersionId = new AlgorithmVersionId(1, 2, 3);
    AlgorithmVersionInfo algorithmVersionInfo = new AlgorithmVersionInfo(
      algorithmInfo, algorithmVersionId,
      new Hashtable<String, List<FileInfo>>(),
      new Hashtable<String, String>());
    return new SimpleAlgorithmConfigurator(algorithmVersionInfo, null,
      ExecutionType.SIMPLE, null, false, "Comando", null, null, true, CMD_PATTERN);
  }

  /**
   * Cria um parmetro qualquer do tipo {@link IntegerListParameter}.
   * 
   * @return .
   */
  private IntegerListParameter createMockIntegerListParameter() {
    String parameterName = "Nome do parmetro";
    return createMockIntegerListParameter(parameterName);
  }

  /**
   * Cria um parmetro qualquer do tipo {@link IntegerListParameter}.
   * 
   * @param parameterName O nome do parmetro (No aceita {@code null}).
   * 
   * @return .
   */
  private IntegerListParameter createMockIntegerListParameter(String parameterName) {
    Integer minimum = null;
    return createMockIntegerListParameter(parameterName, minimum);
  }

  /**
   * Cria um parmetro qualquer do tipo {@link IntegerListParameter}.
   * 
   * @param parameterName O nome do parmetro (No aceita {@code null}).
   * @param minimum O mnimo (Aceita {@code null}).
   * 
   * @return .
   */
  private IntegerListParameter createMockIntegerListParameter(
    String parameterName, Integer minimum) {
    return new IntegerListParameter(parameterName, "Rtulo do parmetro",
      "Descrio do parmetro", false, null, null, minimum);
  }

  /**
   * Cria um parmetro qualquer do tipo {@link IntegerParameter}.
   * 
   * @return .
   */
  private IntegerParameter createMockIntegerParameter() {
    String parameterName = "Nome do parmetro";
    return createMockIntegerParameter(parameterName);
  }

  /**
   * Cria um parmetro qualquer do tipo {@link IntegerParameter}.
   * 
   * @param parameterName O nome do parmetro (No aceita {@code null}).
   * 
   * @return .
   */
  private IntegerParameter createMockIntegerParameter(String parameterName) {
    Integer defaultValue = null;
    Integer minimum = null;
    return createMockIntegerParameter(parameterName, defaultValue, minimum);
  }

  /**
   * Cria um parmetro qualquer do tipo {@link IntegerParameter}.
   * 
   * @param parameterName O nome do parmetro (No aceita {@code null}).
   * @param defaultValue O valor-padro (Aceita {@code null}).
   * @param minimum O mnimo (Aceita {@code null}).
   * 
   * @return .
   */
  private IntegerParameter createMockIntegerParameter(String parameterName,
    Integer defaultValue, Integer minimum) {
    return new IntegerParameter(parameterName, "Rtulo do parmetro",
      "Descrio do parmetro", defaultValue, false, null, null, minimum);
  }

  /**
   * Cria uma condio qualquer.
   * 
   * @return .
   */
  private Condition createMockCondition() {
    return new SimpleCondition("Nome do parmetro", "Valor qualquer");
  }
}
