/**
 * $Id: InitServletTest.java 102965 2010-03-12 23:13:36Z costa $
 */

package csbase.servlet;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.security.Principal;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletInputStream;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import junit.framework.TestCase;

/**
 * Testes unitrios para a classe <code>InitServlet</code>.
 * 
 * @author Leonardo Barros
 */
public class InitServletTest extends TestCase {
  /** Servlet para testes */
  // Embora normalmente no faa sentido instanciar diretamente um Servlet,
  // essa operao permite testar seus mtodos como se fosse uma classe qualquer
  private InitServlet servlet = new InitServlet();
  /** Requisio web "fake", para simular uma real interao */
  private MockHttpServletRequest mockRequest = new MockHttpServletRequest();

  /**
   * Testa chamada default (sem parmetros).
   */
  public void testGetCommonFileUrl1() {
    mockRequest.setServerName("localhost");
    mockRequest.setServerPort(8080);
    mockRequest.setRequestURI("/system/init");
    String expected = "http://localhost:8080/system/index.jnlp";
    assertEquals(expected, servlet.getBaseFileUrl(mockRequest));
  }

  /**
   * Testa chamada passando parmetro "page".
   */
  public void testGetCommonFileUrl2() {
    mockRequest.setServerName("localhost");
    mockRequest.setServerPort(8080);
    mockRequest.setRequestURI("/system/init?page=teste.jnlp");
    String expected = "http://localhost:8080/system/teste.jnlp";
    assertEquals(expected, servlet.getBaseFileUrl(mockRequest));
  }

  /**
   * Testa gerao do JNLP personalizado, somente com os parmetros
   * obrigatrios.
   */
  public void testCustomizeCommonFile1() {
    mockRequest.setServerName("localhost");
    mockRequest.setServerPort(8080);
    mockRequest.setRemoteAddr("10.0.0.0");
    mockRequest.setRequestURI("/system/init");
    String input =
      "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>"
        + "<jnlp spec=\"1.0+\" codebase=\"http://$server_host_name$:$server_port_http$/system/lib\" >"
        + "<information>"
        + "<title>InfoGrid</title>"
        + "<vendor>TECGRAF/PUC-RIO</vendor>"
        + "<homepage href=\"http://www.tecgraf.puc-rio.br\" />"
        + "<description>Aplicao para prover infraestrutura de GRID ao INFOPAE</description>"
        + "</information>"
        + "<resources>"
        + "<j2se version=\"1.5+\" />"
        + "<jar href=\"system.jar\"/>"
        + "<jar href=\"csbase.jar\"/>"
        + "</resources>"
        + "<application-desc main-class=\"csbase.client.kernel.Client\" >"
        + "<argument>--serverUrl $server_host_name$:$server_port_rmi$</argument>"
        + "<argument>--clientAddress $client_address$</argument>"
        + "</application-desc>" + "</jnlp>";
    String expected =
      "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>"
        + "<jnlp spec=\"1.0+\" codebase=\"http://localhost:8080/system/lib\" >"
        + "<information>"
        + "<title>InfoGrid</title>"
        + "<vendor>TECGRAF/PUC-RIO</vendor>"
        + "<homepage href=\"http://www.tecgraf.puc-rio.br\" />"
        + "<description>Aplicao para prover infraestrutura de GRID ao INFOPAE</description>"
        + "</information>" + "<resources>" + "<j2se version=\"1.5+\" />"
        + "<jar href=\"system.jar\"/>" + "<jar href=\"csbase.jar\"/>"
        + "</resources>"
        + "<application-desc main-class=\"csbase.client.kernel.Client\" >"
        + "<argument>--serverUrl localhost:1099</argument>"
        + "<argument>--clientAddress 10.0.0.0</argument>"
        + "</application-desc>" + "</jnlp>";
    //    FIXME no est funcionando aps a reestruturao do
    //    InitServlet
    //    assertEquals(expected, servlet.translateFile(input, mockRequest));
  }

  /**
   * <p>
   * Testa gerao do JNLP personalizado, com parmetros obrigatrios e
   * opcionais.
   * </p>
   * NOTA DE IMPLEMENTAO: Este teste pode falhar com futuras verses do JRE,
   * mesmo sem que seja necessrio corrigir o mtodo sendo testado. Isto se deve
   * ao fato de que este teste assume que a lista de parmetros opcionais ser
   * adicionada na ordem inversa em que foram passadas (implementao atual do
   * <code>HashMap</code>). Entretanto, esta ordem no  garantida pela
   * especificao, o que poder ocasionar a falha citada. Uma implementao
   * mais correta deste teste deveria considerar a hiptese dos parmetros serem
   * adicionados em qualquer ordem.
   */
  public void testCustomizeCommonFile2() {
    mockRequest.setServerName("localhost");
    mockRequest.setServerPort(8080);
    mockRequest.setRemoteAddr("10.0.0.0");
    mockRequest
      .setRequestURI("/system/init?server_host_addr=127.0.0.1&userKey=XPTO&"
        + "server_port_http=8080&server_port_rmi=9999&featureX=enabled&page=teste.jnlp");
    String input =
      "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>"
        + "<jnlp spec=\"1.0+\" codebase=\"http://$server_host_name$:$server_port_http$/system/jar\" >"
        + "<information>"
        + "<title>InfoGrid</title>"
        + "<vendor>TECGRAF/PUC-RIO</vendor>"
        + "<homepage href=\"http://www.tecgraf.puc-rio.br\" />"
        + "<description>Aplicao para prover infraestrutura de GRID ao INFOPAE</description>"
        + "</information>"
        + "<resources>"
        + "<j2se version=\"1.5+\" />"
        + "<jar href=\"system.jar\"/>"
        + "<jar href=\"csbase.jar\"/>"
        + "</resources>"
        + "<application-desc main-class=\"csbase.client.kernel.Client\" >"
        + "<argument>--serverUrl $server_host_addr$:$server_port_rmi$</argument>"
        + "<argument>--properties /local/prod/lbarros/system/system.properties</argument>"
        + "<argument>--clientAddress $client_address$</argument>"
        + "</application-desc>" + "</jnlp>";
    String expected =
      "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>"
        + "<jnlp spec=\"1.0+\" codebase=\"http://localhost:8080/system/jar\" >"
        + "<information>"
        + "<title>InfoGrid</title>"
        + "<vendor>TECGRAF/PUC-RIO</vendor>"
        + "<homepage href=\"http://www.tecgraf.puc-rio.br\" />"
        + "<description>Aplicao para prover infraestrutura de GRID ao INFOPAE</description>"
        + "</information>"
        + "<resources>"
        + "<j2se version=\"1.5+\" />"
        + "<jar href=\"system.jar\"/>"
        + "<jar href=\"csbase.jar\"/>"
        + "</resources>"
        + "<application-desc main-class=\"csbase.client.kernel.Client\" >"
        + "<argument>--serverUrl 127.0.0.1:9999</argument>"
        + "<argument>--properties /local/prod/lbarros/system/system.properties</argument>"
        + "<argument>--clientAddress 10.0.0.0</argument>"
        + "  <argument>--userKey XPTO</argument>\n"
        + "    <argument>--featureX enabled</argument>\n"
        + "  </application-desc>" + "</jnlp>";
    //    FIXME no est funcionando aps a reestruturao do
    //    InitServlet
    //    assertEquals(expected, servlet.translateFile(input, mockRequest));
  }

  /**
   * Testa o mtodo de a adio de parmetros opcionais.
   */
  public void testAddParam() {
    String key = "userKey";
    String value = "XPTOBLABLABLA72372505047258347508345703485";
    String input =
      "<jnlp><resources>...</resources><application-desc>"
        + "<argument>--serverUrl localhost:8080</argument>"
        + "</application-desc></jnlp>";
    String expected =
      "<jnlp><resources>...</resources><application-desc>"
        + "<argument>--serverUrl localhost:8080</argument>"
        + "  <argument>--userKey XPTOBLABLABLA72372505047258347508345703485</argument>\n"
        + "  </application-desc></jnlp>";
    assertEquals(expected, servlet.appendParam(key, value, input));
  }

  private class MockHttpServletRequest implements HttpServletRequest {
    /** Nome do servidor */
    private String serverName;
    /** Porta do servidor */
    private int serverPort;
    /** URI da requisio */
    private String requestURI;
    /** Mapa de parmetros */
    private Map<String, String> parameterMap = new HashMap<String, String>() {
      @Override
      public String remove(Object key) {
        throw new UnsupportedOperationException(
          "O mapa no pode ser modificado");
      }
    };
    /** IP do cliente que fez a requisio */
    private String remoteAddr;

    /**
     * Obtm a URI da requio.
     * 
     * @return URI da requisio.
     */
    public String getRequestURI() {
      return requestURI;
    }

    /**
     * Obtm o nome do servidor.
     * 
     * @return nome do servidor.
     */
    public String getServerName() {
      return serverName;
    }

    /**
     * Obtm a porta do servidor.
     * 
     * @return porta do servidor.
     */
    public int getServerPort() {
      return serverPort;
    }

    /**
     * Obtm o IP do cliente que fez a requisio.
     * 
     * @return IP do cliente que fez a requisio.
     */
    public String getRemoteAddr() {
      return remoteAddr;
    }

    /**
     * Substitui a URI da requisio.
     * 
     * @param requestURI nova URI para a requisio.
     */
    public void setRequestURI(String requestURI) {
      this.requestURI = requestURI;
      String[] URITokens = requestURI.split("\\?");
      if (URITokens != null && URITokens.length > 1) {
        String paramURI = URITokens[1];
        String[] params = paramURI.split("&");
        if (params == null) {
          return;
        }
        for (int i = 0; i < params.length; i++) {
          String[] param = params[i].split("=");
          if (param != null && param.length == 2) {
            String key = param[0];
            String value = param[1];
            parameterMap.put(key, value);
          }
        }
      }
    }

    /**
     * Substitui a porta do servidor.
     * 
     * @param serverPort nova porta do servidor.
     */
    public void setServerPort(int serverPort) {
      this.serverPort = serverPort;
    }

    /**
     * Substitui o nome do servidor.
     * 
     * @param serverName novo nome do servidor.
     */
    public void setServerName(String serverName) {
      this.serverName = serverName;
    }

    /**
     * Obtm o valor do parmetro especificado.
     * 
     * @return valor do parmetro especificado.
     */
    public String getParameter(String key) {
      return parameterMap.get(key);
    }

    /**
     * Obtm uma enumerao contendo os nomes dos parmetros.
     */
    public Enumeration getParameterNames() {
      return new Enumeration() {
        Iterator iter = parameterMap.keySet().iterator();

        public boolean hasMoreElements() {
          return iter.hasNext();
        }

        public Object nextElement() {
          return iter.next();
        }
      };
    }

    /**
     * Substitui o IP do cliente.
     * 
     * @param remoteAddr IP do cliente que fez a requisio.
     */
    public void setRemoteAddr(String remoteAddr) {
      this.remoteAddr = remoteAddr;
    }

    public String getAuthType() {
      return null;
    }

    public Map getParameterMap() {
      return null;
    }

    public Cookie[] getCookies() {
      return null;
    }

    public long getDateHeader(String arg0) {
      return 0;
    }

    public String getHeader(String arg0) {
      return null;
    }

    public Enumeration getHeaders(String arg0) {
      return null;
    }

    public Enumeration getHeaderNames() {
      return null;
    }

    public int getIntHeader(String arg0) {
      return 0;
    }

    public String getMethod() {
      return null;
    }

    public String getPathInfo() {
      return null;
    }

    public String getPathTranslated() {
      return null;
    }

    public String getContextPath() {
      return null;
    }

    public String getQueryString() {
      return null;
    }

    public String getRemoteUser() {
      return null;
    }

    public boolean isUserInRole(String arg0) {
      return false;
    }

    public Principal getUserPrincipal() {
      return null;
    }

    public String getRequestedSessionId() {
      return null;
    }

    public StringBuffer getRequestURL() {
      return null;
    }

    public String getServletPath() {
      return null;
    }

    public HttpSession getSession(boolean arg0) {
      return null;
    }

    public HttpSession getSession() {
      return null;
    }

    public boolean isRequestedSessionIdValid() {
      return false;
    }

    public boolean isRequestedSessionIdFromCookie() {
      return false;
    }

    public boolean isRequestedSessionIdFromURL() {
      return false;
    }

    public boolean isRequestedSessionIdFromUrl() {
      return false;
    }

    public Object getAttribute(String arg0) {
      return null;
    }

    public Enumeration getAttributeNames() {
      return null;
    }

    public String getCharacterEncoding() {
      return null;
    }

    public void setCharacterEncoding(String arg0)
      throws UnsupportedEncodingException {
    }

    public int getContentLength() {
      return 0;
    }

    public String getContentType() {
      return null;
    }

    public ServletInputStream getInputStream() throws IOException {
      return null;
    }

    public String[] getParameterValues(String arg0) {
      return null;
    }

    public String getProtocol() {
      return null;
    }

    public String getScheme() {
      return null;
    }

    public BufferedReader getReader() throws IOException {
      return null;
    }

    public String getRemoteHost() {
      return null;
    }

    public void setAttribute(String arg0, Object arg1) {
    }

    public void removeAttribute(String arg0) {
    }

    public Locale getLocale() {
      return null;
    }

    public Enumeration getLocales() {
      return null;
    }

    public boolean isSecure() {
      return false;
    }

    public RequestDispatcher getRequestDispatcher(String arg0) {
      return null;
    }

    public String getRealPath(String arg0) {
      return null;
    }

  }

}
