package csbase.util.parser;

import csbase.util.parser.symbols.Symbol;

import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * Representa um token utilizado por um autmato finito ({@link
 * csbase.util.parser.FiniteAutomaton}).
 */
public class Token {
  /** Contm os smbolos ({@link Symbol}) que representam o token. */
  private List symbolList;

  /**
   * Cria um token sem smbolos.
   */
  protected Token() {
    this.symbolList = new LinkedList();
  }

  /**
   * Adiciona um smbolo ao token.
   *
   * @param symbol O smbolo a ser inserido.
   *
   * @return true, caso o smbolo tenha sido inserido, ou false, caso
   *         contrrio.
   */
  public final boolean addSymbol(Symbol symbol) {
    return this.symbolList.add(symbol);
  }

  /**
   * Obtm a quantidade de smbolos do token.
   *
   * @return A quantidade de smbolos do token.
   */
  public final int getSize() {
    return this.symbolList.size();
  }

  /**
   * Obtm a lista de smbolos que compe o token.
   *
   * @return A lista de smbolos que compe o token.
   */
  public final List getSymbolList() {
    return Collections.unmodifiableList(this.symbolList);
  }

  public final String toString() {
    StringBuffer buffer = new StringBuffer();
    Iterator iterator = this.symbolList.iterator();
    while (iterator.hasNext()) {
      buffer.append(iterator.next());
    }
    return buffer.toString();
  }
}
