package csbase.util.parser.actions;

import csbase.util.parser.Session;
import csbase.util.parser.exception.AutomatonException;
import csbase.util.parser.symbols.Symbol;

import java.util.LinkedList;
import java.util.List;

/**
 * Representa uma composio de aes de transio. Todas as aes da
 * composio sero executadas, respeitando a ordem em que foram inseridas na
 * composio.
 */
public final class CompositeTransitionAction implements TransitionAction {
  /** Contm as aes que fazem parte da composio. */
  private List<TransitionAction> actions;

  /**
   * Cria uma composio de aes de transio.
   */
  public CompositeTransitionAction() {
    this.actions = new LinkedList<TransitionAction>();
  }

  /**
   * Cria uma composio de aes de transio.
   * 
   * @param actions As aes (No aceita {@code null}).
   */
  public CompositeTransitionAction(TransitionAction...actions) {
    this();
    for (TransitionAction action : actions) {
      addAction(action);
    }
  }
  
  /**
   * Adiciona uma ao  composio.
   *
   * @param action A ao a ser adicionada.
   *
   * @return true, se a ao foi adicionadas, ou false, caso contrrio.
   *
   * @throws IllegalArgumentException Caso a ao recebida seja nula.
   */
  public boolean addAction(TransitionAction action) {
    if (action == null) {
      throw new IllegalArgumentException(
        "No  permitido incluir aes nulas (null).");
    }
    return this.actions.add(action);
  }

  public void execute(Symbol symbol, Session session)
  throws AutomatonException {
    for (TransitionAction action : this.actions) {
      action.execute(symbol, session);
    }
  }
}
