package csbase.util.parser.actions;

import csbase.util.parser.Session;
import csbase.util.parser.exception.AutomatonException;
import csbase.util.parser.exception.ErrorStateReachedException;

/**
 *  a ao que ser executada ao entrarmos no estado de erro ({@link
 * csbase.util.parser.ErrorState}) do autmato finito ({@link
 * csbase.util.parser.FiniteAutomaton}).
 */
public final class ErrorStateAction implements StateAction {
  /** A instncia nica. */
  private static ErrorStateAction instance;

  /**
   * Construtor definido para impedir a criao de objetos desta classe
   * externamente.
   */
  private ErrorStateAction() {}

  /**
   * Obtm a instncia nica.
   *
   * @return A instncia nica.
   */
  public static ErrorStateAction getInstance() {
    if (instance == null) {
      instance = new ErrorStateAction();
    }
    return instance;
  }

  public void execute(Session session)
  throws AutomatonException {
    throw new ErrorStateReachedException(session);
  }
}
