package csbase.util.parser.exception;

import csbase.util.parser.Session;
import csbase.util.parser.State;
import csbase.util.parser.symbols.Symbol;

/**
 * Representa uma exceo gerada quando no existe transio para um
 * determinado smbolo dentro de um estado.
 */
public final class NoTransitionException extends AutomatonException {
  /** O smbolo que no possui transio definida. */
  private Symbol symbol;

  /** O estado para o qual a transio no est definida. */
  private State state;

  /**
   * Cria uma instncia com a sesso onde foi gerada a exceo, o smbolo para
   * o qual no existe transio e o estado de onde se tentou obter a
   * transio.
   *
   * @param session A sesso onde foi gerada a exceo.
   * @param symbol O smbolo para o qual no existe transio.
   * @param state O estado onde a exceo foi gerada.
   */
  public NoTransitionException(Session session, Symbol symbol, State state) {
    super(session);
    this.symbol = symbol;
    this.state = state;
  }

  public String toString() {
    String string = super.toString();
    return string+" - Smbolo: "+symbol.toString()+" - Estado: "+
    state.toString();
  }
}
