package csbase.util.parser.iterators;

import csbase.util.parser.symbols.CharSymbol;
import csbase.util.parser.symbols.Symbol;

/**
 * Representa um iterador de smbolos de caracter ({@link
 * csbase.util.parser.symbols.CharSymbol});
 */
public final class CharSymbolIterator implements SymbolIterator {
  /** Os caracteres que sero iterados. */
  private char[] charArray;

  /** A posio atual. */
  private int currentPosition;

  /**
   * Cria um iterador de smbolos a partir dos caracteres recebidos.
   *
   * @param charArray Um array de caracteres que sero iterados.
   */
  public CharSymbolIterator(char[] charArray) {
    this.charArray = (char[])charArray.clone();
    this.currentPosition = 0;
  }

  /**
   * Cria um iterador de smbolos a partir da String recebida. Cada caracter da
   * String ser um smbolo.
   *
   * @param string A string com os caracteres que sero iterados.
   */
  public CharSymbolIterator(String string) {
    this(string.toCharArray());
  }

  public boolean hasNext() {
    return (currentPosition < charArray.length);
  }

  public Symbol getNext() {
    if (this.hasNext()) {
      CharSymbol symbol = new CharSymbol(this.charArray[this.currentPosition]);
      this.currentPosition++;
      return symbol;
    }
    return null;
  }
}
