package csbase.util.remote;

import java.io.InputStream;
import java.rmi.Remote;
import java.rmi.RemoteException;

/**
 * Interface do {@link RMIInputStreamImpl}.
 * 
 * @author http://www.maelstrom.org.uk/rmi/rmi.html
 */
interface RMIInputStream extends Remote {

  /**
   * @see InputStream#available()
   * @throws RemoteException
   */
  int available() throws RemoteException;

  /**
   * @see InputStream#close()
   * @throws RemoteException
   */
  void close() throws RemoteException;

  /**
   * @see InputStream#mark(int)
   * @throws RemoteException
   */
  void mark(int readlimit) throws RemoteException;

  /**
   * @see InputStream#markSupported()
   * @throws RemoteException
   */
  boolean markSupported() throws RemoteException;

  /**
   * @see InputStream#read()
   * @throws RemoteException
   */
  int read() throws RemoteException;

  /**
   * Similar ao {@link InputStream#read(byte[], int, int)}. Porm, ao invs de
   * receber um array de bytes, retorna um StreamData com o contedo lido.
   * 
   * @param len Nmero mximo de bytes a serem lidos.
   * @return Um {@link StreamData} contendo um buffer e o nmero de bytes lidos. 
   * @throws RemoteException
   */
  StreamData read(int len) throws RemoteException;

  /**
   * @see InputStream#reset()
   * @throws RemoteException
   */
  void reset() throws RemoteException;

  /**
   * @see InputStream#skip(long)
   * @throws RemoteException
   */
  long skip(long n) throws RemoteException;

}