package csbase.util.remote;

import java.io.IOException;
import java.io.InputStream;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

/**
 * Encapsula um {@link InputStream} oferecendo uma interface remota a seus
 * mtodos.
 * 
 * @author http://www.maelstrom.org.uk/rmi/rmi.html
 */
class RMIInputStreamImpl extends UnicastRemoteObject implements RMIInputStream {

  /** {@link InputStream} encapsulado. */
  private InputStream in;
  
  /**
   * Construtor.
   * 
   * @param in {@link InputStream} a ser encapsulado.
   * 
   * @throws RemoteException
   */
  public RMIInputStreamImpl(InputStream in) throws RemoteException {
    super();
    this.in = in;
  }
  
  /**
   * {@inheritDoc}
   */
  public int available() throws RemoteException  {
    try {
      return in.available();
    }
    catch (IOException e) {
      throw new RemoteException("IOException no servidor", e);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void close() throws RemoteException {
    try {
      in.close();
    }
    catch (IOException e) {
      throw new RemoteException("IOException no servidor", e);
    }
  }

  /**
   * {@inheritDoc}
   */
  public synchronized void mark(int readlimit) {
    in.mark(readlimit);
  }

  /**
   * {@inheritDoc}
   */
  public boolean markSupported() {
    return in.markSupported();
  }

  /**
   * {@inheritDoc}
   */
  public int read() throws RemoteException {
    try {
      return in.read();
    }
    catch (IOException e) {
      throw new RemoteException("IOException no servidor", e);
    }
  }

  /**
   * {@inheritDoc}
   */
  public StreamData read(int len) throws RemoteException {
    try {
      byte[] holder = new byte[len];
      int i = in.read(holder, 0, len);
      return new StreamData(holder, i);
    }
    catch (IOException ex) {
      throw new RemoteException("IOException no servidor", ex);
    }
  } 

  /**
   * {@inheritDoc}
   */
  public synchronized void reset() throws RemoteException {
    try {
      in.reset();
    }
    catch (IOException e) {
      throw new RemoteException("IOException no servidor", e);
    }
  }

  /**
   * {@inheritDoc}
   */
  public long skip(long n) throws RemoteException {
    try {
      return in.skip(n);
    }
    catch (IOException e) {
      throw new RemoteException("IOException no servidor", e);
    }
  }
}
