package csbase.util.remote;

import java.io.IOException;
import java.io.OutputStream;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

/**
 * Encapsula um {@link OutputStream} oferecendo uma interface remota a seus
 * mtodos.
 * 
 * @author http://www.maelstrom.org.uk/rmi/rmi.html
 */
class RMIOutputStreamImpl extends UnicastRemoteObject implements
  RMIOutputStream {

  /** {@link OutputStream} encapsulado. */
  private OutputStream out;

  /**
   * Construtor.
   * 
   * @param out {@link OutputStream} a ser encapsulado.
   * 
   * @throws RemoteException
   */
  public RMIOutputStreamImpl(OutputStream out) throws RemoteException {
    super();
    this.out = out;
  }

  /**
   * {@inheritDoc}
   */
  public void close() throws RemoteException {
    try {
      out.close();
    }
    catch (IOException e) {
      throw new RemoteException("IOException no servidor", e);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void flush() throws RemoteException {
    try {
      out.flush();
    }
    catch (IOException e) {
      throw new RemoteException("IOException no servidor", e);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void write(byte[] b, int off, int len) throws RemoteException {
    try {
      out.write(b, off, len);
    }
    catch (IOException e) {
      throw new RemoteException("IOException no servidor", e);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void write(byte[] b) throws RemoteException {
    try {
      out.write(b);
    }
    catch (IOException e) {
      throw new RemoteException("IOException no servidor", e);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void write(int b) throws RemoteException {
    try {
      out.write(b);
    }
    catch (IOException e) {
      throw new RemoteException("IOException no servidor", e);
    }
  }
}