package csbase.util.remote;

import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.rmi.RemoteException;

/**
 * {@link InputStream} remoto para ser enviado via RMI.   
 * 
 * @author http://www.maelstrom.org.uk/rmi/rmi.html
 */
public class RemoteInputStream extends InputStream implements Serializable {
  
  /** Proxy para o {@link InputStream}. */
  private RMIInputStream remoteStream;

  /**
   * Construtor.
   * 
   * @param in Stream a ser acessado.
   * @throws RemoteException
   */
  public RemoteInputStream(InputStream in) throws RemoteException {
    this.remoteStream = new RMIInputStreamImpl(in);
  }
  
  /**
   * {@inheritDoc}
   */
  @Override
  public int available() throws IOException {
      try {
          return remoteStream.available();
      } catch (RemoteException ex) {
          throw new IOException("RemoteException " + ex);
      }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void close() throws IOException {
      try {
          remoteStream.close();
      } catch (RemoteException ex) {
          throw new IOException("RemoteException " + ex);
      }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void mark(int readlimit) {
      try {
          remoteStream.mark(readlimit);
      } catch (RemoteException ex) {
          System.err.println("mark caused RemoteException " + ex);
      }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean markSupported() {
      try {
          return remoteStream.markSupported();
      } catch (RemoteException ex) {
          System.err.println("markSupported caused RemoteException " + ex);
          return false;
      }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public int read() throws IOException {
      try {
          return remoteStream.read();
      } catch (RemoteException ex) {
          throw new IOException("RemoteException " + ex);
      }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public int read(byte[] b) throws IOException {
      try {
          return read(b, 0, b.length);
      } catch (RemoteException ex) {
          throw new IOException("RemoteException " + ex);
      }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public int read(byte[] b, int off, int len) throws IOException {
      try {
          byte[] holder;
          int result;
          StreamData data = remoteStream.read(len);
          holder = data.getData();
          result = data.getResult();
          if (result > 0) {
            System.arraycopy(holder, 0, b, off, result);
          }
          return result;
      } catch (RemoteException ex) {
          throw new IOException("RemoteException " + ex);
      }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void reset() throws IOException {
      try {
          remoteStream.reset();
      } catch (RemoteException ex) {
          throw new IOException("RemoteException " + ex);
      }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public long skip(long n) throws IOException {
      try {
          return remoteStream.skip(n);
      } catch (RemoteException ex) {
          throw new IOException("RemoteException " + ex);
      }
  }
} 
