package csbase.util.remote;

import java.io.IOException;
import java.io.OutputStream;
import java.rmi.RemoteException;

/**
 * {@link OutputStream} remoto para ser enviado via RMI.   
 * 
 * @author http://www.maelstrom.org.uk/rmi/rmi.html
 */
public class RemoteOutputStream extends OutputStream {

  /** Proxy para o {@link OutputStream}. */
  private RMIOutputStream remoteStream;

  /**
   * Construtor.
   * 
   * @param out Stream a ser acessado.
   * @throws RemoteException
   */
  public RemoteOutputStream(OutputStream out) throws RemoteException {
    this.remoteStream = new RMIOutputStreamImpl(out);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void close() throws IOException {
    try {
      remoteStream.close();
    }
    catch (RemoteException e) {
      throw new IOException("RemoteException " + e);
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void flush() throws IOException {
    try {
      remoteStream.flush();
    }
    catch (RemoteException e) {
      throw new IOException("RemoteException " + e);
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void write(byte[] b, int off, int len) throws IOException {
    try {
      remoteStream.write(b, off, len);
    }
    catch (RemoteException e) {
      throw new IOException("RemoteException " + e);
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void write(byte[] b) throws IOException {
    try {
      remoteStream.write(b);
    }
    catch (RemoteException e) {
      throw new IOException("RemoteException " + e);
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void write(int b) throws IOException {
    try {
      remoteStream.write(b);
    }
    catch (RemoteException e) {
      throw new IOException("RemoteException " + e);
    }
  }
}
