package csbase.client.applicationmanager;

import java.util.Hashtable;
import java.util.Set;

import runner.Runner;

import csbase.client.applications.Application;
import csbase.logic.applicationservice.ApplicationRegistry;

/**
 * Modelo de gerente de aplicaes.
 * 
 * @author Tecgraf/PUC-Rio
 */
public final class ApplicationManager {

  /**
   * Instncia de singleton.
   */
  private static ApplicationManager instance = new ApplicationManager();

  /**
   * Lista de tipos de aplicaes registradas.
   */
  final private Hashtable<String, ApplicationRegistry> applications =
    new Hashtable<String, ApplicationRegistry>();

  /**
   * Tomada de um id de aplicao com base em um identificador.
   * 
   * @param id id
   * @return o registro.
   */
  final public ApplicationRegistry getApplicationRegistry(final String id) {
    if (id == null) {
      throw new IllegalArgumentException("invalid null id");
    }
    if (applications.containsKey(id)) {
      return applications.get(id);
    }
    return null;
  }

  /**
   * @return a instncia
   */
  final static public ApplicationManager getInstance() {
    return instance;
  }

  /**
   * Construo de um registry.
   * 
   * @param id identificador da aplicao.
   */
  public void buildNewApplicationRegistry(String id) {
    if (applications.containsKey(id)) {
      throw new IllegalArgumentException("double registered id");
    }
    final ApplicationRegistry reg = new ApplicationRegistry(id);
    applications.put(reg.getId(), reg);
  }

  /**
   * @return o conjunto de aplicaes.
   */
  public Set<String> getAllAplicationsIds() {
    return applications.keySet();
  }

  /**
   * Execuo de uma aplicao com bas em um id.
   * 
   * @param <A> tipo da aplicao
   * @param id id da aplicao
   * @return a instncia da aplicao lanada.
   */
  @SuppressWarnings("unchecked")
  final public <A extends Application> A runApplication(final String id) {
    final Runner runner = Runner.getInstance();
    return (A) runner.runApplication(id);
  }

}
