package csbase.logic;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.rmi.RemoteException;

/**
 * @author Tecgraf/PUC-Rio
 */
public class ClientProjectFileInputStream extends InputStream {

  /**
   * Stream interno.
   */
  final private FileInputStream inputStream;

  /**
   * {@inheritDoc}
   */
  @Override
  public int read() throws IOException {
    return inputStream.read();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public long skip(long n) throws IOException {
    return inputStream.skip(n);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public int available() throws IOException {
    return inputStream.available();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void close() throws IOException {
    inputStream.close();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public synchronized void mark(int readlimit) {
    inputStream.mark(readlimit);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public synchronized void reset() throws IOException {
    inputStream.reset();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean markSupported() {
    return inputStream.markSupported();
  }

  /**
   * Cria um <code>InputStream</code> a partir de um arquivo da rea de
   * projetos, abrindo-o para leitura/escrita e lendo um bloco (cache) inicial.
   * 
   * @param clientProjectFile Arquivo da rea de projetos (
   *        <code>ClientProjectFile</code>).
   * 
   * @param cacheSize Tamanho desejado para a cache.
   * 
   * @throws IOException se houver falha de acesso ao arquivo.
   * @throws RemoteException se houver (simulao) erro de acesso ao arquivo
   *         remoto no servidor.
   */
  public ClientProjectFileInputStream(
    final ClientProjectFile clientProjectFile, final int cacheSize)
    throws IOException, RemoteException {
    final File file = clientProjectFile.getFile();
    inputStream = new FileInputStream(file);
  }
}
