package csbase.logic;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.rmi.RemoteException;

/**
 * @author Tecgraf/PUC-Rio
 */
public class ClientProjectFileOutputStream extends OutputStream {

  /**
   * Stream interno.
   */
  final private FileOutputStream outputStream;

  /**
   * {@inheritDoc}
   */
  @Override
  public void write(int b) throws IOException {
    outputStream.write(b);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void write(byte b[]) throws IOException {
    outputStream.write(b);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void write(byte b[], int off, int len) throws IOException {
    outputStream.write(b, off, len);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void close() throws IOException {
    outputStream.close();
  }

  /**
   * Cria um <code>InputStream</code> a partir de um arquivo da rea de
   * projetos, abrindo-o para leitura/escrita e lendo um bloco (cache) inicial.
   * 
   * @param clientProjectFile Arquivo da rea de projetos (
   *        <code>ClientProjectFile</code>).
   * 
   * @throws IOException se houver falha no acesso ao arquivo.
   * @throws RemoteException se (simulao) houver falha no acesso ao arquivo
   *         remoto no servidor.
   */
  public ClientProjectFileOutputStream(final ClientProjectFile clientProjectFile)
    throws IOException, RemoteException {
    final File file = clientProjectFile.getFile();
    outputStream = new FileOutputStream(file);
  }
}
