package runner;

import java.io.IOException;
import java.io.InputStream;
import java.text.MessageFormat;
import java.util.Locale;
import java.util.PropertyResourceBundle;

/**
 * Simulao de classe tradutora global, que possui a mesma interface da classe
 * <code>LNG</code> do javautils.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class LNG {

  /**
   * Locale Global (inicializado com portugus do Brasil)
   */
  static private Locale locale = new Locale("pt", "BR");

  /**
   * Bundle do sistema.
   */
  static private PropertyResourceBundle bundle = null;

  /**
   * Carga do resource bundle da aplicao com base no locale
   * 
   * @throws IOException em caso de falha na carga do arquivo.
   */
  final static public void loadBundle() throws IOException {
    final String language = locale.getLanguage();
    final String country = locale.getCountry();
    final String idiomFileName = "resources/idiom" + "_" + language + "_"
      + country + ".properties";
    try {
      InputStream in = LNG.class.getResourceAsStream(idiomFileName);
      bundle = new PropertyResourceBundle(in);
    }
    catch (NullPointerException npe) {
      throw new IOException("LNG failed for "+idiomFileName, npe);
    }
  }

  /**
   * Ajuste do locale
   * 
   * @param languageName cdigo do idioma
   * @param countryName cdigo do pas
   */
  final static public void setLocale(final String languageName,
    final String countryName) {
    locale = new Locale(languageName, countryName);
  }

  /**
   * Consulta ao locale
   * 
   * @return o locale
   */
  final static public Locale getLocale() {
    return locale;
  }

  /**
   * <p>
   * Obtm a traduo de uma propriedade para o <code>Locale</code> atualmente
   * em uso.
   * </p>
   * 
   * @param key chave a ser buscada.
   * 
   * @return traduo da chave no <code>Locale</code> atual.
   */
  final public static String get(String key) {
    if (key == null) {
      throw new IllegalStateException("key == null");
    }
    if (bundle == null) {
      throw new IllegalStateException("null bundle!");
    }
    return bundle.getString(key);
  }

  /**
   * Obtem uma string de idioma.
   * 
   * @param key a chave de busca.
   * @param args os objetos de formatao
   * @return o texto no idioma correto.
   */
  final public static String get(final String key, final Object[] args) {
    if (args == null) {
      throw new IllegalStateException("args == null");
    }
    if (args.length == 0) {
      throw new IllegalStateException("args.length == 0");
    }
    String pattern = get(key);
    return MessageFormat.format(pattern, args);
  }
}
