package runner.filetypes;

import java.util.ArrayList;

/**
 * @author Tecgraf/PUC-Rio
 * 
 */
public class FileType {

  /**
   * O tipo
   */
  final private String name;

  /**
   * As extenses padronizadas
   */
  final private ArrayList<String> extensions = new ArrayList<String>();

  /**
   * Consulta as extenses do tipo;
   * 
   * @return extensions
   */
  public final ArrayList<String> getExtensions() {
    return extensions;
  }

  /**
   * Adiciona uma extenso ao tipo
   * 
   * @param extension a extenso.
   */
  final public void addExtension(final String extension) {
    final FileType type = FileTypes.getFileTypeFromExtension(extension);
    if (type != null) {
      final String err = "Extension " + extension
        + " is already assigned to type: " + type;
      throw new IllegalArgumentException(err);
    }
    extensions.add(extension);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  final public String toString() {
    return getName();
  }

  /**
   * Consulta do nome do tipo
   * 
   * @return o nome.
   */
  final public String getName() {
    return name;
  }

  /**
   * Construtor
   * 
   * @param type o tipo
   */
  public FileType(final String type) {
    if (type == null) {
      throw new IllegalArgumentException("Internal error: Type cannot be null.");
    }
    this.name = type.trim().toUpperCase();
  }
}
