package runner.filetypes;

import java.io.File;
import java.util.ArrayList;

import javax.swing.filechooser.FileFilter;

import runner.LNG;

/**
 * Filtro para tipo de arquivo.
 */
public class FileTypeFilter extends FileFilter {

  /**
   * Tipos permitidos.
   */
  final private ArrayList<FileType> typesList;

  /**
   * Retorna a extenso de um nome de arquivo
   * 
   * @param fileName o nome do arquivo.
   * @return a extenso.
   */
  private String getFileExtension(final String fileName) {
    String ext = null;
    int i = fileName.lastIndexOf('.');

    if (i > 0 && i < fileName.length() - 1) {
      ext = fileName.substring(i + 1).toLowerCase();
    }
    return ext;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  final public boolean accept(final File f) {
    if (f.isDirectory()) {
      return true;
    }

    final String extension = getFileExtension(f.getName());
    if (extension == null) {
      return false;
    }

    for (FileType type : typesList) {
      final ArrayList<String> extensions = type.getExtensions();
      for (String ext : extensions) {
        if (ext.equalsIgnoreCase(extension)) return true;
      }
    }

    return false;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  final public String getDescription() {
    String typesStr = "";
    for (FileType type : typesList) {
      typesStr = typesStr + " " + type.getName();
    }
    final String[] args = new String[] { typesStr };
    return LNG.get(getClass().getSimpleName() + ".description", args);
  }

  /**
   * Construtor
   * 
   * @param typesList lista de tipos aceitveis.
   */
  public FileTypeFilter(final ArrayList<FileType> typesList) {
    this.typesList = typesList;
  }
}
