package runner.filetypes;

import java.util.ArrayList;

import csbase.logic.ProjectFileType;

import runner.Runner;

/**
 * @author Tecgraf/PUC-Rio
 * 
 */
public class FileTypes {

  /**
   * Lista de associaes de extenses e tipos de arquivos.
   */
  static private ArrayList<FileType> e2tList = new ArrayList<FileType>();

  /**
   * Cria um tipo de arquivo para o {@link Runner}.
   * 
   * @param typeName o nome do novo tipo.
   * @return o tipo criado.
   */
  final public static FileType createFileType(final String typeName) {
    if (hasFileType(typeName)) {
      final String err = "Type " + typeName + " already exists!";
      throw new IllegalArgumentException(err);
    }
    final FileType typ = new FileType(typeName);
    e2tList.add(typ);
    return typ;
  }

  /**
   * Indica se um tipo j existe.
   * 
   * @param typeName o tipo
   * @return indicativo
   */
  final static public FileType getFileType(final String typeName) {
    if (typeName.equals(ProjectFileType.UNKNOWN)) {
      return null;
    }
    
    for (FileType value : e2tList) {
      final String name = value.getName();
      if (name.equalsIgnoreCase(typeName)) return value;
    }
    return null;
  }

  /**
   * Indica se um tipo j existe.
   * 
   * @param typeName o tipo
   * @return indicativo
   */
  final static public boolean hasFileType(final String typeName) {
    return getFileType(typeName) != null;
  }

  /**
   * Busca um tipo de arquivo aplicvel a extenso sugerida.
   * 
   * @param extension extenso
   * @return o tipo
   */
  static public final FileType getFileTypeFromExtension(final String extension) {
    if (extension == null) {
      throw new RuntimeException("Internal error: null extension not allowed");
    }
    for (FileType value : e2tList) {
      final ArrayList<String> extensions = value.getExtensions();
      for (String ext : extensions) {
        if (extension.trim().equalsIgnoreCase((ext.trim()))) {
          return value;
        }
      }
    }
    return null;
  }

  /**
   * Busca a extenso sugerida com base em um tipo de arquivo aplicvel.
   * 
   * @param typeName o tipo
   * @return as extenses associadas ao tipo
   */
  static public final String[] getExtensionsFromFileType(final String typeName) {
    if (typeName == null) {
      throw new RuntimeException("Internal error: null type name not allowed");
    }
    if (typeName.equals(ProjectFileType.UNKNOWN)) {
      return null;
    }

    for (FileType value : e2tList) {
      final String name = value.getName();
      if (name.trim().equalsIgnoreCase(typeName.trim())) {
        final ArrayList<String> extensions = value.getExtensions();
        final int sz = extensions.size();
        return extensions.toArray(new String[sz]);
      }
    }
    return null;
  }
}
