# -------------------------------------------------------------------
# User Definitions
# -------------------------------------------------------------------
ifndef APP_VERSION
APP_VERSION = 0.0.0
endif

ifndef APP_ID
APP_ID = myapplicationid
endif

ifndef APP_CLASS_NAME
APP_CLASS_NAME = MyApplicationClassName
endif

ifndef APP_LANGUAGE
APP_LANGUAGE = pt
endif

ifndef APP_COUNTRY
APP_COUNTRY = BR
endif

ifndef APP_NAME
APP_NAME = "No name set for this application"
endif

#####################################################################
## Do not change below this line!
#####################################################################

ARGS = \
 --verbose \
 --locale $(APP_LANGUAGE) $(APP_COUNTRY) \
 --file-type TEXT txt \
 --new-id $(APP_ID) \
 --start-id $(APP_ID) \
 --app-bundle $(APP_ID)\
 --app-class $(APP_ID) $(APP_CLASS_NAME) \
 --app-name $(APP_ID) $(APP_NAME)

# -------------------------------------------------------------------
# System definitions 
# -------------------------------------------------------------------
KITDIR = ../../kit/
include $(KITDIR)/csdk.mak

# -------------------------------------------------------------------
# Internal Definitions
# -------------------------------------------------------------------
 
CSDK_JARFILE = $(KITDIR)/deploy/csdk-$(CSDK_VERSION).jar
DEPLOYDIR = deploy
SRCDIR = src
CLASSDIR = classes

JARFILE = $(DEPLOYDIR)/$(APP_ID)-$(APP_VERSION).jar

RC_TARGET_DIR = $(CLASSDIR)/$(APP_ID)/resources
RC_SOURCE_DIR = $(SRCDIR)/$(APP_ID)/resources

# .....................................................................
# .....................................................................
# .....................................................................
# -------------------------------------------------------------------
# Targets
# -------------------------------------------------------------------

default: presentation compile deploy done
rebuild: presentation clean-all compile deploy done
clean: presentation clean-all done
hudson: presentation clean-all compile deploy clean-classes done

# -------------------------------------------------------------------
# Rules
# -------------------------------------------------------------------
                                                                                
done: 
	@(echo "   + $(APP_ID) ended.")

presentation:
	@(echo "--------------------------------------------------")
	@(echo "CSDK / $(APP_ID) - $(APP_VERSION)")
	@(echo "--------------------------------------------------")
                                                                                
clean-all: clean-classes clean-deploy

clean-classes: 
	@(echo "   + Cleaning $(APP_ID) class files...")
	@(rm -f `find $(SRCDIR) -name "*.class" -print`)

clean-deploy: 
	@(echo "   + Cleaning deploy - $(DEPLOYDIR)...")
	@(rm -fr $(DEPLOYDIR))
	@(rm -fr $(CLASSDIR))
                                                                                
update: 
	@(echo "   + Updating $(APP_ID)...")
	@(svn update)
                                                                                
deploy: 
	@(echo "   + Building $(JARFILE) file...")
	@(rm -fr $(DEPLOYDIR))
	@(mkdir -p $(DEPLOYDIR))
	@(cd $(CLASSDIR) ; jar -cf ../$(JARFILE) `find . -name "*.*" -print` > /dev/null)
                                                                                
TMP_RC_FILE = tmp-resources.tgz
OUT_EXTENSIONS = \
  -name "*.png" -or \
  -name "*.jpg" -or \
  -name "*.properties" -or \
  -name "*.gif"  

compile: 
	@(echo "   + Compiling $(APP_ID) v.$(APP_VERSION)...")
	@(mkdir -p $(CLASSDIR))
	@(javac -encoding ISO-8859-1 -d $(CLASSDIR) -classpath $(CSDK_JARFILE):$(APP_JARS) -sourcepath $(SRCDIR) `find $(SRCDIR) -name "*.java" -print` > /dev/null)
	@(echo "   + Copying resources...")
	@(cd $(SRCDIR) ; tar -czf ../$(CLASSDIR)/$(TMP_RC_FILE) `find . \( $(OUT_EXTENSIONS) \) -print` > /dev/null)
	@(cd $(CLASSDIR) ; tar -xzf $(TMP_RC_FILE) ; rm -f $(TMP_RC_FILE))
                                                                                

run:
	@(echo "   + Running $(APP_ID) v.$(APP_VERSION)...")
	@(java -classpath $(CSDK_JARFILE):$(APP_JARS):$(JARFILE) $(CSDK_RUNNER_CLASS) $(ARGS))
