package textviewer;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Font;

import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JToolBar;

import csbase.client.applications.ApplicationAboutAction;
import csbase.client.applications.ApplicationExitAction;
import csbase.client.applications.ApplicationProject;

/**
 * Aplicao de testes mnima do CSDK que permite a abertura e gravao de
 * arquivos ASCII.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class TextViewer extends ApplicationProject {

  /**
   * rea de texto/
   */
  final JTextArea textArea = new JTextArea();

  /**
   * {@inheritDoc}
   */
  @Override
  protected void killApplication() {
  }

  /**
   * Ajusta o texto visvel no painel.
   * 
   * @param string o texto
   */
  public void setText(String string) {
    textArea.setText(string);
  }

  /**
   * Retorna o texto exibido no programa
   * 
   * @return a string visvel no painel
   */
  public String getText() {
    return textArea.getText();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  protected boolean userCanKillApplication() {
    return true;
  }

  /**
   * Montagem do dilogo principal.
   */
  private void buildFrame() {
    final JFrame mainFrame = getApplicationFrame();
    mainFrame.setJMenuBar(buildMenuBar());

    final JPanel panel = new JPanel();

    textArea.setFont(new Font("Monospaced", Font.BOLD, 14));
    textArea.setTabSize(4);

    final JScrollPane scrollPane = new JScrollPane(textArea);
    scrollPane
      .setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
    final Dimension scrDim = new Dimension(600, 400);
    scrollPane.setSize(scrDim);
    scrollPane.setPreferredSize(scrDim);
    scrollPane.setMinimumSize(scrDim);

    panel.setLayout(new BorderLayout());
    panel.add(scrollPane, BorderLayout.CENTER);
    panel.add(buildToolbar(), BorderLayout.NORTH);

    mainFrame.getContentPane().add(panel);
    mainFrame.setSize(new Dimension(600, 400));
  }

  /**
   * Montagem do menu.
   * 
   * @return o menu do programa.
   */
  private JMenuBar buildMenuBar() {
    final JMenuBar menuBar = new JMenuBar();
    menuBar.add(buildOptionsMenu());
    menuBar.add(buildAboutMenu());
    return menuBar;
  }

  /**
   * Montagem do menu de arquivos.
   * 
   * @return o menu de arquivos.
   */
  private JMenu buildOptionsMenu() {
    final JMenu menu = new JMenu(getString("options.menu"));
    menu.add(new OpenAction(this));
    menu.add(new SaveAsAction(this));
    menu.add(new CloneAction(this));
    menu.add(new ApplicationExitAction(this));
    return menu;
  }

  /**
   * Montagem do menu de arquivos.
   * 
   * @return o menu de arquivos.
   */
  private JMenu buildAboutMenu() {
    final JMenu menu = new JMenu(getString("about.menu"));
    menu.add(new ApplicationAboutAction(this));
    return menu;
  }

  /**
   * Montagem da toolbar
   * 
   * @return a toolbar
   */
  private JToolBar buildToolbar() {
    final JToolBar toolbar = new JToolBar();
    toolbar.add(new OpenAction(this));
    toolbar.add(new SaveAsAction(this));
    toolbar.setFloatable(false);
    return toolbar;
  }

  /**
   * Construtor
   * 
   * @param id identificador da aplicao.
   */
  public TextViewer(final String id) {
    super(id);
    buildFrame();
  }

}
