package csfs.client;

import java.util.Random;

import java.io.RandomAccessFile;

import org.omg.CORBA.ORB;
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;

import csfs.remote.FileServer;
import csfs.remote.FileServerHelper;
import csfs.remote.InvalidPathException;
import csfs.remote.InvalidStateException;
import csfs.remote.NotDirectoryException;
import csfs.remote.NotFileException;
import csfs.remote.OctetSequenceHolder;
import csfs.remote.RandomAccessChannel;
import csfs.remote.RemoteFile;
import csfs.remote.ServerException;

/**
 * 
 * A classe RandomAccessTest eh usada para se realizar a comparacao de
 * desempenho entre o acesso randomico via CORBA e o acesso randomico sobre um
 * arquivos servido via NFS. Os arquivos de teste estao hardcoded na classe.
 * 
 * As classes RandomAccess* (nesse pacote) foram definidas para testes simples,
 * e devem ser revistas caso passem a ser usadas com mais frequencia. Por
 * exemplo, um ponto pendente se refere ao fechamento do arquivo (close()).
 * 
 * @author Marcelo
 * 
 */
public class RandomAccessTest {

  public static final int BLOCK_SIZE = 1024 * 512;
  public static final long RANDOM_SEED = 1;
  public static final long BYTES_TO_TRANSFER = 1073741824; // 1GB

  public static void main(String[] args) throws Exception {

    byte[] buffer = new byte[BLOCK_SIZE];

    Random random = new Random(RANDOM_SEED);

    // RandomAccessWrapper raf = createWrapperCORBA(args, buffer);

    RandomAccessWrapper raf = createWrapperFile(buffer);

    long fileSize = raf.length();

    long steps = BYTES_TO_TRANSFER / BLOCK_SIZE;

    long total = 0;
    long start = System.currentTimeMillis();

    int read;
    for (int i = 0; i < steps; i++) {

      long seek = Math.abs(random.nextLong()) % fileSize;

      seek = seek - BLOCK_SIZE;
      if (seek < 0) {
        seek = 0;
      }
      raf.seek(seek);

      read = 0;
      while (read < BLOCK_SIZE) {
        read += raf.read(read, BLOCK_SIZE - read);
      }
      total = total + read;

      if (i % 100 == 0) {
        System.out.println("SEEK: " + seek + " READ: " + total + " bytes in "
          + (System.currentTimeMillis() - start) + " ms");
      }
    }

    long end = System.currentTimeMillis();
    System.out.println("FINISHED!!! BLOCK_SIZE = " + BLOCK_SIZE + " READ: "
      + total + " bytes in " + (end - start) + " ms");
  }

  private static RandomAccessWrapper createWrapperFile(byte[] buffer)
    throws FileNotFoundException {
    String pathToNFSFile = "H:\\csfs\\teste.bin";
    RandomAccessFile rafNFS = new RandomAccessFile(pathToNFSFile, "rw");
    RandomAccessWrapper raf = new RandomAccessWrapperRegularFile(rafNFS, buffer);
    return raf;
  }

  private static RandomAccessWrapper createWrapperCORBA(String[] args,
    byte[] buffer) throws FileNotFoundException, IOException,
    csfs.remote.FileNotFoundException, NotDirectoryException,
    InvalidPathException, ServerException, InvalidStateException,
    NotFileException {
    ORB orb = ORB.init(args, null);

    BufferedReader br = new BufferedReader(new FileReader(args[0]));
    org.omg.CORBA.Object obj = orb.string_to_object(br.readLine());
    br.close();
    FileServer fileServer = FileServerHelper.narrow(obj);

    RemoteFile source = fileServer.getRoot().getChild(
      new String[] { "teste.bin" });

    RandomAccessChannel rafRemote = source.getRandomAccessChannel();

    OctetSequenceHolder holder = new OctetSequenceHolder(buffer);
    RandomAccessWrapper raf = new RandomAccessWrapperRemoteFile(rafRemote,
      holder);
    return raf;
  }
}
