package csfs.datatransfer;

import java.io.File;
import java.io.IOException;

import csfs.remote.RemoteFile;
import csfs.remote.NotFileException;
import csfs.remote.InvalidStateException;
import csfs.remote.ServerException;
import csfs.remote.FileServer;

/**
 * A interface <class>DataTransferMethod</class> define os metodos que
 * representam o envio de um arquivo local para um destino remoto.
 * 
 * @version 1.0 27 Jun 2005
 * @author Marcelo Nery dos Santos
 */
public interface DataTransferMethod {

  /**
   * Copiar arquivo.
   * 
   * @param localFile File O arquivo local que servir de fonte de dados
   * @param destination RemoteFile O arquivo destino que receber os dados
   * @param method O metodo a ser utilizado
   * @return boolean uma indicao de sucesso
   * @throws NotFileException Caso o arquivo destino seja um diretrio
   * @throws InvalidStateException Caso o arquivo destino seja invlido
   */
  public boolean copy(File localFile, RemoteFile destination, String method)
    throws NotFileException, InvalidStateException, IOException,
    ServerException;

  /**
   * Obtem o identificador do metodo de transferencia.
   * 
   * @return
   */
  public String getName();

  /**
   * Obtem detalhes sobre o metodo de transferencia.
   * 
   * @return
   */
  public String getDetailedInfo();

}
