package csfs.datatransfer.blockingnio;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.InetSocketAddress;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.nio.channels.SocketChannel;

public class BlockingNIOSenderThread extends Thread {

  private InetSocketAddress targetAddress;

  private File localFile;
  private long length;

  private String targetFilename;

  public BlockingNIOSenderThread(InetSocketAddress targetAddress,
    File localFile, long length, String targetFilename) {

    this.targetAddress = targetAddress;
    this.localFile = localFile;
    this.length = length;
    this.targetFilename = targetFilename;
  }

  public void run() {

    try {
      /** Abre um canal NIO sobre o endereo do servidor de destino */
      SocketChannel socketChannel = SocketChannel.open(targetAddress);
      /** Abre o arquivo local */
      FileInputStream file = new FileInputStream(localFile);
      /** Recupera o canal NIO de leitura do arquivo */
      FileChannel fileChannel = file.getChannel();
      /** Inicializa o protocolo inicial para envio dos dados */
      String initProtocol = "" + length + "\n" + targetFilename + "\n";

      int headerSize = initProtocol.length();
      String headerSizeStr = headerSize + "\n";
      ByteBuffer headerLength = ByteBuffer
        .allocate(BlockingNIOReceiverThread.HEADER_SIZE);
      headerLength.put(headerSizeStr.getBytes());
      headerLength.rewind();
      socketChannel.write(headerLength);

      /** Aloca um buffer para envio de dados ao servidor */
      ByteBuffer out = ByteBuffer.allocate(initProtocol.length());
      /** Escreve as informacoes do protocolo inical no buffer */
      out.put(initProtocol.getBytes());
      /** Rebobina o buffer */
      out.rewind();
      /** Envia os dados do protocolo para o servidor */
      socketChannel.write(out);
      /** Solicita o envio do arquivo atravs de uma funo de alto desempenho */
      fileChannel.transferTo(0, length, socketChannel);
      /** Fecha o canal de leitura do arquivo local */
      fileChannel.close();
      /** Aloca o buffer de recepcao da confirmao do servidor */
      ByteBuffer result = ByteBuffer.allocate(1);
      /** Bloqueia ate o recebimento da confirmao */
      socketChannel.read(result);
      // TODO verificar o sucesso da operacao.
      /** Fecha o socket utilizado para o envio */
      socketChannel.close();
    }
    catch (IOException exc) {
      exc.printStackTrace();
    }
  }
}
