package csfs.datatransfer.blockingnio;

import java.io.IOException;
import java.net.BindException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.nio.channels.ServerSocketChannel;
import java.util.logging.Logger;

public class BlockingNIOServer extends Thread {

  /** O diretrio raiz para onde os arquivos devem ser criados */
  private String rootDir;

  /** A porta que esse servidor deve ouvir */
  private int port;

  /** O tamanho do buffer TCP */
  private int tcpBuffer;

  /**
   * Contrutor.
   * 
   * @param rootDir String O diretrio raiz para onde os arquivos devem ser
   *        criados
   * @param port int A porta que esse servidor deve ouvir
   */
  public BlockingNIOServer(String rootDir, int port, int tcpBuffer) {
    this.rootDir = rootDir;
    this.port = port;
    this.tcpBuffer = tcpBuffer;
  }

  /**
   * Mtodo principal
   */
  public void run() {
    try {
      /** Define um servidor de sockets capaz de gerenciar sockets NIO */
      ServerSocket server = ServerSocketChannel.open().socket();

      if (tcpBuffer != 0) {
        System.out.println("TCP_BUFFER=" + tcpBuffer);
        server.setReceiveBufferSize(tcpBuffer);
        System.out
          .println("ReceiveBufferSize=" + server.getReceiveBufferSize());
      }

      /** Recupera o nome do servidor atual */
      String localhost = InetAddress.getLocalHost().getHostName();

      /** Associa o servidor e a porta ao servidor de sockets */
      server.bind(new InetSocketAddress(localhost, port));

      Logger.global.info("NIOCopyServer BOUND TO:" + port);
      /** Aguarda solicitaes na porta indefinidamente */
      while (true) {
        /** Para cada solicitao */
        Socket socket = server.accept();

        if (tcpBuffer != 0) {
          socket.setSendBufferSize(tcpBuffer);
          System.out.println("SendBufferSize=" + socket.getSendBufferSize());

        }
        /** Inicia uma thread para trat-la */
        Thread t = new BlockingNIOReceiverThread(socket, rootDir);
        t.setPriority(this.getPriority() - 1);
        t.start();

      }
    }
    catch (BindException exc) {
      Logger.global
        .severe("NIOCopyServer DID NOT start at this process!!! SOMETHING WAS ALREADY BOUND!!!");
    }
    catch (IOException exc) {
      Logger.global.severe("NIOCopyServer SHUT DOWN!");
      exc.printStackTrace();
    }
  }

}
