package csfs.datatransfer.corba;

import java.io.File;
import java.io.RandomAccessFile;
import java.util.ArrayList;

import csfs.remote.RandomAccessChannel;

public class SegmentedCORBAManager {

  private int counter;
  private ArrayList<Thread> threads;

  private Exception exception;

  public SegmentedCORBAManager() {
    threads = new ArrayList<Thread>();
  }

  public void addSegment(long offset, long size, File localFile,
    RandomAccessChannel destination, int bufferSize) {
    threads.add(new Thread(new CopySegment(this, offset, size, localFile,
      destination, bufferSize)));
  }

  public synchronized void copyFile() throws Exception {
    for (int i = 0; i < threads.size(); i++) {
      threads.get(i).start();
    }
    while (counter < threads.size()) {
      wait();
    }
    if (exception != null) {
      exception.printStackTrace();
      throw exception;
    }
  }

  public synchronized void notifyEnd(Exception exc) {
    if (exc != null) {
      this.exception = exc;
    }
    counter++;
    notify();
  }
}

class CopySegment implements Runnable {

  private long offset;
  private long size;
  private File localFile;
  private RandomAccessChannel destination;
  private int bufferSize;

  private SegmentedCORBAManager parent;

  public CopySegment(SegmentedCORBAManager parent, long offset, long size,
    File localFile, RandomAccessChannel destination, int bufferSize) {
    this.parent = parent;
    this.offset = offset;
    this.size = size;
    this.localFile = localFile;
    this.destination = destination;
    this.bufferSize = bufferSize;
  }

  public void run() {
    Exception exc = null;
    try {
      /** define o buffer de envio de dados */
      byte[] buffer = new byte[bufferSize];

      RandomAccessFile file = new RandomAccessFile(localFile, "rw");
      file.seek(offset);
      destination.seek(offset);

      long read = 0;
      int bytesRead = 0;
      while (read < size && bytesRead != -1) {
        int localRead = 0;
        do {
          bytesRead = file.read(buffer, localRead, buffer.length - localRead);
          if (bytesRead != -1) {
            localRead += bytesRead;
          }
        } while (bytesRead != -1 && localRead != buffer.length);
        read += localRead;
        destination.write(localRead, buffer);
      }
      /** fecha o arquivo local */
      file.close();
      /** fecha o canal de escrita */
      destination.close();
    }
    catch (Exception exc2) {
      exc = exc2;
      exc.printStackTrace();
    }
    parent.notifyEnd(exc);
  }

}
