package csfs.datatransfer.corba;

import java.io.File;

import csfs.datatransfer.DataTransferMethod;
import csfs.impl.util.ServerExceptionBuilder;
import csfs.impl.util.SystemProperties;
import csfs.remote.InvalidStateException;
import csfs.remote.NotFileException;
import csfs.remote.RandomAccessChannel;
import csfs.remote.RemoteFile;
import csfs.remote.ServerException;

/**
 * A classe <class>SegmentedCORBAMethod</class>  a implementao da interface
 * DataTransferMethod, para a realizao de transferencias de arquivos via
 * CORBA, utilizando varias threads e canais de acesso randomico.
 * 
 * @version 1.0 27 Jun 2005
 * @author Marcelo Nery dos Santos
 */
public class SegmentedCORBAMethod implements DataTransferMethod {

  /**
   * Copiar arquivo.
   * 
   * @param localFile File O arquivo local que servir de fonte de dados
   * @param destination RemoteFile O arquivo destino que receber os dados
   * @return boolean uma indicao de sucesso
   * @throws NotFileException Caso o arquivo destino seja um diretrio
   * @throws InvalidStateException Caso o arquivo destino seja invlido
   */
  public boolean copy(File localFile, RemoteFile destination, String method)
    throws NotFileException, InvalidStateException, ServerException {
    boolean response = false;
    try {
      /**
       * Essa verificao j foi realizada no Servant de RemoteFile, no 
       * necessrio definir o caminho do arquivo remoto para a exceo.
       */
      if (!localFile.isFile()) {
        throw new InvalidStateException(
          "It is not possible to copy a directory", new String[] {});
      }

      int index = method.lastIndexOf('-') + 1;
      int numChannels = Integer.parseInt(method.substring(index));
      String bufferStr = method.substring(method.indexOf('-') + 1, index - 1);
      int corbaBuffer = Integer.parseInt(bufferStr);

      long size = localFile.length();

      long block = size / numChannels;

      SegmentedCORBAManager manager = new SegmentedCORBAManager();

      for (int i = 0; i < numChannels; i++) {
        RandomAccessChannel channel = destination.getRandomAccessChannel();
        if (i != numChannels - 1) {
          manager.addSegment(i * block, block, localFile, channel, corbaBuffer);
        }
        else {
          manager.addSegment(i * block, size - i * block, localFile, channel,
            corbaBuffer);
        }
      }

      manager.copyFile();
      response = true;

    }
    catch (Exception ex) {
      throw ServerExceptionBuilder.createServerException(ex);
    }
    return response;
  }

  public String getName() {
    return "SegmentedCORBA";
  }

  public String getDetailedInfo() {
    return "SegmentedCORBA" + SystemProperties.getProperty("CORBA_BUFFER_SIZE");
  }
}
