package csfs.datatransfer.nonblockingnio;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.nio.channels.SocketChannel;

/**
 * Estado representando a importao do arquivo em si.
 * 
 * @author Leonardo Barros
 */
public class GettingFile extends ImportState {
  /**
   * Cria o estado.
   * 
   * @param handler referncia para o gerenciador de importao.
   */
  public GettingFile(ImportHandler handler) {
    this.remainingBytes = handler.getFileSize();
    this.handler = handler;
  }

  public void read() throws IOException {
    long size = handler.getFileSize();
    SocketChannel sourceChannel = handler.getSourceChannel();
    FileChannel targetChannel = handler.getTargetChannel();
    long bytesRead = targetChannel.transferFrom(sourceChannel, size
      - remainingBytes, remainingBytes);
    decRemainingBytes(bytesRead);
  }

  protected void decRemainingBytes(long value) throws IOException {
    remainingBytes -= value;
    if (remainingBytes == 0) {
      sendAck();
      handler.close();
    }
  }

  /**
   * Transmite um "acknowledgement" para o cliente, indicando que a
   * transferncia foi concluda com sucesso.
   * 
   * @throws IOException
   */
  private void sendAck() throws IOException {
    ByteBuffer ack = ByteBuffer.allocateDirect(1);
    ack.put((byte) 10);
    ack.flip();
    SocketChannel sourceChannel = handler.getSourceChannel();
    sourceChannel.write(ack);
  }

}
