package csfs.datatransfer.nonblockingnio;

import java.io.IOException;
import java.nio.ByteBuffer;

/**
 * Estado representando a busca do tamanho do arquivo a ser importado.
 * 
 * @author Leonardo Barros
 */
public class GettingFileSize extends ImportState {
  /** Tamanho do buffer para leitura do tamanho. */
  public static final int HEADER_SIZE = 8;

  /** Buffer para leitura do tamanho. */
  private ByteBuffer buffer;

  /**
   * Cria o estado.
   * 
   * @param handler referncia para o gerenciador de importao.
   */
  public GettingFileSize(ImportHandler handler) {
    this.buffer = ByteBuffer.allocate(HEADER_SIZE);
    this.remainingBytes = HEADER_SIZE;
    this.handler = handler;
  }

  public void read() throws IOException {
    long bytesRead = handler.getSourceChannel().read(buffer);
    decRemainingBytes(bytesRead);
  }

  protected void decRemainingBytes(long value) throws IOException {
    remainingBytes -= value;
    if (remainingBytes == 0) {
      buffer.flip();
      // //#TODO LITTLE_ENDIAN
      long fileSize = buffer.getLong();
      // System.out.println("3) Tamanho do arquivo: "+fileSize+ "
      // bytes");//#LEO#
      handler.setFileSize(fileSize);
      handler.changeState(new GettingFile(handler));
    }
  }

}
