package csfs.datatransfer.nonblockingnio;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.nio.charset.Charset;
import java.nio.charset.CharsetDecoder;

/**
 * Estado representando a busca do caminho do arquivo a ser importado.
 * 
 * @author Leonardo Barros
 */
public class GettingPath extends ImportState {
  /** Codificao para transmisso de caracteres */
  private static final String ENCODING = "ISO-8859-1";

  /** Buffer para leitura do caminho. */
  private ByteBuffer buffer;

  /**
   * Cria o estado.
   * 
   * @param handler referncia para o gerenciador de importao.
   * @param size tamanho do caminho do arquivo.
   */
  public GettingPath(ImportHandler handler, int size) {
    this.buffer = ByteBuffer.allocate(size);
    this.remainingBytes = size;
    this.handler = handler;
  }

  public void read() throws IOException {
    long bytesRead = handler.getSourceChannel().read(buffer);
    decRemainingBytes(bytesRead);
  }

  protected void decRemainingBytes(long value) throws IOException {
    remainingBytes -= value;
    if (remainingBytes == 0) {
      buffer.flip();
      // TODO tem outra forma de converter?
      Charset charset = Charset.forName(ENCODING);
      CharsetDecoder decoder = charset.newDecoder();
      CharBuffer charBuffer = decoder.decode(buffer);
      String fileName = charBuffer.toString();
      // System.out.println("2) Caminho do arquivo: "+fileName);//#LEO#
      handler.setFilePath(fileName);
      handler.changeState(new GettingFileSize(handler));
    }
  }

}
