package csfs.datatransfer.nonblockingnio;

import java.io.IOException;
import java.nio.ByteBuffer;

/**
 * Estado representando a busca do tamanho do caminho do arquivo a ser
 * importado.
 * 
 * @author Leonardo Barros
 */
public class GettingPathSize extends ImportState {
  /** Tamanho do buffer para leitura do tamanho. */
  public static final int HEADER_SIZE = 4;

  /** Buffer para leitura do tamanho. */
  private ByteBuffer buffer;

  /**
   * Cria o estado.
   * 
   * @param handler referncia para o gerenciador de importao.
   */
  public GettingPathSize(ImportHandler handler) {
    this.buffer = ByteBuffer.allocate(HEADER_SIZE);
    this.remainingBytes = HEADER_SIZE;
    this.handler = handler;
  }

  public void read() throws IOException {
    int bytesRead = handler.getSourceChannel().read(buffer);
    decRemainingBytes(bytesRead);
  }

  protected void decRemainingBytes(long value) throws IOException {
    remainingBytes -= value;
    if (remainingBytes == 0) {
      buffer.flip();
      // #TODO LITTLE_ENDIAN
      int pathSize = buffer.getInt();
      // System.out.println("1) Tamanho do caminho: "+pathSize+ "
      // bytes");//#LEO#
      handler.changeState(new GettingPath(handler, pathSize));
    }
  }

}
