package csfs.datatransfer.nonblockingnio;

import java.io.File;
import java.io.IOException;

import csfs.datatransfer.DataTransferMethod;
import csfs.impl.util.Util;
import csfs.remote.InvalidStateException;
import csfs.remote.NotFileException;
import csfs.remote.RemoteFile;
import csfs.remote.ServerException;

/**
 * Implementacao do DataTransferMethod baseado em NIO nao bloqueante.
 */
public class NonBlockingNIOMethod implements DataTransferMethod {

  public boolean copy(File localFile, RemoteFile destination, String method)
    throws NotFileException, InvalidStateException, IOException,
    ServerException {
    String nioAddress = destination.getFileServer()
      .getCopyServerAddress(method);
    int index = nioAddress.lastIndexOf(':');
    String host = nioAddress.substring(0, index);
    int port = Integer.parseInt(nioAddress.substring(index + 1));
    String targetFilename = Util.stringArrayToString(destination.getFullName());
    NonBlockingNIOClient client = new NonBlockingNIOClient();
    return client.sendFile(host, port, localFile.getAbsolutePath(),
      targetFilename);
  }

  public String getName() {
    return "BLOCKING_NIO";
  }

  public String getDetailedInfo() {
    return "BLOCKING_NIO METHOD";
  }

}
