package csfs.datatransfer.streamblockingnio;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.InetSocketAddress;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.nio.channels.SocketChannel;
import java.util.logging.Level;
import java.util.logging.Logger;

public class StreamBlockingNIOSenderThread extends Thread {

  private InetSocketAddress targetAddress;

  private File localFile;
  private long offset;
  private long length;

  private String targetFilename;

  private StreamBlockingNIOMethod parent;

  private long copyID;

  private boolean response;

  public StreamBlockingNIOSenderThread(InetSocketAddress targetAddress,
    File localFile, long offset, long length, String targetFilename,
    long copyID, StreamBlockingNIOMethod parent) {

    this.targetAddress = targetAddress;
    this.localFile = localFile;
    this.offset = offset;
    this.length = length;
    this.targetFilename = targetFilename;
    this.copyID = copyID;
    this.parent = parent;

  }

  public void run() {

    try {

      SocketChannel socketChannel = null;
      /** Abre um canal NIO sobre o endereo do servidor de destino */
      while (socketChannel == null || !socketChannel.isConnected()) {
        try {
          socketChannel = SocketChannel.open();
          socketChannel.socket().setSoTimeout(14400000); // 4 hours!!!
          socketChannel.socket().connect(targetAddress, 14400000); // 4
                                                                    // hours!!!
        }
        catch (IOException e) {
          Logger.global
            .log(
              Level.SEVERE,
              "IOException em StreamBlockingNIOSenderThread:run():connect()! Ignoring!!!",
              e);
          try {
            Thread.sleep(10000);
          }
          catch (InterruptedException e1) {
            e1.printStackTrace();
          }
        }
      }

      /** Abre o arquivo local */
      FileInputStream file = new FileInputStream(localFile);
      /** Recupera o canal NIO de leitura do arquivo */
      FileChannel fileChannel = file.getChannel();
      /** Inicializa o protocolo inicial para envio dos dados */
      String initProtocol = "" + offset + "\n" + length + "\n" + "1" + "\n"
        + targetFilename + "\n";

      int headerSize = initProtocol.length();
      String headerSizeStr = headerSize + "\n";
      ByteBuffer headerLength = ByteBuffer
        .allocate(StreamBlockingNIOReceiverThread.HEADER_SIZE);
      headerLength.put(headerSizeStr.getBytes());
      headerLength.rewind();
      socketChannel.write(headerLength);

      /** Aloca um buffer para envio de dados ao servidor */
      ByteBuffer out = ByteBuffer.allocate(initProtocol.length());
      /** Escreve as informacoes do protocolo inical no buffer */
      out.put(initProtocol.getBytes());
      /** Rebobina o buffer */
      out.rewind();
      /** Envia os dados do protocolo para o servidor */
      socketChannel.write(out);
      /** Solicita o envio do arquivo atravs de uma funo de alto desempenho */
      fileChannel.transferTo(offset, length, socketChannel);
      /** Fecha o canal de leitura do arquivo local */
      fileChannel.close();
      /** Aloca o buffer de recepcao da confirmao do servidor */
      ByteBuffer result = ByteBuffer.allocate(1);
      /** Bloqueia ate o recebimento da confirmao */
      socketChannel.read(result);
      /** Verifica o status da cpia */
      if (result.get(0) == 0) {
        response = true;
      }
      /** Fecha o socket utilizado para o envio */
      socketChannel.close();

      parent.notifyThreadEnd(this);
    }
    catch (IOException exc) {
      exc.printStackTrace();
    }

  }

  public long getCopyID() {
    return copyID;
  }

  public boolean getResponse() {
    return response;
  }

  public String getFilename() {
    return targetFilename;
  }

}
