package csfs.datatransfer.streamblockingnio;

import java.io.IOException;

import java.nio.channels.ServerSocketChannel;

import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.BindException;
import java.util.logging.Logger;

/**
 * A classe <class>StreamBlockingNIOServer</class> implementa uma thread que
 * espera pelas solicitaes de cpia de arquivos via NIO.
 * 
 * @version 1.0 27 Jun 2005
 * @author Marcelo Nery dos Santos
 */
public class StreamBlockingNIOServer extends Thread {

  /** O diretrio raiz para onde os arquivos devem ser criados */
  private String rootDir;

  /** A porta que esse servidor deve ouvir */
  private int port;

  /**
   * Contrutor.
   * 
   * @param rootDir String O diretrio raiz para onde os arquivos devem ser
   *        criados
   * @param port int A porta que esse servidor deve ouvir
   */
  public StreamBlockingNIOServer(String rootDir, int port) {
    this.rootDir = rootDir;
    this.port = port;
  }

  /**
   * Mtodo principal
   */
  public void run() {
    try {
      /** Define um servidor de sockets capaz de gerenciar sockets NIO */
      ServerSocket server = ServerSocketChannel.open().socket();

      /** Recupera o nome do servidor atual */
      String localhost = InetAddress.getLocalHost().getHostName();

      /** Associa o servidor e a porta ao servidor de sockets */
      server.bind(new InetSocketAddress(localhost, port));

      Logger.global.info("NIOCopyServer BOUND TO:" + port);
      /** Aguarda solicitaes na porta indefinidamente */
      while (true) {
        /** Para cada solicitao */
        Socket socket = server.accept();
        /** Inicia uma thread para trat-la */
        Thread t = new StreamBlockingNIOReceiverThread(socket, rootDir);
        t.setPriority(this.getPriority() - 1);
        t.start();
      }
    }
    catch (BindException exc) {
      Logger.global
        .severe("NIOCopyServer DID NOT start at this process!!! SOMETHING WAS ALREADY BOUND!!!");
    }
    catch (IOException exc) {
      Logger.global.severe("NIOCopyServer SHUT DOWN!");
      exc.printStackTrace();
    }
  }
}
