package csfs.impl.performancetracker;

/**
 * 
 * Essa classe armazena o total de bytes transferidos para um servidor, bem como
 * o tempo gasto nas operacoes de transferencia.
 * 
 * Essa implementacao basica nao considera a copia de arquivos em paralelo,
 * levando em consideracao simplesmente o valor absoluto do numero de bytes
 * transferidos e do tempo gasto.
 * 
 * Novos modelos, considerando um peso maior para as transferencias mais
 * recentes e algo que leve em consideracao o paralelismo das operacoes de copia
 * podem gerar resultados mais valiosos na obtencao da taxa de trnasferencia
 * para o servidor.
 * 
 */
public class DataTransferInfo {

  private double totalBytesTransferred;
  private double totalTime;

  public DataTransferInfo() {
  }

  public void update(long bytes, long time) {
    totalBytesTransferred = totalBytesTransferred + bytes;
    totalTime = totalTime + time;
  }

  public double getAverageTransferRate() {
    return totalBytesTransferred / (totalTime / 1000);
  }

  public double getTotalBytesTransferred() {
    return totalBytesTransferred;
  }

  public double getTotalTime() {
    return totalTime;
  }

}
