package csfs.impl.performancetracker;

import java.util.HashMap;

/**
 * Essa classe recebe as informacoes do mecanismo de copia do CSFS e armazena os
 * dados localmente (apenas em memoria). Eventualmente pode ser necessario
 * persistir os dados em disco.
 */
public class DataTransferPerformanceTracker {

  private static DataTransferPerformanceTracker instance;

  private HashMap history;

  private DataTransferPerformanceTracker() {
    resetTransferRateData();
  }

  public synchronized static DataTransferPerformanceTracker getInstance() {
    if (instance == null) {
      instance = new DataTransferPerformanceTracker();
    }
    return instance;
  }

  public synchronized void update(String destination, long bytes, long time) {
    System.out.println("destination=" + destination + ",bytes=" + bytes
      + "time=" + time);
    DataTransferInfo info = (DataTransferInfo) history.get(destination);
    if (info == null) {
      info = new DataTransferInfo();
      history.put(destination, info);
    }
    info.update(bytes, time);
  }

  public synchronized double getAverageTransferRateToSpecificHost(
    String destination) throws UnavailableInformationException {
    DataTransferInfo info = (DataTransferInfo) history.get(destination);
    if (info == null) {
      throw new UnavailableInformationException("No data available for host: "
        + destination);
    }
    return info.getAverageTransferRate();
  }

  public double getAverageTransferRate() throws UnavailableInformationException {
    if (history.size() == 0) {
      throw new UnavailableInformationException("No data available!");
    }
    double totalBytes = 0D;
    double totalTime = 0D;
    DataTransferInfo[] infos = (DataTransferInfo[]) history.values().toArray(
      new DataTransferInfo[0]);
    for (int i = 0; i < infos.length; i++) {
      totalBytes = infos[i].getTotalBytesTransferred();
      totalTime = infos[i].getTotalTime();
    }
    return (totalBytes / (totalTime / 1000));
  }

  public void resetTransferRateData() {
    System.out.println("Reset called!");
    this.history = new HashMap();
  }
}
