import java.io.BufferedReader;
import java.io.FileReader;
import java.util.StringTokenizer;
import java.util.Hashtable;
import org.jfree.data.xy.XYSeries;
import org.jfree.chart.ChartFactory;
import java.io.File;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.ChartUtilities;
import java.util.*;
import org.jfree.data.xy.XYSeriesCollection;
import org.jfree.chart.axis.LogarithmicAxis;
import org.jfree.chart.axis.ValueAxis;

public class LogAnalyser {

  private static Hashtable charts = new Hashtable();

  private static double maxTransferRate;

  public static void main(String[] args) throws Exception {
    if (args == null || args.length == 0) {
      args = new String[] { ".." + File.separator + "remotefilesystem"
        + File.separator + "log_performance.txt" };
    }

    BufferedReader in = new BufferedReader(new FileReader(args[0]));
    String line;

    while ((line = in.readLine()) != null) {
      if (line.startsWith("INFO: \t")) {
        StringTokenizer tokens = new StringTokenizer(line, "\t");
        tokens.nextToken(); // "INFO: \t"
        String opName = tokens.nextToken(); // "FILE TRANSFER\t"
        String fileName = tokens.nextToken(); // "random_8MB\t"
        String id = tokens.nextToken(); // "ID=49\t"
        tokens.nextToken(); // "from\t"
        String location = tokens.nextToken(); // "nerynote 6 TO nerynote 11\t"
        String size = tokens.nextToken(); // "SIZE=8388608\t"
        String method = tokens.nextToken(); // "METHOD=CORBA"
        String corbaBufferSize = "BLOCK_SIZE=0";
        if ("METHOD=CORBA".equals(method)) {
          corbaBufferSize = tokens.nextToken(); // "BLOCK_SIZE=131072"
        }
        if ("METHOD=NIO".equals(method)) {
          corbaBufferSize = "BLOCK_SIZE=" + (Integer.MAX_VALUE - 1);
        }
        if ("METHOD=FTP".equals(method)) {
          corbaBufferSize = "BLOCK_SIZE=" + Integer.MAX_VALUE;
        }
        String time = tokens.nextToken(); // "TIME=1261\n"
        handleLogRecord(opName + " " + fileName, id, location, size, time,
          corbaBufferSize);
      }
    }
    generatePNGs();
  }

  private static void handleLogRecord(String opName, String id,
    String location, String size, String time, String corbaBufferSize) {
    Hashtable series = (Hashtable) charts.get(opName + " " + location + " "
      + size);
    if (series == null) {
      series = new Hashtable();
      charts.put(opName + " " + location + " " + size, series);
    }
    XYSeries serie = (XYSeries) series.get(corbaBufferSize);
    if (serie == null) {
      serie = new XYSeries(corbaBufferSize.substring(corbaBufferSize
        .lastIndexOf('=') + 1));
      series.put(corbaBufferSize, serie);
    }
    double sizeValue = Double.parseDouble(size
      .substring((size.lastIndexOf('=') + 1)));
    double timeValue = Double.parseDouble(time
      .substring((time.lastIndexOf('=') + 1)));
    double transferRate = (sizeValue / (timeValue / 1000)) / (1024 * 1024);
    serie.add(Double.parseDouble(id.substring(id.lastIndexOf('=') + 1)),
      transferRate);

    if (transferRate > maxTransferRate) {
      maxTransferRate = transferRate;
    }
  }

  private static void generatePNGs() throws Exception {
    Iterator iter = charts.keySet().iterator();
    while (iter.hasNext()) {
      Object item = (Object) iter.next();
      Hashtable series = (Hashtable) charts.get(item);
      XYSeriesCollection collection = new XYSeriesCollection();
      int[] bufferNumber = new int[series.size()];
      int index = 0;
      Iterator iter2 = series.keySet().iterator();
      while (iter2.hasNext()) {
        String item2 = (String) iter2.next();
        bufferNumber[index] = Integer.parseInt(item2.substring(item2
          .lastIndexOf('=') + 1));
        index++;
      }

      Arrays.sort(bufferNumber);
      for (int i = 0; i < bufferNumber.length; i++) {
        XYSeries serie = (XYSeries) series.get("BLOCK_SIZE=" + bufferNumber[i]);
        collection.addSeries(serie);
        if (serie.getName().equals("" + (Integer.MAX_VALUE - 1))) {
          serie.setName("NIO");
        }
        if (serie.getName().equals("" + (Integer.MAX_VALUE))) {
          serie.setName("FTP");
        }
      }

      JFreeChart chart = ChartFactory.createScatterPlot(item.toString(), // Title
        "Run #", // X-Axis label
        "Transfer Rate (MB/s)", // Y-Axis label
        collection, PlotOrientation.VERTICAL, // Dataset
        true, true, true // Show legend
        );

      // ValueAxis vAxis = new LogarithmicAxis("Running Time (log ms)");
      // ( (XYPlot) chart.getPlot()).setRangeAxis(vAxis);
      ((XYPlot) chart.getPlot()).getRangeAxis().setLowerBound(0D);
      ((XYPlot) chart.getPlot()).getRangeAxis().setUpperBound(
        Math.round(maxTransferRate + 0.5D));

      ChartUtilities.saveChartAsPNG(new File(item + ".png"), chart, 800, 600);

    }

  }
}
