/*
 * $Id: BasicDesktopFrame.java 91606 2009-05-18 14:18:43Z clinio $
 */

package csgrid.client.desktop;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.text.MessageFormat;
import java.util.HashMap;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;

import javax.swing.AbstractAction;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextField;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;

import tecgraf.javautils.LNG;
import tecgraf.javautils.gui.GUIUtils;
import csbase.client.Client;
import csbase.client.applicationmanager.ApplicationManager;
import csbase.client.applicationmanager.ApplicationManagerMenuUtilities;
import csbase.client.applicationmanager.ApplicationPanel;
import csbase.client.applications.ApplicationImages;
import csbase.client.desktop.AboutAction;
import csbase.client.desktop.DesktopFrame;
import csbase.client.desktop.DesktopPreferences;
import csbase.client.desktop.ProjectInfoAction;
import csbase.client.desktop.RemoteTask;
import csbase.client.desktop.TreeFilterPanelListener;
import csbase.client.ias.AdminFrame;
import csbase.client.ias.AdminModifyUserInfoDialog;
import csbase.client.ias.ModifyUserInfoDialog;
import csbase.client.ias.UserInfoDialog;
import csbase.client.kernel.ClientException;
import csbase.client.project.BasicProjectTreeFactory;
import csbase.client.project.ProjectLocationBar;
import csbase.client.project.ProjectTreeAdapter;
import csbase.client.project.ProjectTreeFactory;
import csbase.client.project.ProjectTreeSelectionEvent;
import csbase.client.project.ProjectTreeSelectionListener;
import csbase.client.project.PublishedFileRetrieverDialog;
import csbase.client.project.action.CommonProjectCreationAction;
import csbase.client.project.action.CommonProjectOpenAction;
import csbase.client.project.action.CommonProjectUpdateAction;
import csbase.client.project.action.ProjectCloseAction;
import csbase.client.project.action.ProjectDeleteAction;
import csbase.client.project.action.ProjectHistoryShowAction;
import csbase.client.project.action.ProjectTreeUpdateAction;
import csbase.client.project.action.ProjectUsersManagementAction;
import csbase.client.project.action.ShowHiddenFilesAction;
import csbase.client.project.action.ShowTreeFilterAction;
import csbase.client.remote.ClientRemoteMonitor;
import csbase.client.remote.ProjectAdminObserver;
import csbase.logic.AlgorithmAdminPermission;
import csbase.logic.ClientProjectFile;
import csbase.logic.CommonClientProject;
import csbase.logic.ProjectUserEvent;
import csbase.logic.SharedProjectEvent;
import csbase.logic.User;
import csbase.logic.Utilities;
import csbase.logic.Version;
import csbase.logic.applicationservice.ApplicationRegistry;
import csbase.remote.ClientRemoteLocator;

/**
 * A classe <code>CSGridDesktopFrame</code> implementa a interface principal do
 * sistema no lado cliente. Ela prov a funcionalidade mnima comum a qualquer
 * sistema desenvolvido com base no CSBase. A configurao deste componente
 * (especfica para cada sistema) compreende: - o ttulo do desktop e a imagem a
 * ele associada - a localizao dos arquivos de propriedades que configuram as
 * aplicaes e os grupos de aplicaes.
 * 
 * @author Tecgraf/PUC-Rio
 */
public class CSGridDesktopFrame extends DesktopFrame implements Observer {
  /**
   * Caminho do diretrio selecionado na rvore de projetos
   */
  private JTextField locationBar;
  /**
   * Boto para selecionar projeto corrente (toolbar)
   */
  private JButton openProjButton;
  /**
   * Boto para criar um novo projeto (toolbar)
   */
  private JButton createButton;

  /**
   * Submenu de seleo de projeto
   */
  private JMenu selProjMenu;

  /**
   * Item de menu para abertura de projeto de um usurio
   */
  private JMenuItem openUserProjMenuItem;

  /**
   * Item de menu para abertura de projeto de outros usurios
   */
  private JMenuItem openOthersProjMenuItem;

  /**
   * Item de Menu para Trasnferncia de Arquivos
   */
  private JMenuItem fileTransferMenuItem;

  /**
   * Item de Menu para Remoo de projeto
   */
  private JMenuItem removeProjMenuItem;

  /**
   * Painel de aplicaes.
   */
  private ApplicationPanel applicationPanel;
  /**
   * Texto que ser adicionado ao final do ttulo das janelas dependentes do
   * tipo DesktopComponentFrame.
   */
  private String appendedTitle;

  /** Id da aplicao Gerenciador de Algoritmos */
  private final static String ALGORITHMS_MANAGER_APPLICATION_ID = "algorithmsmanager";

  /**
   * Constri a interface principal do cliente.
   * 
   * @throws ClientException
   */
  public CSGridDesktopFrame() throws ClientException {
    setTitle(Client.getInstance().getSystemName());
    /* Instala a rvore do projetos no desktop */
    installProjectTree();
    /* Constri os painis do desktop, a barra de ferramentas e o menu */
    JFrame mainFrame = getDesktopFrame();
    JComponent mainPanel = composeMainPanel(createProjectPanel(),
      createDirectoryContentsPanel(), createTreeFilterPanel(),
      createApplicationPanel(), createNotificationPanel());
    JPanel firstPanel = new JPanel(new BorderLayout());
    firstPanel.add(createToolBar(), BorderLayout.NORTH);
    firstPanel.add(mainPanel, BorderLayout.CENTER);
    mainFrame.getContentPane().add(firstPanel);
    /* Cria um menu principal do desktop */
    createDesktopMenuBar();
    /* Cadastra o desktop como observador de projetos */
    ProjectAdminObserver.getInstance().addObserver(this);
    /* Salva as configuraes do projeto */
    mainFrame.addWindowListener(new WindowAdapter() {
      @Override
      public void windowClosing(WindowEvent e) {
        exitApplication();
      }
    });
  }

  /**
   * Mtodo para ajustar o ttulo da janela para ter a verso do CSGrid.
   * 
   * @param title ttulo
   */
  @Override
  public String adjustTitle(String title) {
    if (appendedTitle == null) {
      final Version v = Version.getInstance();
      final String version = v.getVersion();
      final User user = User.getLoggedUser();
      final String systemName = LNG.get("SERVER");

      String serverName = null;
      final RemoteTask<String> task = new RemoteTask<String>() {
        @Override
        public void performTask() throws Exception {
          setResult(ClientRemoteLocator.server.getServerName());
        }
      };
      // Esse dilogo no deve aparecer rapidamente pois dar uma impresso
      // ruim ao usurio no start-up do desktop. Ele somente aparecer caso
      // haja alguma demora real na conexo com o servidor.
      task.setProgressDialogDelay(10);
      task.execute(this.getDesktopFrame(), "", systemName + "...");
      if (!task.getStatus()) {
        serverName = "ERROR";
      }
      serverName = task.getResult();
      if (serverName == null) {
        serverName = "[" + systemName + "]";
      }
      final String userLogin = user.getLogin();
      appendedTitle = userLogin + " :: " + serverName + " (" + version + ")";
    }
    return title + " - " + appendedTitle;
  }

  /**
   * Cria o painel que contm a rvore de projetos.
   * 
   * @return o painel com a rvore de projetos
   * @throws ClientException caso ocorra algum problema na criao da rvore
   */
  private JComponent createProjectPanel() throws ClientException {
    JComponent scrollPane = new JScrollPane(getTree().getTree());
    projectTreePanel = new JPanel(new BorderLayout());
    projectTreePanel.add(scrollPane, BorderLayout.CENTER);
    locationBar = new ProjectLocationBar(projectTree);
    projectTreePanel.add(locationBar, BorderLayout.NORTH);
    return projectTreePanel;
  }

  /**
   * Cria o painel que contm as janelas com as aplicaes, agrupadas por
   * categoria.
   * 
   * @return o painel com as aplicaes
   * @throws ClientException caso ocorra algum problema na criao da rvore
   */
  private JComponent createApplicationPanel() throws ClientException {
    applicationPanel = new ApplicationPanel();
    applicationPanel.addProjectTreeListernerForLayoutAutoPersistency(getTree());
    applicationPanel.addMouseListenerForLayoutMenu();
    applicationPanel.addMouseListenerForApplicationsMenu();
    return applicationPanel;
  }

  /**
   * Compe o painel principal, separando os diferentes subpainis com
   * SplitPanes.
   * 
   * @param projectTreePanel painel com a rvore de projeto
   * @param dirContentsPanel painel com detalhes do diretrio selecionado
   * @param filterPanel painel de filtro da rvore de projeto
   * @param appPanel painel de aplicaes
   * @param notificationPanel painel de notificaes
   * @return o painel principal do desktop
   */
  private JComponent composeMainPanel(JComponent projectTreePanel,
    JComponent dirContentsPanel, JComponent filterPanel, JComponent appPanel,
    JComponent notificationPanel) {
    /*
     * Compe um painel com a rvore de diretrios e os detalhes do diretrio
     * corrente, com "arrumao" vertical
     */
    splitPaneBetweenTreeAndTable = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
      projectTreePanel, dirContentsPanel);
    splitPaneBetweenTreeAndTable.setOneTouchExpandable(true);
    splitPaneBetweenTreeAndTable.setResizeWeight(0.75);
    JPanel panel = new JPanel(new BorderLayout());
    panel.add(treeFilterPanel, BorderLayout.NORTH);
    panel.add(splitPaneBetweenTreeAndTable, BorderLayout.CENTER);

    /* Compe este painel com o de aplicaes, com "arrumao" horizontal */
    splitPaneBetweenTreeAndApp = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
      panel, appPanel);
    splitPaneBetweenTreeAndApp.setOneTouchExpandable(true);
    splitPaneBetweenTreeAndApp.setResizeWeight(0.0);

    /* Acrescenta o painel de notificaes, com "arrumao" vertical */
    splitPaneBetweenAppAndNotif = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
      splitPaneBetweenTreeAndApp, notificationPanel);
    splitPaneBetweenAppAndNotif.setOneTouchExpandable(true);
    splitPaneBetweenAppAndNotif.setResizeWeight(1.0);
    getDesktopFrame().setResizable(true);
    return splitPaneBetweenAppAndNotif;
  }

  /**
   * Cria a barra de ferramentas do desktop. Esse elemento  composto por botes
   * para a seleo (open) de um projeto, e para a configurao das janelas que
   * contm grupos de aplicaes. (opes: em cascata, vertical ou horizontal)
   * 
   * @return a barra de ferramentas
   */
  private JToolBar createToolBar() {
    JToolBar toolBar = new JToolBar();
    /* Abrir projeto */
    openProjButton = new JButton(new CommonProjectOpenAction(projectTree));
    openProjButton.setIcon(ApplicationImages.ICON_OPEN_16);
    openProjButton.setToolTipText(LNG.get("desktop.tooltip.open.project"));
    openProjButton.setText("");
    openProjButton.setEnabled(true);
    toolBar.add(openProjButton);
    /* Criar projeto */
    createButton = new JButton(new CommonProjectCreationAction(projectTree));
    createButton.setIcon(ApplicationImages.ICON_NEW_16);
    createButton.setToolTipText(LNG.get("desktop.tooltip.create.project"));
    createButton.setText("");
    createButton.setEnabled(true);
    toolBar.add(createButton);
    /* Habilitar/Desabilitar a exibio de filtro na rvore de projetos */
    final JToggleButton showTreeFilterButton = new JToggleButton(
      new ShowTreeFilterAction(treeFilterPanel));
    showTreeFilterButton.setIcon(ApplicationImages.ICON_FILTER_16);
    showTreeFilterButton.setText("");
    showTreeFilterButton.setToolTipText(LNG
      .get("desktop.tooltip.showTreeFilterButton"));
    treeFilterPanel.addListener(new TreeFilterPanelListener() {
      public void visibilityChanged(boolean panelVisible) {
        showTreeFilterButton.setSelected(panelVisible);
      }
    });
    toolBar.add(showTreeFilterButton);
    /* --------- */
    toolBar.addSeparator();
    /* Executar algoritmo */
    JButton execButton = toolBar.add(applicationPanel
      .getApplicationAction("executor"));
    execButton.setEnabled(false);
    execButton.setToolTipText(LNG.get("desktop.tooltip.executor"));
    enableIfProjectOpened(execButton);
    /* Editor de Fluxos */
    JButton flowButton = toolBar.add(applicationPanel
      .getApplicationAction("flowapplication"));
    flowButton.setToolTipText(LNG.get("desktop.tooltip.flowapplication"));
    /* Monitorar comandos */
    JButton cmdMonitorButton = toolBar.add(applicationPanel
      .getApplicationAction("commandsmonitor"));
    cmdMonitorButton.setToolTipText(LNG.get("desktop.tooltip.commandsmonitor"));
    /* Gerenciar Algoritmos */
    User user = User.getLoggedUser();
    AlgorithmAdminPermission algoPermission = user
      .getPermission(AlgorithmAdminPermission.class);
    if (user.isAdmin() || (algoPermission != null)) {
      JButton algoManagerButton = toolBar.add(applicationPanel
        .getApplicationAction(ALGORITHMS_MANAGER_APPLICATION_ID));
      algoManagerButton.setToolTipText(LNG.get("desktop.tooltip.algorithmsmanager"));
    }
    /* --------- */
    toolBar.addSeparator();
    /* Monitorar servidores */
    JButton srvMonitorButton = toolBar.add(applicationPanel
      .getApplicationAction("srvmonitor"));
    srvMonitorButton.setToolTipText(LNG.get("desktop.tooltip.srvmonitor"));
    /* Monitorar grupo de servidores */
    JButton srvGrpMonitorButton = toolBar.add(applicationPanel
      .getApplicationAction("grpmonitor"));
    srvGrpMonitorButton.setToolTipText(LNG.get("desktop.tooltip.grpmonitor"));
    /* --------- */
    toolBar.addSeparator();
    /* Cascatear janelas */
    JButton cascadeButton = new JButton();
    cascadeButton.setIcon(ApplicationImages.ICON_WINCASCADE_16);
    cascadeButton.setToolTipText(LNG.get("desktop.tooltip.cascade"));
    cascadeButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        applicationPanel.cascadeGroups();
      }
    });
    toolBar.add(cascadeButton);

    /* Janelas na vertical */
    JButton tileVertButton = new JButton();
    tileVertButton.setIcon(ApplicationImages.ICON_WINTILEVERTICAL_16);
    tileVertButton.setToolTipText(LNG.get("desktop.tooltip.tile.vertical"));
    tileVertButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        applicationPanel.tileGroupsVertical();
      }
    });
    toolBar.add(tileVertButton);
    /* Janelas na horizontal */
    JButton tileHorButton = new JButton();
    tileHorButton.setIcon(ApplicationImages.ICON_WINTILEHORIZONTAL_16);
    tileHorButton.setToolTipText(LNG.get("desktop.tooltip.tile.horizontal"));
    tileHorButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        applicationPanel.tileGroupsHorizontal();
      }
    });
    toolBar.add(tileHorButton);
    /* --------- */
    toolBar.addSeparator();
    /* Sobre o sistema */
    JButton aboutButton = new JButton(new AboutAction(this.getDesktopFrame()));
    toolBar.add(aboutButton);
    return toolBar;
  }

  /**
   * Constri a barra de menu do desktop.
   * 
   * @throws ClientException Caso ocorra falha na criao do menu
   */
  private void createDesktopMenuBar() throws ClientException {
    JMenuBar menuBar = new JMenuBar();
    Map<Integer, JMenu> menus = new HashMap<Integer, JMenu>();
    int order = 1;
    /* Menu de Projeto */
    boolean projectMenuVisible = this.getConfiguration()
      .getOptionalBooleanProperty("project.menu.visible", true);
    order = this.getConfiguration().getOptionalIntegerProperty(
      "project.menu.order", order);
    if (projectMenuVisible)
      menus.put(order++, createProjectMenu());
    /* Menu de Algoritmos */
    boolean algorithmMenuVisible = this.getConfiguration()
      .getOptionalBooleanProperty("algorithm.menu.visible", true);
    order = this.getConfiguration().getOptionalIntegerProperty(
      "algorithm.menu.order", order);
    if (algorithmMenuVisible)
      menus.put(order++, createAlgorithmMenu());
    /* Menu de gerenciamento dos SGAs */
    boolean sgaMenuVisible = this.getConfiguration()
      .getOptionalBooleanProperty("sga.menu.visible", true);
    order = this.getConfiguration().getOptionalIntegerProperty(
      "sga.menu.order", order);
    if (sgaMenuVisible)
      menus.put(order++, createSGAMenu());
    /* Menu de Aplicaes */
    boolean applicationMenuVisible = this.getConfiguration()
      .getOptionalBooleanProperty("application.menu.visible", true);
    order = this.getConfiguration().getOptionalIntegerProperty(
      "application.menu.order", order);
    if (applicationMenuVisible)
      menus.put(order++, ApplicationManagerMenuUtilities.buildMenu(LNG
        .get("desktop.applications")));
    /* Menu de Configurao */
    boolean configurationMenuVisible = this.getConfiguration()
      .getOptionalBooleanProperty("config.menu.visible", true);
    order = this.getConfiguration().getOptionalIntegerProperty(
      "config.menu.order", order);
    if (configurationMenuVisible)
      menus.put(order++, createConfigurationMenu());
    /* Menu de Ajuda */
    boolean helpMenuVisible = this.getConfiguration()
      .getOptionalBooleanProperty("help.menu.visible", true);
    order = this.getConfiguration().getOptionalIntegerProperty(
      "help.menu.order", order);
    if (helpMenuVisible)
      menus.put(order++, createHelpMenu());
    /* Monta a barra de menu de acordo com a configurao definida */
    for (JMenu menu : menus.values()) {
      menuBar.add(menu);
    }
    setMenuBar(menuBar);
  }

  /**
   * Constri o menu para gerenciamente dos projetos do usurio.
   * 
   * @return o menu de gerenciamente de projetos.
   */
  protected JMenu createProjectMenu() {
    JMenu menu = new JMenu(LNG.get("desktop.project"));
    boolean hasProject = userHasProject();
    /* cone vazio para alinharmos pelo checkbox do filtro */
    Icon emptyIcon = GUIUtils.createEmptyIcon(6, 6);
    /* Abertura de projeto */
    menu.add(createOpenMenuItem());
    /* Criao de projeto */
    menu.add(createProjectCreationItem());
    /* Remoo do projeto */
    menu.add(createProjectRemoveItem(hasProject)).setIcon(emptyIcon);
    /* -------- */
    menu.addSeparator();
    /* Exibio do painel de filtro para a rvore de projetos */
    menu.add(createShowTreeFilterItem());
    /* Controle da exibio de arquivos ocultos. */
    menu.add(createShowHiddenFilesMenuItem());
    /* Atualizao automtica de projeto */
    menu.add(createProjectAutoUpdateItem());
    /* -------- */
    menu.addSeparator();
    /* Ver informaes */
    menu.add(createProjectInfoItem()).setIcon(emptyIcon);
    /* Ver histrico do projeto */
    menu.add(createProjectHistoryItem()).setIcon(emptyIcon);
    /* Atualizao de informaes do projeto */
    menu.add(createProjectUpdateItem()).setIcon(emptyIcon);
    /* Obteno de Dados */
    menu.add(createDataRetrievalItem()).setIcon(emptyIcon);
    /* Atualizao da rvore do projeto */
    menu.add(createUpdateTreeItem()).setIcon(emptyIcon);
    /* Gerenciamento dos usurios de um projeto */
    menu.add(createUsersManagementMenuItem()).setIcon(emptyIcon);
    /* Fechar projeto */
    menu.add(createProjectCloseMenuItem()).setIcon(emptyIcon);
    /* -------- */
    menu.addSeparator();
    /* Sair do sistema */
    menu.add(createExitMenuItem());
    return menu;
  }

  /**
   * Constri o menu para algoritmos (execuo e monitorao)
   * 
   * @return o menu de algoritmos
   */
  protected JMenu createAlgorithmMenu() {
    JMenu menu = new JMenu(LNG.get("desktop.algorithms"));
    /* Execuo de algoritmo */
    menu.add(createExecuteMenuItem());
    /* Construtor de fluxos */
    menu.add(createAlgoFlowMenuItem());
    /* Monitorao da execuo de algoritmos */
    menu.add(createComandMonitoringMenuItem());
    /* -------- */

    User user = User.getLoggedUser();
    AlgorithmAdminPermission algoPermission = user
      .getPermission(AlgorithmAdminPermission.class);
    if (user.isAdmin() || (algoPermission != null)) {
      menu.addSeparator();
      /* Aplicao Gerenciador de Algoritmos */
      menu.add(createAlgorithmsAdministrationMenuItem());
    }
    return menu;
  }

  /**
   * Constri o menu para gerenciamento e monitarao de SGAs (mquinas para
   * execuo remota de algoritmos)
   * 
   * @return menu de gerenciamento de SGAs
   */
  private JMenu createSGAMenu() {
    JMenu menu = new JMenu(LNG.get("desktop.menu.servers"));
    /* Gerenciamento e monitorao de servidores */
    menu.add(createServersMenuItem());
    /* Gerenciamento e monitorao de grupos de servidores */
    menu.add(createGroupServersMenuItem());
    return menu;
  }

  /**
   * Constri o menu para as opes de configurao do desktop e do servidor
   * 
   * @return o menu de configurao.
   */
  protected JMenu createConfigurationMenu() {
    JMenu menu = new JMenu(LNG.get("desktop.menu.config"));
    /* Configurar usurio corrente */
    menu.add(createUserConfigItem());
    /* Configurar o painel das aplicaes */
    menu.add(applicationPanel.createGroupsConfigurationMenu());
    /* Restaurar o desktop */
    menu.add(createRestoreDesktopItem());
    /* Itens exclusivos do administrador */
    if (User.getLoggedUser().isAdmin()) {
      /* -------- */
      menu.addSeparator();
      menu.add(createAdminMenuItem());
    }
    return menu;
  }

  /**
   * Constri o menu de ajuda.
   * 
   * @return o menu de ajuda.
   */
  protected JMenu createHelpMenu() {
    JMenu menu = new JMenu(LNG.get("desktop.menu.help"));
    /* About */
    menu.add(createAboutMenuItem());
    return menu;
  }

  /**
   * Constri e apresenta os componentes da interface principal do cliente.
   */
  @Override
  public void showDesktop() {
    super.showDesktop();
    applicationPanel.tileGroupsHorizontal();
  }

  /**
   * Fecha o projeto corrente (caso exista) antes de encerrar a aplicao
   * cliente principal.
   * 
   * @param projCloseAction ao de fechamento do projeto
   */
  public void shutdownDesktop(ProjectCloseAction projCloseAction) {
    if (ClientRemoteMonitor.getInstance().isAlive()) {
      // S fecha o projeto se houver conexo com o servidor.
      projCloseAction.close();
    }
    shutdownDesktop();
  }

  /**
   * Este mtodo  chamado nas seguintes situaes: 1 - Um projeto do usurio 
   * criado 2 - Um projeto do usurio  removido 3 - O usurio  adicionado ou
   * removido de um projeto 4 - Um projeto do qual o usurio participa 
   * removido Desta forma, habilita ou desabilita os itens do menu Projeto
   * "(Abrir) Do Usurio" e "(Abrir) De outros usurios".
   * 
   * @param o .
   * @param arg .
   */
  public void update(Observable o, Object arg) {
    if (arg instanceof SharedProjectEvent) {
      SharedProjectEvent event = (SharedProjectEvent) arg;
      if (event.contains(User.getLoggedUser().getId())) {
        openOthersProjMenuItem.setEnabled(true);
      }
      else {
        openOthersProjMenuItem.setEnabled(userHasOthersProject());
      }
    }
    else if (arg instanceof ProjectUserEvent) {
      ProjectUserEvent event = (ProjectUserEvent) arg;
      boolean hasProject = userHasProject();
      switch (event.type) {
        /*
         * Projeto do usurio foi removido ou Projeto do qual o usurio
         * participava foi removido
         */
        case ProjectUserEvent.DELETE: {
          openOthersProjMenuItem.setEnabled(userHasOthersProject());
          openUserProjMenuItem.setEnabled(hasProject);
          removeProjMenuItem.setEnabled(hasProject);
          fileTransferMenuItem.setEnabled(hasProject);
          openProjButton.setEnabled(hasProject);
          break;
        }

          /* Projeto do usurio foi criado */
        case ProjectUserEvent.CREATE: {
          openUserProjMenuItem.setEnabled(true);
          removeProjMenuItem.setEnabled(true);
          fileTransferMenuItem.setEnabled(true);
          openProjButton.setEnabled(true);
          break;
        }
      }
    }
    selProjMenu.setEnabled(openUserProjMenuItem.isEnabled()
      || openOthersProjMenuItem.isEnabled());
  }

  /**
   * Finaliza o cliente.
   */
  public void exitApplication() {
    if (!ClientRemoteMonitor.getInstance().ping()) {
      /*
       * Caso no haja comunicao com o servidor, o usurio  questionado se
       * deseja continuar com o processo de finalizao da aplicao.
       */
      int answer = tecgraf.javautils.gui.StandardDialogs.showYesNoDialog(
        getDesktopFrame(), getTitle(), LNG.get("desktop.confirm_shutdown"));
      if (answer != JOptionPane.YES_OPTION) {
        return;
      }
    }
    notifPanel.setUser(null);
    shutdownDesktop(new ProjectCloseAction(projectTree));
  }

  /**
   * Instala uma rvore de projetos no desktop principal do cliente.
   */
  protected void installProjectTree() throws ClientException {
    new BasicProjectTreeFactory();
    projectTree = ProjectTreeFactory.getInstance().makeProjectTree(
      getDesktopFrame(), getTitle());
    projectTree.addMouseListener();
    /* Listener para reao  mudana de projeto e seleo na rvore */
    projectTree.addProjectTreeListener(new ProjectTreeAdapter() {
      /* Mudana de projeto: atualiza caminho selecionado */
      @Override
      public void projectChanged(CommonClientProject proj) {
        applicationPanel.setProjectFlag(proj != null);
        String location = (proj != null) ? Utilities.makePath(proj.getTree()
          .getPath()) : "";
        locationBar.setText(location);
      }
    });
    projectTree.addTreeSelectionListener(new ProjectTreeSelectionListener() {
      public void update(ProjectTreeSelectionEvent event) {
        ClientProjectFile[] selectedFiles = event.getSelectedFiles();
        if (selectedFiles.length == 1) {
          String pathText = Utilities.makePath(selectedFiles[0].getPath());
          locationBar.setText(pathText);
        }
        else {
          locationBar.setText("");
        }
      }
    });
  }

  /*--------------------------------------------------------------------------*
   * Mtodos para criao dos tens do menu Projeto                           *                             
   *--------------------------------------------------------------------------*/
  /**
   * Constri o submenu para abertura de projetos
   * 
   * @return o menu de abertura de projetos.
   */
  protected JMenuItem createOpenMenuItem() {
    JMenuItem openMenuItem = new JMenuItem(new CommonProjectOpenAction(
      projectTree));
    openMenuItem.setIcon(ApplicationImages.ICON_OPEN_16);
    openMenuItem.setText(LNG.get("desktop.menu.project.open"));
    openMenuItem.setEnabled(true);
    String accelerator = LNG.get("desktop.menu.project.open.accelerator");
    if (null != accelerator) {
      openMenuItem.setAccelerator(KeyStroke.getKeyStroke(accelerator));
    }
    return openMenuItem;
  }

  /**
   * Constri o item de menu para recuperao de dados publicados por outros
   * projetos.
   * 
   * @return o menu de recuperao de dados.
   */
  protected JMenuItem createDataRetrievalItem() {
    JMenuItem byPublishedMenuItem = new JMenuItem(LNG
      .get("desktop.menu.project.data_retrieval.published"));
    byPublishedMenuItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        new PublishedFileRetrieverDialog();
      }
    });
    byPublishedMenuItem.setEnabled(false);
    enableIfProjectOpened(byPublishedMenuItem);
    return byPublishedMenuItem;
  }

  /**
   * Constri o item de menu para criao de um novo projeto.
   * 
   * @return o tem de menu de criao de projeto
   */
  protected JMenuItem createProjectCreationItem() {
    JMenuItem createMenuItem = new JMenuItem(new CommonProjectCreationAction(
      projectTree));
    createMenuItem.setIcon(ApplicationImages.ICON_NEW_16);
    createMenuItem.setText(LNG.get("desktop.project.create"));
    createMenuItem.setEnabled(true);
    return createMenuItem;
  }

  /**
   * Constri o item de menu para remover um projeto.
   * 
   * @return o tem de menu de remoo de projeto
   */
  protected JMenuItem createProjectRemoveItem(boolean hasProject) {
    removeProjMenuItem = new JMenuItem(new ProjectDeleteAction(projectTree));
    removeProjMenuItem.setText(LNG.get("desktop.project.remove"));
    removeProjMenuItem.setEnabled(hasProject);
    return removeProjMenuItem;
  }

  /**
   * Constri o item de menu para atualizao da rvore de projetos.
   * 
   * @return o tem de menu de atualizao da rvore do projeto aberto
   */
  protected JMenuItem createUpdateTreeItem() {
    JMenuItem updateTreeMenuItem = new JMenuItem(new ProjectTreeUpdateAction(
      projectTree));
    updateTreeMenuItem.setEnabled(false);
    enableIfProjectOpened(updateTreeMenuItem);
    return updateTreeMenuItem;
  }

  /**
   * Constri o item de menu para ver o histrico do projeto.
   * 
   * @return o tem de menu de consulta ao histrico do projeto aberto
   */
  protected JMenuItem createProjectHistoryItem() {
    JMenuItem historyViewMenuItem = new JMenuItem(new ProjectHistoryShowAction(
      projectTree));
    historyViewMenuItem.setText(LNG.get("desktop.project.history"));
    historyViewMenuItem.setEnabled(false);
    enableIfProjectOpened(historyViewMenuItem);
    return historyViewMenuItem;
  }

  /**
   * Constri o item de menu para ver informaes sobre o projeto.
   * 
   * @return o tem de menu que exibe informaes do projeto aberto
   */
  protected JMenuItem createProjectInfoItem() {
    final JMenuItem infoItem = new JMenuItem(new ProjectInfoAction(this));
    infoItem.setText(LNG.get("desktop.menu.project.info"));
    String usageAccelerator = LNG.get("desktop.menu.project.info.accelerator");
    if (null != usageAccelerator) {
      infoItem.setAccelerator(KeyStroke.getKeyStroke(usageAccelerator));
    }
    infoItem.setEnabled(false);
    enableIfProjectOpened(infoItem);
    return infoItem;
  }

  /**
   * Constri o item de menu para atualizao de dados do projeto.
   * 
   * @return o tem de menu de atualizao de dados do projeto aberto
   */
  protected JMenuItem createProjectUpdateItem() {
    JMenuItem updateMenuItem = new JMenuItem(new CommonProjectUpdateAction(
      projectTree));
    updateMenuItem.setText(LNG.get("desktop.project.modify"));
    updateMenuItem.setEnabled(false);
    enableIfProjectOpened(updateMenuItem);
    return updateMenuItem;
  }

  /**
   * Constri o item de menu para gerenciamento dos usurios de projetos.
   * 
   * @return o tem de menu de controle de acesso de usurios no projeto
   */
  protected JMenuItem createUsersManagementMenuItem() {
    JMenuItem usersManagementMenuItem = new JMenuItem(
      new ProjectUsersManagementAction(projectTree));
    usersManagementMenuItem.setText(LNG.get("desktop.project.access.control"));
    usersManagementMenuItem.setEnabled(false);
    enableIfAdminProject(usersManagementMenuItem);
    return usersManagementMenuItem;
  }

  /**
   * Constri o item de menu que habilita ou desabilita a exibio de arquivos
   * ocultos (que comeam com "."). O estado do checkbox  obtido das
   * preferncias do usurio.
   */
  protected JMenuItem createShowHiddenFilesMenuItem() {
    final JCheckBoxMenuItem showHiddenFilesMenuItem = new JCheckBoxMenuItem(
      new ShowHiddenFilesAction(projectTree));
    DesktopPreferences prefs = getDesktopPreferences();
    showHiddenFilesMenuItem.setSelected(prefs == null ? false : prefs
      .shouldShowHiddenFiles());
    return showHiddenFilesMenuItem;
  }

  /**
   * Constri o item de menu para fechar o projeto.
   * 
   * @return o item de menu para fechar o projeto aberto
   */
  protected JMenuItem createProjectCloseMenuItem() {
    JMenuItem closeMenuItem = new JMenuItem(new ProjectCloseAction(projectTree));
    closeMenuItem.setEnabled(false);
    enableIfProjectOpened(closeMenuItem);
    return closeMenuItem;
  }

  /**
   * Constri o item de menu para sair do sistema.
   * 
   * @return o item de menu para sair do sistema
   */
  protected JMenuItem createExitMenuItem() {
    JMenuItem exitMenuItem = new JMenuItem(LNG.get("desktop.exit"));
    exitMenuItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        exitApplication();
      }
    });
    return exitMenuItem;
  }

  /*--------------------------------------------------------------------------*
   * Mtodos para criao dos tens do menu Algoritmos                        *                             
   *--------------------------------------------------------------------------*/
  /**
   * Constri o item de menu para execuo de algoritmos.
   * 
   * @return o item de menu para executar um algoritmo remotamente.
   */
  protected JMenuItem createExecuteMenuItem() {
    JMenuItem execMenuItem = new JMenuItem(LNG
      .get("desktop.algorithms.execute"));
    execMenuItem.addActionListener(applicationPanel
      .getApplicationAction("executor"));
    execMenuItem.setEnabled(false);
    execMenuItem.setIcon(ApplicationImages.ICON_EXECUTE_CMD_16);
    enableIfProjectOpened(execMenuItem);
    return execMenuItem;
  }

  /**
   * Constri o item de menu para monitorao da execuo de algoritmos.
   * 
   * @return o item de menu para monitorar a execuo de algoritmos.
   */
  protected JMenuItem createComandMonitoringMenuItem() {
    JMenuItem cmdsMenuItem = new JMenuItem(applicationPanel
      .getApplicationAction("commandsmonitor"));
    cmdsMenuItem.setText(LNG.get("desktop.menu.algoritm.monitor"));
    return cmdsMenuItem;
  }

  /**
   * Constri o item de menu para a aplicao de construtor de fluxos.
   * 
   * @return o item de menu para construir fluxo de algoritmos.
   */
  protected JMenuItem createAlgoFlowMenuItem() {
    JMenuItem cmdsMenuItem = new JMenuItem(applicationPanel
      .getApplicationAction("flowapplication"));
    cmdsMenuItem.setText(LNG.get("desktop.menu.algorithm.flow"));
    return cmdsMenuItem;
  }

  /**
   * Constri o item de menu para gerenciamento do repositrio de algoritmos.
   * Essa opo est disponvel apenas para os administradores ou para usurios
   * com permisso de administrao do repositrio de algoritmos.
   * 
   * @return o item de menu para gerenciamento dos algoritmos no repositrio
   */
  protected JMenuItem createAlgorithmsAdministrationMenuItem() {
    JMenuItem algorithmMenuItem = new JMenuItem(applicationPanel
      .getApplicationAction(ALGORITHMS_MANAGER_APPLICATION_ID));
    algorithmMenuItem.setText(LNG.get("desktop.admin.algorithms"));
    return algorithmMenuItem;
  }

  /*--------------------------------------------------------------------------*
   * Mtodos para criao dos tens do menu Servidores (SGAs)                 *                             
   *--------------------------------------------------------------------------*/
  /**
   * Constri o item de menu para gerenciamento e monitorao dos servidores
   * SGAs.
   * 
   * @return o item de menu para gerenciar e monitorar os servidores.
   */
  protected JMenuItem createServersMenuItem() {
    JMenuItem serversMenuItem = new JMenuItem(applicationPanel
      .getApplicationAction("srvmonitor"));
    serversMenuItem.setText(LNG.get("desktop.monitoring.servers"));
    return serversMenuItem;
  }

  /**
   * Constri o item de menu para gerenciamento e monitorao de grupos de
   * servidores SGAs.
   * 
   * @return o item de menu para gerenciar e monitorar os grupos de servidores.
   */
  protected JMenuItem createGroupServersMenuItem() {
    JMenuItem groupsMenuItem = new JMenuItem(applicationPanel
      .getApplicationAction("grpmonitor"));
    groupsMenuItem.setText(LNG.get("desktop.monitoring.groups"));
    return groupsMenuItem;
  }

  /*--------------------------------------------------------------------------*
   * Mtodos para criao dos tens do menu de Configurao                   *                             
   *--------------------------------------------------------------------------*/

  /**
   * Constri o item de menu para alterar a configurao do usurio corrente. No
   * caso do usurio ser administrador, o dilogo permite que ele altere
   * permisses alm da prpria senha.
   * 
   * @return o item de menu para configurar o usurio corrente.
   */
  protected JMenuItem createUserConfigItem() {
    JMenuItem configUserMenuItem = new JMenuItem(LNG
      .get("desktop.menu.config.user"));
    configUserMenuItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        JFrame frame = getInstance().getDesktopFrame();
        User loggedUser = User.getLoggedUser();
        UserInfoDialog dialog;
        if (loggedUser.isAdmin()) {
          dialog = new AdminModifyUserInfoDialog(frame, loggedUser.getId());
        }
        else {
          dialog = new ModifyUserInfoDialog(frame, loggedUser.getId());
        }
        dialog.setVisible(true);
      }
    });
    return configUserMenuItem;
  }

  /**
   * Constri o menu com as opes de configurao das janelas com as aplicaes
   * do desktop.
   * 
   * @return o menu para configurar a exibio das aplicaes no desktop.
   */
  protected JMenu createGroupsConfigurationMenu() {
    return applicationPanel.createGroupsConfigurationMenu();
  }

  /**
   * Constri o item de menu de administrao do sistema.
   * 
   * @return o item de menu para administrar o sistema.
   */
  protected JMenuItem createAdminMenuItem() {
    JMenuItem adminMenuItem = new JMenuItem(MessageFormat.format(LNG
      .get("desktop.menu.system.admin"), Client.getInstance().getSystemName()));
    adminMenuItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if (adminFrame == null) {
          adminFrame = new AdminFrame();
        }
        adminFrame.showUserManagement();
      }
    });
    return adminMenuItem;
  }

  /*--------------------------------------------------------------------------*
   * Mtodos para criao dos tens do menu de Ajuda                          *                             
   *--------------------------------------------------------------------------*/
  /**
   * Constri o item de menu para exibir o dilogo de "About".
   * 
   * @return o item de menu para exibir o dilogo com as informaes sobre o
   *         sistema
   */
  protected JMenuItem createAboutMenuItem() {
    JMenuItem aboutMenuItem = new JMenuItem(new AboutAction(this
      .getDesktopFrame()));
    aboutMenuItem.setText(MessageFormat.format(LNG.get("desktop.menu.about"),
      Client.getInstance().getSystemName()));
    return aboutMenuItem;
  }

}
