-- $Id: csmake.lua 37080 2006-08-02 22:30:41Z rodrigoh $

-- includes
require "global"
require "oo"
require "getopt"
require "log"

-- sanity check
abort(type(posix) == "table",
    "este script necessita de uma versao de Lua\ncom suporte a funcoes POSIX")

-- variveis recebidas do wrapper bash
-- -----------------------------------
-- SCRIPT_PATH  : dir aonde este script est instalado

-- diretrio a partir de onde foi executado o script
cs_exedir = posix.getcwd()

-- inicializa loggers exportados para todos os mdulos
-- logger de terminal
cs_log = Log:new()
cs_log:setLevel(Log.level.INFO)

-- logger de arquivo
-- flog = FileLogger:new()
-- flog:init(cs_exedir .. "/csmake.log", "w")
-- flog:setLevel(Log.level.INFO)

-- VARIVEIS GLOBAIS

-- funcionalidades disponibilizadas pelos mdulos
MODULE = {
    ALGORITHM     = "alg",
    AUTOUPDATE    = "autoupdate",
    BRANCH        = "branch",
    BUILD         = "build",
    CHECKOUT      = "co",
    CLEAN         = "clean",
    PACK          = "pack",
    DOCS          = "doc",
    WEBDEPLOY     = "webdeploy",
    PATCH         = "patch",
    PROPERTIES    = "props",
    REBUILD       = "rebuild",
    RUN           = "run",
    SERVERLIST    = "servers",
    SINGLEFILE    = "class",
    STATUS        = "status",
    TAG           = "tag",
    TEST          = "test",
    UPDATE        = "update",
}

-- FIXME os conceitos de HARD e SOFT esto trocados!! =/
-- tipo de dependncia entre os mdulos: HARD significa que as
-- dependncias sero executadas caso no sejam satisfeitas;
-- dependncias SOFT causaro fim da operao sem serem executadas
DEP = {
    HARD = 1,
    SOFT = 2,
}

-- estes mdulos so dependncias para outros, mas no podem ser
-- executados diretamente a partir da linha de comando
NONCMD_MODULE = {
    MAIN = "main",
    CFGFILE = "cfgfile",
}

-- VARIVEIS LOCAIS

-- booleano que indica se erros de requisitos devem ser ignorados ou no
local _ignoreReqs

-- aes suportadas (construda como espelho de MODULE)
local _actions = {}
for k, v in pairs(MODULE) do
    _actions[v] = k
end

-- aes disponveis. Cada mdulo registra aqui as possveis aes.
-- Aes so indexadas pela opo associada  sua execuo (p.ex. "i"
-- para "checkout")
local _moduleData = {
    [MODULE.ALGORITHM] = {
        depends = {
            { module = MODULE.CHECKOUT, type = DEP.HARD, },
        },
    },
    [MODULE.AUTOUPDATE] = {
        depends = {
        },
    },
    [MODULE.BUILD] = {
        depends = {
            { module = MODULE.CHECKOUT, type = DEP.HARD, },
        },
    },
    [MODULE.BRANCH] = {
        depends = {
            { module = MODULE.CHECKOUT, type = DEP.HARD, },
        },
    },
    [MODULE.CHECKOUT] = {
        depends = {
            { module = NONCMD_MODULE.CFGFILE, type = DEP.HARD, },
        },
    },
    [MODULE.CLEAN] = {
        depends = {
            { module = MODULE.CHECKOUT, type = DEP.HARD, },
        },
    },
    [MODULE.PACK] = {
        depends = {
            { module = MODULE.CHECKOUT, type = DEP.HARD, },
        },
    },
    [MODULE.DOCS] = {
        depends = {
            { module = MODULE.CHECKOUT, type = DEP.HARD, },
        },
    },
    [MODULE.WEBDEPLOY] = {
        depends = {
            { module = MODULE.CHECKOUT, type = DEP.HARD, },
        },
    },
    [MODULE.PATCH] = {
        depends = {
            { module = MODULE.CHECKOUT, type = DEP.HARD, },
        },
    },
    [MODULE.PROPERTIES] = {
        depends = {
            { module = MODULE.CHECKOUT, type = DEP.HARD, },
        },
    },
    [MODULE.REBUILD] = {
        depends = {
            { module = MODULE.CHECKOUT, type = DEP.HARD, },
        },
    },
    [MODULE.RUN] = {
        depends = {
            { module = MODULE.CHECKOUT, type = DEP.HARD, },
        },
    },
    [MODULE.SERVERLIST] = {
        depends = {},
    },
    [MODULE.SINGLEFILE] = {
        depends = {
            { module = MODULE.CHECKOUT, type = DEP.HARD, },
        },
    },
    [MODULE.STATUS] = {
        depends = {
            { module = MODULE.CHECKOUT, type = DEP.HARD, },
        },
    },
    [MODULE.TAG] = {
        depends = {
            { module = MODULE.CHECKOUT, type = DEP.HARD, },
        },
    },
    [MODULE.TEST] = {
        depends = {
            { module = MODULE.CHECKOUT, type = DEP.HARD, },
        },
    },
    [MODULE.UPDATE] = {
        depends = {
            { module = MODULE.CHECKOUT, type = DEP.HARD, },
        },
    },
    [NONCMD_MODULE.MAIN] = {
        depends = {
            { module = NONCMD_MODULE.CFGFILE, type = DEP.HARD, },
        },
    },
    [NONCMD_MODULE.CFGFILE] = {
        depends = {},
    },
}

-- registra informaes pertinentes a cada mdulo. Cada mdulo deve
-- chamar esta funo com os parmetros apropriados; caso contrrio,
-- suas funcionalidades no estaro disponveis
function cs_registerModule (module)
    local mdata = _moduleData[module.provides]
    abort(mdata, module.provides .. ": funcionalidade nao suportada")
    -- registra a ao
    mdata.module = module
    -- parmetros necessrios  execuo do mdulo
    mdata.params = module.params
    -- requisitos de verses de aplicativos
    mdata.requires = module.requires
end

-- finalizaes. Se for fornecido um cdigo para retorno, termina
-- execuo com este
function cs_finalize (exitCode)
--     flog:close()
    if exitCode then os.exit(exitCode) end
end

-- retorna o mdulo associado a uma ao
function cs_getModule (action)
    return _moduleData[action].module
end

-- iterador sobre os elementos de _moduleData
function cs_moduleDataList ()
    return coroutine.wrap(function ()
        for name, module in pairs(_moduleData) do
            coroutine.yield(name, module)
        end
    end)
end

-- retorna true se act corresponde a uma ao tratada pela aplicao.
-- Retorna true se act == nil
function cs_isAction (action)
    return _actions[action] ~= nil
end

-- retorna true se module  uma das dependncias de refModule, false em
-- caso contrrio
function cs_isDep (refModule, module)
    for _, dep in ipairs(_moduleData[refModule.provides].depends) do
        if dep == module then return true end
    end
    return false
end

function cs_requirementPolicy (ignore)
    _ignoreReqs = ignore
end

-- --------------------------------------------------
-- classe Action

Action = OO:new()

-- iterador: retorna todos os mdulos dos quais um determinado mdulo
-- (associado a 'mdata') depende. A ordem de percorrimento 
-- depth-first. O parmetro 'isDep'  usado para diferenciar chamadas
-- recursivas da chamada "original", porque esta no deve executar o
-- mtodo check(). O parmetro 'depType' especifica a relao de
-- dependncia entre os mdulos (forte ou fraca)
local function _getModuleDeps (mdata)
    local traverse
    traverse = function (md, isDep, depType)
        md = md or mdata
        for _, dep in ipairs(md.depends) do
            traverse(_moduleData[dep.module], true, dep.type)
        end
        coroutine.yield(md, isDep, depType)
    end
    return coroutine.wrap(traverse)
end

-- verifica os requisitos de verses para um determinado mdulo
local function _checkModuleReqs (mdata)
    local reqVersion = mdata.requires
    local versionOk, badExe, reason = Versions:check(reqVersion)
    if reason then
        if reason == VERSION.NOTFOUND then
            cs_log:warn(string.format("'%s' nao disponivel", badExe))
        elseif reason == VERSION.LOWVERSION then
            cs_log:warn(string.format("'%s' : versao incompativel\n       versao esperada: %-10s\n       versao encontrada: %s",
                badExe, reqVersion[badExe] or "", Versions:get(badExe) or ""))
        end
        if not _ignoreReqs then
            local others = Versions:findProviders(badExe, reqVersion[badExe])
            cs_log:info("o executavel pode ser encontrado nos seguintes servidores:")
            table.foreach(others, print)
            cs_log:info("use o parametro '-f' para ignorar dependencias")
        end
    end
    return reason == nil
end

-- construtor
function Action:new (action)
    local data = {
        action = action,
        mdata = _moduleData[action],
        module = _moduleData[action].module,
    }
    return OO.new(self, data)
end

function Action:execute (args, params)
    -- acrescenta aos parmetros repassados s dependncias os
    -- parmetros aceitos pelo mdulo em questo
    for mdescr, isDep, depType in _getModuleDeps(self.mdata) do
        local reqsOk = _checkModuleReqs(mdescr)
        if reqsOk or _ignoreReqs then
            if not reqsOk then
                cs_log:warn("ignorando falhas de requisitos")
            end
            local module = mdescr.module
            -- o parmetro de ajuda  verificado antes dos outros para
            -- permitir a exibio do help para cada mdulo
            -- individualmente (OBS.: no queremos mostrar o help para
            -- as dependncias)
            if not isDep then
                module:parseArgs(args, params)
                if module:opt("h") then
                    module:help()
                    return
                end
            end
            local check = module:check()
            if not check and isDep and depType == DEP.SOFT then
                -- trata-se de uma dependncia fraca que no pde ser
                -- satisfeita
                cs_log:fatal("a dependencia '"..module.actionInfo..
                    "' nao foi satisfeita")
                cs_finalize(1)
            elseif not check and (not isDep or depType == DEP.HARD) then
                -- ou  uma dependncia forte ou no  dependncia; em
                -- ambos os casos, deve ser executada
                cs_log:debug("executando "..(isDep and "dependencia" or "acao")..
                    ": "..module.actionInfo)
                module:execute(args, params)
            end
        end
    end
end

-- ==================================================
-- processamento comea efetivamente aqui

-- carrega os mdulos
require "cssvn"
require "csbuild"
require "csserver"
require "cscfg"
require "csmain"
require "csalg"
require "csapp"
require "cswebdeploy"
require "cspack"
require "csprops"

CSMake = Action:new(NONCMD_MODULE.MAIN)
CSMake:execute(arg)

-- finalizaes
cs_finalize()
