-- $Id: log.lua 35096 2005-01-27 17:48:59Z vge $

require "oo"
require "global"

-- --------------------------------------------------
-- classe Log
-- envia strings para stderr, de acordo com o "nvel" do problema. O
-- nvel  determinado pela funo utilizada (Log.fatal, Log.warn etc.),
-- e a msg s  exibida se corresponde a um nvel menor ou igual ao
-- default

Log = OO:new {
    level = { FATAL = 0, ERROR = 1, WARN = 2, INFO = 3, DEBUG = 4, OFF = 5, },
    appender = io.stderr
}

Log.level.current = Log.level.WARN

-- tabela inversa a Log.level
local levelCodes = {}
for k, v in pairs(Log.level) do
    levelCodes[v] = k
end

-- cria as funes para log, uma para cada nvel
for l, c in pairs(Log.level) do
    local level, code = l, c
    Log[string.lower(level)] = function (self, msg)
        if code <= self.level.current then
            abort(self.appender, "log appender nulo")
            self.appender:write(string.format("[%-5s] %s\n", level, msg))
        end
    end
end

-- redefine o nvel atual
function Log:setLevel (l)
    abort(levelCodes[l], "nivel de log invalido")
    self.level.current = l
end

-- retorna o nivel de log corrente
function Log:getLevel ()
    return self.level.current
end

-- --------------------------------------------------
-- classe FileLogger
-- anexa strings a um arquivo de log

FileLogger = Log:new()

-- inicializa o arquivo de log no modo especificado (ou em modo "a" se
-- 'mode' = nil)
function FileLogger:init (fileName, mode)
    self.fileName = fileName
    mode = mode or "a"
    local err
    abort(string.find("a+w", mode), "modo invalido para arq de log")
    self.appender, err = io.open(fileName, mode)
    return (self.appender ~= nil), err
end

-- fecha o arquivo de log
function FileLogger:close ()
    if self.appender then self.appender:close() end
end
