/*
 * $Id: MockSocketChannel.java 60184 2007-01-22 19:42:10Z lbarros $
 */
package nio.common;

import java.io.IOException;
import java.net.Socket;
import java.net.SocketAddress;
import java.nio.ByteBuffer;
import java.nio.channels.SocketChannel;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;


/**
 * Classe que implementa os mtodos abstratos de SocketChannel como um "mock",
 * isto , uma implementao somente para testes, no  para ser usada em
 * produo.
 *
 * @author Tecgraf
 */
public class MockSocketChannel extends SocketChannel {
  /** Bytes a serem escritos na prxima chamada do mtodo write. */
  private int bytesToWrite;

  /** Bytes escritos no channel. */
  private List<byte[]> bytesWritten;
  
  /**
   * Construtor que no repassa nenhum SelectorProvider ao SocketChannel,
   * somente null.
   */
  public MockSocketChannel() {
    super(null);
    bytesWritten = new ArrayList<byte[]>();
  }
  
  @Override
  public Socket socket() {
    // TODO Auto-generated method stub
    return null;
  }

  @Override
  public boolean isConnected() {
    // TODO Auto-generated method stub
    return false;
  }

  @Override
  public boolean isConnectionPending() {
    // TODO Auto-generated method stub
    return false;
  }

  @Override
  public boolean connect(SocketAddress remote) throws IOException {
    // TODO Auto-generated method stub
    return false;
  }

  @Override
  public boolean finishConnect() throws IOException {
    // TODO Auto-generated method stub
    return false;
  }

  @Override
  public int read(ByteBuffer dst) throws IOException {
    // TODO Auto-generated method stub
    return 0;
  }

  @Override
  public long read(ByteBuffer[] dsts, int offset, int length) throws IOException {
    // TODO Auto-generated method stub
    return 0;
  }

  @Override
  public int write(ByteBuffer src) throws IOException {
    int len = Math.min(src.remaining(), bytesToWrite);
    byte[] tmp = new byte[len];
    src.get(tmp);
    bytesWritten.addAll(Arrays.asList(tmp));
    return len;
  }

  @Override
  public long write(ByteBuffer[] srcs, int offset, int length) throws IOException {
    return 0;
  }

  @Override
  protected void implCloseSelectableChannel() throws IOException {
    // TODO Auto-generated method stub
    
  }

  @Override
  protected void implConfigureBlocking(boolean block) throws IOException {
    // TODO Auto-generated method stub
    
  }

  /**
   * Configura o nmero de bytes que sero "enviados" na prxima chamada ao
   * mtodo write.
   * 
   * @param i
   */
  public void setWillWrite(int i) {
    this.bytesToWrite = i;
  }
  
  /**
   * Reinicia o objeto, isto , apaga da memria todos os bytes lidos assim como
   * o contador de bytes para a prxima operao de escrita.
   */
  public void reset() {
    bytesWritten = new ArrayList<byte[]>();
    bytesToWrite = 0;
  }
  
  /**
   * Retorna o nmero de bytes "escritos" no socket desde que a classe foi
   * iniciada (ou reiniciada).
   * 
   * @return nmero de bytes escritos.
   */
  public int getBytesWrittenCount() {
    int total = 0;
    for (byte[] bytes : bytesWritten) {
      total += bytes.length;
    }
    return total;
  }

  /**
   * Retorna o inteiro recm-escrito (o mtodo assume que ao menos 4 bytes foram
   * escritos desde que o channel foi iniciado).
   * 
   * @return inteiro escrito.
   */
  public int getInt() {
    int number = 0;
    int counter = 1;
    for (byte[] bytes : bytesWritten) {
      for (byte aByte : bytes) {
        number += (aByte & 0xFF) << 8*(IOHandler.INT_SIZE-counter);
        if (counter++ == IOHandler.INT_SIZE) {
          break;
        }
      }
    }
    return number;
  }

  /**
   * Retorna o tipo longo recm-escrito (o mtodo assume que ao menos 8 bytes 
   * foram escritos desde que o channel foi iniciado).
   * 
   * @return tipo longo escrito.
   */
  public long getLong() {
    long number = 0;
    int counter = 1;
    for (byte[] bytes : bytesWritten) {
      for (byte aByte : bytes) {
        number += (long)(aByte & 0xFF) << 8*(IOHandler.LONG_SIZE-counter);
        if (counter++ == IOHandler.LONG_SIZE) {
          break;
        }
      }
    }
    return number;
  }
}
