package nio.common;

import java.io.IOException;

import junit.framework.TestCase;

/**
 *  Testes unitrios para a classe 
 *  {@link nio.common.SimpleFileTransferProtocol}.
 *
 * @author Tecgraf
 */
public class SimpleFileTransferProtocolTest extends TestCase {

  /** Instncia da classe a ser testada. */
  private SimpleFileTransferProtocol protocol;
  
  /**
   * {@inheritDoc}. Cria uma subclasse "mock" da classe a ser testada.
   */
  public void setUp() {
    // Por ora, no foi includo nenhum teste envolvendo o canal remoto
    protocol = new SimpleFileTransferProtocol(0, null) {
      @Override
      protected boolean contentMissing() throws IOException {
        return false;
      }
      @Override
      protected boolean processOperation() throws IOException {
        return false;
      }
      @Override
      protected boolean processPathSize() throws IOException {
        return false;
      }
      @Override
      protected boolean processPath() throws IOException {
        return false;
      }
      @Override
      protected boolean processFileSize() throws IOException {
        return false;
      }
      @Override
      protected boolean processPackSize() throws IOException {
        return false;
      }
      @Override
      protected boolean processPack() throws IOException {
        return false;
      }
      @Override
      protected void closeResources() throws IOException {}
      @Override
      protected boolean hasToWait() {
        return false;
      }
    };
  }
  
  /**
   * Testa o funcionamento normal do mtodo.
   */
  public void testCalcPackSizeThatFitsInBuffer1() {
    assertEquals(18, protocol.calcPackSizeThatFitsInBuffer(20, 3));
  }
  
  /**
   * Testa o funcionamento normal do mtodo.
   */
  public void testCalcPackSizeThatFitsInBuffer2() {
    assertEquals(18, protocol.calcPackSizeThatFitsInBuffer(20, 6));
  }
  
  /**
   * Testa com um valor mnimo de buffer.
   */
  public void testCalcPackSizeThatFitsInBuffer3() {
    assertEquals(20, protocol.calcPackSizeThatFitsInBuffer(20, 1));
  }
  
  /**
   * Testa com um valor de buffer maior do que o tamanho do pacote.
   */
  public void testCalcPackSizeThatFitsInBuffer4() {
    assertEquals(20, protocol.calcPackSizeThatFitsInBuffer(20, 21));
  }
  
  /**
   * Testa com um valor de buffer igual ao do tamanho do pacote.
   */
  public void testCalcPackSizeThatFitsInBuffer5() {
    assertEquals(20, protocol.calcPackSizeThatFitsInBuffer(20, 20));
  }

  /**
   * Testa com buffer zero.
   */
  public void testCalcPackSizeThatFitsInBuffer6() {
    try {
      protocol.calcPackSizeThatFitsInBuffer(20, 0);
      fail("Esperada IllegalArgumentException");
    }
    catch(IllegalArgumentException e) {}
  }

  /**
   * Testa com buffer negativo.
   */
  public void testCalcPackSizeThatFitsInBuffer7() {
    try {
      protocol.calcPackSizeThatFitsInBuffer(20, -2);
      fail("Esperada IllegalArgumentException");
    }
    catch(IllegalArgumentException e) {}
  }  
}
