/** 
 * @file 
 *
 */

import java.util.Properties;
import java.io.FileInputStream;

import org.omg.CORBA.*;
import org.omg.PortableServer.*;

import sgaidl.*;

/**
 * A classe <code>sgaBridge</code> representa uma ponte entre
 * SGAs e o SSI.
 */
public class SGABridge { 

  private Properties properties = null;

  private ORBHandler sgaORB = null;
  private ORBHandler serverORB = null;

  static private SGABridge instance = null;

  private SGABridge() throws Exception {
    properties = new Properties();
    loadProperties("SGABridge.properties");

    String sgaORBHost = getProperty("sgaORBHost");
    String sgaORBPort = getProperty("sgaORBPort");
    String serverORBHost = getProperty("serverORBHost");
    String serverORBPort = getProperty("serverORBPort");

    sgaORB = new ORBHandler(sgaORBPort, "SGAMgr", sgaORBHost);
    serverORB = new ORBHandler(serverORBPort, "ServerMgr", serverORBHost);
  }

  private void start() throws Exception {
    String serverHost = getProperty("serverHost");
    String serverPort = getProperty("serverPort");
    int retry = Integer.parseInt(getProperty("serverRetryIntervalSeconds"));

    String ior = "corbaloc::" + serverHost +":" + serverPort +
           "/SGAMgr/SGAMgrPOA/SGAManager";
    
    org.omg.CORBA.Object srv = serverORB.stringToObject(ior);
    while (srv._non_existent() ) {
       System.out.println("Buscando referncia CORBA para o SSI...");
       Thread.sleep(retry * 1000);  // Espera retry segundos (tempo em ms)
       srv = serverORB.stringToObject(ior);
    }
    SGAManager server = SGAManagerHelper.narrow(srv);
    SGABridgeManager sgaManager = new SGABridgeManager(server);
    sgaManager.activate("SGAManager");

    System.out.println("Acesso ao ambiente ORB inicializado");
  }

  protected void activateObjectAtSGAORB(String name, Servant object)
     throws Exception {
     sgaORB.activateObject(name, object);
  }

  protected void deactivateObjectAtSGAORB(String name)
     throws Exception {
     sgaORB.deactivateObject(name);
  }

  protected void activateObjectAtServerORB(String name, Servant object)
     throws Exception {
     serverORB.activateObject(name, object);
  }

  protected void deactivateObjectAtServerORB(String name)
     throws Exception {
     serverORB.deactivateObject(name);
  }

  protected boolean loadProperties(String fileName) {
    properties = new Properties();
    try {
       FileInputStream in = new FileInputStream(fileName);
       properties.load(in);
       in.close();
    }
    catch (Exception e) {
       System.out.println( "Falha na leitura de: " +
           fileName + " --> " + e.getMessage() );
       return false;
    }
    return true;
  }

  /**
   * Recupera uma propriedade do servio.
   * @param key - a identificao da propriedade
   * @return o valor da propriedade como uma string ou null
   * (caso a propriedade no seja encontrada).
   */
  protected String getProperty(String key) throws Exception {
     if ( properties == null )
        throw new Exception("No carregou arquivo de propriedades.");

     String property = properties.getProperty(key);
     if (property == null)
        throw new Exception("Propriedade '" + key + "' no definida.");

     return property;
  }

  static public SGABridge getInstance() {
     return instance;
  }

  static public void main(String[] args) {
     try {
        instance = new SGABridge();
        instance.start();
     }
     catch (Exception e) {
         System.out.println(e);
         //e.printStackTrace();
     }
  }
}
