/** 
 * Mdulo SGA PBS.
 *
 */

#include <unistd.h>
#include <sys/types.h>
#include <time.h>

#include "sgalib.h"

/* =================================================== */
/* Funes auxiliares                                  */
/* =================================================== */

/**
 * Coloca o processo para dormir por um determinado nmero de segundos.
 *
 */
static int _sleep(lua_State* L)
{
  // Tempo do sleep
  long dtime = (long) luaL_checknumber(L,1);

  /*
  for ( int i = 1; i <= dtime; i++ ) 
    usleep(999999);
  */
  sleep(dtime);

  return 0;
}


/**
 * Retorna o PID atual do processo.
 *
 * @return PID atual do processo.
 *
 */
static int _getpid(void)
{
  return (int)getpid();
}

/**
 * Fornece a hora em segundos desde Epoch (1 de janeiro de 1970).
 *
 */
static int _now(lua_State* L) {
   time_t tm = time(NULL);

   if (tm != ((time_t)-1) )
      lua_pushnumber(L, tm);
   else
      lua_pushnil(L);

   return 1;
}

/* =================================================== */
/* Funes exportadas para o ambiente de execuo Lua. */
/* =================================================== */

/**
 * Registra funes auxiliares no ambiente Lua.
 *
 */
static void init_proc_utils(lua_State *L)
{
  lua_register(L, "now", _now);
  lua_register(L, "sleep", _sleep);
}

/**
 * L e executa um script Lua.
 * 
 * Retorno:
 * 0 : Sucesso
 * 1 : Erro
 */
static int _dofile(lua_State* L, char* file)
{
  if (luaL_loadfile(L, file) || lua_pcall(L, 0, 0, 0)) {
    int n = lua_gettop(L);
    const char *msg = lua_tostring (L, n);
    lua_pop(L, 1);
    fprintf(stderr, "%s\n", msg);

    return 1;
  }

  return 0;
}

/**
 * Funo principal invocada para configurar o ambiente de execuo
 * para o SGA PBS.
 *
 */
int luaopen_sga(lua_State *L) {
   // Ajuste de uma varivel global para ajustar o pid do prprio sga-daemon
   int sga_pid = _getpid();

   // Inicializao das funcionalidades de kernel C relativo ao tratamento
   // de processos (questes de portabilidade so abstradas aqui).
   init_proc_utils(L);

   lua_pushnumber(L, sga_pid);
   lua_setglobal(L, "SGA_PID");

   _dofile(L, "sgaLinuxPBS.lua");

   return 0;
}

