/**
 * TestAdministrationService.java  $Author: cassino $ $Revision: 33587 $ - $Date:
 * 2004/03/17 20:22:07 $
 */
package server.providers;

import csbase.client.util.DateTranslator;
import csbase.logic.*;

import util.*;

import java.io.File;
import java.nio.ByteBuffer;
import java.rmi.RemoteException;
import java.util.Date;
import java.util.ResourceBundle;
import java.util.Vector;

/**
 * A classe <code>TestAdministrationService</code> representa um caso de teste
 * da administrao do CSBase.
 *
 * @see CSBaseAbstractTestCase
 */
public class TestAdministrationService extends CSBaseAbstractTestCase {
  /** Nome do arquivo de propriedades que o ResourceBundle ir ler */
  private static final String RES_FILENAME = "administration_test";
  /* a ser lido da varivel de systema preffix. */
  private static final String PREFFIX_NAME_PROP = "preffix";
  /* Strings usadas para obteno dos valores das propriedades. */
  private static final String MAX_CHANGES = "maxChanges";
  private static final String N_USERS = "nUsers";
  private ResourceBundle prop;
  /** Un associada aos usurios criados */
  private UN un;
  /** Perfil associado aos usurios criados */
  private Role role;
  private String unName = "un";
  private String roleName = "role";

  // prefixo usado para teste.
  private String preffix;
  private int userCount;
  private int max_changes;
  private User[] users;
  private UN[] uns;
  private Role[] roles;

  /**
   * Inicializa os objetos preparando o ambiente de testes.
   *
   * @throws Exception .
   */
  public void setUp() throws Exception {
    int suffix = -1;
    loadSystemVariables();
    loadProperties();
    un = createTestUN(preffix + unName);
    role = createTestRole(preffix + roleName);
    assertNotNull(un);
    assertNotNull(role);
    printAndLog("UN " + un.getName() + " criada.");
    printAndLog("Perfil " + role.getName() + " criado.");
    users = new User[userCount];
    uns = new UN[userCount];
    roles = new Role[userCount];
    // --------- Cria objetos do modelo lgico 
    for (int i = 0; i < userCount; i++) {
      suffix++;
      String strKey = preffix + suffix;

      // cria usurio
      users[i] = createTestUser(strKey, un, role);
      assertNotNull(users[i]);
      assertSameUser(users[i], User.getUserByLogin(strKey));
      printAndLog("Usurio " + users[i].getLogin() + " criado.");
      // cria un
      uns[i] = createTestUN(strKey);
      assertNotNull(uns[i]);
      assertSameUN(uns[i], UN.getUN(uns[i].getId()));
      printAndLog("UN" + uns[i].getName() + " criada.");
      // cria perfil
      roles[i] = createTestRole(strKey);
      assertNotNull(roles[i]);
      assertSameRole(roles[i], Role.getRole(roles[i].getId()));
      printAndLog("Perfil" + roles[i].getLogin() + " criado.");
    }
  }

  /**
   * Verifica se os dois usurios so iguais.
   *
   * @param u1 Usurio a ser comparado.
   * @param u2 Usurio usado como referncia da comparao.
   */
  private void assertSameUser(User u1, User u2) {
    assertEquals(u1.getLogin(), u2.getLogin());
    assertEquals(u1.getName(), u2.getName());
    assertEquals(u1.getKey(), u2.getKey());
    assertEquals(u1.getMail(), u2.getMail());
    assertTrue(u1.equals(u2));
  }

  /**
   * Verifica se os duas uns so iguais.
   *
   * @param un1 UN a ser comparada.
   * @param un2 UN usada como referncia da comparao.
   */
  private void assertSameUN(UN un1, UN un2) {
    assertEquals(un1.getName(), un2.getName());
    assertEquals(un1.getDesc(), un2.getDesc());
    assertTrue(un1.equals(un2));
  }

  /**
   * Verifica se os dois perfis so iguais.
   *
   * @param role1 Perfil a ser comparado.
   * @param role2 Perfil usado como referncia da comparao.
   */
  private void assertSameRole(Role role1, Role role2) {
    assertEquals(role1.getName(), role2.getName());
    assertEquals(role1.getDescription(), role2.getDescription());
    assertEquals(role1.getLogin(), role2.getLogin());
    assertEquals(role1.getPassword(), role2.getPassword());
    assertTrue(role1.equals(role2));
  }

  /**
   * Verifica se os dois usurios possuem os mesmos valores.
   *
   * @param u1 Usurio a ser comparado.
   * @param u2 Usurio usado como referncia da comparao.
   *
   * @return .
   */
  private boolean assertSameUserFields(User u1, User u2) {
    if (
      (u1.getLogin() == u2.getLogin()) && (u1.getName() == u2.getName()) &&
        (u1.getKey() == u2.getKey()) && (u1.getMail() == u2.getMail())) {
      return true;
    }
    return false;
  }

  /**
   * Verifica se as duas uns possuem os mesmos valores.
   *
   * @param un1 UN a ser comparada.
   * @param un2 UN usada como referncia da comparao.
   *
   * @return .
   */
  private boolean assertSameUNFields(UN un1, UN un2) {
    if (un1.getDesc() == un2.getDesc()) {
      return true;
    }
    return false;
  }

  /**
   * Verifica se os dois perfis possuem os mesmos valores.
   *
   * @param role1 Perfil a ser comparado.
   * @param role2 Perfil usado como referncia da comparao.
   *
   * @return .
   */
  private boolean assertSameRoleFields(Role role1, Role role2) {
    if (
      (role1.getLogin() == role2.getLogin()) &&
        (role.getDescription() == role2.getDescription())) {
      return true;
    }
    return false;
  }

  /**
   * Apaga todos os objetos criados.
   *
   * @throws Exception .
   */
  public void tearDown() throws Exception {
    String strKey;
    for (int i = 0; i < users.length; i++) {
      strKey = users[i].getLogin();
      // remove os usurios
      cleanUser(users[i].getLogin());
      assertNull(User.getUser(users[i].getId()));
      printAndLog("Usurio " + users[i].getLogin() + " removido.");
      // remove as uns
      cleanUN(uns[i].getName());
      assertNull(UN.getUN(uns[i].getId()));
      printAndLog("UN " + uns[i].getName() + " removida.");
      // remove os perfis
      cleanRole(strKey);
      assertNull(Role.getRole(roles[i].getId()));
      printAndLog("Perfil " + roles[i].getLogin() + " removido.");
    }
    cleanUN(un.getName());
    cleanRole(role.getName());
    assertNull(UN.getUN(un.getId()));
    assertNull(Role.getRole(role.getId()));
    printAndLog("Un e Perfil dos usurios criados foram removidos");
  }

  /**
   * Mtodo que realiza o teste de carga, verificando se os dados esto ok.
   *
   * @throws Exception .
   */
  public void testAdministration() throws Exception {
    String strKey;

    // testa se todos os usurios foram criados
    for (int i = 0; i < users.length; i++) {
      strKey = users[i].getLogin();
      assertSameUser(users[i], User.getUserByLogin(strKey));
      printAndLog("Usurio " + strKey + " est OK.");
      assertSameUN(uns[i], UN.getUN(uns[i].getId()));
      printAndLog("UN " + uns[i].getName() + " est OK.");
      assertSameRole(roles[i], Role.getRole(roles[i].getId()));
      printAndLog("Perfil " + roles[i].getLogin() + " est OK.");
      printAndLog("iniciando as modificaes de teste...");
      for (int j = 0; j < max_changes; j++) {
        String newData = strKey + "ch" + j;
        UserInfo userInfo =
          new UserInfo(strKey, un.getId(), newData, newData, role.getId());
        UNInfo unInfo = new UNInfo(strKey, newData);
        RoleInfo roleInfo = new RoleInfo(strKey, newData, newData, newData);
        userInfo.passwordDigest = strKey;
        User newUser;
        UN newUN;
        Role newRole;
        try {
          newUser = User.modifyUser(users[i].getId(), userInfo);
          assertNotNull(newUser);
          printAndLog(
            "modificaes realizadas em usurio: " + users[i].getLogin());
          assertFalse(assertSameUserFields(newUser, users[i]));
          printAndLog("confirmao da modificao indica sucesso.");
          users[i] = newUser;
          newUN = UN.modifyUN(uns[i].getId(), unInfo);
          assertNotNull(newUN);
          printAndLog("modificaes realizadas em un: " + uns[i].getName());
          assertFalse(assertSameUNFields(newUN, uns[i]));
          printAndLog("confirmao da modificao indica sucesso.");
          uns[i] = newUN;
          newRole = Role.modifyRole(roles[i].getId(), roleInfo);
          assertNotNull(newRole);
          printAndLog(
            "modificaes realizadas em perfil: " + roles[i].getName());
          assertFalse(assertSameRoleFields(newRole, roles[i]));
          printAndLog("confirmao da modificao indica sucesso.");
          roles[i] = newRole;
        }
        catch (Exception e) {
          printAndLog("Problemas.... ");
          e.printStackTrace();
        }
      }
    }
  }

  /**
   * Obtm as propriedades do arquivo.
   */
  private void loadProperties() {
    prop = getTestBundle(RES_FILENAME);
    try {
      this.userCount = Integer.parseInt(prop.getString(N_USERS));
      this.max_changes = Integer.parseInt(prop.getString(MAX_CHANGES));
    }
    catch (Exception e) {
      e.printStackTrace();
      fail("Falha ao ler propriedades de arquivo");
    }
  }

  /**
   * Obtm as variveis do sistema.
   */
  private void loadSystemVariables() {
    String preffix = System.getProperty(PREFFIX_NAME_PROP);
    if (preffix == null) {
      fail("Prefixo no encontrado");
    }
    this.preffix = preffix;
  }

  /**
   * Imprime a mensagem junto com a data e hora atual.
   *
   * @param message A mensagem a ser exibida na tela.
   */
  private void printAndLog(String message) {
    Date date = new Date(System.currentTimeMillis());

    //String date = DateTranslator.getInstance().translate( time );
    String strDate = date.toString().substring(0, 19);
    System.out.println("<" + strDate + ">: " + message);
  }
}
