/**
 * TestPlatform.java $Author: cassino $ $Revision: 33587 $ - $Date: 2004/03/17
 * 20:22:32 $
 */
package server.providers;

import csbase.logic.*;

import util.*;

import java.util.Vector;

/**
 * A classe <code>TestPlatform</code> representa um caso de teste para os
 * servios de manupulao de Platform do csbase.
 */
public class TestPlatform extends CSBaseAbstractTestCase {
  /**
   * Inicializao do teste.  chamado antes de cada mtodo de teste.
   *
   * @throws Exception .
   */
  public void setUp() throws Exception {
    initializePlatform(ELEMENTS);
  }

  /**
   * Finalizao do teste.  chamado depois de cada mtodo de teste.
   *
   * @throws Exception .
   */
  public void tearDown() throws Exception {
    cleanPlatform(ELEMENTS);
  }

  /**
   * Testa se o objeto criado contm todos os valores informados pelo usurio.
   *
   * @throws Exception .
   */
  public void testCreatePlatform() throws Exception {
    for (int i = 0; i < ELEMENTS; i++) {
      assertEquals(pInfo[i].name, plat[i].getName());
      assertEquals(pInfo[i].description, plat[i].getDescription());
    }
  }

  /**
   * Verifica se a plataforma foi criada no arquivo.
   *
   * @throws Exception .
   */
  public void testCreatePlatformFile() throws Exception {
    Vector allPlatforms = Platform.getAllPlatforms();
    assertEquals(ELEMENTS, allPlatforms.size());
    for (int i = 0; i < ELEMENTS; i++) {
      assertTrue(allPlatforms.contains(plat[i]));
    }
  }

  /**
   * Verifica os testes de obteno de uma plataforma.
   *
   * @throws Exception .
   */
  public void testGetPlatformById() throws Exception {
    Platform platform = Platform.getPlatform(plat[0].getId());
    assertSame(plat[0], platform);
  }

  /**
   * Verifica se o sistema aceita criar uma plataforma com o mesmo nome.
   *
   * @throws Exception .
   */
  public void testCreateSamePlatform() throws Exception {
    Platform platform = Platform.createPlatform(pInfo[0]);
    assertNull(platform);
  }

  /**
   * Testa a remoo de todas as plataformas.
   *
   * @throws Exception .
   */
  public void testDeleteAllPlatforms() throws Exception {
    cleanPlatform(ELEMENTS);
    Vector allPlatforms = Platform.getAllPlatforms();
    assertEquals(0, allPlatforms.size());
  }

  /**
   * Testa se s as plataformas pares foram apagadas, depois tenta reinserir as
   * plataformas pares apagadas.
   *
   * @throws Exception .
   */
  public void testEvenPlatforms() throws Exception {
    /* apagando plataformas pares */
    for (int i = 0; i < ELEMENTS; i++) {
      if ((i % 2) == 0) {
        Platform.deletePlatform(plat[i].getId());
        plat[i] = null;
      }
    }
    /* verifica se as pares foram apagadas e se as impares permanecem */
    Vector allPlatforms = Platform.getAllPlatforms();
    int remainingPlatforms = (ELEMENTS / 2);
    assertEquals(remainingPlatforms, allPlatforms.size());
    for (int i = 0; i < ELEMENTS; i++) {
      if ((i % 2) == 1) {
        assertTrue(allPlatforms.contains(plat[i]));
      }
    }
    /* recria os pares recem apagados */
    for (int i = 0; i < ELEMENTS; i++) {
      if ((i % 2) == 0) {
        plat[i] = Platform.createPlatform(pInfo[i]);
      }
    }
    /* verifica se todos esto presentes */
    allPlatforms = Platform.getAllPlatforms();
    assertEquals(ELEMENTS, allPlatforms.size());
    for (int i = 0; i < ELEMENTS; i++) {
      assertTrue(allPlatforms.contains(plat[i]));
    }
  }

  /**
   * Testa a modificao de plataformas.
   *
   * @throws Exception .
   */
  public void testModifyPlatform() throws Exception {
    String newDescription = "Carambola verso 9.2";
    pInfo[0].description = newDescription;
    Platform platform = Platform.modifyPlatform(plat[0].getId(), pInfo[0]);
    assertNotNull(platform);
    assertEquals(pInfo[0].description, platform.getDescription());
    pInfo[2].description = "Linux Black Shoes 1.0";
    platform = Platform.modifyPlatform(plat[2].getId(), pInfo[2]);
    assertNotNull(platform);
    assertEquals(pInfo[2].description, platform.getDescription());
    pInfo[4].description = "SOS verso 6.22";
    platform = Platform.modifyPlatform(plat[4].getId(), pInfo[4]);
    assertNotNull(platform);
    assertEquals(pInfo[4].description, platform.getDescription());
  }

  /**
   * Testa a modificao do nome das plataformas. (No podem ser modificadas)
   *
   * @throws Exception .
   */
  public void testModifyName() throws Exception {
    String newName = "irix";
    pInfo[0].name = newName;
    Platform platform = Platform.modifyPlatform(plat[0].getId(), pInfo[0]);
    assertNull(platform);
    Platform anotherPlat = Platform.getPlatform(plat[0].getId());
    assertEquals("0", anotherPlat.getName());
  }

  /**
   * Testa a modificao de uma plataforma no existente no arquivo.
   *
   * @throws Exception .
   */
  public void testModifyNonExistentPlatform() throws Exception {
    Platform platform =
      Platform.modifyPlatform(new Integer(ELEMENTS + 10), pInfo[0]);
    assertNull(platform);
    platform = Platform.getPlatform(new Integer(ELEMENTS + 10));
    assertNull(platform);
  }

  /**
   * Tenta modificar uma plataforma com identificador nulo. public void
   * testModifyNull () throws Exception {  Platform platform = null; try {
   * platform = Platform.modifyPlatform( null, pInfo[0] ); fail("Deveria ser
   * lanado Exception");  } catch ( Exception pe ) {} assertNull ( platform
   * ); }
   *
   * @throws Exception .
   */
  /**
   * Tenta modificar uma plataforma usando uma informao nula.
   *
   * @throws Exception .
   */
  public void testModifyNullPlatInfo() throws Exception {
    Platform platform = null;
    try {
      platform = Platform.modifyPlatform(plat[0].getId(), null);
      fail("Deveria ser lanado Exception");
    }
    catch (Exception pe) {}
    assertNull(platform);
  }

  /**
   * Tenta modificar uma plataforma passando uma pInfo com campos nulos. public
   * void testModifyWithNullFields () throws Exception {  Platform platform =
   * null; try { PlatformInfo nullInfo = new PlatformInfo( null, null );
   * platform = Platform.modifyPlatform( plat[0].getId(), nullInfo );
   * fail("Deveria ser lanado Exception"); } catch( Exception pe ) {}
   * assertNull ( platform ); }
   *
   * @throws Exception .
   */
  /**
   * Tenta criar uma plataforma passando uma Informao nula.
   *
   * @throws Exception .
   */
  public void testCreateWithNullInfo() throws Exception {
    Platform platform = null;
    try {
      platform = Platform.createPlatform(null);
      fail("Deveria ser lanado Exception");
    }
    catch (Exception pe) {}
    assertNull(platform);
  }

  /**
   * Tenta criar uma plataforma a partir de uma pInfo com campos nulos.
   *
   * @throws Exception .
   */
  public void testCreateWithNullFields() throws Exception {
    Platform platform = null;
    try {
      PlatformInfo nullInfo = new PlatformInfo(null, null);
      assertNotNull(nullInfo);
      platform = Platform.createPlatform(nullInfo);
      fail("Deveria ser lanado Exception");
    }
    catch (Exception pe) {}
    assertNull(platform);
  }

  /**
   * Tenta obter uma plataforma passando um identificador nulo. public void
   * testGetPlatformWithNullId () throws Exception { Platform platform = null;
   * try { platform = Platform.getPlatform( null ); fail("Deveria ser lanado
   * Exception");  } catch ( Exception pe ) {} assertNull( platform );  }
   *
   * @throws Exception .
   */
  /**
   * Testa o pedido de remoo de plataforma passando um identificador nulo.
   *
   * @throws Exception .
   */
  public void testDeletePlatformWithNullId() throws Exception {
    try {
      Platform.deletePlatform(null);
      fail("Deveria ser lanado Exception");
    }
    catch (Exception pe) {}
  }
}
