/**
 * TestUN.java  $Author: cassino $ $Revision: 33587 $ - $Date: 2004/03/17
 * 20:22:35 $
 */
package server.providers;

import csbase.logic.*;

import util.*;

import java.util.Vector;

/**
 * A classe <code>TestRole</code> representa um caso de teste para os servios
 * de manipulao de Perfis do Websintesi.
 */
public class TestRole extends CSBaseAbstractTestCase {
  /* TESTES QUE NO PASSARAM:
   * testCreateWithNullInfo()
   * testCreateWithNullFields()
   * testDeleteNull()
   * testModifyNull()
   * testModifyNullInfo()
   * testGetRoleNull()
   */
  /**
   * Inicializao do teste.  chamado antes de cada mtodo de teste.
   *
   * @throws Exception .
   */
  protected void setUp() throws Exception {
    initializeRole(ELEMENTS);
  }

  /**
   * Finalizao do teste.  chamado depois de cada mtodo de teste.
   *
   * @throws Exception .
   */
  protected void tearDown() throws Exception {
    cleanRole(ELEMENTS);
  }

  /**
   * Testa a construo de Role. Caso a UN a ser construda seja existente,
   * ento ser apagada e ento ocorrer uma nova tentativa de insero.
   *
   * @throws Exception .
   */
  public void testCreateRole() throws Exception {
    for (int i = 0; i < ELEMENTS; i++) {
      assertNotNull(role[i]);
      assertEquals(roleInfo[i].name, role[i].getName());
      assertEquals(roleInfo[i].description, role[i].getDescription());
      assertEquals(roleInfo[i].login, role[i].getLogin());
      assertEquals(roleInfo[i].password, role[i].getPassword());
    }
  }

  /**
   * Verifica se o sistema aceita criar um novo Perfil com o mesmo nome.
   *
   * @throws Exception .
   */
  public void testCreateSameRole() throws Exception {
    Role aRole = Role.createRole(roleInfo[0]);
    assertNull(aRole);
  }

  /**
   * Testa a remoo dos Perfis.
   *
   * @throws Exception .
   */
  public void testDeleteAllRoles() throws Exception {
    cleanRole(ELEMENTS);
    Vector allRoles = Role.getAllRoles();
    assertEquals(0, allRoles.size());
  }

  /**
   * Testa se s os Perfis pares foram apagados depois, tenta recriar os Perfis
   * pares apagados
   *
   * @throws Exception .
   */
  public void testDeleteEvenRoles() throws Exception {
    /* Apago todos os Perfis pares */
    for (int i = 0; i < ELEMENTS; i++) {
      if ((i % 2) == 0) {
        Role.deleteRole(role[i].getId());
        role[i] = null;
      }
    }
    /* Verifico se os pares foram apagados e se os mpares permanecem */
    Vector roleList = Role.getAllRoles();
    int remainingElements = ELEMENTS / 2;
    assertEquals(remainingElements, roleList.size());
    for (int i = 0; i < ELEMENTS; i++) {
      if ((i % 2) == 1) {
        assertTrue(roleList.contains(role[i]));
      }
    }
    /* Recrio os pares recm-apagados */
    for (int i = 0; i < ELEMENTS; i++) {
      if ((i % 2) == 0) {
        role[i] = Role.createRole(roleInfo[i]);
      }
    }
    /* Verifico se todos esto presentes */
    roleList = Role.getAllRoles();
    assertEquals(ELEMENTS, roleList.size());
    for (int i = 0; i < ELEMENTS; i++) {
      assertTrue(roleList.contains(role[i]));
    }
  }

  /**
   * Tento passar um roleInfo nulo para o mtodo de criao
   *
   * @throws Exception .
   */
  public void testCreateWithNullInfo() throws Exception {
    Role aRole = null;
    try {
      aRole = Role.createRole(null);
      fail("Esperada Exception");
    }
    catch (Exception e) {}
    assertNull(aRole);
  }

  /*
   * Tento passar um unInfo que contm campos nulos
   */
  public void testCreateWithNullFields() throws Exception {
    RoleInfo aRoleInfo = new RoleInfo(null, null, null, null);
    Role aRole = null;
    try {
      aRole = Role.createRole(aRoleInfo);
      fail("Esperada Exception");
    }
    catch (Exception e) {}
    assertNull(aRole);
  }

  /*
   * Tento apagar um Perfil que no existe
   */
  public void testDeleteNonExistentRole() throws Exception {
    Vector RoleList = Role.getAllRoles();
    int listSize = RoleList.size();
    Role.deleteRole(new Integer(3000));
    RoleList = Role.getAllRoles();
    assertEquals(listSize, RoleList.size());
  }

  /*
   * Passo um id nulo pro Delete
   *
       public void testDeleteNull() throws Exception {
         try {
           Role.deleteRole( null );
           fail("Esperada Exception");
         } catch( Exception e ) { }
       }*/
  /*
   * Tento modificar o nome (no pode ser modificado)
   */
  public void testModifyName() throws Exception {
    Role aRole = Role.modifyRole(role[1].getId(), roleInfo[0]);
    assertNull(aRole);
    Role anotherRole = Role.getRole(role[1].getId());
    assertEquals("1", anotherRole.getName());
  }

  /*
   * Tento passar um identificador no existente
   */
  public void testModifyNonExistentRole() throws Exception {
    Role aRole = Role.modifyRole(new Integer(3000), roleInfo[0]);
    assertNull(aRole);
    Role anotherRole = Role.getRole(new Integer(3000));
    assertNull(anotherRole);
  }

  /*
   * Tento passar um identificador nulo
   *
       public void testModifyNull() throws Exception {
         RoleInfo aRoleInfo = new RoleInfo( "alo", "voce", "bla", "coisa" );
         Role aRole = null;
         try {
           aRole = Role.modifyRole( null, aRoleInfo );
           fail("Esperada Exception");
         } catch( Exception e ) { }
         assertNull( aRole );
  
       }*/
  /*
   * Tento passar um roleInfo nulo
   */
  public void testModifyNullInfo() throws Exception {
    Role aRole = null;
    try {
      aRole = Role.modifyRole(role[0].getId(), null);
      fail("Esperada Exception");
    }
    catch (Exception e) {}
    assertNull(aRole);
  }

  /*
   * Tento passar um roleInfo que contm campos nulos
   *
       public void testModifyWithNullFields() throws Exception {
         RoleInfo aRoleInfo = new RoleInfo( null, null, null, null );
         Role aRole = null;
         try {
           aRole = Role.modifyRole( role[0].getId(), aRoleInfo );
           fail("Esperada Exception");
         } catch( Exception e ) { }
  
         assertNull( aRole );
         assertEquals( "0", role[0].getName() );
         assertEquals( "0", role[0].getDescription() );
         assertEquals( "0", role[0].getLogin() );
         assertEquals( "0", role[0].getPassword() );
  
       }*/
  public void testGetRole() throws Exception {
    for (int i = 0; i < ELEMENTS; i++) {
      assertEquals(role[i], Role.getRole(role[i].getId()));
    }
  }

  /**
   * Testa se o vetor recuperado de Perfis corresponde ao esperado. OBS: Esse
   * vetor est sendo retornado na ordem de insero.
   *
   * @throws Exception .
   */
  public void testGetAllRoles() throws Exception {
    Vector allRoles = Role.getAllRoles();
    assertEquals(ELEMENTS, allRoles.size());
    for (int i = 0; i < ELEMENTS; i++) {
      assertTrue(allRoles.contains(role[i]));
    }
  }

  /**
   * Testa a modificao de um Perfil existente no arquivo de dados.
   *
   * @throws Exception .
   */
  public void testModifyRole() throws Exception {
    String desc1 = "imensidao azul";
    String login1 = "cpixaba";
    String password1 = "1234";
    String desc2 = "etiqueta";
    String login2 = "vdias";
    String password2 = "54312";
    RoleInfo aRoleInfo =
      new RoleInfo(role[0].getName(), desc1, login1, password1);
    RoleInfo anotherRoleInfo =
      new RoleInfo(role[1].getName(), desc2, login2, password2);
    Role.modifyRole(role[0].getId(), aRoleInfo);
    Role.modifyRole(role[1].getId(), anotherRoleInfo);
    role[0] = Role.getRole(role[0].getId());
    role[1] = Role.getRole(role[1].getId());
    assertEquals(desc1, role[0].getDescription());
    assertEquals(login1, role[0].getLogin());
    assertEquals(password1, role[0].getPassword());
    assertEquals(desc2, role[1].getDescription());
    assertEquals(login2, role[1].getLogin());
    assertEquals(password2, role[1].getPassword());
  }
  /**
   * Testa passando null pro GetRole public void testGetRoleNull() throws
   * Exception { Role aRole = null; try { aRole = Role.getRole( null );
   * fail("Esperada Exception"); } catch( Exception e ) {} assertNull( aRole
   * ); }
   */
}
