/**
 * TestServerPlatform.java  $Author: cassino $ $Revision: 33587 $ - $Date:
 * 2004/03/17 20:22:41 $
 */
package server.providers;

import csbase.client.ServerMonitor;
import csbase.logic.*;

import util.*;

import java.io.File;
import java.util.ResourceBundle;
import java.util.Vector;

/**
 * A classe <code>TestServerPlatform</code> representa um caso de teste para o
 * contador de IDs quando um servidor  derrubado e momentos depois 
 * levantado novamente.
 */
public class TestServerPlatform extends CSBaseAbstractTestCase {
  /** Atributos auxiliares para os testes. */
  private Platform aPlat;
  private int Platform1;
  private int Platform2;
  /** Numero de elementos que serao testados. */
  private int COUNT_Platform = 5;
  private String[] platNames;
  private String[] platDesc;
  private Object platId1;
  private Object platId2;

  /**
   * Configuraes a serem inicializadas antes de rodar qualquer um dos testes.
   *
   * @throws Exception .
   */
  public void setUp() throws Exception {
    /* indices das plataformas que sero testadas na remoo */
    Platform1 = 1;
    Platform2 = 2;
    platNames = new String[COUNT_Platform];
    platDesc = new String[COUNT_Platform];
    /* inicializa-se todas as plataformas */
    initializePlatform(COUNT_Platform);
    /* guarda os atributos das plataformas para serem comparados no futuro. */
    for (int i = 0; i < COUNT_Platform; i++) {
      platNames[i] = plat[i].getName();
      platDesc[i] = plat[i].getDescription();
    }
    /*  apagada a primeira plataforma (Platform 2) */
    platId1 = plat[Platform1].getId();
    Platform.deletePlatform(plat[Platform1].getId());
    /*  apagada a segunda plataforma (Platform 3) */
    platId2 = plat[Platform2].getId();
    Platform.deletePlatform(plat[Platform2].getId());
  }

  public void tearDown() throws Exception {
    cleanPlatform(COUNT_Platform);
  }

  /**
   * Realiza os testes de plataforma antes do servidor sofrer shutdown. Sero
   * criadas plataformas no servidor, e uma das plataformas ser apagada.
   *
   * @throws Exception .
   */
  public void testPlatformBeforeCrash() throws Exception {
    /* certifica-se que a primeira plataforma (Platform 2) foi apagada */
    aPlat = Platform.getPlatform(platId1);
    assertNull(aPlat);
    /* certifica-se que a segunda plataforma (Platform 3) foi apagada */
    aPlat = Platform.getPlatform(platId2);
    assertNull(aPlat);
    /* certifica-se que as demais esto l */
    for (int i = 0; i < COUNT_Platform; i++) {
      if ((i != 1) && (i != 2)) {
        aPlat = Platform.getPlatform(plat[i].getId());
        assertNotNull(aPlat);
      }
    }
  }

  /**
   * Realiza testes no cliente depois que o servidor for levantado apos um
   * shutdown.
   *
   * @throws Exception .
   */
  public void testPlatformAfterCrash() throws Exception {
    /* so criadas mais 2 plataformas */
    pInfo[0] = new PlatformInfo("Turtle 0.1", "Turtle, sistema baiano");
    pInfo[1] = new PlatformInfo("VRS", "Virus System");
    Platform[] platET = new Platform[2];
    platET[0] = Platform.createPlatform(pInfo[0]);
    platET[1] = Platform.createPlatform(pInfo[1]);
    /* verifica se as plataformas recm criadas esto no arquivo */
    aPlat = Platform.getPlatform(platET[0].getId());
    assertNotNull(aPlat);
    aPlat = Platform.getPlatform(platET[1].getId());
    assertNotNull(aPlat);
    /* verifica se as 3 plataformas restantes de quando o servidor ainda no
       tinha sofrido shutdown estao presentes e NAO foram sobrescrevidas */
    for (int i = 0; i < COUNT_Platform; i++) {
      if ((i != Platform1) && (i != Platform2)) {
        aPlat = Platform.getPlatform(plat[i].getId());
        assertNotNull(aPlat);
        assertEquals(platNames[i], aPlat.getName());
        assertEquals(platDesc[i], aPlat.getDescription());
      }
    }
  }

  public TestServerPlatform(String name) {
    super(name);
  }
}
