/**
 * TestServerRole.java  $Author: cassino $ $Revision: 33587 $ - $Date: 2004/03/17
 * 20:22:44 $
 */
package server.providers;

import csbase.client.ServerMonitor;
import csbase.logic.*;

import util.*;

import java.io.File;
import java.util.ResourceBundle;
import java.util.Vector;

/**
 * A classe <code>TestServerRole</code> representa um caso de teste para os
 * perfis quando o servidor  derrubado e momentos depois  levantado
 * novamente.
 */
public class TestServerRole extends CSBaseAbstractTestCase {
  /** Atributos auxiliares para os testes. */
  private Role aRole;
  private int Role1;
  private int Role2;
  /** Numero de elementos que serao testados. */
  private int COUNT_Role = 5;
  private String[] roleNames;
  private String[] roleDesc;
  private Object roleId1;
  private Object roleId2;

  /**
   * Configuraes a serem inicializadas antes de rodar qualquer um dos testes.
   *
   * @throws Exception .
   */
  public void setUp() throws Exception {
    /* indices dos perfis que sero testadas na remoo */
    Role1 = 1;
    Role2 = 2;
    roleNames = new String[COUNT_Role];
    roleDesc = new String[COUNT_Role];
    /* inicializa-se todos os perfis */
    initializeRole(COUNT_Role);
    /* guarda os atributos dos perfis para serem comparados no futuro. */
    for (int i = 0; i < COUNT_Role; i++) {
      roleNames[i] = role[i].getName();
      roleDesc[i] = role[i].getDescription();
    }
    /*  apagada o primeiro perfil (Role 2) */
    roleId1 = role[Role1].getId();
    Role.deleteRole(role[Role1].getId());
    /*  apagada o segundo perfil (Role 3) */
    roleId2 = role[Role2].getId();
    Role.deleteRole(role[Role2].getId());
  }

  public void tearDown() throws Exception {
    cleanRole(COUNT_Role);
  }

  /**
   * Realiza os testes de perfil antes do servidor sofrer shutdown. Sero
   * criados perfis no servidor, e um dos perfis ser apagada.
   *
   * @throws Exception .
   */
  public void testRoleBeforeCrash() throws Exception {
    /* certifica-se que o primeiro perfil (Role 2) foi apagado */
    aRole = Role.getRole(roleId1);
    assertNull(aRole);
    /* certifica-se que o segundo perfil (Role 3) foi apagado */
    aRole = Role.getRole(roleId2);
    assertNull(aRole);
    /* certifica-se que as demais esto l */
    for (int i = 0; i < COUNT_Role; i++) {
      if ((i != 1) && (i != 2)) {
        aRole = Role.getRole(role[i].getId());
        assertNotNull(aRole);
      }
    }
  }

  /**
   * Realiza testes no cliente depois que o servidor for levantado apos um
   * shutdown.
   *
   * @throws Exception .
   */
  public void testRoleAfterCrash() throws Exception {
    /* so criadas mais 2 perfis */
    roleInfo[0] =
      new RoleInfo(
        "Est120", "Estagirio 120 horas p/ semana", "est120", "est120");
    roleInfo[1] =
      new RoleInfo(
        "Est220", "Estagirio 220 horas p/ semana", "est120", "est120");
    Role[] roleET = new Role[2];
    roleET[0] = Role.createRole(roleInfo[0]);
    roleET[1] = Role.createRole(roleInfo[1]);
    /* verifica se os perfis recm criados esto no arquivo */
    aRole = Role.getRole(roleET[0].getId());
    assertNotNull(aRole);
    aRole = Role.getRole(roleET[1].getId());
    assertNotNull(aRole);
    /* verifica se os 3 perfis restantes de quando o servidor ainda no
       tinha sofrido shutdown estao presentes e NAO foram sobrescrevidos */
    for (int i = 0; i < COUNT_Role; i++) {
      if ((i != Role1) && (i != Role2)) {
        aRole = Role.getRole(role[i].getId());
        assertNotNull(aRole);
        assertEquals(roleNames[i], aRole.getName());
        assertEquals(roleDesc[i], aRole.getDescription());
      }
    }
  }

  public TestServerRole(String name) {
    super(name);
  }
}
