/**
 * TestServerUN.java  $Author: cassino $ $Revision: 33587 $ - $Date: 2004/03/17
 * 20:22:48 $
 */
package server.providers;

import csbase.client.ServerMonitor;
import csbase.logic.*;

import util.*;

import java.io.File;
import java.util.ResourceBundle;
import java.util.Vector;

/**
 * A classe <code>TestServerUN</code> representa um caso de teste para o
 * contador de IDs quando um servidor  derrubado e momentos depois 
 * levantado novamente. O teste consiste em verificar se o contador se mantem
 * confivel quando o servidor  levantado.
 */
public class TestServerUN extends CSBaseAbstractTestCase {
  /** Atributos auxiliares para os testes. */
  private UN aUn;
  private int UN1;
  private int UN2;
  /** Numero de elementos que serao testados. */
  private int COUNT_UN = 5;
  private String[] unNames;
  private String[] unDesc;
  private Integer unId1;
  private Integer unId2;

  /**
   * Configuraes a serem inicializadas antes de rodar qualquer um dos testes.
   *
   * @throws Exception .
   */
  public void setUp() throws Exception {
    /* indices das uns que sero testadas na remoo */
    UN1 = 1;
    UN2 = 2;
    unNames = new String[COUNT_UN];
    unDesc = new String[COUNT_UN];
    /* inicializa-se todas as UNs */
    initializeUN(COUNT_UN);
    /* guarda os atributos das UNS para serem comparados no futuro. */
    for (int i = 0; i < COUNT_UN; i++) {
      unNames[i] = un[i].getName();
      unDesc[i] = un[i].getDesc();
    }
    /*  apagada a primeira UN (UN 2) */
    unId1 = (Integer)un[UN1].getId();
    UN.deleteUN(un[UN1].getId());
    /*  apagada a segunda UN  (UN 3) */
    unId2 = (Integer)un[UN1].getId();
    UN.deleteUN(un[UN2].getId());
  }

  public void tearDown() throws Exception {
    cleanUN(COUNT_UN);
  }

  /**
   * Realiza os testes de UN antes do servidor sofrer shutdown. Sero criadas
   * UNs no servidor, e uma das UNs ser apagada.
   *
   * @throws Exception .
   */
  public void testUNBeforeCrash() throws Exception {
    /* certifica-se que a primeira UN (UN 2) foi apagada */
    aUn = UN.getUN(unId1);
    assertNull(aUn);
    /* certifica-se que a segunda UN (UN 3) foi apagada */
    aUn = UN.getUN(unId2);
    assertNull(aUn);
    /* certifica-se que as demais esto l */
    for (int i = 0; i < COUNT_UN; i++) {
      if ((i != 1) && (i != 2)) {
        aUn = UN.getUN(un[i].getId());
        assertNotNull(aUn);
      }
    }
  }

  /**
   * Realiza testes no cliente depois que o servidor for levantado apos um
   * shutdown. As UNs que foram removidas no teste anterior deixaram,
   * teoricamente, um espao entre as IDs no contador de IDs e que no deveria
   * atrapalhar uma nova insero de UN.   O teste trata-se de certificar que
   * o servidor no esta se perdendo por  causa deste espao.
   *
   * @throws Exception .
   */
  public void testUNAfterCrash() throws Exception {
    /* so criadas mais 2 UNS */
    unInfo[0] = new UNInfo("SJ", "San Jose Sharks");
    unInfo[1] = new UNInfo("PIT", "Pittsburgh Penguins");
    UN[] unET = new UN[2];
    unET[0] = UN.createUN(unInfo[0]);
    unET[1] = UN.createUN(unInfo[1]);
    /* verifica se as uns recm criadas esto no arquivo */
    aUn = UN.getUN(unET[0].getId());
    assertNotNull(aUn);
    aUn = UN.getUN(unET[1].getId());
    assertNotNull(aUn);
    /* verifica se as 3 uns restantes de quando o servidor ainda no tinha
       sofrido shutdown estao presentes e NAO foram sobrescrevidas */
    for (int i = 0; i < COUNT_UN; i++) {
      if ((i != UN1) && (i != UN2)) {
        aUn = UN.getUN(un[i].getId());
        assertNotNull(aUn);
        assertEquals(unNames[i], aUn.getName());
        assertEquals(unDesc[i], aUn.getDesc());
      }
    }
    /* verifica se a as uns recem criadas ocuparam as posies das que foram
       removidas antes do servidor ter sofrido shutdown. Isto testa se o
       comportamento esta correto, se a criacao das UNs esta alocando as UNs
       nas posies certas */

    // obtenho a UN que esta na posio que foi removida anteriormente.
    aUn = UN.getUN(un[UN1].getId());
    // comparo se a UN obtida  a mesma que foi criada recentemente.
    assertSame(aUn, unET[0]);
    aUn = UN.getUN(un[UN2].getId());
    assertSame(aUn, unET[1]);
  }

  public TestServerUN(String name) {
    super(name);
  }
}
