/**
 * TestServerUser.java  $Author: cassino $ $Revision: 33587 $ - $Date: 2004/03/17
 * 20:22:51 $
 */
package server.providers;

import csbase.client.ServerMonitor;
import csbase.logic.*;

import util.*;

import java.io.File;
import java.util.ResourceBundle;
import java.util.Vector;

/**
 * A classe <code>TestServerUser</code> representa um caso de teste para os
 * usurios quando o servidor  derrubado e momentos depois  levantado
 * novamente.
 */
public class TestServerUser extends CSBaseAbstractTestCase {
  /** Atributos auxiliares para os testes. */
  private User aUser;
  private int User1;
  private int User2;
  /** Numero de elementos que serao testados. */
  private int COUNT_User = 5;
  private String[] userNames;
  private String[] userMail;
  private Object userId1;
  private Object userId2;

  /**
   * Configuraes a serem inicializadas antes de rodar qualquer um dos testes.
   *
   * @throws Exception .
   */
  public void setUp() throws Exception {
    /* indices dos usurios que sero testadas na remoo */
    User1 = 1;
    User2 = 2;
    userNames = new String[COUNT_User];
    userMail = new String[COUNT_User];
    /* inicializa-se todos os usurios */
    initializeUser(COUNT_User);
    /* guarda os atributos dos usurios para serem comparados no futuro. */
    for (int i = 0; i < COUNT_User; i++) {
      userNames[i] = user[i].getName();
      userMail[i] = user[i].getMail();
    }
    /*  apagada o primeiro usurio (User 2) */
    userId1 = user[User1].getId();
    User.deleteUser(user[User1].getId());
    /*  apagada o segundo usurio (User 3) */
    userId2 = user[User2].getId();
    User.deleteUser(user[User2].getId());
  }

  public void tearDown() throws Exception {
    cleanUser(COUNT_User);
  }

  /**
   * Realiza os testes de usurio antes do servidor sofrer shutdown. Sero
   * criados usurios no servidor, e um dos usurios ser apagada.
   *
   * @throws Exception .
   */
  public void testUserBeforeCrash() throws Exception {
    /* certifica-se que o primeiro usurio (User 2) foi apagado */
    aUser = User.getUser(userId1);
    assertNull(aUser);
    /* certifica-se que o segundo usurio (User 3) foi apagado */
    aUser = User.getUser(userId2);
    assertNull(aUser);
    /* certifica-se que as demais esto l */
    for (int i = 0; i < COUNT_User; i++) {
      if ((i != 1) && (i != 2)) {
        aUser = User.getUser(user[i].getId());
        assertNotNull(aUser);
      }
    }
  }

  /**
   * Realiza testes no cliente depois que o servidor for levantado apos um
   * shutdown.
   *
   * @throws Exception .
   */
  public void testUserAfterCrash() throws Exception {
    /* so criados mais 2 usurios */
    int NEW_USER_ARRAY = 2;
    initializeUser(un[0], role[0], 0, NEW_USER_ARRAY);
    User[] userET = new User[NEW_USER_ARRAY];
    for (int i = 0; i < NEW_USER_ARRAY; i++) {
      userET[i] = (User)userList.get(i);
      userET[i] = (User)userList.get(i);
    }
    /* verifica se os usurios recm criados esto no arquivo */
    for (int i = 0; i < NEW_USER_ARRAY; i++) {
      aUser = User.getUser(userET[i].getId());
      assertNotNull(aUser);
    }
    /* verifica se os 3 usurios restantes de quando o servidor ainda no
       tinha sofrido shutdown estao presentes e NAO foram sobrescrevidos */
    for (int i = 0; i < COUNT_User; i++) {
      if ((i != User1) && (i != User2)) {
        aUser = User.getUser(user[i].getId());
        assertNotNull(aUser);
        assertEquals(userNames[i], aUser.getName());
        assertEquals(userMail[i], aUser.getMail());
      }
    }
    /* limpa os dois usurio que foram criados no metodo local */
    cleanUser(0, 2);
  }

  public TestServerUser(String name) {
    super(name);
  }
}
