/**
 * TestUN.java  $Author: cassino $ $Revision: 33587 $ - $Date: 2004/03/17
 * 20:22:54 $
 */
package server.providers;

import csbase.logic.*;

import util.*;

import java.util.Vector;

/**
 * A classe <code>TestUN</code> representa um caso de teste para os servios de
 * manipulao de UN's do Websintesi.
 */
public class TestUN extends CSBaseAbstractTestCase {
  /* TESTES QUE NO PASSARAM:
   * OK testCreateWithNullInfo()
   * OK testCreateWithNullFields()
   * OK testDeleteNull()
   * OK testModifyNullInfo()
   * OK testModifyWithNullFields()
   */
  protected void setUp() throws Exception {
    initializeUN(ELEMENTS);
  }

  protected void tearDown() throws Exception {
    cleanUN(ELEMENTS);
  }

  /**
   * Testa a construo de UN. Caso a UN a ser construda seja existente, ento
   * ser apagada e ento ocorrer uma nova tentativa de insero.
   *
   * @throws Exception .
   */
  public void testCreateUN() throws Exception {
    for (int i = 0; i < ELEMENTS; i++) {
      assertNotNull(un[i]);
      assertEquals(unInfo[i].name, un[i].getName());
      assertEquals(unInfo[i].desc, un[i].getDesc());
    }
  }

  /**
   * Verifica se o objeto foi criado no arquivo
   *
   * @throws Exception .
   */
  public void testCreateUNFile() throws Exception {
    Vector unList = UN.getAllUNs();
    assertEquals(ELEMENTS, unList.size());
    for (int i = 0; i < ELEMENTS; i++) {
      assertTrue(unList.contains(un[i]));
    }
  }

  /**
   * Verifica se o sistema aceita criar uma nova UN com o mesmo nome.
   *
   * @throws Exception .
   */
  public void testCreateSameUN() throws Exception {
    UN aUN = UN.createUN(unInfo[0]);
    assertNull(aUN);
  }

  /**
   * Testa a remoo das UNs.
   *
   * @throws Exception .
   */
  public void testDeleteAllUNs() throws Exception {
    cleanUN(ELEMENTS);
    Vector allUNs = UN.getAllUNs();
    assertEquals(0, allUNs.size());
  }

  /**
   * Testa se s as UNs pares foram apagadas depois, tenta recriar as UNs pares
   * apagadas
   *
   * @throws Exception .
   */
  public void testDeleteEvenUNs() throws Exception {
    /* Apago todas as UNs pares */
    for (int i = 0; i < ELEMENTS; i++) {
      if ((i % 2) == 0) {
        UN.deleteUN(un[i].getId());
        un[i] = null;
      }
    }
    /* Verifico se os pares foram apagados e se os mpares permanecem */
    Vector unList = UN.getAllUNs();
    int remainingElements = ELEMENTS / 2;
    assertEquals(remainingElements, unList.size());
    for (int i = 0; i < ELEMENTS; i++) {
      if ((i % 2) == 1) {
        assertTrue(unList.contains(un[i]));
      }
    }
    /* Recrio os pares recm-apagados */
    for (int i = 0; i < ELEMENTS; i++) {
      if ((i % 2) == 0) {
        un[i] = UN.createUN(unInfo[i]);
      }
    }
    /* Verifico se todos esto presentes */
    unList = UN.getAllUNs();
    assertEquals(ELEMENTS, unList.size());
    for (int i = 0; i < ELEMENTS; i++) {
      assertTrue(unList.contains(un[i]));
    }
  }

  /**
   * Tento passar um unInfo nulo para o mtodo de criao
   *
   * @throws Exception .
   */
  public void testCreateWithNullInfo() throws Exception {
    UN aUN = null;
    try {
      aUN = UN.createUN(null);
      fail("Esperada Exception");
    }
    catch (Exception e) {}
    assertNull(aUN);
  }

  /*
   * Tento passar um unInfo que contm o campo-chave nulo/vazio
   */
  public void testCreateWithNullFields() throws Exception {
    UNInfo aUNInfo = new UNInfo(null, "blah");
    UN aUN = null;
    try {
      aUN = UN.createUN(aUNInfo);
      fail("Esperada Exception");
    }
    catch (Exception e) {}
    assertNull(aUN);
    UNInfo anotherUNInfo = new UNInfo("", "blah");
    try {
      aUN = UN.createUN(aUNInfo);
      fail("Esperada Exception");
    }
    catch (Exception e) {}
    assertNull(aUN);
  }

  /*
   * Tento apagar uma UN que no existe
   */
  public void testDeleteNonExistentUN() throws Exception {
    Vector unList = UN.getAllUNs();
    int listSize = unList.size();
    UN.deleteUN(new Integer(3000));
    unList = UN.getAllUNs();
    assertEquals(listSize, unList.size());
  }

  /*
   * Passo um id nulo pro Delete
   */
  public void testDeleteNull() throws Exception {
    try {
      UN.deleteUN(null);
      fail("Esperada Exception");
    }
    catch (Exception e) {}
  }

  /*
   * Tento modificar o nome (no pode ser modificado)
   */
  public void testModifyName() throws Exception {
    UN aUN = UN.modifyUN(un[1].getId(), unInfo[0]);
    assertNull(aUN);
    UN anotherUN = UN.getUN(un[1].getId());
    assertEquals("1", anotherUN.getName());
  }

  /*
   * Tento passar um identificador no existente
   */
  public void testModifyNonExistentUN() throws Exception {
    UN aUN = UN.modifyUN(new Integer(3000), unInfo[0]);
    assertNull(aUN);
    UN anotherUN = UN.getUN(new Integer(3000));
    assertNull(anotherUN);
  }

  /*
   * Tento passar um identificador nulo
   *
       public void testModifyNull() throws Exception {
         UNInfo aUNInfo = new UNInfo( "alo", "voce" );
  
         UN aUN = null;
         try {
           aUN = UN.modifyUN( null, aUNInfo );
           fail("Esperada Exception");
         } catch( Exception e ) { }
         assertNull( aUN );
  
       }*/
  /*
   * Tento passar um unInfo nulo
   */
  public void testModifyNullInfo() throws Exception {
    UN aUN = null;
    try {
      aUN = UN.modifyUN(un[0].getId(), null);
      fail("Esperada Exception");
    }
    catch (Exception e) {}
    assertNull(aUN);
  }

  /*
   * Tento passar um unInfo que contm campos nulos
   *
       public void testModifyWithNullFields() throws Exception {
         UNInfo aUNInfo = new UNInfo( null, "bah" );
         UN aUN = null;
         try {
           aUN = UN.modifyUN( un[0].getId(), aUNInfo );
           fail("Esperada Exception");
         } catch( Exception e ) { }
  
         assertNull( aUN );
         assertEquals( "0", un[0].getName() );
         assertEquals( "0", un[0].getDesc() );
  
       }*/
  public void testGetUN() throws Exception {
    for (int i = 0; i < ELEMENTS; i++) {
      assertEquals(un[i], UN.getUN(un[i].getId()));
    }
  }

  public void testGetUNByName() throws Exception {
    for (int i = 0; i < ELEMENTS; i++) {
      assertEquals(un[i], UN.getUNByName(nameUN[i]));
    }
  }

  /**
   * Testa se o vetor recuperado de UNs corresponde ao esperado.
   *
   * @throws Exception .
   */
  public void testGetAllUNs() throws Exception {
    Vector allUNs = UN.getAllUNs();
    assertEquals(ELEMENTS, allUNs.size());
    for (int i = 0; i < ELEMENTS; i++) {
      assertTrue(allUNs.contains(un[i]));
    }
  }

  /**
   * Testa a modificao de uma UN existente no arquivo de dados.
   *
   * @throws Exception .
   */
  public void testModifyUN() throws Exception {
    String name1 = "blah";
    String name2 = "MICO2000";
    String desc1 = "carrapato";
    String desc2 = "ZYB4743";
    UNInfo aUNInfo = new UNInfo(name1, desc1);
    UNInfo anotherUNInfo = new UNInfo(name2, desc2);
    UN aUN = UN.modifyUN(un[0].getId(), aUNInfo);
    UN anotherUN = UN.modifyUN(un[1].getId(), anotherUNInfo);
    assertNotNull(aUN);
    assertNotNull(anotherUN);
    un[0] = UN.getUN(un[0].getId());
    un[1] = UN.getUN(un[1].getId());
    assertEquals(name1, un[0].getName());
    assertEquals(desc1, un[0].getDesc());
    assertEquals(name2, un[1].getName());
    assertEquals(desc2, un[1].getDesc());
  }

  /**
   * Testa passando null pro GetUN
   *
   * @throws Exception .
   */
  public void testGetUNNull() throws Exception {
    UN aUN = UN.getUN(null);
    assertNull(aUN);
  }
}
