/**
 * TestUser.java $Author: cassino $ $Revision: 33587 $ - $Date: 2004/03/17
 * 20:23:02 $
 */
package server.providers;

import csbase.logic.*;

import junit.framework.TestSuite;

import util.*;

import java.util.Vector;

/**
 * A classe <code>TestUser</code> representa um caso de teste para os  servios
 * de manupulao de User do csbase.
 */
public class TestUser extends CSBaseAbstractTestCase implements DependentTest {
  /** Nmero minimo de usurios necessrios para testar o servidor */
  private int USERS_NUMBER = 5;
  /** Numero de usurios que sero removidos para testar o servidor */
  private int USERS_TO_REMOVE = 2;

  public void setUp() throws Exception {
    initializeUser(ELEMENTS);
  }

  public void tearDown() throws Exception {
    cleanUser(ELEMENTS);
  }

  /**
   * Testa se o objeto criado contm todos os valores informados pelo UserInfo
   *
   * @throws Exception .
   */
  public void testCreateUser() throws Exception {
    for (int i = 0; i < ELEMENTS; i++) {
      assertEquals(userInfo[i].name, user[i].getName());
      assertEquals(userInfo[i].login, user[i].getLogin());
      assertEquals(userInfo[i].mail, user[i].getMail());
      assertEquals(userInfo[i].passwordDigest, user[i].getPasswordDigest());
      assertEquals(un[i], user[i].getUN());
      assertEquals(role[i], user[i].getRole());
    }
  }

  /**
   * Verifica se o objeto foi criado no arquivo
   *
   * @throws Exception .
   */
  public void testCreateUserFile() throws Exception {
    Vector userList = User.getAllUsers();
    /* OBS: Usurio ADMIN  sempre criado ao iniciar o servidor,
     * por isso o nmero de elementos  sempre >= 1
     */
    assertEquals(ELEMENTS + 1, userList.size());
    for (int i = 0; i < ELEMENTS; i++) {
      assertTrue(userList.contains(user[i]));
    }
  }

  /**
   * Verifica se o sistema aceita criar um novo usurio com o mesmo login.
   *
   * @throws Exception .
   */
  public void testCreateSameUser() throws Exception {
    User user = User.createUser(userInfo[0]);
    assertNull(user);
  }

  /**
   * Verificar se todos os usurios foram apagados
   *
   * @throws Exception .
   */
  public void testDeleteAllUsers() throws Exception {
    cleanUser(ELEMENTS);
    Vector userList = User.getAllUsers();
    assertEquals(1, userList.size());
  }

  /**
   * Testa se s os usurios pares foram apagados depois, tenta recriar os
   * usurios pares apagados
   *
   * @throws Exception .
   */
  public void testDeleteEvenUsers() throws Exception {
    /* Apago todos os usurios pares */
    for (int i = 0; i < ELEMENTS; i++) {
      if ((i % 2) == 0) {
        User.deleteUser(user[i].getId());
        user[i] = null;
      }
    }
    /* Verifico se os pares foram apagados e se os mpares permanecem */
    Vector userList = User.getAllUsers();
    int remainingElements = (ELEMENTS / 2) + 1;
    assertEquals(remainingElements, userList.size());
    for (int i = 0; i < ELEMENTS; i++) {
      if ((i % 2) == 1) {
        assertTrue(userList.contains(user[i]));
      }
    }
    /* Recrio os pares recm-apagados */
    for (int i = 0; i < ELEMENTS; i++) {
      if ((i % 2) == 0) {
        user[i] = User.createUser(userInfo[i]);
      }
    }
    /* Verifico se todos esto presentes */
    userList = User.getAllUsers();
    assertEquals(ELEMENTS + 1, userList.size());
    for (int i = 0; i < ELEMENTS; i++) {
      assertTrue(userList.contains(user[i]));
    }
  }

  /**
   * Tento passar um userInfo nulo para o mtodo de criao public void
   * testCreateWithNullInfo() throws Exception { User aUser = null; try {
   * aUser = User.createUser( null ); fail("Esperada Exception"); } catch(
   * Exception e ) {} assertNull( aUser ); }
   *
   * @throws Exception .
   */
  /**
   * Tento passar um userInfo que contm campos nulos public void
   * testCreateWithNullFields() throws Exception { UserInfo aUserInfo = new
   * UserInfo( null, null, null, null, null ); User aUser = null; try { aUser
   * = User.createUser( aUserInfo ); fail("Esperada Exception"); } catch(
   * Exception e ) {} assertNull( aUser ); }
   *
   * @throws Exception .
   */
  /**
   * Tento apagar um usurio que no existe
   *
   * @throws Exception .
   */
  public void testDeleteNonExistentUser() throws Exception {
    Vector userList = User.getAllUsers();
    int listSize = userList.size();
    User.deleteUser(new Integer(3000));
    userList = User.getAllUsers();
    assertEquals(listSize, userList.size());
  }

  /**
   * Passo um id nulo pro Delete
   *
   * @throws Exception .
   */
  public void testDeleteNull() throws Exception {
    try {
      User.deleteUser(null);
      fail("Esperada Exception");
    }
    catch (Exception e) {}
  }

  /**
   * Testa a obteno de um usurio a partir de seu respectivo login passado.
   *
   * @throws Exception .
   */
  public void testGetUserByLogin() throws Exception {
    for (int i = 0; i < ELEMENTS; i++) {
      assertSame(user[i], (User)User.getUserByLogin(user[i].getLogin()));
    }
  }

  /**
   * Tento modificar algumas informaes de um usurio
   *
   * @throws Exception .
   */
  public void testModifyUser() throws Exception {
    /* para ser usado como auxilio em alguns casos de modificacao */
    String newMail = "teste@tecgraf.puc-rio.br";
    String newName = "Asdrubal";
    User aUser;
    /* Para todos os testes: primeiro verifico se foi modificado na memria,
     * depois verifico se foi modificado no arquivo
     */
    /* ------ */
    if (ELEMENTS <= 0) {
      // lancar excecao
    }
    if (ELEMENTS == 1) {
      modifySingleElement(0);
    }
    else if ((ELEMENTS % 2) == 0) {
      int j = ELEMENTS - 1;
      for (int i = 0; i <= (ELEMENTS / 2); i++) {
        userInfo[i].mail = userInfo[j].mail;
        userInfo[j].name = userInfo[i].name;
        modifyCoupleElements(i, j);
        j--;
      }
    }
    else {
      int j = ELEMENTS - 1;
      for (int i = 0; i < (ELEMENTS / 2); i++) {
        userInfo[i].mail = userInfo[j].mail;
        userInfo[j].name = userInfo[i].name;
        modifyCoupleElements(i, j);
        j--;
      }
      j = ELEMENTS / 2;
      modifySingleElement(j);
    }
  }

  /**
   * Modifica um nico elemento, e testa a modificao na memria e no arquivo.
   *
   * @param i ndice do elemento a ser modificado.
   *
   * @throws Exception .
   */
  private void modifySingleElement(int i) throws Exception {
    String newMail = "teste@tecgraf.puc-rio.br";
    userInfo[0].mail = newMail;
    User aUser = User.modifyUser(user[i].getId(), userInfo[i]);
    assertEquals(newMail, aUser.getMail());
    User anotherUser = User.getUser(user[i].getId());
    assertEquals(newMail, anotherUser.getMail());
  }

  /**
   * Modifica um par de elementos, e testa a modificao na memria e no
   * arquivo.
   *
   * @param i ndice do primeiro elemento a ser modificado e testado.
   * @param j ndice do segundo elemento a ser modificado e testado.
   *
   * @throws Exception .
   */
  private void modifyCoupleElements(int i, int j) throws Exception {
    User aUser = User.modifyUser(user[i].getId(), userInfo[i]);
    assertEquals(userInfo[j].mail, aUser.getMail());
    User anotherUser = User.getUser(user[i].getId());
    assertEquals(userInfo[j].mail, anotherUser.getMail());
    aUser = User.modifyUser(user[j].getId(), userInfo[j]);
    assertEquals(userInfo[i].name, aUser.getName());
    anotherUser = User.getUser(user[j].getId());
    assertEquals(userInfo[i].name, anotherUser.getName());
  }

  /**
   * Tento modificar o login (no pode ser modificado)
   *
   * @throws Exception .
   */
  public void testModifyLogin() throws Exception {
    String newLogin = "jasper";
    userInfo[1].login = newLogin;
    User aUser = User.modifyUser(user[1].getId(), userInfo[1]);
    assertNull(aUser);
    User anotherUser = User.getUser(user[1].getId());
    assertEquals("1", anotherUser.getLogin());
  }

  /**
   * Tento passar um identificador no existente
   *
   * @throws Exception .
   */
  public void testModifyNonExistentUser() throws Exception {
    User aUser = User.modifyUser(new Integer(3000), userInfo[0]);
    assertNull(aUser);
    User anotherUser = User.getUser(new Integer(3000));
    assertNull(anotherUser);
  }

  /**
   * Tento passar um identificador nulo public void testModifyNull() throws
   * Exception { User aUser = null; try { aUser = User.modifyUser( null,
   * userInfo[0] ); fail("Esperada Exception"); } catch( Exception e ) {}
   * assertNull( aUser );  }
   *
   * @throws Exception .
   */
  /**
   * Tento passar um userInfo nulo public void testModifyNullInfo() throws
   * Exception { User aUser = null; try { aUser = User.modifyUser(
   * user[0].getId(), null ); fail("Esperada Exception"); } catch( Exception e
   * ) {} assertNull( aUser );  }
   *
   * @throws Exception .
   */
  /**
   * Tento passar um userInfo que contm campos nulos public void
   * testModifyWithNullFields() throws Exception { UserInfo aUserInfo = new
   * UserInfo( null, null, null, null, null ); User aUser = null; try { aUser
   * = User.modifyUser( user[0].getId(), aUserInfo ); fail("Esperada
   * Exception"); } catch( Exception e ) {} assertNull( aUser ); assertEquals(
   * "0", user[0].getName() ); assertEquals( "0", user[0].getLogin() );
   * assertEquals( "0", user[0].getMail() );     }
   *
   * @throws Exception .
   */
  /**
   * Testa a obteno de um usurio a partir de seu respectivo id passado.
   *
   * @throws Exception .
   */
  public void testGetUser() throws Exception {
    for (int i = 0; i < ELEMENTS; i++) {
      assertSame(user[i], (User)User.getUser(user[i].getId()));
    }
  }

  /**
   * Testa passando null pro GetUser public void testGetUserNull() throws
   * Exception { User aUser = null; try { aUser = User.getUser( null );
   * fail("Esperada Exception"); } catch( Exception e ) {} assertNull( aUser
   * ); }
   *
   * @throws Exception .
   */
  /**
   * Comparo os mtodos que retornam colees de usurios
   *
   * @throws Exception .
   */
  public void testGetAll() throws Exception {
    Vector outlineList = User.getAllOutlines();
    Vector userList = User.getAllUsers();
    assertEquals(outlineList.size(), userList.size());
    /* Apago todos os usurios pares */
    for (int i = 0; i < ELEMENTS; i++) {
      if ((i % 2) == 0) {
        User.deleteUser(user[i].getId());
        user[i] = null;
      }
    }
    outlineList = User.getAllOutlines();
    userList = User.getAllUsers();
    assertEquals(outlineList.size(), userList.size());
    /* Recrio os pares recm-apagados */
    for (int i = 0; i < ELEMENTS; i++) {
      if ((i % 2) == 0) {
        user[i] = User.createUser(userInfo[i]);
      }
    }
    outlineList = User.getAllOutlines();
    userList = User.getAllUsers();
    assertEquals(outlineList.size(), userList.size());
  }

  /**
   * Gera um nome para ser usado na insero de um objeto que no pode sofrer
   * coliso com outros testes.
   *
   * @return String O nome do objeto com no maximo 20 caracteres.
   */
  private String generateTestName() {
    String[] name = this.getClass().getName().split("\\.");
    int pos = name.length - 1;
    return (name[pos].length() > 20) ? name[pos].substring(0, 19) : name[pos];
  }

  /**
   * Atribui valores antes do servidor cair para averiguar se estes valores
   * persistem quando o servidor cai e levanta novamente.
   *
   * @throws Exception .
   */
  public void beforeCrashValues() throws Exception {
    int indice = getFirstIndexToRemove();
    if (indice == -1) {
      fail(
        "Nmero de ndices a serem removidos  maior do que o nmero de " +
        "elementos disponiveis.");
    }

    // Associa o nome do usurio ao nome da classe para evitar coliso com
    // os outros testes.
    // String userName = "test";/*this.getClass().getName();NO*/
    String userName = generateTestName();
    for (int i = 0; i < USERS_NUMBER; i++) {
      User newUser = createTestUser(userName + i);
      assertNotNull(newUser);
    }

    // um ou mais usurios so apagados
    for (int i = indice; i < (indice + (USERS_TO_REMOVE - 1)); i++) {
      cleanUser(userName + i);
      cleanUN(userName + i);
      cleanRole(userName + i);
    }
  }

  /**
   * Obtm o primeiro indice que deve ser apagado.
   *
   * @return int O indice que deve ser apagado, caso o numero de elementos  a
   *         ser apagado for maior do que o numero de elementos disponiveis 
   *         retornado -1.
   */
  private int getFirstIndexToRemove() {
    int diff = USERS_NUMBER - USERS_TO_REMOVE;
    return (diff < 1) ? (-1) : (diff / 2);
  }

  /**
   * Realiza testes depois que o servidor caiu e foi levantado novamente.
   *
   * @throws Exception .
   */
  public void afterCrashValues() throws Exception {
    String userName = generateTestName();
    String userNameAf = userName + "after";
    for (int i = 0; i < USERS_NUMBER; i++) {
      User newUser = createTestUser(userNameAf + i);
      assertNotNull(newUser);
    }

    // verifica se os outros usurios nao foram sobrescritos.
    int usersIn = 0;
    for (int i = 0; i < USERS_NUMBER; i++) {
      User loadedUser = (User)User.getUserByLogin(userNameAf + i);
      assertNotNull(loadedUser);
      loadedUser = (User)User.getUserByLogin(userName + i);
      if (loadedUser != null) {
        usersIn++;
      }
    }
    assertEquals(USERS_NUMBER - USERS_TO_REMOVE + 1, usersIn);
  }

  /**
   * Implementa a interface <code>DependentTests</code>.  o mtodo responsvel
   * pelos testes que sero executados antes do servidor ser derrubado.
   *
   * @param suite .
   */
  public void callTestsBeforeCrash(TestSuite suite) {
    suite.addTest(new TestUser("beforeCrashValues"));
  }

  /**
   * Implementa a interface <code>DependentTests</code>.  o mtodo responsvel
   * pelos testes que sero executados para comparar o estado dos objetos no
   * servidor aps este ter sofrido shutdown.
   *
   * @param suite .
   */
  public void callTestsAfterCrash(TestSuite suite) {
    suite.addTest(new TestUser("afterCrashValues"));
  }

  /*
   * PARA TESTAR:
   *          getAllUpdates()
   *          isLocked()
   *          lock(TransactionCallBackInterface cb)
   *          unlock(TransactionCallBackInterface cb)
   */
  /* TESTES QUE NO PASSARAM:
   * OK testCreateWithNullInfo()
   * OK testCreateWithNullFields()
   * OK testDeleteNull()
   * OK testModifyNull()
   * OK testModifyNullInfo()
   * OK testModifyWithNullFields()
   * OK testGetUserNull()
   */
  public TestUser(String test) {
    super(test);
  }

  public TestUser() {}
}
