/**
 * TestLoadPerformance.java
 * 
 * $Author: taciana $
 * $Revision: 33579 $ - $Date: 2004-04-07 15:42:20 -0300 (Wed, 07 Apr 2004) $
 */

package server.providers;


import java.util.Vector;
import java.util.ResourceBundle;
import java.util.Date;
import java.rmi.RemoteException;
import java.io.File;
import java.nio.ByteBuffer;

import util.*;
import csbase.logic.*;
import csbase.client.util.DateTranslator;


/**
 * A classe <code>TestLoadPerformance</code> representa um caso de teste de  
 * sobrecarga no servidor. 
 * @see CSBaseAbstractTestCase
 */
public class TestLoadPerformance extends CSBaseAbstractTestCase {

  /** Nome do arquivo de propriedades que o ResourceBundle ir ler */
  private static final String RES_FILENAME = "load_test";

  private ResourceBundle prop;

  /* atributos a serem lidos do arquivo de propriedades */
  public static final String N_USERS    = "nUsers";
  public static final String N_PROJECTS = "nProjects";
  public static final String N_FILES    = "nFiles";
  public static final String N_SIZE     = "nSize";
  public static final String UN_NAME    = "unName";
  public static final String ROLE_NAME  = "roleName";

  /* a ser lido da varivel de systema preffix. */
  public static final String PREFFIX_NAME_PROP = "preffix";

  /* String usada para gerar o contedo do arquivo a ser escrito */
  private static final String FILE_GEN = "*";

  /** Un associada aos usurios criados */
  private UN un;

  /** Perfil associado aos usurios criados */
  private Role role;  

  private int userCount;
  private int projectCount;
  private int fileCount;
  private int fileSize;
  private String unName;
  private String roleName;

  private int suffix = -1;

  private String preffix;

  private String fileContents;
  
  private User[] users;   
  private CommonClientProject[] projects;
  private ClientProjectFile[] files;     
  

  /**
   * setUp realiza as seguintes inseres:
   * - Cria usurios (n), uns, roles, plats
   *    |_ Cria projetos (n)
   *       |_ Cria N arquivos por projeto.
   */
  public void setUp() throws Exception {
  
     /* carrega as variveis de ambiente para prefixo do nome
     e as propriedades do arquivo */ 
     loadSystemVariables();
     loadProperties();

     // Cria uns e perfis
     un   = createTestUN( preffix + unName );
     role = createTestRole( preffix + roleName );

     assertNotNull( un );
     assertNotNull( role );

     printAndLog("UN " + un.getName() + " criada.");
     printAndLog("Perfil " + role.getName() + " criado.");

     /* gera a String que ser usada para gerar um arquivo regular */
//System.out.println("Antes de gerar arquivo de : " + fileSize );     
     fileContents = createFileContents( fileSize );
//System.out.println("arquivo gerado: " + fileContents );     
     
     users    = new User[ userCount ];
     projects = new CommonClientProject[ projectCount ];
     files    = new ClientProjectFile[ fileCount ];

     // --------- Cria usurios
     for( int i=0; i<userCount; i++ ) {
        suffix++;
        String strKey = preffix + suffix;
        users[i] = createTestUser( strKey, un, role );
        assertNotNull( users[i] );
        printAndLog("Usurio " + users[i].getLogin() + " criado.");

        // --------- Cria projetos
        for( int p=0; p<projectCount; p++ ) {
           String projectName = strKey + "_" + p;                
           projects[p] = createTestProject(projectName, users[i]);
           assertNotNull( projects[p] );
           printAndLog("Projeto " + projectName + " criado.");

           // --------- Cria arquivos em projetos. 
           for( int f=0; f<fileCount; f++ ) {
              String name = strKey + "_" + f;

              files[f] =  createProjectFile( projects[p],
                                                name );
                                                
              // certifica se o arquivo  realmente criado.
              assertNotNull( files[f] );
              printAndLog("Arquivo " + name + " criado.");
           }
        }
     }
  }
  
  public void tearDown() throws Exception {
     String strKey;
     for( int i=0; i< users.length; i++ ) {
        strKey = users[i].getLogin();

        // remove os projetos
        for( int p=0; p < projectCount; p++ ) {
           String projectName = strKey + "_" + p;

           // abro o projeto 
           CommonClientProject project = 
               CommonClientProject.openProject( users[i].getId(), projectName );

           ClientProjectFile root = (ClientProjectFile)project.getTree();
           ProjectFile[] files = root.getChildren();
           
           assertEquals( files.length, fileCount );

           for( int f=0; f < files.length  ; f++ ) { 
              // remove o arquivo
              if( !files[f].isDirectory() ) {
                 printAndLog("Arquivo " + files[f].toString() + " removido.");
                 files[f].remove();
              }
           }
           assertFileExists( root.getAbsolutePath() );
           
           // remove o projeto
           CommonClientProject.remove( users[i].getId(), projectName );
           printAndLog("Projeto " + projectName + " removido.");
           
        }

        //remove o usurio
        printAndLog("Usurio " + users[i].getLogin() + " removido.");
        User.deleteUser( users[i].getId() );
     }
     cleanUN( un.getName() );
     cleanRole( role.getName() );
  }

  private String createFileContents( long size ) {
 
printAndLog("entrei " );
     int max_iteration = 10000;
     String contents   = "";
     
     if( size <= max_iteration ) {
        for( int i=0; i < size; i++ ) {
           contents += FILE_GEN;
        }
        return contents;
     }
     
     long tam = size/max_iteration;
printAndLog("achei tam de " + tam  );

     if( tam > max_iteration ) {
        fail("Tamanho de arquivo muito grande" );
     }
     String bigFile_gen = "";
printAndLog("vou loopar " + tam  );
     for( long i=0; i < tam; i++ ) {
        bigFile_gen += FILE_GEN;
     }
printAndLog("vou loopar o 2o " + max_iteration  );
     for( long i=0; i < max_iteration; i++ ) {
printAndLog("i e max  " + i + " " + max_iteration  );
        contents += bigFile_gen;
     }
     if( (size%max_iteration) != 0 ) {
        long floats = tam * ( size % max_iteration );
        for( long i=0; i < floats; i++ ) {
           contents += FILE_GEN;
        }
     }
     return contents;        
  }  


  public void testPerformance() throws Exception {
     String strKey;
  
     // testa se todos os usurios foram criados
     for( int i=0; i < users.length; i++ ) {
        strKey = users[i].getLogin(); 
        assertSame( users[i], User.getUserByLogin( strKey ) );
        printAndLog("Usurio " + strKey + " est OK.");
        
        // testa a criao dos projetos do usurio
        for( int p=0; p<projectCount; p++ ) {
           String projectName = strKey + "_" + p;
           
           // abro o projeto 
           CommonClientProject project = 
               CommonClientProject.openProject( users[i].getId(), projectName );

           // asseguro que o projeto obtido  de fato o projeto procurado.
           assertNotNull( project );
           assertEquals( project.getName(), projectName );
           printAndLog("Projeto " + projectName + " est OK.");

           // verifico os arquivos criados
           ClientProjectFile root = (ClientProjectFile)project.getTree();

           ProjectFile[] files = root.getChildren();

           for( int f=0; f<files.length; f++ ) {
              if ( compareFileContents( files[f] ) ) {
                 printAndLog("Arquivo " + files[f].toString() + "est OK.");
              }
           }
           project.close();
        }
     }
  }   


 /**
  * Compara o contedo do arquivo de projeto com o contedo esperado.
  * @param projectFile O arquivo de projeto a ser comparado.
  * @return boolean true se o contedo for igual, false seno.
  */
  private boolean compareFileContents( ProjectFile projectFile ) {
     ClientProjectFile file = (ClientProjectFile)projectFile;
     try {
        // tenta abrir o arquivo
        if( !file.open() ) return false;
        
        ByteBuffer buffer = ByteBuffer.allocate( fileContents.length() );
        // l o contedo do arquivo
        int size = file.read( buffer, 0 );
        byte[] bytes = buffer.array();

        // compara com o contedo local
        if ( !fileContents.equals( new String(bytes) ) ) {
           return false;
        }
        // fecha o arquivo
        file.close();
     }
     catch ( Exception e ) {
        return false;
     }
     return true;
  }




 /**
  * Cria um arquivo de projeto contento caracteres de preenchimento, de 
  * acordo com o tamanho de arquivo especificado.
  * @param project Projeto no qual o arquivo ser criado.
  * @param filename Nome do arquivo.
  * @return ClientProjectFile O arquivo criado dentro do projeto.
  */
  private ClientProjectFile createProjectFile( 
                                     CommonClientProject project,
                                     String filename ) {
     ClientProjectFile file;
     
     String absolutePath = project.getTree().getAbsolutePath() +
                           File.separator + filename;
                           
     try {                           
        file = (ClientProjectFile)project.createAndWaitForFile(
                                       project.getTree(), filename, "TEXT", 
                                       "admin" );

                                   
        if ( file == null ) return null;                                         

        // abre arquivo para escrita
        if ( !file.open() ) return null;

        byte[] bytes = fileContents.getBytes();
        ByteBuffer bBuffer = ByteBuffer.wrap( bytes );
        
        // escreve no arquivo 

        // sleep time para a crio de um arquivo e outro.
        try{ Thread.sleep( 90 ); } catch( Exception e ) {}
        

        for( int i=0; i<bBuffer.capacity(); i+=file.write(bBuffer, i) );
           
        file.truncate( fileSize );
        
         // int sizeCreated = file.write( bBuffer, 0 );
        file.close();
     }
     catch (Exception e ) {
        e.printStackTrace();
        return null;
     }
     return file;
  }

 /**
  * Obtm as propriedades do arquivo.
  */
  private void loadProperties() {
     prop = getTestBundle( RES_FILENAME );
     try {
        userCount    = Integer.parseInt( prop.getString( N_USERS ) );
        projectCount = Integer.parseInt( prop.getString( N_PROJECTS ) );
        fileCount    = Integer.parseInt( prop.getString( N_FILES ) );
        fileSize     = Integer.parseInt( prop.getString( N_SIZE ) );
        unName       = prop.getString( UN_NAME );
        roleName     = prop.getString( ROLE_NAME );
     } catch( Exception e ) {
        e.printStackTrace();
        fail("Falha ao ler propriedades de arquivo" );
     }
     
  }

 /**
  * Obtm as variveis do sistema.
  */
  private void loadSystemVariables() {
     String preffix = System.getProperty( PREFFIX_NAME_PROP );
     if( preffix == null ) {
        fail("Prefixo no encontrado");
     }
     this.preffix = preffix;
  }

 /**
  * Imprime a mensagem junto com a data e hora atual.
  * @param message A mensagem a ser exibida na tela.
  */
  private void printAndLog( String message ) {
    Date date = new Date( System.currentTimeMillis() );
    //String date = DateTranslator.getInstance().translate( time );
    System.out.println( "<" + date + ">: " + message );  
  }
}





