/**
 * TestFtpService.java $Author: cassino $ $Revision: 33587 $ - $Date: 2004/04/07
 * 18:42:18 $
 */
package server.services;

import csbase.logic.*;
import csbase.remote.RemoteLocator;
import csbase.server.services.loginservice.LoginService;

import junit.extensions.*;
import junit.framework.*;

import util.*;

import java.io.File;
import java.rmi.RemoteException;
import java.util.ResourceBundle;

/**
 * A classe <code>TestFtpService</code>  responsvel por testar o servio  de
 * transferncia de arquivos via FTP.
 */
public class TestFtpService extends CSBaseAbstractTestCase {
  /*
   * Testes realizados:
   *
   * testDownloadToProject()                              OK
   * testDownloadWrongUserLogin()                         OK
   * testDownloadWrongUserPassword()                      OK
   * testDownloadWrongFile()                              OK
   */
  private static String projDir = System.getProperty("project_dir");
  /** Usurio que ser usado para os testes. */
  private static User user;
  /** Projeto do usurio que ir receber os arquivos transmitidos. */
  private static CommonClientProject project;

  // nome do arquivo que ser salvo localmente.
  private static String[] localfilename = new String[1];
  /* lidos do arquivo de propriedades ftpservice_test */
  private static ResourceBundle prop = getTestBundle(FTP_PROP);
  private static String login;
  private static String password;
  private static String server;
  private static String filename;
  private static String filetype;
  private static int protocol;

  /**
   *  executado antes dos testes serem rodados. Inicializa usurio e projeto
   * para serem usados nos testes.
   *
   * @throws Exception .
   */
  public void setUp() throws Exception {
    user = createTestUser("usertest");
    project = createTestProject("proj", user);
  }

  /**
   *  executado quando todos os testes terminarem. Limpa o(s) usurio(s) e
   * projeto(s) criados no oneTimeSetUp.
   *
   * @throws Exception .
   */
  public void tearDown() throws Exception {
    CommonClientProject.remove(user, "proj");
    cleanUser("usertest");
    cleanRole("usertest");
    cleanUN("usertest");
  }

  /**
   * Carrega as propriedades do teste.
   */
  public static void loadPropertiesBundle() {
    try {
      login = prop.getString(FTP_PROP_LOGIN);
      password = prop.getString(FTP_PROP_PASSWD);
      server = prop.getString(FTP_PROP_SERVER);
      filename = prop.getString(FTP_PROP_FILENAME);
      filetype = prop.getString(FTP_PROP_FILETYPE);
      protocol = Integer.parseInt(prop.getString(FTP_PROP_PROTOCOL));
    }
    catch (Exception e) {
      System.out.println(
        "Problemas ao ler arquivo de propriedades: " + e.getMessage());
    }
  }

  /**
   * Testa se a conexo e pedido de busca em um FTP funciona corretamente.
   * Considerando que este teste esta passando dados vlidos.
   *
   * @throws ServiceException .
   * @throws RemoteException .
   * @throws Exception .
   */
  public void testDownloadToProject()
    throws ServiceException, RemoteException, Exception {
    /* Download de 2 arquivos */
    for (int i = 0; i < 2; i++) {
      localfilename[0] = "arquivo_" + i;
      boolean isOkay =
        RemoteLocator.ftpService.downloadToProject(
          login, password, server, filename, project.getId(), localfilename,
          filetype, protocol, "usertest");
      assertTrue(isOkay);
      assertTrue(isProjectExists(localfilename[0]));
    }
  }

  /**
   * Tenta a conexao em um servidor ftp passando um login errado.
   *
   * @throws ServiceException .
   * @throws RemoteException .
   * @throws Exception .
   */
  public void testDownloadWrongUserLogin()
    throws ServiceException, RemoteException, Exception {
    localfilename[0] = "wrongUser";
    boolean isOkay =
      RemoteLocator.ftpService.downloadToProject(
        "error", password, server, filename, project.getId(), localfilename,
        filetype, protocol, "usertest");
    assertFalse(isOkay);
  }

  /**
   * Tenta a conexao em um servidor ftp passando um login certo com uma senha
   * errada.
   *
   * @throws ServiceException .
   * @throws RemoteException .
   * @throws Exception .
   */
  public void testDownloadWrongUserPassword()
    throws ServiceException, RemoteException, Exception {
    localfilename[0] = "wrongPass";
    boolean isOkay =
      RemoteLocator.ftpService.downloadToProject(
        login, "error", server, filename, project.getId(), localfilename,
        filetype, protocol, "testuser");
    assertFalse(isOkay);
  }

  /**
   * Tenta buscar um arquivo que nao existe no ftp.
   *
   * @throws ServiceException .
   * @throws RemoteException .
   * @throws Exception .
   */
  public void testDownloadWrongFile()
    throws ServiceException, RemoteException, Exception {
    localfilename[0] = "wrongFile";
    boolean isOkay =
      RemoteLocator.ftpService.downloadToProject(
        login, password, server, "error", project.getId(), localfilename,
        filetype, protocol, "testuser");
    assertFalse(isOkay);
  }

  /**
   * Verifica se o arquivo de um projeto existe ou nao.
   *
   * @param localfilename O nome do arquivo a ser verificado.
   *
   * @return boolean true se o arquivo existe, seno false.
   */
  private boolean isProjectExists(String localfilename) {
    String sep = File.separator;
    File projFile =
      new File(
        projDir + sep + user.getName() + sep + project.getName() + sep +
        localfilename);
    return projFile.exists();
  }

  public TestFtpService() {
    super();
    loadPropertiesBundle();
  }

  public TestFtpService(String name) {
    super(name);
    loadPropertiesBundle();
  }
}
