package util;

import csbase.client.ServerMonitor;
import csbase.client.remote.ProjectObserver;
import csbase.logic.*;
import csbase.remote.RemoteLocator;

import junit.framework.TestCase;

import java.io.*;
import java.rmi.RemoteException;
import java.util.*;

public abstract class CSBaseAbstractTestCase extends TestCase {
  /** nmero de elementos a ser usado nos testes */
  protected static int ELEMENTS;

  // Atributos deverao ir para uma classe que contenha todos os BundleNames.
  public static final String WORKING_DIR = "workingdir";
  public static final String RUNSERVER_COMMAND = "runserver";
  public static final String SERVER = "server";
  public static final String RMI_PORT = "rmiport";
  public static final String TEST_ELEM = "testelements";
  public static final String FTP_PROP = "ftpservice_test";
  public static final String FTP_PROP_LOGIN = "login";
  public static final String FTP_PROP_PASSWD = "password";
  public static final String FTP_PROP_SERVER = "server";
  public static final String FTP_PROP_PROTOCOL = "protocol";
  public static final String FTP_PROP_FILENAME = "filename";
  public static final String FTP_PROP_FILETYPE = "filetype";
  private static ResourceBundle prop = null;
  protected String[] nameUN = null; // Nomes das UNs
  protected String[] descUN = null; // Descries das UNs
  protected String[] nameRole = null; // Nomes dos Perfis
  protected String[] descRole = null; // Descries dos Perfis
  protected String[] namePlatform = null; // Nomes das Plataformas
  protected String[] descPlatform = null; // Descries das Plataformas
  protected String[] nameUser = null; // Nomes dos Usurios
  protected String[] descUser = null; // Descries dos Usurios
  protected String[] loginRole = null; // Logins dos Perfis
  protected String[] passRole = null; // Senhas dos Perfis 
  protected String[] loginUser = null; // Logins dos Usurios
  protected String[] passUser = null; // Senhas dos Usurios 
  protected String[] mail = null; // E-mails dos Usurios
  protected UNInfo[] unInfo = null; // Value Objects de UN
  protected UN[] un = null; // Unidades de negcio
  protected RoleInfo[] roleInfo = null; // Value Objects de Perfis
  protected Role[] role = null; // Perfis
  protected PlatformInfo[] pInfo = null; // Value Objects de UN
  protected Platform[] plat = null; // Unidades de negcio
  protected UserInfo[] userInfo = null; // Value Objects de Usurios
  protected User[] user = null; // Usurios
  protected List userInfoList = new ArrayList();

  // Value Objects de Usurios
  protected List userList = new ArrayList();

  // Lista de Usurios
  protected List loginUserList = new ArrayList();

  // Lista de logins p/Usurios
  protected List passUserList = new ArrayList();

  // Lista de senhas p/Usurios
  protected List nameUserList = new ArrayList();

  // Lista de nomes  p/Usurios
  protected List mailUserList = new ArrayList();

  // Lista de e-mails p/Usurios
  private static File serverWorkingDir; // Diretrio de trabalho para levantar

  // o servidor.
  private static String runServerCommand; // Linha de comando para levantar o

  // servidor.
  private static StartServer thread; // Thread responsvel por levantar o serv.
  /** singleton para ser usado nos servidores, ou infopaed ou websintesi */
  protected static ServerMonitor serverMonitor = null;

  public CSBaseAbstractTestCase(String name) {
    super(name);
  }

  public CSBaseAbstractTestCase() {}

  /**
   * Gera nomes de UNs de amostragem.
   *
   * @param elements .
   */
  protected void initializeNameUN(int elements) {
    nameUN = new String[elements];
    for (int i = 0; i < elements; i++) {
      nameUN[i] = "" + i;
    }
  }

  /**
   * Gera nomes de Perfis de amostragem.
   *
   * @param elements .
   */
  protected void initializeNameRole(int elements) {
    nameRole = new String[elements];
    for (int i = 0; i < elements; i++) {
      nameRole[i] = "" + i;
    }
  }

  /**
   * Gera nomes de Plataformas de amostragem.
   *
   * @param elements .
   */
  protected void initializeNamePlatform(int elements) {
    namePlatform = new String[elements];
    for (int i = 0; i < elements; i++) {
      namePlatform[i] = "" + i;
    }
  }

  /**
   * Gera nomes de Usurios de amostragem.
   *
   * @param elements .
   */
  protected void initializeNameUser(int elements) {
    nameUser = new String[elements];
    for (int i = 0; i < elements; i++) {
      nameUser[i] = "" + i;
    }
  }

  /**
   * Gera descries de UNs de amostragem.
   *
   * @param elements .
   */
  protected void initializeDescriptionUN(int elements) {
    descUN = new String[elements];
    for (int i = 0; i < elements; i++) {
      descUN[i] = "" + i;
    }
  }

  /**
   * Gera descries de Perfis de amostragem.
   *
   * @param elements .
   */
  protected void initializeDescriptionRole(int elements) {
    descRole = new String[elements];
    for (int i = 0; i < elements; i++) {
      descRole[i] = "" + i;
    }
  }

  /**
   * Gera descries de Plataformas de amostragem.
   *
   * @param elements .
   */
  protected void initializeDescriptionPlatform(int elements) {
    descPlatform = new String[elements];
    for (int i = 0; i < elements; i++) {
      descPlatform[i] = "" + i;
    }
  }

  /**
   * Gera descries de Usurios de amostragem.
   *
   * @param elements .
   */
  protected void initializeDescriptionUser(int elements) {
    descUser = new String[elements];
    for (int i = 0; i < elements; i++) {
      descUser[i] = "" + i;
    }
  }

  /**
   * Gera logins de Perfis de amostragem.
   *
   * @param elements .
   */
  protected void initializeLoginRole(int elements) {
    loginRole = new String[elements];
    for (int i = 0; i < elements; i++) {
      loginRole[i] = "" + i;
    }
  }

  /**
   * Gera logins de Usurios de amostragem.
   *
   * @param elements .
   */
  protected void initializeLoginUser(int elements) {
    loginUser = new String[elements];
    for (int i = 0; i < elements; i++) {
      loginUser[i] = "" + i;
    }
  }

  protected void initializeLoginUser(int startIndex, int count) {
    for (int i = startIndex; i < (startIndex + count); i++) {
      loginUserList.add(i, "a" + i);
    }
  }

  /**
   * Gera e-mails de amostragem.
   *
   * @param elements .
   */
  protected void initializeMail(int elements) {
    mail = new String[elements];
    for (int i = 0; i < elements; i++) {
      mail[i] = "" + i;
    }
  }

  /**
   * Gera senhas de Perfis de amostragem.
   *
   * @param elements .
   */
  protected void initializePasswordRole(int elements) {
    passRole = new String[elements];
    for (int i = 0; i < elements; i++) {
      passRole[i] = "" + i;
    }
  }

  /**
   * Gera senhas de Usurios de amostragem.
   *
   * @param elements .
   */
  protected void initializePasswordUser(int elements) {
    passUser = new String[elements];
    for (int i = 0; i < elements; i++) {
      passUser[i] = "" + i;
    }
  }

  /**
   * Gera senhas de Usurios de amostragem.
   *
   * @param startIndex .
   * @param count .
   */
  protected void initializePasswordUser(int startIndex, int count) {
    for (int i = startIndex; i < (startIndex + count); i++) {
      passUserList.add(i, "" + i);
    }
  }

  protected void initializeNameUser(int startIndex, int count) {
    for (int i = startIndex; i < (startIndex + count); i++) {
      nameUserList.add(i, "a" + i);
    }
  }

  protected void initializeMail(int startIndex, int count) {
    for (int i = startIndex; i < (startIndex + count); i++) {
      mailUserList.add(i, "" + i);
    }
  }

  /**
   * Inicializa os Value Objects para criao de UNs.
   *
   * @param elements .
   */
  protected void initializeUNInfo(int elements) {
    initializeNameUN(elements);
    initializeDescriptionUN(elements);
    unInfo = new UNInfo[elements];
    for (int i = 0; i < elements; i++) {
      unInfo[i] = new UNInfo(nameUN[i], descUN[i]);
    }
  }

  /**
   * Inicializa os Value Objects para criao de Perfis.
   *
   * @param elements .
   */
  protected void initializeRoleInfo(int elements) {
    initializeNameRole(elements);
    initializeDescriptionRole(elements);
    initializeLoginRole(elements);
    initializePasswordRole(elements);
    roleInfo = new RoleInfo[elements];
    for (int i = 0; i < elements; i++) {
      //??Alterado para mudana de permission
      System.out.println("??CSBaseAbstractTestCase:269");
      roleInfo[i] = new RoleInfo(nameRole[i], descRole[i], null);
    }
  }

  /**
   * Inicializa os Value Objects para criao de Plataformas.
   *
   * @param elements .
   */
  protected void initializePlatformInfo(int elements) {
    initializeNamePlatform(elements);
    initializeDescriptionPlatform(elements);
    pInfo = new PlatformInfo[elements];
    for (int i = 0; i < elements; i++) {
      pInfo[i] = new PlatformInfo(namePlatform[i], descPlatform[i]);
    }
  }

  /**
   * Inicializa os Value Objects para criao de Usurios.
   *
   * @param elements .
   *
   * @throws Exception .
   */
  protected void initializeUserInfo(int elements) throws Exception {
    initializeLoginUser(elements);
    initializeUN(elements);
    initializeNameUser(elements);
    initializeMail(elements);
    initializeRole(elements);
    initializePasswordUser(elements);
    userInfo = new UserInfo[elements];
    for (int i = 0; i < elements; i++) {
      //??Alterado para mudana de permission
      System.out.println("??CSBaseAbstractTestCase:299");
      userInfo[i] =
        new UserInfo(
          loginUser[i], un[i].getId(), nameUser[i], mail[i], null, null);
      userInfo[i].passwordDigest = passUser[i];
    }
  }

  protected void initializeUserInfo(
    UN theUn, Role theRole, int startIndex, int count)
    throws Exception {
    initializeLoginUser(startIndex, count);
    initializeNameUser(startIndex, count);
    initializeMail(startIndex, count);
    initializePasswordUser(startIndex, count);
    for (int i = startIndex; i < (startIndex + count); i++) {
      assertNotNull(loginUserList.get(i));
      assertNotNull(theUn);
      assertNotNull(theRole);
      assertNotNull(nameUserList.get(i));
      assertNotNull(mailUserList.get(i));
      //??Alterado para mudana de permission
      System.out.println("??CSBaseAbstractTestCase:323");
      UserInfo aUserInfo =
        new UserInfo(
          (String)loginUserList.get(i), theUn.getId(),
          (String)nameUserList.get(i), (String)mailUserList.get(i), null, null);
      aUserInfo.passwordDigest = (String)passUserList.get(i);
      userInfoList.add(i, aUserInfo);
    }
  }

  /**
   * Cria Unidades de Negcio, utilizando os VOs criados anteriormente.
   *
   * @param elements .
   *
   * @throws Exception .
   */
  protected void initializeUN(int elements) throws Exception {
    initializeUNInfo(elements);
    un = new UN[elements];
    for (int i = 0; i < elements; i++) {
      un[i] = UN.createUN(unInfo[i]);
    }
  }

  /**
   * Cria Perfis, utilizando os VOs criados anteriormente.
   *
   * @param elements .
   *
   * @throws Exception .
   */
  protected void initializeRole(int elements) throws Exception {
    initializeRoleInfo(elements);
    role = new Role[elements];
    for (int i = 0; i < elements; i++) {
      role[i] = Role.createRole(roleInfo[i]);
    }
  }

  /**
   * Cria Plataformas, utilizando os VOs criados anteriormente.
   *
   * @param elements .
   *
   * @throws Exception .
   */
  protected void initializePlatform(int elements) throws Exception {
    initializePlatformInfo(elements);
    plat = new Platform[elements];
    for (int i = 0; i < elements; i++) {
      plat[i] = Platform.createPlatform(pInfo[i]);
    }
  }

  /**
   * Cria Usurios, utilizando os VOs criados anteriormente.
   *
   * @param elements .
   *
   * @throws Exception .
   */
  protected void initializeUser(int elements) throws Exception {
    initializeUserInfo(elements);
    user = new User[elements];
    for (int i = 0; i < elements; i++) {
      user[i] = User.createUser(userInfo[i]);
    }
  }

  protected void initializeUser(UN un, Role role, int startIndex, int count)
    throws Exception {
    initializeUserInfo(un, role, startIndex, count);
    for (int i = startIndex; i < (startIndex + count); i++) {
      userList.add(i, User.createUser((UserInfo)userInfoList.get(i)));
    }
  }

  /**
   * Libera as referncias dos VOs para o Garbage Collector.
   *
   * @param elements .
   */
  protected void cleanNameUN(int elements) {
    if (nameUN != null) {
      for (int i = 0; i < elements; i++) {
        nameUN[i] = null;
      }
      nameUN = null;
    }
  }

  /**
   * Libera as referncias dos VOs para o Garbage Collector.
   *
   * @param elements .
   */
  protected void cleanDescriptionUN(int elements) {
    if (descUN != null) {
      for (int i = 0; i < elements; i++) {
        descUN[i] = null;
      }
      descUN = null;
    }
  }

  /**
   * Libera as referncias dos VOs para o Garbage Collector.
   *
   * @param elements .
   */
  protected void cleanUNInfo(int elements) {
    cleanNameUN(elements);
    cleanDescriptionUN(elements);
    if (unInfo != null) {
      for (int i = 0; i < elements; i++) {
        unInfo[i] = null;
      }
      unInfo = null;
    }
  }

  /**
   * Libera as referncias dos VOs para o Garbage Collector.
   *
   * @param elements .
   */
  protected void cleanNameRole(int elements) {
    if (nameRole != null) {
      for (int i = 0; i < elements; i++) {
        nameRole[i] = null;
      }
      nameRole = null;
    }
  }

  /**
   * Libera as referncias dos VOs para o Garbage Collector.
   *
   * @param elements .
   */
  protected void cleanDescriptionRole(int elements) {
    if (descRole != null) {
      for (int i = 0; i < elements; i++) {
        descRole[i] = null;
      }
      descRole = null;
    }
  }

  /**
   * Libera as referncias dos VOs para o Garbage Collector.
   *
   * @param elements .
   */
  protected void cleanLoginRole(int elements) {
    if (loginRole != null) {
      for (int i = 0; i < elements; i++) {
        loginRole[i] = null;
      }
      loginRole = null;
    }
  }

  /**
   * Libera as referncias dos VOs para o Garbage Collector.
   *
   * @param elements .
   */
  protected void cleanPasswordRole(int elements) {
    if (passRole != null) {
      for (int i = 0; i < elements; i++) {
        passRole[i] = null;
      }
      passRole = null;
    }
  }

  /**
   * Libera as referncias dos VOs para o Garbage Collector.
   *
   * @param elements .
   */
  protected void cleanRoleInfo(int elements) {
    cleanNameRole(elements);
    cleanDescriptionRole(elements);
    cleanLoginRole(elements);
    cleanPasswordRole(elements);
    if (roleInfo != null) {
      for (int i = 0; i < elements; i++) {
        roleInfo[i] = null;
      }
      roleInfo = null;
    }
  }

  /**
   * Libera as referncias dos VOs para o Garbage Collector.
   *
   * @param elements .
   */
  protected void cleanNamePlatform(int elements) {
    if (namePlatform != null) {
      for (int i = 0; i < elements; i++) {
        namePlatform[i] = null;
      }
      namePlatform = null;
    }
  }

  /**
   * Libera as referncias dos VOs para o Garbage Collector.
   *
   * @param elements .
   */
  protected void cleanDescriptionPlatform(int elements) {
    if (descPlatform != null) {
      for (int i = 0; i < elements; i++) {
        descPlatform[i] = null;
      }
      descPlatform = null;
    }
  }

  /**
   * Libera as referncias dos VOs para o Garbage Collector.
   *
   * @param elements .
   */
  protected void cleanPlatformInfo(int elements) {
    cleanNamePlatform(elements);
    cleanDescriptionPlatform(elements);
    if (pInfo != null) {
      for (int i = 0; i < elements; i++) {
        pInfo[i] = null;
      }
      pInfo = null;
    }
  }

  /**
   * Libera as referncias dos VOs para o Garbage Collector.
   *
   * @param elements .
   */
  protected void cleanNameUser(int elements) {
    if (nameUser != null) {
      for (int i = 0; i < elements; i++) {
        nameUser[i] = null;
      }
      nameUser = null;
    }
  }

  /**
   * Libera as referncias dos VOs para o Garbage Collector.
   *
   * @param elements .
   */
  protected void cleanLoginUser(int elements) {
    if (loginUser != null) {
      for (int i = 0; i < elements; i++) {
        loginUser[i] = null;
      }
      loginUser = null;
    }
  }

  /**
   * Libera as referncias dos VOs para o Garbage Collector.
   *
   * @param elements .
   */
  protected void cleanMail(int elements) {
    if (mail != null) {
      for (int i = 0; i < elements; i++) {
        mail[i] = null;
      }
      mail = null;
    }
  }

  /**
   * Libera as referncias dos VOs para o Garbage Collector.
   *
   * @param elements .
   */
  protected void cleanPasswordUser(int elements) {
    if (passUser != null) {
      for (int i = 0; i < elements; i++) {
        passUser[i] = null;
      }
      passUser = null;
    }
  }

  /**
   * Libera as referncias dos VOs para o Garbage Collector.
   *
   * @param elements .
   *
   * @throws Exception .
   */
  protected void cleanUserInfo(int elements) throws Exception {
    cleanNameUser(elements);
    cleanLoginUser(elements);
    cleanUN(elements);
    cleanMail(elements);
    cleanPasswordUser(elements);
    cleanRole(elements);
    if (userInfo != null) {
      for (int i = 0; i < elements; i++) {
        userInfo[i] = null;
      }
      userInfo = null;
    }
  }

  protected void cleanUserInfo(int startIndex, int count) {
    for (int i = (startIndex + count) - 1; i >= startIndex; i--) {
      nameUserList.remove(i);
      loginUserList.remove(i);
      mailUserList.remove(i);
      passUserList.remove(i);
      userInfoList.remove(i);
    }
  }

  /**
   * Remove as UNs criadas do arquivo de dados. Libera as referncias das UNs
   * para o GC.
   *
   * @param elements .
   *
   * @throws Exception .
   */
  protected void cleanUN(int elements) throws Exception {
    cleanUNInfo(elements);
    if (un != null) {
      for (int i = 0; i < elements; i++) {
        if (un[i] != null) {
          UN.deleteUN(un[i].getId());
        }
        un[i] = null;
      }
      un = null;
    }
  }

  /**
   * Remove os Perfis criados do arquivo de dados. Libera as referncias dos
   * Perfis para o GC.
   *
   * @param elements .
   *
   * @throws Exception .
   */
  protected void cleanRole(int elements) throws Exception {
    cleanRoleInfo(elements);
    if (role != null) {
      for (int i = 0; i < elements; i++) {
        if (role[i] != null) {
          Role.deleteRole(role[i].getId());
        }
        role[i] = null;
      }
      role = null;
    }
  }

  /**
   * Remove as Plataformas criadas do arquivo de dados. Libera as referncias
   * das Plataformas para o GC.
   *
   * @param elements .
   *
   * @throws Exception .
   */
  protected void cleanPlatform(int elements) throws Exception {
    cleanPlatformInfo(elements);
    if (plat != null) {
      for (int i = 0; i < elements; i++) {
        if (plat[i] != null) {
          Platform.deletePlatform(plat[i].getId());
        }
        plat[i] = null;
      }
      plat = null;
    }
  }

  /**
   * Remove os Usurios criados do arquivo de dados. Libera as referncias dos
   * Usurios para o GC.
   *
   * @param elements .
   *
   * @throws Exception .
   */
  protected void cleanUser(int elements) throws Exception {
    if (user != null) {
      for (int i = 0; i < elements; i++) {
        if (user[i] != null) {
          User.deleteUser(user[i].getId());
        }
        user[i] = null;
      }
      user = null;
    }
    cleanUserInfo(elements);
  }

  protected void cleanUser(int startIndex, int count) throws Exception {
    cleanUserInfo(startIndex, count);
    for (int i = (startIndex + count) - 1; i >= startIndex; i--) {
      User aUser = (User)userList.get(i);
      if (aUser != null) {
        User.deleteUser(aUser.getId());
      }
      aUser = null;
      userList.remove(i);
    }
  }

  protected static void setElements(int elements)
    throws Exception {
    if (elements < 10) {
      ELEMENTS = 10;
      throw new Exception(
        "Nmero de elementos deve ser no mnimo 10.");
    }
    ELEMENTS = elements;
  }

  /**
   * Recupera referncias para os provedores do servidor RMI  e inicializa os
   * respectivos Observers.
   *
   * @param serverMonitor .
   * @param serverName .
   * @param rmiPort .
   */
  protected static void startClient(
    ServerMonitor serverMonitor, String serverName, String rmiPort) {
    assertNotNull(serverMonitor);
    try {
      serverMonitor.start(serverName + ":" + rmiPort);
      if (!serverMonitor.isAlive()) {
        System.out.println(
          "Servidor no disponvel:" + serverName + ":" + rmiPort);
        return;
      }
      String bindBase = "rmi://" + serverName + ":" + rmiPort + "/";
      System.out.println("Bind no servidor " + bindBase);
    }
    catch (Exception e) {
      System.out.println("Erro ao conectar com o servidor: " + serverName);
      e.printStackTrace();
      System.exit(0);
    }
  }

  protected static void stopClient(ServerMonitor serverMonitor) {
    serverMonitor.stop();
  }

  protected static void stopServer() {
    try {
      RemoteLocator.killServerService.shutDown();
    }
    catch (RemoteException e) {
      // O sistema vai ser derrubado antes desta thread morrer.
      // De qualquer forma, no me interessa tratar essa exceo. 
    }
  }

  protected static void startServer(
    String runServerCommand, File serverWorkingDir) throws Exception {
    long SLEEP_TIME = 10000;
    CSBaseAbstractTestCase.runServerCommand = runServerCommand;
    CSBaseAbstractTestCase.serverWorkingDir = serverWorkingDir;
    thread = new StartServer();
    thread.start();
    /* Aguardo um tempo mnimo para o servidor subir e fazer o bind dos
     * servios.
     */
    try {
      Thread.sleep(SLEEP_TIME);
    }
    catch (InterruptedException e) {}
  }

  /**
   * Inner class responsvel por levantar o servidor. Utiliza uma chamada para
   * outra virtual machine.
   */
  static class StartServer extends Thread {
    public void run() {
      Process p = null;
      try {
        p = Runtime.getRuntime().exec(runServerCommand, null, serverWorkingDir);
      }
      catch (Exception e) {
        System.out.println("O servidor no pde ser instanciado.");
        e.printStackTrace();
        System.out.println("Terminando o processo...");
        System.exit(0);
      }
      BufferedReader in =
        new BufferedReader(new InputStreamReader(p.getInputStream()));
      String line = null;
      try {
        while ((line = in.readLine()) != null) {
          System.out.println(line);
        }
      }
      catch (IOException e) {
        System.exit(0);
      }
    }
  }

  /**
   * Obtm o ResourceBundle para o arquivo de propriedades de teste.
   *
   * @param bundleName O nome do arquivo de propriedades o qual o
   *        ResourceBundle ir obter as informaes.
   *
   * @return ResourceBundle ResourceBundle que referencia as propriedades de
   *         requeridas.
   */
  public static ResourceBundle getTestBundle(String bundleName) {
    ResourceBundle properties;
    properties = ResourceBundle.getBundle(bundleName);
    return properties;
  }

  /* Mtodos novos ==========================================================*/
  protected static Map testUNMap = new HashMap();
  protected static Map testRoleMap = new HashMap();
  protected static Map testPlatformMap = new HashMap();
  protected static Map testUserMap = new HashMap();

  /**
   * Cria uma Unidade de Negcio fictcia para testes, usando para
   * simplificao a chave repassada em todos os campos da UN. Esta UN  ento
   * armazenada em um Map, podendo ser recuperada usando-se a mesma chave.
   *
   * @param key Chave que ser usada para criar e armazenar a un na tabela.
   *
   * @return UN A Unidade de Negcio criada.
   *
   * @throws Exception .
   */
  protected static UN createTestUN(String key) throws Exception {
    UNInfo unInfo = new UNInfo(key, key);
    UN testUN = UN.createUN(unInfo);
    testUNMap.put(key, testUN);
    return testUN;
  }

  /**
   * Cria um Perfil fictcio para testes, usando para simplificao a chave
   * repassada em todos os campos do Perfil. Este Perfil  ento armazenado em
   * um Map, podendo ser recuperado usando-se a mesma chave.
   *
   * @param key Chave que ser usada para criar e armazenar o perfil na tabela.
   *
   * @return Role O Perfil criado.
   *
   * @throws Exception .
   */
  protected static Role createTestRole(String key) throws Exception {
    //??Alterado para mudana de permission
    System.out.println("??CSBaseAbstractTestCase:823");
    RoleInfo roleInfo = new RoleInfo(key, key, null);
    Role testRole = Role.createRole(roleInfo);
    testRoleMap.put(key, testRole);
    return testRole;
  }

  /**
   * Cria uma Plataforma fictcia para testes, usando para simplificao a
   * chave repassada em todos os campos da Plataforma. Esta Plataforma  ento
   * armazenada em um Map, podendo ser recuperada usando-se a mesma chave.
   *
   * @param key Chave que ser usada para criar e armazenar a plataforma.
   *
   * @return Platform A Plataforma criada.
   *
   * @throws Exception .
   */
  protected Platform createTestPlatform(String key) throws Exception {
    PlatformInfo platInfo = new PlatformInfo(key, key);
    Platform testPlatform = Platform.createPlatform(platInfo);
    testPlatformMap.put(key, testPlatform);
    return testPlatform;
  }

  /**
   * Cria um Usurio fictcio para testes, usando para simplificao a chave
   * repassada em todos os campos de valores do Usurio, com exceo das
   * referncias  Unidade de Negcio e ao Perfil do Usurio, que devem ser
   * repassadas explicitamente. Este Usurio  ento armazenado em um Map,
   * podendo ser recuperado usando-se a mesma chave.
   *
   * @param key Chave que ser usada para criar e armazenar o usurio.
   * @param un A Unidade de Negcio associada ao usurio.
   * @param role O Perfl associado ao usurio.
   *
   * @return User O usurio criado.
   *
   * @throws Exception .
   */
  protected static User createTestUser(String key, UN un, Role role)
    throws Exception {
    //??Alterado para mudana de permission
    System.out.println("??CSBaseAbstractTestCase:867");
    UserInfo userInfo = new UserInfo(key, un.getId(), key, key, null, null);
    userInfo.passwordDigest = key;
    User testUser = User.createUser(userInfo);
    testUserMap.put(key, testUser);
    return testUser;
  }

  /**
   * Cria um Usurio fictcio para testes, usando para simplificao a chave
   * repassada em todos os campos de valores do Usurio. Este Usurio  ento
   * armazenado em um Map, podendo ser recuperado usando-se a mesma chave.
   *
   * @param key Chave que ser usada para criar e armazenar o usurio.
   *
   * @return User O usurio criado.
   *
   * @throws Exception .
   */
  protected static User createTestUser(String key) throws Exception {
    UN un = createTestUN(key);
    Role role = createTestRole(key);

    //??Alterado para mudana de permission
    System.out.println("??CSBaseAbstractTestCase:886");
    UserInfo userInfo = new UserInfo(key, un.getId(), key, key, null, null);
    userInfo.passwordDigest = key;
    User testUser = User.createUser(userInfo);
    testUserMap.put(key, testUser);
    return testUser;
  }

  /**
   * Cria um Projeto fictcio para testes, usando para simplificao a chave
   * repassada em todos os campos do Projeto, com exceo do Usurio, que deve
   * ser repassado explicitamente.
   *
   * @param key Chave que ser usada para criar e armazenar o projeto.
   * @param user O usurio que criou o projeto.
   *
   * @return CommonClientProject O Projeto criado.
   *
   * @throws Exception .
   */
  protected static CommonClientProject createTestProject(String key, User user)
    throws Exception {
    CommonProjectInfo projInfo = new CommonProjectInfo();
    projInfo.userId = user.getId();
    projInfo.name = key;
    projInfo.description = key;
    CommonClientProject project = CommonClientProject.createProject(projInfo);
    project.installObserver(new ProjectObserver(project));
    return project;
  }

  protected static void cleanUser(String key) throws Exception {
    User user = (User)testUserMap.remove(key);
    User.deleteUser(user.getId());
  }

  protected static void cleanUN(String key) throws Exception {
    UN un = (UN)testUNMap.remove(key);
    UN.deleteUN(un.getId());
  }

  protected static void cleanRole(String key) throws Exception {
    Role role = (Role)testRoleMap.remove(key);
    Role.deleteRole(role.getId());
  }

  protected static void cleanPlatform(String key) throws Exception {
    Platform platform = (Platform)testPlatformMap.remove(key);
    Platform.deletePlatform(platform.getId());
  }

  /**
   * Mtodo utilitrio para facilitar a verificao de arquivos no sistema de
   * arquivos. Caso o arquivo no exista, dispara um AssertionFailedError.
   *
   * @param path Caminho absoluto para o arquivo/diretrio, incluindo o nome do
   *        prprio.
   */
  public void assertFileExists(String path) {
    assertTrue(getFileFromPath(path).exists());
  }

  /**
   * Mtodo interno para reuso dos mtodos assertFileExists e
   * assertFileDoesNotExist. Converte uma String com um caminho abstrato,
   * representado por '|', para um caminho real, utilizando o separador de
   * arquivos do sistema operacional.
   *
   * @param path Caminho absoluto para o arquivo/diretrio, incluindo o nome do
   *        prprio.
   *
   * @return Um File representando o arquivo no sistema de arquivos.
   */
  private File getFileFromPath(String path) {
    String fileName = path.replace('|', File.separator.charAt(0));
    return new File(fileName);
  }

  /**
   * Mtodo utilitrio para facilitar a verificao de arquivos no sistema de
   * arquivos. Caso o arquivo exista, dispara um AssertionFailedError.
   *
   * @param path Caminho absoluto para o arquivo/diretrio, incluindo o nome do
   *        prprio.
   */
  public void assertFileDoesNotExist(String path) {
    assertFalse(getFileFromPath(path).exists());
  }
}
