/**
 * TestLoginService.java $Author: cassino $ $Revision: 33587 $ - $Date:
 * 2004/04/07 18:40:52 $
 */
package server.services;

import csbase.client.kernel.ClientException;
import csbase.client.language.LNG;
import csbase.logic.*;
import csbase.remote.RemoteLocator;
import csbase.server.services.projectservice.*;

import junit.framework.TestSuite;

import util.*;

import java.io.*;
import java.nio.*;
import java.rmi.RemoteException;
import java.util.*;

/**
 * A classe <code>TestMultiUserProject</code>  responsvel por testar  as
 * operaes de um projeto multi-usurio.
 */
public class TestMultiUserProject extends CSBaseTestCase {
  private static final String TEXT_TYPE = "TEXT";
  private static final String NOME_PROJETO = "projeto";
  private UN cenpes = null;
  private Role tecnico = null;
  private User ana = null;
  private User bernardo = null;
  private User carlos = null;
  private User daniel = null;
  private User erica = null;
  private CommonClientProject projeto = null;
  private ClientProjectFile arq1 = null;
  private ClientProjectFile arq2 = null;

  public void setUp() throws Exception {
    cenpes = createTestUN("Cenpes");
    tecnico = createTestRole("Tcnico");
    ana = createTestUser("ana", cenpes, tecnico);
    bernardo = createTestUser("bernardo", cenpes, tecnico);
    carlos = createTestUser("carlos", cenpes, tecnico);
    daniel = createTestUser("daniel", cenpes, tecnico);
    erica = createTestUser("erica", cenpes, tecnico);
    projeto = createTestProject(NOME_PROJETO, ana);
    ProjectFile root = projeto.getTree();
    projeto.createAndWaitForFile(root, "arq1", TEXT_TYPE, ana.getId());
    arq1 = (ClientProjectFile)root.getChild("arq1");
    projeto.createAndWaitForFile(root, "arq2", TEXT_TYPE, ana.getId());
    arq2 = (ClientProjectFile)root.getChild("arq2");
    /*Adiciona usuarios ao projeto*/
    Object[] ids = { bernardo.getId(), carlos.getId(), daniel.getId() };
    projeto.updateUsers(ids);
  }

  public void tearDown() {
    try {
      arq1.remove();
      arq2.remove();
      CommonClientProject.remove(ana.getId(), NOME_PROJETO);
      User.deleteUser(ana.getId());
      User.deleteUser(bernardo.getId());
      User.deleteUser(carlos.getId());
      User.deleteUser(daniel.getId());
      User.deleteUser(erica.getId());
      Role.deleteRole(tecnico.getId());
      UN.deleteUN(cenpes.getId());
    }
    catch (Exception e) {
      System.err.println("Erro no tearDown" + e.getMessage());
    }
  }

  public String getClassName() {
    return "TESTMULTIUSERPROJECT";
  }

  public void cleanElements(Vector elements) {}

  public Vector getAllElements() {
    return null;
  }

  /**
   *
   */
  public void testInitial() {
    assertNull(null);
  }

  /**
   * Verifica se os usurios esto no projeto.
   *
   * @throws Exception .
   */
  public void testAreUsersInProject() throws Exception {
    Object[] userIds =
      (Object[])projeto.getAttribute(CommonClientProject.USERS);
    List userList = Arrays.asList(userIds);
    assertTrue(userList.contains(bernardo.getId()));
    assertTrue(userList.contains(carlos.getId()));
    assertTrue(userList.contains(daniel.getId()));
  }

  /**
   * Os usurios adicionados verificam se o projeto esta' na sua lista de
   * projetos.
   *
   * @throws Exception .
   */
  public void testIsProjectInUsers() throws Exception {
    boolean bFlag = false;
    boolean cFlag = false;
    boolean dFlag = false;
    Vector bernardoProjs =
      CommonClientProject.getAllUserProjects(bernardo.getId());
    Vector carlosProjs = CommonClientProject.getAllUserProjects(carlos.getId());
    Vector danielProjs = CommonClientProject.getAllUserProjects(daniel.getId());
    for (int i = 0; i < bernardoProjs.size(); i++) {
      UserProjectInfo upi = (UserProjectInfo)bernardoProjs.get(i);
      if (upi.getProjectName().equals(NOME_PROJETO)) {
        bFlag = true;
      }
    }
    assertTrue(bFlag);
    for (int i = 0; i < carlosProjs.size(); i++) {
      UserProjectInfo upi = (UserProjectInfo)carlosProjs.get(i);
      if (upi.getProjectName().equals(NOME_PROJETO)) {
        cFlag = true;
      }
    }
    assertTrue(cFlag);
    for (int i = 0; i < danielProjs.size(); i++) {
      UserProjectInfo upi = (UserProjectInfo)danielProjs.get(i);
      if (upi.getProjectName().equals(NOME_PROJETO)) {
        dFlag = true;
      }
    }
    assertTrue(dFlag);
  }

  /**
   * Verifica se os arquivos criados possuem o owner Ana
   */
  public void testVerifyFilesOwner() {
    Object ownerId = arq1.whoCreated();
    assertEquals(ownerId, ana.getId());
    ownerId = arq2.whoCreated();
    assertEquals(ownerId, ana.getId());
  }

  /**
   * Os usurios abrem o projeto leem e escrevem dados nele.
   *
   * @throws Exception .
   */
  public void testUpdateAndReadFileInProject() throws Exception {
    CommonClientProject projB =
      CommonClientProject.openProject(ana.getId(), NOME_PROJETO);
    CommonClientProject projC =
      CommonClientProject.openProject(ana.getId(), NOME_PROJETO);
    /*Cliente B escreve no arq1*/
    ProjectFile rootB = projB.getTree();
    ClientProjectFile arq1B = (ClientProjectFile)rootB.getChild("arq1");
    arq1B.open();
    String text = "Texto do arquivo!";
    int size = (int)text.length();
    byte[] array = text.getBytes();
    ByteBuffer buffer = ByteBuffer.wrap(array);
    for (int i = 0; i < buffer.capacity(); i += arq1B.write(buffer, i)) {
      try {
        Thread.sleep(100);
      }
      catch (InterruptedException e) {}
    }
    arq1B.truncate(size);
    arq1B.close();
    /*Cliente C ler o conteudo do arq1*/
    ProjectFile rootC = projC.getTree();
    ClientProjectFile arq1C = (ClientProjectFile)rootC.getChild("arq1");
    arq1C.open();
    StringBuffer sb = new StringBuffer();
    size = (int)arq1C.size();
    buffer = ByteBuffer.allocate(100 * 1024);
    int i = 0;
    do {
      i += arq1C.read(buffer, i);
      buffer.flip();
      int last = buffer.limit();
      array = null;
      if (buffer.hasArray()) {
        array = buffer.array();
      }
      else {
        array = new byte[buffer.remaining()];
        buffer.get(array);
      }
      sb.append(new String(array, 0, last));
      buffer.clear();
    }
    while (i < size);
    arq1C.close();
    Thread.sleep(100);
    String readText = sb.toString();
    assertEquals(readText, text);
  }

  /**
   * A cria um arquivo e D ler.
   *
   * @throws Exception .
   */
  public void testCreateAndReadFileInProject() throws Exception {
    ProjectFile root = projeto.getTree();
    projeto.createAndWaitForFile(root, "arq3", TEXT_TYPE, ana.getId());
    ClientProjectFile arq3 = (ClientProjectFile)root.getChild("arq3");
    assertNotNull(arq3);
    CommonClientProject projD =
      CommonClientProject.openProject(ana.getId(), NOME_PROJETO);
    ProjectFile rootD = projD.getTree();
    ClientProjectFile arq3D = (ClientProjectFile)rootD.getChild("arq3");
    assertNotNull(arq3D);
    assertEquals(arq3.getName(), arq3D.getName());
    arq3D.remove();
  }

  /**
   * B publica um arquivo para E
   *
   * @throws Exception .
   */
  public void testPublishFile() throws Exception {
    CommonClientProject projB =
      CommonClientProject.openProject(ana.getId(), NOME_PROJETO);
    ProjectFile rootB = projB.getTree();
    ClientProjectFile arq1B = (ClientProjectFile)rootB.getChild("arq1");
    assertNotNull(arq1B);
    Object[] ids = { erica.getId() };
    arq1B.publish(ids, bernardo.getId());
    Object[] userIds = arq1B.getPublishAccess();
    List userList = Arrays.asList(userIds);
    assertTrue(userList.contains(erica.getId()));
  }

  /**
   * B publica um arquivo para E e E obtem o arquivo
   *
   * @throws Exception .
   */
  public void testPublishAndGetFile() throws Exception {
    CommonClientProject projB =
      CommonClientProject.openProject(ana.getId(), NOME_PROJETO);
    ProjectFile rootB = projB.getTree();
    ClientProjectFile arq1B = (ClientProjectFile)rootB.getChild("arq1");
    assertNotNull(arq1B);
    Object[] ids = { erica.getId() };
    arq1B.publish(ids, bernardo.getId());
    CommonClientProject projetoTeste = createTestProject("teste", erica);
    ClientProjectFile root = (ClientProjectFile)projetoTeste.getTree();
    Vector pubInfo = CommonClientProject.getAllPublicationInfo(erica.getId());
    assertNotNull(pubInfo);
    PublicationInfo info = (PublicationInfo)pubInfo.get(0);
    root.linkToPublishedFile(info, erica.getId());
    ClientProjectFile arq1Pub = (ClientProjectFile)root.getChild("arq1");
    assertNotNull(arq1Pub);
    assertEquals(arq1Pub.whoPublished(), bernardo.getId());
    assertEquals(arq1Pub.whoCreated(), ana.getId());
    CommonClientProject.remove(erica.getId(), "teste");
  }

  /**
   * Um usurio que no est no projeto tenta abr-lo
   *
   * @throws Exception .
   */
  public void testVerifyProjectList() throws Exception {
    Vector projs = CommonClientProject.getAllUserProjects(erica.getId());
    boolean foundProject = false;
    if (projs != null) {
      for (int i = 0; i < projs.size(); i++) {
        UserProjectInfo upi = (UserProjectInfo)projs.get(i);
        if (NOME_PROJETO.equals(upi.getProjectName())) {
          foundProject = true;
        }
      }
    }
    assertFalse(foundProject);
  }

  /**
   * A remove D de PROJ (mantendo B e C) e D tenta abrir PROJ
   *
   * @throws Exception .
   */
  public void testUpdateUsers() throws Exception {
    Object[] ids = { bernardo.getId(), carlos.getId() };
    projeto.updateUsers(ids);
    Vector projs = CommonClientProject.getAllUserProjects(daniel.getId());
    boolean foundProject = false;
    if (projs != null) {
      for (int i = 0; i < projs.size(); i++) {
        UserProjectInfo upi = (UserProjectInfo)projs.get(i);
        if (NOME_PROJETO.equals(upi.getProjectName())) {
          foundProject = true;
        }
      }
    }
    assertFalse(foundProject);
    projs = CommonClientProject.getAllUserProjects(carlos.getId());
    foundProject = false;
    if (projs != null) {
      for (int i = 0; i < projs.size(); i++) {
        UserProjectInfo upi = (UserProjectInfo)projs.get(i);
        if (NOME_PROJETO.equals(upi.getProjectName())) {
          foundProject = true;
        }
      }
    }
    assertTrue(foundProject);
  }

  /**
   * A adiciona E ao PROJ, E abre PROJ e l um arquivo.
   *
   * @throws Exception .
   */
  public void testAddNewUser() throws Exception {
    Object[] ids =
    { bernardo.getId(), carlos.getId(), daniel.getId(), erica.getId() };
    projeto.updateUsers(ids);
    Vector projs = CommonClientProject.getAllUserProjects(erica.getId());
    boolean foundProject = false;
    if (projs != null) {
      for (int i = 0; i < projs.size(); i++) {
        UserProjectInfo upi = (UserProjectInfo)projs.get(i);
        if (NOME_PROJETO.equals(upi.getProjectName())) {
          foundProject = true;
        }
      }
    }
    assertTrue(foundProject);
    CommonClientProject projE =
      CommonClientProject.openProject(ana.getId(), NOME_PROJETO);
    ProjectFile rootE = projE.getTree();
    ClientProjectFile arq1E = (ClientProjectFile)rootE.getChild("arq1");
    assertNotNull(arq1E);
  }

  public TestMultiUserProject(String name) throws ClientException {
    super(name);
  }

  public TestMultiUserProject() throws ClientException {
    super();
  }
}
