/**
 * TestPlatform.java $Author: cassino $ $Revision: 33587 $ - $Date: 2004/02/05
 * 18:25:12 $
 */
package server.services;

import csbase.logic.*;

import util.*;

import java.util.Vector;

/**
 * A classe <code>TestPlatform</code> representa um caso de teste para os
 * servios de manipulao de Plataforma do csbase.
 */
public class TestPlatform extends CSBaseTestCase {
  PlatformInfo platInfoTest;

  /**
   * Implementao de mtodos abstratos
   *
   * @return .
   */
  protected String getClassName() {
    return "TESTPLATFORM";
  }

  public void initialize() throws Exception {
    platInfoTest = new PlatformInfo(TEST_KEY, TEST_KEY);
  }

  protected Vector getAllElements() throws Exception {
    return Platform.getAllPlatforms();
  }

  protected void cleanElements(Vector elements) throws Exception {
    for (int i = 0; i < elements.size(); i++) {
      Platform platform = (Platform)elements.get(i);
      Platform.deletePlatform(platform.getId());
    }
  }

  /**
   * Testa se o objeto criado contm todos os valores informados pelo usurio.
   *
   * @throws Exception .
   */
  public void testCreatePlatform() throws Exception {
    Platform platform = createTestPlatform(TEST_KEY);
    assertEquals(TEST_KEY, platform.getName());
    assertEquals(TEST_KEY, platform.getDescription());
    Platform.deletePlatform(platform.getId());
  }

  /**
   * Verifica se a plataforma foi criada no arquivo.
   *
   * @throws Exception .
   */
  public void testCreatePlatformFile() throws Exception {
    Platform platform = createTestPlatform(TEST_KEY);
    Vector allPlatforms = Platform.getAllPlatforms();
    assertEquals(1, allPlatforms.size());
    assertTrue(allPlatforms.contains(platform));
    Platform.deletePlatform(platform.getId());
  }

  /**
   * Verifica o mtodo de obteno de uma plataforma.
   *
   * @throws Exception .
   */
  public void testGetPlatformById() throws Exception {
    Platform platform1 = createTestPlatform(TEST_KEY);
    Platform platform2 = Platform.getPlatform(platform1.getId());
    assertSame(platform2, platform1);
    Platform.deletePlatform(platform1.getId());
  }

  /**
   * Verifica se o sistema aceita criar uma plataforma com o mesmo nome.
   *
   * @throws Exception .
   */
  public void testCreateSamePlatform() throws Exception {
    Platform platform1 = createTestPlatform(TEST_KEY);
    Platform platform2 = createTestPlatform(TEST_KEY);
    assertNull(platform2);
    Platform.deletePlatform(platform1.getId());
  }

  /**
   * Testa se s as plataformas pares foram apagadas, depois tenta reinserir as
   * plataformas pares apagadas.
   *
   * @throws Exception .
   */
  public void testEvenPlatforms() throws Exception {
    int ELEMENTS = 6;
    Platform[] plat = new Platform[ELEMENTS];
    for (int i = 0; i < ELEMENTS; i++) {
      plat[i] = createTestPlatform("" + i);
    }
    /* apagando plataformas pares */
    for (int i = 0; i < ELEMENTS; i++) {
      if ((i % 2) == 0) {
        Platform.deletePlatform(plat[i].getId());
        plat[i] = null;
      }
    }
    /* verifica se as pares foram apagadas e se as impares permanecem */
    Vector allPlatforms = Platform.getAllPlatforms();
    int remainingPlatforms = (ELEMENTS / 2);
    assertEquals(remainingPlatforms, allPlatforms.size());
    for (int i = 0; i < ELEMENTS; i++) {
      if ((i % 2) == 1) {
        assertTrue(allPlatforms.contains(plat[i]));
      }
    }
    /* recria os pares recem apagados */
    for (int i = 0; i < ELEMENTS; i++) {
      if ((i % 2) == 0) {
        plat[i] = createTestPlatform("" + i);
      }
    }
    /* verifica se todos esto presentes. Remove um a um. */
    allPlatforms = Platform.getAllPlatforms();
    assertEquals(ELEMENTS, allPlatforms.size());
    for (int i = 0; i < ELEMENTS; i++) {
      assertTrue(allPlatforms.contains(plat[i]));
      Platform.deletePlatform(plat[i].getId());
    }
  }

  /**
   * Testa a modificao de plataformas.
   *
   * @throws Exception .
   */
  public void testModifyPlatform() throws Exception {
    Platform platform = createTestPlatform(TEST_KEY);
    PlatformInfo pInfo = new PlatformInfo(TEST_KEY, "Linux Black Shoes 1.0");
    platform = Platform.modifyPlatform(platform.getId(), pInfo);
    assertNotNull(platform);
    assertEquals(pInfo.description, platform.getDescription());
    Platform.deletePlatform(platform.getId());
  }

  /**
   * Testa a modificao do nome das plataformas. (No podem ser modificadas)
   *
   * @throws Exception .
   */
  public void testModifyName() throws Exception {
    Platform platform = createTestPlatform(TEST_KEY);
    PlatformInfo pInfo = new PlatformInfo("irix", TEST_KEY);
    Platform otherPlat = Platform.modifyPlatform(platform.getId(), pInfo);
    assertNull(otherPlat);
    platform = Platform.getPlatform(platform.getId());
    assertEquals(TEST_KEY, platform.getName());
    Platform.deletePlatform(platform.getId());
  }

  /**
   * Testa a modificao de uma plataforma no existente no arquivo.
   *
   * @throws Exception .
   */
  public void testModifyNonExistentPlatform() throws Exception {
    Platform platform =
      Platform.modifyPlatform(new Integer(3000), platInfoTest);
    assertNull(platform);
    platform = Platform.getPlatform(new Integer(3000));
    assertNull(platform);
  }

  /**
   * Tenta modificar uma plataforma com identificador nulo.
   *
   * @throws Exception .
   */
  public void testModifyNull() throws Exception {
    Platform platform = null;
    try {
      platform = Platform.modifyPlatform(null, platInfoTest);
      fail("Esperada Exception");
    }
    catch (Exception pe) {}
    assertNull(platform);
  }

  /**
   * Tenta modificar uma plataforma usando uma informao nula.
   *
   * @throws Exception .
   */
  public void testModifyNullPlatInfo() throws Exception {
    Platform plat = createTestPlatform(TEST_KEY);
    Platform platform = null;
    try {
      platform = Platform.modifyPlatform(plat.getId(), null);
      fail("Esperada Exception");
    }
    catch (Exception pe) {}
    assertNull(platform);
    Platform.deletePlatform(plat.getId());
  }

  /**
   * Tenta modificar uma plataforma passando uma pInfo com campos nulos.
   *
   * @throws Exception .
   */
  public void testModifyWithNullFields() throws Exception {
    Platform plat = createTestPlatform(TEST_KEY);
    Platform platform = null;
    try {
      PlatformInfo nullInfo = new PlatformInfo(null, null);
      platform = Platform.modifyPlatform(plat.getId(), nullInfo);
      fail("Esperada Exception");
    }
    catch (Exception pe) {}
    assertNull(platform);
    Platform.deletePlatform(plat.getId());
  }

  /**
   * Tenta criar uma plataforma passando uma Informao nula.
   *
   * @throws Exception .
   */
  public void testCreateWithNullInfo() throws Exception {
    Platform platform = null;
    try {
      platform = Platform.createPlatform(null);
      fail("Esperada Exception");
    }
    catch (Exception pe) {}
    assertNull(platform);
  }

  /**
   * Tenta criar uma plataforma a partir de uma pInfo com campos nulos.
   *
   * @throws Exception .
   */
  public void testCreateWithNullFields() throws Exception {
    Platform platform = null;
    try {
      PlatformInfo nullInfo = new PlatformInfo(null, null);
      assertNotNull(nullInfo);
      platform = Platform.createPlatform(nullInfo);
      fail("Esperada Exception");
    }
    catch (Exception pe) {}
    assertNull(platform);
  }

  /**
   * Tenta obter uma plataforma passando um identificador nulo.
   *
   * @throws Exception .
   */
  public void testGetPlatformWithNullId() throws Exception {
    Platform platform = null;
    try {
      platform = Platform.getPlatform(null);
      fail("Esperada Exception");
    }
    catch (Exception pe) {}
    assertNull(platform);
  }

  /**
   * Testa o pedido de remoo de plataforma passando um identificador nulo.
   *
   * @throws Exception .
   */
  public void testDeletePlatformWithNullId() throws Exception {
    try {
      Platform.deletePlatform(null);
      fail("Esperada Exception");
    }
    catch (Exception pe) {}
  }
}
