/**
 * TestLoginService.java $Author: cassino $ $Revision: 33587 $ - $Date:
 * 2004/04/07 18:42:20 $
 */
package server.services;

import csbase.client.kernel.ClientException;
//import csbase.client.project.ProjectFileType;
import csbase.client.language.LNG;
import csbase.logic.*;
import csbase.remote.RemoteLocator;
import csbase.server.services.projectservice.*;

import junit.framework.TestSuite;

import util.*;

import java.io.*;
import java.nio.*;
import java.rmi.RemoteException;
import java.util.*;

/**
 * A classe <code>TestProjectService</code>  responsvel por testar o servio
 * de criao, edio e remoo de Projetos.
 */

/*
   Testes Realizados:
   - SingleProject                                                  OK
   - SameNameSameUser                                               OK
   - SameNameProjectRemoving                                        OK
   - RemoveUserWithProject                                          OK
   - CreateProjectWithNull                                          FAILED
   - CreateProjectInvalidInfo                                       FAILED
   - LongProjectNames                                               FAILED
   - ModifyInfo                                                     OK
   - OpenWithObject                                                 FAILED
   - RemoveProjectTwice                                             OK
   - WritingWithoutOpening                                          OK
   - ReadingWithoutOpening                                          OK
   - JustOpenForReading                                             FAILED
   - JustOpenForWriting                                             OK
   - ReadFromOpenedForWriting                                       OK
   - WriteOnOpenedForReading                                        FAILED
   - ChangeFromWriteToRead                                          FAILED
   - ChangeFromReadToWrite                                          FAILED
   - FilesWithObjects                                               OK
   - WriteOnByteBuffer                                              FAILED
   - OperateWithFilesAndFolders                                     OK
 */
public class TestProjectService extends CSBaseTestCase {
  private UN cenpes = null;
  private Role tecnico = null;
  private User joao = null;
  private User maria = null;
  private CommonClientProject projeto = null;
  private ClientProjectFile objetos = null;
  private String projDir = null;
  private String TEXT_TYPE;
  private String UNKNOWN_TYPE;
  private String HTML_TYPE;

  private void loadProperties() throws ClientException {
    projDir = System.getProperty("project_dir");
    TEXT_TYPE = "TEXT";
    UNKNOWN_TYPE = "UNKNOWN";
    HTML_TYPE = "HTML";
  }

  public void setUp() throws Exception {
    cenpes = createTestUN("Cenpes");
    tecnico = createTestRole("Tcnico");
    joao = createTestUser("joao", cenpes, tecnico);
    maria = createTestUser("maria", cenpes, tecnico);
    if ((joao == null) || (maria == null)) {
      fail("No foi possvel criar os usurios");
    }
    projeto = createTestProject("projeto", maria);
    if (projeto == null) {
      fail("No foi possvel criar o projeto");
    }
    ProjectFile root = projeto.getTree();
    projeto.createAndWaitForFile(root, "objetos", TEXT_TYPE, maria.getId());
    objetos = (ClientProjectFile)root.getChild("objetos");
  }

  public void tearDown() {
    try {
      objetos.remove();
      CommonClientProject.remove(maria.getId(), "projeto");
      User.deleteUser(joao.getId());
      User.deleteUser(maria.getId());
      Role.deleteRole(tecnico.getId());
      UN.deleteUN(cenpes.getId());
    }
    catch (Exception e) {
      System.err.println("Erro no tearDown" + e.getMessage());
    }
  }

  public String getClassName() {
    return "TESTPROJECTSERVICE";
  }

  /**
   * Testa a criao, listagem e remoo de um nico projeto.
   *
   * @throws Exception .
   */
  public void testSingleProject() throws Exception {
    /* Crio um projeto */
    CommonClientProject proj1 = createTestProject("proj1", joao);
    /* Verifico se os diretrios foram criados corretamente */
    String projDir = System.getProperty("project_dir");
    assertFileExists(projDir + "|joao|proj1");
    /* Verifico as propriedades do projeto */
    assertEquals("proj1", proj1.getName());
    assertEquals("proj1", ((ClientProjectFile)proj1.getTree()).getName());
    assertEquals(joao.getId(), proj1.getUserId());
    assertEquals(0, proj1.getTree().getChildren().length);
    /* Verifico se o projeto foi inserido corretamente na lista de projetos */
    String[] projNames = CommonClientProject.getAllProjectNames(joao.getId());
    assertEquals(1, projNames.length);
    assertEquals("proj1", projNames[0]);
    /* Removo tudo o que foi criado */
    CommonClientProject.remove(joao.getId(), "proj1");
    assertFileDoesNotExist(projDir + "|joao|proj1");
    projNames = CommonClientProject.getAllProjectNames(joao.getId());
    assertNull(projNames);
  }

  /**
   * Tento criar dois projetos com o mesmo nome
   *
   * @throws Exception .
   */
  public void testSameNameSameUserProject() throws Exception {
    CommonClientProject proj1 = createTestProject("proj1", joao);
    try {
      proj1 = createTestProject("proj1", joao);
      fail("Esperada Exception");
    }
    catch (ProjectAlreadyExistsException e) {}
    CommonClientProject.remove(joao.getId(), "proj1");
  }

  /**
   * Tento criar dois projetos com o mesmo nome, mas de usurios diferentes
   *
   * @throws Exception .
   */
  public void testSameNameDiffUserProject() throws Exception {
    CommonClientProject proj1 = createTestProject("proj1", joao);
    CommonClientProject proj1b = createTestProject("proj1", maria);
    CommonClientProject.remove(joao.getId(), "proj1");
    CommonClientProject.remove(maria.getId(), "proj1");
  }

  /**
   * Tento criar o mesmo projeto duas vezes, mas removendo
   *
   * @throws Exception .
   */
  public void testSameNameProjectRemoving() throws Exception {
    CommonClientProject proj1 = createTestProject("proj1", joao);
    CommonClientProject.remove(joao.getId(), "proj1");
    proj1 = createTestProject("proj1", joao);
    CommonClientProject.remove(joao.getId(), "proj1");
  }

  /**
   * Tento remover um usurio que tenha um projeto associado
   *
   * @throws Exception .
   */
  public void testRemoveUserWithProject() throws Exception {
    User temporario = createTestUser("temp", cenpes, tecnico);
    CommonClientProject proj1 = createTestProject("proj1", temporario);
    try {
      User.deleteUser(temporario.getId());
      fail("Esperada Exception");
    }
    catch (Exception e) {}
    CommonClientProject.remove(temporario.getId(), "proj1");
    User.deleteUser(temporario.getId());
  }

  /**
   * Tento criar um projeto utilizando informaes nulas OBS: FALTA REFINAR A
   * EXCEPTION PRA VALIDAR O TESTE public void testCreateProjectWithNull() {
   * try { CommonClientProject.createProject(null); fail("Esperada
   * Exception"); } catch(Exception e) {}}
   *
   * @throws Exception .
   */
  /**
   * Tento criar um projeto utilizando informaes invlidas (CommonProjectInfo
   * sem referncia do usurio) OBS: FALTA REFINAR A EXCEPTION PRA VALIDAR O
   * TESTE public void testCreateProjectInvalidInfo() { try {
   * CommonProjectInfo projInfo = new CommonProjectInfo();
   * CommonClientProject.createProject(projInfo); fail("Esperada Exception");
   * } catch(Exception e) {}}
   *
   * @throws Exception .
   */
  /**
   * Testa a criao de projetos nomeados com espaos ou caracteres
   * internacionais. public void testLongProjectNames() { try {
   * createTestProject("nome com espaco",joao); fail("Esperada Exception"); }
   * catch(Exception e) {} try { createTestProject("corao",joao);
   * fail("Esperada Exception"); } catch(Exception e) {}}
   *
   * @throws Exception .
   */
  /**
   * Verifica se informaes como o nome e a descrio de um projeto so
   * efetivamente modificadas.
   *
   * @throws Exception .
   */
  public void testModifyInfo() throws Exception {
    CommonClientProject modifProj = createTestProject("modifproj", joao);
    modifProj.setDescription("descrio");
    modifProj.modify();
    modifProj = CommonClientProject.openProject(joao.getId(), "modifproj");
    assertEquals("descrio", modifProj.getDescription());
    CommonClientProject.remove(joao.getId(), "modifproj");
  }

  /**
   * Passo um parmetro inesperado para o mtodo OBS: FALTA REFINAR A EXCEPTION
   * PRA VALIDAR O TESTE public void testOpenWithObject() { try {
   * CommonClientProject.openProject(new Object(),"lixo"); fail("Esperada
   * Exception"); } catch(Exception e) {}}
   */
  /**
   * Tento remover duas vezes o mesmo projeto public void
   * testRemoveProjectTwice() throws Exception { CommonClientProject temp =
   * createTestProject("temp",maria);
   * CommonClientProject.remove(maria.getId(),"temp");  try {
   * CommonClientProject.remove(maria.getId(),"temp");  fail("Esperada
   * Exception"); } catch(Exception e) {}}
   */
  /**
   * Tenta escrever em um arquivo sem abrir
   */
  public void testWritingWithoutOpening() {
    try {
      objetos.writeObject("Erro");
      fail("Esperada Exception");
    }
    catch (Exception e) {}
  }

  /**
   * Tenta ler em um arquivo sem abrir
   */
  public void testReadingWithoutOpening() {
    try {
      objetos.readObject();
      fail("Esperada Exception");
    }
    catch (Exception e) {}
  }

  /**
   * Abro um arquivo vazio para leitura, mas no tento ler. public void
   * testJustOpenForReading() throws Exception { objetos.openForReading();
   * objetos.close(); }
   *
   * @throws Exception .
   */
  /**
   * Abro um arquivo vazio para escrita, mas no tento escrever.
   *
   * @throws Exception .
   */
  public void testJustOpenForWriting() throws Exception {
    objetos.openForWriting();
    objetos.close();
  }

  /**
   * Tento ler um arquivo aberto para escrita
   *
   * @throws Exception .
   */
  public void testReadFromOpenedForWriting() throws Exception {
    /* Garanto que o arquivo tenha informaes para serem lidas */
    objetos.openForWriting();
    objetos.writeObject("Alguma coisa");
    objetos.close();
    objetos.openForWriting();
    try {
      String tmp = (String)objetos.readObject();
      fail("Esperada Exception");
    }
    catch (Exception e) {}
    objetos.close();
  }

  /**
   * Tento escrever em um arquivo aberto para leitura public void
   * testWriteOnOpenedForReading() throws Exception {
   * objetos.openForReading(); try { objetos.writeObject("falha");
   * fail("Esperada Exception"); } catch(Exception e) {} objetos.close(); }
   *
   * @throws Exception .
   */
  /**
   * Tento abrir o mesmo arquivo em modo leitura, aps t-lo aberto para
   * escrita, sem fechar. public void testChangeFromWriteToRead() throws
   * Exception { objetos.openForWriting(); try { objetos.openForReading();
   * fail("Esperada Exception"); } catch(Exception e) {} objetos.close(); }
   *
   * @throws Exception .
   */
  /**
   * Tento abrir o mesmo arquivo em modo escrita, aps t-lo aberto para
   * leitura, sem fechar. public void testChangeFromReadToWrite() throws
   * Exception { objetos.openForReading(); try { objetos.openForWriting();
   * fail("Esperada Exception"); } catch(Exception e) {} objetos.close(); }
   *
   * @throws Exception .
   */
  /**
   * Abro um arquivo para leitura/escrita, escrevo, leio, fecho. Reabro,
   * modifico e leio mais uma vez.
   *
   * @throws Exception .
   */
  public void testFilesWithObjects() throws Exception {
    objetos.openForWriting();
    objetos.writeObject("Frase 1");
    objetos.writeObject(new Date());
    objetos.close();
    objetos.openForReading();
    assertEquals("Frase 1", (String)objetos.readObject());
    assertTrue(((Date)objetos.readObject()).compareTo(new Date()) < 0);
    objetos.close();
    /* Aqui, o contedo anterior vai ser sobrescrito */
    objetos.openForWriting();
    objetos.writeObject("Carambola");
    objetos.close();
    objetos.openForReading();
    assertEquals("Carambola", (String)objetos.readObject());
    /*    try {
       objetos.readObject();
       fail("Esperada Exception");
       } catch(Exception e) {}*/
    objetos.close();
  }

  /**
   * Tento escrever em um arquivo via ByteBuffer e remover o projeto em
   * seguida. OBS: A FALHA AQUI OCORRE OCASIONALMENTE, PELO FATO DO PROJETO
   * TENTAR SER REMOVIDO ANTES DA THREAD QUE MANIPULA O ARQUIVO TERMINAR SEU
   * TRABALHO.
   *
   * @throws Exception .
   */

  /*
     public void testWriteOnByteBuffer() throws Exception {
       CommonClientProject MeuProjeto = createTestProject("meuprojeto",joao);
       ProjectFile root = MeuProjeto.getTree();
       MeuProjeto.createAndWaitForFile(root,"file",TEXT_TYPE);
       ClientProjectFile file = (ClientProjectFile)root.getChild("file");
       file.open();
       String content = "Isto e um teste";
       int size = (int)content.length();
       byte[] array = content.getBytes();
       ByteBuffer buffer = ByteBuffer.wrap(array);
       for (int i=0; i<buffer.capacity(); i+=file.write(buffer, i));
       file.truncate(size);
       file.close();
       //Thread.sleep(100); // Sem essa linha, ocasionalmente ocorrem excees.
       //CommonClientProject.remove(joao.getId(),"meuprojeto");
     }*/
  /**
   * Crio uma rvore com vrios diretrios e arquivos, apago, renomeio, movo e
   * recrio. Verifico se o file system est de acordo.
   *
   * @throws Exception .
   */
  public void beforeCrashOperateWithFilesAndFolders() throws Exception {
    /* Carrega as propriedades necessrias
       LNG.load("/resources/language/idiom_pt_BR.properties");
       ProjectFileType.load("/resources/filetypes/FileType.properties");
       String projDir = System.getProperty("project_dir");
       int TEXT_TYPE = ProjectFileType.getFileType("TEXT").getId();
     */
    /* Cria a rvore inicial */
    CommonClientProject proj1 = createTestProject("proj1", joao);
    assertFileExists(projDir + "|joao|proj1");
    ProjectFile root = proj1.getTree();
    root.createDirectory("mundo");
    assertFileExists(projDir + "|joao|proj1|mundo");
    proj1.createAndWaitForFile(root, "Deus", TEXT_TYPE, joao.getId());
    assertFileExists(projDir + "|joao|proj1|Deus");
    ProjectFile mundo = root.getChild("mundo");
    mundo.createDirectory("planetas");
    assertFileExists(projDir + "|joao|proj1|mundo|planetas");
    mundo.createDirectory("estrelas");
    assertFileExists(projDir + "|joao|proj1|mundo|estrelas");
    mundo.createDirectory("cometas");
    assertFileExists(projDir + "|joao|proj1|mundo|cometas");
    proj1.createAndWaitForFile(root, "Vida", UNKNOWN_TYPE, joao.getId());
    assertFileExists(projDir + "|joao|proj1|Vida");
    ProjectFile planetas = mundo.getChild("planetas");
    planetas.createDirectory("Urano");
    assertFileExists(projDir + "|joao|proj1|mundo|planetas|Urano");
    planetas.createDirectory("Venus");
    assertFileExists(projDir + "|joao|proj1|mundo|planetas|Venus");
    planetas.createDirectory("Terra");
    assertFileExists(projDir + "|joao|proj1|mundo|planetas|Terra");
    ProjectFile terra = planetas.getChild("Terra");
    terra.createDirectory("paises");
    assertFileExists(projDir + "|joao|proj1|mundo|planetas|Terra|paises");
    terra.createDirectory("mares");
    assertFileExists(projDir + "|joao|proj1|mundo|planetas|Terra|mares");
    ProjectFile paises = terra.getChild("paises");
    proj1.createAndWaitForFile(paises, "Brasil", TEXT_TYPE, joao.getId());
    assertFileExists(
      projDir + "|joao|proj1|mundo|planetas|Terra|paises|Brasil");
    /* Comeam as modificaes nessa estrutura */
    ProjectFile brasil = paises.getChild("Brasil");
    brasil.copy(mundo);
    assertFileExists(projDir + "|joao|proj1|mundo|Brasil");
    ProjectFile novoBrasil = mundo.getChild("Brasil");
    novoBrasil.rename("Lua");
    assertFileDoesNotExist(projDir + "|joao|proj1|mundo|Brasil");
    assertFileExists(projDir + "|joao|proj1|mundo|Lua");
    ProjectFile venus = planetas.getChild("Venus");
    venus.remove();
    assertFileDoesNotExist(projDir + "|joao|proj1|mundo|planetas|Venus");
    ProjectFile cometas = mundo.getChild("cometas");
    ProjectFile urano = planetas.getChild("Urano");
    cometas.move(urano);
    assertFileDoesNotExist(projDir + "|joao|proj1|mundo|cometas");
    assertFileExists(projDir + "|joao|proj1|mundo|planetas|Urano|cometas");
    ProjectFile estrelas = mundo.getChild("estrelas");
    estrelas.move(urano);
    assertFileDoesNotExist(projDir + "|joao|proj1|mundo|estrelas");
    assertFileExists(projDir + "|joao|proj1|mundo|planetas|Urano|estrelas");
    cometas = urano.getChild("cometas");
    cometas.rename("aliens");
    assertFileDoesNotExist(
      projDir + "|joao|proj1|mundo|planetas|Urano|cometas");
    assertFileExists(projDir + "|joao|proj1|mundo|planetas|Urano|aliens");
    ProjectFile deus = root.getChild("Deus");
    deus.remove();
    assertFileDoesNotExist(projDir + "|joao|proj1|Deus");
    ProjectFile mares = terra.getChild("mares");
    proj1.createAndWaitForFile(mares, "Atlantico", HTML_TYPE, joao.getId());
    assertFileExists(
      projDir + "|joao|proj1|mundo|planetas|Terra|mares|Atlantico");
    ProjectFile vida = root.getChild("Vida");
    vida.changeType(HTML_TYPE);
  }

  public void afterCrashOperateWithFilesAndFolders() throws Exception {
    /* Verifico se o estado final condiz com o esperado */
    String projDir = System.getProperty("project_dir");
    assertFileExists(projDir + "|joao|proj1");
    assertFileExists(projDir + "|joao|proj1|mundo");
    assertFileExists(projDir + "|joao|proj1|mundo|planetas");
    assertFileExists(projDir + "|joao|proj1|mundo|planetas|Terra");
    assertFileExists(projDir + "|joao|proj1|mundo|planetas|Terra|mares");
    assertFileExists(
      projDir + "|joao|proj1|mundo|planetas|Terra|mares|Atlantico");
    assertFileExists(projDir + "|joao|proj1|mundo|planetas|Terra|paises");
    assertFileExists(
      projDir + "|joao|proj1|mundo|planetas|Terra|paises|Brasil");
    assertFileExists(projDir + "|joao|proj1|mundo|planetas|Urano");
    assertFileExists(projDir + "|joao|proj1|mundo|planetas|Urano|aliens");
    assertFileExists(projDir + "|joao|proj1|mundo|planetas|Urano|estrelas");
    assertFileExists(projDir + "|joao|proj1|mundo|Lua");
    assertFileExists(projDir + "|joao|proj1|Vida");
  }

  /**
   * Preenche o suite com todos os testes a serem rodados antes do servidor
   * sofrer shutdown.
   *
   * @param suite .
   *
   * @throws Exception .
   */
  public void callTestsBeforeCrash(TestSuite suite) throws Exception {
    suite.addTest(
      new TestProjectService("beforeCrashOperateWithFilesAndFolders"));
  }

  public void callTestsAfterCrash(TestSuite suite) throws Exception {
    suite.addTest(
      new TestProjectService("afterCrashOperateWithFilesAndFolders"));
  }

  public TestProjectService(String name) throws ClientException {
    super(name);
    loadProperties();
  }

  public TestProjectService() throws ClientException {
    super();
    loadProperties();
  }
}
