package server.services;

import csbase.client.kernel.ClientException;
import csbase.client.language.LNG;
import csbase.logic.*;
import csbase.server.services.projectservice.*;

import util.*;

import java.io.File;
import java.util.*;

public class TestPublicationService extends CSBaseTestCase {
  /*
     Testes realizados:
     - testRegularPublish                                          OK
     - testPublishInexistingUser                                   FAILED
     - testPublishOpenedFile                                       FAILED
   */
  private UN cenpes = null;
  private Role tecnico = null;
  private User joao = null;
  private User maria = null;
  private User pedro = null;
  private CommonClientProject projeto = null;
  private ClientProjectFile objetos = null;
  private String projDir = null;
  private String TEXT_TYPE;
  private String UNKNOWN_TYPE;
  private String HTML_TYPE;

  private void loadProperties() throws ClientException {
    /* Carrega as propriedades necessrias */
    projDir = System.getProperty("project_dir");
    TEXT_TYPE = "TEXT";
    UNKNOWN_TYPE = "UNKNOWN";
    HTML_TYPE = "HTML";
  }

  public void setUp() throws Exception {
    cenpes = createTestUN("Cenpes");
    tecnico = createTestRole("Tcnico");
    joao = createTestUser("joao", cenpes, tecnico);
    maria = createTestUser("maria", cenpes, tecnico);
    pedro = createTestUser("pedro", cenpes, tecnico);
    projeto = createTestProject("projeto", maria);
    ProjectFile root = projeto.getTree();
    objetos =
      (ClientProjectFile)projeto.createAndWaitForFile(
        root, "objetos", TEXT_TYPE, maria.getId());
  }

  public void tearDown() throws Exception {
    objetos.remove();
    CommonClientProject.remove(maria.getId(), "projeto");
    User.deleteUser(joao.getId());
    User.deleteUser(maria.getId());
    User.deleteUser(pedro.getId());
    Role.deleteRole(tecnico.getId());
    UN.deleteUN(cenpes.getId());
  }

  /* Testes de publicao */
  /**
   * Publica um arquivo para 3 usurios.
   *
   * @throws Exception .
   */
  public void testRegularPublish() throws Exception {
    assertNull(objetos.getPublishAccess());
    assertFalse(objetos.isPublishedFile());
    objetos.publish(
      new Object[] { joao.getId(), pedro.getId() }, maria.getId());
    try {
      Thread.sleep(200); // A publicao  assncrona
    }
    catch (InterruptedException e) {}
    Object[] publishedArray = objetos.getPublishAccess();
    assertNotNull(publishedArray);
    assertEquals(2, publishedArray.length);
    List publishedList = Arrays.asList(publishedArray);
    assertTrue(publishedList.contains(joao.getId()));
    assertTrue(publishedList.contains(pedro.getId()));
    assertEquals(maria.getId(), objetos.whoPublished());
    assertTrue(objetos.isPublishedFile());
  }

  /**
   * Publica um arquivo para um usurio inexistente. OBS:  PRECISO REFINAR A
   * EXCEO
   *
   * @throws Exception .
   */
  public void testPublishInexistingUser() throws Exception {
    try {
      objetos.publish(new Object[] { new Object() }, maria.getId());
      fail("Esperada Checked Exception");
    }
    catch (Exception e) {}
    try {
      Thread.sleep(200); // A publicao  assncrona
    }
    catch (InterruptedException e) {}
    assertNull(objetos.getPublishAccess());
  }

  /**
   * Tenta publicar um arquivo aberto public void testPublishOpenedFile()
   * throws Exception { objetos.setState(false,false,true); try {
   * objetos.publish(new Object[] { joao.getId() }); fail("Esperada
   * Exception"); } catch(Exception e) {} try { Thread.sleep(200); }
   * catch(InterruptedException e) {} objetos.setState(false,false,false); }
   *
   * @throws Exception .
   */

  /* Testes de obteno de publicao */
  /**
   * Publico um arquivo para dois usurios.  Tento obter a publicao de ambos,
   * mais um terceiro que no est na lista  de acesso.
   *
   * @throws Exception .
   */
  public void testGetPublication() throws Exception {
    /* Carrego o arquivo com informaes */
    objetos.openForWriting();
    objetos.writeObject("Mensagem");
    objetos.close();
    /* Preparo os usurios para receberem arquivos publicados */
    CommonClientProject joaoproj = createTestProject("joaoproj", joao);
    ClientProjectFile joaoroot = (ClientProjectFile)joaoproj.getTree();
    CommonClientProject pedroproj = createTestProject("pedroproj", pedro);
    ClientProjectFile pedroroot = (ClientProjectFile)pedroproj.getTree();
    User laura = createTestUser("Laura", cenpes, tecnico);
    CommonClientProject lauraproj = createTestProject("lauraproj", laura);
    ClientProjectFile lauraroot = (ClientProjectFile)lauraproj.getTree();
    /* Publico o arquivo */
    objetos.publish(
      new Object[] { joao.getId(), pedro.getId() }, maria.getId());
    try {
      Thread.sleep(200); // A publicao  assncrona
    }
    catch (InterruptedException e) {}
    /* Tento recuperar os arquivos */
    Vector joaoFiles = CommonClientProject.getAllPublicationInfo(joao.getId());
    PublicationInfo pubInfo = (PublicationInfo)joaoFiles.get(0);
    joaoroot.linkToPublishedFile(pubInfo, joao.getId());
    try {
      Thread.sleep(200); // A publicao  assncrona
    }
    catch (InterruptedException e) {}
    ProjectFile[] joaoChildren = joaoroot.getChildren();
    assertEquals(1, joaoChildren.length);
    /* Restauro as condies originais */
    CommonClientProject.remove(joao.getId(), "joaoproj");
    CommonClientProject.remove(pedro.getId(), "pedroproj");
    CommonClientProject.remove(laura.getId(), "lauraproj");
    User.deleteUser(laura.getId());
  }

  public TestPublicationService(String name) throws ClientException {
    super(name);
    loadProperties();
  }

  public TestPublicationService() throws ClientException {
    super();
    loadProperties();
  }
}
