/**
 * TestUN.java  $Author: taciana $ $Revision: 34156 $ - $Date: 2004/02/05
 * 18:06:36 $
 */
package server.services;

import csbase.logic.*;

import util.*;

import java.util.Vector;

/**
 * A classe <code>TestRole</code> representa um caso de teste para os servios
 * de manipulao de Perfis do Websintesi.
 */
public class TestRole extends CSBaseTestCase {
  private RoleInfo roleInfoTest;

  /* TESTES QUE NO PASSARAM:
   * testDeleteNull
   * testModifyNull
   * testModifyWithNullFields
   * testGetRoleNull
   */
  protected String getClassName() {
    return "TESTROLE";
  }

  /**
   * Garante que a lista de UNs esteja zerada antes de comear cada teste.
   * Prov um objeto UNInfo para ser usado, se necessrio, no teste (fixture).
   *
   * @throws Exception .
   */
  public void initialize() throws Exception {
    //??Alterado para mudana de permission
    System.out.println("??TestRole:39");
    roleInfoTest = new RoleInfo(TEST_KEY, TEST_KEY, null);
  }

  protected Vector getAllElements() throws Exception {
    return Role.getAllRoles();
  }

  protected void cleanElements(Vector elements) throws Exception {
    for (int i = 0; i < elements.size(); i++) {
      Role role = (Role)elements.get(i);
      Role.deleteRole(role.getId());
    }
  }

  /**
   * Testa a construo de Perfil.
   *
   * @throws Exception .
   */
  public void testCreateRole() throws Exception {
    Role role = createTestRole(TEST_KEY);
    assertNotNull(role);
    assertEquals(TEST_KEY, role.getName());
    assertEquals(TEST_KEY, role.getDescription());
    //??Alterado para mudana de permission
    System.out.println("??TestRole:61");
    //assertEquals(TEST_KEY, role.getLogin());
    //assertEquals(TEST_KEY, role.getPassword());
    Role.deleteRole(role.getId());
  }

  /**
   * Verifica se o sistema aceita criar um novo Perfil com o mesmo nome.
   *
   * @throws Exception .
   */
  public void testCreateSameRole() throws Exception {
    Role aRole = createTestRole(TEST_KEY);
    Role anotherRole = createTestRole(TEST_KEY);
    assertNotNull(aRole);
    assertNull(anotherRole);
    Role.deleteRole(aRole.getId());
  }

  /**
   * Testa se s os Perfis pares foram apagados depois, tenta recriar os Perfis
   * pares apagados
   *
   * @throws Exception .
   */
  public void testDeleteEvenRoles() throws Exception {
    int ELEMENTS = 6;
    Role[] role = new Role[ELEMENTS];
    for (int i = 0; i < ELEMENTS; i++) {
      role[i] = createTestRole("" + i);
    }
    /* Apaga todos os Perfis pares */
    for (int i = 0; i < ELEMENTS; i++) {
      if ((i % 2) == 0) {
        Role.deleteRole(role[i].getId());
        role[i] = null;
      }
    }
    /* Verifica se os pares foram apagados e se os mpares permanecem */
    Vector roleList = Role.getAllRoles();
    int remainingElements = ELEMENTS / 2;
    assertEquals(remainingElements, roleList.size());
    for (int i = 0; i < ELEMENTS; i++) {
      if ((i % 2) == 1) {
        assertTrue(roleList.contains(role[i]));
      }
    }
    /* Recria os pares recm-apagados */
    for (int i = 0; i < ELEMENTS; i++) {
      if ((i % 2) == 0) {
        role[i] = createTestRole("" + i);
      }
    }
    /* Verifica se todos esto presentes. Remove-os um a um. */
    roleList = Role.getAllRoles();
    assertEquals(ELEMENTS, roleList.size());
    for (int i = 0; i < ELEMENTS; i++) {
      assertTrue(roleList.contains(role[i]));
      Role.deleteRole(role[i].getId());
    }
  }

  /**
   * Tenta passar um roleInfo nulo para o mtodo de criao
   *
   * @throws Exception .
   */
  public void testCreateWithNullInfo() throws Exception {
    Role aRole = null;
    try {
      aRole = Role.createRole(null);
      fail("Esperada Exception");
    }
    catch (Exception e) {}
    assertNull(aRole);
  }

  /**
   * Tenta passar um unInfo que contm campos nulos
   *
   * @throws Exception .
   */
  public void testCreateWithNullFields() throws Exception {
    //??Alterado para mudana de permission
    System.out.println("??TestRole:131");
    RoleInfo aRoleInfo = new RoleInfo(null, null, null);
    Role aRole = null;
    try {
      aRole = Role.createRole(aRoleInfo);
      fail("Esperada Exception");
    }
    catch (Exception e) {}
    assertNull(aRole);
  }

  /**
   * Tenta apagar um Perfil que no existe
   *
   * @throws Exception .
   */
  public void testDeleteNonExistentRole() throws Exception {
    Role.deleteRole(new Integer(3000));
  }

  /**
   * Passa um id nulo pro Delete  public void testDeleteNull() throws Exception
   * { try { Role.deleteRole( null ); fail("Esperada Exception"); } catch(
   * Exception e ) {}}
   *
   * @throws Exception .
   */
  /**
   * Tenta modificar o nome (no pode ser modificado)
   *
   * @throws Exception .
   */
  public void testModifyName() throws Exception {
    Role role = createTestRole("1");
    Role anotherRole = Role.modifyRole(role.getId(), roleInfoTest);
    assertNull(anotherRole);
    role = Role.getRole(role.getId());
    assertEquals("1", role.getName());
    Role.deleteRole(role.getId());
  }

  /**
   * Tenta passar um identificador no existente
   *
   * @throws Exception .
   */
  public void testModifyNonExistentRole() throws Exception {
    Role aRole = Role.modifyRole(new Integer(3000), roleInfoTest);
    assertNull(aRole);
    Role anotherRole = Role.getRole(new Integer(3000));
    assertNull(anotherRole);
  }

  /**
   * Tenta passar um identificador nulo  public void testModifyNull() throws
   * Exception { Role aRole = null; try { aRole = Role.modifyRole(null,
   * roleInfoTest); fail("Esperada Exception"); } catch(Exception e) {}
   * assertNull(aRole);  }
   *
   * @throws Exception .
   */
  /**
   * Tenta passar um roleInfo nulo
   *
   * @throws Exception .
   */
  public void testModifyNullInfo() throws Exception {
    Role role = createTestRole(TEST_KEY);
    Role aRole = null;
    try {
      aRole = Role.modifyRole(role.getId(), null);
      fail("Esperada Exception");
    }
    catch (Exception e) {}
    assertNull(aRole);
    Role.deleteRole(role.getId());
  }

  /**
   * Tenta passar um roleInfo que contm campos nulos  public void
   * testModifyWithNullFields() throws Exception { Role role =
   * createTestRole("0"); RoleInfo aRoleInfo = new RoleInfo(null, null, null,
   * null); Role aRole = null; try { aRole = Role.modifyRole(role.getId(),
   * aRoleInfo); fail("Esperada Exception"); }  catch( Exception e ) {}
   * assertNull( aRole ); assertEquals("0", role.getName()); assertEquals("0",
   * role.getDescription()); assertEquals("0", role.getLogin());
   * assertEquals("0", role.getPassword()); Role.deleteRole(role.getId()); }
   *
   * @throws Exception .
   */
  public void testGetRole() throws Exception {
    Role role = createTestRole(TEST_KEY);
    assertEquals(role, Role.getRole(role.getId()));
    Role.deleteRole(role.getId());
  }

  /**
   * Testa se o vetor recuperado de Perfis corresponde ao esperado. OBS: Esse
   * vetor est sendo retornado na ordem de insero.
   *
   * @throws Exception .
   */
  public void testGetAllRoles() throws Exception {
    Role[] role = new Role[3];
    for (int i = 0; i < 3; i++) {
      role[i] = createTestRole("" + i);
    }
    Vector allRoles = Role.getAllRoles();
    assertEquals(3, allRoles.size());
    for (int i = 0; i < 3; i++) {
      assertTrue(allRoles.contains(role[i]));
      Role.deleteRole(role[i].getId());
    }
  }

  /**
   * Testa a modificao de um Perfil existente no arquivo de dados.
   *
   * @throws Exception .
   */
  public void testModifyRole() throws Exception {
    Role role = createTestRole(TEST_KEY);
    String desc = "desc";
    String login = "login";
    String password = "password";

    //??Alterado para mudana de permission
    System.out.println("??TestRole:243");
    RoleInfo aRoleInfo = new RoleInfo(TEST_KEY, desc, null);
    Role.modifyRole(role.getId(), aRoleInfo);
    role = Role.getRole(role.getId());
    assertEquals(desc, role.getDescription());
    //??Alterado para mudana de permission
    System.out.println("??TestRole:249");
    //assertEquals(login, role.getLogin()); 
    //assertEquals(password, role.getPassword()); 
    Role.deleteRole(role.getId());
  }
  /**
   * Testa passando null pro GetRole  public void testGetRoleNull() throws
   * Exception { Role aRole = null; try { aRole = Role.getRole(null);
   * fail("Esperada Exception"); }  catch(Exception e) {} assertNull(aRole); }
   */
}
