/**
 * TestUN.java  $Author: taciana $ $Revision: 34156 $ - $Date: 2004/02/05
 * 18:05:49 $
 */
package server.services;

import csbase.logic.*;

import util.*;

import java.util.Vector;

/**
 * A classe <code>TestUN</code> representa um caso de teste para os servios de
 * manipulao de UN's do CSBase.
 */
public class TestUN extends CSBaseTestCase {
  private UNInfo unInfoTest;

  /* TESTES QUE NO PASSARAM:
   * testModifyNull()
   * testModifyWithNullFields()
   */
  protected String getClassName() {
    return "TESTUN";
  }

  /**
   * Garante que a lista de UNs esteja zerada antes de comear cada teste.
   * Prov um objeto UNInfo para ser usado, se necessrio, no teste (fixture).
   *
   * @throws Exception .
   */
  public void initialize() throws Exception {
    unInfoTest = new UNInfo(TEST_KEY, TEST_KEY);
  }

  protected Vector getAllElements() throws Exception {
    return UN.getAllUNs();
  }

  protected void cleanElements(Vector elements) throws Exception {
    for (int i = 0; i < elements.size(); i++) {
      UN un = (UN)elements.get(i);
      UN.deleteUN(un.getId());
    }
  }

  public void setUp() throws Exception {}

  public void tearDown() {}
  /**
   * Testa a construo de UN. Verifica se os campos da UN assumiram os valores
   * esperados.
   *
   * @throws Exception .
   */
  public void testCreateUN() throws Exception {
    UN un = createTestUN(TEST_KEY);
    assertNotNull(un);
    assertEquals(TEST_KEY, un.getName());
    assertEquals(TEST_KEY, un.getDesc());
    UN.deleteUN(un.getId());
  }

  /**
   * Verifica se o sistema aceita criar uma nova UN com o mesmo nome.
   *
   * @throws Exception .
   */
  public void testCreateSameUN() throws Exception {
    UN un1 = createTestUN(TEST_KEY);
    UN un2 = createTestUN(TEST_KEY);
    assertNotNull(un1);
    assertNull(un2);
    UN.deleteUN(un1.getId());
  }

  /**
   * Teste de remoo: cria vrias UNs, apaga todas as pares. Em seguida, tenta
   * recriar as UNs recm-apagadas.
   *
   * @throws Exception .
   */
  public void testDeleteEvenUNs() throws Exception {
    /* Cria vrias UNs de teste */
    int ELEMENTS = 6;
    UN[] uns = new UN[ELEMENTS];
    for (int i = 0; i < ELEMENTS; i++) {
      uns[i] = createTestUN("" + i);
    }
    /* Apaga todas as UNs pares */
    for (int i = 0; i < ELEMENTS; i++) {
      if ((i % 2) == 0) {
        UN.deleteUN(uns[i].getId());
        uns[i] = null;
      }
    }
    /* Verifica se os pares foram apagados e se os mpares permanecem */
    int remainingElements = ELEMENTS / 2;
    assertListSize(remainingElements);
    Vector unList = UN.getAllUNs();
    for (int i = 0; i < ELEMENTS; i++) {
      if ((i % 2) == 1) {
        assertTrue(unList.contains(uns[i]));
      }
    }
    /* Recria os pares recm-apagados */
    for (int i = 0; i < ELEMENTS; i++) {
      if ((i % 2) == 0) {
        uns[i] = createTestUN("" + i);
      }
    }
    /* Verifica se todos esto presentes */
    assertListSize(ELEMENTS);
    unList = UN.getAllUNs();
    for (int i = 0; i < ELEMENTS; i++) {
      assertTrue(unList.contains(uns[i]));
    }
    /* Faz a limpeza final */
    for (int i = 0; i < ELEMENTS; i++) {
      UN.deleteUN(uns[i].getId());
    }
  }

  /**
   * Tenta passar um unInfo nulo para o mtodo de criao
   *
   * @throws Exception .
   */
  public void testCreateWithNullInfo() throws Exception {
    UN aUN = null;
    try {
      aUN = UN.createUN(null);
      fail("Esperada Exception");
    }
    catch (Exception e) {}
    assertNull(aUN);
  }

  /**
   * Tenta passar um unInfo que contm o campo-chave nulo/vazio
   *
   * @throws Exception .
   */
  public void testCreateWithNullFields() throws Exception {
    UNInfo aUNInfo = new UNInfo(null, TEST_KEY);
    UN aUN = null;
    try {
      aUN = UN.createUN(aUNInfo);
      fail("Esperada Exception");
    }
    catch (Exception e) {}
    assertNull(aUN);
    UNInfo anotherUNInfo = new UNInfo("", TEST_KEY);
    try {
      aUN = UN.createUN(aUNInfo);
      fail("Esperada Exception");
    }
    catch (Exception e) {}
    assertNull(aUN);
  }

  /**
   * Tenta apagar uma UN que no existe
   *
   * @throws Exception .
   */
  public void testDeleteNonExistentUN() throws Exception {
    UN.deleteUN(new Integer(3000));
  }

  /**
   * Tenta apagar uma UN passando nulo
   *
   * @throws Exception .
   */
  public void testDeleteNull() throws Exception {
    try {
      UN.deleteUN(null);
      fail("Esperada Exception");
    }
    catch (Exception e) {}
  }

  /**
   * Tenta modificar o nome das UNs
   *
   * @throws Exception .
   */
  public void testModifyName() throws Exception {
    UN un1 = createTestUN("1");
    UN un2 = createTestUN("2");
    UN modifUn1 = UN.modifyUN(un1.getId(), unInfoTest);
    assertNotNull(modifUn1);
    UN modifUn2 = UN.modifyUN(un2.getId(), unInfoTest);
    assertNull(modifUn2);
    /* Verifica as modificaes */
    assertListSize(2);
    un1 = UN.getUN(un1.getId());
    assertEquals(TEST_KEY, un1.getName());
    assertEquals(TEST_KEY, un1.getDesc());
    un2 = UN.getUN(un2.getId());
    assertEquals("2", un2.getName());
    assertEquals("2", un2.getDesc());
    UN.deleteUN(un1.getId());
    UN.deleteUN(un2.getId());
  }

  /**
   * Tenta passar um identificador no existente
   *
   * @throws Exception .
   */
  public void testModifyNonExistentUN() throws Exception {
    UN aUN = UN.modifyUN(new Integer(3000), unInfoTest);
    assertNull(aUN);
    UN anotherUN = UN.getUN(new Integer(3000));
    assertNull(anotherUN);
  }

  /**
   * Tenta passar um identificador nulo  public void testModifyNull() throws
   * Exception { UN aUN = null; try {   aUN = UN.modifyUN( null, unInfoTest );
   * fail("Esperada Exception"); } catch( Exception e ) {} assertNull( aUN );
   * }
   *
   * @throws Exception .
   */
  /**
   * Tenta passar um unInfo nulo
   *
   * @throws Exception .
   */
  public void testModifyNullInfo() throws Exception {
    UN un = createTestUN(TEST_KEY);
    UN aUN = null;
    try {
      aUN = UN.modifyUN(un.getId(), null);
      fail("Esperada Exception");
    }
    catch (Exception e) {}
    assertNull(aUN);
    UN.deleteUN(un.getId());
  }

  /**
   * Tenta passar um unInfo que contm campos nulos  public void
   * testModifyWithNullFields() throws Exception { UNInfo aUNInfo = new
   * UNInfo(null, TEST_KEY); UN un = createTestUN(TEST_KEY); UN aUN = null;
   * try { aUN = UN.modifyUN(un.getId(), aUNInfo); fail("Esperada Exception");
   * }  catch( Exception e ) {} assertNull( aUN ); UN.deleteUN(un.getId()); }
   *
   * @throws Exception .
   */
  /**
   * Verifica se o mtodo de recuperar elementos por id funciona  corretamente.
   *
   * @throws Exception .
   */
  public void testGetUN() throws Exception {
    UN un = createTestUN(TEST_KEY);
    assertEquals(un, UN.getUN(un.getId()));
    UN.deleteUN(un.getId());
  }

  /**
   * Verifica se o mtodo de recuperar elementos por nome funciona
   * corretamente.
   *
   * @throws Exception .
   */
  public void testGetUNByName() throws Exception {
    UN un = createTestUN(TEST_KEY);
    assertEquals(un, UN.getUNByName(un.getName()));
    UN.deleteUN(un.getId());
  }

  /**
   * Verifica se as UNs so gravadas corretamente em arquivo
   *
   * @throws Exception .
   */
  public void testGetAllUNs() throws Exception {
    UN[] uns = new UN[3];
    for (int i = 0; i < 3; i++) {
      uns[i] = createTestUN("" + i);
    }
    assertListSize(3);
    Vector unList = UN.getAllUNs();
    for (int i = 0; i < 3; i++) {
      assertTrue(unList.contains(uns[i]));
      UN.deleteUN(uns[i].getId());
    }
  }

  /**
   * Testa a modificao de uma UN existente no arquivo de dados.
   *
   * @throws Exception .
   */
  public void testModifyUN() throws Exception {
    UN un = createTestUN(TEST_KEY);
    String name = "nome";
    String desc = "desc";
    UNInfo aUNInfo = new UNInfo(name, desc);
    UN aUN = UN.modifyUN(un.getId(), aUNInfo);
    assertNotNull(aUN);
    un = UN.getUN(un.getId());
    assertEquals(name, un.getName());
    assertEquals(desc, un.getDesc());
    UN.deleteUN(un.getId());
  }

  /**
   * Testa passando null pro GetUN
   *
   * @throws Exception .
   */
  public void testGetUNNull() throws Exception {
    UN aUN = UN.getUN(null);
    assertNull(aUN);
  }
}
