package util;

import csbase.client.ServerMonitor;
import csbase.client.remote.ProjectObserver;
import csbase.logic.*;
import csbase.remote.RemoteLocator;

import junit.framework.*;

import java.io.*;
import java.rmi.RemoteException;

/**
 * Classe utilitria para manipular o servidor e o monitor deste.
 */
public abstract class CSBaseTestSuite extends TestCase {
  /** Parmetros para inicializao do servidor */
  public static final String WORKING_DIR = "workingdir";
  public static final String RUNSERVER_COMMAND = "runserver";
  public static final String SERVER = "server";
  public static final String RMI_PORT = "rmiport";
  /** Thread responsvel por levantar o servidor */
  private static StartServerThread thread;
  /** Linha de comando para levantar o servidor */
  private static String runServerCommand;
  /** Diretrio de trabalho para disparar a linha de comando */
  private static File serverWorkingDir;
  /** singleton para ser usado nos servidores */
  protected static ServerMonitor serverMonitor = null;

  /**
   * Inicializa o servidor em uma thread separada.
   *
   * @param runServerCommand de comando para disparar o servidor.
   * @param serverWorkingDir diretrio de onde disparar a linha de comando.
   */
  protected static void startServer(
    String runServerCommand, File serverWorkingDir) {
    long SLEEP_TIME = 15000;
    CSBaseTestSuite.runServerCommand = runServerCommand;
    CSBaseTestSuite.serverWorkingDir = serverWorkingDir;
    thread = new StartServerThread();
    thread.start();
    /* Aguarda um tempo mnimo para o servidor subir e fazer o bind dos
     * servios. */
    try {
      Thread.sleep(SLEEP_TIME);
    }
    catch (InterruptedException e) {}
  }

  /**
   * Tenta derrubar o servidor, invocando seu servio de finalizao.
   */
  protected static void stopServer() {
    assertNotNull(RemoteLocator.killServerService);
    try {
      RemoteLocator.killServerService.shutDown();
    }
    catch (RemoteException e) {}
  }

  /**
   * Inicializa o monitor do servidor, que vai garantir a conexo do cliente.
   * Recupera referncias para os provedores do servidor RMI  e inicializa os
   * respectivos Observers.
   *
   * @param serverMonitor .
   * @param serverName .
   * @param rmiPort .
   */
  protected static void startClient(
    ServerMonitor serverMonitor, String serverName, String rmiPort) {
    assertNotNull(serverMonitor);
    try {
      serverMonitor.start(serverName + ":" + rmiPort);
      if (!serverMonitor.isAlive()) {
        System.out.println(
          "Servidor no disponvel:" + serverName + ":" + rmiPort);
        return;
      }
      String bindBase = "rmi://" + serverName + ":" + rmiPort + "/";
      System.out.println("Bind no servidor " + bindBase);
    }
    catch (Exception e) {
      System.out.println("Erro ao conectar com o servidor: " + serverName);
      e.printStackTrace();
      System.exit(0);
    }
  }

  /**
   * Finaliza a monitorao do servidor no cliente
   *
   * @param serverMonitor .
   */
  protected static void stopClient(ServerMonitor serverMonitor) {
    serverMonitor.stop();
  }

  /**
   * Thread interna, responsvel por levantar o servidor. A inicializao do
   * servidor  feita em outra virtual machine.
   */
  static class StartServerThread extends Thread {
    public void run() {
      Process p = null;
      try {
        p = Runtime.getRuntime().exec(runServerCommand, null, serverWorkingDir);
      }
      catch (Exception e) {
        System.out.println("O servidor no pde ser instanciado.");
        e.printStackTrace();
        System.out.println("Terminando o processo...");
        System.exit(0);
      }
      /* Envia a sada do comando para a tela */
      BufferedReader in =
        new BufferedReader(new InputStreamReader(p.getInputStream()));
      String line = null;
      try {
        while ((line = in.readLine()) != null) {
          System.out.println(line);
        }
      }
      catch (IOException e) {
        System.exit(0);
      }
    }
  }

  public CSBaseTestSuite() {}

  public CSBaseTestSuite(String name) {
    super(name);
  }
}
